Package: ggraph
Type: Package
Title: An Implementation of Grammar of Graphics for Graphs and Networks
Version: 2.2.2
Authors@R: 
    c(person(given = "Thomas Lin",
             family = "Pedersen",
             role = c("cre", "aut"),
             email = "thomasp85@gmail.com",
             comment = c(ORCID = "0000-0002-5147-4711")),
      person(given = "RStudio",
             role = "cph"))
Maintainer: Thomas Lin Pedersen <thomasp85@gmail.com>
Description: The grammar of graphics as implemented in ggplot2 is a poor fit for
    graph and network visualizations due to its reliance on tabular data input.
    ggraph is an extension of the ggplot2 API tailored to graph visualizations
    and provides the same flexible approach to building up plots layer by layer.
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: TRUE
Imports: dplyr, ggforce (>= 0.3.1), grid, igraph (>= 1.0.0), scales,
        MASS, ggrepel, utils, stats, viridis, rlang, tidygraph,
        graphlayouts (>= 1.1.0), withr, cli, vctrs, lifecycle, memoise
Suggests: network, knitr, rmarkdown, purrr, tibble, seriation, deldir,
        gganimate, covr, sf, sfnetworks
LinkingTo: cpp11
RoxygenNote: 7.3.2
Depends: R (>= 2.10), ggplot2 (>= 3.5.0)
VignetteBuilder: knitr
URL: https://ggraph.data-imaginist.com,
        https://github.com/thomasp85/ggraph
BugReports: https://github.com/thomasp85/ggraph/issues
NeedsCompilation: yes
Packaged: 2025-08-24 10:14:20 UTC; thomas
Author: Thomas Lin Pedersen [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-5147-4711>),
  RStudio [cph]
Repository: CRAN
Date/Publication: 2025-08-24 12:20:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-25 13:33:12 UTC; unix
Archs: ggraph.so.dSYM

# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,layout_ggraph)
S3method(autograph,default)
S3method(collect_connections,default)
S3method(collect_connections,layout_tbl_graph)
S3method(collect_edges,default)
S3method(collect_edges,layout_tbl_graph)
S3method(create_layout,default)
S3method(create_layout,layout_ggraph)
S3method(create_layout,tbl_graph)
S3method(format,ggraph_geometry)
S3method(ggplot_build,ggraph)
S3method(is.na,ggraph_geometry)
S3method(layout_to_table,"function")
S3method(layout_to_table,character)
S3method(layout_to_table,data.frame)
S3method(layout_to_table,default)
S3method(layout_to_table,matrix)
S3method(makeContent,cappedpathgrob)
S3method(makeContent,parallelPath)
S3method(makeContent,textalong)
S3method(scale_type,ggraph_geometry)
S3method(vec_cast,character.ggraph_geometry)
S3method(vec_ptype2,character.ggraph_geometry)
S3method(vec_ptype2,ggraph_geometry.character)
S3method(vec_ptype2,ggraph_geometry.ggraph_geometry)
export(FacetEdges)
export(FacetGraph)
export(FacetNodes)
export(GeomAxisHive)
export(GeomEdgeBezier)
export(GeomEdgeBspline)
export(GeomEdgeParallelPath)
export(GeomEdgeParallelSegment)
export(GeomEdgePath)
export(GeomEdgePoint)
export(GeomEdgeSegment)
export(GeomEdgeSf)
export(GeomEdgeSpanPath)
export(GeomEdgeSpanSegment)
export(GeomEdgeTile)
export(GeomNodeTile)
export(StatAxisHive)
export(StatConnBundle)
export(StatConnBundle0)
export(StatConnBundle2)
export(StatEdgeArc)
export(StatEdgeArc0)
export(StatEdgeArc2)
export(StatEdgeBend)
export(StatEdgeBend0)
export(StatEdgeBend2)
export(StatEdgeBundleForce)
export(StatEdgeBundleForce0)
export(StatEdgeBundleForce2)
export(StatEdgeBundleMinimal)
export(StatEdgeBundleMinimal0)
export(StatEdgeBundleMinimal2)
export(StatEdgeBundlePath)
export(StatEdgeBundlePath0)
export(StatEdgeBundlePath2)
export(StatEdgeDensity)
export(StatEdgeDiagonal)
export(StatEdgeDiagonal0)
export(StatEdgeDiagonal2)
export(StatEdgeElbow)
export(StatEdgeElbow0)
export(StatEdgeElbow2)
export(StatEdgeFan)
export(StatEdgeFan0)
export(StatEdgeFan2)
export(StatEdgeHive)
export(StatEdgeHive0)
export(StatEdgeHive2)
export(StatEdgeLink)
export(StatEdgeLink2)
export(StatEdgeLoop)
export(StatEdgeLoop0)
export(StatEdgeParallel)
export(StatEdgeParallel0)
export(StatEdgeParallel2)
export(StatFilter)
export(StatFilterSf)
export(StatNodeArcBar)
export(StatNodeCircle)
export(StatNodeVoronoi)
export(StatReverse)
export(autograph)
export(circle)
export(collect_connections)
export(collect_edges)
export(create_layout)
export(edge_angle)
export(ellipsis)
export(facet_edges)
export(facet_graph)
export(facet_nodes)
export(geom_axis_hive)
export(geom_conn_bundle)
export(geom_conn_bundle0)
export(geom_conn_bundle2)
export(geom_edge_arc)
export(geom_edge_arc0)
export(geom_edge_arc2)
export(geom_edge_bend)
export(geom_edge_bend0)
export(geom_edge_bend2)
export(geom_edge_bundle_force)
export(geom_edge_bundle_force0)
export(geom_edge_bundle_force2)
export(geom_edge_bundle_minimal)
export(geom_edge_bundle_minimal0)
export(geom_edge_bundle_minimal2)
export(geom_edge_bundle_path)
export(geom_edge_bundle_path0)
export(geom_edge_bundle_path2)
export(geom_edge_density)
export(geom_edge_diagonal)
export(geom_edge_diagonal0)
export(geom_edge_diagonal2)
export(geom_edge_elbow)
export(geom_edge_elbow0)
export(geom_edge_elbow2)
export(geom_edge_fan)
export(geom_edge_fan0)
export(geom_edge_fan2)
export(geom_edge_hive)
export(geom_edge_hive0)
export(geom_edge_hive2)
export(geom_edge_link)
export(geom_edge_link0)
export(geom_edge_link2)
export(geom_edge_loop)
export(geom_edge_loop0)
export(geom_edge_parallel)
export(geom_edge_parallel0)
export(geom_edge_parallel2)
export(geom_edge_point)
export(geom_edge_sf)
export(geom_edge_span)
export(geom_edge_span0)
export(geom_edge_span2)
export(geom_edge_tile)
export(geom_node_arc_bar)
export(geom_node_circle)
export(geom_node_label)
export(geom_node_point)
export(geom_node_range)
export(geom_node_sf)
export(geom_node_text)
export(geom_node_tile)
export(geom_node_voronoi)
export(geometry)
export(get_con)
export(get_edges)
export(get_nodes)
export(ggraph)
export(guide_edge_colorbar)
export(guide_edge_colorsteps)
export(guide_edge_colourbar)
export(guide_edge_coloursteps)
export(guide_edge_direction)
export(is.geometry)
export(label_rect)
export(layout_to_table)
export(node_angle)
export(node_rank_fabric)
export(pack_circles)
export(qgraph)
export(rectangle)
export(scale_color_viridis)
export(scale_edge_alpha)
export(scale_edge_alpha_binned)
export(scale_edge_alpha_continuous)
export(scale_edge_alpha_discrete)
export(scale_edge_alpha_identity)
export(scale_edge_alpha_manual)
export(scale_edge_color_binned)
export(scale_edge_color_brewer)
export(scale_edge_color_continuous)
export(scale_edge_color_discrete)
export(scale_edge_color_distiller)
export(scale_edge_color_fermenter)
export(scale_edge_color_gradient)
export(scale_edge_color_gradient2)
export(scale_edge_color_gradientn)
export(scale_edge_color_grey)
export(scale_edge_color_hue)
export(scale_edge_color_identity)
export(scale_edge_color_manual)
export(scale_edge_color_steps)
export(scale_edge_color_steps2)
export(scale_edge_color_stepsn)
export(scale_edge_color_viridis)
export(scale_edge_colour_binned)
export(scale_edge_colour_brewer)
export(scale_edge_colour_continuous)
export(scale_edge_colour_discrete)
export(scale_edge_colour_distiller)
export(scale_edge_colour_fermenter)
export(scale_edge_colour_gradient)
export(scale_edge_colour_gradient2)
export(scale_edge_colour_gradientn)
export(scale_edge_colour_grey)
export(scale_edge_colour_hue)
export(scale_edge_colour_identity)
export(scale_edge_colour_manual)
export(scale_edge_colour_steps)
export(scale_edge_colour_steps2)
export(scale_edge_colour_stepsn)
export(scale_edge_colour_viridis)
export(scale_edge_fill_binned)
export(scale_edge_fill_brewer)
export(scale_edge_fill_continuous)
export(scale_edge_fill_discrete)
export(scale_edge_fill_distiller)
export(scale_edge_fill_fermenter)
export(scale_edge_fill_gradient)
export(scale_edge_fill_gradient2)
export(scale_edge_fill_gradientn)
export(scale_edge_fill_grey)
export(scale_edge_fill_hue)
export(scale_edge_fill_identity)
export(scale_edge_fill_manual)
export(scale_edge_fill_steps)
export(scale_edge_fill_steps2)
export(scale_edge_fill_stepsn)
export(scale_edge_fill_viridis)
export(scale_edge_linetype)
export(scale_edge_linetype_binned)
export(scale_edge_linetype_continuous)
export(scale_edge_linetype_discrete)
export(scale_edge_linetype_identity)
export(scale_edge_linetype_manual)
export(scale_edge_radius)
export(scale_edge_shape)
export(scale_edge_shape_binned)
export(scale_edge_shape_continuous)
export(scale_edge_shape_discrete)
export(scale_edge_shape_identity)
export(scale_edge_shape_manual)
export(scale_edge_size)
export(scale_edge_size_area)
export(scale_edge_size_binned)
export(scale_edge_size_binned_area)
export(scale_edge_size_continuous)
export(scale_edge_size_discrete)
export(scale_edge_size_identity)
export(scale_edge_size_manual)
export(scale_edge_width)
export(scale_edge_width_binned)
export(scale_edge_width_continuous)
export(scale_edge_width_discrete)
export(scale_edge_width_identity)
export(scale_edge_width_manual)
export(scale_fill_viridis)
export(scale_label_size)
export(scale_label_size_binned)
export(scale_label_size_continuous)
export(scale_label_size_discrete)
export(scale_label_size_identity)
export(scale_label_size_manual)
export(set_graph_style)
export(square)
export(th_foreground)
export(th_no_axes)
export(theme_graph)
export(unset_graph_style)
import(ggplot2)
import(rlang)
import(tidygraph)
import(vctrs)
importFrom(MASS,bandwidth.nrd)
importFrom(MASS,kde2d)
importFrom(dplyr,arrange)
importFrom(dplyr,do)
importFrom(dplyr,group_by)
importFrom(dplyr,group_by_)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,pull)
importFrom(dplyr,slice)
importFrom(dplyr,summarise)
importFrom(dplyr,top_n)
importFrom(dplyr,transmute)
importFrom(dplyr,ungroup)
importFrom(ggforce,GeomArcBar)
importFrom(ggforce,GeomBezier0)
importFrom(ggforce,GeomBspline0)
importFrom(ggforce,GeomCircle)
importFrom(ggforce,GeomShape)
importFrom(ggforce,StatArcBar)
importFrom(ggforce,StatBezier)
importFrom(ggforce,StatBezier0)
importFrom(ggforce,StatBezier2)
importFrom(ggforce,StatBspline)
importFrom(ggforce,StatBspline2)
importFrom(ggforce,StatCircle)
importFrom(ggforce,StatLink)
importFrom(ggforce,StatLink2)
importFrom(ggforce,StatVoronoiTile)
importFrom(ggforce,interpolateDataFrame)
importFrom(ggforce,radial_trans)
importFrom(ggrepel,GeomLabelRepel)
importFrom(ggrepel,GeomTextRepel)
importFrom(graphlayouts,layout_as_backbone)
importFrom(graphlayouts,layout_as_metromap)
importFrom(graphlayouts,layout_with_centrality)
importFrom(graphlayouts,layout_with_centrality_group)
importFrom(graphlayouts,layout_with_constrained_stress)
importFrom(graphlayouts,layout_with_eigen)
importFrom(graphlayouts,layout_with_fixed_coords)
importFrom(graphlayouts,layout_with_focus)
importFrom(graphlayouts,layout_with_focus_group)
importFrom(graphlayouts,layout_with_pmds)
importFrom(graphlayouts,layout_with_sparse_stress)
importFrom(graphlayouts,layout_with_stress)
importFrom(grid,convertHeight)
importFrom(grid,convertWidth)
importFrom(grid,convertX)
importFrom(grid,convertY)
importFrom(grid,gList)
importFrom(grid,gTree)
importFrom(grid,gpar)
importFrom(grid,grob)
importFrom(grid,grobHeight)
importFrom(grid,grobName)
importFrom(grid,grobTree)
importFrom(grid,grobWidth)
importFrom(grid,is.unit)
importFrom(grid,makeContent)
importFrom(grid,nullGrob)
importFrom(grid,pointsGrob)
importFrom(grid,polylineGrob)
importFrom(grid,rasterGrob)
importFrom(grid,rectGrob)
importFrom(grid,segmentsGrob)
importFrom(grid,setChildren)
importFrom(grid,textGrob)
importFrom(grid,unit)
importFrom(igraph,"%--%")
importFrom(igraph,"V<-")
importFrom(igraph,"vertex_attr<-")
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,add_edges)
importFrom(igraph,add_vertices)
importFrom(igraph,adjacent_vertices)
importFrom(igraph,as.undirected)
importFrom(igraph,as_edgelist)
importFrom(igraph,bfs)
importFrom(igraph,components)
importFrom(igraph,count_components)
importFrom(igraph,degree)
importFrom(igraph,delete_edges)
importFrom(igraph,delete_vertex_attr)
importFrom(igraph,distances)
importFrom(igraph,edge_attr)
importFrom(igraph,ends)
importFrom(igraph,gorder)
importFrom(igraph,graph_attr)
importFrom(igraph,gsize)
importFrom(igraph,incident_edges)
importFrom(igraph,induced_subgraph)
importFrom(igraph,is.directed)
importFrom(igraph,is.named)
importFrom(igraph,is_directed)
importFrom(igraph,layout_as_bipartite)
importFrom(igraph,layout_as_star)
importFrom(igraph,layout_as_tree)
importFrom(igraph,layout_in_circle)
importFrom(igraph,layout_nicely)
importFrom(igraph,layout_on_grid)
importFrom(igraph,layout_on_sphere)
importFrom(igraph,layout_randomly)
importFrom(igraph,layout_with_dh)
importFrom(igraph,layout_with_drl)
importFrom(igraph,layout_with_fr)
importFrom(igraph,layout_with_gem)
importFrom(igraph,layout_with_graphopt)
importFrom(igraph,layout_with_kk)
importFrom(igraph,layout_with_lgl)
importFrom(igraph,layout_with_mds)
importFrom(igraph,layout_with_sugiyama)
importFrom(igraph,mst)
importFrom(igraph,neighbors)
importFrom(igraph,permute)
importFrom(igraph,shortest_paths)
importFrom(igraph,simplify)
importFrom(igraph,unfold_tree)
importFrom(igraph,vertex_attr)
importFrom(lifecycle,deprecated)
importFrom(memoise,memoise)
importFrom(rlang,.data)
importFrom(rlang,as_quosure)
importFrom(rlang,enquo)
importFrom(rlang,eval_tidy)
importFrom(rlang,quo_is_null)
importFrom(rlang,quo_is_symbol)
importFrom(rlang,quo_text)
importFrom(rlang,quos)
importFrom(rlang,sym)
importFrom(scales,identity_pal)
importFrom(scales,muted)
importFrom(scales,rescale_pal)
importFrom(stats,setNames)
importFrom(tidygraph,.G)
importFrom(tidygraph,.register_graph_context)
importFrom(tidygraph,activate)
importFrom(tidygraph,arrange)
importFrom(tidygraph,as_tbl_graph)
importFrom(tidygraph,graph_is_forest)
importFrom(tidygraph,graph_is_tree)
importFrom(tidygraph,node_is_root)
importFrom(utils,head)
importFrom(utils,tail)
importFrom(viridis,scale_color_viridis)
importFrom(viridis,scale_colour_viridis)
importFrom(viridis,scale_fill_viridis)
useDynLib(ggraph, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# ggraph <img src="man/figures/logo.png" align="right" />

<!-- badges: start -->

[![R-CMD-check](https://github.com/thomasp85/ggraph/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/thomasp85/ggraph/actions/workflows/R-CMD-check.yaml)
[![CRAN_Release_Badge](http://www.r-pkg.org/badges/version-ago/ggraph)](https://CRAN.R-project.org/package=ggraph)
[![CRAN_Download_Badge](http://cranlogs.r-pkg.org/badges/ggraph)](https://CRAN.R-project.org/package=ggraph)
[![Codecov test
coverage](https://codecov.io/gh/thomasp85/ggraph/branch/main/graph/badge.svg)](https://app.codecov.io/gh/thomasp85/ggraph?branch=main)
<!-- badges: end -->

*/dʒiː.dʒɪˈrɑːf/* (or g-giraffe)

## A grammar of graphics for relational data

ggraph is an extension of [`ggplot2`](https://ggplot2.tidyverse.org)
aimed at supporting relational data structures such as networks, graphs,
and trees. While it builds upon the foundation of `ggplot2` and its API
it comes with its own self-contained set of geoms, facets, etc., as well
as adding the concept of *layouts* to the grammar.

### An example

``` r
library(ggraph)
#> Loading required package: ggplot2
library(tidygraph)
#> 
#> Attaching package: 'tidygraph'
#> The following object is masked from 'package:stats':
#> 
#>     filter

# Create graph of highschool friendships
graph <- as_tbl_graph(highschool) |> 
    mutate(Popularity = centrality_degree(mode = 'in'))

# plot using ggraph
ggraph(graph, layout = 'kk') + 
    geom_edge_fan(aes(alpha = after_stat(index)), show.legend = FALSE) + 
    geom_node_point(aes(size = Popularity)) + 
    facet_edges(~year) + 
    theme_graph(foreground = 'steelblue', fg_text_colour = 'white')
```

![](man/figures/README-unnamed-chunk-2-1.png)<!-- -->

### The core concepts

`ggraph` builds upon three core concepts that are quite easy to
understand:

1.  [**The
    Layout**](https://www.data-imaginist.com/2017/ggraph-introduction-layouts/)
    defines how nodes are placed on the plot, that is, it is a
    conversion of the relational structure into an x and y value for
    each node in the graph. `ggraph` has access to all layout functions
    available in `igraph` and furthermore provides a large selection of
    its own, such as hive plots, treemaps, and circle packing.
2.  [**The
    Nodes**](https://www.data-imaginist.com/2017/ggraph-introduction-nodes/)
    are the connected entities in the relational structure. These can be
    plotted using the `geom_node_*()` family of geoms. Some node geoms
    make more sense for certain layouts, e.g. `geom_node_tile()` for
    treemaps and icicle plots, while others are more general purpose,
    e.g. `geom_node_point()`.
3.  [**The
    Edges**](https://www.data-imaginist.com/2017/ggraph-introduction-edges/)
    are the connections between the entities in the relational
    structure. These can be visualized using the `geom_edge_*()` family
    of geoms that contain a lot of different edge types for different
    scenarios. Sometimes the edges are implied by the layout (e.g. with
    treemaps) and need not be plotted, but often some sort of line is
    warranted.

All of the tree concepts have been discussed in detail in dedicated blog
posts that are also available as vignettes in the package. Please refer
to these for more information.

> **Note:** The linked blog posts are based on ggraph v1. After ggraph
> v1.1 the underlying implementation was moved to tidygraph and cleaned
> up, but this resulted in some breaking changes in the process.
> Therefore the vignette versions are generally recommended as they have
> been updated.

### Supported data types

There are many different ways to store and work with relational data in
R. `ggraph` is built upon `tidygraph` and the large swath of data
structures it supports are thus natively supported in `ggraph`. In order
to get a data type supported by `ggraph`, simply provide an
`as_tbl_graph` method for it.

## Installation

`ggraph` is available through CRAN and can be installed with
`install.packages('ggraph')`. The package is under active development
though and the latest set of features can be obtained by installing from
this repository using `devtools`

``` r
# install.packages("pak")
pak::pak('thomasp85/ggraph')
```

## Related work

`ggraph` is not the only package to provide some sort of support for
relational data in `ggplot2`, though I’m fairly certain that it is the
most ambitious.
[`ggdendro`](https://CRAN.R-project.org/package=ggdendro) provides
support for `dendrogram` and `hclust` objects through conversion of the
structures into line segments that can then be plotted with
`geom_segment()`. [`ggtree`](https://bioconductor.org/packages/ggtree/)
provides more extensive support for all things tree-related, though it
lacks some of the layouts and edge types that `ggraph` offers (it has
other features that `ggraph` lacks though). For more standard *hairball*
network plots
[`ggnetwork`](https://CRAN.R-project.org/package=ggnetwork),
[`geomnet`](https://CRAN.R-project.org/package=geomnet), and
[`GGally`](https://CRAN.R-project.org/package=GGally) all provide some
functionality though none of them are as extensive in scope as `ggraph`.

## Code of Conduct

Please note that the ‘ggraph’ project is released with a [Contributor
Code of
Conduct](https://ggraph.data-imaginist.com/CODE_OF_CONDUCT.html). By
contributing to this project, you agree to abide by its terms.

#' Create a node extractor function
#'
#' This function returns another function that can extract nodes from a
#' ggraph_layout object. As a ggraph_layout object is essentially a data.frame
#' of nodes it might seem useless to provide this function, but since the node
#' data is not necessarily available until after the `ggraph()` call it
#' can be beneficial to be able to add information to the node data on a
#' per-layer basis. Unlike [get_edges()] the use of `get_nodes` is not
#' mandatory and is only required if additional data should be added to selected
#' node layers.
#'
#' @param ... Additional data that should be cbind'ed together with the node
#' data. Accepts expressions that will be evaluated on the node data in it's
#' original order (irrespective of any reordering by the layout)
#'
#' @return A data.frame with the node data as well of any additional data
#' supplied through `...`
#'
#' @family extractors
#'
#' @export
#'
get_nodes <- function(...) {
  dots <- enquos(...)
  function(layout) {
    layout_reorder <- layout[order(layout$.ggraph.orig_index), ]
    extra_data <- lapply(dots, function(x) {
      val <- eval_tidy(x, layout_reorder)
      rep(val, length.out = nrow(layout))[layout$.ggraph.orig_index]
    })
    if (length(extra_data) > 0) {
      layout <- cbind(
        layout,
        data_frame0(!!!extra_data)
      )
    }
    attr(layout, 'type_ggraph') <- 'node_ggraph'
    layout
  }
}

#' Manually specify a layout for layout_tbl_graph
#'
#' This layout function lets you pass the node positions in manually. The
#' supplied positions must match the order of the nodes in the tbl_graph
#'
#' @param graph An `tbl_graph` object
#'
#' @param x,y Expressions with the x and y positions of the nodes
#'
#' @param circular Ignored
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph.
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom rlang enquo eval_tidy quo_is_symbol
#'
layout_tbl_graph_manual <- function(graph, x, y, circular) {
  if (isTRUE(circular)) {
    cli::cli_warn('{.arg circular} argument ignored for manual layout')
  }
  x <- enquo(x)
  y <- enquo(y)
  nodes <- data_frame0(
    x = eval_tidy(x, .N()),
    y = eval_tidy(y, .N()),
    circular = FALSE
  )
  nodes <- combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
  nodes
}

#' A theme tuned for graph visualizations
#'
#' When plotting graphs, networks, and trees the coordinate values are often of
#' no importance and axes are thus a distraction. `ggraph` comes with a
#' build-in theme that removes redundant elements in order to put focus on the
#' data. Furthermore the default behaviour is to use a narrow font so text takes
#' up less space. Theme colour is defined by a background and foreground colour
#' where the background defines the colour of the whole graphics area and the
#' foreground defines the colour of the strip and border. By default strip and
#' border is turned off as it is an unnecessary element unless facetting is
#' used. To add a foreground colour to a plot that is already using
#' `theme_graph` the `th_foreground` helper is provided. In order to
#' use this appearance as default use the `set_graph_style` function. An
#' added benefit of this is that it also changes the default text-related values
#' in the different geoms for a completely coherent look.
#' `unset_graph_style` can be used to revert the defaults back to their
#' default settings (that is, they are not necessarily reverted back to what
#' they were prior to calling `set_graph_style`). The `th_no_axes()` helper is
#' provided to modify an existing theme so that grid and axes are removed.
#'
#' @param background The colour to use for the background. This theme sets all
#' background elements except for plot.background to `element_blank` so
#' this controls the background for all elements of the plot. Set to `NA`
#' to remove the background (thus making the plot transparent)
#'
#' @param foreground The colour of foreground elements, specifically strip and
#' border. Set to `NA` to remove.
#'
#' @param border Logical. Should border be drawn if a foreground colour is
#' provided?
#'
#' @param family,base_family,title_family,subtitle_family,strip_text_family,caption_family The font to use for the different elements
#'
#' @param base_size,size,text_size,title_size,subtitle_size,strip_text_size,caption_size The size to use for the various text elements. `text_size` will be used as geom defaults
#'
#' @param face,title_face,subtitle_face,strip_text_face,caption_face The fontface to use for the various text elements
#'
#' @param title_margin,subtitle_margin,caption_margin The margin to use between the text elements and the plot area
#'
#' @param text_colour,bg_text_colour,fg_text_colour,title_colour,subtitle_colour,strip_text_colour,caption_colour The colour of the text in the various text elements
#'
#' @param plot_margin The plot margin
#'
#' @export
#'
#' @examples
#' library(tidygraph)
#' graph <- as_tbl_graph(highschool)
#'
#' ggraph(graph) + geom_edge_link() + geom_node_point() + theme_graph()
theme_graph <- function(
  base_family = 'Arial Narrow',
  base_size = 11,
  background = 'white',
  foreground = NULL,
  border = TRUE,
  text_colour = 'black',
  bg_text_colour = text_colour,
  fg_text_colour = text_colour,
  title_family = base_family,
  title_size = 18,
  title_face = 'bold',
  title_margin = 10,
  title_colour = bg_text_colour,
  subtitle_family = base_family,
  subtitle_size = 12,
  subtitle_face = 'plain',
  subtitle_margin = 15,
  subtitle_colour = bg_text_colour,
  strip_text_family = base_family,
  strip_text_size = 10,
  strip_text_face = 'bold',
  strip_text_colour = fg_text_colour,
  caption_family = base_family,
  caption_size = 9,
  caption_face = 'italic',
  caption_margin = 10,
  caption_colour = bg_text_colour,
  plot_margin = margin(30, 30, 30, 30)
) {
  style <- theme_bw(base_size = base_size, base_family = base_family)
  style <- style +
    theme(
      text = element_text(colour = text_colour),
      plot.title = element_text(
        family = title_family,
        size = title_size,
        face = title_face,
        colour = title_colour,
        margin = margin(b = title_margin)
      ),
      plot.subtitle = element_text(
        family = subtitle_family,
        size = subtitle_size,
        face = subtitle_face,
        colour = subtitle_colour,
        margin = margin(b = subtitle_margin)
      ),
      plot.caption = element_text(
        family = caption_family,
        size = caption_size,
        face = caption_face,
        colour = caption_colour,
        margin = margin(t = caption_margin)
      ),
      strip.text = element_text(
        family = strip_text_family,
        size = strip_text_size,
        face = strip_text_face,
        colour = strip_text_colour
      ),

      plot.margin = plot_margin,

      legend.background = element_blank(),
      legend.box.background = element_blank(),
      legend.key = element_blank(),
      panel.background = element_blank(),
      axis.title = element_blank(),
      axis.text = element_blank(),
      axis.line = element_blank(),
      axis.ticks = element_blank(),
      panel.grid = element_blank(),

      strip.background = if (is.null(foreground)) {
        element_blank()
      } else {
        element_rect(fill = foreground, colour = foreground)
      },
      plot.background = if (is.null(background)) {
        element_blank()
      } else {
        element_rect(fill = background, colour = NA)
      },
      panel.border = if (border && !is.null(foreground)) {
        element_rect(fill = NA, colour = foreground)
      } else {
        element_blank()
      }
    )

  style
}
#' @rdname theme_graph
#'
#' @export
th_foreground <- function(
  foreground = 'grey80',
  fg_text_colour = NULL,
  border = FALSE
) {
  th <- theme(
    strip.background = if (is.null(foreground)) {
      element_blank()
    } else {
      element_rect(fill = foreground, colour = foreground)
    },
    panel.border = if (border && !is.null(foreground)) {
      element_rect(fill = NA, colour = foreground)
    } else {
      element_blank()
    }
  )
  if (!is.null(fg_text_colour)) {
    th <- th + theme(strip.text = element_text(colour = fg_text_colour))
  }
  th
}
#' @rdname theme_graph
#'
#' @export
th_no_axes <- function() {
  theme(
    panel.grid = element_blank(),
    #panel.grid.major = element_blank(),
    #panel.grid.major.x = element_blank(),
    #panel.grid.major.y = element_blank(),
    #panel.grid.minor = element_blank(),
    #panel.grid.minor.x = element_blank(),
    #panel.grid.minor.y = element_blank(),
    axis.title = element_blank(),
    #axis.title.x = element_blank(),
    #axis.title.x.bottom = element_blank(),
    #axis.title.x.top = element_blank(),
    #axis.title.y = element_blank(),
    #axis.title.y.left = element_blank(),
    #axis.title.y.right = element_blank(),
    axis.text = element_blank(),
    #axis.text.x = element_blank(),
    #axis.text.x.top = element_blank(),
    #axis.text.x.bottom = element_blank(),
    #axis.text.y = element_blank(),
    #axis.text.y.left = element_blank(),
    #axis.text.y.right = element_blank(),
    axis.ticks = element_blank(),
    #axis.ticks.x = element_blank(),
    #axis.ticks.x.top = element_blank(),
    #axis.ticks.x.bottom = element_blank(),
    #axis.ticks.y = element_blank(),
    #axis.ticks.y.left = element_blank(),
    #axis.ticks.y.right = element_blank(),
    axis.line = element_blank(),
    #axis.line.x = element_blank(),
    #axis.line.x.bottom = element_blank(),
    #axis.line.x.top = element_blank(),
    #axis.line.y = element_blank(),
    #axis.line.y.left = element_blank(),
    #axis.line.y.right = element_blank(),
    NULL
  )
}
#' @rdname theme_graph
#'
#' @param ... Parameters passed on the `theme_graph`
#'
#' @export
set_graph_style <- function(
  family = 'Arial Narrow',
  face = 'plain',
  size = 11,
  text_size = 11,
  text_colour = 'black',
  ...
) {
  style <- theme_graph(
    base_family = family,
    base_size = size,
    text_colour = text_colour,
    ...
  )
  theme_set(style)
  text_size <- text_size / .pt

  update_geom_defaults(
    GeomEdgePath,
    list(
      family = family,
      fontface = face,
      label_size = text_size
    )
  )
  update_geom_defaults(
    GeomText,
    list(
      family = family,
      fontface = face,
      size = text_size
    )
  )
  update_geom_defaults(
    GeomTextRepel,
    list(
      family = family,
      fontface = face,
      size = text_size
    )
  )
  update_geom_defaults(
    GeomLabel,
    list(
      family = family,
      fontface = face,
      size = text_size
    )
  )
  update_geom_defaults(
    GeomLabelRepel,
    list(
      family = family,
      fontface = face,
      size = text_size
    )
  )
}
#' @rdname theme_graph
#'
#' @export
unset_graph_style <- function() {
  style <- theme_gray()
  theme_set(style)

  update_geom_defaults(
    GeomEdgePath,
    list(
      family = '',
      fontface = 1,
      label_size = 3.88
    )
  )
  update_geom_defaults(
    GeomText,
    list(
      family = '',
      fontface = 1,
      size = 3.88
    )
  )
  update_geom_defaults(
    GeomTextRepel,
    list(
      family = '',
      fontface = 1,
      size = 3.88
    )
  )
  update_geom_defaults(
    GeomLabel,
    list(
      family = '',
      fontface = 1,
      size = 3.88
    )
  )
  update_geom_defaults(
    GeomLabelRepel,
    list(
      family = '',
      fontface = 1,
      size = 3.88
    )
  )
  update_geom_defaults(
    GeomAxisHive,
    list(
      family = '',
      fontface = 1,
      size = 3.88
    )
  )
}

#' Place nodes on a diagonal
#'
#' This layout puts all nodes on a diagonal, thus preparing the layout for use
#' with [geom_edge_point()] resulting in a matrix layout. While matrix
#' layouts excel in scalability, the interpretation of the visual is very
#' dependent on the sorting of the nodes. Different sorting algorithms have been
#' implemented in `tidygraph` and these can be used directly. Behrisch
#' *et al.* (2016) have provided a nice overview of some of the different
#' sorting algorithms and what insight they might bring, along with a rundown of
#' different patterns to look out for.
#'
#' @param graph An `tbl_graph` object
#'
#' @param circular Ignored
#'
#' @param sort.by An expression providing the sorting of the nodes. If `NULL`
#' the nodes will be ordered by their index in the graph.
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom igraph gorder
#' @importFrom rlang enquo eval_tidy
#'
#' @references
#' Behrisch, M., Bach, B., Riche, N. H., Schreck, T., Fekete, J.-D. (2016).
#' *Matrix Reordering Methods for Table and Network Visualization*.
#' Computer Graphics Forum, 35: 693–716. \doi{10.1111/cgf.12935}
#'
layout_tbl_graph_matrix <- function(graph, circular = FALSE, sort.by = NULL) {
  sort.by <- enquo(sort.by)
  sort.by <- eval_tidy(sort.by, .N())
  if (!is.null(sort.by)) {
    pos <- order(order(sort.by))
  } else {
    pos <- seq_len(gorder(graph))
  }
  nodes <- data_frame0(x = pos, y = pos, circular = FALSE)
  nodes <- combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
  nodes
}

#' Calculate nodes as fractal circle buds
#'
#' The cactustree layout is a hierarchical layout optimised for use with
#' hierarchical edge bundling ([geom_conn_bundle()]). It is a fractal layout
#' that places node children as circles on the periphery of their parent circle,
#' each circle scaled by the total weight of their children.
#'
#' @note
#' cactustree is a layout intended for trees, that is, graphs where nodes
#' only have one parent and zero or more children. If the provided graph does
#' not fit this format an attempt to convert it to such a format will be made.
#'
#' @param graph An `tbl_graph` object
#'
#' @param direction The direction of the tree in the graph. `'out'` (default)
#' means that parents point towards their children, while `'in'` means that
#' children point towards their parent.
#'
#' @param weight An optional node variable to use as weight. If `NULL` all leaf
#' nodes will be assigned a weight of `1`.
#'
#' @param scale_factor A scaling factor for the circles in the layout. The
#' radius will be calculated as `total_weight ^ scale_factor` with `total_weight`
#' being the weight of the node and all it's children.
#'
#' @param overlap How much is the center of child nodes offset from the
#' periphery of their parent as a proportion of their own radius.
#'
#' @param upright Logical. Should the children of the root only be distributed
#' around the top half of the root or all the way around.
#'
#' @param circular Logical. Should the layout be transformed to a circular
#' representation. Ignored.
#'
#' @return A data.frame with the columns `x`, `y`, `r`, `leaf`, `depth`,
#' `circular` as well as any information stored as node variables in the
#' tbl_graph object.
#'
#' @references
#' Dang, T., Forbes, A. (2017). *CactusTree: A Tree Drawing Approach
#' for Hierarchical Edge Bundling*. 2017 IEEE Pacific Visualization Symposium,
#' 210-214. https://doi.org/10.1109/PACIFICVIS.2017.8031596
#'
#' @family layout_tbl_graph_*
#'
layout_tbl_graph_cactustree <- function(
  graph,
  direction = "out",
  weight = NULL,
  scale_factor = 0.75,
  overlap = 0.5,
  upright = FALSE,
  circular = FALSE
) {
  weight <- enquo(weight)
  weight <- eval_tidy(weight, .N())
  if (is.null(weight)) {
    weight <- as.numeric(degree(graph, mode = direction) == 0)
  }
  hierarchy <- tree_to_hierarchy(graph, direction, NULL, weight, NULL)
  layout <- cactusTree(
    as.integer(hierarchy$parent),
    as.integer(hierarchy$order),
    as.numeric(hierarchy$weight),
    as.numeric(scale_factor),
    as.numeric(overlap),
    as.logical(upright)
  )[-1, ]
  nodes <- data_frame0(
    x = layout[, 1],
    y = layout[, 2],
    r = layout[, 3],
    circular = FALSE,
    leaf = degree(graph, mode = direction) == 0,
    depth = node_depth(graph, mode = direction)
  )
  nodes <- combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
  attr(nodes, 'graph') <- add_direction(graph, nodes)
  nodes
}

#' Draw edges as elbows
#'
#' This geom draws edges as an angle in the same manner as known from classic
#' dendrogram plots of hierarchical clustering results. In case a circular
#' transformation has been applied the first line segment will be drawn as an
#' arc as expected. This geom is only applicable to layouts that return a
#' direction for the edges (currently [layout_tbl_graph_dendrogram()],
#' [layout_tbl_graph_partition()] and
#' [layout_tbl_graph_igraph()] with the `"tree"` algorithm).
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_elbow` and `geom_edge_elbow0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - **circular**
#' - **direction**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_elbow2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **group**
#' - **circular**
#' - **direction**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_elbow` and `geom_edge_elbow2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#' @param strength How bend the elbow should be. 1 will give a right angle,
#' while `0` will give a straight line. Ignored for circular layouts
#' @inheritParams geom_edge_diagonal
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' irisDen <- hclust(dist(iris[1:4], method = 'euclidean'), method = 'ward.D2') %>%
#'   as_tbl_graph() %>%
#'   mutate(class = sample(letters[1:3], n(), TRUE)) %>%
#'   activate(edges) %>%
#'   mutate(class = sample(letters[1:3], n(), TRUE))
#'
#' ggraph(irisDen, 'dendrogram', circular = TRUE) +
#'   geom_edge_elbow(aes(alpha = after_stat(index)))
#'
#' ggraph(irisDen, 'dendrogram') +
#'   geom_edge_elbow2(aes(colour = node.class))
#'
#' ggraph(irisDen, 'dendrogram', height = height) +
#'   geom_edge_elbow0(aes(colour = class))
#' @rdname geom_edge_elbow
#' @name geom_edge_elbow
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce radial_trans
#' @export
StatEdgeElbow <- ggproto(
  'StatEdgeElbow',
  Stat,
  compute_panel = function(
    data,
    scales,
    flipped = FALSE,
    n = 100,
    strength = 1
  ) {
    data$group <- make_unique(data$group)
    if (data$circular[1] && n %% 2 == 1) {
      n <- n + 1
    }
    if (!data$circular[1] && n %% 2 == 0) {
      n <- n + 1
    }
    index <- seq(0, 1, length.out = n)
    if (any(data$circular)) {
      if (strength != 1) {
        cli::cli_warn('{.arg strength} is ignored for circular elbow edges')
      }
      circ_id <- which(data$circular)
      data_circ <- data[circ_id, ]
      radial <- radial_trans(c(0, 1), c(2 * pi, 0), pad = 0, offset = 0)
      start <- atan2(data_circ$y, data_circ$x)
      radii_start <- sqrt(data_circ$x^2 + data_circ$y^2)
      radii_end <- sqrt(data_circ$xend^2 + data_circ$yend^2)
      angel_diff <- (data_circ$x *
        data_circ$xend +
        data_circ$y * data_circ$yend) /
        (radii_start * radii_end)
      angel_diff[is.nan(angel_diff)] <- 0
      angel_diff <- suppressWarnings(acos(angel_diff))
      angel_diff[is.nan(angel_diff)] <- 0
      end <- start +
        ifelse(data_circ$direction == 'left', -angel_diff, angel_diff)
      angles <- unlist(Map(seq, from = start, to = end, length.out = n / 2))
      radii <- rep(sqrt(data$y[circ_id]^2 + data$x[circ_id]^2), each = n / 2)
      path_circ <- radial$transform(r = radii, a = angles)
      path_circ$.orig_index <- rep(circ_id, each = n / 2)
      path_circ$group <- rep(data_circ$group, each = n / 2)
      path_circ$index <- rep(index[seq_len(n / 2)], length(circ_id))
      radii_rel <- radii_start / radii_end
      elbow_x <- data_circ$xend * radii_rel
      elbow_y <- data_circ$yend * radii_rel
      elbow_x <- unlist(Map(
        seq,
        from = elbow_x,
        to = data_circ$xend,
        length.out = n / 2
      ))
      elbow_y <- unlist(Map(
        seq,
        from = elbow_y,
        to = data_circ$yend,
        length.out = n / 2
      ))
      path_circ <- vec_rbind(
        path_circ,
        data_frame0(
          x = elbow_x,
          y = elbow_y,
          .orig_index = path_circ$.orig_index,
          group = path_circ$group,
          index = rep(
            index[seq_len(n / 2) + n / 2],
            length(circ_id)
          )
        )
      )
      path_circ <- cbind(
        path_circ,
        data[
          path_circ$.orig_index,
          !names(data) %in%
            c('x', 'y', 'xend', 'yend', 'group')
        ]
      )
      path_circ$.orig_index <- NULL
    }
    if (any(!data$circular)) {
      path_lin <- lapply(which(!data$circular), function(i) {
        if (flipped) {
          path <- data_frame0(
            x = approx(
              c(
                data$x[i],
                data$xend[i] + (data$x[i] - data$xend[i]) * strength,
                data$xend[i]
              ),
              n = n
            )$y,
            y = approx(c(data$y[i], data$yend[i], data$yend[i]), n = n)$y,
            group = data$group[i],
            index = index
          )
        } else {
          path <- data_frame0(
            x = approx(c(data$x[i], data$xend[i], data$xend[i]), n = n)$y,
            y = approx(
              c(
                data$y[i],
                data$yend[i] + (data$y[i] - data$yend[i]) * strength,
                data$yend[i]
              ),
              n = n
            )$y,
            group = data$group[i],
            index = index
          )
        }
        cbind(
          path,
          data[
            rep(i, nrow(path)),
            !names(data) %in%
              c('x', 'y', 'xend', 'yend', 'group')
          ]
        )
      })
      path_lin <- vec_rbind(!!!path_lin)

      if (any(data$circular)) {
        paths <- vec_rbind(path_lin, path_circ)
      } else {
        paths <- path_lin
      }
    } else {
      paths <- path_circ
    }
    paths[order(paths$group), ]
  },
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data <- remove_loop(data)
    if (nrow(data) == 0) {
      return(data)
    }
    data
  },
  default_aes = aes(filter = TRUE),
  required_aes = c('x', 'y', 'xend', 'yend', 'circular', 'direction')
)
#' @rdname geom_edge_elbow
#'
#' @export
geom_edge_elbow <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  flipped = FALSE,
  n = 100,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      circular = circular,
      direction = direction,
      group = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeElbow,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = FALSE,
        flipped = flipped,
        strength = strength,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
StatEdgeElbow2 <- ggproto(
  'StatEdgeElbow2',
  Stat,
  compute_panel = function(
    data,
    scales,
    flipped = FALSE,
    n = 100,
    strength = 1
  ) {
    pos_cols <- c('x', 'y', 'group', 'circular', 'direction', 'PANEL')
    data <- data[order(data$group), ]
    pos_data <- cbind(
      data[c(TRUE, FALSE), pos_cols],
      data[
        c(FALSE, TRUE),
        c('x', 'y')
      ]
    )
    pos_data$group <- make_unique(pos_data$group)
    names(pos_data) <- c(pos_cols, 'xend', 'yend')
    new_data <- StatEdgeElbow$compute_panel(
      pos_data,
      scales,
      flipped,
      n,
      strength
    )
    extra_cols <- !names(data) %in% pos_cols
    index <- match(pos_data$group, new_data$group)
    index <- as.vector(matrix(c(index, index + 1), nrow = 2, byrow = T))
    new_data$.interp <- TRUE
    new_data$.interp[index] <- FALSE
    if (sum(extra_cols) != 0) {
      for (i in names(data)[extra_cols]) {
        new_data[[i]] <- NA
        new_data[[i]][index] <- data[[i]]
      }
    }
    new_data
  },
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data <- remove_loop2(data)
    if (nrow(data) == 0) {
      return(data)
    }
    data
  },
  default_aes = aes(filter = TRUE),
  required_aes = c('x', 'y', 'group', 'circular', 'direction')
)
#' @rdname geom_edge_elbow
#'
#' @export
geom_edge_elbow2 <- function(
  mapping = NULL,
  data = get_edges('long'),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  flipped = FALSE,
  n = 100,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      group = edge.id,
      circular = circular,
      direction = direction
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeElbow2,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = TRUE,
        flipped = flipped,
        strength = strength,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
StatEdgeElbow0 <- ggproto(
  'StatEdgeElbow0',
  Stat,
  compute_panel = function(data, scales, flipped = FALSE, strength = 1) {
    data$group <- make_unique(data$group)
    if (any(data$circular)) {
      if (strength != 1) {
        cli::cli_warn('{.arg strength} is ignored for circular elbow edges')
      }
      circ_id <- which(data$circular)
      data_circ <- data[circ_id, ]
      radial <- radial_trans(c(0, 1), c(2 * pi, 0), pad = 0, offset = 0)
      start <- atan2(data_circ$y, data_circ$x)
      angel_diff <- (data_circ$x *
        data_circ$xend +
        data_circ$y * data_circ$yend) /
        (sqrt(data_circ$x^2 + data_circ$y^2) *
          sqrt(data_circ$xend^2 + data_circ$yend^2))
      angel_diff[is.nan(angel_diff)] <- 0
      angel_diff <- suppressWarnings(acos(angel_diff))
      angel_diff[is.nan(angel_diff)] <- 0
      end <- start +
        ifelse(data_circ$direction == 'left', -angel_diff, angel_diff)
      angles <- unlist(Map(seq, from = start, to = end, length.out = 50))
      radii <- rep(sqrt(data$y[circ_id]^2 + data$x[circ_id]^2), each = 50)
      path_circ <- radial$transform(r = radii, a = angles)
      path_circ$.orig_index <- rep(circ_id, each = 50)
      path_circ$group <- rep(data_circ$group, each = 50)
      path_circ <- vec_rbind(
        path_circ,
        data_frame0(
          x = data$xend[circ_id],
          y = data$yend[circ_id],
          .orig_index = circ_id,
          group = data_circ$group
        )
      )
      path_circ <- cbind(
        path_circ,
        data[
          path_circ$.orig_index,
          !names(data) %in%
            c('x', 'y', 'xend', 'yend', 'group')
        ]
      )
      path_circ$.orig_index <- NULL
    }
    if (any(!data$circular)) {
      path_lin <- lapply(which(!data$circular), function(i) {
        if (flipped) {
          path <- data_frame0(
            x = c(
              data$x[i],
              data$xend[i] + (data$x[i] - data$xend[i]) * strength,
              data$xend[i]
            ),
            y = c(data$y[i], data$yend[i], data$yend[i]),
            group = data$group
          )
        } else {
          path <- data_frame0(
            x = c(data$x[i], data$xend[i], data$xend[i]),
            y = c(
              data$y[i],
              data$yend[i] + (data$y[i] - data$yend[i]) * strength,
              data$yend[i]
            ),
            group = data$group[i]
          )
        }
        cbind(
          path,
          data[
            rep(i, nrow(path)),
            !names(data) %in%
              c('x', 'y', 'xend', 'yend', 'group')
          ]
        )
      })
      path_lin <- vec_rbind(!!!path_lin)

      if (any(data$circular)) {
        paths <- vec_rbind(path_lin, path_circ)
      } else {
        paths <- path_lin
      }
    } else {
      paths <- path_circ
    }
    paths[order(paths$group), ]
  },
  setup_data = function(data, params) {
    StatEdgeElbow$setup_data(data, params)
  },
  default_aes = aes(filter = TRUE),
  required_aes = c('x', 'y', 'xend', 'yend', 'circular', 'direction')
)
#' @rdname geom_edge_elbow
#'
#' @export
geom_edge_elbow0 <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  arrow = NULL,
  flipped = FALSE,
  lineend = 'butt',
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      circular = circular,
      direction = direction
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeElbow0,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        interpolate = FALSE,
        flipped = flipped,
        ...
      )
    )
  )
}

#' Draw edges as diagonals
#'
#' This geom draws edges as cubic bezier curves with the control points
#' positioned along the elbow edge. It has the appearance of a softened elbow
#' edge with the hard angle substituted by a tapered bend.
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_bend` and `geom_edge_bend0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - **circular**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_bend2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **group**
#' - **circular**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_bend` and `geom_edge_bend2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_diagonal
#'
#' @param strength The strength of the curvature of the bend. `0` will
#' result in a straight line while `1` will give a strong arc.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' gr <- create_tree(20, 4) %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE)) %>%
#'   activate(edges) %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE))
#'
#' ggraph(gr, 'tree') +
#'   geom_edge_bend(aes(alpha = after_stat(index)))
#'
#' ggraph(gr, 'tree') +
#'   geom_edge_bend2(aes(colour = node.class))
#'
#' ggraph(gr, 'tree') +
#'   geom_edge_bend0(aes(colour = class))
#' @rdname geom_edge_bend
#' @name geom_edge_bend
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier
#' @export
StatEdgeBend <- ggproto(
  'StatEdgeBend',
  StatBezier,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data <- remove_loop(data)
    if (nrow(data) == 0) {
      return(data)
    }
    data$group <- make_unique(data$group)
    data2 <- data
    data2$x <- data2$xend
    data2$y <- data2$yend
    create_bend(data, data2, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'circular'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'flipped', 'n', 'strength')
)
#' @rdname geom_edge_bend
#'
#' @export
geom_edge_bend <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  flipped = FALSE,
  n = 100,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      circular = circular,
      group = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeBend,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = FALSE,
        flipped = flipped,
        strength = strength,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier2
#' @export
StatEdgeBend2 <- ggproto(
  'StatEdgeBend2',
  StatBezier2,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data <- remove_loop2(data)
    if (nrow(data) == 0) {
      return(data)
    }
    data <- data[order(data$group), ]
    data2 <- data[c(FALSE, TRUE), ]
    data <- data[c(TRUE, FALSE), ]
    create_bend(data, data2, params)
  },
  required_aes = c('x', 'y', 'group', 'circular'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'flipped', 'n', 'strength')
)
#' @rdname geom_edge_bend
#'
#' @export
geom_edge_bend2 <- function(
  mapping = NULL,
  data = get_edges('long'),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  flipped = FALSE,
  n = 100,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      group = edge.id,
      circular = circular
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeBend2,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = TRUE,
        flipped = flipped,
        strength = strength,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier0
#' @export
StatEdgeBend0 <- ggproto(
  'StatEdgeBend0',
  StatBezier0,
  setup_data = function(data, params) {
    StatEdgeBend$setup_data(data, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'circular'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'flipped', 'strength')
)
#' @rdname geom_edge_bend
#'
#' @export
geom_edge_bend0 <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  flipped = FALSE,
  lineend = 'butt',
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      circular = circular
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeBend0,
    geom = GeomEdgeBezier,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        strength = strength,
        flipped = flipped,
        ...
      )
    )
  )
}

create_bend <- function(from, to, params) {
  bezier_start <- seq(1, by = 4, length.out = nrow(from))
  from$index <- bezier_start
  to$index <- bezier_start + 3
  data2 <- from
  data3 <- to
  data2$index <- bezier_start + 1
  data3$index <- bezier_start + 2
  if (any(from$circular)) {
    r0 <- sqrt(from$x[from$circular]^2 + from$y[from$circular]^2)
    r1 <- sqrt(to$x[to$circular]^2 + to$y[to$circular]^2)
    root <- r0 == 0 | r1 == 0

    dotprod <- from$x * to$x + from$y * to$y

    crossing <- r0 * r0 / dotprod
    cross_x <- to$x * crossing
    cross_y <- to$y * crossing

    data2$x[from$circular] <- from$x + (cross_x - from$x) * params$strength
    data2$y[from$circular] <- from$y + (cross_y - from$y) * params$strength
    data3$x[from$circular] <- to$x + (cross_x - to$x) * params$strength
    data3$y[from$circular] <- to$y + (cross_y - to$y) * params$strength

    data2$x[root] <- from$x[root]
    data2$y[root] <- from$y[root]
    data3$x[root] <- to$x[root]
    data3$y[root] <- to$y[root]
  }
  if (any(!from$circular)) {
    if (params$flipped) {
      h_diff <- from$x[!from$circular] - to$x[!from$circular]
      w_diff <- from$y[!from$circular] - to$y[!from$circular]
      data2$y[!from$circular] <- from$y[!from$circular] -
        w_diff * params$strength
      data3$x[!from$circular] <- to$x[!from$circular] + h_diff * params$strength
    } else {
      h_diff <- from$y[!from$circular] - to$y[!from$circular]
      w_diff <- from$x[!from$circular] - to$x[!from$circular]
      data2$x[!from$circular] <- from$x[!from$circular] -
        w_diff * params$strength
      data3$y[!from$circular] <- to$y[!from$circular] + h_diff * params$strength
    }
  }
  data <- vec_rbind(from, data2, data3, to)
  data[order(data$index), names(data) != 'index']
}

#' Calculate nodes as areas dividing their parent
#'
#' The partition layout is a way to show hierarchical data in the same way as
#' [layout_tbl_graph_treemap()]. Instead of subdividing the parent area
#' the partition layout shows the division of a nodes children next to the area
#' of the node itself. As such the node positions will be very reminiscent of
#' a reingold-tilford tree layout but by plotting nodes as areas it better
#' communicate the total weight of a node by summing up all its children.
#' Often partition layouts are called icicle plots or sunburst diagrams (in case
#' a radial transform is applied).
#'
#' @note
#' partition is a layout intended for trees, that is, graphs where nodes
#' only have one parent and zero or more children. If the provided graph does
#' not fit this format an attempt to convert it to such a format will be made.
#'
#' @param graph An `tbl_graph` object
#'
#' @param weight An optional node variable to use as weight. Will only affect
#' the weight of leaf nodes as the weight of non-leaf nodes are derived from
#' their children.
#'
#' @param circular Logical. Should the layout be transformed to a circular
#' representation. If `TRUE` the resulting layout will be a sunburst
#' diagram.
#'
#' @param height An optional node variable to use as height. If `NULL`
#' all nodes will be given a height of 1.
#'
#' @param sort.by The name of a node variable to sort the nodes by.
#'
#' @param direction The direction of the tree in the graph. `'out'` (default)
#' means that parents point towards their children, while `'in'` means that
#' children point towards their parent.
#'
#' @param const.area Logical. Should 'height' be scaled for area proportionality
#' when using `circular = TRUE`. Defaults to `TRUE`.
#'
#' @param offset If `circular = TRUE`, where should it begin. Defaults to
#' `pi/2` which is equivalent to 12 o'clock.
#'
#' @return If `circular = FALSE` A data.frame with the columns `x`,
#' `y`, `width`, `height`, `leaf`,
#' `depth`, `circular` as well as any information stored as node
#' variables in the tbl_graph object.
#' If `circular = TRUE` A data.frame with the columns `x`, `y`,
#' `r0`, `r`, `start`, `end`, `leaf`,
#' `depth`, `circular` as well as any information stored as node
#' variables in the tbl_graph object.
#'
#' @references
#' Kruskal, J. B., Landwehr, J. M. (1983). *Icicle Plots: Better Displays
#' for Hierarchical Clustering*. American Statistician Vol 37(2), 162-168.
#' https://doi.org/10.2307/2685881
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom ggforce radial_trans
#'
layout_tbl_graph_partition <- function(
  graph,
  weight = NULL,
  circular = FALSE,
  height = NULL,
  sort.by = NULL,
  direction = 'out',
  offset = pi / 2,
  const.area = TRUE
) {
  weight <- enquo(weight)
  weight <- eval_tidy(weight, .N())
  height <- enquo(height)
  height <- eval_tidy(height, .N())
  sort.by <- enquo(sort.by)
  sort.by <- eval_tidy(sort.by, .N())
  hierarchy <- tree_to_hierarchy(graph, direction, sort.by, weight, height)
  layout <- partitionTree(
    as.integer(hierarchy$parent),
    as.integer(hierarchy$order),
    as.numeric(hierarchy$weight),
    as.numeric(hierarchy$height)
  )[-1, ]
  if (circular) {
    if (const.area) {
      y0 <- sqrt(layout[, 2])
      y1 <- sqrt(layout[, 2] + layout[, 4])
      layout[, 2] <- y0
      layout[, 4] <- y1 - y0
    }
    width_range <- c(0, max(rowSums(layout[, c(1, 3)])))
    radial <- radial_trans(
      r.range = c(0, 1),
      a.range = width_range,
      offset = offset,
      pad = 0
    )
    coords <- radial$transform(
      layout[, 2] + layout[, 4] / 2,
      layout[, 1] + layout[, 3] / 2
    )
    nodes <- data_frame0(
      x = coords$x,
      y = coords$y,
      r0 = layout[, 2],
      r = layout[, 2] + layout[, 4],
      start = 2 * pi * layout[, 1] / width_range[2],
      end = 2 * pi * (layout[, 1] + layout[, 3]) / width_range[2],
      circular = TRUE
    )
    nodes$x[1] <- 0
    nodes$y[1] <- 0
  } else {
    nodes <- data_frame0(
      x = layout[, 1] + layout[, 3] / 2,
      y = layout[, 2] + layout[, 4] / 2,
      width = layout[, 3],
      height = layout[, 4],
      circular = FALSE
    )
  }
  nodes$leaf <- degree(graph, mode = direction) == 0
  nodes$depth <- node_depth(graph, mode = direction)
  nodes <- combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
  attr(nodes, 'graph') <- add_direction(graph, nodes)
  nodes
}

#' Apply a dendrogram layout to layout_tbl_graph
#'
#' This layout mimics the [igraph::layout_as_tree()] algorithm
#' supplied by igraph, but puts all leaves at 0 and builds it up from there,
#' instead of starting from the root and building it from there. The height of
#' branch points are related to the maximum distance to an edge from the branch
#' node, or read from a node variable.
#'
#' @note This function is not intended to be used directly but by setting
#' `layout = 'dendrogram'` in [create_layout()]
#'
#' @param graph A `tbl_graph` object
#'
#' @param circular Logical. Should the layout be transformed to a circular
#' representation. Defaults to `FALSE`.
#'
#' @param offset If `circular = TRUE`, where should it begin. Defaults to
#' `pi/2` which is equivalent to 12 o'clock.
#'
#' @param height The node variable holding the height of each node in the
#' dendrogram. If `NULL` it will be calculated as the maximal distance to a
#' leaf.
#'
#' @param length An edge parameter giving the length of each edge. The node
#' height will be calculated from the maximal length to the root node (ignored
#' if `height` does not evaluate to `NULL`)
#'
#' @param repel Should leafs repel each other relative to the height of their
#' common ancestor. Will emphasize clusters
#'
#' @param ratio The strength of repulsion if `repel = TRUE`. Higher values will
#' give more defined clusters
#'
#' @param direction The direction to the leaves. Defaults to 'out'
#'
#' @return A data.frame with the columns `x`, `y`, `circular`, `depth` and
#' `leaf` as well as any information stored as node variables on the
#' tbl_graph
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom igraph gorder degree adjacent_vertices distances neighbors
#' @importFrom tidygraph node_is_root
#' @importFrom rlang enquo eval_tidy
#'
layout_tbl_graph_dendrogram <- function(
  graph,
  circular = FALSE,
  offset = pi / 2,
  height = NULL,
  length = NULL,
  repel = FALSE,
  ratio = 1,
  direction = 'out'
) {
  height <- enquo(height)
  length <- enquo(length)
  reverse_dir <- if (direction == 'out') 'in' else 'out'
  if (quo_is_null(height)) {
    if (quo_is_null(length)) {
      height <- NA
    } else {
      length <- eval_tidy(length, .E())
      full_lengths <- distances(
        graph,
        to = node_is_root(),
        weights = length,
        mode = reverse_dir
      )
      full_lengths[is.infinite(full_lengths)] <- 0
      height <- unname(apply(full_lengths, 1, max))
      height <- abs(height - max(height))
    }
  } else {
    height <- eval_tidy(height, .N())
  }
  nodes <- data_frame0(
    x = rep(NA_real_, gorder(graph)),
    y = height,
    leaf = degree(graph, mode = direction) == 0
  )
  if (all(is.na(nodes$y))) {
    nodes$y <- vapply(
      seq_len(gorder(graph)),
      function(i) {
        max(
          bfs(
            graph,
            i,
            direction,
            unreachable = FALSE,
            order = FALSE,
            dist = TRUE
          )$dist,
          na.rm = TRUE
        )
      },
      numeric(1)
    )
  }
  if (repel) {
    pad <- min(nodes$y[nodes$y != 0]) / 2
  } else {
    pad <- 0
  }
  startnode <- which(degree(graph, mode = reverse_dir) == 0)
  if (length(startnode) < 1) {
    cli::cli_abort('The graph doesn\'t contain a root node')
  }
  neighbors <- lapply(
    adjacent_vertices(graph, seq_len(gorder(graph)), direction),
    as.integer
  )
  nodes$x <- dendrogram_spread(
    neighbors,
    as.integer(startnode),
    as.numeric(nodes$y),
    as.logical(nodes$leaf),
    as.logical(repel),
    as.numeric(pad),
    as.numeric(ratio)
  )
  graph <- add_direction(graph, nodes)
  if (circular) {
    radial <- radial_trans(
      r.range = rev(range(nodes$y)),
      a.range = range(nodes$x),
      offset = offset
    )
    coords <- radial$transform(nodes$y, nodes$x)
    nodes$x <- coords$x
    nodes$y <- coords$y
  }
  nodes <- combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
  nodes$circular <- circular
  attr(nodes, 'graph') <- graph
  nodes
}

textAlongGrob <- function(
  label,
  x = unit(0.5, 'npc'),
  y = unit(0.5, 'npc'),
  just = 'centre',
  hjust = NULL,
  vjust = NULL,
  rot = 0,
  check.overlap = FALSE,
  rot.type = 'rot',
  x0 = 0,
  y0 = 0,
  x1 = 0,
  y1 = 0,
  force.rot = TRUE,
  dodge = NULL,
  push = NULL,
  default.units = 'npc',
  name = NULL,
  gp = gpar(),
  vp = NULL
) {
  if (!is.unit(x)) {
    x <- unit(x, default.units)
  }
  if (!is.unit(y)) {
    y <- unit(y, default.units)
  }
  if (rot.type == 'rot') {
    textGrob(
      label = label,
      x = x,
      y = y,
      just = just,
      hjust = hjust,
      vjust = vjust,
      rot = rot,
      check.overlap = check.overlap,
      default.units = default.units,
      name = name,
      gp = gp,
      vp = vp
    )
  } else {
    if (!rot.type %in% c('along', 'across')) {
      cli::cli_abort(
        '{.arg rot.type} must be either {.val rot}, {.val along), or {.val across}'
      )
    }
    if (!is.unit(x0)) {
      x0 <- unit(x0, default.units)
    }
    if (!is.unit(y0)) {
      y0 <- unit(y0, default.units)
    }
    if (!is.unit(x1)) {
      x1 <- unit(x1, default.units)
    }
    if (!is.unit(y1)) {
      y1 <- unit(y1, default.units)
    }
    grob(
      label = label,
      x = x,
      y = y,
      just = just,
      hjust = hjust,
      vjust = vjust,
      rot.type = rot.type,
      x0 = x0,
      y0 = y0,
      x1 = x1,
      y1 = y1,
      force.rot = force.rot,
      dodge = dodge,
      push = push,
      check.overlap = check.overlap,
      name = name,
      gp = gp,
      vp = vp,
      cl = 'textalong'
    )
  }
}
#' Text angled according to line
#'
#' This function takes care of recalculating the angle of the text as the device
#' size changes
#'
#' @importFrom grid makeContent
#' @export
#' @keywords internal
makeContent.textalong <- function(x) {
  x0 <- convertX(x$x0, 'mm', TRUE)
  y0 <- convertY(x$y0, 'mm', TRUE)
  x1 <- convertX(x$x1, 'mm', TRUE)
  y1 <- convertY(x$y1, 'mm', TRUE)
  xpos <- convertX(x$x, 'mm', TRUE)
  ypos <- convertY(x$y, 'mm', TRUE)
  angle <- edge_angle(x0, y0, x1, y1)
  if (x$rot.type == 'across') {
    angle <- angle - 90
  }
  if (!is.null(x$dodge)) {
    dodge <- convertHeight(x$dodge, 'mm', TRUE)
    dodge_angle <- (angle + 90) / 360 * 2 * pi
    dodge_x <- cos(dodge_angle) * dodge
    dodge_y <- sin(dodge_angle) * dodge
    xpos <- xpos + dodge_x
    ypos <- ypos + dodge_y
  }
  if (!is.null(x$push)) {
    push <- convertHeight(x$push, 'mm', TRUE)
    push_angle <- angle / 360 * 2 * pi
    push_x <- cos(push_angle) * push
    push_y <- sin(push_angle) * push
    xpos <- xpos + push_x
    ypos <- ypos + push_y
  }
  if (x$force.rot) {
    fix <- angle > 90 & angle < 270
    angle[fix] <- angle[fix] + 180
  }
  grob(
    label = x$label,
    x = unit(xpos, 'mm'),
    y = unit(ypos, 'mm'),
    just = x$just,
    hjust = x$hjust,
    vjust = x$vjust,
    rot = angle,
    check.overlap = x$check.overlap,
    name = x$name,
    gp = x$gp,
    vp = x$vp,
    cl = 'text'
  )
}

#' @keywords internal
'_PACKAGE'

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
#' @import ggplot2 tidygraph rlang vctrs
#' @importFrom memoise memoise
#' @importFrom lifecycle deprecated
#' @useDynLib ggraph, .registration = TRUE
## usethis namespace: end
NULL

#' Draw multi edges as parallel lines
#'
#' This geom draws multi edges as parallel lines. The edges are first sorted by
#' direction and then shifted a fixed amount so that all edges are visible.
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_parallel` and `geom_edge_parallel0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - **from**
#' - **to**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_parallel2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **group**
#' - **from**
#' - **to**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_parallel` and `geom_edge_parallel2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#'
#' @param sep The separation between parallel edges, given as a [grid::unit()]
#'
#' @author David Schoch and Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('bull') %>%
#'   convert(to_directed) %>%
#'   bind_edges(data.frame(from = c(1, 2, 2, 3), to = c(2, 1, 3, 2))) %E>%
#'   mutate(class = sample(letters[1:3], 9, TRUE)) %N>%
#'   mutate(class = sample(c('x', 'y'), 5, TRUE))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_parallel(aes(alpha = after_stat(index)))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_parallel2(aes(colour = node.class))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_parallel0(aes(colour = class))
#'
#' # Use capping and sep to fine tune the look
#' ggraph(gr, 'stress') +
#'   geom_edge_parallel(start_cap = circle(1), end_cap = circle(1),
#'                      arrow = arrow(length = unit(2, 'mm')), sep = unit(4, 'mm')) +
#'   geom_node_point(size = 12)
#'
#' @rdname geom_edge_parallel
#' @name geom_edge_parallel
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatLink
#' @export
StatEdgeParallel <- ggproto(
  'StatEdgeParallel',
  StatLink,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data <- remove_loop(data)
    if (nrow(data) == 0) {
      return(data)
    }
    data2 <- data
    data2$x <- data2$xend
    data2$y <- data2$yend
    data$.position <- edge_positions(data, data2, params)
    StatLink$setup_data(data, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'from', 'to'),
  default_aes = aes(filter = TRUE),
  extra_params = c("na.rm", "n")
)

#' @rdname geom_edge_parallel
#'
#' @importFrom ggforce StatLink
#' @export
geom_edge_parallel <- function(
  mapping = NULL,
  data = get_edges(),
  position = "identity",
  arrow = NULL,
  sep = unit(2, 'mm'),
  n = 100,
  lineend = "butt",
  linejoin = "round",
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      from = from,
      to = to,
      group = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeParallel,
    geom = GeomEdgeParallelPath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        sep = sep,
        n = n,
        interpolate = FALSE,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier2
#' @export
StatEdgeParallel2 <- ggproto(
  'StatEdgeParallel2',
  StatLink2,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data <- remove_loop2(data)
    if (nrow(data) == 0) {
      return(data)
    }
    data <- data[order(data$group), ]
    data2 <- data[c(FALSE, TRUE), ]
    data1 <- data[c(TRUE, FALSE), ]
    data$.position <- rep(edge_positions(data1, data2), each = 2)
    StatLink2$setup_data(data, params)
  },
  required_aes = c('x', 'y', 'group', 'from', 'to'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n')
)
#' @rdname geom_edge_parallel
#'
#' @export
geom_edge_parallel2 <- function(
  mapping = NULL,
  data = get_edges('long'),
  position = 'identity',
  arrow = NULL,
  sep = unit(2, 'mm'),
  n = 100,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      group = edge.id,
      from = from,
      to = to
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeParallel2,
    geom = GeomEdgeParallelPath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        sep = sep,
        n = n,
        interpolate = TRUE,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier0
#' @export
StatEdgeParallel0 <- ggproto(
  'StatEdgeFan0',
  StatIdentity,
  setup_data = function(data, params) {
    StatEdgeParallel$setup_data(data, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'from', 'to'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm')
)
#' @rdname geom_edge_parallel
#'
#' @export
geom_edge_parallel0 <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  arrow = NULL,
  sep = unit(2, 'mm'),
  lineend = 'butt',
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      from = from,
      to = to
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeParallel0,
    geom = GeomEdgeParallelSegment,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        sep = sep,
        ...
      )
    )
  )
}
#' @importFrom dplyr group_by arrange summarise n ungroup pull
edge_positions <- function(from, to, params) {
  from$.id <- paste(pmin(from$from, to$to), pmax(from$from, to$to), sep = '-')
  from$.orig_ind <- seq_len(nrow(from))
  from %>%
    group_by(.data$PANEL, .data$.id) %>%
    arrange(.data$from) %>%
    mutate(position = seq_len(n()) - 0.5 - n() / 2) %>%
    mutate(position = .data$position * ifelse(.data$from < .data$to, 1, -1)) %>%
    ungroup() %>%
    arrange(.data$.orig_ind) %>%
    pull(.data$position)
}

#' Draw edges as vertical spans
#'
#' This edge geom is mainly intended for use with [fabric][layout_tbl_graph_fabric]
#' layouts. It draws edges as vertical segments with an optional end shape
#' adornment. Due to the special nature of fabric layouts where nodes are not
#' a single point in space but a line, this geom doesn't derive the x position
#' from the location of the terminal nodes, but defaults to using the `edge_x`
#' variable calculated by the fabric layout. If this geom is used with other
#' layouts `x`and `xend` must be given explicitly.
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_span` and `geom_edge_span0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_span2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **group**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_span` and `geom_edge_span2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#'
#' @param end_shape The adornment to put at the ends of the span. The naming
#' follows the conventions of the shape aesthetic in [ggplot2::geom_point()]
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' gr <- play_smallworld(n_dim = 3, dim_size = 3, order = 1, p_rewire = 0.6)
#'
#' # Standard use
#' ggraph(gr, 'fabric', sort.by = node_rank_fabric()) +
#'   geom_node_range(colour = 'grey80') +
#'   geom_edge_span()
#'
#' # Add end shapes
#' ggraph(gr, 'fabric', sort.by = node_rank_fabric()) +
#'   geom_node_range(colour = 'grey80') +
#'   geom_edge_span(end_shape = 'circle')
#'
#' # If the layout include shadow edges these can be styled differently
#' ggraph(gr, 'fabric', sort.by = node_rank_fabric(), shadow.edges = TRUE) +
#'   geom_node_range(colour = 'grey80') +
#'   geom_edge_span(aes(colour = shadow_edge), end_shape = 'square') +
#'   scale_edge_colour_manual(values = c('FALSE' = 'black', 'TRUE' = 'grey'))
#'
#' @rdname geom_edge_span
#' @name geom_edge_span
#'
NULL

#' @rdname geom_edge_span
#'
#' @importFrom ggforce StatLink
#' @export
geom_edge_span <- function(
  mapping = NULL,
  data = get_edges('short'),
  position = 'identity',
  end_shape = NA,
  arrow = NULL,
  n = 100,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(x = edge_x, y = y, xend = edge_x, yend = yend, group = edge.id)
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeLink,
    geom = GeomEdgeSpanPath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = FALSE,
        end_shape = end_shape,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname geom_edge_span
#'
#' @importFrom ggforce StatLink2
#' @export
geom_edge_span2 <- function(
  mapping = NULL,
  data = get_edges('long'),
  position = 'identity',
  end_shape = NA,
  arrow = NULL,
  n = 100,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = edge_x, y = y, group = edge.id))
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeLink2,
    geom = GeomEdgeSpanPath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = TRUE,
        end_shape = end_shape,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname geom_edge_span
#'
#' @importFrom ggforce StatLink2
#' @export
geom_edge_span0 <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  end_shape = NA,
  arrow = NULL,
  lineend = 'butt',
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(x = edge_x, y = y, xend = edge_x, yend = yend)
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatFilter,
    geom = GeomEdgeSpanSegment,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(end_shape = end_shape, arrow = arrow, lineend = lineend, ...)
    )
  )
}

#' Bundle edges along the minimal spanning tree
#'
#' This geom performs edge bundling by letting edges follow the shortest path
#' along the minimal spanning tree of the graph. Due to it's simplicity it is
#' very fast but does enforce a tree-like appearance to the bundling. Adjusting
#' the `max_distortion` and `tension` parameters may alleviate this to some
#' extend.
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_force_minimal` and `geom_edge_force_minimal0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - **edge_id** (should not be overwritten)
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_force_minimal2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **group**
#' - **edge_id** (should not be overwritten)
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_force_minimal` and `geom_edge_force_minimal2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#' @inheritParams geom_edge_bundle_path
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @rdname geom_edge_bundle_minimal
#' @name geom_edge_bundle_minimal
#'
#' @examples
#' ggraph(highschool) +
#'   geom_edge_bundle_minimal()
#'
#' # Allow more edges to bundle
#' ggraph(highschool) +
#'   geom_edge_bundle_minimal(max_distortion = 5, tension = 0.9)
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBspline
#' @export
StatEdgeBundleMinimal <- ggproto(
  "StatEdgeBundleMinimal",
  Stat,
  setup_data = function(data, params) {
    StatEdgeBundleMinimal0$setup_data(data, params)
  },
  compute_panel = function(
    data,
    scales,
    n = 100,
    max_distortion = 2,
    weight_fac = 2,
    tension = 1
  ) {
    edges <- StatEdgeBundleMinimal0$compute_panel(
      data,
      scales,
      max_distortion = max_distortion,
      weight_fac = weight_fac,
      tension = tension
    )
    StatBspline$compute_layer(edges, list(n = n), NULL)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'edge_id'),
  default_aes = aes(filter = TRUE),
  extra_params = c("na.rm")
)

#' @rdname geom_edge_bundle_minimal
#'
#' @export
geom_edge_bundle_minimal <- function(
  mapping = NULL,
  data = get_edges(),
  position = "identity",
  arrow = NULL,
  n = 100,
  max_distortion = 2,
  weight_fac = 2,
  tension = 1,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      group = edge.id,
      edge_id = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeBundleMinimal,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = FALSE,
        max_distortion = max_distortion,
        weight_fac = weight_fac,
        tension = tension,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBspline
#' @export
StatEdgeBundleMinimal2 <- ggproto(
  "StatEdgeBundleMinimal2",
  Stat,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    remove_loop2(data)
  },
  compute_panel = function(
    data,
    scales,
    n = 100,
    max_distortion = 2,
    weight_fac = 2,
    tension = 1
  ) {
    graph <- .G()
    nodes <- data_frame0(x = .N()$.ggraph_layout_x, y = .N()$.ggraph_layout_y)
    data <- data[order(data$group), ]
    edge_id <- data$edge_id[c(TRUE, FALSE)]
    edges <- minimal_bundle_mem(
      graph,
      nodes,
      .E()$from[edge_id],
      .E()$to[edge_id],
      max_distortion = max_distortion,
      weight_fac = weight_fac
    )
    if (tension < 1) {
      edges <- relax(edges, tension)
    }
    edges$PANEL <- data$PANEL[1]
    edges$group <- data$group[edges$group * 2]
    edges <- StatBspline$compute_layer(edges, list(n = n), NULL)
    extra_data <- data[1, !names(data) %in% c("x", "y", "group", "PANEL")][
      rep(NA, nrow(edges)),
    ]
    edges$.interp <- TRUE
    ends <- !duplicated(edges$group) | !duplicated(edges$group, fromLast = TRUE)
    edges$.interp[ends] <- FALSE
    extra_data[ends, ] <- data[, names(extra_data)]
    cbind(edges, extra_data)
  },
  required_aes = c("x", "y", "edge_id"),
  default_aes = aes(filter = TRUE),
  extra_params = c("na.rm")
)

#' @rdname geom_edge_bundle_minimal
#'
#' @export
geom_edge_bundle_minimal2 <- function(
  mapping = NULL,
  data = get_edges("long"),
  position = "identity",
  arrow = NULL,
  n = 100,
  max_distortion = 2,
  weight_fac = 2,
  tension = 1,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      group = edge.id,
      edge_id = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeBundleMinimal2,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = TRUE,
        max_distortion = max_distortion,
        weight_fac = weight_fac,
        tension = tension,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
StatEdgeBundleMinimal0 <- ggproto(
  'StatEdgeBundleMinimal0',
  Stat,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    remove_loop(data)
  },
  compute_panel = function(
    data,
    scales,
    max_distortion = 2,
    weight_fac = 2,
    tension = 1
  ) {
    graph <- .G()
    nodes <- data_frame0(x = .N()$.ggraph_layout_x, y = .N()$.ggraph_layout_y)
    edges <- minimal_bundle_mem(
      graph,
      nodes,
      .E()$from[data$edge_id],
      .E()$to[data$edge_id],
      max_distortion = max_distortion,
      weight_fac = weight_fac
    )
    if (tension < 1) {
      edges <- relax(edges, tension)
    }
    edges$PANEL <- data$PANEL[1]
    edges$group <- data$group[edges$group]
    cbind(
      edges,
      data[
        edges$group,
        !names(data) %in% c("x", "y", "xend", "yend", "PANEL", "group")
      ]
    )
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'edge_id'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm')
)
#' @rdname geom_edge_bundle_minimal
#'
#' @export
geom_edge_bundle_minimal0 <- function(
  mapping = NULL,
  data = get_edges(),
  position = "identity",
  arrow = NULL,
  max_distortion = 2,
  weight_fac = 2,
  tension = 1,
  lineend = 'butt',
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      group = edge.id,
      edge_id = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeBundleMinimal0,
    geom = GeomEdgeBspline,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        max_distortion = max_distortion,
        weight_fac = weight_fac,
        tension = tension,
        ...
      )
    )
  )
}
#' @importFrom igraph mst shortest_paths
minimal_bundle <- function(
  graph,
  nodes,
  from,
  to,
  max_distortion = 2,
  weight_fac = 2
) {
  edge_length <- sqrt(
    (nodes$x[from] - nodes$x[to])^2 + (nodes$y[from] - nodes$y[to])^2
  )
  weights <- edge_length^weight_fac
  g_temp <- mst(graph, weights)
  paths <- lapply(seq_along(from), function(f) {
    s <- from[f]
    t <- to[f]
    path <- suppressWarnings(shortest_paths(
      g_temp,
      s,
      t,
      mode = "all",
      output = "vpath"
    )$vpath[[1]])
    path_length <- sum(sqrt(
      (nodes$x[path[-length(path)]] - nodes$x[path[-1]])^2 +
        (nodes$y[path[-length(path)]] - nodes$y[path[-1]])^2
    ))
    if (path_length >= max_distortion * edge_length[f]) {
      c(s, t)
    } else {
      path
    }
  })
  ids <- rep(seq_along(from), lengths(paths))
  paths <- unlist(paths)
  data_frame0(x = nodes$x[paths], y = nodes$y[paths], group = ids)
}

minimal_bundle_mem <- memoise(minimal_bundle)

#' Edge colour scales
#'
#' This set of scales defines new colour scales for edge geoms equivalent to the
#' ones already defined by ggplot2. The parameters are equivalent to the ones
#' from ggplot2 so there is nothing new under the sun. The different geoms will
#' know whether to use edge scales or the standard scales so it is not necessary
#' to write `edge_colour` in the call to the geom - just use `colour`.
#'
#' @return A ggproto object inheriting from `Scale`
#'
#' @family scale_edge_*
#'
#' @name scale_edge_colour
#' @rdname scale_edge_colour
#'
NULL

#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_hue
#'
#' @export
scale_edge_colour_hue <- function(
  ...,
  h = c(0, 360) + 15,
  c = 100,
  l = 65,
  h.start = 0,
  direction = 1,
  na.value = "grey50",
  aesthetics = "edge_colour"
) {
  scale_colour_hue(
    ...,
    h = h,
    c = c,
    l = l,
    h.start = h.start,
    direction = direction,
    na.value = na.value,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_brewer
#'
#' @export
scale_edge_colour_brewer <- function(
  ...,
  type = "seq",
  palette = 1,
  direction = 1,
  aesthetics = "edge_colour"
) {
  scale_colour_brewer(
    ...,
    type = type,
    palette = palette,
    direction = direction,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_distiller
#'
#' @export
scale_edge_colour_distiller <- function(
  ...,
  type = "seq",
  palette = 1,
  direction = -1,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
) {
  scale_color_distiller(
    ...,
    type = type,
    palette = palette,
    direction = direction,
    values = values,
    space = space,
    na.value = na.value,
    guide = guide,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_gradient
#'
#' @export
scale_edge_colour_gradient <- function(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
) {
  scale_colour_gradient(
    ...,
    low = low,
    high = high,
    space = space,
    na.value = na.value,
    guide = guide,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_gradient2
#'
#' @importFrom scales muted
#' @export
scale_edge_colour_gradient2 <- function(
  ...,
  low = muted("red"),
  mid = "white",
  high = muted("blue"),
  midpoint = 0,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
) {
  scale_colour_gradient2(
    ...,
    low = low,
    mid = mid,
    high = high,
    midpoint = midpoint,
    space = space,
    na.value = na.value,
    guide = guide,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_gradientn
#'
#' @export
scale_edge_colour_gradientn <- function(
  ...,
  colours,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour",
  colors
) {
  scale_colour_gradientn(
    ...,
    colours = colours,
    values = values,
    space = space,
    na.value = na.value,
    guide = guide,
    aesthetics = aesthetics,
    colors = colors
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_grey
#'
#' @export
scale_edge_colour_grey <- function(
  ...,
  start = 0.2,
  end = 0.8,
  na.value = "red",
  aesthetics = "edge_colour"
) {
  scale_colour_grey(
    ...,
    start = start,
    end = end,
    na.value = na.value,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_identity
#'
#' @export
scale_edge_colour_identity <- function(
  ...,
  guide = "none",
  aesthetics = "edge_colour"
) {
  scale_colour_identity(..., guide = guide, aesthetics = aesthetics)
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_manual
#'
#' @export
scale_edge_colour_manual <- function(
  ...,
  values,
  aesthetics = "edge_colour",
  breaks = waiver(),
  na.value = "grey50"
) {
  scale_colour_manual(
    ...,
    values = values,
    aesthetics = aesthetics,
    breaks = breaks,
    na.value = na.value
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams viridis::scale_colour_viridis
#'
#' @importFrom viridis scale_colour_viridis
#' @export
scale_edge_colour_viridis <- function(
  ...,
  alpha = 1,
  begin = 0,
  end = 1,
  direction = 1,
  discrete = FALSE,
  option = "D",
  aesthetics = 'edge_colour'
) {
  scale_colour_viridis(
    ...,
    alpha = alpha,
    begin = begin,
    end = end,
    direction = direction,
    discrete = discrete,
    option = option,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_steps
#' @export
scale_edge_colour_steps <- function(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
) {
  scale_colour_steps(
    ...,
    low = low,
    high = high,
    space = space,
    na.value = na.value,
    guide = guide,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_steps2
#' @export
scale_edge_colour_steps2 <- function(
  ...,
  low = muted("red"),
  mid = "white",
  high = muted("blue"),
  midpoint = 0,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
) {
  scale_colour_steps2(
    ...,
    low = low,
    mid = mid,
    high = high,
    midpoint = midpoint,
    space = space,
    na.value = na.value,
    guide = guide,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_steps2
#' @export
scale_edge_colour_stepsn <- function(
  ...,
  colours,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour",
  colors
) {
  scale_colour_stepsn(
    ...,
    colours = colours,
    values = values,
    space = space,
    na.value = na.value,
    guide = guide,
    aesthetics = aesthetics,
    colors = colors
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_steps2
#' @export
scale_edge_colour_fermenter <- function(
  ...,
  type = "seq",
  palette = 1,
  direction = -1,
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
) {
  scale_colour_fermenter(
    ...,
    type = type,
    palette = palette,
    direction = direction,
    na.value = na.value,
    guide = guide,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_colour_continuous <- scale_edge_colour_gradient
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_colour_discrete <- scale_edge_colour_hue
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_colour_binned <- scale_edge_colour_steps

#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_hue <- scale_edge_colour_hue
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_brewer <- scale_edge_colour_brewer
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_distiller <- scale_edge_colour_distiller
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_gradient <- scale_edge_colour_gradient
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_gradient2 <- scale_edge_colour_gradient2
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_gradientn <- scale_edge_colour_gradientn
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_grey <- scale_edge_colour_grey
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_identity <- scale_edge_colour_identity
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_manual <- scale_edge_colour_manual
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_continuous <- scale_edge_colour_continuous
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_discrete <- scale_edge_colour_discrete
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_viridis <- scale_edge_colour_viridis
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_steps <- scale_edge_colour_steps
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_steps2 <- scale_edge_colour_steps2
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_stepsn <- scale_edge_colour_stepsn
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_fermenter <- scale_edge_colour_fermenter
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_binned <- scale_edge_colour_binned

#' ggraph extensions to ggplot2
#'
#' This help page lists all exported ggproto classes defined by ggraph. In
#' general these should be of no concern to the user as the main interface to
#' the functionality is, as with ggplot2, the `geom_*` format. As opposed
#' to ggplot2 there really aren't any use for separate `stat_*` functions
#' as they are intimately linked to each geom and mixing and matching stats and
#' geoms would only cause a lot of trouble.
#'
#' @details
#' Many of the `geom_edge_*` geoms comes in different flavors dependent on
#' the functionality required. There will always be a base geom and some will
#' have a `geom_edge_*0` and `geom_edge_*2` version. The base geom
#' will, in the case of multiple versions, draw the edge as a sequence of small
#' segments. The different aesthetics will be repeated for each segment and a
#' counter will be added the enumerates the progression of segments, so that a
#' gradient of colour or size can be added along the edge by assigning the
#' respective aesthetic to `after_stat(index)`. `geom_edge_*2` will also draw
#' the edge as segments but will interpolate between the aesthetics at the end
#' points. This makes it possible for an edge to interpolate node properties of
#' its end nodes. `geom_edge_*2` is less performant than the base geom so
#' use only when interpolation is needed. `geom_edge_*0` is a
#' high-performance version that usually maps directly to a grid grob. It does
#' not allow for drawing gradients or interpolations though, so use only for
#' simple edge drawings.
#'
#' @name ggraph-extensions
#' @rdname ggraph-extensions
#' @keywords internal
#'
NULL

#' Show nodes as points
#'
#' This geom is equivalent in functionality to [ggplot2::geom_point()]
#' and allows for simple plotting of nodes in different shapes, colours and sizes.
#'
#' @section Aesthetics:
#' `geom_node_point` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - alpha
#' - colour
#' - fill
#' - shape
#' - size
#' - stroke
#' - filter
#'
#' @inheritParams ggplot2::geom_point
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x and y are mapped to x and y in
#' the node data.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_node_*
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('bull') %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE))
#'
#' ggraph(gr, 'stress') + geom_node_point()
#' @export
#'
geom_node_point <- function(
  mapping = NULL,
  data = NULL,
  position = 'identity',
  show.legend = NA,
  ...
) {
  mapping <- aes_intersect(mapping, aes(x = x, y = y))
  layer(
    data = data,
    mapping = mapping,
    stat = StatFilter,
    geom = GeomPoint,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = list2(...)
  )
}

#' Show nodes as circles
#'
#' This geom is equivalent in functionality to [ggforce::geom_circle()]
#' and allows for plotting of nodes as circles with a radius scaled by the
#' coordinate system. Because of the geoms reliance on the coordinate system
#' it will only produce true circles when combined with
#' [ggplot2::coord_fixed()]
#'
#' @section Aesthetics:
#' `geom_node_circle` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x0**
#' - **y0**
#' - **r**
#' - alpha
#' - colour
#' - fill
#' - shape
#' - size
#' - stroke
#' - filter
#'
#' @inheritParams ggforce::geom_circle
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x and y are mapped to x0 and y0 in
#' the node data.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_node_*
#'
#' @examples
#' require(tidygraph)
#' gr <- tbl_graph(flare$vertices, flare$edges)
#' ggraph(gr, 'circlepack', weight = size) +
#'   geom_node_circle() +
#'   coord_fixed()
#' @export
#' @importFrom ggforce GeomCircle
#'
geom_node_circle <- function(
  mapping = NULL,
  data = NULL,
  position = 'identity',
  show.legend = NA,
  ...
) {
  mapping <- aes_intersect(mapping, aes(x0 = x, y0 = y, r = r))
  layer(
    data = data,
    mapping = mapping,
    stat = StatNodeCircle,
    geom = GeomCircle,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = list2(...)
  )
}

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatCircle
#' @export
StatNodeCircle <- ggproto(
  'StatNodeCircle',
  StatCircle,
  setup_data = function(data, params) {
    StatFilter$setup_data(data, params)
  },
  default_aes = aes(filter = TRUE)
)

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom dplyr group_by mutate slice ungroup
#' @export
StatAxisHive <- ggproto(
  'StatAxisHive',
  StatFilter,
  setup_data = function(data, params) {
    data <- data %>%
      group_by(.data$angle, .data$section, .data$PANEL) %>%
      mutate(
        x = min(.data$r) * cos(.data$angle[1]) * 1.1,
        y = min(.data$r) * sin(.data$angle[1]) * 1.1,
        xend = max(.data$r) * cos(.data$angle[1]) * 1.1,
        yend = max(.data$r) * sin(.data$angle[1]) * 1.1,
        max_r = max(.data$r),
        min_r = min(.data$r)
      ) %>%
      slice(1) %>%
      ungroup()
    data_frame0(data)
  },
  required_aes = c('r', 'angle', 'center_size', 'axis', 'section'),
  extra_params = c('na.rm', 'n', 'curvature')
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid textGrob nullGrob
#' @importFrom dplyr group_by summarise
#' @export
GeomAxisHive <- ggproto(
  'GeomAxisHive',
  GeomSegment,
  draw_panel = function(
    data,
    panel_scales,
    coord,
    label = TRUE,
    axis = TRUE,
    label_colour = 'black'
  ) {
    data$x <- data$x / 1.1
    data$y <- data$y / 1.1
    data$xend <- data$xend / 1.1
    data$yend <- data$yend / 1.1
    data <- coord$transform(data, panel_scales)
    label_data <- data %>%
      group_by(.data$axis) %>%
      summarise(
        x = max(.data$max_r) * cos(mean(.data$angle)),
        y = max(.data$max_r) * sin(mean(.data$angle)),
        label = .data$axis[1],
        angle = mean(.data$angle) / (2 * pi) * 360 - 90,
        colour = .data$colour[1],
        label_size = .data$label_size[1],
        family = .data$family[1],
        fontface = .data$fontface[1],
        lineheight = .data$lineheight[1]
      )
    label_data <- data_frame0(label_data)
    lab_dist <- sqrt(label_data$x^2 + label_data$y^2)
    dist_dodge <- max(lab_dist) * 1.05 - max(lab_dist)
    label_data$x <- label_data$x * (dist_dodge + lab_dist) / lab_dist
    label_data$y <- label_data$y * (dist_dodge + lab_dist) / lab_dist
    label_data$angle <- label_data$angle + ifelse(label_data$angle < 0, 360, 0)
    label_data$angle <- label_data$angle -
      ifelse(label_data$angle > 360, 360, 0)
    upside_label <- label_data$angle > 90 & label_data$angle < 270
    label_data$angle[upside_label] <- label_data$angle[upside_label] + 180
    label_data <- coord$transform(label_data, panel_scales)
    label_data$label_colour <- if (is.na(label_colour)) {
      label_data$colour
    } else {
      label_colour
    }
    label_grob <- if (label) {
      textGrob(
        label_data$label,
        label_data$x,
        label_data$y,
        default.units = 'native',
        rot = label_data$angle,
        gp = gpar(
          col = label_data$label_colour,
          fontsize = label_data$label_size * .pt,
          fontfamily = label_data$family,
          fontface = label_data$fontface,
          lineheight = label_data$lineheight
        )
      )
    } else {
      nullGrob()
    }
    axis_grob <- if (axis) {
      segmentsGrob(
        data$x,
        data$y,
        data$xend,
        data$yend,
        default.units = 'native',
        gp = gpar(
          col = alpha(data$colour, data$alpha),
          fill = alpha(data$colour, data$alpha),
          lwd = data$linewidth * .pt,
          lty = data$linetype,
          lineend = 'square'
        )
      )
    } else {
      nullGrob()
    }
    gList(axis_grob, label_grob)
  },
  default_aes = aes(
    colour = 'black',
    linewidth = 0.5,
    linetype = 1,
    alpha = NA,
    label_size = 3.88,
    family = '',
    fontface = 1,
    lineheight = 1.2
  )
)

#' Draw rectangular bars and labels on hive axes
#'
#' This function lets you annotate the axes in a hive plot with labels and
#' color coded bars.
#'
#' @inheritParams ggplot2::geom_point
#'
#' @param label Should the axes be labelled. Defaults to `TRUE`
#'
#' @param axis Should a rectangle be drawn along the axis. Defaults to `TRUE`
#'
#' @section Aesthetics:
#' geom_axis_hive understand the following aesthetics.
#' \itemize{
#'  \item{alpha}
#'  \item{colour}
#'  \item{fill}
#'  \item{size}
#'  \item{linetype}
#'  \item{label_size}
#'  \item{family}
#'  \item{fontface}
#'  \item{lineheight}
#' }
#'
#' @author Thomas Lin Pedersen
#'
#' @export
#'
#' @examples
#' # Plot the flare import graph as a hive plot
#' library(tidygraph)
#' flareGr <- as_tbl_graph(flare$imports) %>%
#'   mutate(
#'     type = dplyr::case_when(
#'       centrality_degree(mode = 'in') == 0 ~ 'Source',
#'       centrality_degree(mode = 'out') == 0 ~ 'Sink',
#'       TRUE ~ 'Both'
#'     )
#'   ) %>%
#'   activate(edges) %>%
#'   mutate(
#'     type = dplyr::case_when(
#'       grepl('flare.analytics', paste(.N()$name[from], .N()$name[to])) ~ 'Analytics',
#'       TRUE ~ 'Other'
#'     )
#'   )
#' ggraph(flareGr, 'hive', axis = type) +
#'   geom_edge_hive(aes(colour = type), edge_alpha = 0.1) +
#'   geom_axis_hive(aes(colour = type)) +
#'   coord_fixed()
geom_axis_hive <- function(
  mapping = NULL,
  data = NULL,
  position = 'identity',
  label = TRUE,
  axis = TRUE,
  show.legend = NA,
  ...
) {
  mapping <- aes_intersect(
    mapping,
    aes(
      r = r,
      angle = angle,
      center_size = center_size,
      axis = axis,
      section = section
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatAxisHive,
    geom = GeomAxisHive,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = list2(label = label, axis = axis, ...)
  )
}

#' Calculate nodes as circles packed within their parent circle
#'
#' The circle packing algorithm is basically a treemap using circles instead of
#' rectangles. Due to the nature of circles they cannot be packed as efficiently
#' leading to increased amount of "empty space" as compared to a treemap. This
#' can be beneficial though, as the added empty space can aid in visually
#' showing the hierarchy.
#'
#' @details
#' The circle packing is based on the algorithm developed by Weixin Wang and
#' collaborators which tries to find the most dense packing of circles as they
#' are added, one by one. This makes the algorithm very dependent on the order
#' in which circles are added and it is possible that layouts could sometimes
#' be optimized by choosing a different ordering. The algorithm for finding the
#' enclosing circle is the randomized incremental algorithm proposed by Emo
#' Welzl. Both of the above algorithms are the same as used in the D3.js
#' implementation of circle packing and their C++ implementation in ggraph is
#' inspired by Mike Bostocks JavaScript implementation.
#'
#' @note
#' Circle packing is a layout intended for trees, that is, graphs where nodes
#' only have one parent and zero or more children. If the provided graph does
#' not fit this format an attempt to convert it to such a format will be made.
#'
#' @param graph An `tbl_graph` object
#'
#' @param weight An optional node variable to use as weight. Will only affect
#' the weight of leaf nodes as the weight of non-leaf nodes are derived from
#' their children.
#'
#' @param circular Logical. Should the layout be transformed to a circular
#' representation. Ignored.
#'
#' @param sort.by The name of a node variable to sort the nodes by.
#'
#' @param direction The direction of the tree in the graph. `'out'` (default)
#' means that parents point towards their children, while `'in'` means that
#' children point towards their parent.
#'
#' @return A data.frame with the columns `x`, `y`, `r`, `leaf`,
#' `depth`, `circular` as well as any information stored as node
#' variables in the tbl_graph object.
#'
#' @references
#' Wang, W., Wang, H. H., Dai, G., & Wang, H. (2006). *Visualization of
#' large hierarchical data by circle packing*. Chi, 517-520.
#'
#' Welzl, E. (1991). *Smallest enclosing disks (balls and ellipsoids)*. New
#' Results and New Trends in Computer Science, 359-370.
#'
#' @family layout_tbl_graph_*
#'
layout_tbl_graph_circlepack <- function(
  graph,
  weight = NULL,
  circular = FALSE,
  sort.by = NULL,
  direction = 'out'
) {
  weight <- enquo(weight)
  weight <- eval_tidy(weight, .N())
  sort.by <- enquo(sort.by)
  sort.by <- eval_tidy(sort.by, .N())
  hierarchy <- tree_to_hierarchy(graph, direction, sort.by, weight)
  nodes <- circlePackLayout(
    as.integer(hierarchy$parent),
    as.numeric(hierarchy$weight)
  )[-1, ]
  nodes <- data_frame0(
    x = nodes[, 1],
    y = nodes[, 2],
    r = nodes[, 3],
    circular = FALSE,
    leaf = degree(graph, mode = direction) == 0,
    depth = node_depth(graph, mode = direction)
  )
  combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
}

#' Edge size scales
#'
#' This set of scales defines new size scales for edge geoms equivalent to the
#' ones already defined by ggplot2. See [ggplot2::scale_size()] for
#' more information. The different geoms will know whether to use edge scales or
#' the standard scales so it is not necessary to write `edge_size` in
#' the call to the geom - just use `size`.
#'
#' @note In ggplot2 size conflates both line width and point size into one
#' scale. In ggraph there is also a width scale ([scale_edge_width()])
#' that is used for linewidth. As edges are often represented by lines the width
#' scale is the most common.
#'
#' @return A ggproto object inheriting from `Scale`
#'
#' @family scale_edge_*
#'
#' @name scale_edge_size
#' @rdname scale_edge_size
#'
NULL

#' @rdname scale_edge_size
#'
#' @inheritParams ggplot2::scale_size_continuous
#'
#' @export
scale_edge_size_continuous <- function(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  trans = "identity",
  guide = "legend"
) {
  sc <- scale_size_continuous(
    name = name,
    breaks = breaks,
    labels = labels,
    limits = limits,
    range = range,
    trans = trans,
    guide = guide
  )
  sc$aesthetics <- 'edge_size'
  sc
}
#' @rdname scale_edge_size
#'
#' @inheritParams ggplot2::scale_radius
#'
#' @export
scale_edge_radius <- function(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  trans = "identity",
  guide = "legend"
) {
  sc <- scale_radius(
    name = name,
    breaks = breaks,
    labels = labels,
    limits = limits,
    range = range,
    trans = trans,
    guide = guide
  )
  sc$aesthetics <- 'edge_size'
  sc
}
#' @rdname scale_edge_size
#'
#' @export
scale_edge_size <- scale_edge_size_continuous
#' @rdname scale_edge_size
#'
#' @inheritParams ggplot2::scale_size_discrete
#' @export
scale_edge_size_discrete <- function(...) {
  cli::cli_warn(
    "Using {.field edge_size} for a discrete variable is not advised."
  )
  sc <- scale_size_ordinal(...)
  sc$aesthetics <- 'edge_size'
  sc
}
#' @rdname scale_edge_size
#'
#' @inheritParams ggplot2::scale_size_binned
#' @export
scale_edge_size_binned <- function(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  n.breaks = NULL,
  nice.breaks = TRUE,
  trans = "identity",
  guide = "bins"
) {
  sc <- scale_size_binned(
    name = name,
    breaks = breaks,
    labels = labels,
    limits = limits,
    range = range,
    n.breaks = n.breaks,
    nice.breaks = nice.breaks,
    trans = trans,
    guide = guide
  )
  sc$aesthetics <- 'edge_size'
  sc
}
#' @rdname scale_edge_size
#'
#' @inheritParams ggplot2::scale_size_area
#'
#' @export
scale_edge_size_area <- function(..., max_size = 6) {
  sc <- scale_size_area(..., max_size = max_size)
  sc$aesthetics <- 'edge_size'
  sc
}
#' @rdname scale_edge_size
#'
#'
#' @inheritParams ggplot2::scale_size_binned_area
#'
#' @export
scale_edge_size_binned_area <- function(..., max_size = 6) {
  sc <- scale_size_binned_area(..., max_size = max_size)
  sc$aesthetics <- 'edge_size'
  sc
}
#' @rdname scale_edge_size
#'
#' @inheritParams ggplot2::scale_size_manual
#'
#' @export
scale_edge_size_manual <- function(
  ...,
  values,
  breaks = waiver(),
  na.value = NA
) {
  sc <- scale_size_manual(
    ...,
    values = values,
    breaks = breaks,
    na.value = na.value
  )
  sc$aesthetics <- 'edge_size'
  sc
}
#' @rdname scale_edge_size
#'
#' @inheritParams ggplot2::scale_size_identity
#'
#' @export
scale_edge_size_identity <- function(..., guide = 'none') {
  sc <- scale_size_identity(..., guide = guide)
  sc$aesthetics <- 'edge_size'
  sc
}

#' Colourbar legend for edges
#'
#' This function is equivalent to [ggplot2::guide_colourbar()] but
#' works for edge aesthetics.
#'
#' @inheritParams ggplot2::guide_colourbar
#' @inheritDotParams ggplot2::guide_colourbar
#'
#' @return A guide object
#'
#' @export
guide_edge_colourbar <- function(
  ...,
  available_aes = c("edge_colour", "edge_fill")
) {
  guide <- guide_colourbar(..., available_aes = available_aes)
  guide$params$name <- 'edge_colourbar'
  guide
}
#' @rdname guide_edge_colourbar
#' @export
guide_edge_colorbar <- guide_edge_colourbar

#' Coloursteps legend for edges
#'
#' This function is equivalent to [ggplot2::guide_coloursteps()] but
#' works for edge aesthetics.
#'
#' @inheritParams ggplot2::guide_coloursteps
#' @inheritParams ggplot2::guide_colourbar
#' @inheritDotParams ggplot2::guide_colourbar
#'
#' @return A guide object
#'
#' @export
guide_edge_coloursteps <- function(
  even.steps = TRUE,
  show.limits = NULL,
  ...,
  available_aes = c("edge_colour", "edge_fill")
) {
  guide <- guide_coloursteps(
    even.steps = even.steps,
    show.limits = show.limits,
    ...,
    available_aes = available_aes
  )
  guide$params$name <- 'edge_coloursteps'
  guide
}
#' @rdname guide_edge_coloursteps
#' @export
guide_edge_colorsteps <- guide_edge_coloursteps

#' Place nodes in circles based on distance to a specific node
#'
#' This layout constrains node placement to a radius relative to its distance to
#' a given node. It then uses stress majorisation to find an optimal node
#' distribution according to this constraint.
#'
#' @param focus An expression evaluating to a selected node. Can either be a
#' single integer or a logical vector with a single `TRUE` element.
#' @inheritParams layout_tbl_graph_stress
#' @inheritParams layout_tbl_graph_centrality
#'
#' @return A data.frame with the columns `x`, `y`, `circular`, `distance` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @references
#' Brandes, U., & Pich, C. (2011). *More flexible radial layout.* Journal of
#' Graph Algorithms and Applications, 15(1), 157-173.
#'
#' @family layout_tbl_graph_*
#'
#' @author The underlying algorithm is implemented in the graphlayouts package
#' by David Schoch
#'
#' @importFrom graphlayouts layout_with_focus layout_with_focus_group
#' @importFrom rlang eval_tidy enquo
layout_tbl_graph_focus <- function(
  graph,
  focus,
  weights = NULL,
  niter = 500,
  tolerance = 1e-4,
  group = NULL,
  shrink = 10,
  circular = TRUE
) {
  focus <- eval_tidy(enquo(focus), .N())
  if (is.logical(focus)) {
    focus <- which(focus)[1]
  }

  weights <- eval_tidy(enquo(weights), .E())
  if (is.null(weights)) {
    weights <- NA
  }

  group <- eval_tidy(enquo(group), .N())

  if (is.null(group)) {
    layout <- layout_with_focus(
      graph,
      v = focus,
      weights = weights,
      iter = niter,
      tol = tolerance
    )
  } else {
    layout <- layout_with_focus_group(
      graph,
      v = focus,
      group = group,
      shrink = shrink,
      weights = weights,
      iter = niter,
      tol = tolerance
    )
  }

  xy <- layout$xy

  nodes <- data_frame0(
    x = xy[, 1],
    y = xy[, 2],
    distance = layout$distance,
    circular = FALSE
  )
  combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
}

#' Automatically pick a layout based on graph type
#'
#' This function infers the layout from the graph structure and is the default
#' when calling [ggraph()]. If an `x` and `y` argument is passed along, the
#' manual layout is chosen. Otherwise if the graph is either a rooted tree or a
#' rooted forest the layout will be `dendrogram` if the nodes contains a height
#' variable or `tree` if not. If the tree is unrooted the `unrooted` layout will
#' be used. If the tree is a DAG the `sygiyama` layout will be used. Otherwise
#' the `stress` layout will be used (or `sparse_tree` if the graph contains more
#' than 2000 nodes).
#'
#' @param graph A tbl_graph object
#'
#' @param circular Logical. Should the layout be transformed to a circular
#' representation. Defaults to `FALSE`. Only applicable if the graph is a tree
#' structure
#'
#' @param ... Arguments passed on to the chosen layout
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom rlang .data quos
#'
layout_tbl_graph_auto <- function(graph, circular, ...) {
  if (all(c('x', 'y') %in% names(quos(...)))) {
    layout_tbl_graph_manual(graph, circular = circular, ...)
  } else if (with_graph(graph, graph_is_tree() || graph_is_forest())) {
    if (is.null(with_graph(graph, .N()[['height']]))) {
      cli::cli_inform('Using {.val tree} as default layout')
      layout_tbl_graph_igraph(
        graph,
        algorithm = 'tree',
        circular = circular,
        ...
      )
    } else {
      cli::cli_inform('Using {.val dendrogram} as default layout')
      layout_tbl_graph_dendrogram(
        graph,
        circular = circular,
        height = .data$height,
        ...
      )
    }
  } else if (with_graph(graph, graph_is_dag())) {
    cli::cli_inform('Using {.val sugiyama} as default layout')
    layout_tbl_graph_igraph(
      graph,
      algorithm = 'sugiyama',
      circular = circular,
      ...
    )
  } else if (with_graph(graph, graph_order()) < 2000) {
    cli::cli_inform('Using {.val stress} as default layout')
    layout_tbl_graph_stress(graph, circular = circular, ...)
  } else {
    cli::cli_inform(
      'Using {.val sparse_stress} with 250 pivots as default layout'
    )
    layout_tbl_graph_sparse_stress(
      graph,
      pivots = 250,
      circular = circular,
      ...
    )
  }
}

#' Edge fill scales
#'
#' This set of scales defines new fill scales for edge geoms equivalent to the
#' ones already defined by ggplot2. The parameters are equivalent to the ones
#' from ggplot2 so there is nothing new under the sun. The different geoms will
#' know whether to use edge scales or the standard scales so it is not necessary
#' to write `edge_fill` in the call to the geom - just use `fill`.
#'
#' @return A ggproto object inheriting from `Scale`
#'
#' @family scale_edge_*
#'
#' @name scale_edge_fill
#' @rdname scale_edge_fill
#'
NULL

#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_hue
#'
#' @export
scale_edge_fill_hue <- function(
  ...,
  h = c(0, 360) + 15,
  c = 100,
  l = 65,
  h.start = 0,
  direction = 1,
  na.value = "grey50",
  aesthetics = "edge_fill"
) {
  scale_fill_hue(
    ...,
    h = h,
    c = c,
    l = l,
    h.start = h.start,
    direction = direction,
    na.value = na.value,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_brewer
#'
#' @export
scale_edge_fill_brewer <- function(
  ...,
  type = "seq",
  palette = 1,
  direction = 1,
  aesthetics = "edge_fill"
) {
  scale_fill_brewer(
    ...,
    type = type,
    palette = palette,
    direction = direction,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_distiller
#'
#' @export
scale_edge_fill_distiller <- function(
  ...,
  type = "seq",
  palette = 1,
  direction = -1,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_fill"
) {
  scale_color_distiller(
    ...,
    type = type,
    palette = palette,
    direction = direction,
    values = values,
    space = space,
    na.value = na.value,
    guide = guide,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_gradient
#'
#' @export
scale_edge_fill_gradient <- function(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_fill"
) {
  scale_fill_gradient(
    ...,
    low = low,
    high = high,
    space = space,
    na.value = na.value,
    guide = guide,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_gradient2
#'
#' @importFrom scales muted
#' @export
scale_edge_fill_gradient2 <- function(
  ...,
  low = muted("red"),
  mid = "white",
  high = muted("blue"),
  midpoint = 0,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_fill"
) {
  scale_fill_gradient2(
    ...,
    low = low,
    mid = mid,
    high = high,
    midpoint = midpoint,
    space = space,
    na.value = na.value,
    guide = guide,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_gradientn
#'
#' @export
scale_edge_fill_gradientn <- function(
  ...,
  colours,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_fill",
  colors
) {
  scale_fill_gradientn(
    ...,
    colours = colours,
    values = values,
    space = space,
    na.value = na.value,
    guide = guide,
    aesthetics = aesthetics,
    colors = colors
  )
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_grey
#'
#' @export
scale_edge_fill_grey <- function(
  ...,
  start = 0.2,
  end = 0.8,
  na.value = "red",
  aesthetics = "edge_fill"
) {
  scale_fill_grey(
    ...,
    start = start,
    end = end,
    na.value = na.value,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_identity
#'
#' @export
scale_edge_fill_identity <- function(
  ...,
  guide = "none",
  aesthetics = "edge_fill"
) {
  scale_fill_identity(..., guide = guide, aesthetics = aesthetics)
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_manual
#'
#' @export
scale_edge_fill_manual <- function(
  ...,
  values,
  aesthetics = "edge_fill",
  breaks = waiver(),
  na.value = "grey50"
) {
  scale_fill_manual(
    ...,
    values = values,
    aesthetics = aesthetics,
    breaks = breaks,
    na.value = na.value
  )
}
#' @rdname scale_edge_fill
#'
#' @inheritParams viridis::scale_fill_viridis
#'
#' @importFrom viridis scale_fill_viridis
#' @export
scale_edge_fill_viridis <- function(
  ...,
  alpha = 1,
  begin = 0,
  end = 1,
  direction = 1,
  discrete = FALSE,
  option = "D",
  aesthetics = 'edge_fill'
) {
  scale_fill_viridis(
    ...,
    alpha = alpha,
    begin = begin,
    end = end,
    direction = direction,
    discrete = discrete,
    option = option,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_steps
#' @export
scale_edge_fill_steps <- function(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_fill"
) {
  scale_fill_steps(
    ...,
    low = low,
    high = high,
    space = space,
    na.value = na.value,
    guide = guide,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_steps2
#' @export
scale_edge_fill_steps2 <- function(
  ...,
  low = muted("red"),
  mid = "white",
  high = muted("blue"),
  midpoint = 0,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_fill"
) {
  scale_fill_steps2(
    ...,
    low = low,
    mid = mid,
    high = high,
    midpoint = midpoint,
    space = space,
    na.value = na.value,
    guide = guide,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_stepsn
#' @export
scale_edge_fill_stepsn <- function(
  ...,
  colours,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_fill",
  colors
) {
  scale_fill_stepsn(
    ...,
    colours = colours,
    values = values,
    space = space,
    na.value = na.value,
    guide = guide,
    aesthetics = aesthetics,
    colors = colors
  )
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_fermenter
#' @export
scale_edge_fill_fermenter <- function(
  ...,
  type = "seq",
  palette = 1,
  direction = -1,
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_fill"
) {
  scale_fill_fermenter(
    ...,
    type = type,
    palette = palette,
    direction = direction,
    na.value = na.value,
    guide = guide,
    aesthetics = aesthetics
  )
}
#' @rdname scale_edge_fill
#'
#' @export
scale_edge_fill_continuous <- scale_edge_fill_gradient
#' @rdname scale_edge_fill
#'
#' @export
scale_edge_fill_discrete <- scale_edge_fill_hue
#' @rdname scale_edge_fill
#'
#' @export
scale_edge_fill_binned <- scale_edge_fill_steps

#' Edge direction guide
#'
#' This guide is intended to show the direction of edges based on the aesthetics
#' mapped to its progression, such as changing width, colour and opacity.
#'
#' @inheritParams ggplot2::guide_colourbar
#' @inheritParams ggplot2::guide_legend
#'
#' @param arrow Logical. Should an arrow be drawn to illustrate the direction.
#' Defaults to `TRUE`. The arrow is styled with the `legend.axis.line` theme
#' element. If `FALSE` the direction will be indicated by the text
#' given in `labels`
#'
#' @param labels A vector with two strings giving the labels to place at the
#' start and the end of the legend to indicate direction if `arrow = FALSE`
#'
#' @param arrow.position `r lifecycle::badge('deprecated')` The position of the
#' arrow relative to the example edge. Use the `legend.text.position` argument
#' in `theme()` instead.
#'
#' @export
#'
#' @examples
#' gr <- tidygraph::as_tbl_graph(highschool)
#' ggraph(gr, layout = 'kk') +
#'   geom_edge_fan(aes(alpha = after_stat(index))) +
#'   guides(edge_alpha = guide_edge_direction())
#'
#' # Use text labels instead of an arrow
#' ggraph(gr, layout = 'kk') +
#'   geom_edge_fan(aes(alpha = after_stat(index))) +
#'   guides(edge_alpha = guide_edge_direction(labels = c('start', 'end')))
#'
#' # Style the indicator arrow
#' arrow_style <- element_line(linewidth = 3, arrow = grid::arrow(type = "closed"))
#' ggraph(gr, layout = 'kk') +
#'   geom_edge_fan(aes(alpha = after_stat(index))) +
#'   guides(
#'     edge_alpha = guide_edge_direction(
#'       theme = theme(legend.axis.line = arrow_style)
#'     )
#'   )
#'
guide_edge_direction <- function(
  title = NULL,
  theme = NULL,
  arrow = NULL,
  labels = NULL,
  nbin = 500,
  position = NULL,
  direction = NULL,
  reverse = FALSE,
  order = 0,
  override.aes = list(),
  ...,
  available_aes = c("edge_colour", "edge_alpha", "edge_width"),
  arrow.position = deprecated()
) {
  # Piggyback on non-exported `deprecated_guide_args()`
  theme <- guide_colourbar(theme = theme, ...)$params$theme

  if (!is.null(arrow) && !is.null(labels)) {
    cli::cli_abort(
      "{.arg arrow} and {.arg labels} can't be used at the same time"
    )
  }
  if (is.null(arrow) && is.null(labels)) {
    arrow <- TRUE
  }
  if (!is.null(labels)) {
    arrow <- FALSE
  }
  if (!arrow && is.null(labels)) {
    labels <- c("from", "to")
  }

  if (lifecycle::is_present(arrow.position)) {
    lifecycle::deprecate_warn(
      '2.2.0',
      'guide_edge_direction(arrow.position)',
      'guide_edge_direction(theme = "theme(legend.text.position)")'
    )
    if (!is.null(arrow.position)) {
      if (is.null(theme)) {
        theme <- theme()
      }
      theme$legend.text.position <- arrow.position
    }
  }
  if (!is.null(position)) {
    position <- arg_match0(
      position,
      c("top", "right", "bottom", "left", "inside")
    )
  }
  if (!is.null(labels) && (length(labels) != 2 || !is.character(labels))) {
    cli::cli_abort('{.arg label} must be a vector of two strings')
  }
  new_guide(
    title = title,
    theme = theme,
    arrow = arrow,
    labels = labels,
    nbin = nbin,
    position = position,
    direction = direction,
    override.aes = expand_edge_aes(rename_aes(override.aes)),
    reverse = reverse,
    order = order,
    available_aes = available_aes,
    name = 'direction',
    super = GuideEdgeDirection
  )
}
GuideEdgeDirection <- ggproto(
  "GuideEdgeDirection",
  GuideLegend,

  params = list(
    # title
    title = NULL,

    # theming
    theme = NULL,

    # bar
    nbin = 300,
    arrow = TRUE,
    labels = c("from", "to"),

    # general
    direction = NULL,
    override.aes = list(),
    reverse = FALSE,
    order = 0,

    # parameter
    name = "direction",
    hash = character(),
    position = NULL
  ),

  available_aes = c('edge_colour', 'edge_alpha', 'edge_width'),

  hashables = exprs(title, decor$value, name),

  elements = list(
    background = "legend.background",
    margin = "legend.margin",
    key = "legend.key",
    key_height = "legend.key.height",
    key_width = "legend.key.width",
    text = "legend.text",
    theme.title = "legend.title",
    text_position = "legend.text.position",
    title_position = "legend.title.position",
    arrow_line = "legend.axis.line"
  ),

  extract_key = function(scale, aesthetic, ...) {
    if (scale$is_discrete()) {
      cli::cli_warn("{.fn guide_edge_direction} needs continuous scales.")
      return(NULL)
    }
    Guide$extract_key(scale, aesthetic, ...)
  },

  extract_decor = function(scale, aesthetic, nbin = 300, reverse = FALSE, ...) {
    limits <- scale$get_limits()
    bar <- seq(limits[1], limits[2], length.out = nbin)
    if (length(bar) == 0) {
      bar <- unique0(limits)
    }
    aes <- scale$aesthetics[1]
    bar <- data_frame0(
      {{ aes }} := scale$map(bar),
      value = bar,
      .size = length(bar)
    )
    if (reverse) {
      bar <- bar[rev(seq_len(nrow(bar))), , drop = FALSE]
    }
    return(bar)
  },

  merge = function(self, params, new_guide, new_params) {
    new_params$decor$value <- NULL
    params$decor <- vec_cbind(params$decor, new_params$decor)
    return(list(guide = self, params = params))
  },

  get_layer_key = function(params, ...) {
    decor <- GuideLegend$get_layer_key(params, ...)$decor
    extra_data <- lapply(decor, `[[`, 'data')
    missing_aes <- setdiff(
      c("edge_colour", "edge_alpha", "edge_width"),
      names(params$decor)
    )
    for (d in extra_data) {
      for (aes in missing_aes) {
        if (!is.null(d[[aes]])) params$decor[[aes]] <- d[[aes]][1]
      }
    }
    params
  },

  setup_params = function(params) {
    params$direction <- arg_match0(
      params$direction,
      c("horizontal", "vertical"),
      arg_nm = "direction"
    )
    params
  },

  setup_elements = function(params, elements, theme) {
    elements <- GuideColourbar$setup_elements(params, elements, theme)
    if (is.logical(elements$arrow_line$arrow)) {
      elements$arrow_line$arrow <- grid::arrow()
    }
    elements$arrow_line$arrow$ends <- if (params$reverse) 1L else 2L
    elements
  },

  build_labels = function(key, elements, params) {
    if (params$arrow) {
      list(
        labels = flip_element_grob(
          elements$arrow_line,
          x = unit(c(0, 1), "npc"),
          y = unit(c(0.5, 0.5), "npc"),
          flip = params$direction == "vertical"
        )
      )
    } else {
      list(
        labels = flip_element_grob(
          elements$text,
          label = params$labels,
          x = unit(c(0, 1), "npc"),
          margin_x = FALSE,
          margin_y = TRUE,
          flip = params$direction == "vertical"
        )
      )
    }
  },

  build_ticks = function(key, elements, params, position = params$position) {
    zeroGrob()
  },

  build_decor = function(decor, grobs, elements, params) {
    y <- seq(0, 1, length.out = nrow(decor) + 1)
    yend <- y[-1]
    y <- y[-length(y)]
    x <- rep(0.5, length(y))
    xend <- x
    if (params$direction == "horizontal") {
      tmp <- x
      x <- y
      y <- tmp
      tmp <- xend
      xend <- yend
      yend <- tmp
    }
    grob <- segmentsGrob(
      x0 = x,
      y0 = y,
      x1 = xend,
      y1 = yend,
      default.units = "npc",
      gp = gpar(
        col = alpha(decor$edge_colour, decor$edge_alpha),
        lwd = decor$edge_width * .pt,
        lty = decor$edge_linetyoe,
        lineend = "butt"
      )
    )

    list(bar = grob)
  },

  measure_grobs = function(grobs, params, elements) {
    params$sizes <- list(
      widths = elements$width_cm,
      heights = elements$height_cm
    )
    sizes <- GuideLegend$measure_grobs(grobs, params, elements)
    if (params$arrow) {
      l <- convertHeight(
        elements$arrow_line$arrow$length,
        "cm",
        valueOnly = TRUE
      )
      a <- 2 * pi * elements$arrow_line$arrow$angle / 360
      width <- sin(a) * l * 2 + elements$arrow_line$linewidth / 10
      if (params$direction == "vertical") {
        sizes$widths[sizes$widths == 0] <- width
      } else {
        sizes$heights[sizes$heights == 0] <- width
      }
    }
    sizes
  }
)

#' Define simple shapes for line capping
#'
#' This set of functions makes it easy to define shapes at the terminal points
#' of edges that are used to shorten the edges. The shapes themselves are not
#' drawn, but the edges will end at the boundary of the shape rather than at
#' the node position. This is especially relevant when drawing arrows at the
#' edges as the arrows will be partly obscured by the node unless the edge is
#' shortened. Edge shortening is dynamic and will update as the plot is resized,
#' making sure that the capping remains at an absolute distance to the end
#' point.
#'
#' @details
#' `geometry` is the base constructor, while the rest are helpers to save
#' typing. `circle` creates circles width a given radius, `square`
#' creates squares at a given side length, `ellipsis` creates ellipses with
#' given a and b values (width and height radii), and `rectangle` makes
#' rectangles of a given width and height. label_rect is a helper that, given
#' a list of strings and potentially formatting options creates a rectangle that
#' encloses the string.
#'
#' @param type The type of geometry to use. Currently `'circle'` and
#' `'rect'` is supported.
#'
#' @param width,height,length,radius,a,b The dimensions of the shape.
#'
#' @param unit,width_unit,height_unit,a_unit,b_unit The unit for the numbers
#' given.
#'
#' @return A geometry object encoding the specified shape.
#'
#' @examples
#' geometry(c('circle', 'rect', 'rect'), 1:3, 3:1)
#'
#' circle(1:4, 'mm')
#'
#' label_rect(c('some', 'different', 'words'), fontsize = 18)
#' @export
#'
geometry <- function(
  type = 'circle',
  width = 1,
  height = width,
  width_unit = 'cm',
  height_unit = width_unit
) {
  l <- max(length(type), length(width), length(height))
  g <- rep_len(vec_cast(type, character()), l)
  g_na <- is.na(g)
  g[g_na] <- 'circle'
  width <- rep_len(vec_cast(width, double()), l)
  width[g_na] <- 0
  height <- rep_len(vec_cast(height, double()), l)
  height[g_na] <- 0
  uwidth <- rep_len(vec_cast(width_unit, character()), l)
  uheight <- rep_len(vec_cast(height_unit, character()), l)
  new_geometry(
    geometry = g,
    width = width,
    height = height,
    width_unit = uwidth,
    height_unit = uheight
  )
}
new_geometry <- function(
  geometry = character(),
  width = numeric(),
  height = numeric(),
  width_unit = character(),
  height_unit = character()
) {
  vec_assert(geometry, character())
  vec_assert(width, double())
  vec_assert(height, double())
  vec_assert(width_unit, character())
  vec_assert(height_unit, character())
  new_rcrd(
    list(
      geometry = geometry,
      width = width,
      height = height,
      width_unit = width_unit,
      height_unit = height_unit
    ),
    class = 'ggraph_geometry'
  )
}
#' @rdname geometry
#'
#' @export
circle <- function(radius = 1, unit = 'cm') {
  geometry('circle', width = radius * 2, width_unit = unit)
}
#' @rdname geometry
#'
#' @export
square <- function(length = 1, unit = 'cm') {
  geometry('rect', width = length, width_unit = unit)
}
#' @rdname geometry
#'
#' @export
ellipsis <- function(a = 1, b = 1, a_unit = 'cm', b_unit = a_unit) {
  geometry(
    'circle',
    width = a * 2,
    height = b * 2,
    width_unit = a_unit,
    height_unit = b_unit
  )
}
#' @rdname geometry
#'
#' @export
rectangle <- function(
  width = 1,
  height = 1,
  width_unit = 'cm',
  height_unit = width_unit
) {
  geometry(
    'rect',
    width = width,
    height = height,
    width_unit = width_unit,
    height_unit = height_unit
  )
}
#' @rdname geometry
#'
#' @param label The text to be enclosed
#'
#' @param padding extra size to be added around the text using the
#' [ggplot2::margin()] function
#'
#' @param ... Passed on to [grid::gpar()]
#'
#' @export
#' @importFrom grid convertWidth convertHeight textGrob grobWidth grobHeight
label_rect <- function(label, padding = margin(1, 1, 1.5, 1, 'mm'), ...) {
  grobs <- lapply(label, textGrob, gp = gpar(...))
  width <- abs_width(grobs)
  height <- abs_height(grobs)
  width <- width + sum(convertWidth(padding[c(2, 4)], 'cm', TRUE))
  height <- height + sum(convertHeight(padding[c(1, 3)], 'cm', TRUE))
  geometry('rect', width = width, height = height)
}
#' @rdname geometry
#'
#' @param x An object to test for geometry inheritance
#'
#' @export
is.geometry <- function(x) inherits(x, 'ggraph_geometry')
#' @export
format.ggraph_geometry <- function(x, ...) {
  paste0(
    field(x, 'geometry'),
    '(',
    field(x, 'width'),
    field(x, 'width_unit'),
    ', ',
    field(x, 'height'),
    field(x, 'height_unit'),
    ')'
  )
}
#' @export
is.na.ggraph_geometry <- function(x) {
  is.na(field(x, 'geometry'))
}
geo_type <- function(x) {
  if (!is.geometry(x)) {
    cli::cli_abort('{.arg x} must be a {.cls ggraph_geometry} object')
  }
  field(x, 'geometry')
}
geo_width <- function(x) {
  if (!is.geometry(x)) {
    cli::cli_abort('{.arg x} must be a {.cls geometry} object')
  }
  unit(field(x, 'width'), field(x, 'width_unit'))
}
geo_height <- function(x) {
  if (!is.geometry(x)) {
    cli::cli_abort('{.arg x} must be a {.cls geometry} object')
  }
  unit(field(x, 'height'), field(x, 'height_unit'))
}
#' @importFrom grid convertHeight grobHeight
abs_height <- function(grobs) {
  vapply(
    grobs,
    function(g) convertHeight(grobHeight(g), 'cm', TRUE),
    numeric(1)
  )
}
#' @importFrom grid convertWidth grobWidth
abs_width <- function(grobs) {
  vapply(grobs, function(g) convertWidth(grobWidth(g), 'cm', TRUE), numeric(1))
}

#' @export
vec_ptype2.ggraph_geometry.ggraph_geometry <- function(x, y, ...) new_geometry()
#' @export
vec_ptype2.ggraph_geometry.character <- function(x, y, ...) character()
#' @export
vec_ptype2.character.ggraph_geometry <- function(x, y, ...) character()
#' @export
vec_cast.character.ggraph_geometry <- function(x, to, ...) {
  as.character(field(x, 'geometry'))
}

#' Define default scale type for geometry
#'
#' This function is quite useless as geometry is not meant to be scaled, but it
#' is a requirement for ggplot2 to handle it correctly.
#'
#' @export
#'
#' @keywords internal
scale_type.ggraph_geometry <- function(x) 'identity'

#' Create small multiples based on edge attributes
#'
#' This function is equivalent to [ggplot2::facet_wrap()] but only
#' facets edges. Nodes are repeated in every panel.
#'
#' @inheritParams ggplot2::facet_wrap
#'
#' @family ggraph-facets
#'
#' @examples
#' gr <- tidygraph::as_tbl_graph(highschool)
#'
#' ggraph(gr) +
#'   geom_edge_link() +
#'   geom_node_point() +
#'   facet_edges(~year)
#' @export
#'
facet_edges <- function(
  facets,
  nrow = NULL,
  ncol = NULL,
  scales = 'fixed',
  shrink = TRUE,
  labeller = 'label_value',
  as.table = TRUE,
  switch = deprecated(),
  drop = TRUE,
  dir = 'h',
  strip.position = 'top'
) {
  # Work around non-lifecycle deprecation
  if (
    !lifecycle::is_present(switch) &&
      utils::packageVersion('ggplot2') <= '3.3.6'
  ) {
    switch <- NULL
  }
  facet <- facet_wrap(
    facets = facets,
    nrow = nrow,
    ncol = ncol,
    scales = scales,
    shrink = shrink,
    labeller = labeller,
    as.table = as.table,
    switch = switch,
    drop = drop,
    dir = dir,
    strip.position = strip.position
  )
  seed <- sample(.Machine$integer.max, 1L)
  facet$params$facets[] <- lapply(seq_along(facet$params$facets), function(i) {
    rlang::quo(withr::with_seed(seed, !!facet$params$facets[[i]]))
  })
  ggproto(NULL, FacetEdges, shrink = shrink, params = facet$params)
}

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
FacetEdges <- ggproto(
  'FacetEdges',
  FacetWrap,
  compute_layout = function(data, params) {
    plot_data <- data[[1]]
    data <- split(data, vapply(data, data_type, character(1)))
    facet_data <- data$edge_ggraph

    panels <- FacetWrap$compute_layout(facet_data, params)
    node_placement <- rep(list(plot_data$.ggraph.index), nrow(panels))
    names(node_placement) <- as.character(seq_along(node_placement))
    attr(panels, 'node_placement') <- node_placement
    panels
  },
  map_data = function(data, layout, params) {
    switch(
      data_type(data),
      node_ggraph = {
        node_map <- rep(list(data), length(attr(layout, 'node_placement')))
        panel <- rep(seq_along(node_map), vapply(node_map, nrow, numeric(1)))
        node_map <- vec_rbind(!!!node_map)
        node_map$PANEL <- factor(panel, levels = levels(layout$PANEL))
        node_map
      },
      edge_ggraph = ,
      {
        FacetWrap$map_data(data, layout, params)
      }
    )
  }
)

#' Calculate nodes as rectangles subdividing that of their parent
#'
#' A treemap is a space filling hierarchical layout that maps nodes to
#' rectangles. The rectangles of the children of a node is packed into the
#' rectangle of the node so that the size of a rectangle is a function of the
#' size of the children. The size of the leaf nodes can be mapped arbitrarily
#' (defaults to 1). Many different algorithms exists for dividing a rectangle
#' into smaller bits, some optimizing the aspect ratio and some focusing on the
#' ordering of the rectangles. See details for more discussions on this. The
#' treemap layout was first developed by Ben Shneiderman for visualizing disk
#' usage in the early '90 and has seen many improvements since.
#'
#' @details
#' Different approaches to dividing the rectangles in a treemap exists; all with
#' their strengths and weaknesses. Currently only the split algorithm is
#' implemented which strikes a good balance between aspect ratio and order
#' preservation, but other, more well-known, algorithms such as squarify and
#' slice-and-dice will eventually be implemented.
#'
#' \strong{Algorithms}
#'
#' *Split* (default)
#'
#' The Split algorithm was developed by Bjorn Engdahl in order to address the
#' downsides of both the original slice-and-dice algorithm (poor aspect ratio)
#' and the popular squarify algorithm (no ordering of nodes). It works by
#' finding the best cut in the ordered list of children in terms of making sure
#' that the two rectangles associated with the split will have optimal aspect
#' ratio.
#'
#'
#' @note
#' Treemap is a layout intended for trees, that is, graphs where nodes
#' only have one parent and zero or more children. If the provided graph does
#' not fit this format an attempt to convert it to such a format will be made.
#'
#' @param graph A `tbl_graph` object
#'
#' @param algorithm The name of the tiling algorithm to use. Defaults to 'split'
#'
#' @param weight An optional node variable to use as weight. Will only affect
#' the weight of leaf nodes as the weight of non-leaf nodes are derived from
#' their children.
#'
#' @param circular Logical. Should the layout be transformed to a circular
#' representation. Ignored.
#'
#' @param sort.by The name of a node variables to sort the nodes by.
#'
#' @param direction The direction of the tree in the graph. `'out'` (default)
#' means that parents point towards their children, while `'in'` means that
#' children point towards their parent.
#'
#' @param height The height of the bounding rectangle
#'
#' @param width The width of the bounding rectangle
#'
#' @return A data.frame with the columns `x`, `y`, `width`,
#' `height`, `leaf`, `depth`, `circular` as well as any
#' information stored as node variables in the tbl_graph object.
#'
#' @references
#' Engdahl, B. (2005). *Ordered and unordered treemap algorithms and their
#' applications on handheld devices*. Master's Degree Project.
#'
#' Johnson, B., & Ben Shneiderman. (1991). *Tree maps: A Space-Filling
#' Approach to the Visualization of Hierarchical Information Structures*. IEEE
#' Visualization, 284-291. \doi{10.1109/VISUAL.1991.175815}
#'
#' @family layout_tbl_graph_*
#'
layout_tbl_graph_treemap <- function(
  graph,
  algorithm = 'split',
  weight = NULL,
  circular = FALSE,
  sort.by = NULL,
  direction = 'out',
  height = 1,
  width = 1
) {
  weight <- enquo(weight)
  weight <- eval_tidy(weight, .N())
  sort.by <- enquo(sort.by)
  sort.by <- eval_tidy(sort.by)
  hierarchy <- tree_to_hierarchy(graph, direction, sort.by, weight)
  layout <- switch(
    algorithm,
    split = splitTreemap(
      as.integer(hierarchy$parent),
      as.integer(hierarchy$order),
      as.numeric(hierarchy$weight),
      as.numeric(width),
      as.numeric(height)
    ),
    cli::cli_abort('Unknown treemap algorithm')
  )[-1, ]
  nodes <- data_frame0(
    x = layout[, 1] + layout[, 3] / 2,
    y = layout[, 2] + layout[, 4] / 2,
    width = layout[, 3],
    height = layout[, 4],
    circular = FALSE,
    leaf = degree(graph, mode = direction) == 0,
    depth = node_depth(graph, mode = direction)
  )
  combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
}

#' Get the angle of nodes and edges
#'
#' These helper functions makes it easy to calculate the angle associated with
#' nodes and edges. For nodes the angle is defined as the angle of the vector
#' pointing towards the node position, and is thus mainly suited for circular
#' layouts where it can be used to calculate the angle of labels. For edges it
#' is simply the angle of the vector describing the edge.
#'
#' @param x,y A vector of positions
#'
#' @param xend,yend The end position of the edge
#'
#' @param degrees Logical. Should the angle be returned in degree (`TRUE`)
#' or radians (`FALSE`). Defaults to `TRUE`.
#'
#' @param avoid_flip Logical. Should the angle be adjusted so that text is
#' always upside-down
#'
#' @return A vector with the angle of each node/edge
#'
#' @examples
#' require(tidygraph)
#' flareGraph <- tbl_graph(flare$vertices, flare$edges)
#'
#' ggraph(flareGraph, 'dendrogram', circular = TRUE) +
#'   geom_edge_diagonal0() +
#'   geom_node_text(aes(filter = leaf, angle = node_angle(x, y), label = shortName),
#'     hjust = 'outward', size = 2
#'   ) +
#'   expand_limits(x = c(-1.3, 1.3), y = c(-1.3, 1.3))
#' @export
#'
node_angle <- function(x, y, degrees = TRUE, avoid_flip = TRUE) {
  angles <- atan2(y, x)
  angles[angles < 0] <- angles[angles < 0] + 2 * pi
  if (avoid_flip) {
    needs_flip <- angles > pi / 2 & angles < 3 * pi / 2
    angles[needs_flip] <- angles[needs_flip] + pi
  }
  if (degrees) {
    angles * 360 / (2 * pi)
  } else {
    angles
  }
}
#' @rdname node_angle
#'
#' @export
edge_angle <- function(x, y, xend, yend, degrees = TRUE, avoid_flip = TRUE) {
  x <- xend - x
  y <- yend - y
  node_angle(x, y, degrees, avoid_flip = avoid_flip)
}


### COPY FROM GGPLOT2 NON-EXPORTS
#' @importFrom grid grobName
ggname <- function(prefix, grob) {
  grob$name <- grobName(grob, prefix)
  grob
}
rename_aes <- function(x) {
  names(x) <- standardise_aes_names(names(x))
  duplicated_names <- names(x)[duplicated(names(x))]
  if (length(duplicated_names) > 0L) {
    cli::cli_warn(
      "Duplicated aesthetics after name standardisation: {.field {unique0(duplicated_names)}}"
    )
  }
  x
}
flip_names <- c(
  x = "y",
  y = "x",
  width = "height",
  height = "width",
  hjust = "vjust",
  vjust = "hjust",
  margin_x = "margin_y",
  margin_y = "margin_x"
)
flip_element_grob <- function(..., flip = FALSE) {
  if (!flip) {
    ans <- element_grob(...)
    return(ans)
  }
  args <- list(...)
  translate <- names(args) %in% names(flip_names)
  names(args)[translate] <- flip_names[names(args)[translate]]
  do.call(element_grob, args)
}

#' @importFrom viridis scale_color_viridis
#' @export
viridis::scale_color_viridis

#' @importFrom viridis scale_fill_viridis
#' @export
viridis::scale_fill_viridis

# Wrapping vctrs data_frame constructor with no name repair
data_frame0 <- function(...) data_frame(..., .name_repair = "minimal")

# Wrapping unique0() to accept NULL
unique0 <- function(x, ...) if (is.null(x)) x else vec_unique(x, ...)

# Adapted from plyr:::id_vars
# Create a unique id for elements in a single vector
id_var <- function(x, drop = FALSE) {
  if (length(x) == 0) {
    id <- integer()
    n <- 0L
  } else if (!is.null(attr(x, 'n')) && !drop) {
    return(x)
  } else if (is.factor(x) && !drop) {
    x <- addNA(x, ifany = TRUE)
    id <- as.integer(x)
    n <- length(levels(x))
  } else {
    levels <- sort(unique0(x), na.last = TRUE)
    id <- match(x, levels)
    n <- max(id)
  }
  attr(id, 'n') <- n
  id
}
#' Create an unique integer id for each unique row in a data.frame
#'
#' Properties:
#' - `order(id)` is equivalent to `do.call(order, df)`
#' - rows containing the same data have the same value
#' - if `drop = FALSE` then room for all possibilites
#'
#' @param .variables list of variables
#' @param drop Should unused factor levels be dropped?
#'
#' @return An integer vector with attribute `n` giving the total number of
#' possible unique rows
#'
#' @keywords internal
#' @noRd
#'
id <- function(.variables, drop = FALSE) {
  nrows <- NULL
  if (is.data.frame(.variables)) {
    nrows <- nrow(.variables)
    .variables <- unclass(.variables)
  }
  lengths <- vapply(.variables, length, integer(1))
  .variables <- .variables[lengths != 0]
  if (length(.variables) == 0) {
    n <- nrows %||% 0L
    id <- seq_len(n)
    attr(id, 'n') <- n
    return(id)
  }
  if (length(.variables) == 1) {
    return(id_var(.variables[[1]], drop = drop))
  }
  ids <- rev(lapply(.variables, id_var, drop = drop))
  p <- length(ids)
  ndistinct <- vapply(ids, attr, 'n', FUN.VALUE = numeric(1), USE.NAMES = FALSE)
  n <- prod(ndistinct)
  if (n > 2^31) {
    char_id <- inject(paste(!!!ids, sep = '\r'))
    res <- match(char_id, unique0(char_id))
  } else {
    combs <- c(1, cumprod(ndistinct[-p]))
    mat <- inject(cbind(!!!ids))
    res <- c((mat - 1L) %*% combs + 1L)
  }
  if (drop) {
    id_var(res, drop = TRUE)
  } else {
    res <- as.integer(res)
    attr(res, 'n') <- n
    res
  }
}

# Use chartr() for safety since toupper() fails to convert i to I in Turkish locale
lower_ascii <- "abcdefghijklmnopqrstuvwxyz"
upper_ascii <- "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
to_lower_ascii <- function(x) chartr(upper_ascii, lower_ascii, x)
to_upper_ascii <- function(x) chartr(lower_ascii, upper_ascii, x)

tolower <- function(x) {
  cli::cli_abort(
    "Please use {.fn to_lower_ascii}, which works fine in all locales."
  )
}

toupper <- function(x) {
  cli::cli_abort(
    "Please use {.fn to_upper_ascii}, which works fine in all locales."
  )
}

snakeize <- function(x) {
  x <- gsub("([A-Za-z])([A-Z])([a-z])", "\\1_\\2\\3", x)
  x <- gsub(".", "_", x, fixed = TRUE)
  x <- gsub("([a-z])([A-Z])", "\\1_\\2", x)
  to_lower_ascii(x)
}

snake_class <- function(x) {
  snakeize(class(x)[1])
}

#' Layout binary trees in a fractal H formation
#'
#' This is a spac efficient layout only useful for binary trees. It is fractal
#' and works by offsetting child nodes from their parent either horizontally or
#' vertically depending on depth. The offset is decreased at each step by a
#' factor of the square root of 2.
#'
#' @note
#' H Tree is a layout intended for trees, that is, graphs where nodes
#' only have one parent and zero or more children. If the provided graph does
#' not fit this format an attempt to convert it to such a format will be made.
#'
#' @param graph An `tbl_graph` object
#'
#' @param sort.by The name of a node variable to sort the nodes by.
#'
#' @param direction The direction of the tree in the graph. `'out'` (default)
#' means that parents point towards their children, while `'in'` means that
#' children point towards their parent.
#'
#' @param circular Logical. Should the layout be transformed to a circular
#' representation. Ignored
#'
#' @return A data.frame with the columns `x`, `y`, `leaf`, `depth`, `circular`
#' as well as any information stored as node variables in the tbl_graph object.
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom igraph degree count_components
#'
layout_tbl_graph_htree <- function(
  graph,
  sort.by = NULL,
  direction = 'out',
  circular = FALSE
) {
  if (any(degree(graph, mode = direction) > 2)) {
    cli::cli_abort("H-Tree layouts can only be used with binary trees")
  }
  if (count_components(graph, "weak") != 1) {
    cli::cli_abort(
      "H-Tree layouts can only be used with fully connected graphs"
    )
  }
  sort.by <- enquo(sort.by)
  sort.by <- eval_tidy(sort.by, .N())
  hierarchy <- tree_to_hierarchy(graph, direction, sort.by, NULL)
  layout <- hTree(as.integer(hierarchy$parent), as.integer(hierarchy$order))[
    -1,
  ]
  nodes <- data_frame0(
    x = layout[, 1],
    y = layout[, 2],
    circular = FALSE
  )
  nodes$leaf <- degree(graph, mode = direction) == 0
  nodes$depth <- node_depth(graph, mode = direction)
  nodes <- combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
  attr(nodes, 'graph') <- add_direction(graph, nodes)
  nodes
}

#' @rdname ggraph
#'
#' @aliases layout_ggraph
#'
#' @export
create_layout <- function(graph, layout, circular, ...) {
  if (is_layout(layout)) {
    cli::cli_warn(c(
      'Ignoring {.arg graph} as layout is already calculated',
      i = 'Pass the calculated layout to the {.arg graph} argument to silence this warning'
    ))
    return(layout)
  }
  UseMethod('create_layout', graph)
}
is_layout <- function(x) inherits(x, 'layout_ggraph')
#' @rdname ggraph
#' @export
create_layout.default <- function(graph, layout, ...) {
  graph <- try_fetch(as_tbl_graph(graph), error = function(e) {
    cli::cli_abort(
      'No layout function defined for objects of class {.cls {class(graph)[1]}}'
    )
  })
  create_layout(graph, layout, ...)
}
#' @rdname ggraph
#' @export
create_layout.layout_ggraph <- function(graph, ...) {
  graph
}
#' @export
as.data.frame.layout_ggraph <- function(x, ...) {
  extra_attr <- names(attributes(x))
  extra_attr <- extra_attr[!extra_attr %in% c('names', 'row.names')]
  attributes(x)[extra_attr] <- NULL
  class(x) <- 'data.frame'
  x
}
check_layout <- function(layout) {
  if (!is.data.frame(layout)) {
    cli::cli_abort('{.arg layout} must be a {.cls data.frame}')
  }
  if (!(is.numeric(layout$x) && is.numeric(layout$y))) {
    cli::cli_abort(
      '{.arg layout} must contain numeric {.col x} and {.col y} columns'
    )
  }
  graph <- attr(layout, 'graph')
  if (!is.tbl_graph(graph)) {
    cli::cli_abort(
      '{.arg layout} must have a {.cls tbl_graph} as the {.arg graph} attribute'
    )
  }
  if (nrow(layout) != gorder(graph)) {
    cli::cli_abort(
      '{.arg layout} must contain the same number of rows as the number of nodes in the graph'
    )
  }
  if (!'circular' %in% names(layout)) {
    layout$circular <- FALSE
  }
  if (!is.logical(layout$circular)) {
    cli::cli_abort('The {.col circular} column must be logical')
  }
  layout
}

#' Place nodes based on a multidimensional scaling of a set of pivot nodes
#'
#' This layout is similar to the 'mds' layout but uses only a subset of pivot
#' nodes for the mds calculation, making it considerably faster and thus suited
#' for large graphs
#'
#' @param graph A tbl_graph object
#' @param pivots The number of pivot nodes
#' @param weights An expression evaluated on the edge data to provide edge
#' weights for the layout. Currently ignored for the sparse version
#' @param circular ignored
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @references
#' Brandes, U. and Pich, C. (2006). *Eigensolver Methods for Progressive
#' Multidimensional Scaling of Large Data.* In International Symposium on Graph
#' Drawing (pp. 42-53). Springer
#'
#' @family layout_tbl_graph_*
#'
#' @author The underlying algorithm is implemented in the graphlayouts package
#' by David Schoch
#'
#' @importFrom graphlayouts layout_with_pmds
layout_tbl_graph_pmds <- function(
  graph,
  pivots,
  weights = NULL,
  circular = FALSE
) {
  weights <- eval_tidy(enquo(weights), .E())
  if (is.null(weights)) {
    weights <- NA
  }
  xy <- layout_with_pmds(graph, pivots = pivots, weights = weights)
  nodes <- data_frame0(x = xy[, 1], y = xy[, 2], circular = FALSE)
  combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
}

#' Show nodes as a line spanning a horizontal range
#'
#' This geom is most useful together with the [fabric][layout_tbl_graph_fabric]
#' layout for showing the horizontal span of each node.
#'
#' @section Aesthetics:
#' `geom_node_point` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x**
#' - **xend**
#' - **y**
#' - **yend**
#' - alpha
#' - colour
#' - linetype
#' - size
#' - filter
#'
#' @inheritParams ggplot2::geom_linerange
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x is mapped to xmin, xend is mapped to xmax
#' and y and yend are mapped to y in the node data.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_node_*
#'
#' @examples
#' require(tidygraph)
#' gr <- as_tbl_graph(highschool)
#'
#' ggraph(gr, layout = 'fabric') +
#'   geom_node_range()
#' @export
#'
geom_node_range <- function(
  mapping = NULL,
  data = NULL,
  position = 'identity',
  show.legend = NA,
  ...
) {
  mapping <- aes_intersect(mapping, aes(x = xmin, xend = xmax, y = y, yend = y))
  layer(
    data = data,
    mapping = mapping,
    stat = StatFilter,
    geom = GeomSegment,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = list2(...)
  )
}

#' Draw the rectangles in a treemap
#'
#' A treemap is a space filling layout that recursively divides a rectangle to
#' the children of the node. Often only the leaf nodes are drawn as nodes higher
#' up in the hierarchy would obscure what is below. `geom_treemap` is a
#' shorthand for `geom_node_treemap` as node is implicit in the case of
#' treemap drawing
#'
#' @section Aesthetics:
#' `geom_treemap` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **width**
#' - **height**
#' - alpha
#' - colour
#' - fill
#' - size
#' - stroke
#' - filter
#'
#' @inheritParams ggplot2::geom_tile
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x, y, width and height are mapped to
#' x, y, width and height in the node data.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_node_*
#'
#' @examples
#' # Create a graph of the flare class system
#' library(tidygraph)
#' flareGraph <- tbl_graph(flare$vertices, flare$edges) %>%
#'   mutate(
#'     class = map_bfs_chr(node_is_root(), .f = function(node, dist, path, ...) {
#'       if (dist <= 1) {
#'         return(shortName[node])
#'       }
#'       path$result[[nrow(path)]]
#'     })
#'   )
#'
#' ggraph(flareGraph, 'treemap', weight = size) +
#'   geom_node_tile(aes(fill = class, filter = leaf, alpha = depth), colour = NA) +
#'   geom_node_tile(aes(linewidth = depth), colour = 'white') +
#'   scale_alpha(range = c(1, 0.5), guide = 'none') +
#'   scale_size(range = c(4, 0.2), guide = 'none')
#' @export
#'
geom_node_tile <- function(
  mapping = NULL,
  data = NULL,
  position = 'identity',
  show.legend = NA,
  ...
) {
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      width = width,
      height = height
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatFilter,
    geom = GeomNodeTile,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = list2(...)
  )
}

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
#'
GeomNodeTile <- ggproto(
  'GeomNodeTile',
  GeomTile,
  default_aes = combine_aes(
    GeomTile$default_aes,
    aes(fill = NA, colour = 'black', width = 1, height = 1)
  ),
  required_aes = c('x', 'y')
)

#' Quickplot wrapper for networks
#'
#' This function is intended to quickly show an overview of your network data.
#' While it returns a ggraph object that layers etc can be added to it is
#' limited in use and should not be used as a foundation for more complicated
#' plots. It allows colour, labeling and sizing of nodes and edges, and the
#' exact combination of layout and layers will depend on these as well as the
#' features of the network. The output of this function may be fine-tuned at any
#' release and should not be considered stable. If a plot should be reproducible
#' it should be created manually.
#'
#' @param graph An object coercible to a tbl_graph
#' @param ... arguments passed on to methods
#'
#' @importFrom tidygraph as_tbl_graph .register_graph_context graph_is_tree graph_is_forest
#' @importFrom rlang quo_is_null quo_text as_quosure sym
#' @export
#'
#' @examples
#' library(tidygraph)
#' gr <- create_notable('herschel') %>%
#'   mutate(class = sample(letters[1:3], n(), TRUE)) %E>%
#'   mutate(weight = runif(n()))
#'
#' # Standard graph
#' autograph(gr)
#'
#' # Adding node labels will cap edges
#' autograph(gr, node_label = class)
#'
#' # Use tidygraph calls for mapping
#' autograph(gr, node_size = centrality_pagerank())
#'
#' # Trees are plotted as dendrograms
#' iris_tree <- hclust(dist(iris[1:4], method = 'euclidean'), method = 'ward.D2')
#' autograph(iris_tree)
#'
autograph <- function(graph, ...) {
  UseMethod('autograph')
}

#' @export
#' @rdname autograph
#' @param node_colour,edge_colour Colour mapping for nodes and edges
#' @param node_size,edge_width Size/width mapping for nodes and edges
#' @param node_label,edge_label Label mapping for nodes and edges
autograph.default <- function(
  graph,
  ...,
  node_colour = NULL,
  edge_colour = NULL,
  node_size = NULL,
  edge_width = NULL,
  node_label = NULL,
  edge_label = NULL
) {
  node_colour <- enquo(node_colour)
  edge_colour <- enquo(edge_colour)
  node_size <- enquo(node_size)
  edge_width <- enquo(edge_width)
  node_label <- enquo(node_label)
  edge_label <- enquo(edge_label)
  graph <- as_tbl_graph(graph)
  .register_graph_context(graph, TRUE)

  if (graph_is_tree() || graph_is_forest()) {
    node_names <- names(as_tibble(graph, active = 'nodes'))
    height <- if ('height' %in% node_names) quo(height) else quo(NULL)
    p <- ggraph(graph, 'dendrogram', height = !!height) +
      geom_edge_elbow0(aes(colour = !!edge_colour, width = !!edge_width))
    if (!quo_is_null(node_colour) || !quo_is_null(node_size)) {
      p <- p +
        geom_node_point(aes(
          filter = leaf,
          colour = !!node_colour,
          size = !!node_size
        ))
    }
    if (!quo_is_null(node_label)) {
      p <- p +
        geom_node_text(
          aes(filter = leaf, label = !!node_label, colour = !!node_colour),
          angle = 40,
          hjust = 1,
          vjust = 1
        ) +
        coord_cartesian(clip = 'off')
    }
  } else {
    p <- suppressMessages(ggraph(graph) + coord_fixed())

    if (!quo_is_null(node_label)) {
      label_col <- quo_text(node_label)
      label_col <- paste0(c('node1.', 'node2.'), label_col)
      start_label <- sym(label_col[1])
      end_label <- sym(label_col[2])
      p <- p +
        geom_edge_link(
          aes(
            colour = !!edge_colour,
            label = !!edge_label,
            width = !!edge_width,
            start_cap = label_rect(!!start_label),
            end_cap = label_rect(!!end_label)
          ),
          angle_calc = 'along',
          label_dodge = unit(2.5, 'mm')
        )
    } else if (!quo_is_null(edge_label)) {
      p <- p +
        geom_edge_link(
          aes(
            colour = !!edge_colour,
            label = !!edge_label,
            width = !!edge_width
          ),
          angle_calc = 'along',
          label_dodge = unit(2.5, 'mm')
        )
    } else {
      p <- p +
        geom_edge_link0(aes(colour = !!edge_colour, width = !!edge_width))
    }
    if (quo_is_null(node_label)) {
      p <- p +
        geom_node_point(aes(colour = !!node_colour, size = !!node_size))
    } else {
      p <- p +
        geom_node_text(aes(label = !!node_label, colour = !!node_colour))
    }
  }

  p
}

#' Deprecated autograph predecessor
#'
#' @export
#' @keywords internal
qgraph <- function(...) {
  lifecycle::deprecate_warn('2.0.6', 'qgraph()', 'autograph()')
}

utils::globalVariables(c(
  'leaf'
))

#' Create a connection extractor function
#'
#' Connections within the ggraph terminology are links between nodes that are
#' not part of the network structure itself. In that sense connections do not
#' affect the layout calculation in any way and will not be drawn by the
#' standard `geom_edge_*` functions. A connection does not need to only be
#' defined by a start and end node, but can include intermediary nodes.
#' `get_con` helps in creating connection data by letting you specify start
#' and end nodes and automatically finds the shortest path within the graph
#' structure that connects the given points. If this is not what is needed it is
#' also possible to supply a list of vectors giving node indices that define a
#' connection.
#'
#' @param from,to The index of the start and end nodes for the connections
#'
#' @param paths A list of integer vectors giving the index of nodes defining
#' connections
#'
#' @param ... Additional information to be added to the final data output.
#' Accepts expressions that will be evaluated on the node data in it's
#' original order (irrespective of any reordering by the layout)
#'
#' @param weight An expression to be evaluated on the edge data to provide
#' weights for the shortest path calculations
#'
#' @inheritParams igraph::shortest_paths
#'
#' @return A function that takes a layout_ggraph object and returns the given
#' connections
#'
#' @family extractors
#'
#' @export
get_con <- function(
  from = integer(),
  to = integer(),
  paths = NULL,
  ...,
  weight = NULL,
  mode = 'all'
) {
  if (length(from) != length(to)) {
    cli::cli_abort('{.arg from} and {.arg to} must be of equal length')
  }
  dots <- enquos(...)
  function(layout) {
    if (length(from) == 0) {
      return(NULL)
    }
    connections <- collect_connections(
      layout = layout,
      from = from,
      to = to,
      weight = {{ weight }},
      mode = mode
    )
    nodes <- data_frame0(layout)[unlist(connections), ]
    nodes$con.id <- rep(seq_along(connections), lengths(connections))
    if (!is.null(paths)) {
      extra <- data_frame0(layout)[unlist(paths), ]
      extra$con.id <- rep(
        seq_along(paths) + length(connections),
        lengths(paths)
      )
      nodes <- vec_rbind(nodes, extra)
    }
    layout <- layout[order(layout$.ggraph.orig_index), ]
    extra_data <- lapply(dots, function(x) {
      val <- eval_tidy(x, layout)
      rep(val, length.out = length(from))[nodes$con.id]
    })
    if (length(extra_data) > 0) {
      nodes <- cbind(
        nodes,
        data_frame0(!!!extra_data)
      )
    }
    attr(nodes, 'type') <- 'connection_ggraph'
    nodes
  }
}
#' Internal data extractors
#'
#' These functions exists for supporting different data structures. There is no
#' need to call these directly
#'
#' @param layout The layout data
#'
#' @param from,to A numeric vector giving the indexes of the start and end nodes
#'
#' @param ... Additional parameters passed on to the specific method
#'
#' @keywords internal
#' @export
#' @rdname internal_extractors
#' @name internal_extractors
collect_connections <- function(layout, from, to, ...) {
  UseMethod('collect_connections', layout)
}
#' @export
collect_connections.default <- function(layout, ...) {
  cli::cli_abort(
    'Don\'t know how to get connections from an object of class {.cls {class(layout)[1]}}'
  )
}

#' Friendship among high school boys
#'
#' This dataset shows the friendship among high school boys as assessed by the
#' question: "What fellows here in school do you go around with most often?".
#' The question was posed twice, with one year in between (1957 and 1958) and
#' shows the evolution in friendship between the two timepoints.
#'
#' @format
#' The graph is stored as an unnamed edgelist with a year attribute.
#' \describe{
#'  \item{from}{The boy answering the question}
#'  \item{to}{The boy being the answer to the question}
#'  \item{year}{The year the friendship was reported}
#' }
#'
#' @source
#' Coleman, J. S. *Introduction to Mathematical Sociology*. New York: Free
#' Press, pp.450-451.
#'
'highschool'

#' Membership network of American Whigs
#'
#' This dataset shows the membership of 136 colonial Americans in 5 whig
#' organization and is a bipartite graph. The data appeared in the appendix to
#' David Hackett Fischer's *Paul Revere's Ride* (Oxford University Press,
#' 1995) and compiled by Kieran Healy for the blog post
#' [Using Metadata to Find Paul Revere](https://kieranhealy.org/blog/archives/2013/06/09/using-metadata-to-find-paul-revere/).
#'
#' @format
#' The data is stored as an incidence matrix with persons as rows and
#' organizations as columns. A 0 means no membership while a one means
#' membership.
#'
#' @source <https://github.com/kjhealy/revere/blob/master/data/PaulRevereAppD.csv>
#' adapted from:
#'
#' Fischer, David H. (1995) *Paul Revere's Ride*. Oxford University Press
#'
'whigs'

#' Place nodes on a line or circle
#'
#' This layout puts all nodes on a line, possibly sorted by a node attribute. If
#' `circular = TRUE` the nodes will be laid out on the unit circle instead.
#' In the case where the `sort.by` attribute is numeric, the numeric values
#' will be used as the x-position and it is thus possible to have uneven spacing
#' between the nodes.
#'
#' @param graph An `tbl_graph` object
#'
#' @param circular Logical. Should the layout be transformed to a circular
#' representation. Defaults to `FALSE`.
#'
#' @param sort.by The name of a node variable to sort the nodes by.
#'
#' @param use.numeric Logical. Should a numeric sort.by attribute be used as the
#' actual x-coordinates in the layout. May lead to overlapping nodes. Defaults
#' to FALSE
#'
#' @param offset If `circular = TRUE`, where should it begin. Defaults to
#' `pi/2` which is equivalent to 12 o'clock.
#'
#' @param weight A weight for each node. Nodes will be spread out according to
#' their weight so that nodes with heigher weight will have more space around
#' them. Ignored if `use.numeric = TRUE`
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#' Further, if `circular = FALSE` a `width` column and if `circular = TRUE` a
#' `start`, `end`, and `r0` column.
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom igraph gorder
#'
layout_tbl_graph_linear <- function(
  graph,
  circular,
  sort.by = NULL,
  use.numeric = FALSE,
  offset = pi / 2,
  weight = NULL
) {
  sort.by <- enquo(sort.by)
  sort.by <- eval_tidy(sort.by, .N())
  weight <- enquo(weight)
  weight <- eval_tidy(weight, .N())

  if (use.numeric && !is.null(weight)) {
    cli::cli_warn('Ignoring {.arg weight} when {.code use.numeric = TRUE}')
  }
  weight <- weight %||% rep(1, gorder(graph))

  if (length(weight) != gorder(graph)) {
    cli::cli_abort('{.arg weight} must match the order of the graph')
  }
  if (any(weight < 0)) {
    cli::cli_abort("{.arg weight} must be positive")
  }

  if (!is.null(sort.by)) {
    if (length(sort.by) != gorder(graph)) {
      cli::cli_abort('{.arg sort.by} must match the order of the graph')
    }
    if (is.numeric(sort.by) && use.numeric) {
      x <- sort.by
      weight <- 0
    } else {
      x <- c(0, cumsum(weight[order(sort.by)]))
      x <- (x[-1] + x[-length(x)]) / 2
      x <- x[order(order(sort.by))]
    }
  } else {
    x <- c(0, cumsum(weight))
    x <- (x[-1] + x[-length(x)]) / 2
  }
  nodes <- data_frame0(x = x, y = 0, width = weight)
  if (circular) {
    full_width <- sum(nodes$width)
    if (full_width > 0) {
      nodes$start <- 2 * pi * (nodes$x - nodes$width / 2) / full_width
      nodes$end <- 2 * pi * (nodes$x + nodes$width / 2) / full_width
    } else {
      full_width <- max(nodes$x)
      nodes$start <- 2 * pi * nodes$x / full_width
      nodes$end <- nodes$start
    }
    mid <- (nodes$start + nodes$end) / 2
    nodes$x <- sin(mid)
    nodes$y <- cos(mid)
    nodes$r0 <- 1
  }
  nodes <- combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
  nodes$circular <- circular
  nodes
}

#' Bundle edges using force directed edge bundling
#'
#' This geom performs force directed edge bundling to reduce visual clutter.
#' It uses a self-organizing approach to bundling in which edges are modeled as
#' flexible springs that can attract each other without the need of a hierarchy.
#' Be aware that this bundling technique works globally and thus may bundle
#' edges that is otherwise unrelated together. Care should be taken when
#' interpreting the resulting visual. An alternative approach to edge bundling
#' that uses the graph topology is provided by [geom_edge_bundle_path()].
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_bundle_force` and `geom_edge_bundle_force0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_bundle_force2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **group**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_bundle_force` and `geom_edge_bundle_force2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#'
#' @param force The spring force during bundling
#' @param n_cycle number of iteration cycles
#' @param cuts_start initial number of edge divisions
#' @param step initial step size
#' @param cuts_new factor for how many new division points to add after a cycle
#' @param n_iter number of iteration steps per cycle
#' @param iter_new factor of how to decrease the number of iterations per cycle
#' @param threshold threshold for considering two edges to be interacting
#' @param eps tolerance
#'
#' @author David Schoch
#'
#' @family geom_edge_*
#'
#' @references
#' Holten, D. and Wijk, J.J.V. (2009). *Force‐Directed Edge Bundling for Graph
#' Visualization.* Computer Graphics Forum (Blackwell Publishing Ltd) 28, no. 3:
#' 983-990. https://doi.org/10.1111/j.1467-8659.2009.01450.x
#'
#' @rdname geom_edge_bundle_force
#' @name geom_edge_bundle_force
#'
#' @examples
#' # (not necessarily an insightful use)
#' ggraph(highschool) +
#'   geom_edge_bundle_force(n_cycle = 2, threshold = 0.4)
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBspline
#' @export
StatEdgeBundleForce <- ggproto(
  "StatEdgeBundleForce",
  Stat,
  setup_data = function(data, params) {
    StatEdgeBundleForce0$setup_data(data, params)
  },
  compute_panel = function(
    data,
    scales,
    n = 100,
    force = 1,
    n_cycle = 6,
    cuts_start = 1,
    step = 0.04,
    cuts_new = 2,
    n_iter = 50,
    iter_new = 2 / 3,
    threshold = 0.6,
    eps = 1e-8
  ) {
    edges <- StatEdgeBundleForce0$compute_panel(
      data,
      scales,
      force = force,
      n_cycle = n_cycle,
      cuts_start = cuts_start,
      step = step,
      cuts_new = cuts_new,
      n_iter = n_iter,
      iter_new = iter_new,
      threshold = threshold,
      eps = eps
    )
    StatBspline$compute_layer(edges, list(n = n), NULL)
  },
  required_aes = c("x", "y", "xend", "yend"),
  default_aes = aes(filter = TRUE),
  extra_params = c("na.rm")
)

#' @rdname geom_edge_bundle_force
#'
#' @export
geom_edge_bundle_force <- function(
  mapping = NULL,
  data = get_edges(),
  position = "identity",
  arrow = NULL,
  n = 100,
  force = 1,
  n_cycle = 6,
  cuts_start = 1,
  step = 0.04,
  cuts_new = 2,
  n_iter = 50,
  iter_new = 2 / 3,
  threshold = 0.6,
  eps = 1e-8,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      group = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeBundleForce,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = FALSE,
        force = force,
        n_cycle = n_cycle,
        cuts_start = cuts_start,
        step = step,
        cuts_new = cuts_new,
        n_iter = n_iter,
        iter_new = iter_new,
        threshold = threshold,
        eps = eps,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBspline
#' @export
StatEdgeBundleForce2 <- ggproto(
  "StatEdgeBundleForce2",
  Stat,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    remove_loop2(data)
  },
  compute_panel = function(
    data,
    scales,
    n = 100,
    force = 1,
    n_cycle = 6,
    cuts_start = 1,
    step = 0.04,
    cuts_new = 2,
    n_iter = 50,
    iter_new = 2 / 3,
    threshold = 0.6,
    eps = 1e-8
  ) {
    data <- data[order(data$group), ]
    edges <- cbind(
      data$x[c(TRUE, FALSE)],
      data$y[c(TRUE, FALSE)],
      data$x[c(FALSE, TRUE)],
      data$y[c(FALSE, TRUE)]
    )
    edges <- force_bundle_mem(
      edges,
      K = force,
      C = n_cycle,
      P = cuts_start,
      S = step,
      P_rate = cuts_new,
      I = n_iter,
      I_rate = iter_new,
      compatibility_threshold = threshold,
      eps = eps
    )
    edges$PANEL <- data$PANEL[1]
    edges$group <- data$group[edges$group * 2]
    edges <- StatBspline$compute_layer(edges, list(n = n), NULL)
    extra_data <- data[1, !names(data) %in% c("x", "y", "group", "PANEL")][
      rep(NA, nrow(edges)),
    ]
    edges$.interp <- TRUE
    ends <- !duplicated(edges$group) | !duplicated(edges$group, fromLast = TRUE)
    edges$.interp[ends] <- FALSE
    extra_data[ends, ] <- data[, names(extra_data)]
    cbind(edges, extra_data)
  },
  required_aes = c("x", "y"),
  default_aes = aes(filter = TRUE),
  extra_params = c("na.rm")
)

#' @rdname geom_edge_bundle_force
#'
#' @export
geom_edge_bundle_force2 <- function(
  mapping = NULL,
  data = get_edges("long"),
  position = "identity",
  arrow = NULL,
  n = 100,
  force = 1,
  n_cycle = 6,
  cuts_start = 1,
  step = 0.04,
  cuts_new = 2,
  n_iter = 50,
  iter_new = 2 / 3,
  threshold = 0.6,
  eps = 1e-8,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      group = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeBundleForce2,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = TRUE,
        force = force,
        n_cycle = n_cycle,
        cuts_start = cuts_start,
        step = step,
        cuts_new = cuts_new,
        n_iter = n_iter,
        iter_new = iter_new,
        threshold = threshold,
        eps = eps,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
StatEdgeBundleForce0 <- ggproto(
  'StatEdgeBundleForce0',
  Stat,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    remove_loop(data)
  },
  compute_panel = function(
    data,
    scales,
    force = 1,
    n_cycle = 6,
    cuts_start = 1,
    step = 0.04,
    cuts_new = 2,
    n_iter = 50,
    iter_new = 2 / 3,
    threshold = 0.6,
    eps = 1e-8
  ) {
    edges <- cbind(data$x, data$y, data$xend, data$yend)
    edges <- force_bundle_mem(
      edges,
      K = force,
      C = n_cycle,
      P = cuts_start,
      S = step,
      P_rate = cuts_new,
      I = n_iter,
      I_rate = iter_new,
      compatibility_threshold = threshold,
      eps = eps
    )
    edges$PANEL <- data$PANEL[1]
    edges$group <- data$group[edges$group]
    cbind(
      edges,
      data[
        edges$group,
        !names(data) %in% c("x", "y", "xend", "yend", "PANEL", "group")
      ]
    )
  },
  required_aes = c('x', 'y', 'xend', 'yend'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm')
)
#' @rdname geom_edge_bundle_force
#'
#' @export
geom_edge_bundle_force0 <- function(
  mapping = NULL,
  data = get_edges(),
  position = "identity",
  arrow = NULL,
  force = 1,
  n_cycle = 6,
  cuts_start = 1,
  step = 0.04,
  cuts_new = 2,
  n_iter = 50,
  iter_new = 2 / 3,
  threshold = 0.6,
  eps = 1e-8,
  lineend = 'butt',
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      group = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeBundleForce0,
    geom = GeomEdgeBspline,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        force = force,
        n_cycle = n_cycle,
        cuts_start = cuts_start,
        step = step,
        cuts_new = cuts_new,
        n_iter = n_iter,
        iter_new = iter_new,
        threshold = threshold,
        eps = eps,
        ...
      )
    )
  )
}

force_bundle <- function(
  data,
  K,
  C,
  P,
  S,
  P_rate,
  I,
  I_rate,
  compatibility_threshold,
  eps
) {
  # initialize matrix with coordinates
  m <- nrow(data)

  mode(data) <- 'numeric'

  # main force bundling routine
  force_bundle_iter(
    data,
    as.numeric(K),
    as.integer(C),
    as.integer(P),
    as.integer(P_rate),
    as.numeric(S),
    as.integer(I),
    as.numeric(I_rate),
    as.numeric(compatibility_threshold),
    as.numeric(eps)
  )
}

force_bundle_mem <- memoise(force_bundle)

#' Bundle edges using edge path bundling
#'
#' This geom performs edge bundling using the edge path algorithm. This approach
#' uses the underlying graph structure to find shortest paths for each edge in
#' a graph the is gradually removed of it's edges. Since it is based on the
#' topology of the graph it should lead to less spurious bundling of unrelated
#' edges compared to [geom_edge_bundle_force()] and also has a simpler parameter
#' space.
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_force_path` and `geom_edge_force_path0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - **edge_id** (should not be overwritten)
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_force_path2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **group**
#' - **edge_id** (should not be overwritten)
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_force_path` and `geom_edge_force_path2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#'
#' @param directed Logical. Should the shortest paths be calculated using
#' direction information of the graph. Setting this to `TRUE` can help split up
#' bundles that flows in opposite directions. Ignored for undirected graphs
#' @param max_distortion A multiplication factor to determine the maximum
#' allowed distortion of the path during bundling. If the new edge is longer
#' than `max_distortion` times the old length it is rejected.
#' @param weight_fac The exponent used to assign weights to the graph when
#' calculating the shortest path. The final weights are given as
#' `edge_length ^ weight_fac` meaning that sorter edges are prioritised when
#' calculating the weights.
#' @param tension A loosening factor when calculating the b-spline of the edge
#' based on the shortest path. Will move control points closer and closer to
#' the direct line as it approaches 0
#'
#' @author Thomas Lin Pedersen and David Schoch
#'
#' @family geom_edge_*
#'
#' @references
#' Wallinger, M., Archambault, D., Auber, D., Nöllenburg, M., and Peltonen, J.
#' (2022). *Edge-Path Bundling: A Less Ambiguous Edge Bundling Approach.* IEEE
#' Transactions on Visualization and Computer Graphics 28(1) 313-323.
#' https://doi.org/10.1109/TVCG.2021.3114795
#'
#' @rdname geom_edge_bundle_path
#' @name geom_edge_bundle_path
#'
#' @examples
#' ggraph(highschool) +
#'   geom_edge_bundle_path()
#'
#' # Use tension to lessen the effect
#' ggraph(highschool) +
#'   geom_edge_bundle_path(tension = 0.8)
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBspline
#' @export
StatEdgeBundlePath <- ggproto(
  "StatEdgeBundlePath",
  Stat,
  setup_data = function(data, params) {
    StatEdgeBundlePath0$setup_data(data, params)
  },
  compute_panel = function(
    data,
    scales,
    n = 100,
    directed = NULL,
    max_distortion = 2,
    weight_fac = 2,
    tension = 1
  ) {
    edges <- StatEdgeBundlePath0$compute_panel(
      data,
      scales,
      directed = directed,
      max_distortion = max_distortion,
      weight_fac = weight_fac,
      tension = tension
    )
    StatBspline$compute_layer(edges, list(n = n), NULL)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'edge_id'),
  default_aes = aes(filter = TRUE),
  extra_params = c("na.rm")
)

#' @rdname geom_edge_bundle_path
#'
#' @export
geom_edge_bundle_path <- function(
  mapping = NULL,
  data = get_edges(),
  position = "identity",
  arrow = NULL,
  n = 100,
  directed = NULL,
  max_distortion = 2,
  weight_fac = 2,
  tension = 1,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      group = edge.id,
      edge_id = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeBundlePath,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = FALSE,
        directed = directed,
        max_distortion = max_distortion,
        weight_fac = weight_fac,
        tension = tension,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBspline
#' @export
StatEdgeBundlePath2 <- ggproto(
  "StatEdgeBundlePath2",
  Stat,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    remove_loop2(data)
  },
  compute_panel = function(
    data,
    scales,
    n = 100,
    directed = NULL,
    max_distortion = 2,
    weight_fac = 2,
    tension = 1
  ) {
    graph <- .G()
    nodes <- data_frame0(x = .N()$.ggraph_layout_x, y = .N()$.ggraph_layout_y)
    data <- data[order(data$group), ]
    edge_id <- data$edge_id[c(TRUE, FALSE)]
    edges <- path_bundle_mem(
      graph,
      nodes,
      .E()$from[edge_id],
      .E()$to[edge_id],
      directed = directed,
      max_distortion = max_distortion,
      weight_fac = weight_fac
    )
    if (tension < 1) {
      edges <- relax(edges, tension)
    }
    edges$PANEL <- data$PANEL[1]
    edges$group <- data$group[edges$group * 2]
    edges <- StatBspline$compute_layer(edges, list(n = n), NULL)
    extra_data <- data[1, !names(data) %in% c("x", "y", "group", "PANEL")][
      rep(NA, nrow(edges)),
    ]
    edges$.interp <- TRUE
    ends <- !duplicated(edges$group) | !duplicated(edges$group, fromLast = TRUE)
    edges$.interp[ends] <- FALSE
    extra_data[ends, ] <- data[, names(extra_data)]
    cbind(edges, extra_data)
  },
  required_aes = c("x", "y", "edge_id"),
  default_aes = aes(filter = TRUE),
  extra_params = c("na.rm")
)

#' @rdname geom_edge_bundle_path
#'
#' @export
geom_edge_bundle_path2 <- function(
  mapping = NULL,
  data = get_edges("long"),
  position = "identity",
  arrow = NULL,
  n = 100,
  directed = NULL,
  max_distortion = 2,
  weight_fac = 2,
  tension = 1,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      group = edge.id,
      edge_id = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeBundlePath2,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = TRUE,
        directed = directed,
        max_distortion = max_distortion,
        weight_fac = weight_fac,
        tension = tension,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
StatEdgeBundlePath0 <- ggproto(
  'StatEdgeBundlePath0',
  Stat,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    remove_loop(data)
  },
  compute_panel = function(
    data,
    scales,
    directed = NULL,
    max_distortion = 2,
    weight_fac = 2,
    tension = 1
  ) {
    graph <- .G()
    nodes <- data_frame0(x = .N()$.ggraph_layout_x, y = .N()$.ggraph_layout_y)
    from <- .E()$from[data$edge_id]
    to <- .E()$to[data$edge_id]
    edges <- path_bundle_mem(
      graph,
      nodes,
      from,
      to,
      directed = directed,
      max_distortion = max_distortion,
      weight_fac = weight_fac
    )
    if (tension < 1) {
      edges <- relax(edges, tension)
    }
    edges$PANEL <- data$PANEL[1]
    edges$group <- data$group[edges$group]
    cbind(
      edges,
      data[
        edges$group,
        !names(data) %in% c("x", "y", "xend", "yend", "PANEL", "group")
      ]
    )
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'edge_id'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm')
)
#' @rdname geom_edge_bundle_path
#'
#' @export
geom_edge_bundle_path0 <- function(
  mapping = NULL,
  data = get_edges(),
  position = "identity",
  arrow = NULL,
  directed = NULL,
  max_distortion = 2,
  weight_fac = 2,
  tension = 1,
  lineend = 'butt',
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      group = edge.id,
      edge_id = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeBundlePath0,
    geom = GeomEdgeBspline,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        directed = directed,
        max_distortion = max_distortion,
        weight_fac = weight_fac,
        tension = tension,
        ...
      )
    )
  )
}
#' @importFrom igraph gsize delete_edges shortest_paths is_directed as_edgelist
path_bundle <- function(
  graph,
  nodes,
  from,
  to,
  directed = directed,
  max_distortion = 2,
  weight_fac = 2
) {
  m <- gsize(graph)
  lock <- rep(FALSE, m)
  skip <- rep(FALSE, m)

  edge_length <- sqrt(
    (nodes$x[from] - nodes$x[to])^2 + (nodes$y[from] - nodes$y[to])^2
  )
  weights <- edge_length^weight_fac
  edges_order <- order(weights, decreasing = TRUE)
  paths <- vector("list", m)
  if (is_directed(graph)) {
    directed <- directed %||% TRUE
  } else if (!is.null(directed)) {
    cli::cli_warn("Ignoring {.arg directed} for undirected graphs")
  } else {
    directed <- FALSE
  }
  mode <- if (is.null(directed) || !directed) "all" else "out"

  all_edges <- as_edgelist(graph)
  if (directed) {
    all_edges <- paste(all_edges[, 1], "-", all_edges[, 2])
  } else {
    all_edges <- paste(
      pmin(all_edges[, 1], all_edges[, 2]),
      "-",
      pmax(all_edges[, 1], all_edges[, 2])
    )
  }
  # iterate
  for (e in edges_order) {
    s <- from[e]
    t <- to[e]
    paths[[e]] <- c(s, t)
    if (lock[e]) {
      next()
    }
    skip[e] <- TRUE
    g_temp <- delete_edges(graph, which(skip))
    path <- suppressWarnings(shortest_paths(
      g_temp,
      s,
      t,
      weights = weights[!skip],
      mode = mode,
      output = "vpath"
    )$vpath[[1]])
    if (length(path) < 2) {
      skip[e] <- FALSE
      next()
    }
    path_length <- sum(sqrt(
      (nodes$x[path[-length(path)]] - nodes$x[path[-1]])^2 +
        (nodes$y[path[-length(path)]] - nodes$y[path[-1]])^2
    ))
    if (path_length >= max_distortion * edge_length[e]) {
      skip[e] <- FALSE
      next()
    }
    all_edges_on_path <- rep(as.integer(path), each = 2)
    all_edges_on_path <- matrix(
      all_edges_on_path[-c(1, length(all_edges_on_path))],
      ncol = 2,
      byrow = TRUE
    )
    if (directed) {
      all_edges_on_path <- paste(
        all_edges_on_path[, 1],
        "-",
        all_edges_on_path[, 2]
      )
    } else {
      all_edges_on_path <- paste(
        pmin(all_edges_on_path[, 1], all_edges_on_path[, 2]),
        "-",
        pmax(all_edges_on_path[, 1], all_edges_on_path[, 2])
      )
    }
    all_edges_on_path <- all_edges %in% all_edges_on_path
    lock[all_edges_on_path] <- TRUE
    paths[[e]] <- path
  }
  ids <- rep(seq_along(from), lengths(paths))
  paths <- unlist(paths)
  data_frame0(x = nodes$x[paths], y = nodes$y[paths], group = ids)
}

path_bundle_mem <- memoise(path_bundle)

#' The class hierarchy of the flare visualization library
#'
#' This dataset contains the graph that describes the class hierarchy for the
#' [Flare](https://blokt.com/tool/prefuse-flare) ActionScript visualization library. It
#' contains both the class hierarchy as well as the import connections between
#' classes. This dataset has been used extensively in the D3.js documentation
#' and examples and are included here to make it easy to redo the examples in
#' ggraph.
#'
#' @format A list of three data.frames describing the software structure of
#' flare:
#' \describe{
#'  \item{edges}{This data.frame maps the hierarchical structure of the class
#'  hierarchy as an edgelist, with the class in `from` being the superclass
#'  of the class in `to`.}
#'  \item{vertices}{This data.frame gives additional information on the classes.
#'  It contains the full name, size and short name of each class.}
#'  \item{imports}{This data.frame contains the class imports for each class
#'  implementation. The `from` column gives the importing class and the
#'  `to` column gives the import.}
#' }
#'
#' @source The data have been adapted from the JSON downloaded from
#' <https://gist.github.com/mbostock/1044242#file-readme-flare-imports-json>
#' courtesy of Mike Bostock. The Flare framework is the work of the
#' [UC Berkeley Visualization Lab](http://vis.berkeley.edu/).
#'
'flare'

#' Place nodes in a Hive Plot layout
#'
#' Hive plots were invented by Martin Krzywinski as a perceptually uniform and
#' scalable alternative to standard node-edge layouts. In hive plots nodes are
#' positioned on axes radiating out from a center based on their own information
#' e.g. membership of a class, size of neighborhood, etc. Edges are then drawn
#' between nodes as bezier curves. As the placement of nodes is not governed by
#' convoluted algorithms but directly reflects the qualities of the nodes itself
#' the resulting plot can be easier to interpret as well as compare to other
#' graphs.
#'
#' @details
#' In order to be able to draw all edges without edges crossing axes you should
#' not assign nodes to axes based on a variable with more than three levels.
#'
#' @param graph An `tbl_graph` object
#'
#' @param axis The node attribute to use for assigning nodes to axes
#'
#' @param axis.pos The relative distance to the prior axis. Default
#' (`NULL`) places axes equidistant.
#'
#' @param sort.by The node attribute to use for placing nodes along their axis.
#' Defaults (`NULL`) places nodes sequentially.
#'
#' @param divide.by An optional node attribute to subdivide each axis by.
#'
#' @param divide.order The order the axis subdivisions should appear in
#'
#' @param normalize Logical. Should axis lengths be equal or reflect the number
#' of nodes in each axis. Defaults to `TRUE`.
#'
#' @param center.size The size of the blank center, that is, the start position
#' of the axes.
#'
#' @param divide.size The distance between subdivided axis segments.
#'
#' @param use.numeric Logical, If the `sort.by` attribute is numeric,
#' should these values be used directly in positioning the nodes along the axes.
#' Defaults to `FALSE` which sorts the numeric values and positions them
#' equidistant from each other.
#'
#' @param offset Change the overall rotation of the hive plot by changing the
#' offset of the first axis.
#'
#' @param split.axes Should axes be split to show edges between nodes on the
#' same axis? One of:
#' \describe{
#'   \item{`'none'`}{Do not split axes and show in-between edges}
#'   \item{`'loops'`}{Only split axes that contain in-between edges}
#'   \item{`'all'`}{Split all axes}
#' }
#'
#' @param split.angle The angular distance between the two axes resulting from a
#' split.
#'
#' @param circular Ignored.
#'
#' @return A data.frame with the columns `x`, `y`, `r`,
#' `center_size`, `split`, `axis`, `section`, `angle`,
#' `circular` as well as any information stored as node variables in the
#' tbl_graph object.
#'
#' @references
#' Krzywinski, M., Birol, I., Jones, SJM., and Marra, MA. (2012). *Hive
#' plots-rational approach to visualizing networks*. Brief Bioinform 13 (5):
#' 627-644. https://doi.org/10.1093/bib/bbr069
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom igraph gorder vertex_attr gsize induced_subgraph add_vertices E ends add_edges delete_edges %--% edge_attr
#' @importFrom utils tail
layout_tbl_graph_hive <- function(
  graph,
  axis,
  axis.pos = NULL,
  sort.by = NULL,
  divide.by = NULL,
  divide.order = NULL,
  normalize = TRUE,
  center.size = 0.1,
  divide.size = 0.05,
  use.numeric = FALSE,
  offset = pi / 2,
  split.axes = 'none',
  split.angle = pi / 6,
  circular = FALSE
) {
  axis <- enquo(axis)
  axis <- eval_tidy(axis, .N())
  sort.by <- enquo(sort.by)
  sort.by <- eval_tidy(sort.by, .N())
  divide.by <- enquo(divide.by)
  divide.by <- eval_tidy(divide.by, .N())

  axes <- split(seq_len(gorder(graph)), axis)
  if (is.null(axis.pos)) {
    axis.pos <- rep(1, length(axes))
  } else {
    if (length(axis.pos) != length(axes)) {
      cli::cli_warn(
        'Number of axes {.val {length(axes)}} not matching {.arg axis.pos} argument. Recycling as needed'
      )
      axis.pos <- rep(axis.pos, length.out = length(axes))
    }
  }
  axis.pos <- -cumsum(axis.pos)
  axis.pos <- c(0, axis.pos[-length(axis.pos)]) /
    -tail(axis.pos, 1) *
    2 *
    pi +
    offset
  if (use.numeric) {
    if (is.null(sort.by) || !is.numeric(sort.by)) {
      cli::cli_abort(
        '{.arg sort.by} must be a numeric vertex attribute when {.code use.numeric = TRUE}'
      )
    }
    numeric.range <- range(sort.by)
  }
  if (normalize) {
    normalize_to <- rep(1, length(axes))
  } else {
    normalize_to <- lengths(axes) / max(lengths(axes))
  }
  node.pos <- Map(
    function(nodes, axis_length, axis, angle) {
      if (length(nodes) == 0) {
        return(data_frame0())
      }
      split_axis <- switch(
        split.axes,
        all = TRUE,
        loops = gsize(induced_subgraph(graph, nodes)) > 0,
        none = FALSE,
        cli::cli_abort(
          'Unknown {.arg split} argument. Use {.val all}, {.val loops} or {.val none}'
        )
      )
      node_div <- axis_length / length(nodes)
      if (is.null(divide.by)) {
        node_split <- list(`1` = nodes)
      } else {
        if (use.numeric) {
          cli::cli_abort('Cannot divide axis when {.code use.numeric = TRUE}')
        }
        node_split <- split(nodes, divide.by[nodes])
        if (!is.null(divide.order)) {
          if (!all(divide.order %in% names(node_split))) {
            cli::cli_abort(
              'All {.arg divide.by} levels must be present in {.arg divide.order}'
            )
          }
          node_split <- node_split[order(match(
            names(node_split),
            divide.order
          ))]
        }
      }
      node_pos <- lapply(node_split, function(nodes) {
        if (length(nodes) == 0) {
          return(numeric())
        }
        if (is.null(sort.by)) {
          pos <- match(seq_along(nodes), order(nodes)) - 1
          pos <- pos * node_div
        } else {
          pos <- sort.by[nodes]
          if (use.numeric) {
            if (!is.numeric(pos)) {
              cli::cli_abort(
                '{.arg sort.by} must contain numeric data when {.code use.numeric = TRUE}'
              )
            }
            if (normalize) {
              if (diff(range(pos)) == 0) {
                pos <- rep(0.5, length.out = length(pos))
              } else {
                pos <- (pos - min(pos)) / diff(range(pos))
              }
            } else {
              pos <- (pos - numeric.range[1]) / diff(numeric.range)
            }
          } else {
            pos <- match(seq_along(pos), order(pos)) - 1
            pos <- pos * node_div
          }
        }
        pos
      })
      node_pos <- Reduce(
        function(l, r) {
          append(l, list(r + node_div + divide.size + max(l[[length(l)]])))
        },
        x = node_pos[-1],
        init = node_pos[1]
      )
      node_pos <- unlist(node_pos) + center.size

      data_frame0(
        node = nodes,
        r = node_pos[match(nodes, unlist(node_split))],
        center_size = center.size,
        split = split_axis,
        axis = axis,
        section = rep(names(node_split), lengths(node_split))[match(
          nodes,
          unlist(node_split)
        )],
        angle = angle,
        circular = FALSE
      )
    },
    nodes = axes,
    axis_length = normalize_to,
    axis = names(axes),
    angle = axis.pos
  )
  for (i in seq_along(node.pos)) {
    if (nrow(node.pos[[i]]) > 0 && node.pos[[i]]$split[1]) {
      n_new_nodes <- nrow(node.pos[[i]])
      new_node_start <- gorder(graph) + 1
      extra_nodes <- node.pos[[i]]
      extra_nodes$node <- seq(new_node_start, length.out = n_new_nodes)
      vattr <- lapply(vertex_attr(graph), `[`, i = node.pos[[i]]$node)
      graph <- add_vertices(graph, n_new_nodes, attr = vattr)

      loop_edges <- E(graph)[node.pos[[i]]$node %--% node.pos[[i]]$node]
      if (length(loop_edges) != 0) {
        loop_edges_ends <- ends(graph, loop_edges, names = FALSE)
        correct_order_ends <- node.pos[[i]]$r[match(
          loop_edges_ends[, 1],
          node.pos[[i]]$node
        )] <
          node.pos[[i]]$r[match(loop_edges_ends[, 2], node.pos[[i]]$node)]
        loop_edges_ends <- data_frame0(
          from = ifelse(
            correct_order_ends,
            loop_edges_ends[, 1],
            loop_edges_ends[, 2]
          ),
          to = ifelse(
            correct_order_ends,
            loop_edges_ends[, 2],
            loop_edges_ends[, 1]
          )
        )
        loop_edges_ends$to <- extra_nodes$node[match(
          loop_edges_ends$to,
          node.pos[[i]]$node
        )]
        loop_edges_ends <- matrix(
          c(
            ifelse(
              correct_order_ends,
              loop_edges_ends$from,
              loop_edges_ends$to
            ),
            ifelse(correct_order_ends, loop_edges_ends$to, loop_edges_ends$from)
          ),
          nrow = 2,
          byrow = TRUE
        )
        eattr <- lapply(edge_attr(graph), `[`, i = as.numeric(loop_edges))
        graph <- add_edges(graph, as.vector(loop_edges_ends), attr = eattr)
        graph <- delete_edges(graph, as.numeric(loop_edges))
      }

      node_correction <- unlist(lapply(node.pos[-i], function(ax) {
        correct <- if (ax$angle[1] < node.pos[[i]]$angle[1]) {
          ax$angle[1] - node.pos[[i]]$angle[1] < -pi
        } else {
          ax$angle[1] - node.pos[[i]]$angle[1] < pi
        }
        if (correct) ax$node
      }))
      if (length(node_correction) != 0) {
        correct_edges <- E(graph)[node.pos[[i]]$node %--% node_correction]
        correct_edges_ends <- ends(graph, correct_edges, names = FALSE)
        new_node_ind <- correct_edges_ends %in% node.pos[[i]]$node
        correct_edges_ends[new_node_ind] <- extra_nodes$node[match(
          correct_edges_ends[new_node_ind],
          node.pos[[i]]$node
        )]
        eattr <- lapply(edge_attr(graph), `[`, i = as.numeric(correct_edges))
        graph <- add_edges(
          graph,
          as.vector(t(correct_edges_ends)),
          attr = eattr
        )
        graph <- delete_edges(graph, as.numeric(correct_edges))
      }

      node.pos[[i]]$angle <- node.pos[[i]]$angle - split.angle / 2
      extra_nodes$angle <- extra_nodes$angle + split.angle / 2
      node.pos <- append(node.pos, list(extra_nodes))
    }
  }
  node.pos <- lapply(node.pos, function(nodes) {
    if (nrow(nodes) > 0) {
      nodes$x <- nodes$r * cos(nodes$angle)
      nodes$y <- nodes$r * sin(nodes$angle)
    }
    nodes
  })
  node.pos <- vec_rbind(!!!node.pos)
  node.pos <- node.pos[order(node.pos$node), names(node.pos) != 'node']
  extra_data <- as_tibble(as_tbl_graph(graph), active = 'nodes')
  node.pos <- cbind(
    node.pos,
    extra_data[, !names(extra_data) %in% names(node.pos), drop = FALSE]
  )
  attr(node.pos, 'graph') <- as_tbl_graph(graph)
  node.pos
}

#' Draw edges as Arcs
#'
#' This geom is mainly intended for arc linear and circular diagrams (i.e. used
#' together with [layout_tbl_graph_linear()]), though it can be used
#' elsewhere. It draws edges as arcs with a height proportional to the distance
#' between the nodes. Arcs are calculated as beziers. For linear layout the
#' placement of control points are related to the `curvature` argument and
#' the distance between the two nodes. For circular layout the control points
#' are placed on the same angle as the start and end node at a distance related
#' to the distance between the nodes.
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_arc` and `geom_edge_arc0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - **circular**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_arc2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **group**
#' - **circular**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_arc` and `geom_edge_arc2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#'
#' @param strength The bend of the curve. 1 approximates a halfcircle while 0
#' will give a straight line. Negative number will change the direction of the
#' curve. Only used if `circular = FALSE`.
#'
#' @param fold Logical. Should arcs appear on the same side of the nodes despite
#' different directions. Default to `FALSE`.
#'
#' @param curvature Deprecated. Use `strength` instead.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' # Make a graph with different directions of edges
#' gr <- create_notable('Meredith') %>%
#'   convert(to_directed) %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE)) %>%
#'   activate(edges) %>%
#'   mutate(
#'     class = sample(letters[1:3], n(), replace = TRUE),
#'     switch = sample(c(TRUE, FALSE), n(), replace = TRUE)
#'   ) %>%
#'   reroute(from = to, to = from, subset = switch)
#'
#' ggraph(gr, 'linear') +
#'   geom_edge_arc(aes(alpha = after_stat(index)))
#'
#' ggraph(gr, 'linear') +
#'   geom_edge_arc2(aes(colour = node.class), strength = 0.6)
#'
#' ggraph(gr, 'linear', circular = TRUE) +
#'   geom_edge_arc0(aes(colour = class))
#' @rdname geom_edge_arc
#' @name geom_edge_arc
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier
#' @export
StatEdgeArc <- ggproto(
  'StatEdgeArc',
  StatBezier,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data <- remove_loop(data)
    if (nrow(data) == 0) {
      return(data)
    }
    data$group <- make_unique(data$group)
    data2 <- data
    data2$x <- data2$xend
    data2$y <- data2$yend
    create_arc(data, data2, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'circular'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n', 'strength', 'fold', 'curvature')
)
#' @rdname geom_edge_arc
#'
#' @export
geom_edge_arc <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  n = 100,
  fold = FALSE,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...,
  curvature
) {
  if (!missing(curvature)) {
    lifecycle::deprecate_warn(
      '2.0.0',
      'geom_edge_arc(curvature)',
      'geom_edge_arc(strength)'
    )
    strength <- curvature
  }
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      circular = circular,
      group = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeArc,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = FALSE,
        strength = strength,
        fold = fold,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier2
#' @export
StatEdgeArc2 <- ggproto(
  'StatEdgeArc2',
  StatBezier2,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data <- remove_loop2(data)
    if (nrow(data) == 0) {
      return(data)
    }
    data <- data[order(data$group), ]
    data2 <- data[c(FALSE, TRUE), ]
    data <- data[c(TRUE, FALSE), ]
    create_arc(data, data2, params)
  },
  required_aes = c('x', 'y', 'group', 'circular'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n', 'strength', 'fold', 'curvature')
)
#' @rdname geom_edge_arc
#'
#' @export
geom_edge_arc2 <- function(
  mapping = NULL,
  data = get_edges('long'),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  n = 100,
  fold = FALSE,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...,
  curvature
) {
  if (!missing(curvature)) {
    lifecycle::deprecate_warn(
      '2.0.0',
      'geom_edge_arc2(curvature)',
      'geom_edge_arc2(strength)'
    )
    strength <- curvature
  }
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      group = edge.id,
      circular = circular
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeArc2,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = TRUE,
        strength = strength,
        fold = fold,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier0
#' @export
StatEdgeArc0 <- ggproto(
  'StatEdgeArc0',
  StatBezier0,
  setup_data = function(data, params) {
    StatEdgeArc$setup_data(data, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'circular'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'strength', 'fold', 'curvature')
)
#' @rdname geom_edge_arc
#'
#' @export
geom_edge_arc0 <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  lineend = 'butt',
  show.legend = NA,
  fold = fold,
  ...,
  curvature
) {
  if (!missing(curvature)) {
    lifecycle::deprecate_warn(
      '2.0.0',
      'geom_edge_arc0(curvature)',
      'geom_edge_arc0(strength)'
    )
    strength <- curvature
  }
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      circular = circular
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeArc0,
    geom = GeomEdgeBezier,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        strength = strength,
        fold = FALSE,
        ...
      )
    )
  )
}

create_arc <- function(from, to, params) {
  bezier_start <- seq(1, by = 4, length.out = nrow(from))
  from$index <- bezier_start
  to$index <- bezier_start + 3
  data2 <- from
  data3 <- to
  data2$index <- bezier_start + 1
  data3$index <- bezier_start + 2
  node_dist <- sqrt((to$x - from$x)^2 + (to$y - from$y)^2) / 2
  circ <- from$circular
  if (any(circ)) {
    r0 <- sqrt(to$x[circ]^2 + to$y[circ]^2)
    r1 <- sqrt(to$x[circ]^2 + to$y[circ]^2)

    data2$x[circ] <- from$x[circ] * (1 - (node_dist[circ] / r0))
    data2$y[circ] <- from$y[circ] * (1 - (node_dist[circ] / r0))
    data3$x[circ] <- to$x[circ] * (1 - (node_dist[circ] / r1))
    data3$y[circ] <- to$y[circ] * (1 - (node_dist[circ] / r1))
  }
  if (any(!circ)) {
    strength <- pi / 2 * -params$strength
    edge_angle <- atan2(
      to$y[!circ] - from$y[!circ],
      to$x[!circ] - from$x[!circ]
    )
    start_angle <- edge_angle - strength
    end_angle <- edge_angle - pi + strength
    data2$x[!circ] <- data2$x[!circ] + cos(start_angle) * node_dist[!circ]
    data2$y[!circ] <- data2$y[!circ] + sin(start_angle) * node_dist[!circ]
    data3$x[!circ] <- data3$x[!circ] + cos(end_angle) * node_dist[!circ]
    data3$y[!circ] <- data3$y[!circ] + sin(end_angle) * node_dist[!circ]
    if (params$fold) {
      # data2$x[!circ] <- abs(data2$x[!circ]) * sign(params$strength)
      data2$y[!circ] <- abs(data2$y[!circ]) * sign(params$strength)
      # data3$x[!circ] <- abs(data3$x[!circ]) * sign(params$strength)
      data3$y[!circ] <- abs(data3$y[!circ]) * sign(params$strength)
    }
  }
  data <- vec_rbind(from, data2, data3, to)
  data[order(data$index), names(data) != 'index']
}

#' Place nodes on their geographical space
#'
#' This layout is built for objects of class `sfnetwork` and is meant to
#' plot a graph on its geographical space, by extracting its X and Y coordinates
#'
#' @param graph An sfnetwork object
#' @param circular ignored
#'
#' @importFrom tidygraph as_tbl_graph
#'
#' @return A data.frame with the columns `x`, `y`, `circular`.
#'
#' @family layout_tbl_graph_*
layout_tbl_graph_sf <- function(graph, circular = FALSE) {
  # Check the presence of sf.
  check_installed("sf")
  # Check if network is an sfnetwork
  if (!inherits(graph, "sfnetwork")) {
    cli::cli_abort("The 'sf' layout needs an {.cls sfnetwork} graph.")
  }
  # Extract X and Y coordinates from the nodes
  graph <- activate(graph, "nodes")
  x <- sf::st_coordinates(graph)[, "X"]
  y <- sf::st_coordinates(graph)[, "Y"]
  # Create layout data frame
  nodes <- data_frame0(x = x, y = y, circular = FALSE)
  # Convert to tbl_graph to 'unstick' geometry column
  extra_data <- as_tibble(as_tbl_graph(graph), active = "nodes")
  nodes <- combine_layout_nodes(nodes, extra_data)
  attr(nodes, 'graph') <- graph
  nodes
}

#' Edge alpha scales
#'
#' This set of scales defines new alpha scales for edge geoms equivalent to the
#' ones already defined by ggplot2. See [ggplot2::scale_alpha()] for
#' more information. The different geoms will know whether to use edge scales or
#' the standard scales so it is not necessary to write `edge_alpha` in
#' the call to the geom - just use `alpha`.
#'
#' @return A ggproto object inheriting from `Scale`
#'
#' @family scale_edge_*
#'
#' @name scale_edge_alpha
#' @rdname scale_edge_alpha
#'
NULL

#' @rdname scale_edge_alpha
#'
#' @inheritParams ggplot2::scale_alpha
#'
#' @importFrom scales rescale_pal
#' @export
scale_edge_alpha <- function(..., range = c(0.1, 1)) {
  sc <- scale_alpha(..., range = range)
  sc$aesthetics <- 'edge_alpha'
  sc
}

#' @rdname scale_edge_alpha
#'
#' @export
scale_edge_alpha_continuous <- scale_edge_alpha

#' @rdname scale_edge_alpha
#'
#'
#' @inheritParams ggplot2::scale_alpha_discrete
#'
#' @export
scale_edge_alpha_discrete <- function(..., range = c(0.1, 1)) {
  sc <- scale_alpha_discrete(..., range = range)
  sc$aesthetics <- 'edge_alpha'
  sc
}

#' @rdname scale_edge_alpha
#'
#'
#' @inheritParams ggplot2::scale_alpha_binned
#'
#' @export
scale_edge_alpha_binned <- function(..., range = c(0.1, 1)) {
  sc <- scale_alpha_binned(..., range = range)
  sc$aesthetics <- 'edge_alpha'
  sc
}

#' @rdname scale_edge_alpha
#'
#' @inheritParams ggplot2::scale_alpha_manual
#'
#' @export
scale_edge_alpha_manual <- function(
  ...,
  values,
  breaks = waiver(),
  na.value = NA
) {
  sc <- scale_alpha_manual(
    ...,
    values = values,
    breaks = breaks,
    na.value = na.value
  )
  sc$aesthetics <- 'edge_alpha'
  sc
}

#' @rdname scale_edge_alpha
#'
#' @inheritParams ggplot2::scale_alpha_identity
#'
#' @importFrom scales identity_pal
#' @export
scale_edge_alpha_identity <- function(..., guide = 'none') {
  sc <- scale_alpha_identity(..., guide = guide)
  sc$aesthetics <- 'edge_alpha'
  sc
}

#' @export
#' @keywords internal
makeContent.parallelPath <- function(x) {
  will_cap <- inherits(x, 'cappedpathgrob')
  if (!is.null(x$id)) {
    sep <- x$sep[!duplicated(x$id)]
  }
  x <- NextMethod()

  if (will_cap) {
    x$children[[1]]$sep <- sep[x$children[[1]]$id]
    if (inherits(x$children[[1]], 'segments')) {
      x$children[[1]] <- shift_segments(x$children[[1]])
    } else {
      x$children[[1]] <- shift_paths(x$children[[1]])
    }
    x
  } else if (inherits(x, 'segments')) {
    shift_segments(x)
  } else {
    shift_paths(x)
  }
}

shift_segments <- function(x) {
  x0_new <- convertX(x$x0, 'mm', TRUE)
  x1_new <- convertX(x$x1, 'mm', TRUE)
  y0_new <- convertY(x$y0, 'mm', TRUE)
  y1_new <- convertY(x$y1, 'mm', TRUE)

  sep <- convertWidth(x$sep, 'mm', TRUE)

  xdiff <- x1_new - x0_new
  ydiff <- y1_new - y0_new
  lengths <- sqrt(xdiff * xdiff + ydiff * ydiff)
  xshift <- -ydiff / lengths * sep
  yshift <- xdiff / lengths * sep

  x$x0 <- unit(x0_new + xshift, 'mm')
  x$x1 <- unit(x1_new + xshift, 'mm')
  x$y0 <- unit(y0_new + yshift, 'mm')
  x$y1 <- unit(y1_new + yshift, 'mm')

  x
}

shift_paths <- function(x) {
  first <- which(!duplicated(x$id))
  last <- which(!duplicated(x$id, fromLast = TRUE))

  sep <- convertWidth(x$sep[first], 'mm', TRUE)

  x_new <- convertX(x$x, 'mm', TRUE)
  y_new <- convertY(x$y, 'mm', TRUE)
  xdiff <- x_new[last] - x_new[first]
  ydiff <- y_new[last] - y_new[first]
  lengths <- sqrt(xdiff * xdiff + ydiff * ydiff)
  xshift <- -ydiff / lengths * sep
  yshift <- xdiff / lengths * sep

  x$x <- unit(x_new + xshift[x$id], 'mm')
  x$y <- unit(y_new + yshift[x$id], 'mm')

  x
}

#' Draw edges as LINESTRINGs in geographical space
#'
#' This geom is equivalent in functionality to [ggplot2::geom_sf()] for `LINESTRING`
#' geometries and allows for plotting of edges in their geographical space in
#' different colours, linetypes and widths.
#'
#' @section Aesthetics:
#' `geom_edge_sf` understand the following aesthetics.
#'
#' - alpha
#' - colour
#' - linetype
#' - filter
#'
#' @inheritParams ggplot2::geom_sf
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default geometry is mapped to the geometry in
#' the edge data.
#'
#' @author Lorena Abad
#'
#' @family geom_edge_*
#'
#' @examples
#' if (require("sfnetworks", quietly = TRUE)) {
#'   gr <- sfnetworks::as_sfnetwork(roxel)
#'   ggraph(gr, 'sf') + geom_edge_sf()
#' }
#'
#' @export
#'
geom_edge_sf <- function(
  mapping = NULL,
  data = get_sf_edges(),
  position = 'identity',
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(geometry = geometry))
  c(
    layer_sf(
      geom = GeomEdgeSf,
      data = data,
      mapping = mapping,
      stat = StatFilterSf,
      position = position,
      show.legend = show.legend,
      inherit.aes = FALSE,
      params = expand_edge_aes(list2(na.rm = FALSE, ...))
    ),
    coord_sf(default = TRUE)
  )
}
#' @rdname get_edges
get_sf_edges <- function() {
  function(layout) {
    edges <- sf::st_as_sf(attr(layout, "graph"), "edges")
    attr(edges, 'type_ggraph') <- 'edge_ggraph'
    edges
  }
}

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
GeomEdgeSf = ggproto(
  "GeomEdgeSf",
  GeomSf,
  draw_panel = function(data, panel_params, coords) {
    names(data) <- sub('edge_', '', names(data))
    names(data)[names(data) == 'width'] <- 'linewidth'
    GeomSf$draw_panel(data, panel_params, coords)
  },
  draw_key = GeomEdgePath$draw_key,
  default_aes = aes(
    edge_colour = 'black',
    edge_width = 0.5,
    edge_linetype = 1,
    edge_alpha = NA
  ),
  rename_size = FALSE
)

#' Show nodes as voronoi tiles
#'
#' This geom is equivalent in functionality to [ggforce::geom_voronoi_tile()]
#' and allows for plotting of nodes as tiles from a voronoi tesselation. As with
#' [ggforce::geom_voronoi_tile()] it is possible to restrict the size of the
#' tile to a fixed radius, as well as round corners and expand/contract the
#' tile.
#'
#' @section Aesthetics:
#' `geom_node_voronoi` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - alpha
#' - colour
#' - fill
#' - shape
#' - size
#' - stroke
#' - filter
#'
#' @inheritParams ggforce::geom_voronoi_tile
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x and y are mapped to x and y in
#' the node data and group set to `-1`.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_node_*
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('meredith') %>%
#'   mutate(group = sample(letters[1:4], n(), TRUE))
#'
#' ggraph(gr) +
#'   geom_node_voronoi(aes(fill = group, colour = group), alpha = 0.3) +
#'   geom_edge_link(alpha = 0.3) +
#'   geom_node_point()
#'
#' # Use max.radius to make the tesselation more "node"-like
#' ggraph(gr) +
#'   geom_node_voronoi(aes(fill = group, colour = group), alpha = 0.3, max.radius = 1) +
#'   geom_edge_link(alpha = 0.3) +
#'   geom_node_point()
#' @export
#' @importFrom ggforce GeomShape
#'
geom_node_voronoi <- function(
  mapping = NULL,
  data = NULL,
  position = 'identity',
  show.legend = NA,
  bound = NULL,
  eps = 1e-09,
  max.radius = NULL,
  normalize = FALSE,
  asp.ratio = 1,
  expand = 0,
  radius = 0,
  ...
) {
  mapping <- aes_intersect(mapping, aes(x = x, y = y, group = -1))
  layer(
    data = data,
    mapping = mapping,
    stat = StatNodeVoronoi,
    geom = GeomShape,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = list2(
      bound = bound,
      eps = eps,
      max.radius = max.radius,
      normalize = normalize,
      asp.ratio = asp.ratio,
      expand = expand,
      radius = radius,
      ...
    )
  )
}

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatVoronoiTile
#' @export
StatNodeVoronoi <- ggproto(
  'StatNodeVoronoi',
  StatVoronoiTile,
  finish_layer = function(data, params) {
    StatFilter$setup_data(data, params)
  },
  default_aes = aes(filter = TRUE)
)

utils::globalVariables(c(
  'angle',
  'center_size',
  'circular',
  'con.id',
  'direction',
  'edge.id',
  'edge_x',
  'end',
  'from',
  'height',
  'r',
  'r0',
  'section',
  'start',
  'to',
  'width',
  'x',
  'xend',
  'xmin',
  'xmax',
  'y',
  'yend'
))

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
StatFilter <- ggproto(
  'StatFilter',
  StatIdentity,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        cli::cli_abort('{.field filter} must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    data
  },
  default_aes = aes(filter = TRUE)
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
StatReverse <- ggproto(
  'StatReverse',
  StatFilter,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data[rev(seq_len(nrow(data))), ]
  }
)

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
StatFilterSf <- ggproto(
  'StatFilterSf',
  StatSf,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        cli::cli_abort('{.field filter} must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    data
  },
  default_aes = aes(filter = TRUE)
)

aes_intersect <- function(aes1, aes2) {
  aes <- c(as.list(aes1), aes2[!names(aes2) %in% names(aes1)])
  class(aes) <- 'uneval'
  aes
}

data_type <- function(data) {
  type <- attr(data, 'type_ggraph')
  if (is.null(type)) {
    if (inherits(data, 'layout_ggraph')) {
      return('node_ggraph')
    } else {
      return('other')
    }
  }
  type
}

# non-orderaltering version of make.unique
make_unique <- function(x, sep = '.') {
  if (!anyDuplicated(x)) {
    return(x)
  }
  groups <- match(x, unique0(x))
  suffix <- unsplit(lapply(split(x, groups), seq_along), groups)
  max_chars <- nchar(max(suffix))
  suffix_format <- paste0('%0', max_chars, 'd')
  paste0(x, sep, sprintf(suffix_format, suffix))
}

warn_dropped_vars <- function(layout, data) {
  overlap <- intersect(names(layout), names(data))
  if (
    length(overlap) > 0 &&
      !identical(as.list(layout[overlap]), as.list(data[overlap]))
  ) {
    cli::cli_warn(
      'Existing variables {.var {overlap}} overwritten by layout variables'
    )
  }
}

combine_layout_nodes <- function(layout, nodes) {
  warn_dropped_vars(layout, nodes)
  cbind(layout, nodes[, !names(nodes) %in% names(layout), drop = FALSE])
}

empty_data <- function(x) {
  length(x) == 0 || nrow(x) == 0
}

combine_aes <- function(aes1, aes2) {
  aes_all <- c(aes1[setdiff(names(aes1), names(aes2))], aes2)
  class(aes_all) <- class(aes1)
  aes_all
}

#' Edge shape scales
#'
#' This set of scales defines new shape scales for edge geoms equivalent to the
#' ones already defined by ggplot2. See [ggplot2::scale_shape()] for
#' more information. The different geoms will know whether to use edge scales or
#' the standard scales so it is not necessary to write `edge_shape` in
#' the call to the geom - just use `shape`.
#'
#' @param guide Guide to use for this scale.
#'
#' @return A ggproto object inheriting from `Scale`
#'
#' @family scale_edge_*
#'
#' @name scale_edge_shape
#' @rdname scale_edge_shape
#'
NULL

#' @rdname scale_edge_shape
#'
#' @inheritParams ggplot2::scale_shape
#'
#' @export
scale_edge_shape <- function(..., solid = TRUE) {
  sc <- scale_shape(..., solid = solid)
  sc$aesthetics <- 'edge_shape'
  sc
}
#' @rdname scale_edge_shape
#'
#' @export
scale_edge_shape_discrete <- scale_edge_shape
#' @rdname scale_edge_shape
#'
#' @export
scale_edge_shape_continuous <- function(...) {
  cli::cli_abort(c(
    "A continuous variable cannot be mapped to the {.field edge_shape} aesthetic",
    "i" = "choose a different aesthetic or use {.fn scale_edge_shape_binned}"
  ))
}
#' @rdname scale_edge_shape
#'
#' @export
scale_edge_shape_binned <- function(..., solid = TRUE) {
  sc <- scale_shape_binned(..., solid = solid)
  sc$aesthetics <- 'edge_shape'
  sc
}
#' @rdname scale_edge_shape
#'
#' @inheritParams ggplot2::scale_shape_manual
#'
#' @export
scale_edge_shape_manual <- function(
  ...,
  values,
  breaks = waiver(),
  na.value = NA
) {
  sc <- scale_shape_manual(
    ...,
    values = values,
    breaks = breaks,
    na.value = na.value
  )
  sc$aesthetics <- 'edge_shape'
  sc
}
#' @rdname scale_edge_shape
#'
#' @inheritParams ggplot2::scale_shape_identity
#'
#' @importFrom scales identity_pal
#' @export
scale_edge_shape_identity <- function(..., guide = 'none') {
  sc <- scale_shape_identity(..., guide = guide)
  sc$aesthetics <- 'edge_shape'
  sc
}

#' Create a grid of small multiples by node and/or edge attributes
#'
#' This function is equivalent to [ggplot2::facet_grid()] in that it
#' allows for building a grid of small multiples where rows and columns
#' correspond to a specific data value. While [ggplot2::facet_grid()]
#' could be used it would lead to unexpected results as it is not possible to
#' specify whether you are referring to a node or an edge attribute. Furthermore
#' [ggplot2::facet_grid()] will draw edges in panels even though the
#' panel does not contain both terminal nodes. `facet_graph` takes care of
#' all of these issues, allowing you to define which data type the rows and
#' columns are referencing as well as filtering the edges based on the nodes in
#' each panel (even when nodes are not drawn).
#'
#' @param row_type,col_type Either `'node'` or `'edge'`. Which data
#' type is being facetted in the rows and columns. Default is to facet on nodes
#' column wise and on edges row wise.
#'
#' @inheritParams ggplot2::facet_grid
#'
#' @family ggraph-facets
#'
#' @examples
#' library(tidygraph)
#' gr <- as_tbl_graph(highschool) %>%
#'   mutate(popularity = as.character(cut(centrality_degree(mode = 'in'),
#'     breaks = 3,
#'     labels = c('low', 'medium', 'high')
#'   )))
#' ggraph(gr) +
#'   geom_edge_link() +
#'   geom_node_point() +
#'   facet_graph(year ~ popularity)
#' @export
#'
facet_graph <- function(
  facets,
  row_type = 'edge',
  col_type = 'node',
  margins = FALSE,
  scales = 'fixed',
  space = 'fixed',
  shrink = TRUE,
  labeller = 'label_value',
  as.table = TRUE,
  switch = NULL,
  drop = TRUE
) {
  facet <- facet_grid(
    facets,
    margins = margins,
    scales = scales,
    space = space,
    shrink = shrink,
    labeller = labeller,
    as.table = as.table,
    switch = switch,
    drop = drop
  )
  seed <- sample(.Machine$integer.max, 1L)
  facet$params$rows[] <- lapply(seq_along(facet$params$rows), function(i) {
    rlang::quo(withr::with_seed(seed, !!facet$params$rows[[i]]))
  })
  facet$params$cols[] <- lapply(seq_along(facet$params$cols), function(i) {
    rlang::quo(withr::with_seed(seed, !!facet$params$cols[[i]]))
  })

  ggproto(
    NULL,
    FacetGraph,
    shrink = shrink,
    params = c(
      facet$params,
      list(
        row_type = row_type,
        col_type = col_type
      )
    )
  )
}

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
FacetGraph <- ggproto(
  'FacetGraph',
  FacetGrid,
  compute_layout = function(self, data, params) {
    plot_data <- data[[1]]
    data <- split(data, vapply(data, data_type, character(1)))

    rows <- params$rows
    cols <- params$cols
    row_data <- switch(
      params$row_type,
      node = data$node_ggraph,
      edge = data$edge_ggraph,
      cli::cli_abort(
        '{.arg row_type} must be either {.val node} or {.val edge}'
      )
    )
    col_data <- switch(
      params$col_type,
      node = data$node_ggraph,
      edge = data$edge_ggraph,
      cli::cli_abort(
        '{.arg col_type} must be either {.val node} or {.val edge}'
      )
    )

    dups <- intersect(names(rows), names(cols))
    if (length(dups) > 0 && params$col_type == params$row_type) {
      cli::cli_abort(
        c(
          "Faceting variables can only appear in {.arg rows} or {.arg cols}, not both.\n",
          "i" = "Duplicated variables: {.val {dups}}"
        ),
        call = call2(snake_class(self))
      )
    }

    base_rows <- combine_vars(
      row_data,
      params$plot_env,
      rows,
      drop = params$drop
    )
    if (!params$as.table) {
      rev_order <- function(x) factor(x, levels = rev(ulevels(x)))
      base_rows[] <- lapply(base_rows, rev_order)
    }
    base_cols <- combine_vars(
      col_data,
      params$plot_env,
      cols,
      drop = params$drop
    )
    base <- df.grid(base_rows, base_cols)

    # Add margins
    base <- reshape_add_margins(
      base,
      list(names(rows), names(cols)),
      params$margins
    )
    base <- unique0(base)

    # Create panel info dataset
    panel <- id(base, drop = TRUE)
    panel <- factor(panel, levels = seq_len(attr(panel, 'n')))

    rows <- if (length(names(rows)) == 0) {
      1L
    } else {
      id(base[names(rows)], drop = TRUE)
    }
    cols <- if (length(names(cols)) == 0) {
      1L
    } else {
      id(base[names(cols)], drop = TRUE)
    }

    panels <- data_frame0(PANEL = panel, ROW = rows, COL = cols, base)
    panels <- panels[order(panels$PANEL), , drop = FALSE]
    rownames(panels) <- NULL

    panels$SCALE_X <- if (params$free$x) panels$COL else 1L
    panels$SCALE_Y <- if (params$free$y) panels$ROW else 1L

    node_placement <- if (nrow(base) == 0) {
      list(`1` = plot_data$.ggraph_index)
    } else {
      if (params$row_type == 'edge') {
        params$rows <- NULL
      }
      if (params$col_type == 'edge') {
        params$cols <- NULL
      }
      if (length(params$cols) == 0 && length(params$rows) == 0) {
        node_map <- rep(list(node_map$.ggraph.index), nrow(panels))
        names(node_map) <- panels$PANEL
        node_map
      } else {
        node_map <- FacetGrid$map_data(plot_data, panels, params)
        node_map <- expand_facet_map(node_map, panels)
        node_map <- node_map[order(node_map$.ggraph.index), , drop = FALSE]
        split(node_map$.ggraph.index, node_map$PANEL)
      }
    }

    attr(panels, 'node_placement') <- node_placement
    panels
  },
  map_data = function(data, layout, params) {
    switch(
      data_type(data),
      edge_ggraph = {
        if (params$row_type == 'node') {
          params$rows <- NULL
        }
        if (params$col_type == 'node') {
          params$cols <- NULL
        }
        if (length(params$cols) == 0 && length(params$rows) == 0) {
          edge_map <- cbind(
            vec_rep(data, nrow(layout)),
            PANEL = rep(layout$PANEL, each = nrow(data))
          )
        } else {
          edge_map <- FacetGrid$map_data(data, layout, params)
          edge_map <- expand_facet_map(edge_map, layout)
        }
        edge_map <- Map(
          function(map, nodes) {
            map[map$from %in% nodes & map$to %in% nodes, , drop = FALSE]
          },
          map = split(edge_map, edge_map$PANEL),
          nodes = attr(layout, 'node_placement')
        )
        vec_rbind(!!!edge_map)
      },
      node_ggraph = {
        node_map <- lapply(attr(layout, 'node_placement'), function(nodes) {
          data[data$.ggraph.index %in% nodes, , drop = FALSE]
        })
        panel <- rep(seq_along(node_map), vapply(node_map, nrow, numeric(1)))
        node_map <- vec_rbind(!!!node_map)
        node_map$PANEL <- factor(panel, levels = levels(layout$PANEL))
        node_map
      },
      {
        FacetGrid$map_data(data, layout, params)
      }
    )
  }
)

expand_facet_map <- function(map, layout) {
  ncols <- max(layout$COL)
  nrows <- max(layout$ROW)
  if (!(ncols == 1 && nrows == 1)) {
    map$PANEL[map$PANEL == -1] <- 1
  }
  if (ncols != 1) {
    data_cols <- unique0(layout$COL[layout$PANEL %in% map$PANEL])
    if (length(data_cols) == 1) {
      map <- lapply(split(map, as.integer(map$PANEL)), function(data) {
        row <- layout$ROW[layout$PANEL == data$PANEL[1]]
        panels <- layout$PANEL[layout$ROW == row]
        data_expand <- data[
          rep(seq_len(nrow(data)), length(panels)),
          ,
          drop = FALSE
        ]
        data_expand$PANEL <- rep(panels, each = nrow(data))
        data_expand
      })
      map <- vec_rbind(!!!map)
    }
  }
  if (nrows != 1) {
    data_rows <- unique0(layout$ROW[layout$PANEL %in% map$PANEL])
    if (length(data_rows) == 1) {
      map <- lapply(split(map, as.integer(map$PANEL)), function(data) {
        col <- layout$COL[layout$PANEL == data$PANEL[1]]
        panels <- layout$PANEL[layout$COL == col]
        data_expand <- data[
          rep(seq_len(nrow(data)), length(panels)),
          ,
          drop = FALSE
        ]
        data_expand$PANEL <- rep(panels, each = nrow(data))
        data_expand
      })
      map <- vec_rbind(!!!map)
    }
  }
  map$PANEL <- factor(map$PANEL, levels = levels(layout$PANEL))
  map
}

# From ggplot2
df.grid <- function(a, b) {
  if (is.null(a) || nrow(a) == 0) {
    return(b)
  }
  if (is.null(b) || nrow(b) == 0) {
    return(a)
  }

  indexes <- expand.grid(
    i_a = seq_len(nrow(a)),
    i_b = seq_len(nrow(b))
  )
  grid <- cbind(
    a[indexes$i_a, , drop = FALSE],
    b[indexes$i_b, , drop = FALSE]
  )
  attr(grid, 'row.names') <- .set_row_names(.row_names_info(grid, 2L))
  grid
}
ulevels <- function(x) {
  if (is.factor(x)) {
    x <- addNA(x, TRUE)
    factor(levels(x), levels(x), exclude = NULL)
  } else {
    sort(unique0(x))
  }
}

#' Show nodes as POINTs in geographical space
#'
#' This geom is equivalent in functionality to [ggplot2::geom_sf()] for `POINT`
#' geometries and allows for plotting of nodes in their geographical space in
#' different shapes, colours and sizes.
#'
#' @section Aesthetics:
#' `geom_node_sf` understand the following aesthetics.
#'
#' - alpha
#' - colour
#' - shape
#' - size
#' - filter
#'
#' @inheritParams ggplot2::geom_sf
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default geometry is mapped to the geometry in
#' the node data.
#'
#' @author Lorena Abad
#'
#' @family geom_node_*
#'
#' @examples
#' library(tidygraph)
#'
#' if (require("sfnetworks", quietly = TRUE)) {
#'   gr <- sfnetworks::as_sfnetwork(roxel)
#'   ggraph(gr, 'sf') +
#'     geom_node_sf(aes(color = centrality_betweenness()))
#' }
#'
#' @export
#'
geom_node_sf <- function(
  mapping = NULL,
  data = get_sf_nodes(),
  position = 'identity',
  show.legend = NA,
  ...
) {
  mapping <- aes_intersect(mapping, aes(geometry = geometry))
  c(
    layer_sf(
      geom = GeomSf,
      data = data,
      mapping = mapping,
      stat = StatFilterSf,
      position = position,
      show.legend = show.legend,
      inherit.aes = FALSE,
      params = list2(na.rm = FALSE, ...)
    ),
    coord_sf(default = TRUE)
  )
}

#' @rdname get_nodes
get_sf_nodes <- function() {
  function(layout) {
    nodes <- sf::st_as_sf(layout)
    attr(nodes, 'type_ggraph') <- 'node_ggraph'
    nodes
  }
}

#' Create a ggraph plot
#'
#' This function is the equivalent of [ggplot2::ggplot()] in ggplot2.
#' It takes care of setting up the plot object along with creating the layout
#' for the plot based on the graph and the specification passed in.
#' Alternatively a layout can be prepared in advance using
#' `create_layout` and passed as the data argument. See *Details* for
#' a description of all available layouts.
#'
#' @details
#' Following is a short description of the different layout types available in
#' ggraph. Each layout is further described in its own help pages. Any type of
#' regular graph/network data can be represented as a tbl_graph object. Because
#' of this the different layouts that can be applied to tbl_graph
#' objects are quite diverse, but not all layouts makes sense to all types of
#' graphs. It is up to the user to understand their data and choose an
#' appropriate layout. For standard node-edge diagrams igraph defines a
#' long range of different layout functions that are all available through the
#' `igraph` layout where the specific layout is specified using the
#' `algorithm` argument. In order to minimize typing all igraph algorithms
#' can also be passed directly into the `layout` argument.
#'
#' Any object that has an appropriate `as_tbl_graph` method can be passed
#' into `ggraph()` and will automatically be converted underneath.
#'
#' \describe{
#'   \item{`auto`}{The default layout. See [layout_tbl_graph_auto()] for further
#'   details}
#'   \item{`igraph`}{Use one of the internal igraph layout algorithms.
#'   The algorithm is specified using the `algorithm` argument. All strings
#'   accepted by the `algorithm` argument can also be supplied directly
#'   into `layout`. See [layout_tbl_graph_igraph()] for further
#'   details}
#'   \item{`dendrogram`}{Lays out the nodes in a tree-like graph as a
#'   dendrogram with leaves set at 0 and parents 1 unit above its tallest child.
#'   See [layout_tbl_graph_dendrogram()] for further details}
#'   \item{`manual`}{Lets the user manually specify the location of each
#'   node. See [layout_tbl_graph_manual()] for further details}
#'   \item{`linear`}{Arranges the nodes linearly or circularly in order to
#'   make an arc diagram. See [layout_tbl_graph_linear()] for further
#'   details}
#'   \item{`matrix`}{Arranges nodes on a diagonal thus preparing it for use with
#'   [geom_edge_point()] to make a matrix plot. See [layout_tbl_graph_matrix()]
#'   for further details}
#'   \item{`treemap`}{Creates a treemap from the graph, that is, a
#'   space-filing subdivision of rectangles showing a weighted hierarchy. See
#'   [layout_tbl_graph_treemap()] for further details}
#'   \item{`circlepack`}{Creates a layout showing a hierarchy as circles
#'   within circles. Conceptually equal to treemaps. See
#'   [layout_tbl_graph_circlepack()] for further details}
#'   \item{`partition`}{Create icicle or sunburst charts, where each layer
#'   subdivides the division given by the preceding layer. See
#'   [layout_tbl_graph_partition()] for further details}
#'   \item{`hive`}{Positions nodes on axes spreading out from the center
#'   based on node attributes. See [layout_tbl_graph_hive()] for further
#'   details}
#'   \item{`cactustree`}{Positions nodes as circles on the periphery of their
#'   parent circle. See [layout_tbl_graph_cactustree()] for further
#'   details}
#'   \item{`backbone`}{Layout optimised for highly connected small-world graphs
#'   such as social networks. See [layout_tbl_graph_backbone()] for further
#'   details}
#'   \item{`centrality`}{Place nodes around origin based on their centrality.
#'   See [layout_tbl_graph_centrality()] for further
#'   details}
#'   \item{`eigen`}{Spectral layout based on the eigenvector of a matrix
#'   representation of the graph. See [layout_tbl_graph_eigen()] for further
#'   details}
#'   \item{`fabric`}{Draw nodes as horizontal lines and connect them with
#'   vertical lines if an edge exists between them. See
#'   [layout_tbl_graph_fabric()] for further
#'   details}
#'   \item{`focus`}{Place nodes around a focus node based on their distance to
#'   that node. See [layout_tbl_graph_focus()] for further
#'   details}
#'   \item{`pmds`}{Layout based on multidimensional scaling of a set of pivot
#'   nodes, allowing MDS layout to be used on larger graphs. See
#'   [layout_tbl_graph_pmds()] for further
#'   details}
#'   \item{`stress`}{Layout based on stress minimisation with better stability
#'   than Kamada-Kawai layout. See [layout_tbl_graph_stress()] for further
#'   details}
#'   \item{`unrooted`}{Draws unrooted trees based on equal angle with optional
#'   equal daylight modification. See [layout_tbl_graph_unrooted()] for further
#'   details}
#'   \item{`htree`}{Draws binary trees as a space filling fractal. See
#'   [layout_tbl_graph_htree()] for further details}
#' }
#'
#' Alternatively a matrix or a data.frame can be provided to the `layout`
#' argument. In the former case the first column will be used as x coordinates
#' and the second column will by used as y coordinates, further columns are
#' dropped. In the latter case the data.frame is used as the layout table and
#' must thus contain a numeric x and y column.
#'
#' Lastly a function can be provided to the `layout` argument. It will be called
#' with the graph object as its first argument and any additional argument
#' passed into `ggraph()`/`create_layout()`. The function must return either a
#' data.frame or an object coercible to one and have an `x` and `y` column, or
#' an object coercible to a `tbl_graph`. In the latter case the node data is
#' extracted and used as layout (and must thus contain an `x` and `y` column)
#' and the graph will be added as the `graph` attribute.
#'
#' @param graph The object containing the graph. See *Details* for a list
#' of supported classes. Or a `layout_ggraph` object as returned from
#' `create_layout` in which case all subsequent arguments is ignored.
#'
#' @param layout The type of layout to create. Either a valid string, a
#' function, a matrix, or a data.frame (see Details)
#'
#' @param circular Should the layout be transformed into a radial
#' representation. Only possible for some layouts. Defaults to `FALSE`
#'
#' @param ... Arguments passed on to the layout function.
#'
#' @return For `ggraph()` an object of class gg onto which layers, scales,
#'   etc. can be added. For `create_layout()` an object inheriting from
#'   `layout_ggraph`. `layout_ggraph` itself inherits from
#'   `data.frame` and can be considered as such. The data.frame contains
#'   the node positions in the `x` and `y` column along with
#'   additional columns generated by the specific layout, as well as node
#'   parameters inherited from the graph. Additional information is stored as
#'   attributes to the data.frame. The original graph object is stored in the
#'   `graph` attribute and the `circular` attribute contains a logical
#'   indicating whether the layout has been transformed to a circular
#'   representation.
#'
#' @keywords layout network graph hierarchy visualisation
#'
#' @seealso [get_edges()] for extracting edge information from the
#' layout and [get_con()] for extracting path information.
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('bull')
#' layout <- create_layout(gr, layout = 'igraph', algorithm = 'kk')
#' @export
#'
ggraph <- function(graph, layout = 'auto', ...) {
  envir <- parent.frame()
  p <- ggplot(data = create_layout(graph, layout, ...), environment = envir) +
    th_no_axes()
  class(p) <- c('ggraph', class(p))
  p
}
#' @export
ggplot_build.ggraph <- function(plot, ...) {
  .register_graph_context(attr(plot$data, 'graph'), free = TRUE)
  NextMethod()
}

#' Draw edges as diagonals
#'
#' This geom draws edge loops (edges starting and ending at the same node).
#' Loops are drawn as bezier curves starting and ending at the position of the
#' node and with control points protruding at an angle and in a direction
#' specified in the call. As the start and end node is always the same no *2
#' method is provided. Loops can severely clutter up your visualization which is
#' why they are decoupled from the other edge drawings. Only plot them if they
#' are of importance. If the graph doesn't contain any loops the geom adds
#' nothing silently.
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_loop` and `geom_edge_loop0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **from**
#' - **to**
#' - **span** *90*
#' - **direction** *45*
#' - **strength** *1*
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_loop` furthermore takes the following aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' gr <- as_tbl_graph(
#'   data.frame(from = c(1, 1, 2, 2, 3, 3, 3), to = c(1, 2, 2, 3, 3, 1, 2))
#' )
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_loop(aes(alpha = after_stat(index))) +
#'   geom_edge_fan(aes(alpha = after_stat(index)))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_loop0() +
#'   geom_edge_fan0()
#' @rdname geom_edge_loop
#' @name geom_edge_loop
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier
#' @export
StatEdgeLoop <- ggproto(
  'StatEdgeLoop',
  StatBezier,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    if (nrow(data) == 0) {
      return(data)
    }
    data <- data[data$from == data$to, ]
    data$group <- make_unique(data$group)
    if (nrow(data) != 0) {
      create_loops(data, params)
    } else {
      data
    }
  },
  required_aes = c('x', 'y', 'from', 'to', 'span', 'direction', 'strength'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n')
)
#' @rdname geom_edge_loop
#'
#' @export
geom_edge_loop <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  arrow = NULL,
  n = 100,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      from = from,
      to = to,
      group = edge.id,
      span = 90,
      direction = 45,
      strength = 1
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeLoop,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = FALSE,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier0
#' @export
StatEdgeLoop0 <- ggproto(
  'StatEdgeLoop0',
  StatBezier0,
  setup_data = function(data, params) {
    StatEdgeLoop$setup_data(data, params)
  },
  required_aes = c('x', 'y', 'from', 'to', 'span', 'direction', 'strength'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm')
)
#' @rdname geom_edge_loop
#'
#' @export
geom_edge_loop0 <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  arrow = NULL,
  lineend = 'butt',
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      from = from,
      to = to,
      span = 90,
      direction = 45,
      strength = 1
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeLoop0,
    geom = GeomEdgeBezier,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(arrow = arrow, lineend = lineend, ...)
    )
  )
}

create_loops <- function(loops, params) {
  control_angle1 <- loops$direction - loops$span / 2
  control_angle2 <- loops$direction + loops$span / 2
  controls1 <- find_loop_controls(loops, control_angle1)
  controls2 <- find_loop_controls(loops, control_angle2)
  end <- loops
  bezier_start <- seq(1, by = 4, length.out = nrow(loops))
  loops$index <- bezier_start
  controls1$index <- bezier_start + 1
  controls2$index <- bezier_start + 2
  end$index <- bezier_start + 3
  loops <- vec_rbind(loops, controls1, controls2, end)
  loops[order(loops$index), names(loops) != 'index']
}
find_loop_controls <- function(loops, angle) {
  angle <- angle / 360 * 2 * pi
  loops$x <- loops$x + cos(angle) * loops$strength
  loops$y <- loops$y + sin(angle) * loops$strength
  loops
}

#' Place nodes according to the standard design of metro maps
#'
#' This layouttries to optimise the placement of nodes so that they adhere to
#' the classic look of metro maps. As such it optimizes on the distribution of
#' incident edge angles, conformity of edge length, and edge angles in multiples
#' of 45 degrees. As it works as a refinement of an already existing layout
#' (the physical location of metro stations) it requires an a priori node
#' location. Due to it's purpose it probably works best with planar graphs.
#'
#' @param graph A tbl_graph object
#' @param x,y The start location of the nodes
#' @param length Desired multiple of grid point spacing. (`length * grid_space`
#' determines desired edge length)
#' @param grid_space The distance between consecitive grid points
#' @param optimization_weights The relative weight to be placed on the 5
#' criteria during optimization as a numeric vector of length 4. The criteria
#' are:
#' * `Angular Resolution Criterion`: The angles of incident edges at each
#'   station should be maximized, because if there is only a small angle between
#'   any two adjacent edges, then it can become difficult to distinguish between
#'   them.
#' * `Edge Length Criterion`: The edge lengths across the whole map should be
#'   approximately equal to ensure regular spacing between stations. It is based
#'   on the preferred multiple, l, of the grid spacing, g. The purpose of the
#'   criterion is to penalize edges that are longer than or shorter than lg.
#' * `Balanced Edge Length Criterion`: The length of edges incident to a
#'   particular station should be similar.
#' * `Line Straightness Criterion`: (not yet implemented) Edges that form part
#'   of a line should, where possible, be co-linear either side of each station
#'   that the line passes through.
#' * `Octiinearity Criterion`: Each edge should be drawn horizontally,
#'   vertically, or diagonally at 45 degree, so we penalize edges that are not
#'   at a desired angle.
#' If `NULL` all criteria are given equal weight.
#' @param max_movement Number of grid points a station can move away rom its
#' original position
#' @param circular ignored
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @references
#' Stott, J., Rodgers, P., Martinez-Ovando, J. C., and Walker, S. G. (2011).
#' *Automatic metro map layout using multicriteria optimization* In IEEE Trans
#' Vis Comput Graph 17(1) pp. 101-114. https://doi.org/10.1109/tvcg.2010.24
#'
#' @family layout_tbl_graph_*
#'
#' @author The underlying algorithm is implemented in the graphlayouts package
#' by David Schoch
#'
#' @importFrom graphlayouts layout_as_metromap
#'
layout_tbl_graph_metro <- function(
  graph,
  x,
  y,
  length = 2,
  grid_space = 0.0025,
  optimization_weights = NULL,
  max_movement = 5,
  circular = FALSE
) {
  x <- eval_tidy(enquo(x), .N())
  y <- eval_tidy(enquo(y), .N())
  if (is.null(x) || is.null(y)) {
    cli::cli_abort(
      "The metro layout must have a prior location of nodes to start with"
    )
  }
  if (is.null(optimization_weights)) {
    optimization_weights <- rep(1, 5)
  }
  xy <- layout_as_metromap(
    graph,
    xy = cbind(x, y),
    l = length,
    gr = grid_space,
    w = optimization_weights,
    bsize = max_movement
  )
  nodes <- data_frame0(x = xy[, 1], y = xy[, 2], circular = FALSE)
  combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
}

#' Place nodes using stress majorisation
#'
#' This layout is related to the stress-minimization algorithm known as
#' Kamada-Kawai (available as the 'kk' layout), but uses another optimization
#' strategy. It generally have better runtime, quality, and stability compared
#' to the Kamada-Kawai layout and is thus generally preferred. The sparse
#' version of the layout have better performance (especially on larger networks)
#' at the expense of layout quality, but will generally outperform many other
#' algorithms for large graphs in both runtime and quality (e.g. the 'drl'
#' layout from igraph).
#'
#' @param graph a tbl_graph object
#' @param weights An expression evaluated on the edge data to provide edge
#' weights for the layout. Currently ignored for the sparse version
#' @param pivots The number of pivot nodes.
#' @param niter number of iterations during stress optimization
#' @param tolerance stopping criterion for stress optimization
#' @param mds should an MDS layout be used as initial layout (default: TRUE)
#' @param bbox constrain dimension of output. Only relevant to determine the
#' placement of disconnected graphs.
#' @param x,y Expressions evaluated on the node data giving
#' coordinates along x and/or y axis to fix nodes to. You can chose to only fix
#' selected nodes by leaving the remaining nodes with `NA` values.
#' @param circular ignored
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @references
#' Gansner, E. R., Koren, Y., & North, S. (2004). *Graph drawing by stress
#' majorization.* In International Symposium on Graph Drawing (pp. 239-250). Springer, Berlin, Heidelberg.
#'
#' Ortmann, M. and Klimenta, M. and Brandes, U. (2016). *A Sparse Stress Model.* https://arxiv.org/pdf/1608.08909.pdf
#'
#' @family layout_tbl_graph_*
#'
#' @author The underlying algorithm is implemented in the graphlayouts package
#' by David Schoch
#'
#' @importFrom graphlayouts layout_with_stress layout_with_constrained_stress layout_with_fixed_coords
#' @importFrom igraph gorder
#' @importFrom rlang eval_tidy enquo
#'
layout_tbl_graph_stress <- function(
  graph,
  weights = NULL,
  niter = 500,
  tolerance = 1e-4,
  mds = TRUE,
  bbox = 50,
  x = NULL,
  y = NULL,
  circular = FALSE
) {
  weights <- eval_tidy(enquo(weights), .E())
  if (is.null(weights)) {
    weights <- NA
  } else {
    weights <- 1 / weights
  }
  x <- eval_tidy(enquo(x), .N())
  y <- eval_tidy(enquo(y), .N())
  if (is.null(x) && is.null(y)) {
    xy <- layout_with_stress(
      graph,
      weights = weights,
      iter = niter,
      tol = tolerance,
      mds = mds,
      bbox = bbox
    )
  } else {
    xy <- cbind(x %||% NA, y %||% NA)
    if (nrow(xy) != gorder(graph)) {
      cli::cli_abort(
        "If {.arg x_coord} and/or {.arg y_coord} is given they must equal the number of nodes in the graph"
      )
    }
    if (anyNA(xy)) {
      xy <- layout_with_fixed_coords(
        graph,
        xy,
        weights = weights,
        iter = niter,
        tol = tolerance,
        mds = mds,
        bbox = bbox
      )
    }
  }

  nodes <- data_frame0(x = xy[, 1], y = xy[, 2], circular = FALSE)
  combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
}
#' @rdname layout_tbl_graph_stress
#' @importFrom graphlayouts layout_with_sparse_stress
layout_tbl_graph_sparse_stress <- function(
  graph,
  pivots,
  weights = NULL,
  niter = 500,
  circular = FALSE
) {
  weights <- eval_tidy(enquo(weights))
  if (!is.null(weights)) {
    cli::cli_warn(
      '{.arg weights} is currently ignored for sparse stress layouts'
    )
  }
  xy <- layout_with_sparse_stress(
    graph,
    pivots = pivots,
    weights = weights,
    iter = niter
  )
  nodes <- data_frame0(x = xy[, 1], y = xy[, 2], circular = FALSE)
  combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
}

#' Edge width scales
#'
#' This set of scales defines width scales for edge geoms. Of all the new edge
#' scales defined in ggraph, this is the only one not having an equivalent in
#' ggplot2. In essence it mimics the use of size in
#' [ggplot2::geom_line()] and related. As almost all edge
#' representations are lines of some sort, edge_width will be used much more
#' often than edge_size. It is not necessary to spell out that it is an edge
#' scale as the geom knows if it is drawing an edge. Just write `width` and
#' not `edge_width` in the call to geoms.
#'
#' @return A ggproto object inheriting from `Scale`
#'
#' @family scale_edge_*
#'
#' @name scale_edge_width
#' @rdname scale_edge_width
#'
NULL

#' @rdname scale_edge_width
#'
#' @inheritParams ggplot2::scale_size_continuous
#'
#' @export
scale_edge_width_continuous <- function(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  trans = "identity",
  guide = "legend"
) {
  sc <- scale_radius(
    name = name,
    breaks = breaks,
    labels = labels,
    limits = limits,
    range = range,
    trans = trans,
    guide = guide
  )
  sc$scale_name <- 'width_c'
  sc$aesthetics <- 'edge_width'
  sc
}
#' @rdname scale_edge_width
#'
#' @export
scale_edge_width <- scale_edge_width_continuous
#' @rdname scale_edge_width
#'
#' @inheritParams ggplot2::scale_size_discrete
#' @export
scale_edge_width_discrete <- function(...) {
  cli::cli_warn(
    "Using {.field edge_width} for a discrete variable is not advised."
  )
  sc <- scale_size_ordinal(...)
  sc$scale_name <- 'width_d'
  sc$aesthetics <- 'edge_width'
  sc
}
#' @rdname scale_edge_width
#'
#' @inheritParams ggplot2::scale_size_binned
#' @export
scale_edge_width_binned <- function(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  n.breaks = NULL,
  nice.breaks = TRUE,
  trans = "identity",
  guide = "bins"
) {
  sc <- scale_size_binned(
    name = name,
    breaks = breaks,
    labels = labels,
    limits = limits,
    range = range,
    n.breaks = n.breaks,
    nice.breaks = nice.breaks,
    trans = trans,
    guide = guide
  )
  sc$scale_name <- 'width_b'
  sc$aesthetics <- 'edge_width'
  sc
}
#' @rdname scale_edge_width
#'
#' @inheritParams ggplot2::scale_size_manual
#'
#' @export
scale_edge_width_manual <- function(
  ...,
  values,
  breaks = waiver(),
  na.value = NA
) {
  sc <- scale_size_manual(
    ...,
    values = values,
    breaks = breaks,
    na.value = na.value
  )
  sc$aesthetics <- 'edge_width'
  sc
}
#' @rdname scale_edge_width
#'
#' @inheritParams ggplot2::scale_size_identity
#'
#' @export
scale_edge_width_identity <- function(..., guide = 'none') {
  sc <- scale_size_identity(..., guide = guide)
  sc$aesthetics <- 'edge_width'
  sc
}

#' Create a fabric layout
#'
#' This layout is a bit unusual in that it shows nodes as horizontal line ranges
#' end edges as evenly spaced vertical spans connecting the nodes. As with the
#' matrix layout the strength comes from better scalability but its use require
#' some experience recognising the patterns that different connectivity features
#' gives rise to. As with matrix layouts the ordering of nodes have huge power
#' over the look of the plot. The `node_rank_fabric()` mimics the default
#' ordering from the original BioFabric implementation, but other ranking
#' algorithms from tidygraph can be used with the `sort.by` argument as well.
#' Fabric layouts tend to become quite wide as the graph grows which is
#' something that should be handled with care - e.g. by only zooming in on a
#' specific region.
#'
#' @param graph An `tbl_graph` object
#'
#' @param circular Ignored
#'
#' @param sort.by An expression providing the sorting of the nodes. If `NULL`
#' the nodes will be ordered by their index in the graph.
#'
#' @param shadow.edges Should shadow edges be shown.
#'
#' @return A data.frame with the columns `x`, `xmin`, `xmax`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#' Further, the edges of the graph will gain a `edge_x` variable giving the
#' horizontal position of the edge as well as a `shadow_edge` variable denoting
#' whether the edge is a shadow edge added by the layout.
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom igraph incident_edges V
#' @importFrom rlang enquo eval_tidy
#'
#' @references
#' BioFabric website: <https://biofabric.systemsbiology.net>
#'
#' Longabaugh, William J.R. (2012).
#' *Combing the hairball with BioFabric: a new approach for visualization of large networks*.
#' BMC Bioinformatics, 13: 275. \doi{10.1186/1471-2105-13-275}
#'
layout_tbl_graph_fabric <- function(
  graph,
  circular = FALSE,
  sort.by = NULL,
  shadow.edges = FALSE
) {
  sort.by <- enquo(sort.by)
  sort.by <- eval_tidy(sort.by, .N())
  if (!is.null(sort.by)) {
    pos <- order(order(sort.by))
  } else {
    pos <- seq_len(gorder(graph))
  }

  edges <- as_edgelist(graph, names = FALSE)
  edges <- cbind(pos[edges[, 1]], pos[edges[, 2]])
  first_node <- pmin(edges[, 1], edges[, 2])
  second_node <- pmax(edges[, 1], edges[, 2])
  edge_order <- order(first_node, second_node)

  if (shadow.edges) {
    shadow_order <- order(second_node, first_node)
    edge_order <- split(
      edge_order,
      factor(first_node[edge_order], seq_along(pos))
    )
    shadow_order <- split(
      shadow_order + length(second_node),
      factor(second_node[shadow_order], seq_along(pos))
    )
    edge_order <- unlist(c(shadow_order, edge_order)[matrix(
      seq_len(length(pos) * 2),
      nrow = 2,
      byrow = T
    )])
    graph <- bind_edges(graph, as_tibble(graph, active = 'edges'))
    shadow <- rep(c(FALSE, TRUE), each = length(first_node))
  } else {
    shadow <- rep_len(FALSE, length(first_node))
  }

  edge_rank <- match(seq_along(edge_order), edge_order)

  node_span <- vapply(
    incident_edges(graph, V(graph), mode = 'all'),
    function(e) {
      range(edge_rank[as.integer(e)])
    },
    numeric(2)
  )

  nodes <- data_frame0(
    x = colMeans(node_span),
    xmin = node_span[1, ],
    xmax = node_span[2, ],
    y = abs(pos - max(pos)),
    circular = FALSE
  )
  nodes <- combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))

  graph <- activate(graph, 'edges')
  graph <- mutate(graph, edge_x = edge_rank, shadow_edge = shadow)
  graph <- activate(graph, 'nodes')
  attr(nodes, 'graph') <- graph
  nodes
}

#' @rdname layout_tbl_graph_fabric
#' @importFrom igraph bfs degree
#' @importFrom tidygraph activate .G arrange
#' @export
node_rank_fabric <- function() {
  graph <- activate(.G(), 'nodes')
  graph <- mutate(graph, node_order_orig = seq_len(n()))
  graph <- arrange(graph, -degree(graph))
  node_order_orig <- pull(graph, node_order_orig)
  graph <- activate(graph, 'edges')
  graph <- arrange(graph, pmin(from, to), pmax(from, to))
  order <- as.integer(bfs(graph, 1, 'all', order = TRUE)$order)
  order <- node_order_orig[order]
  match(seq_along(order), order)
}

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid segmentsGrob polylineGrob gpar
#' @importFrom dplyr group_by_ do ungroup
#' @importFrom ggforce interpolateDataFrame
#' @export
GeomEdgePath <- ggproto(
  'GeomEdgePath',
  GeomPath,
  draw_panel = function(
    self,
    data,
    panel_scales,
    coord,
    arrow = NULL,
    lineend = 'butt',
    linejoin = 'round',
    linemitre = 1,
    na.rm = FALSE,
    interpolate = TRUE,
    label_colour = 'black',
    label_alpha = 1,
    label_parse = FALSE,
    check_overlap = FALSE,
    angle_calc = 'none',
    force_flip = TRUE,
    label_dodge = NULL,
    label_push = NULL
  ) {
    if (!anyDuplicated(data$group)) {
      cli::cli_inform(c(
        "{.fn {snake_class(self)}}: Each group consists of only one observation.",
        i = "Do you need to adjust the {.field group} aesthetic?"
      ))
    }
    data <- data[order(data$group), , drop = FALSE]
    data$group <- match(data$group, unique0(data$group))
    if (interpolate) {
      geometries <- vapply(data, is.geometry, logical(1))
      geom_cols <- data[geometries]
      data <- cbind(interpolateDataFrame(data[!geometries]), geom_cols)
    }
    data <- coord$transform(data, panel_scales)
    zero <- coord$transform(data_frame0(x = 0, y = 0), panel_scales)
    if (nrow(data) < 2) {
      return(zeroGrob())
    }
    attr <- pathAttr(
      as.integer(data$group),
      as.numeric(data$edge_alpha),
      as.numeric(data$edge_width),
      as.character(data$edge_linetype),
      as.character(data$edge_colour),
      length(unique0(data$group))
    )

    if (all(is.na(data$start_cap))) {
      start_captype <- 'circle'
      start_cap <- NULL
      start_cap2 <- NULL
    } else {
      scap <- data$start_cap[!duplicated(data$group)]
      start_captype <- geo_type(scap)
      start_cap <- geo_width(scap)
      start_cap2 <- geo_height(scap)
      native_start <- grepl('native', as.character(start_cap))
      native_start2 <- grepl('native', as.character(start_cap2))
      if (any(native_start) || any(native_start2)) {
        recalc <- coord$transform(
          data_frame0(x = as.vector(start_cap), y = as.vector(start_cap2)),
          panel_scales
        )
        start_cap[native_start] <- unit(recalc$x - zero$x, 'npc')[native_start]
        start_cap2[native_start2] <- unit(recalc$y - zero$y, 'npc')[
          native_start2
        ]
      }
    }
    if (all(is.na(data$end_cap))) {
      end_captype <- 'circle'
      end_cap <- NULL
      end_cap2 <- NULL
    } else {
      ecap <- data$end_cap[!duplicated(data$group)]
      end_captype <- geo_type(ecap)
      end_cap <- geo_width(ecap)
      end_cap2 <- geo_height(ecap)
      native_end <- grepl('native', as.character(end_cap))
      native_end2 <- grepl('native', as.character(end_cap2))
      if (any(native_end) || any(native_end2)) {
        recalc <- coord$transform(
          data_frame0(x = as.vector(end_cap), y = as.vector(end_cap2)),
          panel_scales
        )
        end_cap[native_end] <- unit(recalc$x - zero$x, 'native')[native_end]
        end_cap2[native_end2] <- unit(recalc$y - zero$y, 'native')[native_end2]
      }
    }
    if (
      (is.null(start_cap) && !is.null(end_cap)) ||
        (!is.null(start_cap) && is.null(end_cap))
    ) {
      if (is.null(start_cap)) {
        start_cap <- 0
      }
      if (is.null(end_cap)) end_cap <- 0
    }

    solid_lines <- all(attr$solid)
    constant <- all(attr$constant)
    if (!solid_lines && !constant) {
      cli::cli_abort(
        "{.fn {snake_class(self)}} can't have varying {.field edge_colour}, {.field edge_width}, and/or {.field edge_alpha} along the line when {.field edge_linetype} isn't solid"
      )
    }

    gp <- gpar(
      col = alpha(data$edge_colour, data$edge_alpha),
      fill = alpha(data$edge_colour, data$edge_alpha),
      lwd = data$edge_width * .pt,
      lty = data$edge_linetype,
      lineend = lineend,
      linejoin = linejoin,
      linemitre = linemitre
    )
    edge_grob <- cappedPathGrob(
      x = data$x,
      y = data$y,
      id = data$group,
      arrow = arrow,
      start.cap = start_cap,
      start.cap2 = start_cap2,
      start.captype = start_captype,
      end.cap = end_cap,
      end.cap2 = end_cap2,
      end.captype = end_captype,
      default.units = 'native',
      gp = gp,
      constant = constant
    )
    if (any(!is.na(data$label))) {
      if (angle_calc == 'none') {
        angle_calc <- 'rot'
      }
      edge_ind <- split(seq_len(nrow(data)), data$group)
      edge_length <- lengths(edge_ind)
      edge_start <- c(0, cumsum(edge_length)[-length(edge_length)]) + 1
      label_pos <- data$label_pos[vapply(edge_ind, head, integer(1), n = 1)]
      label_pos <- 1 + floor((edge_length - 1) * label_pos)
      label_ind <- edge_start + label_pos
      label_data <- data[label_ind, ]
      label_data$label_colour <- if (is.na(label_colour)) {
        label_data$edge_colour
      } else {
        label_colour
      }
      label_data$label_alpha <- if (is.na(label_alpha)) {
        label_data$edge_alpha
      } else {
        label_alpha
      }
      angle_start <- ifelse(label_pos == 1, 1, label_pos - 1)
      angle_end <- ifelse(label_pos == edge_length, edge_length, label_pos + 1)
      label_x0 <- data$x[angle_start + edge_start]
      label_y0 <- data$y[angle_start + edge_start]
      label_x1 <- data$x[angle_end + edge_start]
      label_y1 <- data$y[angle_end + edge_start]
      lab <- as.character(label_data$label)
      if (label_parse) {
        lab_expressions <- sapply(X = lab, FUN = str2expression)
        lab_len <- sapply(X = lab_expressions, FUN = length)
        if (any(lab_len == 0)) {
          lab[lab_len == 0] <- "` `"
          lab <- sapply(X = lab, FUN = str2expression)
        } else {
          lab <- lab_expressions
        }
      }
      label_grob <- textAlongGrob(
        lab,
        label_data$x,
        label_data$y,
        default.units = 'native',
        hjust = label_data$hjust,
        vjust = label_data$vjust,
        rot = label_data$angle,
        gp = gpar(
          col = alpha(label_data$label_colour, label_data$label_alpha),
          fontsize = label_data$label_size * .pt,
          fontfamily = label_data$family,
          fontface = label_data$fontface,
          lineheight = label_data$lineheight
        ),
        check.overlap = check_overlap,
        rot.type = angle_calc,
        x0 = label_x0,
        y0 = label_y0,
        x1 = label_x1,
        y1 = label_y1,
        force.rot = force_flip,
        dodge = label_dodge,
        push = label_push
      )
      gList(edge_grob, label_grob)
    } else {
      edge_grob
    }
  },
  draw_key = function(data, params, size) {
    segmentsGrob(
      0.1,
      0.5,
      0.9,
      0.5,
      gp = gpar(
        col = alpha(data$edge_colour, data$edge_alpha),
        fill = alpha(
          params$arrow.fill %||%
            data$edge_colour %||%
            data$edge_fill %||%
            "black",
          data$edge_alpha
        ),
        lwd = data$edge_width * .pt,
        lty = data$edge_linetype,
        lineend = 'butt'
      ),
      arrow = params$arrow
    )
  },
  handle_na = function(self, data, params) {
    if (params$interpolate) {
      return(data)
    }
    keep <- function(x) {
      first <- match(FALSE, x, nomatch = 1) - 1
      last <- length(x) - match(FALSE, rev(x), nomatch = 1) + 1
      c(
        rep(FALSE, first),
        rep(TRUE, last - first),
        rep(FALSE, length(x) - last)
      )
    }
    missing <- !stats::complete.cases(
      data[c('x', 'y', 'edge_width', 'edge_colour', 'edge_linetype')]
    )
    kept <- stats::ave(missing, data$group, FUN = keep)
    data <- data[kept, ]
    if (!all(kept) && !params$na.rm) {
      cli::cli_warn(
        "Removed {sum(!kept)} row{?s} containing missing values ({.fn {snake_class(self)}})."
      )
    }
    data
  },
  default_aes = aes(
    edge_colour = 'black',
    edge_width = 0.5,
    edge_linetype = 'solid',
    edge_alpha = NA,
    start_cap = NA,
    end_cap = NA,
    label = NA,
    label_pos = 0.5,
    label_size = 3.88,
    angle = 0,
    hjust = 0.5,
    vjust = 0.5,
    family = '',
    fontface = 1,
    lineheight = 1.2
  ),
  rename_size = FALSE
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid gpar segmentsGrob
#' @export
GeomEdgeParallelPath <- ggproto(
  'GeomEdgeParallelPath',
  GeomEdgePath,
  draw_panel = function(
    data,
    panel_scales,
    coord,
    arrow = NULL,
    sep = unit(2, 'mm'),
    lineend = 'butt',
    linejoin = 'round',
    linemitre = 1,
    na.rm = FALSE,
    interpolate = TRUE,
    label_colour = 'black',
    label_alpha = 1,
    label_parse = FALSE,
    check_overlap = FALSE,
    angle_calc = 'none',
    force_flip = TRUE,
    label_dodge = NULL,
    label_push = NULL
  ) {
    data <- data[order(data$group), , drop = FALSE]
    panel <- GeomEdgePath$draw_panel(
      data,
      panel_scales,
      coord,
      arrow = arrow,
      lineend = lineend,
      linejoin = linejoin,
      linemitre = linemitre,
      na.rm = na.rm,
      interpolate = interpolate,
      label_colour = label_colour,
      label_alpha = label_alpha,
      label_parse = label_parse,
      check_overlap = check_overlap,
      angle_calc = angle_calc,
      force_flip = force_flip,
      label_dodge = label_dodge,
      label_push = label_push
    )
    if (inherits(panel, 'gList')) {
      panel[[1]]$sep <- (data$.position[!duplicated(data$group)] * sep)[
        panel[[1]]$id
      ]
      class(panel[[1]]) <- c('parallelPath', class(panel[[1]]))
    } else {
      panel$sep <- (data$.position[!duplicated(data$group)] * sep)[panel$id]
      class(panel) <- c('parallelPath', class(panel))
    }
    panel
  }
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid gpar segmentsGrob
#' @export
GeomEdgeSegment <- ggproto(
  'GeomEdgeSegment',
  GeomSegment,
  draw_panel = function(
    data,
    panel_scales,
    coord,
    arrow = NULL,
    lineend = 'butt',
    na.rm = FALSE
  ) {
    if (empty_data(data)) {
      return(zeroGrob())
    }
    coord <- coord$transform(data, panel_scales)
    segmentsGrob(
      coord$x,
      coord$y,
      coord$xend,
      coord$yend,
      default.units = 'native',
      gp = gpar(
        col = alpha(coord$edge_colour, coord$edge_alpha),
        fill = alpha(coord$edge_colour, coord$edge_alpha),
        lwd = coord$edge_width * .pt,
        lty = coord$edge_linetype,
        lineend = lineend
      ),
      arrow = arrow
    )
  },
  draw_key = function(data, params, size) {
    segmentsGrob(
      0.1,
      0.5,
      0.9,
      0.5,
      gp = gpar(
        col = alpha(data$edge_colour, data$edge_alpha),
        fill = alpha(
          params$arrow.fill %||%
            data$edge_colour %||%
            data$edge_fill %||%
            "black",
          data$edge_alpha
        ),
        lwd = data$edge_width * .pt,
        lty = data$edge_linetype,
        lineend = 'butt'
      ),
      arrow = params$arrow
    )
  },
  default_aes = aes(
    edge_colour = 'black',
    edge_width = 0.5,
    edge_linetype = 1,
    edge_alpha = NA
  ),
  rename_size = FALSE
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
GeomEdgeParallelSegment <- ggproto(
  'GeomEdgeParallelSegment',
  GeomEdgeSegment,
  draw_panel = function(
    data,
    panel_scales,
    coord,
    arrow = NULL,
    lineend = 'butt',
    na.rm = FALSE,
    sep = unit(2, 'mm')
  ) {
    data <- data[order(data$group), , drop = FALSE]
    panel <- GeomEdgeSegment$draw_panel(
      data,
      panel_scales,
      coord,
      arrow = arrow,
      lineend = lineend,
      na.rm = na.rm
    )
    panel$sep <- data$.position * sep
    class(panel) <- c('parallelPath', class(panel))
    panel
  }
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid pointsGrob unit gpar grobTree
#' @export
GeomEdgeSpanPath <- ggproto(
  'GeomEdgeSpanPath',
  GeomEdgePath,
  draw_panel = function(
    data,
    panel_scales,
    coord,
    arrow = NULL,
    end_shape = NA,
    lineend = 'butt',
    linejoin = 'round',
    linemitre = 1,
    na.rm = FALSE,
    interpolate = TRUE,
    label_colour = 'black',
    label_alpha = 1,
    label_parse = FALSE,
    check_overlap = FALSE,
    angle_calc = 'none',
    force_flip = TRUE,
    label_dodge = NULL,
    label_push = NULL
  ) {
    panel <- GeomEdgePath$draw_panel(
      data,
      panel_scales,
      coord,
      arrow = arrow,
      lineend = lineend,
      linejoin = linejoin,
      linemitre = linemitre,
      na.rm = na.rm,
      interpolate = interpolate,
      label_colour = label_colour,
      label_alpha = label_alpha,
      label_parse = label_parse,
      check_overlap = FALSE,
      angle_calc = 'none',
      force_flip = TRUE,
      label_dodge = label_dodge,
      label_push = label_push
    )
    if (is.na(end_shape)) {
      return(panel)
    }

    data <- data[data$index == 0 | data$index == 1, ]
    end_shape <- translate_shape_string(end_shape)
    coords <- coord$transform(data, panel_scales)
    ends <- pointsGrob(
      coords$x,
      coords$y,
      pch = end_shape,
      size = unit(
        1 / (panel_scales$x.range[2] - (panel_scales$x.range[1])),
        'npc'
      ),
      gp = gpar(
        col = alpha(coords$edge_colour, coords$edge_alpha),
        fill = alpha(coords$edge_colour, coords$edge_alpha),
        lwd = 0
      )
    )
    grobTree(panel, ends)
  }
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid pointsGrob unit gpar grobTree
#' @export
GeomEdgeSpanSegment <- ggproto(
  'GeomEdgeSpanSegment',
  GeomEdgeSegment,
  draw_panel = function(
    data,
    panel_scales,
    coord,
    arrow = NULL,
    lineend = 'butt',
    na.rm = FALSE,
    end_shape = NA
  ) {
    panel <- GeomEdgeSegment$draw_panel(
      data,
      panel_scales,
      coord,
      arrow = arrow,
      lineend = lineend,
      na.rm = na.rm
    )
    if (is.na(end_shape)) {
      return(panel)
    }

    data2 <- data
    data2$x <- data2$xend
    data2$y <- data2$yend
    data <- vec_rbind(data, data2)
    data$xend <- NULL
    data$yend <- NULL

    end_shape <- translate_shape_string(end_shape)
    coords <- coord$transform(data, panel_scales)
    ends <- pointsGrob(
      coords$x,
      coords$y,
      pch = end_shape,
      size = unit(
        1 / (panel_scales$x.range[2] - (panel_scales$x.range[1])),
        'npc'
      ),
      gp = gpar(
        col = alpha(coords$edge_colour, coords$edge_alpha),
        fill = alpha(coords$edge_colour, coords$edge_alpha),
        lwd = 0
      )
    )
    grobTree(panel, ends)
  }
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid gpar pointsGrob
#' @export
GeomEdgePoint <- ggproto(
  'GeomEdgePoint',
  GeomPoint,
  setup_data = function(data, params) {
    if (params$mirror) {
      data2 <- data
      data2[, c('x', 'y')] <- data2[, c('y', 'x'), drop = FALSE]
      data2$x <- abs(data2$x) * sign(data$x)
      data2$y <- abs(data2$y) * sign(data$y)
      data <- vec_rbind(data, data2)
    }
    data
  },
  draw_panel = function(data, panel_scales, coord, na.rm = FALSE) {
    if (empty_data(data)) {
      return(zeroGrob())
    }
    if (is.character(data$edge_shape)) {
      data$edge_shape <- translate_shape_string(data$edge_shape)
    }
    coords <- coord$transform(data, panel_scales)
    coords <- coords[order(coords$edge_size, decreasing = TRUE), , drop = FALSE]
    pointsGrob(
      coords$x,
      coords$y,
      pch = coords$edge_shape,
      gp = gpar(
        col = alpha(coords$edge_colour, coords$edge_alpha),
        fill = alpha(coords$edge_fill, coords$edge_alpha),
        fontsize = coords$edge_size *
          .pt +
          coords$stroke *
            .stroke /
            2,
        lwd = coords$stroke * .stroke / 2
      )
    )
  },
  draw_key = function(data, params, size) {
    pointsGrob(
      0.5,
      0.5,
      pch = data$edge_shape,
      gp = gpar(
        col = alpha(data$edge_colour, data$edge_alpha),
        fill = alpha(data$edge_fill, data$edge_alpha),
        fontsize = data$edge_size * .pt + data$stroke * .stroke / 2,
        lwd = data$stroke * .stroke / 2
      )
    )
  },
  default_aes = aes(
    edge_shape = 19,
    edge_colour = 'black',
    edge_size = 1.5,
    edge_fill = NA,
    edge_alpha = NA,
    stroke = 0.5
  ),
  extra_params = c('na.rm', 'mirror')
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid gpar rectGrob
#' @export
GeomEdgeTile <- ggproto(
  'GeomEdgeTile',
  GeomTile,
  setup_data = function(data, params) {
    if (params$mirror) {
      data2 <- data
      data2[, c('x', 'y')] <- data2[, c('y', 'x'), drop = FALSE]
      data2$x <- abs(data2$x) * sign(data$x)
      data2$y <- abs(data2$y) * sign(data$y)
      data <- vec_rbind(data, data2)
    }
    data
  },
  draw_panel = function(
    data,
    panel_params,
    coord,
    lineend = 'butt',
    linejoin = 'mitre',
    na.rm = FALSE
  ) {
    if (empty_data(data)) {
      return(zeroGrob())
    }
    data$xmin <- data$x - 0.5
    data$xmax <- data$x + 0.5
    data$ymin <- data$y - 0.5
    data$ymax <- data$y + 0.5
    coords <- coord$transform(data, panel_params)
    rectGrob(
      coords$xmin,
      coords$ymax,
      width = coords$xmax - coords$xmin,
      height = coords$ymax - coords$ymin,
      default.units = "native",
      just = 'centre',
      gp = gpar(
        col = coords$edge_colour,
        fill = alpha(coords$edge_fill, coords$edge_alpha),
        lwd = coords$edge_width * .pt,
        lty = coords$edge_linetype,
        linejoin = linejoin,
        lineend = lineend
      )
    )
  },
  draw_key = function(data, params, size) {
    if (is.null(data$edge_width)) {
      data$edge_width <- 0.5
    }

    lwd <- min(data$edge_width, min(size) / 4)

    rectGrob(
      width = unit(1, "npc") - unit(lwd, "mm"),
      height = unit(1, "npc") - unit(lwd, "mm"),
      gp = gpar(
        col = data$edge_colour %||% NA,
        fill = alpha(data$edge_fill %||% "grey20", data$edge_alpha),
        lty = data$edge_linetype %||% 1,
        lwd = lwd * .pt,
        linejoin = "mitre",
        # `lineend` is a workaround for Windows and intentionally kept unexposed
        # as an argument. (c.f. https://github.com/tidyverse/ggplot2/issues/3037#issuecomment-457504667)
        lineend = "square"
      )
    )
  },
  default_aes = aes(
    edge_fill = 'grey20',
    edge_colour = NA,
    edge_width = 0.1,
    edge_linetype = 1,
    edge_alpha = NA
  ),
  rename_size = FALSE,
  extra_params = c('na.rm', 'mirror')
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce GeomBezier0
#' @export
GeomEdgeBezier <- ggproto(
  'GeomEdgeBezier',
  GeomBezier0,
  draw_panel = function(
    data,
    panel_scales,
    coord,
    arrow = NULL,
    lineend = 'butt',
    linejoin = 'round',
    linemitre = 1,
    na.rm = FALSE
  ) {
    names(data) <- sub('edge_', '', names(data))
    names(data)[names(data) == 'width'] <- 'linewidth'
    GeomBezier0$draw_panel(
      data,
      panel_scales,
      coord,
      arrow,
      lineend,
      linejoin,
      linemitre,
      na.rm
    )
  },
  draw_key = function(data, params, size) {
    segmentsGrob(
      0.1,
      0.5,
      0.9,
      0.5,
      gp = gpar(
        col = alpha(data$edge_colour, data$edge_alpha),
        fill = alpha(
          params$arrow.fill %||%
            data$edge_colour %||%
            data$edge_fill %||%
            "black",
          data$edge_alpha
        ),
        lwd = data$edge_width * .pt,
        lty = data$edge_linetype,
        lineend = 'butt'
      ),
      arrow = params$arrow
    )
  },
  default_aes = aes(
    edge_colour = 'black',
    edge_width = 0.5,
    edge_linetype = 1,
    edge_alpha = NA
  ),
  rename_size = FALSE,
  handle_na = function(data, ...) {
    data
  }
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce GeomBspline0
#' @export
GeomEdgeBspline <- ggproto(
  'GeomEdgeBspline',
  GeomBspline0,
  draw_panel = function(
    data,
    panel_scales,
    coord,
    arrow = NULL,
    lineend = 'butt',
    linejoin = 'round',
    linemitre = 1,
    na.rm = FALSE
  ) {
    names(data) <- sub('edge_', '', names(data))
    names(data)[names(data) == 'width'] <- 'linewidth'
    GeomBspline0$draw_panel(
      data,
      panel_scales,
      coord,
      arrow,
      type = "clamped",
      lineend,
      linejoin,
      linemitre,
      na.rm
    )
  },
  draw_key = function(data, params, size) {
    segmentsGrob(
      0.1,
      0.5,
      0.9,
      0.5,
      gp = gpar(
        col = alpha(data$edge_colour, data$edge_alpha),
        fill = alpha(
          params$arrow.fill %||%
            data$edge_colour %||%
            data$edge_fill %||%
            "black",
          data$edge_alpha
        ),
        lwd = data$edge_width * .pt,
        lty = data$edge_linetype,
        lineend = 'butt'
      ),
      arrow = params$arrow
    )
  },
  default_aes = aes(
    edge_colour = 'black',
    edge_width = 0.5,
    edge_linetype = 1,
    edge_alpha = NA
  ),
  rename_size = FALSE,
  handle_na = function(data, ...) {
    data
  }
)

remove_loop <- function(data) {
  if (nrow(data) == 0) {
    return(data)
  }

  data[!(data$x == data$xend & data$y == data$yend), , drop = FALSE]
}
remove_loop2 <- function(data) {
  if (nrow(data) == 0) {
    return(data)
  }

  data <- data[order(data$group), ]
  loop <- data$x[c(TRUE, FALSE)] == data$x[c(FALSE, TRUE)] &
    data$y[c(TRUE, FALSE)] == data$y[c(FALSE, TRUE)]
  data[rep(!loop, each = 2), , drop = FALSE]
}

# We need this so that edge geoms is consider 'points' by gganimate. This is
# safe as all edges have the same number of points. We don't want to depend on
# gganimate so we register it on load
layer_type.GeomEdgePath <- function(x) 'point'

#' Create hierarchical edge bundles between node connections
#'
#' Hierarchical edge bundling is a technique to introduce some order into the
#' hairball structure that can appear when there's a lot of overplotting and
#' edge crossing in a network plot. The concept requires that the network has
#' an intrinsic hierarchical structure that defines the layout but is not shown.
#' Connections between points (that is, not edges) are then drawn so that they
#' loosely follows the underlying hierarchical structure. This results in a
#' flow-like structure where lines that partly move in the same direction will
#' be bundled together.
#'
#' @note In order to avoid excessive typing edge aesthetic names are
#' automatically expanded. Because of this it is not necessary to write
#' `edge_colour` within the `aes()` call as `colour` will
#' automatically be renamed appropriately.
#'
#' @section Aesthetics:
#' geom_conn_bundle* understands the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#' \itemize{
#'  \item{\strong{x}}
#'  \item{\strong{y}}
#'  \item{\strong{group}}
#'  \item{\strong{circular}}
#'  \item{edge_colour}
#'  \item{edge_width}
#'  \item{edge_linetype}
#'  \item{edge_alpha}
#'  \item{filter}
#' }
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#'
#' @param data The result of a call to [get_con()]
#'
#' @param tension How "loose" should the bundles be. 1 will give very tight
#' bundles, while 0 will turn of bundling completely and give straight lines.
#' Defaults to 0.8
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_conn_*
#'
#' @references
#' Holten, D. (2006). *Hierarchical edge bundles: visualization
#' of adjacency relations in hierarchical data.* IEEE Transactions on
#' Visualization and Computer Graphics, **12**(5), 741-748.
#' \doi{10.1109/TVCG.2006.147}
#'
#' @examples
#' # Create a graph of the flare class system
#' library(tidygraph)
#' flareGraph <- tbl_graph(flare$vertices, flare$edges) %>%
#'   mutate(
#'     class = map_bfs_chr(node_is_root(), .f = function(node, dist, path, ...) {
#'       if (dist <= 1) {
#'         return(shortName[node])
#'       }
#'       path$result[[nrow(path)]]
#'     })
#'   )
#' importFrom <- match(flare$imports$from, flare$vertices$name)
#' importTo <- match(flare$imports$to, flare$vertices$name)
#'
#' # Use class inheritance for layout but plot class imports as bundles
#' ggraph(flareGraph, 'dendrogram', circular = TRUE) +
#'   geom_conn_bundle(aes(colour = after_stat(index)),
#'     data = get_con(importFrom, importTo),
#'     edge_alpha = 0.25
#'   ) +
#'   geom_node_point(aes(filter = leaf, colour = class)) +
#'   scale_edge_colour_distiller('', direction = 1, guide = 'edge_direction') +
#'   coord_fixed() +
#'   ggforce::theme_no_axes()
#' @rdname geom_conn_bundle
#' @name geom_conn_bundle
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBspline
#' @export
StatConnBundle <- ggproto(
  'StatConnBundle',
  StatBspline,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    if (nrow(data) == 0) {
      return(data)
    }
    relax(data, params$tension)
  },
  setup_params = function(data, params) {
    if (is.null(params$tension)) {
      params$tension <- 0.8
    }
    if (params$tension < 0) {
      params$tension <- 0
    }
    if (params$tension > 1) {
      params$tension <- 1
    }
    params
  },
  required_aes = c('x', 'y'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n', 'tension', 'type')
)
#' @rdname geom_conn_bundle
#'
#' @export
geom_conn_bundle <- function(
  mapping = NULL,
  data = get_con(),
  position = 'identity',
  arrow = NULL,
  lineend = 'butt',
  show.legend = NA,
  n = 100,
  tension = 0.8,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y, group = con.id))
  layer(
    data = data,
    mapping = mapping,
    stat = StatConnBundle,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        n = n,
        interpolate = FALSE,
        tension = tension,
        type = 'clamped',
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBspline2
#' @export
StatConnBundle2 <- ggproto(
  'StatConnBundle2',
  StatBspline2,
  setup_data = function(data, params) {
    StatConnBundle$setup_data(data, params)
  },
  setup_params = function(data, params) {
    StatConnBundle$setup_params(data, params)
  },
  required_aes = c('x', 'y'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n', 'tension', 'type')
)
#' @rdname geom_conn_bundle
#'
#' @export
geom_conn_bundle2 <- function(
  mapping = NULL,
  data = get_con(),
  position = 'identity',
  arrow = NULL,
  lineend = 'butt',
  show.legend = NA,
  n = 100,
  tension = 0.8,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y, group = con.id))
  layer(
    data = data,
    mapping = mapping,
    stat = StatConnBundle2,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow,
        lineend = lineend,
        n = n,
        interpolate = TRUE,
        tension = tension,
        type = 'clamped',
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
StatConnBundle0 <- ggproto(
  'StatConnBundle0',
  StatIdentity,
  setup_data = function(data, params) {
    StatConnBundle$setup_data(data, params)
  },
  setup_params = function(data, params) {
    StatConnBundle$setup_params(data, params)
  },
  required_aes = c('x', 'y'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n', 'tension', 'type')
)
#' @rdname geom_conn_bundle
#'
#' @export
geom_conn_bundle0 <- function(
  mapping = NULL,
  data = get_con(),
  position = 'identity',
  arrow = NULL,
  lineend = 'butt',
  show.legend = NA,
  tension = 0.8,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y, group = con.id))
  layer(
    data = data,
    mapping = mapping,
    stat = StatConnBundle0,
    geom = GeomEdgeBspline,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        tension = tension,
        type = 'clamped',
        ...
      )
    )
  )
}
#' @importFrom utils head tail
relax <- function(data, strength) {
  formula <- function(p, startInd, endInd, pathLengths) {
    start <- rep(p[startInd], pathLengths)
    range <- rep(p[endInd] - p[startInd], pathLengths)
    ind <- unlist(lapply(pathLengths, seq_len)) - 1
    length <- rep(pathLengths, pathLengths)
    strength * p + (1 - strength) * (start + (ind / (length - 1)) * range)
  }
  idInds <- split(seq_len(nrow(data)), data$group)
  pathLengths <- lengths(idInds)
  startInd <- vapply(idInds, head, integer(1), n = 1)
  endInd <- vapply(idInds, tail, integer(1), n = 1)
  data$x <- formula(data$x, startInd, endInd, pathLengths)
  data$y <- formula(data$y, startInd, endInd, pathLengths)
  data
}

#' Draw edges as glyphs
#'
#' This geom draws edges as glyphs with their x-position defined by the
#' x-position of the start node, and the y-position defined by the y-position of
#' the end node. As such it will result in a matrix layout when used in
#' conjunction with [layout_tbl_graph_matrix()]
#'
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_point` understands the following
#' aesthetics. Bold aesthetics are automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - edge_shape
#' - edge_colour
#' - edge_size
#' - edge_alpha
#' - filter
#'
#' @inheritParams ggplot2::geom_point
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x, y, xend, yend, group and
#' circular are mapped to x, y, xend, yend, edge.id and circular in the edge
#' data.
#'
#' @param data The return of a call to `get_edges()` or a data.frame
#' giving edges in correct format (see details for for guidance on the format).
#' See [get_edges()] for more details on edge extraction.
#'
#' @param mirror Logical. Should edge points be duplicated on both sides of the
#' diagonal. Intended for undirected graphs. Default to `FALSE`
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @rdname geom_edge_point
#' @name geom_edge_point
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('zachary') %>%
#'   mutate(group = group_infomap()) %>%
#'   morph(to_split, group) %>%
#'   activate(edges) %>%
#'   mutate(edge_group = as.character(.N()$group[1])) %>%
#'   unmorph()
#'
#' ggraph(gr, 'matrix', sort.by = node_rank_hclust()) +
#'   geom_edge_point(aes(colour = edge_group), mirror = TRUE, edge_size = 3) +
#'   scale_y_reverse() +
#'   coord_fixed() +
#'   labs(edge_colour = 'Infomap Cluster') +
#'   ggtitle("Zachary' Karate Club")
NULL

#' @rdname geom_edge_point
#'
#' @export
geom_edge_point <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  mirror = FALSE,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = yend))
  layer(
    data = data,
    mapping = mapping,
    stat = StatFilter,
    geom = GeomEdgePoint,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = list2(mirror = mirror, ...)
  )
}

#' Show edges as a density map
#'
#' This geom makes it possible to add a layer showing edge presence as a density
#' map. Each edge is converted to `n` points along the line and a jitter is
#' applied. Based on this dataset a two-dimensional kernel density estimation is
#' applied and plotted as a raster image. The density is mapped to the alpha
#' level, making it possible to map a variable to the fill.
#'
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_density` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' **x**
#' **y**
#' **xend**
#' **yend**
#' edge_fill
#' filter
#'
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{x, y}{The coordinates for each pixel in the raster}
#'  \item{density}{The density associated with the pixel}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_raster
#'
#' @param n The number of points to estimate in the x and y direction, i.e. the
#' resolution of the raster.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('bull') %>%
#'   activate(edges) %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_density(aes(fill = class)) +
#'   geom_edge_link() + geom_node_point()
#' @rdname geom_edge_density
#' @name geom_edge_density
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatLink
#' @importFrom MASS bandwidth.nrd kde2d
#' @export
StatEdgeDensity <- ggproto(
  'StatEdgeDensity',
  Stat,
  compute_group = function(
    data,
    scales,
    na.rm = FALSE,
    h = NULL,
    n = 100,
    bins = NULL,
    binwidth = NULL
  ) {
    group <- data$group[1]
    x_range <- diff(range(c(data$x, data$xend)))
    y_range <- diff(range(c(data$y, data$yend)))
    x_extend <- x_range / 10
    y_extend <- y_range / 10
    data <- StatLink$compute_panel(data, n = 50)
    data <- PositionJitter$compute_layer(
      data,
      list(
        width = x_extend,
        height = y_extend
      )
    )

    if (is.null(h)) {
      h <- c(MASS::bandwidth.nrd(data$x), MASS::bandwidth.nrd(data$y))
    }
    dens <- MASS::kde2d(
      data$x,
      data$y,
      h = h,
      n = n,
      lims = c(
        scales$x$dimension(),
        scales$y$dimension()
      ) +
        c(
          -x_extend,
          x_extend,
          -y_extend,
          y_extend
        )
    )
    df <- expand.grid(x = dens$x, y = dens$y)
    df$z <- as.vector(dens$z)
    df$group <- group
    names(df) <- c('x', 'y', 'density', 'group')
    df
  },
  setup_data = function(data, params) {
    StatFilter$setup_data(data, params)
  },
  default_aes = aes(filter = TRUE),
  required_aes = c('x', 'y', 'xend', 'yend')
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid gTree gList grobName rasterGrob rectGrob gpar
GeomEdgeDensity <- ggproto(
  'GeomEdgeDensity',
  GeomRaster,
  draw_panel = function(self, data, panel_scales, coord, ...) {
    groups <- split(data, factor(data$group))
    max_density <- max(rowSums(inject(cbind(
      !!!lapply(groups, `[[`, i = 'density')
    ))))
    grobs <- lapply(groups, function(group) {
      self$draw_group(group, panel_scales, coord, max.alpha = max_density, ...)
    })
    grobs <- gTree(children = inject(gList(!!!grobs)))
    grobs$name <- grobName(grobs, 'geom_edge_density')
    grobs
  },
  draw_group = function(self, data, panel_scales, coord, max.alpha) {
    if (!inherits(coord, 'CoordCartesian')) {
      cli::cli_abort(
        '{.fn {snake_class(self)}} only works with {.fn coord_cartesian'
      )
    }
    data <- coord$transform(data, panel_scales)
    x_pos <- as.integer(
      (data$x - min(data$x)) /
        resolution(
          data$x,
          FALSE
        )
    )
    y_pos <- as.integer(
      (data$y - min(data$y)) /
        resolution(
          data$y,
          FALSE
        )
    )
    nrow <- max(y_pos) + 1
    ncol <- max(x_pos) + 1
    raster <- matrix(NA_character_, nrow = nrow, ncol = ncol)
    raster[cbind(nrow - y_pos, x_pos + 1)] <- alpha(
      data$edge_fill,
      data$density / max.alpha
    )
    x_rng <- c(min(data$xmin, na.rm = TRUE), max(data$xmax, na.rm = TRUE))
    y_rng <- c(min(data$ymin, na.rm = TRUE), max(data$ymax, na.rm = TRUE))
    rasterGrob(
      raster,
      x = mean(x_rng),
      y = mean(y_rng),
      width = diff(x_rng),
      height = diff(y_rng),
      default.units = 'native',
      interpolate = TRUE
    )
  },
  draw_key = function(data, params, size) {
    rectGrob(
      gp = gpar(
        col = NA,
        fill = alpha(data$edge_fill %||% data$fill %||% "darkgrey"),
        lty = 0
      )
    )
  },
  required_aes = c('x', 'y'),
  default_aes = aes(edge_fill = 'darkgrey')
)
#' @rdname geom_edge_density
#'
#' @importFrom ggforce StatLink
#' @export
geom_edge_density <- function(
  mapping = NULL,
  data = get_edges('short'),
  position = 'identity',
  show.legend = NA,
  n = 100,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y, xend = xend, yend = yend))
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeDensity,
    geom = GeomEdgeDensity,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(n = n, ...)
    )
  )
}

#' Edge linetype scales
#'
#' This set of scales defines new linetype scales for edge geoms equivalent to
#' the ones already defined by ggplot2. See
#' [ggplot2::scale_linetype()] for more information. The different
#' geoms will know whether to use edge scales or the standard scales so it is
#' not necessary to write `edge_linetype` in the call to the geom - just
#' use `linetype`.
#'
#' @return A ggproto object inheriting from `Scale`
#'
#' @family scale_edge_*
#'
#' @name scale_edge_linetype
#' @rdname scale_edge_linetype
#'
NULL

#' @rdname scale_edge_linetype
#'
#' @inheritParams ggplot2::scale_linetype
#' @export
scale_edge_linetype <- function(..., na.value = 'blank') {
  sc <- scale_linetype(..., na.value = na.value)
  sc$aesthetics <- 'edge_linetype'
  sc
}
#' @rdname scale_edge_linetype
#'
#' @export
scale_edge_linetype_continuous <- function(...) {
  cli::cli_abort(c(
    "A continuous variable cannot be mapped to the {.field edge_linetype} aesthetic",
    "i" = "choose a different aesthetic or use {.fn scale_edge_linetype_binned}"
  ))
}
#' @rdname scale_edge_linetype
#'
#' @export
scale_edge_linetype_discrete <- scale_edge_linetype
#' @rdname scale_edge_linetype
#'
#' @export
scale_edge_linetype_binned <- function(..., na.value = "blank") {
  sc <- scale_linetype_binned(..., na.value = na.value)
  sc$aesthetics <- 'edge_linetype'
  sc
}
#' @rdname scale_edge_linetype
#'
#' @inheritParams ggplot2::scale_linetype_manual
#'
#' @export
scale_edge_linetype_manual <- function(
  ...,
  values,
  breaks = waiver(),
  na.value = "blank"
) {
  sc <- scale_linetype_manual(
    ...,
    values = values,
    breaks = breaks,
    na.value = na.value
  )
  sc$aesthetics <- 'edge_linetype'
  sc
}
#' @rdname scale_edge_linetype
#'
#' @inheritParams ggplot2::scale_linetype_identity
#' @export
scale_edge_linetype_identity <- function(..., guide = 'none') {
  sc <- scale_linetype_identity(..., guide = guide)
  sc$aesthetics <- 'edge_linetype'
  sc
}

#' Use igraph layout algorithms for layout_tbl_graph
#'
#' This layout function makes it easy to apply one of the layout algorithms
#' supplied in igraph when plotting with ggraph. Layout names are auto completed
#' so there is no need to write `layout_with_graphopt` or
#' `layout_as_tree`, just `graphopt` and `tree` (though the
#' former will also work if you want to be super explicit). Circular layout is
#' only supported for tree-like layout (`tree` and `sugiyama`) and
#' will throw an error when applied to other layouts.
#'
#' @details
#' igraph provides a huge amount of possible layouts. They are all briefly
#' described below:
#'
#' \strong{Hierarchical layouts}
#'
#' \describe{
#'   \item{`tree`}{Uses the *Reingold-Tilford* algorithm to place the
#'   nodes below their parent with the parent centered above its children. See
#'   [igraph::as_tree()]}
#'   \item{`sugiyama`}{Designed for directed acyclic graphs (that is,
#'   hierarchies where multiple parents are allowed) it minimizes the number of
#'   crossing edges. See [igraph::with_sugiyama()]}
#' }
#'
#' \strong{Standard layouts}
#'
#' \describe{
#'   \item{`bipartite`}{Minimize edge-crossings in a simple two-row (or
#'   column) layout for bipartite graphs. See [igraph::as_bipartite()]}
#'   \item{`star`}{Place one node in the center and the rest equidistantly
#'   around it. See [igraph::as_star()]}
#'   \item{`circle`}{Place nodes in a circle in the order of their index.
#'   Consider using [layout_tbl_graph_linear()] with `circular=TRUE`
#'   for more control. See [igraph::in_circle()]}
#'   \item{`nicely`}{Tries to pick an appropriate layout. See
#'   [igraph::nicely()] for a description of the simple decision tree
#'   it uses}
#'   \item{`dh`}{Uses *Davidson and Harels* simulated annealing
#'   algorithm to place nodes. See [igraph::with_dh()]}
#'   \item{`gem`}{Place nodes on the plane using the GEM force-directed
#'   layout algorithm. See [igraph::with_gem()]}
#'   \item{`graphopt`}{Uses the Graphopt algorithm based on alternating
#'   attraction and repulsion to place nodes. See
#'   [igraph::with_graphopt()]}
#'   \item{`grid`}{Place nodes on a rectangular grid. See
#'   [igraph::on_grid()]}
#'   \item{`mds`}{Perform a multidimensional scaling of nodes using either
#'   the shortest path or a user supplied distance. See
#'   [igraph::with_mds()]}
#'   \item{`sphere`}{Place nodes uniformly on a sphere - less relevant for
#'   2D visualizations of networks. See [igraph::on_sphere()]}
#'   \item{`randomly`}{Places nodes uniformly random. See
#'   [igraph::randomly()]}
#'   \item{`fr`}{Places nodes according to the force-directed algorithm of
#'   Fruchterman and Reingold. See [igraph::with_fr()]}
#'   \item{`kk`}{Uses the spring-based algorithm by Kamada and Kawai to
#'   place nodes. See [igraph::with_kk()]}
#'   \item{`drl`}{Uses the force directed algorithm from the DrL toolbox to
#'   place nodes. See [igraph::with_drl()]}
#'   \item{`lgl`}{Uses the algorithm from Large Graph Layout to place
#'   nodes. See [igraph::with_lgl()]}
#' }
#'
#' @note This function is not intended to be used directly but by setting
#' `layout = 'igraph'` in [create_layout()]
#'
#' @param graph A `tbl_graph` object.
#'
#' @param algorithm The type of layout algorithm to apply. See *Details* or
#' [igraph::layout_()] for links to the layouts supplied by igraph.
#'
#' @param circular Logical. Should the layout be transformed to a circular
#' representation. Defaults to `FALSE`. Only applicable to
#' `algorithm = 'tree'` and `algorithm = 'sugiyama'`.
#'
#' @param offset If `circular = TRUE`, where should it begin. Defaults to
#' `pi/2` which is equivalent to 12 o'clock.
#'
#' @param use.dummy Logical. In the case of `algorithm = 'sugiyama'` should the
#' dummy-infused graph be used rather than the original. Defaults to
#' `FALSE`.
#'
#' @param ... Arguments passed on to the respective layout functions
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom igraph graph_attr components layout_as_bipartite layout_as_star layout_as_tree layout_in_circle layout_nicely layout_with_dh layout_with_drl layout_with_gem layout_with_graphopt layout_on_grid layout_with_mds layout_with_sugiyama layout_on_sphere layout_randomly layout_with_fr layout_with_kk layout_with_lgl
#' @importFrom rlang quos eval_tidy
#' @importFrom stats setNames
#'
layout_tbl_graph_igraph <- function(
  graph,
  algorithm,
  circular,
  offset = pi / 2,
  use.dummy = FALSE,
  ...
) {
  algorithm <- as.igraphlayout(algorithm)
  dots <- quos(...)
  dots <- setNames(
    lapply(names(dots), function(nq) {
      if (nq == 'weights') {
        eval_tidy(dots[[nq]], as_tibble(graph, active = 'edges'))
      } else {
        eval_tidy(dots[[nq]])
      }
    }),
    names(dots)
  )
  alg_fun <- try_fetch(
    utils::getFromNamespace(algorithm, 'igraph'),
    error = function(cnd) {
      cli::cli_abort(
        "Could not find the {.val {algorithm}} layout algorithm in the {.var igraph} namespace"
      )
    }
  )
  layout <- inject(alg_fun(graph, !!!dots))
  if (algorithm == 'layout_with_sugiyama') {
    if (use.dummy) {
      graph <- as_tbl_graph(layout$extd_graph)
      layout <- graph_attr(layout$extd_graph, 'layout')
    } else {
      layout <- layout$layout
    }
  }
  if (algorithm == 'layout_as_tree') {
    layout[, 1] <- layout[, 1] + components(graph)$membership - 1
  }
  if ('dim' %in% names(dots) && isTRUE(dots$dim > 2)) {
    cli::cli_warn(
      'Only the first two dimensions will be used despite requesting more'
    )
  }
  nodes <- combine_layout_nodes(
    data_frame0(x = layout[, 1], y = layout[, 2]),
    as_tibble(graph, active = 'nodes')
  )
  graph <- add_direction(graph, nodes)
  if (circular) {
    if (!algorithm %in% c('layout_as_tree', 'layout_with_sugiyama')) {
      cli::cli_abort('Circular layout only applicable to tree and DAG layout')
    }
    radial <- radial_trans(
      r.range = rev(range(nodes$y)),
      a.range = range(nodes$x),
      offset = offset
    )
    coords <- radial$transform(nodes$y, nodes$x)
    nodes$x <- coords$x
    nodes$y <- coords$y
  }
  nodes$circular <- circular
  attr(nodes, 'graph') <- graph
  nodes
}

#' Annotate nodes with text
#'
#' These geoms are equivalent in functionality to [ggplot2::geom_text()] and
#' [ggplot2::geom_label()] and allows for simple annotation of nodes.
#'
#' @section Aesthetics:
#' `geom_node_text` understands the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten. Italic aesthetics are required but
#' not set by default
#'
#' - **x**
#' - **y**
#' - *label*
#' - alpha
#' - angle
#' - colour
#' - family
#' - fontface
#' - hjust
#' - lineheight
#' - size
#' - vjust
#'
#' @inheritParams ggplot2::geom_text
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x and y are mapped to x and y in
#' the node data.
#'
#' @param nudge_x,nudge_y Horizontal and vertical adjustment to nudge labels by.
#' Useful for offsetting text from points, particularly on discrete scales.
#'
#' @param repel If `TRUE`, text labels will be repelled from each other
#' to avoid overlapping, using the `GeomTextRepel` geom from the
#' ggrepel package.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_node_*
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('bull') %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE))
#'
#' ggraph(gr, 'stress') +
#'   geom_node_point(aes(label = class))
#'
#' ggraph(gr, 'stress') +
#'   geom_node_label(aes(label = class), repel = TRUE)
#' @importFrom ggrepel GeomTextRepel
#' @export
#'
geom_node_text <- function(
  mapping = NULL,
  data = NULL,
  position = 'identity',
  parse = FALSE,
  nudge_x = 0,
  nudge_y = 0,
  check_overlap = FALSE,
  show.legend = NA,
  repel = FALSE,
  ...
) {
  if (!missing(nudge_x) || !missing(nudge_y)) {
    if (!missing(position)) {
      cli::cli_abort(c(
        "both {.arg position} and {.arg nudge_x}/{.arg nudge_y} are supplied",
        "i" = "Only use one approach to alter the position"
      ))
    }
    position <- position_nudge(nudge_x, nudge_y)
  }
  params <- list2(parse = parse, ...)
  stat <- StatFilter
  if (repel) {
    geom <- GeomTextRepel
  } else {
    geom <- GeomText
    params$check_overlap <- check_overlap
    if (check_overlap && is.null(data)) {
      stat <- StatReverse
    }
  }

  mapping <- aes_intersect(mapping, aes(x = x, y = y))
  layer(
    data = data,
    mapping = mapping,
    stat = stat,
    geom = geom,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = params
  )
}
#' @rdname geom_node_text
#'
#' @inheritParams ggplot2::geom_label
#'
#' @importFrom ggrepel GeomLabelRepel
#' @export
#'
geom_node_label <- function(
  mapping = NULL,
  data = NULL,
  position = 'identity',
  parse = FALSE,
  nudge_x = 0,
  nudge_y = 0,
  label.padding = unit(0.25, 'lines'),
  label.r = unit(0.15, 'lines'),
  show.legend = NA,
  repel = FALSE,
  ...
) {
  if (!missing(nudge_x) || !missing(nudge_y)) {
    if (!missing(position)) {
      cli::cli_abort(c(
        "both {.arg position} and {.arg nudge_x}/{.arg nudge_y} are supplied",
        "i" = "Only use one approach to alter the position"
      ))
    }
    position <- position_nudge(nudge_x, nudge_y)
  }
  params <- list2(
    parse = parse,
    label.padding = label.padding,
    label.r = label.r,
    ...
  )
  if (repel) {
    geom <- GeomLabelRepel
  } else {
    geom <- GeomLabel
  }

  mapping <- aes_intersect(mapping, aes(x = x, y = y))
  layer(
    data = data,
    mapping = mapping,
    stat = StatFilter,
    geom = geom,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = params
  )
}

#' Create small multiples based on node attributes
#'
#' This function is equivalent to [ggplot2::facet_wrap()] but only
#' facets nodes. Edges are drawn if their terminal nodes are both present in a
#' panel.
#'
#' @inheritParams ggplot2::facet_wrap
#'
#' @family ggraph-facets
#'
#' @examples
#' library(tidygraph)
#' gr <- as_tbl_graph(highschool) %>%
#'   mutate(popularity = as.character(cut(centrality_degree(mode = 'in'),
#'     breaks = 3,
#'     labels = c('low', 'medium', 'high')
#'   )))
#' ggraph(gr) +
#'   geom_edge_link() +
#'   geom_node_point() +
#'   facet_nodes(~popularity)
#' @export
#'
facet_nodes <- function(
  facets,
  nrow = NULL,
  ncol = NULL,
  scales = 'fixed',
  shrink = TRUE,
  labeller = 'label_value',
  as.table = TRUE,
  switch = deprecated(),
  drop = TRUE,
  dir = 'h',
  strip.position = 'top'
) {
  # Work around non-lifecycle deprecation
  if (
    !lifecycle::is_present(switch) &&
      utils::packageVersion('ggplot2') <= '3.3.6'
  ) {
    switch <- NULL
  }
  facet <- facet_wrap(
    facets = facets,
    nrow = nrow,
    ncol = ncol,
    scales = scales,
    shrink = shrink,
    labeller = labeller,
    as.table = as.table,
    switch = switch,
    drop = drop,
    dir = dir,
    strip.position = strip.position
  )
  seed <- sample(.Machine$integer.max, 1L)
  facet$params$facets[] <- lapply(seq_along(facet$params$facets), function(i) {
    rlang::quo(withr::with_seed(seed, !!facet$params$facets[[i]]))
  })
  ggproto(NULL, FacetNodes, shrink = shrink, params = facet$params)
}

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
FacetNodes <- ggproto(
  'FacetNodes',
  FacetWrap,
  compute_layout = function(data, params) {
    plot_data <- data[[1]]
    data <- split(data, vapply(data, data_type, character(1)))
    facet_data <- data$node_ggraph

    panels <- FacetWrap$compute_layout(facet_data, params)
    node_placement <- if (nrow(panels) == 1) {
      list(`1` = plot_data$ggraph_index)
    } else {
      node_map <- FacetWrap$map_data(plot_data, panels, params)
      node_map <- node_map[order(node_map$.ggraph.index), , drop = FALSE]
      split(node_map$.ggraph.index, node_map$PANEL)
    }
    attr(panels, 'node_placement') <- node_placement
    panels
  },
  map_data = function(data, layout, params) {
    switch(
      data_type(data),
      edge_ggraph = {
        node_placement <- attr(layout, 'node_placement')
        edge_map <- Map(
          function(map, nodes) {
            map[map$from %in% nodes & map$to %in% nodes, , drop = FALSE]
          },
          map = rep(list(data), length(node_placement)),
          nodes = node_placement
        )
        panel <- rep(seq_along(edge_map), vapply(edge_map, nrow, numeric(1)))
        edge_map <- vec_rbind(!!!edge_map)
        edge_map$PANEL <- factor(panel, levels = levels(layout$PANEL))
        edge_map
      },
      node_ggraph = ,
      {
        FacetWrap$map_data(data, layout, params)
      }
    )
  }
)

#' @importFrom grid grob is.unit unit gTree
cappedPathGrob <- function(
  x,
  y,
  id = NULL,
  id.lengths = NULL,
  arrow = NULL,
  start.cap = NULL,
  start.cap2 = NULL,
  start.captype = 'circle',
  end.cap = NULL,
  end.cap2 = NULL,
  end.captype = 'circle',
  default.units = 'npc',
  name = NULL,
  gp = gpar(),
  vp = NULL,
  constant = TRUE
) {
  if (!is.unit(x)) {
    x <- unit(x, default.units)
  }
  if (!is.unit(y)) {
    y <- unit(y, default.units)
  }
  if (is.null(id)) {
    if (!is.null(id.lengths)) {
      id <- rep(seq_along(id.lengths), id.lengths)
    } else {
      id <- rep(1, length(x))
    }
  }
  n <- length(unique0(id))
  start.cap <- validate_cap(start.cap, default.units, n)
  if (is.null(start.cap2)) {
    start.cap2 <- start.cap
  } else {
    start.cap2 <- validate_cap(start.cap2, default.units, n)
  }
  end.cap <- validate_cap(end.cap, default.units, n)
  if (is.null(end.cap2)) {
    end.cap2 <- end.cap
  } else {
    end.cap2 <- validate_cap(end.cap2, default.units, n)
  }
  if (!all(c(start.captype, end.captype) %in% c('circle', 'rect'))) {
    cli::cli_abort(
      '{.arg captype} must be either {.val circle} or {.val rect}',
      call. = FALSE
    )
  }
  start.captype <- rep(start.captype, length.out = n)
  end.captype <- rep(end.captype, length.out = n)

  n_points <- length(x)
  group_diff <- id[-1] != id[-n_points]
  start <- c(TRUE, group_diff)
  end <- c(group_diff, TRUE)

  if (constant) {
    gp <- lapply(gp, function(par) {
      if (length(par) == length(end)) {
        par[start]
      } else {
        par
      }
    })
  } else {
    gp <- lapply(gp, function(par) {
      if (length(par) == length(end)) {
        par[!end]
      } else {
        par
      }
    })
  }
  class(gp) <- 'gpar'

  if (is.null(start.cap) && is.null(end.cap)) {
    if (constant) {
      grob(
        x = x,
        y = y,
        id = id,
        id.lengths = NULL,
        arrow = arrow,
        name = name,
        gp = gp,
        vp = vp,
        cl = 'polyline'
      )
    } else {
      grob(
        x0 = x[!end],
        y0 = y[!end],
        x1 = x[!start],
        y1 = y[!start],
        id = id[!end],
        arrow = arrow,
        name = name,
        gp = gp,
        vp = vp,
        cl = 'segments'
      )
    }
  } else {
    gTree(
      x = x,
      y = y,
      id = id,
      arrow = arrow,
      constant = constant,
      start = start,
      end = end,
      start.cap = start.cap,
      start.cap2 = start.cap2,
      start.captype = start.captype,
      end.cap = end.cap,
      end.cap2 = end.cap2,
      end.captype = end.captype,
      name = name,
      gp = gp,
      vp = vp,
      cl = 'cappedpathgrob'
    )
  }
}
#' Dynamic capping of paths
#'
#' This function takes care of updating which parts of the paths get removed
#' when the device dimensions are updated.
#'
#' @importFrom grid convertX convertY convertWidth convertHeight unit grob makeContent setChildren gList
#' @export
#' @keywords internal
makeContent.cappedpathgrob <- function(x) {
  x_new <- convertX(x$x, 'mm', TRUE)
  y_new <- convertY(x$y, 'mm', TRUE)
  start.cap <- convertWidth(x$start.cap, 'mm', TRUE)
  end.cap <- convertWidth(x$end.cap, 'mm', TRUE)
  start.cap2 <- convertHeight(x$start.cap2, 'mm', TRUE)
  end.cap2 <- convertHeight(x$end.cap2, 'mm', TRUE)
  truncated <- cut_lines(
    as.numeric(x_new),
    as.numeric(y_new),
    as.integer(x$id),
    as.numeric(start.cap),
    as.numeric(start.cap2),
    as.numeric(end.cap),
    as.numeric(end.cap2),
    as.character(x$start.captype),
    as.character(x$end.captype)
  )
  keep <- !is.na(truncated$x)
  if (!any(keep)) {
    lines <- zeroGrob()
  } else if (x$constant) {
    x_new <- truncated$x[keep]
    y_new <- truncated$y[keep]
    id <- x$id[keep]
    all_id <- unique0(id)
    gp <- lapply(x$gp, function(par) {
      if (length(par) == 1) {
        par
      } else {
        par[all_id]
      }
    })
    class(gp) <- 'gpar'
    id <- match(id, all_id)
    lines <- grob(
      x = unit(x_new, 'mm'),
      y = unit(y_new, 'mm'),
      id = id,
      id.lengths = NULL,
      arrow = x$arrow,
      name = x$name,
      gp = gp,
      vp = x$vp,
      cl = 'polyline'
    )
  } else {
    x0 <- truncated$x[!x$end]
    y0 <- truncated$y[!x$end]
    x1 <- truncated$x[!x$start]
    y1 <- truncated$y[!x$start]
    lines <- grob(
      x0 = unit(x0, 'mm'),
      y0 = unit(y0, 'mm'),
      x1 = unit(x1, 'mm'),
      id = x$id[!x$end],
      y1 = unit(y1, 'mm'),
      arrow = x$arrow,
      name = x$name,
      gp = x$gp,
      vp = x$vp,
      cl = 'segments'
    )
  }
  setChildren(x, gList(lines))
}
#' @importFrom grid is.unit unit
validate_cap <- function(cap, default.units, n) {
  if (is.null(cap)) {
    return()
  }
  if (!is.unit(cap)) {
    cap <- unit(cap, default.units)
  }
  rep(cap, length.out = n)
}

#' Create an unrooted layout using equal-angle or equal-daylight
#'
#' When drawing unrooted trees the standard dendrogram layout is a bad fit as it
#' implicitly creates a visual root node. Instead it is possible to spread the
#' leafs out on the plane without putting any special emphasis on a particular
#' node using an unrooted layout. The standard algorithm is the equal angle
#' algorithm, but it can struggle with optimising the leaf distribution for
#' large trees with very uneven branch length. The equal daylight
#' algorithm modifies the output of the equal angle algorithm to better disperse
#' the leaves, at the cost of higher computational cost and the possibility of
#' edge crossings for very large unbalanced trees. For standard sized trees the
#' daylight algorithm is far superior and not too heavy so it is the default.
#'
#' @note
#' Unrooted is a layout intended for undirected trees, that is, graphs with no
#' cycles. If the provided graph does not fit this format an attempt to convert
#' it to such a format will be made.
#'
#' @param graph A tbl_graph object
#' @param daylight Should equal-daylight adjustments be made
#' @param length An expression evaluating to the branch length of each edge
#' @param tolerance The threshold for mean angular adjustment before terminating
#' the daylight adjustment
#' @param rotation_mod A modifier for the angular adjustment of each branch. Set
#' it below 1 to let the daylight adjustment progress more slowly
#' @param maxiter The maximum number of iterations in the the daylight
#' adjustment
#' @param circular ignored
#'
#' @return A data.frame with the columns `x`, `y`, `circular`, `leaf` as well as
#' any information stored as node variables in the tbl_graph object.
#'
#' @references
#' Felsenstein, J. (2004) *Drawing Trees*, in Inferring Phylogenies. Sinauer
#' Assoc., pp 573-584
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom igraph as.undirected gorder
layout_tbl_graph_unrooted <- function(
  graph,
  daylight = TRUE,
  length = NULL,
  tolerance = 0.05,
  rotation_mod = 1,
  maxiter = 100,
  circular = FALSE
) {
  extra_data <- as_tibble(graph, active = 'nodes')
  graph <- as_tbl_graph(as.undirected(graph))
  length <- enquo(length)
  length <- eval_tidy(length, .E())
  hierarchy <- tree_to_hierarchy(
    graph,
    'out',
    seq_len(gorder(graph)),
    weight = NULL,
    length
  )
  layout <- unrooted(
    as.integer(hierarchy$parent),
    as.integer(hierarchy$order),
    as.numeric(hierarchy$height),
    as.logical(daylight),
    as.numeric(tolerance),
    as.numeric(rotation_mod),
    as.integer(maxiter)
  )[-1, ]
  nodes <- data_frame0(
    x = layout[, 1],
    y = layout[, 2],
    circular = FALSE,
    leaf = degree(graph) == 1
  )
  combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
}

#' @rdname ggraph
#' @aliases layout_tbl_graph
#'
#' @importFrom igraph gorder V<-
#' @export
#'
create_layout.tbl_graph <- function(graph, layout, circular = FALSE, ...) {
  graph <- mutate(
    ungroup(activate(graph, 'nodes')),
    .ggraph.orig_index = seq_len(graph_order())
  )
  graph <- prepare_graph(graph, layout, ...)
  .register_graph_context(graph, free = TRUE)
  if (gorder(graph) == 0) {
    layout <- data_frame0(
      x = numeric(),
      y = numeric(),
      circular = logical(),
      .N()
    )
  } else {
    layout <- layout_to_table(layout, graph, circular = circular, ...)
  }
  layout <- as_tibble(layout)
  layout$.ggraph.index <- seq_len(nrow(layout))
  graph <- attr(layout, 'graph') %||% graph
  V(graph)$.ggraph_layout_x <- layout$x
  V(graph)$.ggraph_layout_y <- layout$y
  attr(layout, 'graph') <- graph
  attr(layout, 'circular') <- circular
  class(layout) <- c(
    'layout_tbl_graph',
    'layout_ggraph',
    class(layout)
  )
  check_layout(layout)
}
#' @export
collect_edges.layout_tbl_graph <- function(layout) {
  gr <- attr(layout, 'graph')
  edges <- as_tibble(gr, active = 'edges')
  edges$circular <- rep(attr(layout, 'circular'), nrow(edges))
  data_frame0(edges)
}
#' @importFrom igraph shortest_paths
#' @importFrom rlang enquo eval_tidy
#' @export
collect_connections.layout_tbl_graph <- function(
  layout,
  from,
  to,
  weight = NULL,
  mode = 'all',
  ...
) {
  from <- match(from, layout$.ggraph.orig_index)
  to <- match(to, layout$.ggraph.orig_index)
  weight <- eval_tidy(enquo(weight), collect_edges(layout))
  if (is.null(weight)) {
    weight <- NA
  }
  graph <- attr(layout, 'graph')
  to_ind <- split(seq_along(to), from)
  connections <- lapply(seq_along(to_ind), function(i) {
    paths <- shortest_paths(
      graph,
      as.integer(names(to_ind)[i]),
      to[to_ind[[i]]],
      mode = mode,
      weights = weight
    )$vpath
    lapply(paths, as.numeric)
  })
  connections <- unlist(connections, recursive = FALSE)
  connections[match(seq_along(to), unlist(to_ind))]
}

# HELPERS -----------------------------------------------------------------

is.igraphlayout <- function(type) {
  if (type %in% igraphlayouts) {
    TRUE
  } else if (
    any(paste0(c('as_', 'in_', 'with_', 'on_'), type) %in% igraphlayouts)
  ) {
    TRUE
  } else {
    FALSE
  }
}
as.igraphlayout <- function(type, call = caller_env()) {
  if (type %in% igraphlayouts) {
    layout <- type
  } else {
    new_type <- paste0(c('as_', 'in_', 'with_', 'on_'), type)
    type_ind <- which(new_type %in% igraphlayouts)
    if (length(type_ind) == 0) {
      cli::cli_abort('Cannot find the igraph layout {.val {type}}', call = call)
    }
    layout <- new_type[type_ind]
  }
  paste0('layout_', layout)
}
#' @importFrom igraph gorder permute
prepare_graph <- function(graph, layout, direction = 'out', ...) {
  if (!is.character(layout)) {
    return(graph)
  }
  is_hierarchy <- layout %in%
    c(
      'dendrogram',
      'treemap',
      'circlepack',
      'partition',
      'cactustree',
      'htree'
    )
  graph_is_treeish <- with_graph(graph, graph_is_tree() || graph_is_forest())
  if (is_hierarchy || (layout == 'auto' && graph_is_treeish)) {
    if (!graph_is_treeish) {
      graph <- graph_to_tree(graph, mode = direction)
    }
    graph <- permute(
      graph,
      match(seq_len(gorder(graph)), order(node_depth(graph, direction)))
    )
  }
  if (inherits(graph, "sfnetwork")) {
    graph
  } else {
    as_tbl_graph(graph)
  }
}
#' @importFrom igraph degree unfold_tree components induced_subgraph vertex_attr vertex_attr<- is.directed simplify
graph_to_tree <- function(graph, mode) {
  if (!is.directed(graph)) {
    cli::cli_abort('{.arg graph} must be directed')
  }
  graph <- simplify(graph, edge.attr.comb = 'first')
  parent_dir <- if (mode == 'out') 'in' else 'out'
  comp <- components(graph, 'weak')
  graphs <- lapply(seq_len(comp$no), function(i) {
    graph <- induced_subgraph(graph, which(comp$membership == i))
    n_parents <- degree(graph, mode = parent_dir)
    if (!any(n_parents == 0)) {
      cli::cli_abort(c(
        '{.arg graph} doesn\'t contain a root.',
        i = ' Provide graph with one parentless node'
      ))
    }
    if (any(n_parents > 1)) {
      cli::cli_inform('Multiple parents. Unfolding graph')
      root <- which(degree(graph, mode = parent_dir) == 0)
      if (length(root) > 1) {
        cli::cli_inform('Multiple roots in graph. Choosing the first')
        root <- root[1]
      }
      tree <- unfold_tree(graph, mode = mode, roots = root)
      vattr <- lapply(vertex_attr(graph), `[`, i = tree$vertex_index)
      vertex_attr(tree$tree) <- vattr
      graph <- tree$tree
    }
    as_tbl_graph(graph)
  })
  inject(bind_graphs(!!!graphs))
}
#' @importFrom igraph gorder as_edgelist delete_vertex_attr is.named
tree_to_hierarchy <- function(graph, mode, sort.by, weight, height = NULL) {
  if (is.named(graph)) {
    graph <- delete_vertex_attr(graph, 'name')
  }
  parent_col <- if (mode == 'out') 1 else 2
  node_col <- if (mode == 'out') 2 else 1
  edges <- as_edgelist(graph)
  hierarchy <- data_frame0(parent = rep(0, gorder(graph)))
  hierarchy$parent[edges[, node_col]] <- edges[, parent_col]
  if (is.null(sort.by)) {
    hierarchy$order <- seq_len(nrow(hierarchy)) + 1
  } else {
    hierarchy$order <- order(sort.by) + 1
  }
  if (is.null(height)) {
    hierarchy$height <- 1
  } else {
    hierarchy$height <- 0
    hierarchy$height[edges[, node_col]] <- height
  }
  leaf <- degree(graph, mode = mode) == 0
  if (is.null(weight)) {
    hierarchy$weight <- 0
    hierarchy$weight[leaf] <- 1
  } else {
    if (!is.numeric(weight)) {
      cli::cli_abort('{.arg weight} must be numeric')
    }
    hierarchy$weight <- weight
    if (any(hierarchy$weight[!leaf] != 0)) {
      cli::cli_inform('Non-leaf weights ignored')
    }
    if (any(hierarchy$weight[leaf] == 0)) {
      cli::cli_abort('leaf nodes must have a weight')
    }
    hierarchy$weight[!leaf] <- 0
  }
  hierarchy <- hierarchy[c(1, seq_len(nrow(hierarchy))), ]
  hierarchy$parent[1] <- -1
  hierarchy$order[1] <- 1
  hierarchy
}
#' @importFrom igraph bfs degree gorder
node_depth <- function(graph, mode) {
  mode_rev <- switch(
    mode,
    `in` = 'out',
    out = 'in',
    cli::cli_abort(c(
      'Unknown graph mode {.val {mode}}',
      i = "use either {.val in} or {.val out}"
    ))
  )
  root <- which(degree(graph, mode = mode_rev) == 0)
  depth <- rep(NA_integer_, gorder(graph))
  for (i in root) {
    depth[!is.finite(depth)] <- unname(
      bfs(graph, root = i, unreachable = FALSE, dist = TRUE)$dist
    )[!is.finite(depth)]
  }
  depth
}
#' @importFrom rlang .data
add_direction <- function(graph, pos, direction = 'out') {
  if (igraph::gsize(graph) == 0) {
    return(graph)
  }
  graph <- activate(graph, 'edges')
  graph <- mutate(
    graph,
    direction = ifelse(pos$x[.data$to] < pos$x[.data$from], 'right', 'left')
  )
  if (direction == 'in') {
    graph <- mutate(graph, ifelse(.data$direction == 'left', 'right', 'left'))
  }
  graph <- mutate(graph, direction = factor(.data$direction))
  graph
}

#' Convert a layout to a table
#'
#' This generic takes care of dispatching various layout types (names,
#' functions, tables) to their respective functions that will return a valid
#' layout table.
#'
#' @param layout A supported object
#' @param graph A `tbl_graph`
#' @param ... passed on to implementations
#'
#' @return A valid data.frame
#'
#' @keywords internal
#' @export
layout_to_table <- function(layout, graph, ...) {
  UseMethod('layout_to_table')
}
#' @export
layout_to_table.default <- function(layout, graph, ...) {
  cli::cli_abort('Unknown {.arg layout}')
}
#' @export
layout_to_table.character <- function(layout, graph, circular, ...) {
  if (is.igraphlayout(layout)) {
    layout_tbl_graph_igraph(graph, layout, circular, ...)
  } else {
    layout_fun <- get(paste0('layout_tbl_graph_', layout))
    layout_fun(graph, circular = circular, ...)
  }
}
#' @export
layout_to_table.matrix <- function(layout, graph, ...) {
  layout <- data_frame0(x = layout[, 1], y = layout[, 2])
  layout_to_table(layout, graph, ...)
}
#' @export
layout_to_table.data.frame <- function(layout, graph, ...) {
  combine_layout_nodes(layout, as_tibble(graph, active = 'nodes'))
}
#' @export
layout_to_table.function <- function(layout, graph, circular, ...) {
  layout <- if ('circular' %in% names(formals(layout))) {
    layout(graph, circular = circular, ...)
  } else {
    layout(graph, ...)
  }
  if (!is.tbl_graph(layout) && !is.data.frame(layout)) {
    layout <- try_fetch(
      data_frame0(layout),
      error = function(e) {
        try_fetch(
          as_tbl_graph(layout),
          error = function(e) {
            cli::cli_abort(
              'layout function must return an object coerceble to either a {.cls data.frame} or {.cls tbl_graph}'
            )
          }
        )
      }
    )
  }
  if (is.tbl_graph(layout)) {
    graph <- layout
    nodes <- as_tibble(graph, active = 'nodes')
    attr(nodes, 'graph') <- graph
  } else {
    nodes <- combine_layout_nodes(layout, as_tibble(graph, active = 'nodes'))
  }
  nodes
}

igraphlayouts <- c(
  'as_bipartite',
  'as_star',
  'as_tree',
  'in_circle',
  'nicely',
  'with_dh',
  'with_drl',
  'with_gem',
  'with_graphopt',
  'on_grid',
  'with_mds',
  'with_sugiyama',
  'on_sphere',
  'randomly',
  'with_fr',
  'with_kk',
  'with_lgl'
)

#' Create edge extractor function
#'
#' This function returns another function that can extract edges from a
#' ggraph_layout object. The functionality of the returned function is decided
#' by the arguments to `get_edges`. The need for `get_edges` is mainly to
#' pass to the `data` argument of the different `geom_edge_*`
#' functions in order to present them with the right kind of data. In general
#' each `geom_edge_*` has the default set correctly so there is only need
#' to modify the data argument if parallel edges should be collapsed.
#'
#' @details
#' There are two types of return formats possible for the result of the returned
#' function:
#'
#' \describe{
#'   \item{short}{In this format each edge is described in one line in the
#'   format expected for [ggplot2::geom_segment()], that is, the start
#'   node position is encoded in the `x` and `y` column and the end
#'   node position is encoded in the `xend` and `yend` column. If node
#'   parameters are added to the edge the name of the parameters will be
#'   prefixed with `node1.` for the start node and `node2.` for the
#'   end node.}
#'   \item{long}{In this format each edge consists of two rows with matching
#'   `edge.id` value. The start and end position are both encoded in the
#'   `x` and `y` column. The relative position of the rows determines
#'   which is the start and end node, the first occurring being the start node.
#'   If node parameters are added to the edge data the name of the parameters
#'   will be prefixed with `node.`.}
#' }
#'
#' Node parameters are automatically added so it is possible to format edge
#' aesthetics according to start or end node parameters, or interpolate edge
#' aesthetics between start and end node parameters. Node parameters will be
#' prefixed to avoid name clash with edge parameters. The prefix depends on the
#' format (see above).
#'
#' If the graph is not simple (it contains at most one edge between each node
#' pair) it can be collapsed so either all edges between two nodes or all edges
#' of the same direction between two nodes are merged. The edge parameters are
#' taken from the first occurring edge, so if some more sophisticated summary is
#' needed it is suggested that the graph be tidied up before plotting with
#' ggraph.
#'
#' @param format Either `'short'` (the default) or `'long'`. See
#' details for a descriptions of the differences
#'
#' @param collapse Either `'none'` (the default), `'all'` or
#' `'direction'`. Specifies whether parallel edges should be merged. See
#' details for more information
#'
#' @param ... Additional data that will be cbind'ed together with the returned
#' edge data. Accepts expressions that will be evaluated on the edge data
#'
#' @return A data.frame with columns dependent on format as well as the graph
#' type. In addition to the columns discussed in the details section,
#' the data.frame will always contain the columns `from`, `to` and
#' `circular`, the two former giving the indexes of the start and end node
#'  and the latter if the layout is circular (needed for correct formatting of
#'  some `geom_edge_*`). The graph dependent information is:
#'
#' \describe{
#'   \item{dendrogram}{A `label` column will hold the value of the
#'   `edgetext` attribute. In addition any value stored in the
#'   `edgePar` attribute will be added. Lastly a `direction` column
#'   will hold the relative position between the start and end nodes (needed for
#'   correct formatting of [geom_edge_elbow()]).}
#'   \item{igraph}{All edge attributes of the original graph object is added as
#'   columns to the data.frame}
#' }
#'
#' @family extractors
#'
#' @export
#'
get_edges <- function(format = 'short', collapse = 'none', ...) {
  if (!collapse %in% c('none', 'all', 'direction')) {
    cli::cli_abort(
      '{.arg collapse} must be either {.val none}, {.val all} or {.val direction}'
    )
  }
  dots <- enquos(...)
  function(layout) {
    edges <- collect_edges(layout)
    edges <- switch(
      collapse,
      none = edges,
      all = collapse_all_edges(edges),
      direction = collapse_dir_edges(edges)
    )
    edges <- switch(
      format,
      short = format_short_edges(edges, layout),
      long = format_long_edges(edges, layout),
      cli::cli_abort(
        'Unknown {.arg format}. Use either {.val short} or {.val long}'
      )
    )
    extra_data <- lapply(dots, function(x) {
      val <- eval_tidy(x, edges)
      rep(val, length.out = nrow(edges))
    })
    if (length(extra_data) > 0) {
      edges <- cbind(
        edges,
        data_frame0(!!!extra_data)
      )
    }
    attr(edges, 'type_ggraph') <- 'edge_ggraph'
    edges
  }
}
#' @rdname internal_extractors
#' @export
collect_edges <- function(layout) {
  UseMethod('collect_edges', layout)
}
#' @export
collect_edges.default <- function(layout) {
  attr(layout, 'edges')
}
check_short_edges <- function(edges) {
  if (!inherits(edges, 'data.frame')) {
    cli::cli_abort('{.arg edges} must by of class {.cls data.frame}')
  }
  if (
    !all(
      c('from', 'to', 'x', 'y', 'xend', 'yend', 'circular', 'edge.id') %in%
        names(edges)
    )
  ) {
    cli::cli_abort(
      '{.arg edges} must contain the columns {.and {.col {c("from", "to", "x", "y", "xend", "yend", "circular", "edge.id")}}}'
    )
  }
  if (!is.logical(edges$circular)) {
    cli::cli_abort('The {.col circular} column must be logical')
  }
  edges
}
check_long_edges <- function(edges) {
  if (!inherits(edges, 'data.frame')) {
    cli::cli_abort('{.arg edges} must by of class {.cls data.frame}')
  }
  if (!all(c('edge.id', 'node', 'x', 'y', 'circular') %in% names(edges))) {
    cli::cli_abort(
      '{.arg edges} must contain the columns {.and {.col {c("edge.id", "node", "x", "y", "circular")}}}'
    )
  }
  if (!all(range(table(edges$edge.id)) == 2)) {
    cli::cli_abort('Each edge must consist of two rows')
  }
  if (!is.logical(edges$circular)) {
    cli::cli_abort('The {.col circular} column must be logical')
  }
  edges
}
add_edge_coordinates <- function(edges, layout) {
  edges$x <- layout$x[edges$from]
  edges$y <- layout$y[edges$from]
  edges$xend <- layout$x[edges$to]
  edges$yend <- layout$y[edges$to]
  edges
}
format_short_edges <- function(edges, layout) {
  edges <- add_edge_coordinates(edges, layout)
  nodes1 <- layout[edges$from, , drop = FALSE]
  names(nodes1) <- paste0('node1.', names(nodes1))
  nodes2 <- layout[edges$to, , drop = FALSE]
  names(nodes2) <- paste0('node2.', names(nodes2))
  edges <- cbind(edges, nodes1, nodes2)
  rownames(edges) <- NULL
  edges$edge.id <- seq_len(nrow(edges))
  check_short_edges(edges)
}
format_long_edges <- function(edges, layout) {
  from <- cbind(
    edge.id = seq_len(nrow(edges)),
    node = edges$from,
    layout[edges$from, c('x', 'y')],
    edges
  )
  to <- cbind(
    edge.id = seq_len(nrow(edges)),
    node = edges$to,
    layout[edges$to, c('x', 'y')],
    edges
  )
  edges <- vec_rbind(from, to)
  node <- layout[edges$node, , drop = FALSE]
  names(node) <- paste0('node.', names(node))
  edges <- cbind(edges, node)
  rownames(edges) <- NULL
  check_long_edges(edges[order(edges$edge.id), ])
}
complete_edge_aes <- function(aesthetics) {
  if (is.null(aesthetics)) {
    return(aesthetics)
  }
  if (any(names(aesthetics) == 'color')) {
    names(aesthetics)[names(aesthetics) == 'color'] <- 'colour'
  }
  expand_edge_aes(aesthetics)
}
expand_edge_aes <- function(x) {
  short_names <- names(x) %in%
    c(
      'colour',
      'color',
      'fill',
      'linetype',
      'shape',
      'size',
      'width',
      'alpha',
      'linewidth'
    )
  names(x)[short_names] <- paste0('edge_', names(x)[short_names])
  if (all(c('edge_linewidth', 'edge_width') %in% names(x) == c(TRUE, FALSE))) {
    names(x)[names(x) == 'edge_linewidth'] <- 'edge_width'
  }
  x
}
#' @importFrom dplyr group_by top_n ungroup
collapse_all_edges <- function(edges) {
  from <- pmin(edges$from, edges$to)
  to <- pmax(edges$to, edges$from)
  id <- paste(from, to, sep = '-')
  edges[!duplicated(id), , drop = FALSE]
}
#' @importFrom dplyr group_by top_n ungroup
collapse_dir_edges <- function(edges) {
  id <- paste(edges$from, edges$to, sep = '-')
  edges[!duplicated(id), , drop = FALSE]
}

#' Place node to emphasize group structure
#'
#' This layout is optimised for drawing small-world types of graphs often found
#' in social networks, where distinct groups are still highly connected to the
#' remaining graph. Typical layouts struggle with this as they attempt to
#' minimise the edge length of all edges equally. The backbone layout is based
#' on weighing edges based on how well they hold together communities. The end
#' result is that communities tend to stick together despite high
#' interconnectivity.
#'
#' @param graph A tbl_graph object
#' @param keep The fraction of edges to use for creating the backbone
#' @param circular ignored
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#' Further an edge attribute called `backbone` is added giving whether the edge
#' was selected as backbone.
#'
#' @references
#' Nocaj, A., Ortmann, M., & Brandes, U. (2015). *Untangling the hairballs of
#' multi-centered, small-world online social media networks.* Journal of Graph
#' Algorithms and Applications: JGAA, 19(2), 595-618.
#'
#' @family layout_tbl_graph_*
#'
#' @author The underlying algorithm is implemented in the graphlayouts package
#' by David Schoch
#'
#' @importFrom graphlayouts layout_as_backbone
#'
layout_tbl_graph_backbone <- function(graph, keep = 0.2, circular = FALSE) {
  layout <- layout_as_backbone(graph, keep = keep, backbone = TRUE)
  xy <- layout$xy
  nodes <- data_frame0(x = xy[, 1], y = xy[, 2], circular = FALSE)
  nodes <- combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
  graph <- activate(graph, 'edges')
  graph <- mutate(graph, backbone = seq_len(graph_size()) %in% layout$backbone)
  attr(nodes, 'graph') <- activate(graph, 'nodes')
  nodes
}

#' Place nodes in circles according to centrality measure
#'
#' This layout places nodes in circles with the radii relative to a given
#' centrality measure. Under the hood it use stress majorisation to place nodes
#' optimally given the radius constraint.
#'
#' @param graph A tbl_graph object
#' @param centrality An expression evaluating to a centrality measure for the
#' nodes. See the different `centrality_*()` algorithms in tidygraph for a
#' selection.
#' @param scale Should the centrality measure be scaled between 0 and 100
#' @param tseq Transitioning steps
#' @param group An expression evaluating to a grouping of the nodes. If given
#' the layout will keep grouped nodes within an angle range of the origin
#' @param shrink shrink the reserved angle range for a group to increase the
#' gaps between groups
#' @inheritParams layout_tbl_graph_stress
#'
#' @return A data.frame with the columns `x`, `y`, `circular`, `centrality` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @references
#' Brandes, U., & Pich, C. (2011). *More flexible radial layout.* Journal of
#' Graph Algorithms and Applications, 15(1), 157-173.
#'
#' @family layout_tbl_graph_*
#'
#' @author The underlying algorithm is implemented in the graphlayouts package
#' by David Schoch
#'
#' @importFrom graphlayouts layout_with_centrality layout_with_centrality_group
#' @importFrom rlang eval_tidy enquo
layout_tbl_graph_centrality <- function(
  graph,
  centrality,
  scale = TRUE,
  niter = 500,
  tolerance = 1e-4,
  tseq = seq(0, 1, 0.2),
  group = NULL,
  shrink = 10,
  circular = FALSE
) {
  centrality <- eval_tidy(enquo(centrality), .N())
  group <- eval_tidy(enquo(group), .N())
  if (is.null(group)) {
    xy <- layout_with_centrality(
      graph,
      cent = centrality,
      scale = scale,
      iter = niter,
      tol = tolerance,
      tseq = tseq
    )
  } else {
    xy <- layout_with_centrality_group(
      graph,
      cent = centrality,
      group = group,
      shrink = shrink,
      scale = scale,
      iter = niter,
      tol = tolerance,
      tseq = tseq
    )
  }

  nodes <- data_frame0(
    x = xy[, 1],
    y = xy[, 2],
    centrality = centrality,
    group = group,
    circular = FALSE
  )
  combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
}

#' Place nodes according to their eigenvalues
#'
#' This layout is based on the idea of spectral layouts where node coordinates
#' are calculated directly by decomposing a matrix representation of the graph
#' and extracting the eigenvectors.
#'
#' @param graph A tbl_graph object
#' @param type The type of matrix to extract the eigenvectors from. Either
#' `'laplacian'` or `'adjacency'`
#' @param eigenvector The eigenvector to use for coordinates. Either `'smallest'`
#' or `'largest'`
#' @param circular ignored
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @family layout_tbl_graph_*
#'
#' @author The underlying algorithm is implemented in the graphlayouts package
#' by David Schoch
#'
#' @importFrom graphlayouts layout_with_eigen
layout_tbl_graph_eigen <- function(
  graph,
  type = 'laplacian',
  eigenvector = 'smallest',
  circular = FALSE
) {
  type <- match.arg(type, c('laplacian', 'adjacency'))
  eigenvector <- match.arg(eigenvector, c('smallest', 'largest'))
  xy <- layout_with_eigen(graph, type = type, ev = eigenvector)
  nodes <- data_frame0(x = xy[, 1], y = xy[, 2], circular = FALSE)
  combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
}

#' Draw edges as straight lines between nodes
#'
#' This geom draws edges in the simplest way - as straight lines between the
#' start and end nodes. Not much more to say about that...
#'
#' @section Edge variants:
#' Many geom_edge_* layers comes in 3 flavors depending on the level of control
#' needed over the drawing. The default (no numeric postfix) generate a number
#' of points (`n`) along the edge and draws it as a path. Each point along
#' the line has a numeric value associated with it giving the position along the
#' path, and it is therefore possible to show the direction of the edge by
#' mapping to this e.g. `colour = after_stat(index)`. The version postfixed with a
#' "2" uses the "long" edge format (see [get_edges()]) and makes it
#' possible to interpolate node parameter between the start and end node along
#' the edge. It is considerable less performant so should only be used if this
#' is needed. The version postfixed with a "0" draws the edge in the most
#' performant way, often directly using an appropriate grob from the grid
#' package, but does not allow for gradients along the edge.
#'
#' Often it is beneficial to stop the drawing of the edge before it reaches the
#' node, for instance in cases where an arrow should be drawn and the arrowhead
#' shouldn't lay on top or below the node point. geom_edge_* and geom_edge_*2
#' supports this through the start_cap and end_cap aesthetics that takes a
#' [geometry()] specification and dynamically caps the termini of the
#' edges based on the given specifications. This means that if
#' `end_cap = circle(1, 'cm')` the edges will end at a distance of 1cm even
#' during resizing of the plot window.
#'
#' All `geom_edge_*` and `geom_edge_*2` have the ability to draw a
#' label along the edge. The reason this is not a separate geom is that in order
#' for the label to know the location of the edge it needs to know the edge type
#' etc. Labels are drawn by providing a label aesthetic. The label_pos can be
#' used to specify where along the edge it should be drawn by supplying a number
#' between 0 and 1. The label_size aesthetic can be used to control the size of
#' the label. Often it is needed to have the label written along the direction
#' of the edge, but since the actual angle is dependent on the plot dimensions
#' this cannot be calculated beforehand. Using the angle_calc argument allows
#' you to specify whether to use the supplied angle aesthetic or whether to draw
#' the label along or across the edge.
#'
#' @section Edge aesthetic name expansion:
#' In order to avoid excessive typing edge aesthetic names are
#' automatically expanded. Because of this it is not necessary to write
#' `edge_colour` within the `aes()` call as `colour` will
#' automatically be renamed appropriately.
#'
#' @section Aesthetics:
#' `geom_edge_link` and `geom_edge_link0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_link2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **group**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_link` and `geom_edge_link2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams ggplot2::geom_path
#' @inheritParams ggplot2::geom_text
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x, y, xend, yend, group and
#' circular are mapped to x, y, xend, yend, edge.id and circular in the edge
#' data.
#'
#' @param data The return of a call to `get_edges()` or a data.frame
#' giving edges in correct format (see details for for guidance on the format).
#' See [get_edges()] for more details on edge extraction.
#'
#' @param n The number of points to create along the path.
#'
#' @param label_colour The colour of the edge label. If `NA` it will use
#' the colour of the edge.
#'
#' @param label_alpha The opacity of the edge label. If `NA` it will use
#' the opacity of the edge.
#'
#' @param label_parse If `TRUE`, the labels will be parsed into expressions
#' and displayed as described in [grDevices::plotmath()].
#'
#' @param angle_calc Either 'none', 'along', or 'across'. If 'none' the label will
#' use the angle aesthetic of the geom. If 'along' The label will be written
#' along the edge direction. If 'across' the label will be written across the
#' edge direction.
#'
#' @param force_flip Logical. If `angle_calc` is either 'along' or 'across'
#' should the label be flipped if it is on it's head. Default to `TRUE`.
#'
#' @param label_dodge A [grid::unit()] giving a fixed vertical shift
#' to add to the label in case of `angle_calc` is either 'along' or 'across'
#'
#' @param label_push A [grid::unit()] giving a fixed horizontal shift
#' to add to the label in case of `angle_calc` is either 'along' or 'across'
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('bull') %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE)) %>%
#'   activate(edges) %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_link(aes(alpha = after_stat(index)))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_link2(aes(colour = node.class))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_link0(aes(colour = class))
#' @rdname geom_edge_link
#' @name geom_edge_link
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatLink
#' @export
StatEdgeLink <- ggproto(
  'StatEdgeLink',
  StatLink,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data <- remove_loop(data)
    if (nrow(data) == 0) {
      return(data)
    }
    StatLink$setup_data(data, params)
  },
  default_aes = aes(filter = TRUE)
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatLink2
#' @export
StatEdgeLink2 <- ggproto(
  'StatEdgeLink2',
  StatLink2,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data <- remove_loop2(data)
    if (nrow(data) == 0) {
      return(data)
    }
    StatLink2$setup_data(data, params)
  },
  default_aes = aes(filter = TRUE)
)
#' @rdname geom_edge_link
#'
#' @importFrom ggforce StatLink
#' @export
geom_edge_link <- function(
  mapping = NULL,
  data = get_edges('short'),
  position = 'identity',
  arrow = NULL,
  n = 100,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      group = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeLink,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = FALSE,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname geom_edge_link
#'
#' @importFrom ggforce StatLink2
#' @export
geom_edge_link2 <- function(
  mapping = NULL,
  data = get_edges('long'),
  position = 'identity',
  arrow = NULL,
  n = 100,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y, group = edge.id))
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeLink2,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = TRUE,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname geom_edge_link
#'
#' @importFrom ggforce StatLink2
#' @export
geom_edge_link0 <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  arrow = NULL,
  lineend = 'butt',
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y, xend = xend, yend = yend))
  layer(
    data = data,
    mapping = mapping,
    stat = StatFilter,
    geom = GeomEdgeSegment,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(arrow = arrow, lineend = lineend, ...)
    )
  )
}

#' Create a node extractor function
#'
#' This function returns another function that can extract nodes from a
#' ggraph_layout object. As a ggraph_layout object is essentially a data.frame
#' of nodes it might seem useless to provide this function, but since the node
#' data is not necessarily available until after the `ggraph()` call it
#' can be beneficial to be able to add information to the node data on a
#' per-layer basis. Unlike [get_edges()] the use of `get_nodes` is not
#' mandatory and is only required if additional data should be added to selected
#' node layers.
#'
#' @param ... Additional data that should be cbind'ed together with the node
#' data.
#'
#' @return A data.frame with the node data as well of any additional data
#' supplied through `...`
#'
#' @family extractors
#'
#' @export
#'
get_nodes <- function(...) {
  function(layout) {
    nodes <- do.call(
      cbind,
      c(
        list(layout),
        lapply(list(...), rep, length.out = nrow(layout)),
        list(stringsAsFactors = FALSE)
      )
    )
    attr(nodes, 'type_ggraph') <- 'node_ggraph'
    nodes
  }
}

#' Manually specify a layout for layout_tbl_graph
#'
#' This layout function lets you pass the node positions in manually. The
#' supplied positions must match the order of the nodes in the tbl_graph
#'
#' @param graph An `tbl_graph` object
#'
#' @param x,y Expressions with the x and y positions of the nodes
#'
#' @param circular Ignored
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph.
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom rlang enquo eval_tidy quo_is_symbol
#'
layout_tbl_graph_manual <- function(graph, x, y, circular) {
  if (circular) {
    warning('circular argument ignored for manual layout')
  }
  x <- enquo(x)
  y <- enquo(y)
  layout <- new_data_frame(list(
    x = eval_tidy(x, .N()),
    y = eval_tidy(y, .N())
  ))
  extra_data <- as_tibble(graph, active = 'nodes')
  warn_dropped_vars(layout, extra_data)
  layout <- cbind(layout, extra_data[, !names(extra_data) %in% names(layout), drop = FALSE])
  layout$circular <- FALSE
  layout
}

# Generated by cpp11: do not edit by hand

cactusTree <- function(parent, order, weight, scale, overlap, upright) {
  .Call(`_ggraph_cactusTree`, parent, order, weight, scale, overlap, upright)
}

pack <- function(areas) {
  .Call(`_ggraph_pack`, areas)
}

circlePackLayout <- function(parent, weight) {
  .Call(`_ggraph_circlePackLayout`, parent, weight)
}

dendrogram_spread <- function(graph, starts, y, leaf, repel, pad, ratio) {
  .Call(`_ggraph_dendrogram_spread`, graph, starts, y, leaf, repel, pad, ratio)
}

force_bundle_iter <- function(edges_xy, K, C, P, P_rate, S, I, I_rate, compatibility_threshold, eps) {
  .Call(`_ggraph_force_bundle_iter`, edges_xy, K, C, P, P_rate, S, I, I_rate, compatibility_threshold, eps)
}

hTree <- function(parent, order) {
  .Call(`_ggraph_hTree`, parent, order)
}

partitionTree <- function(parent, order, weight, height) {
  .Call(`_ggraph_partitionTree`, parent, order, weight, height)
}

cut_lines <- function(x, y, id, start_width, start_height, end_width, end_height, start_type, end_type) {
  .Call(`_ggraph_cut_lines`, x, y, id, start_width, start_height, end_width, end_height, start_type, end_type)
}

pathAttr <- function(group, alpha, width, lty, colour, ngroups) {
  .Call(`_ggraph_pathAttr`, group, alpha, width, lty, colour, ngroups)
}

splitTreemap <- function(parent, order, weight, width, height) {
  .Call(`_ggraph_splitTreemap`, parent, order, weight, width, height)
}

unrooted <- function(parent, order, length, daylight, tol, rotation_mod, maxiter) {
  .Call(`_ggraph_unrooted`, parent, order, length, daylight, tol, rotation_mod, maxiter)
}

#' Draw edges in hive plots
#'
#' This geom is only intended for use together with the hive layout. It draws
#' edges between nodes as bezier curves, with the control points positioned at
#' the same radii as the start or end point, and at a distance defined by the
#' curvature argument.
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_hive` and `geom_edge_hive0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_hive2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **group**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_hive` and `geom_edge_hive2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#'
#' @param strength The curvature of the bezier. Defines the distance from the
#' control points to the midpoint between the start and end node. 1 means the
#' control points are positioned halfway between the nodes and the middle of the
#' two axes, while 0 means it coincide with the nodes (resulting in straight
#' lines)
#'
#' @param curvature Deprecated. Use `strength` instead.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' # Plot the flare import graph as a hive plot
#' library(tidygraph)
#' flareGr <- as_tbl_graph(flare$imports) %>%
#'   mutate(
#'     type = dplyr::case_when(
#'       centrality_degree(mode = 'in') == 0 ~ 'Source',
#'       centrality_degree(mode = 'out') == 0 ~ 'Sink',
#'       TRUE ~ 'Both'
#'     )
#'   ) %>%
#'   activate(edges) %>%
#'   mutate(
#'     type = dplyr::case_when(
#'       grepl('flare.analytics', paste(.N()$name[from], .N()$name[to])) ~ 'Analytics',
#'       TRUE ~ 'Other'
#'     )
#'   )
#'
#' ggraph(flareGr, 'hive', axis = type) +
#'   geom_edge_hive(aes(colour = type), edge_alpha = 0.1) +
#'   coord_fixed()
#' @rdname geom_edge_hive
#' @name geom_edge_hive
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier
#' @export
StatEdgeHive <- ggproto(
  'StatEdgeHive',
  StatBezier,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data <- remove_loop(data)
    if (nrow(data) == 0) {
      return(data)
    }
    data$group <- make_unique(data$group)
    data2 <- data
    data2$x <- data2$xend
    data2$y <- data2$yend
    keep <- atan2(data$y, data$x) != atan2(data2$y, data2$x)
    create_hive_bezier(data[keep, ], data2[keep, ], params)
  },
  required_aes = c('x', 'y', 'xend', 'yend'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n', 'strength', 'curvature')
)
#' @rdname geom_edge_hive
#'
#' @export
geom_edge_hive <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  n = 100,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...,
  curvature
) {
  if (!missing(curvature)) {
    lifecycle::deprecate_warn(
      '2.0.0',
      'geom_edge_hive(curvature)',
      'geom_edge_hive(strength)'
    )
    strength <- curvature * 2
  }
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      group = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeHive,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = FALSE,
        strength = strength,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier2
#' @export
StatEdgeHive2 <- ggproto(
  'StatEdgeHive2',
  StatBezier2,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data <- remove_loop2(data)
    if (nrow(data) == 0) {
      return(data)
    }
    data <- data[order(data$group), ]
    data2 <- data[c(FALSE, TRUE), ]
    data <- data[c(TRUE, FALSE), ]
    keep <- atan2(data$y, data$x) != atan2(data2$y, data2$x)
    create_hive_bezier(data[keep, ], data2[keep, ], params)
  },
  required_aes = c('x', 'y', 'group'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n', 'strength', 'curvature')
)
#' @rdname geom_edge_hive
#'
#' @export
geom_edge_hive2 <- function(
  mapping = NULL,
  data = get_edges('long'),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  n = 100,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...,
  curvature
) {
  if (!missing(curvature)) {
    lifecycle::deprecate_warn(
      '2.0.0',
      'geom_edge_hive2(curvature)',
      'geom_edge_hive2(strength)'
    )
    strength <- curvature * 2
  }
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y, group = edge.id))
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeHive2,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = TRUE,
        strength = strength,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier0
#' @export
StatEdgeHive0 <- ggproto(
  'StatEdgeHive0',
  StatBezier0,
  setup_data = function(data, params) {
    StatEdgeHive$setup_data(data, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'strength', 'curvature')
)
#' @rdname geom_edge_hive
#'
#' @export
geom_edge_hive0 <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  lineend = 'butt',
  show.legend = NA,
  ...,
  curvature
) {
  if (!missing(curvature)) {
    lifecycle::deprecate_warn(
      '2.0.0',
      'geom_edge_hive0(curvature)',
      'geom_edge_hive0(strength)'
    )
    strength <- curvature * 2
  }
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y, xend = xend, yend = yend))
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeHive0,
    geom = GeomEdgeBezier,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        strength = strength,
        ...
      )
    )
  )
}
create_hive_bezier <- function(from, to, params) {
  bezier_start <- seq(1, by = 4, length.out = nrow(from))
  from$index <- bezier_start
  to$index <- bezier_start + 3
  data2 <- from
  data3 <- to
  data2$index <- bezier_start + 1
  data3$index <- bezier_start + 2
  from_axis <- atan2(from$y, from$x)
  from_axis[from_axis < 0] <- from_axis[from_axis < 0] + 2 * pi
  to_axis <- atan2(to$y, to$x)
  to_axis[to_axis < 0] <- to_axis[to_axis < 0] + 2 * pi
  from_first <- ifelse(
    from_axis < to_axis,
    to_axis - from_axis < pi,
    to_axis - from_axis < -pi
  )
  middle_axis1 <- ifelse(from_first, from_axis, to_axis)
  middle_axis2 <- ifelse(from_first, to_axis, from_axis)
  middle_axis2 <- ifelse(
    middle_axis2 < middle_axis1,
    middle_axis2 + 2 * pi,
    middle_axis2
  )
  mean_axis <- (middle_axis2 - middle_axis1) / 2
  middle_axis1 <- middle_axis1 + mean_axis * params$strength / 2
  middle_axis2 <- middle_axis2 - mean_axis * params$strength / 2
  node_r2 <- sqrt(data2$x^2 + data2$y^2)
  node_r3 <- sqrt(data3$x^2 + data3$y^2)
  data2$x <- node_r2 * cos(ifelse(from_first, middle_axis1, middle_axis2))
  data2$y <- node_r2 * sin(ifelse(from_first, middle_axis1, middle_axis2))
  data3$x <- node_r3 * cos(ifelse(from_first, middle_axis2, middle_axis1))
  data3$y <- node_r3 * sin(ifelse(from_first, middle_axis2, middle_axis1))
  data <- vec_rbind(from, data2, data3, to)
  data[order(data$index), names(data) != 'index']
}

#' A theme tuned for graph visualizations
#'
#' When plotting graphs, networks, and trees the coordinate values are often of
#' no importance and axes are thus a distraction. `ggraph` comes with a
#' build-in theme that removes redundant elements in order to put focus on the
#' data. Furthermore the default behaviour is to use a narrow font so text takes
#' up less space. Theme colour is defined by a background and foreground colour
#' where the background defines the colour of the whole graphics area and the
#' foreground defines the colour of the strip and border. By default strip and
#' border is turned off as it is an unnecessary element unless facetting is
#' used. To add a foreground colour to a plot that is already using
#' `theme_graph` the `th_foreground` helper is provided. In order to
#' use this appearance as default use the `set_graph_style` function. An
#' added benefit of this is that it also changes the default text-related values
#' in the different geoms for a completely coherent look.
#' `unset_graph_style` can be used to revert the defaults back to their
#' default settings (that is, they are not necessarily reverted back to what
#' they were prior to calling `set_graph_style`). The `th_no_axes()` helper is
#' provided to modify an existing theme so that grid and axes are removed.
#'
#' @param background The colour to use for the background. This theme sets all
#' background elements except for plot.background to `element_blank` so
#' this controls the background for all elements of the plot. Set to `NA`
#' to remove the background (thus making the plot transparent)
#'
#' @param foreground The colour of foreground elements, specifically strip and
#' border. Set to `NA` to remove.
#'
#' @param border Logical. Should border be drawn if a foreground colour is
#' provided?
#'
#' @param family,base_family,title_family,subtitle_family,strip_text_family,caption_family The font to use for the different elements
#'
#' @param base_size,size,text_size,title_size,subtitle_size,strip_text_size,caption_size The size to use for the various text elements. `text_size` will be used as geom defaults
#'
#' @param face,title_face,subtitle_face,strip_text_face,caption_face The fontface to use for the various text elements
#'
#' @param title_margin,subtitle_margin,caption_margin The margin to use between the text elements and the plot area
#'
#' @param text_colour,bg_text_colour,fg_text_colour,title_colour,subtitle_colour,strip_text_colour,caption_colour The colour of the text in the various text elements
#'
#' @param plot_margin The plot margin
#'
#' @export
#'
#' @examples
#' library(tidygraph)
#' graph <- as_tbl_graph(highschool)
#'
#' ggraph(graph) + geom_edge_link() + geom_node_point() + theme_graph()
theme_graph <- function(base_family = 'Arial Narrow', base_size = 11,
                        background = 'white', foreground = NULL, border = TRUE,
                        text_colour = 'black', bg_text_colour = text_colour,
                        fg_text_colour = text_colour,
                        title_family = base_family, title_size = 18,
                        title_face = 'bold', title_margin = 10,
                        title_colour = bg_text_colour,
                        subtitle_family = base_family, subtitle_size = 12,
                        subtitle_face = 'plain', subtitle_margin = 15,
                        subtitle_colour = bg_text_colour,
                        strip_text_family = base_family, strip_text_size = 10,
                        strip_text_face = 'bold', strip_text_colour = fg_text_colour,
                        caption_family = base_family, caption_size = 9,
                        caption_face = 'italic', caption_margin = 10,
                        caption_colour = bg_text_colour,
                        plot_margin = margin(30, 30, 30, 30)) {
  style <- theme_bw(base_size = base_size, base_family = base_family)
  style <- style + theme(
    text = element_text(colour = text_colour),
    plot.title = element_text(
      family = title_family,
      size = title_size,
      face = title_face,
      colour = title_colour,
      margin = margin(b = title_margin)
    ),
    plot.subtitle = element_text(
      family = subtitle_family,
      size = subtitle_size,
      face = subtitle_face,
      colour = subtitle_colour,
      margin = margin(b = subtitle_margin)
    ),
    plot.caption = element_text(
      family = caption_family,
      size = caption_size,
      face = caption_face,
      colour = caption_colour,
      margin = margin(t = caption_margin)
    ),
    strip.text = element_text(
      family = strip_text_family,
      size = strip_text_size,
      face = strip_text_face,
      colour = strip_text_colour
    ),

    plot.margin = plot_margin,

    legend.background = element_blank(),
    legend.box.background = element_blank(),
    legend.key = element_blank(),
    panel.background = element_blank(),
    axis.title = element_blank(),
    axis.text = element_blank(),
    axis.line = element_blank(),
    axis.ticks = element_blank(),
    panel.grid = element_blank(),

    strip.background = if (is.null(foreground)) element_blank() else element_rect(fill = foreground, colour = foreground),
    plot.background = if (is.null(background)) element_blank() else element_rect(fill = background, colour = NA),
    panel.border = if (border && !is.null(foreground)) element_rect(fill = NA, colour = foreground) else element_blank()
  )

  style
}
#' @rdname theme_graph
#'
#' @export
th_foreground <- function(foreground = 'grey80', fg_text_colour = NULL, border = FALSE) {
  th <- theme(
    strip.background = if (is.null(foreground)) element_blank() else element_rect(fill = foreground, colour = foreground),
    panel.border = if (border && !is.null(foreground)) element_rect(fill = NA, colour = foreground) else element_blank()
  )
  if (!is.null(fg_text_colour)) {
    th <- th + theme(strip.text = element_text(colour = fg_text_colour))
  }
  th
}
#' @rdname theme_graph
#'
#' @export
th_no_axes <- function() {
  theme(
    panel.grid = element_blank(),
    #panel.grid.major = element_blank(),
    #panel.grid.major.x = element_blank(),
    #panel.grid.major.y = element_blank(),
    #panel.grid.minor = element_blank(),
    #panel.grid.minor.x = element_blank(),
    #panel.grid.minor.y = element_blank(),
    axis.title = element_blank(),
    #axis.title.x = element_blank(),
    #axis.title.x.bottom = element_blank(),
    #axis.title.x.top = element_blank(),
    #axis.title.y = element_blank(),
    #axis.title.y.left = element_blank(),
    #axis.title.y.right = element_blank(),
    axis.text = element_blank(),
    #axis.text.x = element_blank(),
    #axis.text.x.top = element_blank(),
    #axis.text.x.bottom = element_blank(),
    #axis.text.y = element_blank(),
    #axis.text.y.left = element_blank(),
    #axis.text.y.right = element_blank(),
    axis.ticks = element_blank(),
    #axis.ticks.x = element_blank(),
    #axis.ticks.x.top = element_blank(),
    #axis.ticks.x.bottom = element_blank(),
    #axis.ticks.y = element_blank(),
    #axis.ticks.y.left = element_blank(),
    #axis.ticks.y.right = element_blank(),
    axis.line = element_blank(),
    #axis.line.x = element_blank(),
    #axis.line.x.bottom = element_blank(),
    #axis.line.x.top = element_blank(),
    #axis.line.y = element_blank(),
    #axis.line.y.left = element_blank(),
    #axis.line.y.right = element_blank(),
    NULL
  )
}
#' @rdname theme_graph
#'
#' @param ... Parameters passed on the `theme_graph`
#'
#' @export
set_graph_style <- function(family = 'Arial Narrow', face = 'plain', size = 11,
                            text_size = 11, text_colour = 'black', ...) {
  style <- theme_graph(
    base_family = family, base_size = size,
    text_colour = text_colour, ...
  )
  theme_set(style)
  text_size <- text_size / .pt

  update_geom_defaults(GeomEdgePath, list(
    family = family,
    fontface = face,
    label_size = text_size
  ))
  update_geom_defaults(GeomText, list(
    family = family,
    fontface = face,
    size = text_size
  ))
  update_geom_defaults(GeomTextRepel, list(
    family = family,
    fontface = face,
    size = text_size
  ))
  update_geom_defaults(GeomLabel, list(
    family = family,
    fontface = face,
    size = text_size
  ))
  update_geom_defaults(GeomLabelRepel, list(
    family = family,
    fontface = face,
    size = text_size
  ))
}
#' @rdname theme_graph
#'
#' @export
unset_graph_style <- function() {
  style <- theme_gray()
  theme_set(style)

  update_geom_defaults(GeomEdgePath, list(
    family = '',
    fontface = 1,
    label_size = 3.88
  ))
  update_geom_defaults(GeomText, list(
    family = '',
    fontface = 1,
    size = 3.88
  ))
  update_geom_defaults(GeomTextRepel, list(
    family = '',
    fontface = 1,
    size = 3.88
  ))
  update_geom_defaults(GeomLabel, list(
    family = '',
    fontface = 1,
    size = 3.88
  ))
  update_geom_defaults(GeomLabelRepel, list(
    family = '',
    fontface = 1,
    size = 3.88
  ))
  update_geom_defaults(GeomAxisHive, list(
    family = '',
    fontface = 1,
    size = 3.88
  ))
}

#' Place nodes on a diagonal
#'
#' This layout puts all nodes on a diagonal, thus preparing the layout for use
#' with [geom_edge_point()] resulting in a matrix layout. While matrix
#' layouts excel in scalability, the interpretation of the visual is very
#' dependent on the sorting of the nodes. Different sorting algorithms have been
#' implemented in `tidygraph` and these can be used directly. Behrisch
#' *et al.* (2016) have provided a nice overview of some of the different
#' sorting algorithms and what insight they might bring, along with a rundown of
#' different patterns to look out for.
#'
#' @param graph An `tbl_graph` object
#'
#' @param circular Ignored
#'
#' @param sort.by An expression providing the sorting of the nodes. If `NULL`
#' the nodes will be ordered by their index in the graph.
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom igraph gorder
#' @importFrom rlang enquo eval_tidy
#'
#' @references
#' Behrisch, M., Bach, B., Riche, N. H., Schreck, T., Fekete, J.-D. (2016).
#' *Matrix Reordering Methods for Table and Network Visualization*.
#' Computer Graphics Forum, 35: 693–716. <https://doi.org/10.1111/cgf.12935>
#'
layout_tbl_graph_matrix <- function(graph, circular = FALSE, sort.by = NULL) {
  sort.by <- enquo(sort.by)
  sort.by <- eval_tidy(sort.by, .N())
  if (!is.null(sort.by)) {
    pos <- order(order(sort.by))
  } else {
    pos <- seq_len(gorder(graph))
  }
  nodes <- new_data_frame(list(x = pos, y = pos))
  extra_data <- as_tibble(graph, active = 'nodes')
  warn_dropped_vars(nodes, extra_data)
  nodes <- cbind(nodes, extra_data[, !names(extra_data) %in% names(nodes), drop = FALSE])
  nodes$circular <- FALSE
  nodes
}

#' Pack circles together
#'
#' This function is a direct interface to the circle packing algorithm used by
#' \code{\link{layout_tbl_graph_circlepack}}. It takes a vector of sizes and
#' returns the x and y position of each circle as a two-column matrix.
#'
#' @param areas A vector of circle areas
#'
#' @return A matrix with two columns and the same number of rows as the length
#' of the "areas" vector. The matrix has the following attributes added:
#' "enclosing_radius" giving the radius of the smallest enclosing circle, and
#' "front_chain" giving the terminating members of the front chain (see
#' Wang \emph{et al}. 2006).
#'
#' @references
#' Wang, W., Wang, H. H., Dai, G., & Wang, H. (2006). \emph{Visualization of
#' large hierarchical data by circle packing}. Chi, 517-520.
#'
#' @export
#'
#' @examples
#' library(ggforce)
#' sizes <- sample(10, 100, TRUE)
#'
#' position <- pack_circles(sizes)
#' data <- data.frame(x = position[,1], y = position[,2], r = sqrt(sizes/pi))
#'
#' ggplot() +
#'   geom_circle(aes(x0 = x, y0 = y, r = r), data = data, fill = 'steelblue') +
#'   geom_circle(aes(x0 = 0, y0 = 0, r = attr(position, 'enclosing_radius'))) +
#'   geom_polygon(aes(x = x, y = y),
#'                data = data[attr(position, 'front_chain'), ],
#'                fill = NA,
#'                colour = 'black')
#'
pack_circles <- function(areas) {
  pack(as.numeric(areas))
}

#' Draw edges as elbows
#'
#' This geom draws edges as an angle in the same manner as known from classic
#' dendrogram plots of hierarchical clustering results. In case a circular
#' transformation has been applied the first line segment will be drawn as an
#' arc as expected. This geom is only applicable to layouts that return a
#' direction for the edges (currently [layout_tbl_graph_dendrogram()],
#' [layout_tbl_graph_partition()] and
#' [layout_tbl_graph_igraph()] with the `"tree"` algorithm).
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_elbow` and `geom_edge_elbow0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - **circular**
#' - **direction**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_elbow2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - **group**
#' - **circular**
#' - **direction**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_elbow` and `geom_edge_elbow2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#' @param strength How bend the elbow should be. 1 will give a right angle,
#' while `0` will give a straight line. Ignored for circular layouts
#' @inheritParams geom_edge_diagonal
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' irisDen <- hclust(dist(iris[1:4], method = 'euclidean'), method = 'ward.D2') %>%
#'   as_tbl_graph() %>%
#'   mutate(class = sample(letters[1:3], n(), TRUE)) %>%
#'   activate(edges) %>%
#'   mutate(class = sample(letters[1:3], n(), TRUE))
#'
#' ggraph(irisDen, 'dendrogram', circular = TRUE) +
#'   geom_edge_elbow(aes(alpha = stat(index)))
#'
#' ggraph(irisDen, 'dendrogram') +
#'   geom_edge_elbow2(aes(colour = node.class))
#'
#' ggraph(irisDen, 'dendrogram', height = height) +
#'   geom_edge_elbow0(aes(colour = class))
#' @rdname geom_edge_elbow
#' @name geom_edge_elbow
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce radial_trans
#' @export
StatEdgeElbow <- ggproto('StatEdgeElbow', Stat,
  compute_panel = function(data, scales, flipped = FALSE, n = 100, strength = 1) {
    data$group <- make_unique(data$group)
    if (data$circular[1] && n %% 2 == 1) {
      n <- n + 1
    }
    if (!data$circular[1] && n %% 2 == 0) {
      n <- n + 1
    }
    index <- seq(0, 1, length.out = n)
    if (any(data$circular)) {
      if (strength != 1) warning('strength is ignored for circular elbow edges', call. = FALSE)
      circ_id <- which(data$circular)
      data_circ <- data[circ_id, ]
      radial <- radial_trans(c(0, 1), c(2 * pi, 0), pad = 0, offset = 0)
      start <- atan2(data_circ$y, data_circ$x)
      radii_start <- sqrt(data_circ$x^2 + data_circ$y^2)
      radii_end <- sqrt(data_circ$xend^2 + data_circ$yend^2)
      angel_diff <- (data_circ$x * data_circ$xend + data_circ$y * data_circ$yend) /
        (radii_start * radii_end)
      angel_diff[is.nan(angel_diff)] <- 0
      angel_diff <- suppressWarnings(acos(angel_diff))
      angel_diff[is.nan(angel_diff)] <- 0
      end <- start + ifelse(data_circ$direction == 'left',
        -angel_diff, angel_diff
      )
      angles <- unlist(Map(seq, from = start, to = end, length.out = n / 2))
      radii <- rep(sqrt(data$y[circ_id]^2 + data$x[circ_id]^2), each = n / 2)
      path_circ <- radial$transform(r = radii, a = angles)
      path_circ$.orig_index <- rep(circ_id, each = n / 2)
      path_circ$group <- rep(data_circ$group, each = n / 2)
      path_circ$index <- rep(index[seq_len(n / 2)], length(circ_id))
      radii_rel <- radii_start / radii_end
      elbow_x <- data_circ$xend * radii_rel
      elbow_y <- data_circ$yend * radii_rel
      elbow_x <- unlist(Map(seq,
        from = elbow_x, to = data_circ$xend,
        length.out = n / 2
      ))
      elbow_y <- unlist(Map(seq,
        from = elbow_y, to = data_circ$yend,
        length.out = n / 2
      ))
      path_circ <- rbind_dfs(list(
        path_circ,
        new_data_frame(list(
          x = elbow_x,
          y = elbow_y,
          .orig_index = path_circ$.orig_index,
          group = path_circ$group,
          index = rep(
            index[seq_len(n / 2) + n / 2],
            length(circ_id)
          )
        ))
      ))
      path_circ <- cbind(path_circ, data[path_circ$.orig_index, !names(data) %in%
        c('x', 'y', 'xend', 'yend', 'group')])
      path_circ$.orig_index <- NULL
    }
    if (any(!data$circular)) {
      path_lin <- lapply(which(!data$circular), function(i) {
        if (flipped) {
          path <- new_data_frame(list(
            x = approx(c(data$x[i], data$xend[i] + (data$x[i] - data$xend[i]) * strength, data$xend[i]),
              n = n
            )$y,
            y = approx(c(data$y[i], data$yend[i], data$yend[i]),
              n = n
            )$y,
            group = data$group[i],
            index = index
          ))
        } else {
          path <- new_data_frame(list(
            x = approx(c(data$x[i], data$xend[i], data$xend[i]),
              n = n
            )$y,
            y = approx(c(data$y[i], data$yend[i] + (data$y[i] - data$yend[i]) * strength, data$yend[i]),
              n = n
            )$y,
            group = data$group[i],
            index = index
          ))
        }
        cbind(path, data[rep(i, nrow(path)), !names(data) %in%
          c('x', 'y', 'xend', 'yend', 'group')])
      })
      path_lin <- rbind_dfs(path_lin)

      if (any(data$circular)) {
        paths <- rbind_dfs(list(path_lin, path_circ))
      } else {
        paths <- path_lin
      }
    } else {
      paths <- path_circ
    }
    paths[order(paths$group), ]
  },
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    data <- remove_loop(data)
    if (nrow(data) == 0) return(NULL)
    data
  },
  default_aes = aes(filter = TRUE),
  required_aes = c('x', 'y', 'xend', 'yend', 'circular', 'direction')
)
#' @rdname geom_edge_elbow
#'
#' @export
geom_edge_elbow <- function(mapping = NULL, data = get_edges(),
                            position = 'identity', arrow = NULL, strength = 1,
                            flipped = FALSE, n = 100, lineend = 'butt',
                            linejoin = 'round', linemitre = 1,
                            label_colour = 'black', label_alpha = 1,
                            label_parse = FALSE, check_overlap = FALSE,
                            angle_calc = 'rot', force_flip = TRUE,
                            label_dodge = NULL, label_push = NULL,
                            show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(
    x = x, y = y, xend = xend, yend = yend,
    circular = circular, direction = direction, group = edge.id
  ))
  layer(
    data = data, mapping = mapping, stat = StatEdgeElbow,
    geom = GeomEdgePath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, linejoin = linejoin,
        linemitre = linemitre, na.rm = FALSE, n = n,
        interpolate = FALSE, flipped = flipped, strength = strength,
        label_colour = label_colour, label_alpha = label_alpha,
        label_parse = label_parse, check_overlap = check_overlap,
        angle_calc = angle_calc, force_flip = force_flip,
        label_dodge = label_dodge, label_push = label_push, ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
StatEdgeElbow2 <- ggproto('StatEdgeElbow2', Stat,
  compute_panel = function(data, scales, flipped = FALSE, n = 100, strength = 1) {
    pos_cols <- c('x', 'y', 'group', 'circular', 'direction', 'PANEL')
    data <- data[order(data$group), ]
    pos_data <- cbind(data[c(TRUE, FALSE), pos_cols], data[
      c(FALSE, TRUE),
      c('x', 'y')
    ])
    pos_data$group <- make_unique(pos_data$group)
    names(pos_data) <- c(pos_cols, 'xend', 'yend')
    new_data <- StatEdgeElbow$compute_panel(pos_data, scales, flipped, n, strength)
    extra_cols <- !names(data) %in% pos_cols
    index <- match(pos_data$group, new_data$group)
    index <- as.vector(matrix(c(index, index + 1), nrow = 2, byrow = T))
    new_data$.interp <- TRUE
    new_data$.interp[index] <- FALSE
    if (sum(extra_cols) != 0) {
      for (i in names(data)[extra_cols]) {
        new_data[[i]] <- NA
        new_data[[i]][index] <- data[[i]]
      }
    }
    new_data
  },
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    data <- remove_loop2(data)
    if (nrow(data) == 0) return(NULL)
    data
  },
  default_aes = aes(filter = TRUE),
  required_aes = c('x', 'y', 'group', 'circular', 'direction')
)
#' @rdname geom_edge_elbow
#'
#' @export
geom_edge_elbow2 <- function(mapping = NULL, data = get_edges('long'),
                             position = 'identity', arrow = NULL, strength = 1,
                             flipped = FALSE, n = 100, lineend = 'butt',
                             linejoin = 'round', linemitre = 1,
                             label_colour = 'black', label_alpha = 1,
                             label_parse = FALSE, check_overlap = FALSE,
                             angle_calc = 'rot', force_flip = TRUE,
                             label_dodge = NULL, label_push = NULL,
                             show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(
    x = x, y = y, group = edge.id,
    circular = circular, direction = direction
  ))
  layer(
    data = data, mapping = mapping, stat = StatEdgeElbow2,
    geom = GeomEdgePath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, linejoin = linejoin,
        linemitre = linemitre, na.rm = FALSE, n = n,
        interpolate = TRUE, flipped = flipped, strength = strength,
        label_colour = label_colour, label_alpha = label_alpha,
        label_parse = label_parse, check_overlap = check_overlap,
        angle_calc = angle_calc, force_flip = force_flip,
        label_dodge = label_dodge, label_push = label_push, ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
StatEdgeElbow0 <- ggproto('StatEdgeElbow0', Stat,
  compute_panel = function(data, scales, flipped = FALSE, strength = 1) {
    data$group <- make_unique(data$group)
    if (any(data$circular)) {
      if (strength != 1) warning('strength is ignored for circular elbow edges', call. = FALSE)
      circ_id <- which(data$circular)
      data_circ <- data[circ_id, ]
      radial <- radial_trans(c(0, 1), c(2 * pi, 0), pad = 0, offset = 0)
      start <- atan2(data_circ$y, data_circ$x)
      angel_diff <- (data_circ$x * data_circ$xend + data_circ$y * data_circ$yend) /
        (sqrt(data_circ$x^2 + data_circ$y^2) *
          sqrt(data_circ$xend^2 + data_circ$yend^2))
      angel_diff[is.nan(angel_diff)] <- 0
      angel_diff <- suppressWarnings(acos(angel_diff))
      angel_diff[is.nan(angel_diff)] <- 0
      end <- start + ifelse(data_circ$direction == 'left',
        -angel_diff, angel_diff
      )
      angles <- unlist(Map(seq, from = start, to = end, length.out = 50))
      radii <- rep(sqrt(data$y[circ_id]^2 + data$x[circ_id]^2), each = 50)
      path_circ <- radial$transform(r = radii, a = angles)
      path_circ$.orig_index <- rep(circ_id, each = 50)
      path_circ$group <- rep(data_circ$group, each = 50)
      path_circ <- rbind_dfs(list(
        path_circ,
        new_data_frame(list(
          x = data$xend[circ_id],
          y = data$yend[circ_id],
          .orig_index = circ_id,
          group = data_circ$group
        ))
      ))
      path_circ <- cbind(path_circ, data[path_circ$.orig_index, !names(data) %in%
        c('x', 'y', 'xend', 'yend')])
      path_circ$.orig_index <- NULL
    }
    if (any(!data$circular)) {
      path_lin <- lapply(which(!data$circular), function(i) {
        if (flipped) {
          path <- new_data_frame(list(
            x = c(data$x[i], data$xend[i] + (data$x[i] - data$xend[i]) * strength, data$xend[i]),
            y = c(data$y[i], data$yend[i], data$yend[i]),
            group = data$group
          ))
        } else {
          path <- new_data_frame(list(
            x = c(data$x[i], data$xend[i], data$xend[i]),
            y = c(data$y[i], data$yend[i] + (data$y[i] - data$yend[i]) * strength, data$yend[i]),
            group = data$group[i]
          ))
        }
        cbind(path, data[rep(i, nrow(path)), !names(data) %in%
          c('x', 'y', 'xend', 'yend')])
      })
      path_lin <- rbind_dfs(path_lin)

      if (any(data$circular)) {
        paths <- rbind_dfs(list(path_lin, path_circ))
      } else {
        paths <- path_lin
      }
    } else {
      paths <- path_circ
    }
    paths[order(paths$group), ]
  },
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    if (nrow(data) == 0) return(NULL)
    data
  },
  default_aes = aes(filter = TRUE),
  required_aes = c('x', 'y', 'xend', 'yend', 'circular', 'direction')
)
#' @rdname geom_edge_elbow
#'
#' @export
geom_edge_elbow0 <- function(mapping = NULL, data = get_edges(),
                             position = 'identity', arrow = NULL, flipped = FALSE,
                             lineend = 'butt', show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(
    x = x, y = y, xend = xend, yend = yend,
    circular = circular, direction = direction
  ))
  layer(
    data = data, mapping = mapping, stat = StatEdgeElbow0,
    geom = GeomEdgePath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, na.rm = FALSE,
        interpolate = FALSE, flipped = flipped, ...
      )
    )
  )
}

#' Draw edges as diagonals
#'
#' This geom draws edges as cubic bezier curves with the control points
#' positioned along the elbow edge. It has the appearance of a softened elbow
#' edge with the hard angle substituted by a tapered bend.
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_bend` and `geom_edge_bend0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - **circular**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_bend2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - **group**
#' - **circular**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_bend` and `geom_edge_bend2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_diagonal
#'
#' @param strength The strength of the curvature of the bend. `0` will
#' result in a straight line while `1` will give a strong arc.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' gr <- create_tree(20, 4) %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE)) %>%
#'   activate(edges) %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE))
#'
#' ggraph(gr, 'tree') +
#'   geom_edge_bend(aes(alpha = stat(index)))
#'
#' ggraph(gr, 'tree') +
#'   geom_edge_bend2(aes(colour = node.class))
#'
#' ggraph(gr, 'tree') +
#'   geom_edge_bend0(aes(colour = class))
#' @rdname geom_edge_bend
#' @name geom_edge_bend
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier
#' @export
StatEdgeBend <- ggproto('StatEdgeBend', StatBezier,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    data <- remove_loop(data)
    if (nrow(data) == 0) return(NULL)
    data$group <- make_unique(data$group)
    data2 <- data
    data2$x <- data2$xend
    data2$y <- data2$yend
    create_bend(data, data2, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'circular'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'flipped', 'n', 'strength')
)
#' @rdname geom_edge_bend
#'
#' @export
geom_edge_bend <- function(mapping = NULL, data = get_edges(),
                           position = 'identity', arrow = NULL, strength = 1,
                           flipped = FALSE, n = 100, lineend = 'butt',
                           linejoin = 'round', linemitre = 1,
                           label_colour = 'black', label_alpha = 1,
                           label_parse = FALSE, check_overlap = FALSE,
                           angle_calc = 'rot', force_flip = TRUE,
                           label_dodge = NULL, label_push = NULL,
                           show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(
    x = x, y = y, xend = xend, yend = yend,
    circular = circular, group = edge.id
  ))
  layer(
    data = data, mapping = mapping, stat = StatEdgeBend,
    geom = GeomEdgePath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, linejoin = linejoin,
        linemitre = linemitre, na.rm = FALSE, n = n,
        interpolate = FALSE, flipped = flipped, strength = strength,
        label_colour = label_colour, label_alpha = label_alpha,
        label_parse = label_parse, check_overlap = check_overlap,
        angle_calc = angle_calc, force_flip = force_flip,
        label_dodge = label_dodge, label_push = label_push, ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier2
#' @export
StatEdgeBend2 <- ggproto('StatEdgeBend2', StatBezier2,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    data <- remove_loop2(data)
    if (nrow(data) == 0) return(NULL)
    data <- data[order(data$group), ]
    data2 <- data[c(FALSE, TRUE), ]
    data <- data[c(TRUE, FALSE), ]
    create_bend(data, data2, params)
  },
  required_aes = c('x', 'y', 'group', 'circular'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'flipped', 'n', 'strength')
)
#' @rdname geom_edge_bend
#'
#' @export
geom_edge_bend2 <- function(mapping = NULL, data = get_edges('long'),
                            position = 'identity', arrow = NULL, strength = 1,
                            flipped = FALSE, n = 100, lineend = 'butt',
                            linejoin = 'round', linemitre = 1,
                            label_colour = 'black', label_alpha = 1,
                            label_parse = FALSE, check_overlap = FALSE,
                            angle_calc = 'rot', force_flip = TRUE,
                            label_dodge = NULL, label_push = NULL,
                            show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(
    x = x, y = y, group = edge.id,
    circular = circular
  ))
  layer(
    data = data, mapping = mapping, stat = StatEdgeBend2,
    geom = GeomEdgePath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, linejoin = linejoin,
        linemitre = linemitre, na.rm = FALSE, n = n,
        interpolate = TRUE, flipped = flipped, strength = strength,
        label_colour = label_colour, label_alpha = label_alpha,
        label_parse = label_parse, check_overlap = check_overlap,
        angle_calc = angle_calc, force_flip = force_flip,
        label_dodge = label_dodge, label_push = label_push, ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier0
#' @export
StatEdgeBend0 <- ggproto('StatEdgeBend0', StatBezier0,
  setup_data = function(data, params) {
    StatEdgeBend$setup_data(data, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'circular'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'flipped', 'strength')
)
#' @rdname geom_edge_bend
#'
#' @export
geom_edge_bend0 <- function(mapping = NULL, data = get_edges(),
                            position = 'identity', arrow = NULL, strength = 1,
                            flipped = FALSE, lineend = 'butt',
                            show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(
    x = x, y = y, xend = xend, yend = yend,
    circular = circular
  ))
  layer(
    data = data, mapping = mapping, stat = StatEdgeBend0,
    geom = GeomEdgeBezier, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, na.rm = FALSE, strength = strength,
        flipped = flipped, ...
      )
    )
  )
}

create_bend <- function(from, to, params) {
  bezier_start <- seq(1, by = 4, length.out = nrow(from))
  from$index <- bezier_start
  to$index <- bezier_start + 3
  data2 <- from
  data3 <- to
  data2$index <- bezier_start + 1
  data3$index <- bezier_start + 2
  if (any(from$circular)) {
    r0 <- sqrt(from$x[from$circular]^2 + from$y[from$circular]^2)
    r1 <- sqrt(to$x[to$circular]^2 + to$y[to$circular]^2)
    root <- r0 == 0 | r1 == 0

    dotprod <- from$x * to$x + from$y * to$y

    crossing <- r0 * r0 / dotprod
    cross_x <- to$x * crossing
    cross_y <- to$y * crossing

    data2$x[from$circular] <- from$x + (cross_x - from$x) * params$strength
    data2$y[from$circular] <- from$y + (cross_y - from$y) * params$strength
    data3$x[from$circular] <- to$x + (cross_x - to$x) * params$strength
    data3$y[from$circular] <- to$y + (cross_y - to$y) * params$strength

    data2$x[root] <- from$x[root]
    data2$y[root] <- from$y[root]
    data3$x[root] <- to$x[root]
    data3$y[root] <- to$y[root]
  }
  if (any(!from$circular)) {
    if (params$flipped) {
      h_diff <- from$x[!from$circular] - to$x[!from$circular]
      w_diff <- from$y[!from$circular] - to$y[!from$circular]
      data2$y[!from$circular] <- from$y[!from$circular] - w_diff * params$strength
      data3$x[!from$circular] <- to$x[!from$circular] + h_diff * params$strength
    } else {
      h_diff <- from$y[!from$circular] - to$y[!from$circular]
      w_diff <- from$x[!from$circular] - to$x[!from$circular]
      data2$x[!from$circular] <- from$x[!from$circular] - w_diff * params$strength
      data3$y[!from$circular] <- to$y[!from$circular] + h_diff * params$strength
    }
  }
  data <- rbind_dfs(list(from, data2, data3, to))
  data[order(data$index), names(data) != 'index']
}

#' Calculate nodes as areas dividing their parent
#'
#' The partition layout is a way to show hierarchical data in the same way as
#' [layout_tbl_graph_treemap()]. Instead of subdividing the parent area
#' the partition layout shows the division of a nodes children next to the area
#' of the node itself. As such the node positions will be very reminiscent of
#' a reingold-tilford tree layout but by plotting nodes as areas it better
#' communicate the total weight of a node by summing up all its children.
#' Often partition layouts are called icicle plots or sunburst diagrams (in case
#' a radial transform is applied).
#'
#' @note
#' partition is a layout intended for trees, that is, graphs where nodes
#' only have one parent and zero or more children. If the provided graph does
#' not fit this format an attempt to convert it to such a format will be made.
#'
#' @param graph An `tbl_graph` object
#'
#' @param weight An optional node variable to use as weight. Will only affect
#' the weight of leaf nodes as the weight of non-leaf nodes are derived from
#' their children.
#'
#' @param circular Logical. Should the layout be transformed to a circular
#' representation. If `TRUE` the resulting layout will be a sunburst
#' diagram.
#'
#' @param height An optional node variable to use as height. If `NULL`
#' all nodes will be given a height of 1.
#'
#' @param sort.by The name of a node variable to sort the nodes by.
#'
#' @param direction The direction of the tree in the graph. `'out'` (default)
#' means that parents point towards their children, while `'in'` means that
#' children point towards their parent.
#'
#' @param const.area Logical. Should 'height' be scaled for area proportionality
#' when using `circular = TRUE`. Defaults to `TRUE`.
#'
#' @param offset If `circular = TRUE`, where should it begin. Defaults to
#' `pi/2` which is equivalent to 12 o'clock.
#'
#' @return If `circular = FALSE` A data.frame with the columns `x`,
#' `y`, `width`, `height`, `leaf`,
#' `depth`, `circular` as well as any information stored as node
#' variables in the tbl_graph object.
#' If `circular = TRUE` A data.frame with the columns `x`, `y`,
#' `r0`, `r`, `start`, `end`, `leaf`,
#' `depth`, `circular` as well as any information stored as node
#' variables in the tbl_graph object.
#'
#' @references
#' Kruskal, J. B., Landwehr, J. M. (1983). *Icicle Plots: Better Displays
#' for Hierarchical Clustering*. American Statistician Vol 37(2), 162-168.
#' https://doi.org/10.2307/2685881
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom ggforce radial_trans
#'
layout_tbl_graph_partition <- function(graph, weight = NULL, circular = FALSE, height = NULL, sort.by = NULL, direction = 'out', offset = pi / 2, const.area = TRUE) {
  weight <- enquo(weight)
  weight <- eval_tidy(weight, .N())
  height <- enquo(height)
  height <- eval_tidy(height, .N())
  sort.by <- enquo(sort.by)
  sort.by <- eval_tidy(sort.by, .N())
  hierarchy <- tree_to_hierarchy(graph, direction, sort.by, weight, height)
  layout <- partitionTree(hierarchy$parent, hierarchy$order, hierarchy$weight, hierarchy$height)[-1, ]
  if (circular) {
    if (const.area) {
      y0 <- sqrt(layout[, 2])
      y1 <- sqrt(layout[, 2] + layout[, 4])
      layout[, 2] <- y0
      layout[, 4] <- y1 - y0
    }
    width_range <- c(0, max(rowSums(layout[, c(1, 3)])))
    radial <- radial_trans(
      r.range = c(0, 1),
      a.range = width_range,
      offset = offset,
      pad = 0
    )
    coords <- radial$transform(
      layout[, 2] + layout[, 4] / 2,
      layout[, 1] + layout[, 3] / 2
    )
    layout <- new_data_frame(list(
      x = coords$x,
      y = coords$y,
      r0 = layout[, 2],
      r = layout[, 2] + layout[, 4],
      start = 2 * pi * layout[, 1] / width_range[2],
      end = 2 * pi * (layout[, 1] + layout[, 3]) / width_range[2],
      circular = TRUE
    ))
    layout$x[1] <- 0
    layout$y[1] <- 0
  } else {
    layout <- new_data_frame(list(
      x = layout[, 1] + layout[, 3] / 2,
      y = layout[, 2] + layout[, 4] / 2,
      width = layout[, 3],
      height = layout[, 4],
      circular = FALSE
    ))
  }
  layout$leaf <- degree(graph, mode = direction) == 0
  layout$depth <- node_depth(graph, mode = direction)
  extra_data <- as_tibble(graph, active = 'nodes')
  warn_dropped_vars(layout, extra_data)
  layout <- cbind(layout, extra_data[, !names(extra_data) %in% names(layout), drop = FALSE])
  attr(layout, 'graph') <- add_direction(graph, layout)
  layout
}

#' Apply a dendrogram layout to layout_tbl_graph
#'
#' This layout mimics the [igraph::layout_as_tree()] algorithm
#' supplied by igraph, but puts all leaves at 0 and builds it up from there,
#' instead of starting from the root and building it from there. The height of
#' branch points are related to the maximum distance to an edge from the branch
#' node, or read from a node variable.
#'
#' @note This function is not intended to be used directly but by setting
#' `layout = 'dendrogram'` in [create_layout()]
#'
#' @param graph A `tbl_graph` object
#'
#' @param circular Logical. Should the layout be transformed to a circular
#' representation. Defaults to `FALSE`.
#'
#' @param offset If `circular = TRUE`, where should it begin. Defaults to
#' `pi/2` which is equivalent to 12 o'clock.
#'
#' @param height The node variable holding the height of each node in the
#' dendrogram. If `NULL` it will be calculated as the maximal distance to a
#' leaf.
#'
#' @param length An edge parameter giving the length of each edge. The node
#' height will be calculated from the maximal length to the root node (ignored
#' if `height` does not evaluate to `NULL`)
#'
#' @param repel Should leafs repel each other relative to the height of their
#' common ancestor. Will emphasize clusters
#'
#' @param ratio The strength of repulsion if `repel = TRUE`. Higher values will
#' give more defined clusters
#'
#' @param direction The direction to the leaves. Defaults to 'out'
#'
#' @return A data.frame with the columns `x`, `y`, `circular`, `depth` and
#' `leaf` as well as any information stored as node variables on the
#' tbl_graph
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom igraph gorder degree neighbors distances
#' @importFrom tidygraph node_is_root
#' @importFrom rlang enquo eval_tidy
#'
layout_tbl_graph_dendrogram <- function(graph, circular = FALSE, offset = pi / 2, height = NULL, length = NULL, repel = FALSE, ratio = 1, direction = 'out') {
  height <- enquo(height)
  length <- enquo(length)
  reverse_dir <- if (direction == 'out') 'in' else 'out'
  if (quo_is_null(height)) {
    if (quo_is_null(length)) {
      height <- NA
    } else {
      length <- eval_tidy(length, .E())
      full_lengths <- distances(graph, to = node_is_root(), weights = length, mode = reverse_dir)
      full_lengths[is.infinite(full_lengths)] <- 0
      height <- unname(apply(full_lengths, 1, max))
      height <- abs(height - max(height))
    }
  } else {
    height <- eval_tidy(height, .N())
  }
  nodes <- new_data_frame(list(
    x = rep(NA_real_, gorder(graph)),
    y = height,
    leaf = degree(graph, mode = direction) == 0
  ))
  if (all(is.na(nodes$y))) {
    nodes$y <- vapply(seq_len(gorder(graph)), function(i) {
      max(bfs(graph, i, direction, unreachable = FALSE, order = FALSE, dist = TRUE)$dist, na.rm = TRUE)
    }, numeric(1))
  }
  if (repel) {
    pad <- min(nodes$y[nodes$y != 0]) / 2
  } else {
    pad <- 0
  }
  startnode <- which(degree(graph, mode = reverse_dir) == 0)
  if (length(startnode) < 1) stop('No root nodes in graph')
  recurse_layout <- function(gr, node, layout, direction, offset = 0) {
    children <- as.numeric(neighbors(gr, node, direction))
    if (length(children) == 0) {
      layout$x[node] <- offset
      layout
    } else {
      children_missing <- children[is.na(layout$x[children])]
      for (i in children_missing) {
        layout <- recurse_layout(gr, i, layout, direction, offset)
        offset <- if (repel) {
          max(layout$x[layout$leaf], na.rm = TRUE) + (layout$y[node] + pad) * ratio
        } else {
          max(layout$x[layout$leaf], na.rm = TRUE) + 1 + pad
        }
      }
      layout$x[node] <- mean(range(layout$x[children]))
      layout
    }
  }
  offset <- 0
  for (i in startnode) {
    nodes <- recurse_layout(graph, i, nodes, direction = direction, offset)
    offset <- max(nodes$x[nodes$leaf], na.rm = TRUE) + 1
  }
  graph <- add_direction(graph, nodes)
  if (circular) {
    radial <- radial_trans(
      r.range = rev(range(nodes$y)),
      a.range = range(nodes$x),
      offset = offset
    )
    coords <- radial$transform(nodes$y, nodes$x)
    nodes$x <- coords$x
    nodes$y <- coords$y
  }
  extra_data <- as_tibble(graph, active = 'nodes')
  warn_dropped_vars(nodes, extra_data)
  nodes <- cbind(nodes, extra_data[, !names(extra_data) %in% names(nodes), drop = FALSE])
  nodes$circular <- circular
  attr(nodes, 'graph') <- graph
  nodes
}

textAlongGrob <- function(label, x = unit(0.5, 'npc'), y = unit(0.5, 'npc'),
                          just = 'centre', hjust = NULL, vjust = NULL, rot = 0,
                          check.overlap = FALSE, rot.type = 'rot', x0 = 0,
                          y0 = 0, x1 = 0, y1 = 0, force.rot = TRUE, dodge = NULL,
                          push = NULL,
                          default.units = 'npc', name = NULL, gp = gpar(),
                          vp = NULL) {
  if (!is.unit(x)) {
    x <- unit(x, default.units)
  }
  if (!is.unit(y)) {
    y <- unit(y, default.units)
  }
  if (rot.type == 'rot') {
    textGrob(
      label = label, x = x, y = y, just = just, hjust = hjust,
      vjust = vjust, rot = rot, check.overlap = check.overlap,
      default.units = default.units, name = name, gp = gp, vp = vp
    )
  } else {
    if (!rot.type %in% c('along', 'across')) {
      stop('rot.type must be either `rot`, `along`, or `across`',
        call. = FALSE
      )
    }
    if (!is.unit(x0)) {
      x0 <- unit(x0, default.units)
    }
    if (!is.unit(y0)) {
      y0 <- unit(y0, default.units)
    }
    if (!is.unit(x1)) {
      x1 <- unit(x1, default.units)
    }
    if (!is.unit(y1)) {
      y1 <- unit(y1, default.units)
    }
    grob(
      label = label, x = x, y = y, just = just, hjust = hjust,
      vjust = vjust, rot.type = rot.type, x0 = x0, y0 = y0, x1 = x1,
      y1 = y1, force.rot = force.rot, dodge = dodge, push = push,
      check.overlap = check.overlap, name = name, gp = gp, vp = vp,
      cl = 'textalong'
    )
  }
}
#' Text angled according to line
#'
#' This function takes care of recalculating the angle of the text as the device
#' size changes
#'
#' @importFrom grid makeContent
#' @export
#' @keywords internal
makeContent.textalong <- function(x) {
  x0 <- convertX(x$x0, 'mm', TRUE)
  y0 <- convertY(x$y0, 'mm', TRUE)
  x1 <- convertX(x$x1, 'mm', TRUE)
  y1 <- convertY(x$y1, 'mm', TRUE)
  xpos <- convertX(x$x, 'mm', TRUE)
  ypos <- convertY(x$y, 'mm', TRUE)
  angle <- edge_angle(x0, y0, x1, y1)
  if (x$rot.type == 'across') {
    angle <- angle - 90
  }
  if (!is.null(x$dodge)) {
    dodge <- convertHeight(x$dodge, 'mm', TRUE)
    dodge_angle <- (angle + 90) / 360 * 2 * pi
    dodge_x <- cos(dodge_angle) * dodge
    dodge_y <- sin(dodge_angle) * dodge
    xpos <- xpos + dodge_x
    ypos <- ypos + dodge_y
  }
  if (!is.null(x$push)) {
    push <- convertHeight(x$push, 'mm', TRUE)
    push_angle <- angle / 360 * 2 * pi
    push_x <- cos(push_angle) * push
    push_y <- sin(push_angle) * push
    xpos <- xpos + push_x
    ypos <- ypos + push_y
  }
  if (x$force.rot) {
    fix <- angle > 90 & angle < 270
    angle[fix] <- angle[fix] + 180
  }
  grob(
    label = x$label, x = unit(xpos, 'mm'), y = unit(ypos, 'mm'),
    just = x$just, hjust = x$hjust, vjust = x$vjust, rot = angle,
    check.overlap = x$check.overlap, name = x$name, gp = x$gp, vp = x$vp,
    cl = 'text'
  )
}

#' @aliases ggraph-package
'_PACKAGE'

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
#' @useDynLib ggraph
#' @import ggplot2 tidygraph
#' @importFrom Rcpp sourceCpp
#' @importFrom rlang .data
## usethis namespace: end
NULL

#' Draw multi edges as parallel lines
#'
#' This geom draws multi edges as parallel lines. The edges are first sorted by
#' direction and then shifted a fixed amount so that all edges are visible.
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_parallel` and `geom_edge_parallel0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - **from**
#' - **to**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_parallel2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - **group**
#' - **from**
#' - **to**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_parallel` and `geom_edge_parallel2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#'
#' @param sep The separation between parallel edges, given as a [grid::unit()]
#'
#' @author David Schoch and Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('bull') %>%
#'   convert(to_directed) %>%
#'   bind_edges(data.frame(from = c(1, 2, 2, 3), to = c(2, 1, 3, 2))) %E>%
#'   mutate(class = sample(letters[1:3], 9, TRUE)) %N>%
#'   mutate(class = sample(c('x', 'y'), 5, TRUE))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_parallel(aes(alpha = stat(index)))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_parallel2(aes(colour = node.class))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_parallel0(aes(colour = class))
#'
#' # Use capping and sep to fine tune the look
#' ggraph(gr, 'stress') +
#'   geom_edge_parallel(start_cap = circle(1), end_cap = circle(1),
#'                      arrow = arrow(length = unit(2, 'mm')), sep = unit(4, 'mm')) +
#'   geom_node_point(size = 12)
#'
#' @rdname geom_edge_parallel
#' @name geom_edge_parallel
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatLink
#' @export
StatEdgeParallel <- ggproto('StatEdgeParallel', StatLink,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    data <- remove_loop(data)
    if (nrow(data) == 0) return(NULL)
    data2 <- data
    data2$x <- data2$xend
    data2$y <- data2$yend
    data$.position <- edge_positions(data, data2, params)
    StatLink$setup_data(data, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'from', 'to'),
  default_aes = aes(filter = TRUE),
  extra_params = c("na.rm", "n")
)

#' @rdname geom_edge_parallel
#'
#' @importFrom ggforce StatLink
#' @export
geom_edge_parallel <- function(mapping = NULL, data = get_edges(),
                               position = "identity", arrow = NULL,
                               sep = unit(2, 'mm'), n = 100, lineend = "butt",
                               linejoin = "round", linemitre = 1,
                               label_colour = 'black',  label_alpha = 1,
                               label_parse = FALSE, check_overlap = FALSE,
                               angle_calc = 'rot', force_flip = TRUE,
                               label_dodge = NULL, label_push = NULL,
                               show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y, xend = xend, yend = yend,
                                        from = from, to = to, group = edge.id))
  layer(data = data, mapping = mapping, stat = StatEdgeParallel,
        geom = GeomEdgeParallelPath, position = position, show.legend = show.legend,
        inherit.aes = FALSE,
        params = expand_edge_aes(
          list(arrow = arrow, lineend = lineend, linejoin = linejoin,
               linemitre = linemitre, na.rm = FALSE, sep = sep, n = n,
               interpolate = FALSE,
               label_colour = label_colour, label_alpha = label_alpha,
               label_parse = label_parse, check_overlap = check_overlap,
               angle_calc = angle_calc, force_flip = force_flip,
               label_dodge = label_dodge, label_push = label_push, ...)
        )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier2
#' @export
StatEdgeParallel2 <- ggproto('StatEdgeParallel2', StatLink2,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    data <- remove_loop2(data)
    if (nrow(data) == 0) return(NULL)
    data <- data[order(data$group), ]
    data2 <- data[c(FALSE, TRUE), ]
    data1 <- data[c(TRUE, FALSE), ]
    data$.position <- rep(edge_positions(data1, data2), each = 2)
    StatLink2$setup_data(data, params)
  },
  required_aes = c('x', 'y', 'group', 'from', 'to'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n')
)
#' @rdname geom_edge_parallel
#'
#' @export
geom_edge_parallel2 <- function(mapping = NULL, data = get_edges('long'),
                                position = 'identity', arrow = NULL,
                                sep = unit(2, 'mm'), n = 100, lineend = 'butt',
                                linejoin = 'round', linemitre = 1,
                                label_colour = 'black', label_alpha = 1,
                                label_parse = FALSE, check_overlap = FALSE,
                                angle_calc = 'rot', force_flip = TRUE,
                                label_dodge = NULL, label_push = NULL,
                                show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(
    x = x, y = y, group = edge.id,
    from = from, to = to
  ))
  layer(
    data = data, mapping = mapping, stat = StatEdgeParallel2,
    geom = GeomEdgeParallelPath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(arrow = arrow, lineend = lineend, linejoin = linejoin,
           linemitre = linemitre, na.rm = FALSE, sep = sep, n = n,
           interpolate = TRUE,
           label_colour = label_colour, label_alpha = label_alpha,
           label_parse = label_parse, check_overlap = check_overlap,
           angle_calc = angle_calc, force_flip = force_flip,
           label_dodge = label_dodge, label_push = label_push, ...)
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier0
#' @export
StatEdgeParallel0 <- ggproto('StatEdgeFan0', StatIdentity,
  setup_data = function(data, params) {
    StatEdgeParallel$setup_data(data, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'from', 'to'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm')
)
#' @rdname geom_edge_parallel
#'
#' @export
geom_edge_parallel0 <- function(mapping = NULL, data = get_edges(),
                           position = 'identity', arrow = NULL, sep = unit(2, 'mm'),
                           lineend = 'butt', show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(
    x = x, y = y, xend = xend, yend = yend,
    from = from, to = to
  ))
  layer(
    data = data, mapping = mapping, stat = StatEdgeParallel0,
    geom = GeomEdgeParallelSegment, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, na.rm = FALSE,
        sep = sep, ...
      )
    )
  )
}
#' @importFrom dplyr %>% group_by arrange summarise n ungroup pull
edge_positions <- function(from, to, params) {
  from$.id <- paste(pmin(from$from, to$to), pmax(from$from, to$to), sep = '-')
  from$.orig_ind <- seq_len(nrow(from))
  from %>%
    group_by(.data$PANEL, .data$.id) %>%
    arrange(.data$from) %>%
    mutate(position = seq_len(n()) - 0.5 - n() / 2) %>%
    mutate(position = .data$position * ifelse(.data$from < .data$to, 1, -1)) %>%
    ungroup() %>%
    arrange(.data$.orig_ind) %>%
    pull(.data$position)
}

#' Draw edges as vertical spans
#'
#' This edge geom is mainly intended for use with [fabric][layout_tbl_graph_fabric]
#' layouts. It draws edges as vertical segments with an optional end shape
#' adornment. Due to the special nature of fabric layouts where nodes are not
#' a single point in space but a line, this geom doesn't derive the x position
#' from the location of the terminal nodes, but defaults to using the `edge_x`
#' variable calculated by the fabric layout. If this geom is used with other
#' layouts `x`and `xend` must be given explicitly.
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_span` and `geom_edge_span0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_span2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - **group**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_span` and `geom_edge_span2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#'
#' @param end_shape The adornment to put at the ends of the span. The naming
#' follows the conventions of the shape aesthetic in [ggplot2::geom_point()]
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' gr <- play_smallworld(n_dim = 3, dim_size = 3, order = 1, p_rewire = 0.6)
#'
#' # Standard use
#' ggraph(gr, 'fabric', sort.by = node_rank_fabric()) +
#'   geom_node_range(colour = 'grey80') +
#'   geom_edge_span()
#'
#' # Add end shapes
#' ggraph(gr, 'fabric', sort.by = node_rank_fabric()) +
#'   geom_node_range(colour = 'grey80') +
#'   geom_edge_span(end_shape = 'circle')
#'
#' # If the layout include shadow edges these can be styled differently
#' ggraph(gr, 'fabric', sort.by = node_rank_fabric(), shadow.edges = TRUE) +
#'   geom_node_range(colour = 'grey80') +
#'   geom_edge_span(aes(colour = shadow_edge), end_shape = 'square') +
#'   scale_edge_colour_manual(values = c('FALSE' = 'black', 'TRUE' = 'grey'))
#'
#' @rdname geom_edge_span
#' @name geom_edge_span
#'
NULL

#' @rdname geom_edge_span
#'
#' @importFrom ggforce StatLink
#' @export
geom_edge_span <- function(mapping = NULL, data = get_edges('short'),
                           position = 'identity', end_shape = NA, arrow = NULL, n = 100,
                           lineend = 'butt', linejoin = 'round', linemitre = 1,
                           label_colour = 'black', label_alpha = 1,
                           label_parse = FALSE, check_overlap = FALSE,
                           angle_calc = 'rot', force_flip = TRUE,
                           label_dodge = NULL, label_push = NULL,
                           show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = edge_x, y = y, xend = edge_x,
                                        yend = yend, group = edge.id))
  layer(
    data = data, mapping = mapping, stat = StatEdgeLink,
    geom = GeomEdgeSpanPath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, linejoin = linejoin,
        linemitre = linemitre, na.rm = FALSE, n = n,
        interpolate = FALSE, end_shape = end_shape,
        label_colour = label_colour, label_alpha = label_alpha,
        label_parse = label_parse, check_overlap = check_overlap,
        angle_calc = angle_calc, force_flip = force_flip,
        label_dodge = label_dodge, label_push = label_push, ...
      )
    )
  )
}
#' @rdname geom_edge_span
#'
#' @importFrom ggforce StatLink2
#' @export
geom_edge_span2 <- function(mapping = NULL, data = get_edges('long'),
                            position = 'identity', end_shape = NA, arrow = NULL, n = 100,
                            lineend = 'butt', linejoin = 'round', linemitre = 1,
                            label_colour = 'black', label_alpha = 1,
                            label_parse = FALSE, check_overlap = FALSE,
                            angle_calc = 'rot', force_flip = TRUE,
                            label_dodge = NULL, label_push = NULL,
                            show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = edge_x, y = y,
                                        group = edge.id))
  layer(
    data = data, mapping = mapping, stat = StatEdgeLink2,
    geom = GeomEdgeSpanPath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, linejoin = linejoin,
        linemitre = linemitre, na.rm = FALSE, n = n,
        interpolate = TRUE, end_shape = end_shape,
        label_colour = label_colour, label_alpha = label_alpha,
        label_parse = label_parse, check_overlap = check_overlap,
        angle_calc = angle_calc, force_flip = force_flip,
        label_dodge = label_dodge, label_push = label_push, ...
      )
    )
  )
}
#' @rdname geom_edge_span
#'
#' @importFrom ggforce StatLink2
#' @export
geom_edge_span0 <- function(mapping = NULL, data = get_edges(),
                            position = 'identity', end_shape = NA, arrow = NULL,
                            lineend = 'butt', show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = edge_x, y = y,
                                        xend = edge_x, yend = yend))
  layer(
    data = data, mapping = mapping, stat = StatFilter,
    geom = GeomEdgeSpanSegment, position = position,
    show.legend = show.legend, inherit.aes = FALSE,
    params = expand_edge_aes(
      list(end_shape = end_shape, arrow = arrow, lineend = lineend, na.rm = FALSE, ...)
    )
  )
}

#' Edge colour scales
#'
#' This set of scales defines new colour scales for edge geoms equivalent to the
#' ones already defined by ggplot2. The parameters are equivalent to the ones
#' from ggplot2 so there is nothing new under the sun. The different geoms will
#' know whether to use edge scales or the standard scales so it is not necessary
#' to write `edge_colour` in the call to the geom - just use `colour`.
#'
#' @return A ggproto object inheriting from `Scale`
#'
#' @family scale_edge_*
#'
#' @name scale_edge_colour
#' @rdname scale_edge_colour
#'
NULL

#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_hue
#'
#' @importFrom scales hue_pal
#' @export
scale_edge_colour_hue <- function(..., h = c(0, 360) + 15, c = 100, l = 65, h.start = 0, direction = 1, na.value = 'grey50') {
  discrete_scale('edge_colour', 'hue', hue_pal(h, c, l, h.start, direction),
    na.value = na.value, ...
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_brewer
#'
#' @importFrom scales brewer_pal
#' @export
scale_edge_colour_brewer <- function(..., type = 'seq', palette = 1, direction = 1) {
  discrete_scale('edge_colour', 'brewer', brewer_pal(type, palette, direction), ...)
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_distiller
#'
#' @importFrom scales gradient_n_pal brewer_pal
#' @export
scale_edge_colour_distiller <- function(..., type = 'seq', palette = 1, direction = -1, values = NULL, space = 'Lab', na.value = 'grey50', guide = 'edge_colourbar') {
  # warn about using a qualitative brewer palette to generate the gradient
  type <- match.arg(type, c('seq', 'div', 'qual'))
  if (type == 'qual') {
    warning('Using a discrete colour palette in a continuous scale.\n  Consider using type = "seq" or type = "div" instead', call. = FALSE)
  }
  continuous_scale('edge_colour', 'distiller',
    gradient_n_pal(brewer_pal(type, palette, direction)(6), values, space),
    na.value = na.value, guide = guide, ...
  )
  # NB: 6 colours per palette gives nice gradients; more results in more saturated colours which do not look as good
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_gradient
#' @param low,high Colours for low and high ends of the gradient.
#'
#' @importFrom scales seq_gradient_pal
#' @export
scale_edge_colour_gradient <- function(..., low = '#132B43', high = '#56B1F7', space = 'Lab', na.value = 'grey50', guide = 'edge_colourbar') {
  continuous_scale('edge_colour', 'gradient', seq_gradient_pal(low, high, space),
    na.value = na.value, guide = guide, ...
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_gradient2
#'
#' @importFrom scales div_gradient_pal muted
#' @export
scale_edge_colour_gradient2 <- function(..., low = muted('red'), mid = 'white', high = muted('blue'), midpoint = 0, space = 'Lab', na.value = 'grey50', guide = 'edge_colourbar') {
  continuous_scale('edge_colour', 'gradient2',
    div_gradient_pal(low, mid, high, space),
    na.value = na.value, guide = guide, ...,
    rescaler = mid_rescaler(mid = midpoint)
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_gradientn
#' @param colours,colors Vector of colours to use for n-colour gradient.
#'
#' @importFrom scales gradient_n_pal
#' @export
scale_edge_colour_gradientn <- function(..., colours, values = NULL, space = 'Lab', na.value = 'grey50', guide = 'edge_colourbar', colors) {
  colours <- if (missing(colours)) colors else colours

  continuous_scale('edge_colour', 'gradientn',
    gradient_n_pal(colours, values, space),
    na.value = na.value, guide = guide, ...
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_grey
#'
#' @importFrom scales grey_pal
#' @export
scale_edge_colour_grey <- function(..., start = 0.2, end = 0.8, na.value = 'red') {
  discrete_scale('edge_colour', 'grey', grey_pal(start, end),
    na.value = na.value, ...
  )
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_identity
#'
#' @importFrom scales identity_pal
#' @export
scale_edge_colour_identity <- function(..., guide = 'none') {
  sc <- discrete_scale('edge_colour', 'identity', identity_pal(), ...,
    guide = guide, super = ScaleDiscreteIdentity
  )
  sc
}
#' @rdname scale_edge_colour
#'
#' @inheritParams ggplot2::scale_colour_manual
#'
#' @export
scale_edge_colour_manual <- function(..., values) {
  manual_scale('edge_colour', values, ...)
}
#' @rdname scale_edge_colour
#'
#' @inheritParams viridis::scale_colour_viridis
#'
#' @importFrom viridis viridis viridis_pal
#' @export
scale_edge_colour_viridis <- function(..., alpha = 1, begin = 0, end = 1,
                                      discrete = FALSE, option = 'D',
                                      direction = 1) {
  if (direction == -1) {
    tmp <- begin
    begin <- end
    end <- tmp
  }
  if (discrete) {
    discrete_scale(
      'edge_colour', 'viridis',
      viridis_pal(
        alpha = alpha, begin = begin, end = end,
        option = option
      ), ...
    )
  } else {
    scale_edge_colour_gradientn(colours = viridis(256,
      alpha = alpha,
      begin = begin, end = end,
      option = option
    ), ...)
  }
}
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_colour_continuous <- scale_edge_colour_gradient
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_colour_discrete <- scale_edge_colour_hue

#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_hue <- scale_edge_colour_hue
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_brewer <- scale_edge_colour_brewer
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_distiller <- scale_edge_colour_distiller
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_gradient <- scale_edge_colour_gradient
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_gradient2 <- scale_edge_colour_gradient2
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_gradientn <- scale_edge_colour_gradientn
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_grey <- scale_edge_colour_grey
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_identity <- scale_edge_colour_identity
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_manual <- scale_edge_colour_manual
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_continuous <- scale_edge_colour_continuous
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_discrete <- scale_edge_colour_discrete
#' @rdname scale_edge_colour
#'
#' @export
scale_edge_color_viridis <- scale_edge_colour_viridis

#' ggraph extensions to ggplot2
#'
#' This help page lists all exported ggproto classes defined by ggraph. In
#' general these should be of no concern to the user as the main interface to
#' the functionality is, as with ggplot2, the `geom_*` format. As opposed
#' to ggplot2 there really aren't any use for separate `stat_*` functions
#' as they are intimately linked to each geom and mixing and matching stats and
#' geoms would only cause a lot of trouble.
#'
#' @details
#' Many of the `geom_edge_*` geoms comes in different flavors dependent on
#' the functionality required. There will always be a base geom and some will
#' have a `geom_edge_*0` and `geom_edge_*2` version. The base geom
#' will, in the case of multiple versions, draw the edge as a sequence of small
#' segments. The different aesthetics will be repeated for each segment and a
#' counter will be added the enumerates the progression of segments, so that a
#' gradient of colour or size can be added along the edge by assigning the
#' respective aesthetic to `stat(index)`. `geom_edge_*2` will also draw
#' the edge as segments but will interpolate between the aesthetics at the end
#' points. This makes it possible for an edge to interpolate node properties of
#' its end nodes. `geom_edge_*2` is less performant than the base geom so
#' use only when interpolation is needed. `geom_edge_*0` is a
#' high-performance version that usually maps directly to a grid grob. It does
#' not allow for drawing gradients or interpolations though, so use only for
#' simple edge drawings.
#'
#' @name ggraph-extensions
#' @rdname ggraph-extensions
#' @keywords internal
#'
NULL

#' Show nodes as points
#'
#' This geom is equivalent in functionality to [ggplot2::geom_point()]
#' and allows for simple plotting of nodes in different shapes, colours and sizes.
#'
#' @section Aesthetics:
#' `geom_node_point` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - alpha
#' - colour
#' - fill
#' - shape
#' - size
#' - stroke
#' - filter
#'
#' @inheritParams ggplot2::geom_point
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x and y are mapped to x and y in
#' the node data.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_node_*
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('bull') %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE))
#'
#' ggraph(gr, 'stress') + geom_node_point()
#' @export
#'
geom_node_point <- function(mapping = NULL, data = NULL, position = 'identity',
                            show.legend = NA, ...) {
  mapping <- aes_intersect(mapping, aes(x = x, y = y))
  layer(
    data = data, mapping = mapping, stat = StatFilter, geom = GeomPoint,
    position = position, show.legend = show.legend, inherit.aes = FALSE,
    params = list(na.rm = FALSE, ...)
  )
}

#' Show nodes as circles
#'
#' This geom is equivalent in functionality to [ggforce::geom_circle()]
#' and allows for plotting of nodes as circles with a radius scaled by the
#' coordinate system. Because of the geoms reliance on the coordinate system
#' it will only produce true circles when combined with
#' [ggplot2::coord_fixed()]
#'
#' @section Aesthetics:
#' `geom_node_circle` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overridden.
#'
#' - **x0**
#' - **y0**
#' - **r**
#' - alpha
#' - colour
#' - fill
#' - shape
#' - size
#' - stroke
#' - filter
#'
#' @inheritParams ggforce::geom_circle
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x and y are mapped to x0 and y0 in
#' the node data.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_node_*
#'
#' @examples
#' require(tidygraph)
#' gr <- tbl_graph(flare$vertices, flare$edges)
#' ggraph(gr, 'circlepack', weight = size) +
#'   geom_node_circle() +
#'   coord_fixed()
#' @export
#' @importFrom ggforce GeomCircle
#'
geom_node_circle <- function(mapping = NULL, data = NULL, position = 'identity',
                             show.legend = NA, ...) {
  mapping <- aes_intersect(mapping, aes(x0 = x, y0 = y, r = r))
  layer(
    data = data, mapping = mapping, stat = StatNodeCircle, geom = GeomCircle,
    position = position, show.legend = show.legend, inherit.aes = FALSE,
    params = list(na.rm = FALSE, ...)
  )
}

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatCircle
#' @export
StatNodeCircle <- ggproto('StatNodeCircle', StatCircle,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    if (nrow(data) == 0) return(NULL)
    data$group <- make_unique(data$group)
    data
  },
  default_aes = aes(filter = TRUE)
)

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom dplyr %>% group_by mutate slice ungroup
#' @export
StatAxisHive <- ggproto('StatAxisHive', StatFilter,
  setup_data = function(data, params) {
    data <- data %>%
      group_by(.data$angle, .data$section, .data$PANEL) %>%
      mutate(
        x = min(.data$r) * cos(.data$angle[1]) * 1.1,
        y = min(.data$r) * sin(.data$angle[1]) * 1.1,
        xend = max(.data$r) * cos(.data$angle[1]) * 1.1,
        yend = max(.data$r) * sin(.data$angle[1]) * 1.1,
        max_r = max(.data$r),
        min_r = min(.data$r)
      ) %>%
      slice(1) %>%
      ungroup()
    as.data.frame(data, stringsAsFactors = FALSE)
  },
  required_aes = c('r', 'angle', 'center_size', 'axis', 'section'),
  extra_params = c('na.rm', 'n', 'curvature')
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid textGrob nullGrob
#' @importFrom dplyr %>% group_by summarise
#' @export
GeomAxisHive <- ggproto('GeomAxisHive', GeomSegment,
  draw_panel = function(data, panel_scales, coord, label = TRUE, axis = TRUE, label_colour = 'black') {
    data$x <- data$x / 1.1
    data$y <- data$y / 1.1
    data$xend <- data$xend / 1.1
    data$yend <- data$yend / 1.1
    data <- coord$transform(data, panel_scales)
    label_data <- data %>%
      group_by(.data$axis) %>%
      summarise(
        x = max(.data$max_r) * cos(mean(.data$angle)),
        y = max(.data$max_r) * sin(mean(.data$angle)),
        label = .data$axis[1],
        angle = mean(.data$angle) / (2 * pi) * 360 - 90,
        colour = .data$colour[1],
        label_size = .data$label_size[1],
        family = .data$family[1],
        fontface = .data$fontface[1],
        lineheight = .data$lineheight[1]
      )
    label_data <- as.data.frame(label_data, stringsAsFactors = FALSE)
    lab_dist <- sqrt(label_data$x^2 + label_data$y^2)
    dist_dodge <- max(lab_dist) * 1.05 - max(lab_dist)
    label_data$x <- label_data$x * (dist_dodge + lab_dist) / lab_dist
    label_data$y <- label_data$y * (dist_dodge + lab_dist) / lab_dist
    label_data$angle <- label_data$angle + ifelse(label_data$angle < 0, 360, 0)
    label_data$angle <- label_data$angle - ifelse(label_data$angle > 360, 360, 0)
    upside_label <- label_data$angle > 90 & label_data$angle < 270
    label_data$angle[upside_label] <- label_data$angle[upside_label] + 180
    label_data <- coord$transform(label_data, panel_scales)
    label_data$label_colour <- if (is.na(label_colour)) {
      label_data$colour
    } else {
      label_colour
    }
    label_grob <- if (label) {
      textGrob(label_data$label, label_data$x, label_data$y,
        default.units = 'native', rot = label_data$angle,
        gp = gpar(
          col = label_data$label_colour,
          fontsize = label_data$label_size * .pt,
          fontfamily = label_data$family,
          fontface = label_data$fontface,
          lineheight = label_data$lineheight
        )
      )
    } else {
      nullGrob()
    }
    axis_grob <- if (axis) {
      segmentsGrob(data$x, data$y, data$xend, data$yend,
        default.units = 'native',
        gp = gpar(
          col = alpha(data$colour, data$alpha),
          fill = alpha(data$colour, data$alpha),
          lwd = data$size * .pt,
          lty = data$linetype,
          lineend = 'square'
        )
      )
    } else {
      nullGrob()
    }
    gList(axis_grob, label_grob)
  },
  default_aes = aes(
    colour = 'black', size = 0.5, linetype = 1, alpha = NA,
    label_size = 3.88, family = '', fontface = 1,
    lineheight = 1.2
  )
)

#' Draw rectangular bars and labels on hive axes
#'
#' This function lets you annotate the axes in a hive plot with labels and
#' color coded bars.
#'
#' @inheritParams ggplot2::geom_point
#'
#' @param label Should the axes be labelled. Defaults to `TRUE`
#'
#' @param axis Should a rectangle be drawn along the axis. Defaults to `TRUE`
#'
#' @section Aesthetics:
#' geom_axis_hive understand the following aesthetics.
#' \itemize{
#'  \item{alpha}
#'  \item{colour}
#'  \item{fill}
#'  \item{size}
#'  \item{linetype}
#'  \item{label_size}
#'  \item{family}
#'  \item{fontface}
#'  \item{lineheight}
#' }
#'
#' @author Thomas Lin Pedersen
#'
#' @export
#'
#' @examples
#' # Plot the flare import graph as a hive plot
#' library(tidygraph)
#' flareGr <- as_tbl_graph(flare$imports) %>%
#'   mutate(
#'     type = dplyr::case_when(
#'       centrality_degree(mode = 'in') == 0 ~ 'Source',
#'       centrality_degree(mode = 'out') == 0 ~ 'Sink',
#'       TRUE ~ 'Both'
#'     )
#'   ) %>%
#'   activate(edges) %>%
#'   mutate(
#'     type = dplyr::case_when(
#'       grepl('flare.analytics', paste(.N()$name[from], .N()$name[to])) ~ 'Analytics',
#'       TRUE ~ 'Other'
#'     )
#'   )
#' ggraph(flareGr, 'hive', axis = type) +
#'   geom_edge_hive(aes(colour = type), edge_alpha = 0.1) +
#'   geom_axis_hive(aes(colour = type)) +
#'   coord_fixed()
geom_axis_hive <- function(mapping = NULL, data = NULL,
                           position = 'identity', label = TRUE, axis = TRUE, show.legend = NA, ...) {
  mapping <- aes_intersect(mapping, aes(r = r, angle = angle,
                                        center_size = center_size,
                                        axis = axis, section = section))
  layer(
    data = data, mapping = mapping, stat = StatAxisHive,
    geom = GeomAxisHive, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = list(na.rm = FALSE, label = label, axis = axis, ...)
  )
}

#' Calculate nodes as circles packed within their parent circle
#'
#' The circle packing algorithm is basically a treemap using circles instead of
#' rectangles. Due to the nature of circles they cannot be packed as efficiently
#' leading to increased amount of "empty space" as compared to a treemap. This
#' can be beneficial though, as the added empty space can aid in visually
#' showing the hierarchy.
#'
#' @details
#' The circle packing is based on the algorithm developed by Weixin Wang and
#' collaborators which tries to find the most dense packing of circles as they
#' are added, one by one. This makes the algorithm very dependent on the order
#' in which circles are added and it is possible that layouts could sometimes
#' be optimized by choosing a different ordering. The algorithm for finding the
#' enclosing circle is the randomized incremental algorithm proposed by Emo
#' Welzl. Both of the above algorithms are the same as used in the D3.js
#' implementation of circle packing and their C++ implementation in ggraph is
#' inspired by Mike Bostocks JavaScript implementation.
#'
#' @note
#' Circle packing is a layout intended for trees, that is, graphs where nodes
#' only have one parent and zero or more children. If the provided graph does
#' not fit this format an attempt to convert it to such a format will be made.
#'
#' @param graph An `tbl_graph` object
#'
#' @param weight An optional node variable to use as weight. Will only affect
#' the weight of leaf nodes as the weight of non-leaf nodes are derived from
#' their children.
#'
#' @param circular Logical. Should the layout be transformed to a circular
#' representation. Ignored.
#'
#' @param sort.by The name of a node variable to sort the nodes by.
#'
#' @param direction The direction of the tree in the graph. `'out'` (default)
#' means that parents point towards their children, while `'in'` means that
#' children point towards their parent.
#'
#' @return A data.frame with the columns `x`, `y`, `r`, `leaf`,
#' `depth`, `circular` as well as any information stored as node
#' variables in the tbl_graph object.
#'
#' @references
#' Wang, W., Wang, H. H., Dai, G., & Wang, H. (2006). *Visualization of
#' large hierarchical data by circle packing*. Chi, 517-520.
#'
#' Welzl, E. (1991). *Smallest enclosing disks (balls and ellipsoids)*. New
#' Results and New Trends in Computer Science, 359-370.
#'
#' @family layout_tbl_graph_*
#'
layout_tbl_graph_circlepack <- function(graph, weight = NULL, circular = FALSE, sort.by = NULL, direction = 'out') {
  weight <- enquo(weight)
  weight <- eval_tidy(weight, .N())
  sort.by <- enquo(sort.by)
  sort.by <- eval_tidy(sort.by, .N())
  hierarchy <- tree_to_hierarchy(graph, direction, sort.by, weight)
  layout <- circlePackLayout(hierarchy$parent, hierarchy$weight)[-1, ]
  layout <- new_data_frame(list(
    x = layout[, 1],
    y = layout[, 2],
    r = layout[, 3],
    circular = FALSE,
    leaf = degree(graph, mode = direction) == 0,
    depth = node_depth(graph, mode = direction)
  ))
  extra_data <- as_tibble(graph, active = 'nodes')
  warn_dropped_vars(layout, extra_data)
  layout <- cbind(layout, extra_data[, !names(extra_data) %in% names(layout), drop = FALSE])
  layout
}

#' Colourbar legend for edges
#'
#' This function is equivalent to [ggplot2::guide_colourbar()] but
#' works for edge aesthetics.
#'
#' @inheritParams ggplot2::guide_colourbar
#'
#' @return A guide object
#'
#' @importFrom grid is.unit unit
#' @importFrom digest digest
#' @export
guide_edge_colourbar <- function(title = waiver(), title.position = NULL,
                                 title.theme = NULL, title.hjust = NULL,
                                 title.vjust = NULL, label = TRUE,
                                 label.position = NULL, label.theme = NULL,
                                 label.hjust = NULL, label.vjust = NULL,
                                 barwidth = NULL, barheight = NULL, nbin = 20,
                                 raster = TRUE, ticks = TRUE, draw.ulim = TRUE,
                                 draw.llim = TRUE, direction = NULL,
                                 default.unit = 'line', reverse = FALSE, order = 0,
                                 ...) {
  if (!is.null(barwidth) && !is.unit(barwidth)) {
    barwidth <- unit(barwidth, default.unit)
  }
  if (!is.null(barheight) && !is.unit(barheight)) {
    barheight <- unit(barheight, default.unit)
  }
  guide <- list(
    title = title, title.position = title.position,
    title.theme = title.theme, title.hjust = title.hjust,
    title.vjust = title.vjust, label = label,
    label.position = label.position, label.theme = label.theme,
    label.hjust = label.hjust, label.vjust = label.vjust,
    barwidth = barwidth, barheight = barheight, nbin = nbin,
    raster = raster, ticks = ticks, draw.ulim = draw.ulim,
    draw.llim = draw.llim, direction = direction,
    default.unit = default.unit, reverse = reverse, order = order,
    available_aes = c('edge_colour', 'edge_fill'), ...,
    name = 'edge_colourbar'
  )
  class(guide) <- c('guide', 'edge_colourbar', 'colorbar')
  guide
}
#' @rdname guide_edge_colourbar
#' @export
guide_edge_colorbar <- guide_edge_colourbar

#' @importFrom scales discard
#' @importFrom stats setNames
#' @rdname guide-helpers
#' @export
guide_train.edge_colourbar <- function(guide, scale, aesthetic = NULL) {
  if (length(intersect(scale$aesthetics, c(
    'edge_colour',
    'edge_fill'
  ))) == 0) {
    warning('edge_colourbar guide needs edge_colour or edge_fill scales.')
    return(NULL)
  }
  if (scale$is_discrete()) {
    warning('edge_colourbar guide needs continuous scales.')
    return(NULL)
  }
  breaks <- scale$get_breaks()
  if (length(breaks) == 0 || all(is.na(breaks))) {
    return()
  }
  ticks <- as.data.frame(setNames(
    list(scale$map(breaks)),
    aesthetic %||% scale$aesthetics[1]
  ), stringsAsFactors = FALSE)
  ticks$.value <- breaks
  ticks$.label <- scale$get_labels(breaks)
  guide$key <- ticks
  .limits <- scale$get_limits()
  .bar <- discard(pretty(.limits, n = guide$nbin), scale$get_limits())
  if (length(.bar) == 0) {
    .bar <- unique(.limits)
  }
  guide$bar <- new_data_frame(list(
    colour = scale$map(.bar), value = .bar,
    stringsAsFactors = FALSE
  ))
  if (guide$reverse) {
    guide$key <- guide$key[nrow(guide$key):1, ]
    guide$bar <- guide$bar[nrow(guide$bar):1, ]
  }
  guide$hash <- with(guide, digest::digest(list(
    title, key$.label,
    bar, name
  )))
  guide
}

#' Place nodes in circles based on distance to a specific node
#'
#' This layout constrains node placement to a radius relative to its distance to
#' a given node. It then uses stress majorisation to find an optimal node
#' distribution according to this constraint.
#'
#' @param focus An expression evaluating to a selected node. Can either be a
#' single integer or a logical vector with a single `TRUE` element.
#' @inheritParams layout_tbl_graph_stress
#'
#' @return A data.frame with the columns `x`, `y`, `circular`, `distance` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @references
#' Brandes, U., & Pich, C. (2011). *More flexible radial layout.* Journal of
#' Graph Algorithms and Applications, 15(1), 157-173.
#'
#' @family layout_tbl_graph_*
#'
#' @author The underlying algorithm is implemented in the graphlayouts package
#' by David Schoch
#'
#' @importFrom graphlayouts layout_with_focus
#' @importFrom rlang eval_tidy enquo
layout_tbl_graph_focus <- function(graph, focus, weights = NULL, niter = 500, tolerance = 1e-4,
                                   circular = TRUE) {
  focus <- eval_tidy(enquo(focus), .N())
  if (is.logical(focus)) focus <- which(focus)[1]

  weights <- eval_tidy(enquo(weights), .E())
  if (is.null(weights)) {
    weights <- NA
  }

  layout <- layout_with_focus(graph, v = focus, weights = weights, iter = niter,
                              tol = tolerance)
  xy <- layout$xy

  nodes <- new_data_frame(list(x = xy[,1],y = xy[,2], distance = layout$distance))
  nodes$circular <- FALSE
  extra_data <- as_tibble(graph, active = 'nodes')
  warn_dropped_vars(nodes, extra_data)
  nodes <- cbind(nodes, extra_data[, !names(extra_data) %in% names(nodes), drop = FALSE])
  nodes
}

#' Edge label size scales
#'
#' This set of scales defines new size scales for edge labels in order to allow
#' for separate sizing of edges and their labels.
#'
#' @return A ggproto object inheriting from `Scale`
#'
#' @family scale_edge_*
#'
#' @name scale_label_size
#' @rdname scale_label_size
#'
NULL

#' @rdname scale_label_size
#'
#' @inheritParams ggplot2::scale_size_continuous
#'
#' @export
scale_label_size_continuous <- function(name = waiver(), breaks = waiver(), labels = waiver(),
                                        limits = NULL, range = c(1, 6),
                                        trans = "identity", guide = "legend") {
  sc <- scale_size_continuous(name = name, breaks = breaks, labels = labels, limits = limits,
                              range = range, trans = trans, guide = guide)
  sc$aesthetics <- 'label_size'
  sc
}
#' @rdname scale_label_size
#'
#' @export
scale_label_size <- scale_label_size_continuous
#' @rdname scale_label_size
#'
#' @inheritParams ggplot2::scale_size_discrete
#'
#' @export
scale_label_size_discrete <- function(...) {
  cli::cli_warn("Using {.field label_size} for a discrete variable is not advised.")
  sc <- scale_size_ordinal(...)
  sc$aesthetics <- 'label_size'
  sc
}
#' @rdname scale_label_size
#'
#' @inheritParams ggplot2::scale_size_binned
#' @export
scale_label_size_binned <- function(name = waiver(), breaks = waiver(), labels = waiver(),
                                    limits = NULL, range = c(1, 6), n.breaks = NULL,
                                    nice.breaks = TRUE, trans = "identity", guide = "bins") {
  sc <- scale_size_binned(name = name, breaks = breaks, labels = labels, limits = limits,
                          range = range, n.breaks = n.breaks, nice.breaks = nice.breaks,
                          trans = trans, guide = guide)
  sc$aesthetics <- 'label_size'
  sc
}
#' @rdname scale_label_size
#'
#' @inheritParams ggplot2::scale_size_manual
#'
#' @export
scale_label_size_manual <- function(..., values, breaks = waiver(), na.value = NA) {
  sc <- scale_size_manual(..., values = values, breaks = breaks, na.value = na.value)
  sc$aesthetics <- 'label_size'
  sc
}
#' @rdname scale_label_size
#'
#' @inheritParams ggplot2::scale_size_identity
#'
#' @importFrom scales identity_pal
#' @export
scale_label_size_identity <- function(..., guide = 'none') {
  sc <- scale_size_identity(..., guide = guide)
  sc$aesthetics <- 'label_size'
  sc
}

#' Automatically pick a layout based on graph type
#'
#' This function infers the layout from the graph structure and is the default
#' when calling [ggraph()]. If an `x` and `y` argument is passed along, the
#' manual layout is chosen. Otherwise if the graph is either a rooted tree or a
#' rooted forest the layout will be `dendrogram` if the nodes contains a height
#' variable or `tree` if not. If the tree is unrooted the `unrooted` layout will
#' be used. If the tree is a DAG the `sygiyama` layout will be used. Otherwise
#' the `stress` layout will be used (or `sparse_tree` if the graph contains more
#' than 2000 nodes).
#'
#' @param graph A tbl_graph object
#'
#' @param circular Logical. Should the layout be transformed to a circular
#' representation. Defaults to `FALSE`. Only applicable if the graph is a tree
#' structure
#'
#' @param ... Arguments passed on to the chosen layout
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom rlang .data quos
#'
layout_tbl_graph_auto <- function(graph, circular, ...) {
  if (all(c('x', 'y') %in% names(quos(...)))) {
    layout_tbl_graph_manual(graph, circular = circular, ...)
  } else if (with_graph(graph, graph_is_tree() || graph_is_forest())) {
    if (is.null(with_graph(graph, .N()[['height']]))) {
      message('Using `tree` as default layout')
      layout_tbl_graph_igraph(graph, algorithm = 'tree', circular = circular, ...)
    } else {
      message('Using `dendrogram` as default layout')
      layout_tbl_graph_dendrogram(graph, circular = circular, height = .data$height, ...)
    }
  } else if (with_graph(graph, graph_is_dag())) {
    message('Using `sugiyama` as default layout')
    layout_tbl_graph_igraph(graph, algorithm = 'sugiyama', circular = circular, ...)
  } else if (with_graph(graph, graph_order()) < 2000) {
    message('Using `stress` as default layout')
    layout_tbl_graph_stress(graph, circular = circular, ...)
  } else {
    message('Using `sparse_stress` with 250 pivots as default layout')
    layout_tbl_graph_sparse_stress(graph, pivots = 250, circular = circular, ...)
  }
}

#' Edge direction guide
#'
#' This guide is intended to show the direction of edges based on the aesthetics
#' mapped to its progression, such as changing width, colour and opacity.
#'
#' @inheritParams ggplot2::guide_colourbar
#'
#' @param arrow Logical. Should an arrow be drawn to illustrate the direction.
#' Defaults to `TRUE`
#'
#' @param arrow.position The position of the arrow relative to the example edge.
#'
#' @param override.aes A list specifying aesthetic parameters of legend key.
#'
#' @importFrom grid is.unit unit
#' @importFrom digest digest
#' @export
#'
#' @examples
#' gr <- tidygraph::as_tbl_graph(highschool)
#' ggraph(gr, layout = 'kk') +
#'   geom_edge_fan(aes(alpha = stat(index))) +
#'   guides(edge_alpha = guide_edge_direction())
guide_edge_direction <- function(title = waiver(), title.position = NULL,
                                 title.theme = NULL, title.hjust = NULL,
                                 title.vjust = NULL, arrow = TRUE,
                                 arrow.position = NULL,
                                 barwidth = NULL, barheight = NULL, nbin = 500,
                                 direction = NULL, default.unit = 'line',
                                 reverse = FALSE, order = 0,
                                 override.aes = list(), ...) {
  if (!is.null(barwidth) && !is.unit(barwidth)) {
    barwidth <- unit(barwidth, default.unit)
  }
  if (!is.null(barheight) && !is.unit(barheight)) {
    barheight <- unit(barheight, default.unit)
  }
  guide <- list(
    title = title, title.position = title.position,
    title.theme = title.theme, title.hjust = title.hjust,
    title.vjust = title.vjust, arrow = arrow,
    arrow.position = arrow.position, barwidth = barwidth,
    barheight = barheight, nbin = nbin, direction = direction,
    default.unit = default.unit, reverse = reverse, order = order,
    available_aes = c('edge_colour', 'edge_alpha', 'edge_width'),
    override.aes = expand_edge_aes(rename_aes(override.aes)), ...,
    name = 'edge_direction'
  )
  class(guide) <- c('guide', 'edge_direction')
  guide

}
#' Helper methods for guides
#'
#' @importFrom scales discard
#' @export
#' @rdname guide-helpers
#' @keywords internal
guide_train.edge_direction <- function(guide, scale, aesthetic = NULL) {
  if (length(intersect(scale$aesthetics, c(
    'edge_colour',
    'edge_alpha',
    'edge_width'
  ))) == 0) {
    warning('edge_colourbar guide needs edge_colour or edge_alpha or edge_width scales.')
    return(NULL)
  }
  if (scale$is_discrete()) {
    warning('edge_colourbar guide needs continuous scales.')
    return(NULL)
  }
  breaks <- scale$get_breaks()
  if (length(breaks) == 0 || all(is.na(breaks))) {
    return()
  }
  ticks <- as.data.frame(setNames(
    list(scale$map(breaks)),
    aesthetic %||% scale$aesthetics[1]
  ), stringsAsFactors = FALSE)
  ticks$.value <- breaks
  ticks$.label <- scale$get_labels(breaks)
  guide$key <- ticks
  .limits <- scale$get_limits()
  .bar <- discard(pretty(.limits, n = guide$nbin), scale$get_limits())
  if (length(.bar) == 0) {
    .bar <- unique(.limits)
  }
  guide$bar <- as.data.frame(setNames(
    list(scale$map(.bar)),
    scale$aesthetics[1]
  ), stringsAsFactors = FALSE)
  guide$bar$.value <- .bar
  guide$bar <- guide$bar[order(.bar), ]
  if (guide$reverse) {
    guide$key <- guide$key[nrow(guide$key):1, ]
    guide$bar <- guide$bar[nrow(guide$bar):1, ]
  }
  guide$hash <- with(guide, digest::digest(list(
    title, bar$.value,
    direction, name
  )))
  guide
}
#' @rdname guide-helpers
#' @export
guide_merge.edge_direction <- function(guide, new_guide) {
  guide$bar <- merge(guide$bar, new_guide$bar, sort = FALSE)
  guide$override.aes <- c(guide$override.aes, new_guide$override.aes)
  if (any(duplicated(names(guide$override.aes)))) warning('Duplicated override.aes is ignored.')
  guide$override.aes <- guide$override.aes[!duplicated(names(guide$override.aes))]
  guide
}
#' @export
#' @rdname guide-helpers
guide_geom.edge_direction <- function(guide, layers, default_mapping) {
  guide$geoms <- lapply(layers, function(layer) {
    all <- names(c(
      layer$mapping, if (layer$inherit.aes) default_mapping,
      layer$stat$default_aes
    ))
    geom <- c(layer$geom$required_aes, names(layer$geom$default_aes))
    matched <- intersect(intersect(all, geom), names(guide$bar))
    matched <- setdiff(matched, names(layer$geom_params))
    matched <- setdiff(matched, names(layer$aes_params))
    if (length(matched) > 0) {
      if (is.na(layer$show.legend) || layer$show.legend) {
        data <- layer$geom$use_defaults(
          guide$bar[matched],
          layer$aes_params
        )
      }
      else {
        return(NULL)
      }
    }
    else {
      if (is.na(layer$show.legend) || !layer$show.legend) {
        return(NULL)
      }
      else {
        data <- layer$geom$use_defaults(
          NULL,
          layer$aes_params
        )[rep(1, nrow(guide$bar)), ]
      }
    }
    data <- utils::modifyList(data, guide$override.aes)
    list(data = data, params = c(layer$geom_params, layer$stat_params))
  })
  guide$geoms <- guide$geoms[!vapply(guide$geoms, is.null, logical(1))]
  if (length(guide$geoms) == 0) {
    guide <- NULL
  }
  guide
}
#' @importFrom grid convertWidth convertHeight unit grobWidth grobHeight arrow grobName
#' @importFrom gtable gtable gtable_add_grob
#' @importFrom utils tail
#' @importFrom stats setNames
#' @rdname guide-helpers
#' @export
guide_gengrob.edge_direction <- function(guide, theme) {
  switch(guide$direction, horizontal = {
    arrow.position <- guide$arrow.position %||% 'bottom'
    if (!arrow.position %in% c('top', 'bottom')) {
      stop('label position "', arrow.position, '" is invalid')
    }
  }, vertical = {
    arrow.position <- guide$arrow.position %||% 'right'
    if (!arrow.position %in% c('left', 'right')) {
      stop('label position "', arrow.position, '" is invalid')
    }
  })
  arrowlength <- convertWidth(guide$arrowlength %||%
    (theme$legend.key.width * 5), 'mm')
  arrowwidth <- convertWidth(unit(sin(30 / 360 * 2 * pi) * 0.25 * 2, 'in'), 'mm')
  edgewidth <- max(vapply(guide$geoms, function(g) {
    max(g$data$edge_width)
  }, numeric(1))) * .pt
  edgewidth <- convertWidth(unit(edgewidth, 'points'), 'mm')
  hgap <- c(convertWidth(unit(0.3, 'lines'), 'mm'))
  vgap <- hgap
  x <- rep(c(edgewidth) / 2, nrow(guide$bar))
  xend <- x
  y <- seq(0, 1, length.out = nrow(guide$bar) + 1) * c(arrowlength)
  yend <- y[-1]
  y <- y[-length(y)]
  grob.bar <- switch(
    guide$direction,
    horizontal = {
      lapply(guide$geoms, function(g) {
        segmentsGrob(
          x0 = y, y0 = x, x1 = yend, y1 = xend,
          default.units = 'mm',
          gp = gpar(
            col = alpha(
              g$data$edge_colour,
              g$data$edge_alpha
            ),
            lwd = g$data$edge_width * .pt,
            lty = g$data$edge_linetype,
            lineend = 'butt'
          )
        )
      })
    },
    vertical = {
      lapply(guide$geoms, function(g) {
        segmentsGrob(
          x0 = x, y0 = y, x1 = xend, y1 = yend,
          default.units = 'mm',
          gp = gpar(
            col = alpha(
              g$data$edge_colour,
              g$data$edge_alpha
            ),
            lwd = g$data$edge_width * .pt,
            lty = g$data$edge_linetype,
            lineend = 'butt'
          )
        )
      })
    }
  )
  grob.bar <- do.call(gList, grob.bar)
  grob.title <- ggname(
    'guide.title',
    element_grob(guide$title.theme %||%
      calc_element('legend.title', theme),
    label = guide$title,
    hjust = guide$title.hjust %||%
      theme$legend.title.align %||% 0,
    vjust = guide$title.vjust %||% 0.5
    )
  )
  title_width <- convertWidth(grobWidth(grob.title), 'mm')
  title_width.c <- c(title_width)
  title_height <- convertHeight(grobHeight(grob.title), 'mm')
  title_height.c <- c(title_height)
  grob.arrow <- {
    if (!guide$arrow) {
      zeroGrob()
    } else {
      switch(
        guide$direction,
        horizontal = {
          segmentsGrob(
            x0 = y[1], y0 = c(arrowwidth) / 2, x1 = tail(yend, 1),
            y1 = c(arrowwidth) / 2, default.units = 'mm',
            gp = gpar(
              col = 'black', lwd = 0.5 * .pt,
              lty = 'solid', lineend = 'round'
            ),
            arrow = arrow(ends = if (guide$reverse) {
              'first'
            } else {
              'last'
            })
          )
        },
        vertical = {
          segmentsGrob(
            x0 = c(arrowwidth) / 2, y0 = y[1], x1 = c(arrowwidth) / 2,
            y1 = tail(yend, 1), default.units = 'mm',
            gp = gpar(
              col = 'black', lwd = 0.5 * .pt,
              lty = 'solid', lineend = 'round'
            ),
            arrow = arrow(ends = if (guide$reverse) {
              'first'
            } else {
              'last'
            })
          )
        }
      )
    }
  }
  switch(guide$direction, horizontal = {
    switch(arrow.position, top = {
      bl_widths <- c(arrowlength)
      bl_heights <- c(c(arrowwidth), vgap, c(edgewidth))
      vps <- list(
        bar.row = 3, bar.col = 1, label.row = 1,
        label.col = 1
      )
    }, bottom = {
      bl_widths <- c(arrowlength)
      bl_heights <- c(c(edgewidth), vgap, c(arrowwidth))
      vps <- list(
        bar.row = 1, bar.col = 1, label.row = 3,
        label.col = 1
      )
    })
  }, vertical = {
    switch(arrow.position, left = {
      bl_widths <- c(c(arrowwidth), vgap, c(edgewidth))
      bl_heights <- c(arrowlength)
      vps <- list(
        bar.row = 1, bar.col = 3, label.row = 1,
        label.col = 1
      )
    }, right = {
      bl_widths <- c(c(edgewidth), vgap, c(arrowwidth))
      bl_heights <- c(arrowlength)
      vps <- list(
        bar.row = 1, bar.col = 1, label.row = 1,
        label.col = 3
      )
    })
  })
  switch(guide$title.position, top = {
    widths <- c(bl_widths, max(0, title_width.c - sum(bl_widths)))
    heights <- c(title_height.c, vgap, bl_heights)
    vps <- with(vps, list(
      bar.row = bar.row + 2, bar.col = bar.col,
      label.row = label.row + 2, label.col = label.col,
      title.row = 1, title.col = 1:length(widths)
    ))
  }, bottom = {
    widths <- c(bl_widths, max(0, title_width.c - sum(bl_widths)))
    heights <- c(bl_heights, vgap, title_height.c)
    vps <- with(vps, list(
      bar.row = bar.row, bar.col = bar.col,
      label.row = label.row, label.col = label.col,
      title.row = length(heights),
      title.col = 1:length(widths)
    ))
  }, left = {
    widths <- c(title_width.c, hgap, bl_widths)
    heights <- c(bl_heights, max(0, title_height.c - sum(bl_heights)))
    vps <- with(vps, list(bar.row = bar.row, bar.col = bar.col +
      2, label.row = label.row, label.col = label.col +
      2, title.row = 1:length(heights), title.col = 1))
  }, right = {
    widths <- c(bl_widths, hgap, title_width.c)
    heights <- c(bl_heights, max(0, title_height.c - sum(bl_heights)))
    vps <- with(vps, list(
      bar.row = bar.row, bar.col = bar.col,
      label.row = label.row, label.col = label.col,
      title.row = 1:length(heights),
      title.col = length(widths)
    ))
  })
  grob.background <- element_render(theme, 'legend.background')
  padding <- unit(1.5, 'mm')
  widths <- c(padding, widths, padding)
  heights <- c(padding, heights, padding)
  gt <- gtable(widths = unit(widths, 'mm'), heights = unit(
    heights,
    'mm'
  ))
  gt <- gtable_add_grob(gt, grob.background,
    name = 'background',
    clip = 'off', t = 1, r = -1, b = -1, l = 1
  )
  gt <- gtable_add_grob(gt, grob.bar,
    name = 'bar', clip = 'off',
    t = 1 + min(vps$bar.row), r = 1 + max(vps$bar.col), b = 1 +
      max(vps$bar.row), l = 1 + min(vps$bar.col)
  )
  gt <- gtable_add_grob(gt, grob.arrow,
    name = 'label', clip = 'off',
    t = 1 + min(vps$label.row), r = 1 + max(vps$label.col),
    b = 1 + max(vps$label.row), l = 1 + min(vps$label.col)
  )
  gt <- gtable_add_grob(gt, grob.title,
    name = 'title', clip = 'off',
    t = 1 + min(vps$title.row), r = 1 + max(vps$title.col),
    b = 1 + max(vps$title.row), l = 1 + min(vps$title.col)
  )
  gt
}

#' Define simple shapes for line capping
#'
#' This set of functions makes it easy to define shapes at the terminal points
#' of edges that are used to shorten the edges. The shapes themselves are not
#' drawn, but the edges will end at the boundary of the shape rather than at
#' the node position. This is especially relevant when drawing arrows at the
#' edges as the arrows will be partly obscured by the node unless the edge is
#' shortened. Edge shortening is dynamic and will update as the plot is resized,
#' making sure that the capping remains at an absolute distance to the end
#' point.
#'
#' @details
#' `geometry` is the base constructor, while the rest are helpers to save
#' typing. `circle` creates circles width a given radius, `square`
#' creates squares at a given side length, `ellipsis` creates ellipses with
#' given a and b values (width and height radii), and `rectangle` makes
#' rectangles of a given width and height. label_rect is a helper that, given
#' a list of strings and potentially formatting options creates a rectangle that
#' encloses the string.
#'
#' @param type The type of geometry to use. Currently `'circle'` and
#' `'rect'` is supported.
#'
#' @param width,height,length,radius,a,b The dimensions of the shape.
#'
#' @param unit,width_unit,height_unit,a_unit,b_unit The unit for the numbers
#' given.
#'
#' @return A geometry object encoding the specified shape.
#'
#' @examples
#' geometry(c('circle', 'rect', 'rect'), 1:3, 3:1)
#'
#' circle(1:4, 'mm')
#'
#' label_rect(c('some', 'different', 'words'), fontsize = 18)
#' @export
#'
geometry <- function(type = 'circle', width = 1, height = width,
                     width_unit = 'cm', height_unit = width_unit) {
  l <- max(length(type), length(width), length(height))
  g <- rep(type, length.out = l)
  g_na <- is.na(g)
  width <- rep(width, length.out = l)
  width[g_na] <- 0
  height <- rep(height, length.out = l)
  height[g_na] <- 0
  uwidth <- rep(width_unit, length.out = l)
  uheight <- rep(height_unit, length.out = l)
  g[g_na] <- 'circle'
  attributes(g) <- list(
    width = width, uwidth = uwidth, height = height,
    uheight = uheight, class = 'geometry'
  )
  g
}
#' @rdname geometry
#'
#' @export
circle <- function(radius = 1, unit = 'cm') {
  geometry('circle', width = radius * 2, width_unit = unit)
}
#' @rdname geometry
#'
#' @export
square <- function(length = 1, unit = 'cm') {
  geometry('rect', width = length, width_unit = unit)
}
#' @rdname geometry
#'
#' @export
ellipsis <- function(a = 1, b = 1, a_unit = 'cm', b_unit = a_unit) {
  geometry('circle',
    width = a * 2, height = b * 2, width_unit = a_unit,
    height_unit = b_unit
  )
}
#' @rdname geometry
#'
#' @export
rectangle <- function(width = 1, height = 1, width_unit = 'cm',
                      height_unit = width_unit) {
  geometry('rect',
    width = width, height = height, width_unit = width_unit,
    height_unit = height_unit
  )
}
#' @rdname geometry
#'
#' @param label The text to be enclosed
#'
#' @param padding extra size to be added around the text using the
#' [ggplot2::margin()] function
#'
#' @param ... Passed on to [grid::gpar()]
#'
#' @export
#' @importFrom grid convertWidth convertHeight textGrob grobWidth grobHeight
label_rect <- function(label, padding = margin(1, 1, 1.5, 1, 'mm'), ...) {
  grobs <- lapply(label, textGrob, gp = gpar(...))
  width <- abs_width(grobs)
  height <- abs_height(grobs)
  width <- width + sum(convertWidth(padding[c(2, 4)], 'cm', TRUE))
  height <- height + sum(convertHeight(padding[c(1, 3)], 'cm', TRUE))
  geometry('rect', width = width, height = height)
}
#' @rdname geometry
#'
#' @param x An object to test for geometry inheritance
#'
#' @export
is.geometry <- function(x) inherits(x, 'geometry')
#' @export
length.geometry <- function(x) length(unclass(x))
#' @export
`[.geometry` <- function(x, i, ...) {
  g <- unclass(x)[i]
  attributes(g) <- list(
    width = attr(x, 'width')[i],
    height = attr(x, 'height')[i],
    uwidth = attr(x, 'uwidth')[i],
    uheight = attr(x, 'uheight')[i],
    class = 'geometry'
  )
  g
}
#' @export
`[<-.geometry` <- function(x, ..., value) {
  if (!is.geometry(value)) stop('Only possible to insert geometries', call. = FALSE)
  type <- unclass(x)
  type[...] <- unclass(value)
  width <- attr(x, 'width')
  width[...] <- attr(value, 'width')
  height <- attr(x, 'height')
  height[...] <- attr(value, 'height')
  uwidth <- attr(x, 'uwidth')
  uwidth[...] <- attr(value, 'uwidth')
  uheight <- attr(x, 'uheight')
  uheight[...] <- attr(value, 'uheight')
  attributes(type) <- list(
    width = width,
    height = height,
    uwidth = uwidth,
    uheight = uheight,
    class = 'geometry'
  )
  type
}
#' @export
format.geometry <- function(x, ...) {
  paste0(
    unclass(x), '(', attr(x, 'width'), attr(x, 'uwidth'),
    ', ', attr(x, 'height'), attr(x, 'uheight'), ')'
  )
}
#' @export
print.geometry <- function(x, ...) {
  print(format(x, ...))
}
#' @export
rep.geometry <- function(x, ...) {
  i <- rep(seq_along(x), ...)
  x[i]
}
#' @export
as.data.frame.geometry <- function(x, row.names = NULL, optional = FALSE, ...) {
  new_data_frame(list(x))
}
#' @export
c.geometry <- function(...) {
  geometries <- list(...)
  base <- do.call(c, lapply(geometries, unclass))
  g_attr <- do.call(Map, c(list(f = c), lapply(geometries, attributes)))
  g_attr$class <- 'geometry'
  attributes(base) <- g_attr
  base
}
#' @export
is.na.geometry <- function(x) {
  is.na(unclass(x))
}
geo_type <- function(x) {
  if (!is.geometry(x)) stop('x must be a geometry object', call. = FALSE)
  unclass(x)
}
geo_width <- function(x) {
  if (!is.geometry(x)) stop('x must be a geometry object', call. = FALSE)
  unit(attr(x, 'width'), attr(x, 'uwidth'))
}
geo_height <- function(x) {
  if (!is.geometry(x)) stop('x must be a geometry object', call. = FALSE)
  unit(attr(x, 'height'), attr(x, 'uheight'))
}
#' @importFrom grid convertHeight grobHeight
abs_height <- function(grobs) {
  vapply(grobs, function(g) convertHeight(grobHeight(g), 'cm', TRUE), numeric(1))
}
#' @importFrom grid convertWidth grobWidth
abs_width <- function(grobs) {
  vapply(grobs, function(g) convertWidth(grobWidth(g), 'cm', TRUE), numeric(1))
}
#' Define default scale type for geometry
#'
#' This function is quite useless as geometry is not meant to be scaled, but it
#' is a requirement for ggplot2 to handle it correctly.
#'
#' @export
#'
#' @keywords internal
scale_type.geometry <- function(x) 'identity'

#' Create small multiples based on edge attributes
#'
#' This function is equivalent to [ggplot2::facet_wrap()] but only
#' facets edges. Nodes are repeated in every panel.
#'
#' @inheritParams ggplot2::facet_wrap
#'
#' @family ggraph-facets
#'
#' @examples
#' gr <- tidygraph::as_tbl_graph(highschool)
#'
#' ggraph(gr) +
#'   geom_edge_link() +
#'   geom_node_point() +
#'   facet_edges(~year)
#' @export
#'
facet_edges <- function(facets, nrow = NULL, ncol = NULL, scales = 'fixed',
                        shrink = TRUE, labeller = 'label_value', as.table = TRUE,
                        switch = NULL, drop = TRUE, dir = 'h',
                        strip.position = 'top') {
  facet <- facet_wrap(
    facets = facets, nrow = nrow, ncol = ncol,
    scales = scales, shrink = shrink, labeller = labeller,
    as.table = as.table, switch = switch, drop = drop,
    dir = dir, strip.position = strip.position
  )
  seed <- sample(.Machine$integer.max, 1L)
  facet$params$facets[] <- lapply(seq_along(facet$params$facets), function(i) {
    rlang::quo(withr::with_seed(seed, !!facet$params$facets[[i]]))
  })
  ggproto(NULL, FacetEdges,
    shrink = shrink,
    params = facet$params
  )
}

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
FacetEdges <- ggproto('FacetEdges', FacetWrap,
  compute_layout = function(data, params) {
    plot_data <- data[[1]]
    data <- split(data, vapply(data, data_type, character(1)))
    facet_data <- data$edge_ggraph

    panels <- FacetWrap$compute_layout(facet_data, params)
    node_placement <- rep(list(plot_data$.ggraph.index), nrow(panels))
    names(node_placement) <- as.character(seq_along(node_placement))
    attr(panels, 'node_placement') <- node_placement
    panels
  },
  map_data = function(data, layout, params) {
    switch(
      data_type(data),
      node_ggraph = {
        node_map <- rep(list(data), length(attr(layout, 'node_placement')))
        panel <- rep(seq_along(node_map), vapply(node_map, nrow, numeric(1)))
        node_map <- rbind_dfs(node_map)
        node_map$PANEL <- factor(panel, levels = levels(layout$PANEL))
        node_map
      },
      edge_ggraph = , {
        FacetWrap$map_data(data, layout, params)
      }
    )
  }
)

#' Calculate nodes as rectangles subdividing that of their parent
#'
#' A treemap is a space filling hierarchical layout that maps nodes to
#' rectangles. The rectangles of the children of a node is packed into the
#' rectangle of the node so that the size of a rectangle is a function of the
#' size of the children. The size of the leaf nodes can be mapped arbitrarily
#' (defaults to 1). Many different algorithms exists for dividing a rectangle
#' into smaller bits, some optimizing the aspect ratio and some focusing on the
#' ordering of the rectangles. See details for more discussions on this. The
#' treemap layout was first developed by Ben Shneiderman for visualizing disk
#' usage in the early '90 and has seen many improvements since.
#'
#' @details
#' Different approaches to dividing the rectangles in a treemap exists; all with
#' their strengths and weaknesses. Currently only the split algorithm is
#' implemented which strikes a good balance between aspect ratio and order
#' preservation, but other, more well-known, algorithms such as squarify and
#' slice-and-dice will eventually be implemented.
#'
#' \strong{Algorithms}
#'
#' *Split* (default)
#'
#' The Split algorithm was developed by Bjorn Engdahl in order to address the
#' downsides of both the original slice-and-dice algorithm (poor aspect ratio)
#' and the popular squarify algorithm (no ordering of nodes). It works by
#' finding the best cut in the ordered list of children in terms of making sure
#' that the two rectangles associated with the split will have optimal aspect
#' ratio.
#'
#'
#' @note
#' Treemap is a layout intended for trees, that is, graphs where nodes
#' only have one parent and zero or more children. If the provided graph does
#' not fit this format an attempt to convert it to such a format will be made.
#'
#' @param graph A `tbl_graph` object
#'
#' @param algorithm The name of the tiling algorithm to use. Defaults to 'split'
#'
#' @param weight An optional node variable to use as weight. Will only affect
#' the weight of leaf nodes as the weight of non-leaf nodes are derived from
#' their children.
#'
#' @param circular Logical. Should the layout be transformed to a circular
#' representation. Ignored.
#'
#' @param sort.by The name of a node variables to sort the nodes by.
#'
#' @param direction The direction of the tree in the graph. `'out'` (default)
#' means that parents point towards their children, while `'in'` means that
#' children point towards their parent.
#'
#' @param height The height of the bounding rectangle
#'
#' @param width The width of the bounding rectangle
#'
#' @return A data.frame with the columns `x`, `y`, `width`,
#' `height`, `leaf`, `depth`, `circular` as well as any
#' information stored as node variables in the tbl_graph object.
#'
#' @references
#' Engdahl, B. (2005). *Ordered and unordered treemap algorithms and their
#' applications on handheld devices*. Master's Degree Project.
#'
#' Johnson, B., & Ben Shneiderman. (1991). *Tree maps: A Space-Filling
#' Approach to the Visualization of Hierarchical Information Structures*. IEEE
#' Visualization, 284-291. <https://doi.org/10.1109/VISUAL.1991.175815>
#'
#' @family layout_tbl_graph_*
#'
layout_tbl_graph_treemap <- function(graph, algorithm = 'split', weight = NULL, circular = FALSE, sort.by = NULL, direction = 'out', height = 1, width = 1) {
  weight <- enquo(weight)
  weight <- eval_tidy(weight, .N())
  sort.by <- enquo(sort.by)
  sort.by <- eval_tidy(sort.by)
  hierarchy <- tree_to_hierarchy(graph, direction, sort.by, weight)
  layout <- switch(
    algorithm,
    split = splitTreemap(hierarchy$parent, hierarchy$order, hierarchy$weight, width, height),
    stop('Unknown algorithm')
  )[-1, ]
  layout <- new_data_frame(list(
    x = layout[, 1] + layout[, 3] / 2,
    y = layout[, 2] + layout[, 4] / 2,
    width = layout[, 3],
    height = layout[, 4],
    circular = FALSE,
    leaf = degree(graph, mode = direction) == 0,
    depth = node_depth(graph, mode = direction)
  ))
  extra_data <- as_tibble(graph, active = 'nodes')
  warn_dropped_vars(layout, extra_data)
  layout <- cbind(layout, extra_data[, !names(extra_data) %in% names(layout), drop = FALSE])
  layout
}

#' Edge size scales
#'
#' This set of scales defines new size scales for edge geoms equivalent to the
#' ones already defined by ggplot2. See [ggplot2::scale_size()] for
#' more information. The different geoms will know whether to use edge scales or
#' the standard scales so it is not necessary to write `edge_size` in
#' the call to the geom - just use `size`.
#'
#' @note In ggplot2 size conflates both line width and point size into one
#' scale. In ggraph there is also a width scale ([scale_edge_width()])
#' that is used for linewidth. As edges are often represented by lines the width
#' scale is the most common.
#'
#' @return A ggproto object inheriting from `Scale`
#'
#' @family scale_edge_*
#'
#' @name scale_edge_size
#' @rdname scale_edge_size
#'
NULL

#' @rdname scale_edge_size
#'
#' @inheritParams ggplot2::scale_size_continuous
#'
#' @export
scale_edge_size_continuous <- function(name = waiver(), breaks = waiver(), labels = waiver(),
                                       limits = NULL, range = c(1, 6),
                                       trans = "identity", guide = "legend") {
  sc <- scale_size_continuous(name = name, breaks = breaks, labels = labels, limits = limits,
                              range = range, trans = trans, guide = guide)
  sc$aesthetics <- 'edge_size'
  sc
}
#' @rdname scale_edge_size
#'
#' @inheritParams ggplot2::scale_radius
#'
#' @export
scale_edge_radius <- function(name = waiver(), breaks = waiver(), labels = waiver(),
                              limits = NULL, range = c(1, 6),
                              trans = "identity", guide = "legend") {
  sc <- scale_radius(name = name, breaks = breaks, labels = labels, limits = limits,
                     range = range, trans = trans, guide = guide)
  sc$aesthetics <- 'edge_size'
  sc
}
#' @rdname scale_edge_size
#'
#' @export
scale_edge_size <- scale_edge_size_continuous
#' @rdname scale_edge_size
#'
#' @inheritParams ggplot2::scale_size_discrete
#' @export
scale_edge_size_discrete <- function(...) {
  cli::cli_warn("Using {.field edge_size} for a discrete variable is not advised.")
  sc <- scale_size_ordinal(...)
  sc$aesthetics <- 'edge_size'
  sc
}
#' @rdname scale_edge_size
#'
#' @inheritParams ggplot2::scale_size_binned
#' @export
scale_edge_size_binned <- function(name = waiver(), breaks = waiver(), labels = waiver(),
                                   limits = NULL, range = c(1, 6), n.breaks = NULL,
                                   nice.breaks = TRUE, trans = "identity", guide = "bins") {
  sc <- scale_size_binned(name = name, breaks = breaks, labels = labels, limits = limits,
                          range = range, n.breaks = n.breaks, nice.breaks = nice.breaks,
                          trans = trans, guide = guide)
  sc$aesthetics <- 'edge_size'
  sc
}
#' @rdname scale_edge_size
#'
#' @inheritParams ggplot2::scale_size_area
#'
#' @export
scale_edge_size_area <- function(..., max_size = 6) {
  sc <- scale_size_area(..., max_size = max_size)
  sc$aesthetics <- 'edge_size'
  sc
}
#' @rdname scale_edge_size
#'
#'
#' @inheritParams ggplot2::scale_size_binned_area
#'
#' @export
scale_edge_size_binned_area <- function(..., max_size = 6) {
  sc <- scale_size_binned_area(..., max_size = max_size)
  sc$aesthetics <- 'edge_size'
  sc
}
#' @rdname scale_edge_size
#'
#' @inheritParams ggplot2::scale_size_manual
#'
#' @export
scale_edge_size_manual <- function(..., values, breaks = waiver(), na.value = NA) {
  sc <- scale_size_manual(..., values = values, breaks = breaks, na.value = na.value)
  sc$aesthetics <- 'edge_size'
  sc
}
#' @rdname scale_edge_size
#'
#' @inheritParams ggplot2::scale_size_identity
#'
#' @export
scale_edge_size_identity <- function(..., guide = 'none') {
  sc <- scale_size_identity(..., guide = guide)
  sc$aesthetics <- 'edge_size'
  sc
}

#' @rdname ggraph
#'
#' @aliases layout_ggraph
#'
#' @export
create_layout <- function(graph, layout, circular, ...) {
  if (is_layout(layout)) {
    warning('Ignoring `graph` as layout is already calculated. Pass the calculated layout to the `graph` argument to silence this warning', call. = FALSE)
    return(layout)
  }
  UseMethod('create_layout', graph)
}
is_layout <- function(x) inherits(x, 'layout_ggraph')
#' @rdname ggraph
#' @export
create_layout.default <- function(graph, layout, ...) {
  graph <- tryCatch(as_tbl_graph(graph), error = function(e) {
    stop('No layout function defined for objects of class ', class(graph), call. = FALSE)
  })
  create_layout(graph, layout, ...)
}
#' @rdname ggraph
#' @export
create_layout.layout_ggraph <- function(graph, ...) {
  graph
}
#' @export
as.data.frame.layout_ggraph <- function(x, ...) {
  extra_attr <- names(attributes(x))
  extra_attr <- extra_attr[!extra_attr %in% c('names', 'row.names')]
  attributes(x)[extra_attr] <- NULL
  class(x) <- 'data.frame'
  x
}
check_layout <- function(layout) {
  if (!is.data.frame(layout)) {
    stop('layout must be a data.frame', call. = FALSE)
  }
  if (!(is.numeric(layout$x) && is.numeric(layout$y))) {
    stop('layout must contain numeric `x` and `y` columns', call. = FALSE)
  }
  graph <- attr(layout, 'graph')
  if (!is.tbl_graph(graph)) {
    stop('layout must have a tbl_graph as the `graph` attribute', call. = FALSE)
  }
  if (nrow(layout) != gorder(graph)) {
    stop('layout must contain the same number of rows as nodes', call. = FALSE)
  }
  if (!'circular' %in% names(layout)) {
    layout$circular <- FALSE
  }
  if (!is.logical(layout$circular)) {
    stop('circular column must be logical', call. = FALSE)
  }
  layout
}

#' Get the angle of nodes and edges
#'
#' These helper functions makes it easy to calculate the angle associated with
#' nodes and edges. For nodes the angle is defined as the angle of the vector
#' pointing towards the node position, and is thus mainly suited for circular
#' layouts where it can be used to calculate the angle of labels. For edges it
#' is simply the angle of the vector describing the edge.
#'
#' @param x,y A vector of positions
#'
#' @param xend,yend The end position of the edge
#'
#' @param degrees Logical. Should the angle be returned in degree (`TRUE`)
#' or radians (`FALSE`). Defaults to `TRUE`.
#'
#' @return A vector with the angle of each node/edge
#'
#' @examples
#' require(tidygraph)
#' flareGraph <- tbl_graph(flare$vertices, flare$edges)
#'
#' ggraph(flareGraph, 'dendrogram', circular = TRUE) +
#'   geom_edge_diagonal0() +
#'   geom_node_text(aes(filter = leaf, angle = node_angle(x, y), label = shortName),
#'     hjust = 'outward', size = 2
#'   ) +
#'   expand_limits(x = c(-1.3, 1.3), y = c(-1.3, 1.3))
#' @export
#'
node_angle <- function(x, y, degrees = TRUE) {
  angles <- atan2(y, x)
  angles[angles < 0] <- angles[angles < 0] + 2 * pi
  if (degrees) {
    angles * 360 / (2 * pi)
  } else {
    angles
  }
}
#' @rdname node_angle
#'
#' @export
edge_angle <- function(x, y, xend, yend, degrees = TRUE) {
  x <- xend - x
  y <- yend - y
  node_angle(x, y, degrees)
}


### COPY FROM GGPLOT2 NON-EXPORTS
#' @importFrom scales rescale_mid
mid_rescaler <- function(mid) {
  function(x, to = c(0, 1), from = range(x, na.rm = TRUE)) {
    rescale_mid(x, to, from, mid)
  }
}
manual_scale <- function(aesthetic, values = NULL, breaks = waiver(), ..., limits = NULL) {
  # check for missing `values` parameter, in lieu of providing
  # a default to all the different scale_*_manual() functions
  if (is_missing(values)) {
    values <- NULL
  } else {
    force(values)
  }

  if (is.null(limits) && !is.null(names(values))) {
    # Limits as function to access `values` names later on (#4619)
    limits <- function(x) intersect(x, names(values))
  }

  # order values according to breaks
  if (is.vector(values) && is.null(names(values)) && !inherits(breaks, 'waiver') &&
      !is.null(breaks) && !is.function(breaks)) {
    if (length(breaks) <= length(values)) {
      names(values) <- breaks
    } else {
      names(values) <- breaks[1:length(values)]
    }
  }

  pal <- function(n) {
    if (n > length(values)) {
      cli::cli_abort("Insufficient values in manual scale. {n} needed but only {length(values)} provided.")
    }
    values
  }
  discrete_scale(aesthetic, "manual", pal, breaks = breaks, limits = limits, ...)
}
#' @importFrom scales zero_range
resolution <- function(x, zero = TRUE) {
  if (is.integer(x) || zero_range(range(x, na.rm = TRUE))) {
    return(1)
  }
  x <- unique0(as.numeric(x))
  if (zero) {
    x <- unique0(c(0, x))
  }
  min(diff(sort(x)))
}
#' @importFrom grid grobName
ggname <- function(prefix, grob) {
  grob$name <- grobName(grob, prefix)
  grob
}
element_render <- function(theme, element, ..., name = NULL) {
  el <- calc_element(element, theme)
  if (is.null(el)) {
    cli::cli_inform("Theme element {.var {element}} is missing")
    return(zeroGrob())
  }
  ggname(paste(element, name, sep = '.'), element_grob(el, ...))
}
.all_aesthetics <- c(
  'adj', 'alpha', 'angle', 'bg', 'cex', 'col', 'color', 'colour',
  'fg', 'fill', 'group', 'hjust', 'label', 'linetype', 'lower',
  'lty', 'lwd', 'max', 'middle', 'min', 'pch', 'radius', 'sample',
  'shape', 'size', 'srt', 'upper', 'vjust', 'weight', 'width',
  'x', 'xend', 'xmax', 'xmin', 'xintercept', 'y', 'yend', 'ymax',
  'ymin', 'yintercept', 'z'
)
.base_to_ggplot <- structure(
  c(
    'colour', 'colour', 'shape', 'size', 'linetype', 'size', 'angle', 'hjust',
    'fill', 'colour', 'ymin', 'ymax'
  ),
  .Names = c(
    'col', 'color', 'pch', 'cex', 'lty', 'lwd', 'srt', 'adj', 'bg',
    'fg', 'min', 'max'
  )
)
rename_aes <- function(x) {
  # Convert prefixes to full names
  full <- match(names(x), .all_aesthetics)
  names(x)[!is.na(full)] <- .all_aesthetics[full[!is.na(full)]]

  old_names <- match(names(x), names(.base_to_ggplot))
  names(x)[!is.na(old_names)] <- .base_to_ggplot[old_names[!is.na(old_names)]]

  x
}

translate_shape_string <- function(shape_string) {
  if (is.numeric(shape_string)) return(shape_string)
  # strings of length 0 or 1 are interpreted as symbols by grid
  if (nchar(shape_string[1]) <= 1) {
    return(shape_string)
  }

  pch_table <- c(
    "square open"           = 0,
    "circle open"           = 1,
    "triangle open"         = 2,
    "plus"                  = 3,
    "cross"                 = 4,
    "diamond open"          = 5,
    "triangle down open"    = 6,
    "square cross"          = 7,
    "asterisk"              = 8,
    "diamond plus"          = 9,
    "circle plus"           = 10,
    "star"                  = 11,
    "square plus"           = 12,
    "circle cross"          = 13,
    "square triangle"       = 14,
    "triangle square"       = 14,
    "square"                = 15,
    "circle small"          = 16,
    "triangle"              = 17,
    "diamond"               = 18,
    "circle"                = 19,
    "bullet"                = 20,
    "circle filled"         = 21,
    "square filled"         = 22,
    "diamond filled"        = 23,
    "triangle filled"       = 24,
    "triangle down filled"  = 25
  )

  shape_match <- charmatch(shape_string, names(pch_table))

  invalid_strings <- is.na(shape_match)
  nonunique_strings <- shape_match == 0

  if (any(invalid_strings)) {
    bad_string <- unique0(shape_string[invalid_strings])
    cli::cli_abort("Shape aesthetic contains invalid value{?s}: {.val {bad_string}}")
  }

  if (any(nonunique_strings)) {
    bad_string <- unique0(shape_string[nonunique_strings])
    cli::cli_abort(c(
      "shape names must be given unambiguously",
      "i" = "Fix {.val {bad_string}}"
    ))
  }

  unname(pch_table[shape_match])
}

#' @importFrom viridis scale_color_viridis
#' @export
viridis::scale_color_viridis

#' @importFrom viridis scale_fill_viridis
#' @export
viridis::scale_fill_viridis

# Wrapping vctrs data_frame constructor with no name repair
data_frame0 <- function(...) data_frame(..., .name_repair = "minimal")

# Wrapping unique0() to accept NULL
unique0 <- function(x, ...) if (is.null(x)) x else vec_unique(x, ...)

df_rows <- function(x, i) {
  cols <- lapply(x, `[`, i = i)
  data_frame0(!!!cols, .size = length(i))
}
split_matrix <- function(x, col_names = colnames(x)) {
  force(col_names)
  x <- lapply(seq_len(ncol(x)), function(i) x[, i])
  if (!is.null(col_names)) names(x) <- col_names
  x
}
# More performant modifyList without recursion
modify_list <- function(old, new) {
  for (i in names(new)) old[[i]] <- new[[i]]
  old
}
empty <- function(df) {
  is.null(df) || nrow(df) == 0 || ncol(df) == 0
}
split_indices <- function(group) {
  split(seq_along(group), group)
}
# Adapted from plyr:::id_vars
# Create a unique id for elements in a single vector
id_var <- function(x, drop = FALSE) {
  if (length(x) == 0) {
    id <- integer()
    n <- 0L
  } else if (!is.null(attr(x, 'n')) && !drop) {
    return(x)
  } else if (is.factor(x) && !drop) {
    x <- addNA(x, ifany = TRUE)
    id <- as.integer(x)
    n <- length(levels(x))
  } else {
    levels <- sort(unique0(x), na.last = TRUE)
    id <- match(x, levels)
    n <- max(id)
  }
  attr(id, 'n') <- n
  id
}
#' Create an unique integer id for each unique row in a data.frame
#'
#' Properties:
#' - `order(id)` is equivalent to `do.call(order, df)`
#' - rows containing the same data have the same value
#' - if `drop = FALSE` then room for all possibilites
#'
#' @param .variables list of variables
#' @param drop Should unused factor levels be dropped?
#'
#' @return An integer vector with attribute `n` giving the total number of
#' possible unique rows
#'
#' @keywords internal
#' @noRd
#'
id <- function(.variables, drop = FALSE) {
  nrows <- NULL
  if (is.data.frame(.variables)) {
    nrows <- nrow(.variables)
    .variables <- unclass(.variables)
  }
  lengths <- vapply(.variables, length, integer(1))
  .variables <- .variables[lengths != 0]
  if (length(.variables) == 0) {
    n <- nrows %||% 0L
    id <- seq_len(n)
    attr(id, 'n') <- n
    return(id)
  }
  if (length(.variables) == 1) {
    return(id_var(.variables[[1]], drop = drop))
  }
  ids <- rev(lapply(.variables, id_var, drop = drop))
  p <- length(ids)
  ndistinct <- vapply(ids, attr, 'n', FUN.VALUE = numeric(1), USE.NAMES = FALSE)
  n <- prod(ndistinct)
  if (n > 2^31) {
    char_id <- inject(paste(!!!ids, sep = '\r'))
    res <- match(char_id, unique0(char_id))
  }
  else {
    combs <- c(1, cumprod(ndistinct[-p]))
    mat <- inject(cbind(!!!ids))
    res <- c((mat - 1L) %*% combs + 1L)
  }
  if (drop) {
    id_var(res, drop = TRUE)
  }
  else {
    res <- as.integer(res)
    attr(res, 'n') <- n
    res
  }
}

# Use chartr() for safety since toupper() fails to convert i to I in Turkish locale
lower_ascii <- "abcdefghijklmnopqrstuvwxyz"
upper_ascii <- "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
to_lower_ascii <- function(x) chartr(upper_ascii, lower_ascii, x)
to_upper_ascii <- function(x) chartr(lower_ascii, upper_ascii, x)

tolower <- function(x) {
  cli::cli_abort("Please use {.fn to_lower_ascii}, which works fine in all locales.")
}

toupper <- function(x) {
  cli::cli_abort("Please use {.fn to_upper_ascii}, which works fine in all locales.")
}

# Convert a snake_case string to camelCase
camelize <- function(x, first = FALSE) {
  x <- gsub("_(.)", "\\U\\1", x, perl = TRUE)
  if (first) x <- firstUpper(x)
  x
}

snakeize <- function(x) {
  x <- gsub("([A-Za-z])([A-Z])([a-z])", "\\1_\\2\\3", x)
  x <- gsub(".", "_", x, fixed = TRUE)
  x <- gsub("([a-z])([A-Z])", "\\1_\\2", x)
  to_lower_ascii(x)
}

firstUpper <- function(s) {
  paste0(to_upper_ascii(substring(s, 1, 1)), substring(s, 2))
}

snake_class <- function(x) {
  snakeize(class(x)[1])
}

#' Place nodes based on a multidimensional scaling of a set of pivot nodes
#'
#' This layout is similar to the 'mds' layout but uses only a subset of pivot
#' nodes for the mds calculation, making it considerably faster and thus suited
#' for large graphs
#'
#' @param graph A tbl_graph object
#' @param pivots The number of pivot nodes
#' @param weights An expression evaluated on the edge data to provide edge
#' weights for the layout. Currently ignored for the sparse version
#' @param circular ignored
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @references
#' Brandes, U. and Pich, C. (2006). *Eigensolver Methods for Progressive
#' Multidimensional Scaling of Large Data.* In International Symposium on Graph
#' Drawing (pp. 42-53). Springer
#'
#' @family layout_tbl_graph_*
#'
#' @author The underlying algorithm is implemented in the graphlayouts package
#' by David Schoch
#'
#' @importFrom graphlayouts layout_with_pmds
layout_tbl_graph_pmds <- function(graph, pivots, weights = NULL, circular = FALSE) {
  weights <- eval_tidy(enquo(weights), .E())
  if (is.null(weights)) weights <- NA
  xy <- layout_with_pmds(graph, pivots = pivots, weights = weights)
  nodes <- new_data_frame(list(x = xy[,1], y = xy[,2]))
  nodes$circular <- FALSE
  extra_data <- as_tibble(graph, active = 'nodes')
  warn_dropped_vars(nodes, extra_data)
  nodes <- cbind(nodes, extra_data[, !names(extra_data) %in% names(nodes), drop = FALSE])
  nodes
}

#' Show nodes as a line spanning a horizontal range
#'
#' This geom is most useful together with the [fabric][layout_tbl_graph_fabric]
#' layout for showing the horizontal span of each node.
#'
#' @section Aesthetics:
#' `geom_node_point` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overridden.
#'
#' - **x**
#' - **xend**
#' - **y**
#' - **yend**
#' - alpha
#' - colour
#' - linetype
#' - size
#' - filter
#'
#' @inheritParams ggplot2::geom_linerange
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x is mapped to xmin, xend is mapped to xmax
#' and y and yend are mapped to y in the node data.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_node_*
#'
#' @examples
#' require(tidygraph)
#' gr <- as_tbl_graph(highschool)
#'
#' ggraph(gr, layout = 'fabric') +
#'   geom_node_range()
#' @export
#'
geom_node_range <- function(mapping = NULL, data = NULL, position = 'identity',
                            show.legend = NA, ...) {
  mapping <- aes_intersect(mapping, aes(x = xmin, xend = xmax, y = y, yend = y))
  layer(
    data = data, mapping = mapping, stat = StatFilter, geom = GeomSegment,
    position = position, show.legend = show.legend, inherit.aes = FALSE,
    params = list(na.rm = FALSE, ...)
  )
}

#' Draw the rectangles in a treemap
#'
#' A treemap is a space filling layout that recursively divides a rectangle to
#' the children of the node. Often only the leaf nodes are drawn as nodes higher
#' up in the hierarchy would obscure what is below. `geom_treemap` is a
#' shorthand for `geom_node_treemap` as node is implicit in the case of
#' treemap drawing
#'
#' @section Aesthetics:
#' `geom_treemap` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - **width**
#' - **height**
#' - alpha
#' - colour
#' - fill
#' - size
#' - stroke
#' - filter
#'
#' @inheritParams ggplot2::geom_tile
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x, y, width and height are mapped to
#' x, y, width and height in the node data.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_node_*
#'
#' @examples
#' # Create a graph of the flare class system
#' library(tidygraph)
#' flareGraph <- tbl_graph(flare$vertices, flare$edges) %>%
#'   mutate(
#'     class = map_bfs_chr(node_is_root(), .f = function(node, dist, path, ...) {
#'       if (dist <= 1) {
#'         return(shortName[node])
#'       }
#'       path$result[[nrow(path)]]
#'     })
#'   )
#'
#' ggraph(flareGraph, 'treemap', weight = size) +
#'   geom_node_tile(aes(fill = class, filter = leaf, alpha = depth), colour = NA) +
#'   geom_node_tile(aes(size = depth), colour = 'white') +
#'   scale_alpha(range = c(1, 0.5), guide = 'none') +
#'   scale_size(range = c(4, 0.2), guide = 'none')
#' @export
#'
geom_node_tile <- function(mapping = NULL, data = NULL, position = 'identity',
                           show.legend = NA, ...) {
  mapping <- aes_intersect(mapping, aes(
    x = x, y = y, width = width, height = height
  ))
  layer(
    data = data, mapping = mapping, stat = StatFilter, geom = GeomNodeTile,
    position = position, show.legend = show.legend, inherit.aes = FALSE,
    params = list(na.rm = FALSE, ...)
  )
}

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
#'
GeomNodeTile <- ggproto('GeomNodeTile', GeomTile,
  default_aes = aes(
    fill = NA, colour = 'black', size = 0.5, linetype = 1,
    alpha = NA, width = 1, height = 1
  ),
  required_aes = c('x', 'y')
)

#' Quickplot wrapper for networks
#'
#' This function is intended to quickly show an overview of your network data.
#' While it returns a ggraph object that layers etc can be added to it is
#' limited in use and should not be used as a foundation for more complicated
#' plots. It allows colour, labeling and sizing of nodes and edges, and the
#' exact combination of layout and layers will depend on these as well as the
#' features of the network. The output of this function may be fine-tuned at any
#' release and should not be considered stable. If a plot should be reproducible
#' it should be created manually.
#'
#' @param graph An object coercible to a tbl_graph
#' @param ... arguments passed on to methods
#'
#' @importFrom tidygraph as_tbl_graph .register_graph_context graph_is_tree graph_is_forest
#' @importFrom rlang quo_is_null quo_text as_quosure sym
#' @export
#'
#' @examples
#' library(tidygraph)
#' gr <- create_notable('herschel') %>%
#'   mutate(class = sample(letters[1:3], n(), TRUE)) %E>%
#'   mutate(weight = runif(n()))
#'
#' # Standard graph
#' autograph(gr)
#'
#' # Adding node labels will cap edges
#' autograph(gr, node_label = class)
#'
#' # Use tidygraph calls for mapping
#' autograph(gr, node_size = centrality_pagerank())
#'
#' # Trees are plotted as dendrograms
#' iris_tree <- hclust(dist(iris[1:4], method = 'euclidean'), method = 'ward.D2')
#' autograph(iris_tree)
#'
autograph <- function(graph, ...) {
  UseMethod('autograph')
}

#' @export
#' @rdname autograph
#' @param node_colour,edge_colour Colour mapping for nodes and edges
#' @param node_size,edge_width Size/width mapping for nodes and edges
#' @param node_label,edge_label Label mapping for nodes and edges
autograph.default <- function(graph, ..., node_colour = NULL, edge_colour = NULL,
                              node_size = NULL, edge_width = NULL,
                              node_label = NULL, edge_label = NULL) {
  node_colour <- enquo(node_colour)
  edge_colour <- enquo(edge_colour)
  node_size <- enquo(node_size)
  edge_width <- enquo(edge_width)
  node_label <- enquo(node_label)
  edge_label <- enquo(edge_label)
  graph <- as_tbl_graph(graph)
  .register_graph_context(graph, TRUE)

  if (graph_is_tree() || graph_is_forest()) {
    node_names <- names(as_tibble(graph, active = 'nodes'))
    height <- if ('height' %in% node_names) quo(height) else quo(NULL)
    p <- ggraph(graph, 'dendrogram', height = !!height) +
      geom_edge_elbow0(aes(colour = !!edge_colour, width = !!edge_width))
    if (!quo_is_null(node_colour) || !quo_is_null(node_size)) {
      p <- p +
        geom_node_point(aes(filter = leaf, colour = !!node_colour, size = !!node_size))
    }
    if (!quo_is_null(node_label)) {
      p <- p +
        geom_node_text(aes(filter = leaf, label = !!node_label, colour = !!node_colour), angle = 40, hjust = 1, vjust = 1) +
        coord_cartesian(clip = 'off')
    }
  } else {
    p <- suppressMessages(ggraph(graph) + coord_fixed())

    if (!quo_is_null(node_label)) {
      label_col <- quo_text(node_label)
      label_col <- paste0(c('node1.', 'node2.'), label_col)
      start_label <- sym(label_col[1])
      end_label <- sym(label_col[2])
      p <- p +
        geom_edge_link(aes(colour = !!edge_colour, label = !!edge_label,
                           width = !!edge_width, start_cap = label_rect(!!start_label),
                           end_cap = label_rect(!!end_label)),
                       angle_calc = 'along', label_dodge = unit(2.5, 'mm'))
    } else if (!quo_is_null(edge_label)) {
      p <- p +
        geom_edge_link(aes(colour = !!edge_colour, label = !!edge_label,
                            width = !!edge_width),
                        angle_calc = 'along', label_dodge = unit(2.5, 'mm'))
    } else {
      p <- p +
        geom_edge_link0(aes(colour = !!edge_colour, width = !!edge_width))
    }
    if (quo_is_null(node_label)) {
      p <- p +
        geom_node_point(aes(colour = !!node_colour, size = !!node_size))
    } else {
      p <- p +
        geom_node_text(aes(label = !!node_label, colour = !!node_colour))
    }
  }

  p
}

#' Deprecated autograph predecessor
#'
#' @export
#' @keywords internal
qgraph <- function(...) {
  .Deprecated('autograph', msg = 'To avoid name clashing with qgraph::qgraph(), ggraph::qgraph() has been renamed to ggraph::autograph()')
}

utils::globalVariables(c(
  'leaf'
))

#' Edge fill scales
#'
#' This set of scales defines new fill scales for edge geoms equivalent to the
#' ones already defined by ggplot2. The parameters are equivalent to the ones
#' from ggplot2 so there is nothing new under the sun. The different geoms will
#' know whether to use edge scales or the standard scales so it is not necessary
#' to write `edge_fill` in the call to the geom - just use `fill`.
#'
#' @return A ggproto object inheriting from `Scale`
#'
#' @family scale_edge_*
#'
#' @name scale_edge_fill
#' @rdname scale_edge_fill
#'
NULL

#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_hue
#'
#' @export
scale_edge_fill_hue <- function(..., h = c(0, 360) + 15, c = 100, l = 65, h.start = 0,
                                  direction = 1, na.value = "grey50", aesthetics = "edge_fill") {
  scale_fill_hue(..., h = h, c = c, l = l, h.start = h.start,
                   direction = direction, na.value = na.value, aesthetics = aesthetics)
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_brewer
#'
#' @export
scale_edge_fill_brewer <- function(..., type = "seq", palette = 1, direction = 1,
                                     aesthetics = "edge_fill") {
  scale_fill_brewer(..., type = type, palette = palette, direction = direction,
                      aesthetics = aesthetics)
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_distiller
#'
#' @export
scale_edge_fill_distiller <- function(..., type = "seq", palette = 1, direction = -1,
                                        values = NULL, space = "Lab", na.value = "grey50",
                                        guide = "edge_colourbar", aesthetics = "edge_fill") {
  scale_color_distiller(..., type = type, palette = palette, direction = direction,
                        values = values, space = space, na.value = na.value,
                        guide = guide, aesthetics = aesthetics)
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_gradient
#'
#' @export
scale_edge_fill_gradient <- function(..., low = "#132B43", high = "#56B1F7", space = "Lab",
                                       na.value = "grey50", guide = "edge_colourbar",
                                       aesthetics = "edge_fill") {
  scale_fill_gradient(..., low = low, high = high, space = space, na.value = na.value,
                        guide = guide, aesthetics = aesthetics)
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_gradient2
#'
#' @importFrom scales muted
#' @export
scale_edge_fill_gradient2 <- function(..., low = muted("red"), mid = "white",
                                        high = muted("blue"), midpoint = 0,
                                        space = "Lab", na.value = "grey50",
                                        guide = "edge_colourbar", aesthetics = "edge_fill") {
  scale_fill_gradient2(..., low = low, mid = mid, high = high, midpoint = midpoint,
                         space = space, na.value = na.value, guide = guide,
                         aesthetics = aesthetics)
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_gradientn
#'
#' @export
scale_edge_fill_gradientn <- function(..., colours, values = NULL, space = "Lab",
                                        na.value = "grey50", guide = "edge_colourbar",
                                        aesthetics = "edge_fill", colors) {
  scale_fill_gradientn(..., colours = colours, values = values, space = space,
                         na.value = na.value, guide = guide, aesthetics = aesthetics,
                         colors = colors)
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_grey
#'
#' @export
scale_edge_fill_grey <- function(..., start = 0.2, end = 0.8, na.value = "red",
                                   aesthetics = "edge_fill") {
  scale_fill_grey(..., start = start, end = end, na.value = na.value, aesthetics = aesthetics)
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_identity
#'
#' @export
scale_edge_fill_identity <- function(..., guide = "none", aesthetics = "edge_fill") {
  scale_fill_identity(..., guide = guide, aesthetics = aesthetics)
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_manual
#'
#' @export
scale_edge_fill_manual <- function(..., values, aesthetics = "edge_fill", breaks = waiver(), na.value = "grey50") {
  scale_fill_manual(..., values = values, aesthetics = aesthetics, breaks = breaks,
                      na.value = na.value)
}
#' @rdname scale_edge_fill
#'
#' @inheritParams viridis::scale_fill_viridis
#'
#' @importFrom viridis scale_fill_viridis
#' @export
scale_edge_fill_viridis <- function (..., alpha = 1, begin = 0, end = 1, direction = 1,
                                       discrete = FALSE, option = "D", aesthetics = 'edge_fill') {
  scale_fill_viridis(..., alpha = alpha, begin = begin, end = end, direction = direction,
                       discrete = discrete, option = option, aesthetics = aesthetics)
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_steps
#' @export
scale_edge_fill_steps <- function(..., low = "#132B43", high = "#56B1F7", space = "Lab",
                                    na.value = "grey50", guide = "edge_coloursteps",
                                    aesthetics = "edge_fill") {
  scale_fill_steps(..., low = low, high = high, space = space, na.value = na.value,
                     guide = guide, aesthetics = aesthetics)
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_steps2
#' @export
scale_edge_fill_steps2 <- function(..., low = muted("red"), mid = "white", high = muted("blue"),
                                     midpoint = 0, space = "Lab", na.value = "grey50",
                                     guide = "edge_coloursteps",
                                     aesthetics = "edge_fill") {
  scale_fill_steps2(..., low = low, mid = mid, high = high, midpoint = midpoint, space = space,
                      na.value = na.value, guide = guide, aesthetics = aesthetics)
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_stepsn
#' @export
scale_edge_fill_stepsn <- function(..., colours, values = NULL, space = "Lab", na.value = "grey50",
                                     guide = "edge_coloursteps", aesthetics = "edge_fill", colors) {
  scale_fill_stepsn(..., colours = colours, values = values, space = space,
                      na.value = na.value, guide = guide, aesthetics = aesthetics,
                      colors = colors)
}
#' @rdname scale_edge_fill
#'
#' @inheritParams ggplot2::scale_fill_fermenter
#' @export
scale_edge_fill_fermenter <- function(..., type = "seq", palette = 1, direction = -1,
                                        na.value = "grey50", guide = "edge_coloursteps",
                                        aesthetics = "edge_fill") {
  scale_fill_fermenter(..., type = type, palette = palette, direction = direction,
                         na.value = na.value, guide = guide, aesthetics = aesthetics)
}
#' @rdname scale_edge_fill
#'
#' @export
scale_edge_fill_continuous <- scale_edge_fill_gradient
#' @rdname scale_edge_fill
#'
#' @export
scale_edge_fill_discrete <- scale_edge_fill_hue
#' @rdname scale_edge_fill
#'
#' @export
scale_edge_fill_binned <- scale_edge_fill_steps

#' Create a connection extractor function
#'
#' Connections within the ggraph terminology are links between nodes that are
#' not part of the network structure itself. In that sense connections do not
#' affect the layout calculation in any way and will not be drawn by the
#' standard `geom_edge_*` functions. A connection does not need to only be
#' defined by a start and end node, but can include intermediary nodes.
#' `get_con` helps in creating connection data by letting you specify start
#' and end nodes and automatically finds the shortest path within the graph
#' structure that connects the given points. If this is not what is needed it is
#' also possible to supply a list of vectors giving node indices that define a
#' connection.
#'
#' @param from,to The index of the start and end nodes for the connections
#'
#' @param paths A list of integer vectors giving the index of nodes defining
#' connections
#'
#' @param ... Additional information to be added to the final data output
#'
#' @param weight An expression to be evaluated on the edge data to provide
#' weights for the shortest path calculations
#'
#' @inheritParams igraph::shortest_paths
#'
#' @return A function that takes a layout_ggraph object and returns the given
#' connections
#'
#' @family extractors
#'
#' @export
get_con <- function(from = integer(), to = integer(), paths = NULL, ..., weight = NULL, mode = 'all') {
  if (length(from) != length(to)) {
    stop('from and to must be of equal length')
  }
  function(layout) {
    if (length(from) == 0) {
      return(NULL)
    }
    connections <- collect_connections(
      layout = layout, from = from, to = to,
      weight = {
        {
          weight
        }
      }, mode = mode
    )
    nodes <- as.data.frame(layout, stringsAsFactors = FALSE)[unlist(connections), ]
    nodes$con.id <- rep(seq_along(connections), lengths(connections))
    if (!is.null(paths)) {
      extra <- as.data.frame(layout, stringsAsFactors = FALSE)[unlist(paths), ]
      extra$con.id <- rep(
        seq_along(paths) + length(connections),
        lengths(paths)
      )
      nodes <- rbind_dfs(list(nodes, extra))
    }
    nodes <- do.call(
      cbind,
      c(
        list(nodes),
        lapply(list(...), function(x) rep_len(x, length(from))[nodes$con.id]),
        list(stringsAsFactors = FALSE)
      )
    )
    attr(nodes, 'type') <- 'connection_ggraph'
    nodes
  }
}
#' Internal data extractors
#'
#' These functions exists for supporting different data structures. There is no
#' need to call these directly
#'
#' @param layout The layout data
#'
#' @param from,to A numeric vector giving the indexes of the start and end nodes
#'
#' @param ... Additional parameters passed on to the specific method
#'
#' @keywords internal
#' @export
#' @rdname internal_extractors
#' @name internal_extractors
collect_connections <- function(layout, from, to, ...) {
  UseMethod('collect_connections', layout)
}
collect_connections.default <- function(layout, ...) {
  stop('Don\'t know how to get connections from an object of class ', class(layout))
}

#' Place nodes on a line or circle
#'
#' This layout puts all nodes on a line, possibly sorted by a node attribute. If
#' `circular = TRUE` the nodes will be laid out on the unit circle instead.
#' In the case where the `sort.by` attribute is numeric, the numeric values
#' will be used as the x-position and it is thus possible to have uneven spacing
#' between the nodes.
#'
#' @param graph An `tbl_graph` object
#'
#' @param circular Logical. Should the layout be transformed to a circular
#' representation. Defaults to `FALSE`.
#'
#' @param sort.by The name of a node variable to sort the nodes by.
#'
#' @param use.numeric Logical. Should a numeric sort.by attribute be used as the
#' actual x-coordinates in the layout. May lead to overlapping nodes. Defaults
#' to FALSE
#'
#' @param offset If `circular = TRUE`, where should it begin. Defaults to
#' `pi/2` which is equivalent to 12 o'clock.
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom igraph gorder
#'
layout_tbl_graph_linear <- function(graph, circular, sort.by = NULL, use.numeric = FALSE, offset = pi / 2) {
  sort.by <- enquo(sort.by)
  sort.by <- eval_tidy(sort.by, .N())
  if (!is.null(sort.by)) {
    if (is.numeric(sort.by) && use.numeric) {
      x <- sort.by
    } else {
      x <- order(order(sort.by))
    }
  } else {
    x <- seq_len(gorder(graph))
  }
  nodes <- new_data_frame(list(x = x, y = 0))
  if (circular) {
    radial <- radial_trans(
      r.range = rev(range(nodes$y)),
      a.range = range(nodes$x),
      offset = offset
    )
    coords <- radial$transform(nodes$y, nodes$x)
    nodes$x <- coords$x
    nodes$y <- coords$y
  }
  extra_data <- as_tibble(graph, active = 'nodes')
  warn_dropped_vars(nodes, extra_data)
  nodes <- cbind(nodes, extra_data[, !names(extra_data) %in% names(nodes), drop = FALSE])
  nodes$circular <- circular
  nodes
}

#' Place nodes in a Hive Plot layout
#'
#' Hive plots were invented by Martin Krzywinski as a perceptually uniform and
#' scalable alternative to standard node-edge layouts. In hive plots nodes are
#' positioned on axes radiating out from a center based on their own information
#' e.g. membership of a class, size of neighborhood, etc. Edges are then drawn
#' between nodes as bezier curves. As the placement of nodes is not governed by
#' convoluted algorithms but directly reflects the qualities of the nodes itself
#' the resulting plot can be easier to interpret as well as compare to other
#' graphs.
#'
#' @details
#' In order to be able to draw all edges without edges crossing axes you should
#' not assign nodes to axes based on a variable with more than three levels.
#'
#' @param graph An `tbl_graph` object
#'
#' @param axis The node attribute to use for assigning nodes to axes
#'
#' @param axis.pos The relative distance to the prior axis. Default
#' (`NULL`) places axes equidistant.
#'
#' @param sort.by The node attribute to use for placing nodes along their axis.
#' Defaults (`NULL`) places nodes sequentially.
#'
#' @param divide.by An optional node attribute to subdivide each axis by.
#'
#' @param divide.order The order the axis subdivisions should appear in
#'
#' @param normalize Logical. Should axis lengths be equal or reflect the number
#' of nodes in each axis. Defaults to `TRUE`.
#'
#' @param center.size The size of the blank center, that is, the start position
#' of the axes.
#'
#' @param divide.size The distance between subdivided axis segments.
#'
#' @param use.numeric Logical, If the `sort.by` attribute is numeric,
#' should these values be used directly in positioning the nodes along the axes.
#' Defaults to `FALSE` which sorts the numeric values and positions them
#' equidistant from each other.
#'
#' @param offset Change the overall rotation of the hive plot by changing the
#' offset of the first axis.
#'
#' @param split.axes Should axes be split to show edges between nodes on the
#' same axis? One of:
#' \describe{
#'   \item{`'none'`}{Do not split axes and show in-between edges}
#'   \item{`'loops'`}{Only split axes that contain in-between edges}
#'   \item{`'all'`}{Split all axes}
#' }
#'
#' @param split.angle The angular distance between the two axes resulting from a
#' split.
#'
#' @param circular Ignored.
#'
#' @return A data.frame with the columns `x`, `y`, `r`,
#' `center_size`, `split`, `axis`, `section`, `angle`,
#' `circular` as well as any information stored as node variables in the
#' tbl_graph object.
#'
#' @references
#' Krzywinski, M., Birol, I., Jones, SJM., and Marra, MA. (2012). *Hive
#' plots-rational approach to visualizing networks*. Brief Bioinform 13 (5):
#' 627-644. https://doi.org/10.1093/bib/bbr069
#'
#' <http://www.hiveplot.net>
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom igraph gorder vertex_attr gsize induced_subgraph add_vertices E ends add_edges delete_edges %--% edge_attr
#' @importFrom utils tail
layout_tbl_graph_hive <- function(graph, axis, axis.pos = NULL, sort.by = NULL, divide.by = NULL, divide.order = NULL, normalize = TRUE, center.size = 0.1, divide.size = 0.05, use.numeric = FALSE, offset = pi / 2, split.axes = 'none', split.angle = pi / 6, circular = FALSE) {
  axis <- enquo(axis)
  axis <- eval_tidy(axis, .N())
  sort.by <- enquo(sort.by)
  sort.by <- eval_tidy(sort.by, .N())
  divide.by <- enquo(divide.by)
  divide.by <- eval_tidy(divide.by, .N())

  axes <- split(seq_len(gorder(graph)), axis)
  if (is.null(axis.pos)) {
    axis.pos <- rep(1, length(axes))
  } else {
    if (length(axis.pos) != length(axes)) {
      warning('Number of axes not matching axis.pos argument. Recycling as needed')
      axis.pos <- rep(axis.pos, length.out = length(axes))
    }
  }
  axis.pos <- -cumsum(axis.pos)
  axis.pos <- c(0, axis.pos[-length(axis.pos)]) / -tail(axis.pos, 1) * 2 * pi + offset
  if (use.numeric) {
    if (is.null(sort.by) || !is.numeric(sort.by)) {
      stop('sort.by must be a numeric vertex attribute when use.numeric = TRUE')
    }
    numeric.range <- range(sort.by)
  }
  if (normalize) {
    normalize_to <- rep(1, length(axes))
  } else {
    normalize_to <- lengths(axes) / max(lengths(axes))
  }
  node.pos <- Map(function(nodes, axis_length, axis, angle) {
    if (length(nodes) == 0) {
      return(new_data_frame())
    }
    split_axis <- switch(
      split.axes,
      all = TRUE,
      loops = gsize(induced_subgraph(graph, nodes)) > 0,
      none = FALSE,
      stop('Unknown split argument. Use "all", "loops" or "none"')
    )
    node_div <- axis_length / length(nodes)
    if (is.null(divide.by)) {
      node_split <- list(`1` = nodes)
    } else {
      if (use.numeric) {
        stop('Cannot divide axis while use.numeric = TRUE')
      }
      node_split <- split(nodes, divide.by[nodes])
      if (!is.null(divide.order)) {
        if (!all(divide.order %in% names(node_split))) {
          stop('All ', divide.by, ' levels must be present in divide.order')
        }
        node_split <- node_split[order(match(names(node_split), divide.order))]
      }
    }
    node_pos <- lapply(node_split, function(nodes) {
      if (length(nodes) == 0) {
        return(numeric())
      }
      if (is.null(sort.by)) {
        pos <- match(seq_along(nodes), order(nodes)) - 1
        pos <- pos * node_div
      } else {
        pos <- sort.by[nodes]
        if (use.numeric) {
          if (!is.numeric(pos)) {
            stop('sort.by must contain numeric data when use.numeric = TRUE')
          }
          if (normalize) {
            if (diff(range(pos)) == 0) {
              pos <- rep(0.5, length.out = length(pos))
            } else {
              pos <- (pos - min(pos)) / diff(range(pos))
            }
          } else {
            pos <- (pos - numeric.range[1]) / diff(numeric.range)
          }
        } else {
          pos <- match(seq_along(pos), order(pos)) - 1
          pos <- pos * node_div
        }
      }
      pos
    })
    node_pos <- Reduce(function(l, r) {
      append(l, list(r + node_div + divide.size + max(l[[length(l)]])))
    }, x = node_pos[-1], init = node_pos[1])
    node_pos <- unlist(node_pos) + center.size

    new_data_frame(list(
      node = nodes,
      r = node_pos[match(nodes, unlist(node_split))],
      center_size = center.size,
      split = split_axis,
      axis = axis,
      section = rep(names(node_split), lengths(node_split))[match(nodes, unlist(node_split))],
      angle = angle,
      circular = FALSE
    ))
  }, nodes = axes, axis_length = normalize_to, axis = names(axes), angle = axis.pos)
  for (i in seq_along(node.pos)) {
    if (nrow(node.pos[[i]]) > 0 && node.pos[[i]]$split[1]) {
      n_new_nodes <- nrow(node.pos[[i]])
      new_node_start <- gorder(graph) + 1
      extra_nodes <- node.pos[[i]]
      extra_nodes$node <- seq(new_node_start, length.out = n_new_nodes)
      vattr <- lapply(vertex_attr(graph), `[`, i = node.pos[[i]]$node)
      graph <- add_vertices(graph, n_new_nodes, attr = vattr)

      loop_edges <- E(graph)[node.pos[[i]]$node %--% node.pos[[i]]$node]
      if (length(loop_edges) != 0) {
        loop_edges_ends <- ends(graph, loop_edges, names = FALSE)
        correct_order_ends <- node.pos[[i]]$r[match(loop_edges_ends[, 1], node.pos[[i]]$node)] <
          node.pos[[i]]$r[match(loop_edges_ends[, 2], node.pos[[i]]$node)]
        loop_edges_ends <- new_data_frame(list(
          from = ifelse(correct_order_ends, loop_edges_ends[, 1], loop_edges_ends[, 2]),
          to = ifelse(correct_order_ends, loop_edges_ends[, 2], loop_edges_ends[, 1])
        ))
        loop_edges_ends$to <- extra_nodes$node[match(loop_edges_ends$to, node.pos[[i]]$node)]
        loop_edges_ends <- matrix(c(
          ifelse(correct_order_ends, loop_edges_ends$from, loop_edges_ends$to),
          ifelse(correct_order_ends, loop_edges_ends$to, loop_edges_ends$from)
        ), nrow = 2, byrow = TRUE)
        eattr <- lapply(edge_attr(graph), `[`, i = as.numeric(loop_edges))
        graph <- add_edges(graph, as.vector(loop_edges_ends), attr = eattr)
        graph <- delete_edges(graph, as.numeric(loop_edges))
      }

      node_correction <- unlist(lapply(node.pos[-i], function(ax) {
        correct <- if (ax$angle[1] < node.pos[[i]]$angle[1]) {
          ax$angle[1] - node.pos[[i]]$angle[1] < -pi
        } else {
          ax$angle[1] - node.pos[[i]]$angle[1] < pi
        }
        if (correct) ax$node
      }))
      if (length(node_correction) != 0) {
        correct_edges <- E(graph)[node.pos[[i]]$node %--% node_correction]
        correct_edges_ends <- ends(graph, correct_edges, names = FALSE)
        new_node_ind <- correct_edges_ends %in% node.pos[[i]]$node
        correct_edges_ends[new_node_ind] <- extra_nodes$node[match(correct_edges_ends[new_node_ind], node.pos[[i]]$node)]
        eattr <- lapply(edge_attr(graph), `[`, i = as.numeric(correct_edges))
        graph <- add_edges(graph, as.vector(t(correct_edges_ends)), attr = eattr)
        graph <- delete_edges(graph, as.numeric(correct_edges))
      }

      node.pos[[i]]$angle <- node.pos[[i]]$angle - split.angle / 2
      extra_nodes$angle <- extra_nodes$angle + split.angle / 2
      node.pos <- append(node.pos, list(extra_nodes))
    }
  }
  node.pos <- lapply(node.pos, function(nodes) {
    if (nrow(nodes) > 0) {
      nodes$x <- nodes$r * cos(nodes$angle)
      nodes$y <- nodes$r * sin(nodes$angle)
    }
    nodes
  })
  node.pos <- rbind_dfs(node.pos)
  node.pos <- node.pos[order(node.pos$node), names(node.pos) != 'node']
  extra_data <- as_tibble(as_tbl_graph(graph), active = 'nodes')
  node.pos <- cbind(node.pos, extra_data[, !names(extra_data) %in% names(node.pos), drop = FALSE])
  attr(node.pos, 'graph') <- as_tbl_graph(graph)
  node.pos
}

#' Draw edges as Arcs
#'
#' This geom is mainly intended for arc linear and circular diagrams (i.e. used
#' together with [layout_tbl_graph_linear()]), though it can be used
#' elsewhere. It draws edges as arcs with a height proportional to the distance
#' between the nodes. Arcs are calculated as beziers. For linear layout the
#' placement of control points are related to the `curvature` argument and
#' the distance between the two nodes. For circular layout the control points
#' are placed on the same angle as the start and end node at a distance related
#' to the distance between the nodes.
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_arc` and `geom_edge_arc0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - **circular**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_arc2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - **group**
#' - **circular**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_arc` and `geom_edge_arc2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#'
#' @param strength The bend of the curve. 1 approximates a halfcircle while 0
#' will give a straight line. Negative number will change the direction of the
#' curve. Only used if `circular = FALSE`.
#'
#' @param fold Logical. Should arcs appear on the same side of the nodes despite
#' different directions. Default to `FALSE`.
#'
#' @param curvature Deprecated. Use `strength` instead.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' # Make a graph with different directions of edges
#' gr <- create_notable('Meredith') %>%
#'   convert(to_directed) %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE)) %>%
#'   activate(edges) %>%
#'   mutate(
#'     class = sample(letters[1:3], n(), replace = TRUE),
#'     switch = sample(c(TRUE, FALSE), n(), replace = TRUE)
#'   ) %>%
#'   reroute(from = to, to = from, subset = switch)
#'
#' ggraph(gr, 'linear') +
#'   geom_edge_arc(aes(alpha = stat(index)))
#'
#' ggraph(gr, 'linear') +
#'   geom_edge_arc2(aes(colour = node.class), strength = 0.6)
#'
#' ggraph(gr, 'linear', circular = TRUE) +
#'   geom_edge_arc0(aes(colour = class))
#' @rdname geom_edge_arc
#' @name geom_edge_arc
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier
#' @export
StatEdgeArc <- ggproto('StatEdgeArc', StatBezier,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    data <- remove_loop(data)
    if (nrow(data) == 0) return(NULL)
    data$group <- make_unique(data$group)
    data2 <- data
    data2$x <- data2$xend
    data2$y <- data2$yend
    create_arc(data, data2, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'circular'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n', 'strength', 'fold', 'curvature')
)
#' @rdname geom_edge_arc
#'
#' @export
geom_edge_arc <- function(mapping = NULL, data = get_edges(),
                          position = 'identity', arrow = NULL, strength = 1,
                          n = 100, fold = FALSE, lineend = 'butt',
                          linejoin = 'round', linemitre = 1,
                          label_colour = 'black', label_alpha = 1,
                          label_parse = FALSE, check_overlap = FALSE,
                          angle_calc = 'rot', force_flip = TRUE,
                          label_dodge = NULL, label_push = NULL,
                          show.legend = NA, ..., curvature) {
  if (!missing(curvature)) {
    .Deprecated(msg = 'The curvature argument has been deprecated in favour of strength')
    strength <- curvature
  }
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(
    x = x, y = y, xend = xend, yend = yend,
    circular = circular, group = edge.id
  ))
  layer(
    data = data, mapping = mapping, stat = StatEdgeArc,
    geom = GeomEdgePath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, linejoin = linejoin,
        linemitre = linemitre, na.rm = FALSE, n = n,
        interpolate = FALSE, strength = strength, fold = fold,
        label_colour = label_colour, label_alpha = label_alpha,
        label_parse = label_parse, check_overlap = check_overlap,
        angle_calc = angle_calc, force_flip = force_flip,
        label_dodge = label_dodge, label_push = label_push, ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier2
#' @export
StatEdgeArc2 <- ggproto('StatEdgeArc2', StatBezier2,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    data <- remove_loop2(data)
    if (nrow(data) == 0) return(NULL)
    data <- data[order(data$group), ]
    data2 <- data[c(FALSE, TRUE), ]
    data <- data[c(TRUE, FALSE), ]
    create_arc(data, data2, params)
  },
  required_aes = c('x', 'y', 'group', 'circular'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n', 'strength', 'fold', 'curvature')
)
#' @rdname geom_edge_arc
#'
#' @export
geom_edge_arc2 <- function(mapping = NULL, data = get_edges('long'),
                           position = 'identity', arrow = NULL, strength = 1,
                           n = 100, fold = FALSE, lineend = 'butt',
                           linejoin = 'round', linemitre = 1,
                           label_colour = 'black', label_alpha = 1,
                           label_parse = FALSE, check_overlap = FALSE,
                           angle_calc = 'rot', force_flip = TRUE,
                           label_dodge = NULL, label_push = NULL,
                           show.legend = NA, ..., curvature) {
  if (!missing(curvature)) {
    .Deprecated(msg = 'The curvature argument has been deprecated in favour of strength')
    strength <- curvature
  }
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(
    x = x, y = y, group = edge.id,
    circular = circular
  ))
  layer(
    data = data, mapping = mapping, stat = StatEdgeArc2,
    geom = GeomEdgePath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, linejoin = linejoin,
        linemitre = linemitre, na.rm = FALSE, n = n,
        interpolate = TRUE, strength = strength, fold = fold,
        label_colour = label_colour, label_alpha = label_alpha,
        label_parse = label_parse, check_overlap = check_overlap,
        angle_calc = angle_calc, force_flip = force_flip,
        label_dodge = label_dodge, label_push = label_push, ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier0
#' @export
StatEdgeArc0 <- ggproto('StatEdgeArc0', StatBezier0,
  setup_data = function(data, params) {
    StatEdgeArc$setup_data(data, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'circular'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'strength', 'fold', 'curvature')
)
#' @rdname geom_edge_arc
#'
#' @export
geom_edge_arc0 <- function(mapping = NULL, data = get_edges(),
                           position = 'identity', arrow = NULL, strength = 1,
                           lineend = 'butt', show.legend = NA, fold = fold, ..., curvature) {
  if (!missing(curvature)) {
    .Deprecated(msg = 'The curvature argument has been deprecated in favour of strength')
    strength <- curvature
  }
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(
    x = x, y = y, xend = xend, yend = yend,
    circular = circular
  ))
  layer(
    data = data, mapping = mapping, stat = StatEdgeArc0,
    geom = GeomEdgeBezier, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, na.rm = FALSE,
        strength = strength, fold = FALSE, ...
      )
    )
  )
}

create_arc <- function(from, to, params) {
  bezier_start <- seq(1, by = 4, length.out = nrow(from))
  from$index <- bezier_start
  to$index <- bezier_start + 3
  data2 <- from
  data3 <- to
  data2$index <- bezier_start + 1
  data3$index <- bezier_start + 2
  node_dist <- sqrt((to$x - from$x)^2 + (to$y - from$y)^2) / 2
  circ <- from$circular
  if (any(circ)) {
    r0 <- sqrt(to$x[circ]^2 + to$y[circ]^2)
    r1 <- sqrt(to$x[circ]^2 + to$y[circ]^2)

    data2$x[circ] <- from$x[circ] * (1 - (node_dist[circ] / r0))
    data2$y[circ] <- from$y[circ] * (1 - (node_dist[circ] / r0))
    data3$x[circ] <- to$x[circ] * (1 - (node_dist[circ] / r1))
    data3$y[circ] <- to$y[circ] * (1 - (node_dist[circ] / r1))
  }
  if (any(!circ)) {
    strength <- pi / 2 * -params$strength
    edge_angle <- atan2(
      to$y[!circ] - from$y[!circ],
      to$x[!circ] - from$x[!circ]
    )
    start_angle <- edge_angle - strength
    end_angle <- edge_angle - pi + strength
    data2$x[!circ] <- data2$x[!circ] + cos(start_angle) * node_dist[!circ]
    data2$y[!circ] <- data2$y[!circ] + sin(start_angle) * node_dist[!circ]
    data3$x[!circ] <- data3$x[!circ] + cos(end_angle) * node_dist[!circ]
    data3$y[!circ] <- data3$y[!circ] + sin(end_angle) * node_dist[!circ]
    if (params$fold) {
      # data2$x[!circ] <- abs(data2$x[!circ]) * sign(params$strength)
      data2$y[!circ] <- abs(data2$y[!circ]) * sign(params$strength)
      # data3$x[!circ] <- abs(data3$x[!circ]) * sign(params$strength)
      data3$y[!circ] <- abs(data3$y[!circ]) * sign(params$strength)
    }
  }
  data <- rbind_dfs(list(from, data2, data3, to))
  data[order(data$index), names(data) != 'index']
}

#' Edge alpha scales
#'
#' This set of scales defines new alpha scales for edge geoms equivalent to the
#' ones already defined by ggplot2. See [ggplot2::scale_alpha()] for
#' more information. The different geoms will know whether to use edge scales or
#' the standard scales so it is not necessary to write `edge_alpha` in
#' the call to the geom - just use `alpha`.
#'
#' @return A ggproto object inheriting from `Scale`
#'
#' @family scale_edge_*
#'
#' @name scale_edge_alpha
#' @rdname scale_edge_alpha
#'
NULL

#' @rdname scale_edge_alpha
#'
#' @inheritParams ggplot2::scale_alpha
#'
#' @importFrom scales rescale_pal
#' @export
scale_edge_alpha <- function(..., range = c(0.1, 1)) {
  continuous_scale('edge_alpha', 'alpha_c', rescale_pal(range), ...)
}

#' @rdname scale_edge_alpha
#'
#' @export
scale_edge_alpha_continuous <- scale_edge_alpha

#' @rdname scale_edge_alpha
#'
#'
#' @inheritParams ggplot2::scale_alpha_discrete
#'
#' @export
scale_edge_alpha_discrete <- function(..., range = c(0.1, 1)) {
  discrete_scale(
    'edge_alpha', 'alpha_d',
    function(n) seq(range[1], range[2], length.out = n), ...
  )
}
#' @rdname scale_edge_alpha
#'
#' @inheritParams ggplot2::scale_alpha_manual
#'
#' @export
scale_edge_alpha_manual <- function(..., values) {
  manual_scale('edge_alpha', values, ...)
}

#' @rdname scale_edge_alpha
#'
#' @inheritParams ggplot2::scale_alpha_identity
#'
#' @importFrom scales identity_pal
#' @export
scale_edge_alpha_identity <- function(..., guide = 'none') {
  sc <- continuous_scale('edge_alpha', 'identity', identity_pal(), ...,
    guide = guide, super = ScaleContinuousIdentity
  )
  sc
}

#' Show nodes as voronoi tiles
#'
#' This geom is equivalent in functionality to [ggforce::geom_voronoi_tile()]
#' and allows for plotting of nodes as tiles from a voronoi tesselation. As with
#' [ggforce::geom_voronoi_tile()] it is possible to restrict the size of the
#' tile to a fixed radius, as well as round corners and expand/contract the
#' tile.
#'
#' @section Aesthetics:
#' `geom_node_voronoi` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - alpha
#' - colour
#' - fill
#' - shape
#' - size
#' - stroke
#' - filter
#'
#' @inheritParams ggforce::geom_voronoi_tile
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x and y are mapped to x and y in
#' the node data and group set to `-1`.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_node_*
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('meredith') %>%
#'   mutate(group = sample(letters[1:4], n(), TRUE))
#'
#' ggraph(gr) +
#'   geom_node_voronoi(aes(fill = group, colour = group), alpha = 0.3) +
#'   geom_edge_link(alpha = 0.3) +
#'   geom_node_point()
#'
#' # Use max.radius to make the tesselation more "node"-like
#' ggraph(gr) +
#'   geom_node_voronoi(aes(fill = group, colour = group), alpha = 0.3, max.radius = 1) +
#'   geom_edge_link(alpha = 0.3) +
#'   geom_node_point()
#' @export
#' @importFrom ggforce GeomShape
#'
geom_node_voronoi <- function(mapping = NULL, data = NULL, position = 'identity',
                              show.legend = NA, bound = NULL, eps = 1e-09,
                              max.radius = NULL, normalize = FALSE,
                              asp.ratio = 1, expand = 0, radius = 0, ...) {
  mapping <- aes_intersect(mapping, aes(x = x, y = y, group = -1))
  layer(
    data = data, mapping = mapping, stat = StatNodeVoronoi, geom = GeomShape,
    position = position, show.legend = show.legend, inherit.aes = FALSE,
    params = list(na.rm = FALSE, bound = bound, eps = eps, max.radius = max.radius,
                  normalize = normalize, asp.ratio = asp.ratio, expand = expand,
                  radius = radius, ...)
  )
}

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatVoronoiTile
#' @export
StatNodeVoronoi <- ggproto('StatNodeVoronoi', StatVoronoiTile,
  finish_layer = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    if (nrow(data) == 0) return(NULL)
    data
  },
  default_aes = aes(filter = TRUE)
)

utils::globalVariables(c(
  'angle',
  'center_size',
  'circular',
  'con.id',
  'direction',
  'edge.id',
  'edge_x',
  'end',
  'from',
  'height',
  'r',
  'r0',
  'section',
  'start',
  'to',
  'width',
  'x',
  'xend',
  'xmin',
  'xmax',
  'y',
  'yend'
))

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
StatFilter <- ggproto('StatFilter', StatIdentity,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    data
  },
  default_aes = aes(filter = TRUE)
)

aes_intersect <- function(aes1, aes2) {
  aes <- c(as.list(aes1), aes2[!names(aes2) %in% names(aes1)])
  class(aes) <- 'uneval'
  aes
}

data_type <- function(data) {
  type <- attr(data, 'type_ggraph')
  if (is.null(type)) {
    if (inherits(data, 'layout_ggraph')) {
      return('node_ggraph')
    } else {
      return('other')
    }
  }
  type
}

# non-orderaltering version of make.unique
make_unique <- function(x, sep = '.') {
  if (!anyDuplicated(x)) return(x)
  groups <- match(x, unique(x))
  suffix <- unsplit(lapply(split(x, groups), seq_along), groups)
  max_chars <- nchar(max(suffix))
  suffix_format <- paste0('%0', max_chars, 'd')
  paste0(x, sep, sprintf(suffix_format, suffix))
}

warn_dropped_vars <- function(layout, data) {
  overlap <- intersect(names(layout), names(data))
  if (length(overlap) > 0 && !identical(as.list(layout[overlap]), as.list(data[overlap]))) {
    warning('Existing variables ', paste(paste0('`', overlap, '`'), collapse = ', '), ' overwritten by layout variables', call. = FALSE)
  }
}

#' Edge shape scales
#'
#' This set of scales defines new shape scales for edge geoms equivalent to the
#' ones already defined by ggplot2. See [ggplot2::scale_shape()] for
#' more information. The different geoms will know whether to use edge scales or
#' the standard scales so it is not necessary to write `edge_shape` in
#' the call to the geom - just use `shape`.
#'
#' @param guide Guide to use for this scale.
#'
#' @return A ggproto object inheriting from `Scale`
#'
#' @family scale_edge_*
#'
#' @name scale_edge_shape
#' @rdname scale_edge_shape
#'
NULL

#' @rdname scale_edge_shape
#'
#' @inheritParams ggplot2::scale_shape
#'
#' @importFrom scales shape_pal
#' @export
scale_edge_shape <- function(..., solid = TRUE) {
  discrete_scale('edge_shape', 'shape_d', shape_pal(solid), ...)
}
#' @rdname scale_edge_shape
#'
#' @export
scale_edge_shape_discrete <- scale_edge_shape
#' @rdname scale_edge_shape
#'
#' @export
scale_edge_shape_continuous <- function(...) {
  stop('A continuous variable can not be mapped to shape', call. = FALSE)
}
#' @rdname scale_edge_shape
#'
#' @inheritParams ggplot2::scale_shape_manual
#'
#' @export
scale_edge_shape_manual <- function(..., values) {
  manual_scale('edge_shape', values, ...)
}
#' @rdname scale_edge_shape
#'
#' @inheritParams ggplot2::scale_shape_identity
#'
#' @importFrom scales identity_pal
#' @export
scale_edge_shape_identity <- function(..., guide = 'none') {
  sc <- discrete_scale('edge_shape', 'identity', identity_pal(), ...,
    guide = guide, ScaleDiscreteIdentity
  )
  sc
}

#' Create a grid of small multiples by node and/or edge attributes
#'
#' This function is equivalent to [ggplot2::facet_grid()] in that it
#' allows for building a grid of small multiples where rows and columns
#' correspond to a specific data value. While [ggplot2::facet_grid()]
#' could be used it would lead to unexpected results as it is not possible to
#' specify whether you are referring to a node or an edge attribute. Furthermore
#' [ggplot2::facet_grid()] will draw edges in panels even though the
#' panel does not contain both terminal nodes. `facet_graph` takes care of
#' all of these issues, allowing you to define which data type the rows and
#' columns are referencing as well as filtering the edges based on the nodes in
#' each panel (even when nodes are not drawn).
#'
#' @param row_type,col_type Either `'node'` or `'edge'`. Which data
#' type is being facetted in the rows and columns. Default is to facet on nodes
#' column wise and on edges row wise.
#'
#' @inheritParams ggplot2::facet_grid
#'
#' @family ggraph-facets
#'
#' @examples
#' library(tidygraph)
#' gr <- as_tbl_graph(highschool) %>%
#'   mutate(popularity = as.character(cut(centrality_degree(mode = 'in'),
#'     breaks = 3,
#'     labels = c('low', 'medium', 'high')
#'   )))
#' ggraph(gr) +
#'   geom_edge_link() +
#'   geom_node_point() +
#'   facet_graph(year ~ popularity)
#' @export
#'
facet_graph <- function(facets, row_type = 'edge', col_type = 'node',
                        margins = FALSE, scales = 'fixed', space = 'fixed',
                        shrink = TRUE, labeller = 'label_value', as.table = TRUE,
                        switch = NULL, drop = TRUE) {
  facet <- facet_grid(facets,
    margins = margins, scales = scales,
    space = space, shrink = shrink, labeller = labeller,
    as.table = as.table, switch = switch, drop = drop
  )
  seed <- sample(.Machine$integer.max, 1L)
  facet$params$rows[] <- lapply(seq_along(facet$params$rows), function(i) {
    rlang::quo(withr::with_seed(seed, !!facet$params$rows[[i]]))
  })
  facet$params$cols[] <- lapply(seq_along(facet$params$cols), function(i) {
    rlang::quo(withr::with_seed(seed, !!facet$params$cols[[i]]))
  })

  ggproto(NULL, FacetGraph,
    shrink = shrink,
    params = c(facet$params, list(
      row_type = row_type,
      col_type = col_type
    ))
  )
}

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
FacetGraph <- ggproto('FacetGraph', FacetGrid,
  compute_layout = function(data, params) {
    plot_data <- data[[1]]
    data <- split(data, vapply(data, data_type, character(1)))

    rows <- params$rows
    cols <- params$cols
    row_data <- switch(
      params$row_type,
      node = data$node_ggraph,
      edge = data$edge_ggraph,
      stop('row_type must be either "node" or "edge"', call. = FALSE)
    )
    col_data <- switch(
      params$col_type,
      node = data$node_ggraph,
      edge = data$edge_ggraph,
      stop('col_type must be either "node" or "edge"', call. = FALSE)
    )

    base_rows <- combine_vars(row_data, params$plot_env, rows, drop = params$drop)
    if (!params$as.table) {
      rev_order <- function(x) factor(x, levels = rev(ulevels(x)))
      base_rows[] <- lapply(base_rows, rev_order)
    }
    base_cols <- combine_vars(col_data, params$plot_env, cols, drop = params$drop)
    base <- df.grid(
      as.data.frame(base_rows, stringsAsFactors = FALSE),
      as.data.frame(base_cols, stringsAsFactors = FALSE)
    )

    # Add margins
    base <- reshape_add_margins(base, list(names(rows), names(cols)), params$margins)
    base <- unique(base)

    # Create panel info dataset
    panel <- id(base, drop = TRUE)
    panel <- factor(panel, levels = seq_len(attr(panel, 'n')))

    rows <- if (is.null(names(rows))) 1L else id(base[names(rows)], drop = TRUE)
    cols <- if (is.null(names(cols))) 1L else id(base[names(cols)], drop = TRUE)

    panels <- new_data_frame(c(list(
      PANEL = panel, ROW = rows, COL = cols), base
    ))
    panels <- panels[order(panels$PANEL), , drop = FALSE]
    rownames(panels) <- NULL

    panels$SCALE_X <- if (params$free$x) panels$COL else 1L
    panels$SCALE_Y <- if (params$free$y) panels$ROW else 1L

    node_placement <- if (nrow(base_cols) == 0) {
      list(`1` = plot_data$ggraph_index)
    } else {
      if (params$row_type == 'edge') params$rows <- NULL
      if (params$col_type == 'edge') params$cols <- NULL
      node_map <- FacetGrid$map_data(plot_data, panels, params)
      node_map <- expand_facet_map(node_map, panels)
      node_map <- node_map[order(node_map$.ggraph.index), , drop = FALSE]
      split(node_map$.ggraph.index, node_map$PANEL)
    }

    attr(panels, 'node_placement') <- node_placement
    panels
  },
  map_data = function(data, layout, params) {
    switch(
      data_type(data),
      edge_ggraph = {
        if (params$row_type == 'node') params$rows <- NULL
        if (params$col_type == 'node') params$cols <- NULL
        edge_map <- FacetGrid$map_data(data, layout, params)
        edge_map <- expand_facet_map(edge_map, layout)
        edge_map <- Map(function(map, nodes) {
          map[map$from %in% nodes & map$to %in% nodes, , drop = FALSE]
        }, map = split(edge_map, edge_map$PANEL), nodes = attr(layout, 'node_placement'))
        rbind_dfs(edge_map)
      },
      node_ggraph = {
        node_map <- lapply(attr(layout, 'node_placement'), function(nodes) {
          data[data$.ggraph.index %in% nodes, , drop = FALSE]
        })
        panel <- rep(seq_along(node_map), vapply(node_map, nrow, numeric(1)))
        node_map <- rbind_dfs(node_map)
        node_map$PANEL <- factor(panel, levels = levels(layout$PANEL))
        node_map
      }, {
        FacetGrid$map_data(data, layout, params)
      }
    )
  }
)

expand_facet_map <- function(map, layout) {
  ncols <- max(layout$COL)
  nrows <- max(layout$ROW)
  if (!(ncols == 1 && nrows == 1)) {
    map$PANEL[map$PANEL == -1] <- 1
  }
  if (ncols != 1) {
    data_cols <- unique(layout$COL[layout$PANEL %in% map$PANEL])
    if (length(data_cols) == 1) {
      map <- lapply(split(map, as.integer(map$PANEL)), function(data) {
        row <- layout$ROW[layout$PANEL == data$PANEL[1]]
        panels <- layout$PANEL[layout$ROW == row]
        data_expand <- data[rep(seq_len(nrow(data)), length(panels)), , drop = FALSE]
        data_expand$PANEL <- rep(panels, each = nrow(data))
        data_expand
      })
      map <- rbind_dfs(map)
    }
  }
  if (nrows != 1) {
    data_rows <- unique(layout$ROW[layout$PANEL %in% map$PANEL])
    if (length(data_rows) == 1) {
      map <- lapply(split(map, as.integer(map$PANEL)), function(data) {
        col <- layout$COL[layout$PANEL == data$PANEL[1]]
        panels <- layout$PANEL[layout$COL == col]
        data_expand <- data[rep(seq_len(nrow(data)), length(panels)), , drop = FALSE]
        data_expand$PANEL <- rep(panels, each = nrow(data))
        data_expand
      })
      map <- rbind_dfs(map)
    }
  }
  map$PANEL <- factor(map$PANEL, levels = levels(layout$PANEL))
  map
}
df.grid <- function(a, b) {
  if (is.null(a) || nrow(a) == 0) {
    return(b)
  }
  if (is.null(b) || nrow(b) == 0) {
    return(a)
  }
  indexes <- expand.grid(i_a = seq_len(nrow(a)), i_b = seq_len(nrow(b)))
  grid <- cbind(a[indexes$i_a, , drop = FALSE], b[indexes$i_b, , drop = FALSE])
  attr(grid, 'row.names') <- .set_row_names(nrow(grid))
  grid
}
ulevels <- function(x) {
  if (is.factor(x)) {
    x <- addNA(x, TRUE)
    factor(levels(x), levels(x), exclude = NULL)
  } else {
    sort(unique(x))
  }
}

#' Show nodes as thick arcs
#'
#' This geom is equivalent in functionality to [ggforce::geom_arc_bar()]
#' and allows for plotting of nodes as arcs with an inner and outer radius
#' scaled by the coordinate system. Its main use is currently in sunburst plots
#' as created with circular partition layouts
#'
#' @section Aesthetics:
#' `geom_node_point` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overridden.
#'
#' - **x0**
#' - **y0**
#' - **r0**
#' - **r**
#' - **start**
#' - **end**
#' - alpha
#' - colour
#' - fill
#' - shape
#' - size
#' - stroke
#' - filter
#'
#' @inheritParams ggforce::geom_circle
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x and y are mapped to x0 and y0 in
#' the node data.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_node_*
#'
#' @examples
#' require(tidygraph)
#' gr <- tbl_graph(flare$vertices, flare$edges)
#' ggraph(gr, 'partition', circular = TRUE, weight = size) +
#'   geom_node_arc_bar()
#' @export
#' @importFrom ggforce GeomArcBar
#'
geom_node_arc_bar <- function(mapping = NULL, data = NULL, position = 'identity',
                              show.legend = NA, ...) {
  mapping <- aes_intersect(mapping, aes(x0 = 0, y0 = 0, r0 = r0,
                                        r = r, start = start,
                                        end = end))
  layer(
    data = data, mapping = mapping, stat = StatNodeArcBar, geom = GeomArcBar,
    position = position, show.legend = show.legend, inherit.aes = FALSE,
    params = list(na.rm = FALSE, ...)
  )
}

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatArcBar
#' @export
StatNodeArcBar <- ggproto('StatNodeArcBar', StatArcBar,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    if (nrow(data) == 0) return(NULL)
    data
  },
  default_aes = aes(filter = TRUE)
)

#' Create a ggraph plot
#'
#' This function is the equivalent of [ggplot2::ggplot()] in ggplot2.
#' It takes care of setting up the plot object along with creating the layout
#' for the plot based on the graph and the specification passed in.
#' Alternatively a layout can be prepared in advance using
#' `create_layout` and passed as the data argument. See *Details* for
#' a description of all available layouts.
#'
#' @details
#' Following is a short description of the different layout types available in
#' ggraph. Each layout is further described in its own help pages. Any type of
#' regular graph/network data can be represented as a tbl_graph object. Because
#' of this the different layouts that can be applied to tbl_graph
#' objects are quite diverse, but not all layouts makes sense to all types of
#' graphs. It is up to the user to understand their data and choose an
#' appropriate layout. For standard node-edge diagrams igraph defines a
#' long range of different layout functions that are all available through the
#' `igraph` layout where the specific layout is specified using the
#' `algorithm` argument. In order to minimize typing all igraph algorithms
#' can also be passed directly into the `layout` argument.
#'
#' Any object that has an appropriate `as_tbl_graph` method can be passed
#' into `ggraph()` and will automatically be converted underneath.
#'
#' \describe{
#'   \item{`auto`}{The default layout. See [layout_tbl_graph_auto()] for further
#'   details}
#'   \item{`igraph`}{Use one of the internal igraph layout algorithms.
#'   The algorithm is specified using the `algorithm` argument. All strings
#'   accepted by the `algorithm` argument can also be supplied directly
#'   into `layout`. See [layout_tbl_graph_igraph()] for further
#'   details}
#'   \item{`dendrogram`}{Lays out the nodes in a tree-like graph as a
#'   dendrogram with leaves set at 0 and parents 1 unit above its tallest child.
#'   See [layout_tbl_graph_dendrogram()] for further details}
#'   \item{`manual`}{Lets the user manually specify the location of each
#'   node. See [layout_tbl_graph_manual()] for further details}
#'   \item{`linear`}{Arranges the nodes linearly or circularly in order to
#'   make an arc diagram. See [layout_tbl_graph_linear()] for further
#'   details}
#'   \item{`matrix`}{Arranges nodes on a diagonal thus preparing it for use with
#'   [geom_edge_point()] to make a matrix plot. See [layout_tbl_graph_matrix()]
#'   for further details}
#'   \item{`treemap`}{Creates a treemap from the graph, that is, a
#'   space-filing subdivision of rectangles showing a weighted hierarchy. See
#'   [layout_tbl_graph_treemap()] for further details}
#'   \item{`circlepack`}{Creates a layout showing a hierarchy as circles
#'   within circles. Conceptually equal to treemaps. See
#'   [layout_tbl_graph_circlepack()] for further details}
#'   \item{`partition`}{Create icicle or sunburst charts, where each layer
#'   subdivides the division given by the preceding layer. See
#'   [layout_tbl_graph_partition()] for further details}
#'   \item{`hive`}{Positions nodes on axes spreading out from the center
#'   based on node attributes. See [layout_tbl_graph_hive()] for further
#'   details}
#' }
#'
#' Alternatively a matrix or a data.frame can be provided to the `layout`
#' argument. In the former case the first column will be used as x coordinates
#' and the second column will by used as y coordinates, further columns are
#' dropped. In the latter case the data.frame is used as the layout table and
#' must thus contain a numeric x and y column.
#'
#' Lastly a function can be provided to the `layout` argument. It will be called
#' with the graph object as its first argument and any additional argument
#' passed into `ggraph()`/`create_layout()`. The function must return either a
#' data.frame or an object coercible to one and have an `x` and `y` column, or
#' an object coercible to a `tbl_graph`. In the latter case the node data is
#' extracted and used as layout (and must thus contain an `x` and `y` column)
#' and the graph will be added as the `graph` attribute.
#'
#' @param graph The object containing the graph. See *Details* for a list
#' of supported classes. Or a `layout_ggraph` object as returned from
#' `create_layout` in which case all subsequent arguments is ignored.
#'
#' @param layout The type of layout to create. Either a valid string, a
#' function, a matrix, or a data.frame (see Details)
#'
#' @param circular Should the layout be transformed into a radial
#' representation. Only possible for some layouts. Defaults to `FALSE`
#'
#' @param ... Arguments passed on to the layout function.
#'
#' @return For `ggraph()` an object of class gg onto which layers, scales,
#'   etc. can be added. For `create_layout()` an object inheriting from
#'   `layout_ggraph`. `layout_ggraph` itself inherits from
#'   `data.frame` and can be considered as such. The data.frame contains
#'   the node positions in the `x` and `y` column along with
#'   additional columns generated by the specific layout, as well as node
#'   parameters inherited from the graph. Additional information is stored as
#'   attributes to the data.frame. The original graph object is stored in the
#'   `graph` attribute and the `circular` attribute contains a logical
#'   indicating whether the layout has been transformed to a circular
#'   representation.
#'
#' @keywords layout network graph hierarchy visualisation
#'
#' @seealso [get_edges()] for extracting edge information from the
#' layout and [get_con()] for extracting path information.
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('bull')
#' layout <- create_layout(gr, layout = 'igraph', algorithm = 'kk')
#' @export
#'
ggraph <- function(graph, layout = 'auto', ...) {
  envir <- parent.frame()
  p <- ggplot(data = create_layout(graph, layout, ...), environment = envir) +
    th_no_axes()
  class(p) <- c('ggraph', class(p))
  p
}
#' @export
ggplot_build.ggraph <- function(plot) {
  .register_graph_context(attr(plot$data, 'graph'), free = TRUE)
  NextMethod()
}

#' Draw edges as diagonals
#'
#' This geom draws edge loops (edges starting and ending at the same node).
#' Loops are drawn as bezier curves starting and ending at the position of the
#' node and with control points protruding at an angle and in a direction
#' specified in the call. As the start and end node is always the same no *2
#' method is provided. Loops can severely clutter up your visualization which is
#' why they are decoupled from the other edge drawings. Only plot them if they
#' are of importance. If the graph doesn't contain any loops the geom adds
#' nothing silently.
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_loop` and `geom_edge_loop0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - **from**
#' - **to**
#' - **span** *90*
#' - **direction** *45*
#' - **strength** *1*
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_loop` furthermore takes the following aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' gr <- as_tbl_graph(
#'   data.frame(from = c(1, 1, 2, 2, 3, 3, 3), to = c(1, 2, 2, 3, 3, 1, 2))
#' )
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_loop(aes(alpha = stat(index))) +
#'   geom_edge_fan(aes(alpha = stat(index)))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_loop0() +
#'   geom_edge_fan0()
#' @rdname geom_edge_loop
#' @name geom_edge_loop
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier
#' @export
StatEdgeLoop <- ggproto('StatEdgeLoop', StatBezier,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    if (nrow(data) == 0) return(NULL)
    data <- data[data$from == data$to, ]
    data$group <- make_unique(data$group)
    if (nrow(data) != 0) {
      create_loops(data, params)
    } else {
      NULL
    }
  },
  required_aes = c('x', 'y', 'from', 'to', 'span', 'direction', 'strength'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n')
)
#' @rdname geom_edge_loop
#'
#' @export
geom_edge_loop <- function(mapping = NULL, data = get_edges(),
                           position = 'identity', arrow = NULL, n = 100,
                           lineend = 'butt', linejoin = 'round', linemitre = 1,
                           label_colour = 'black', label_alpha = 1,
                           label_parse = FALSE, check_overlap = FALSE,
                           angle_calc = 'rot', force_flip = TRUE,
                           label_dodge = NULL, label_push = NULL,
                           show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(
    x = x, y = y, from = from, to = to, group = edge.id,
    span = 90, direction = 45, strength = 1
  ))
  layer(
    data = data, mapping = mapping, stat = StatEdgeLoop,
    geom = GeomEdgePath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, linejoin = linejoin,
        linemitre = linemitre, na.rm = FALSE, n = n,
        interpolate = FALSE,
        label_colour = label_colour, label_alpha = label_alpha,
        label_parse = label_parse, check_overlap = check_overlap,
        angle_calc = angle_calc, force_flip = force_flip,
        label_dodge = label_dodge, label_push = label_push, ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier0
#' @export
StatEdgeLoop0 <- ggproto('StatEdgeLoop0', StatBezier0,
  setup_data = function(data, params) {
    StatEdgeLoop$setup_data(data, params)
  },
  required_aes = c('x', 'y', 'from', 'to', 'span', 'direction', 'strength'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm')
)
#' @rdname geom_edge_loop
#'
#' @export
geom_edge_loop0 <- function(mapping = NULL, data = get_edges(),
                            position = 'identity', arrow = NULL,
                            lineend = 'butt', show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(
    x = x, y = y, from = from, to = to,
    span = 90, direction = 45, strength = 1
  ))
  layer(
    data = data, mapping = mapping, stat = StatEdgeLoop0,
    geom = GeomEdgeBezier, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(arrow = arrow, lineend = lineend, na.rm = FALSE, ...)
    )
  )
}

create_loops <- function(loops, params) {
  control_angle1 <- loops$direction - loops$span / 2
  control_angle2 <- loops$direction + loops$span / 2
  controls1 <- find_loop_controls(loops, control_angle1)
  controls2 <- find_loop_controls(loops, control_angle2)
  end <- loops
  bezier_start <- seq(1, by = 4, length.out = nrow(loops))
  loops$index <- bezier_start
  controls1$index <- bezier_start + 1
  controls2$index <- bezier_start + 2
  end$index <- bezier_start + 3
  loops <- rbind_dfs(list(loops, controls1, controls2, end))
  loops[order(loops$index), names(loops) != 'index']
}
find_loop_controls <- function(loops, angle) {
  angle <- angle / 360 * 2 * pi
  loops$x <- loops$x + cos(angle) * loops$strength
  loops$y <- loops$y + sin(angle) * loops$strength
  loops
}

#' Place nodes using stress majorisation
#'
#' This layout is related to the stress-minimization algorithm known as
#' Kamada-Kawai (available as the 'kk' layout), but uses another optimization
#' strategy. It generally have better runtime, quality, and stability compared
#' to the Kamada-Kawai layout and is thus generally preferred. The sparse
#' version of the layout have better performance (especially on larger networks)
#' at the expense of layout quality, but will generally outperform many other
#' algorithms for large graphs in both runtime and quality (e.g. the 'drl'
#' layout from igraph).
#'
#' @param graph a tbl_graph object
#' @param weights An expression evaluated on the edge data to provide edge
#' weights for the layout. Currently ignored for the sparse version
#' @param pivots The number of pivot nodes.
#' @param niter number of iterations during stress optimization
#' @param tolerance stopping criterion for stress optimization
#' @param mds should an MDS layout be used as initial layout (default: TRUE)
#' @param bbox constrain dimension of output. Only relevant to determine the
#' placement of disconnected graphs.
#' @param circular ignored
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @references
#' Gansner, E. R., Koren, Y., & North, S. (2004). *Graph drawing by stress
#' majorization.* In International Symposium on Graph Drawing (pp. 239-250). Springer, Berlin, Heidelberg.
#'
#' Ortmann, M. and Klimenta, M. and Brandes, U. (2016). *A Sparse Stress Model.* https://arxiv.org/pdf/1608.08909.pdf
#'
#' @family layout_tbl_graph_*
#'
#' @author The underlying algorithm is implemented in the graphlayouts package
#' by David Schoch
#'
#' @importFrom graphlayouts layout_with_stress
#' @importFrom rlang eval_tidy enquo
#'
layout_tbl_graph_stress <- function(graph, weights = NULL, niter = 500,
                                    tolerance = 1e-4, mds = TRUE, bbox = 50,
                                    circular = FALSE) {
  weights <- eval_tidy(enquo(weights), .E())
  if (is.null(weights)) {
    weights <- NA
  } else {
    weights <- 1 / weights
  }
  xy <- layout_with_stress(graph, weights = weights, iter = niter,
                           tol = tolerance, mds = mds, bbox = bbox)

  nodes <- new_data_frame(list(x = xy[,1],y = xy[,2]))
  nodes$circular <- FALSE
  extra_data <- as_tibble(graph, active = 'nodes')
  warn_dropped_vars(nodes, extra_data)
  nodes <- cbind(nodes, extra_data[, !names(extra_data) %in% names(nodes), drop = FALSE])
  nodes
}
#' @rdname layout_tbl_graph_stress
#' @importFrom graphlayouts layout_with_sparse_stress
layout_tbl_graph_sparse_stress <- function(graph, pivots, weights = NULL,
                                           niter = 500, circular = FALSE) {
  weights <- eval_tidy(enquo(weights))
  if (!is.null(weights)) {
    warning('weights is currently ignored for sparse stress layouts', call. = FALSE)
  }
  xy <- layout_with_sparse_stress(graph, pivots = pivots, weights = weights,
                                  iter = niter)
  nodes <- new_data_frame(list(x = xy[,1], y = xy[,2]))
  nodes$circular <- FALSE
  extra_data <- as_tibble(graph, active = 'nodes')
  warn_dropped_vars(nodes, extra_data)
  nodes <- cbind(nodes, extra_data[, !names(extra_data) %in% names(nodes), drop = FALSE])
  nodes
}

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

#' Pack circles together
#'
#' This function is a direct interface to the circle packing algorithm used by
#' \code{\link{layout_tbl_graph_circlepack}}. It takes a vector of sizes and
#' returns the x and y position of each circle as a two-column matrix.
#'
#' @param areas A vector of circle areas
#'
#' @return A matrix with two columns and the same number of rows as the length
#' of the "areas" vector. The matrix has the following attributes added:
#' "enclosing_radius" giving the radius of the smallest enclosing circle, and
#' "front_chain" giving the terminating members of the front chain (see
#' Wang \emph{et al}. 2006).
#'
#' @references
#' Wang, W., Wang, H. H., Dai, G., & Wang, H. (2006). \emph{Visualization of
#' large hierarchical data by circle packing}. Chi, 517-520.
#'
#' @export
#'
#' @examples
#' library(ggforce)
#' sizes <- sample(10, 100, TRUE)
#'
#' position <- pack_circles(sizes)
#' data <- data.frame(x = position[,1], y = position[,2], r = sqrt(sizes/pi))
#'
#' ggplot() +
#'   geom_circle(aes(x0 = x, y0 = y, r = r), data = data, fill = 'steelblue') +
#'   geom_circle(aes(x0 = 0, y0 = 0, r = attr(position, 'enclosing_radius'))) +
#'   geom_polygon(aes(x = x, y = y),
#'                data = data[attr(position, 'front_chain'), ],
#'                fill = NA,
#'                colour = 'black')
#'
pack_circles <- function(areas) {
    .Call('_ggraph_pack', PACKAGE = 'ggraph', areas)
}

circlePackLayout <- function(parent, weight) {
    .Call('_ggraph_circlePackLayout', PACKAGE = 'ggraph', parent, weight)
}

partitionTree <- function(parent, order, weight, height) {
    .Call('_ggraph_partitionTree', PACKAGE = 'ggraph', parent, order, weight, height)
}

cut_lines <- function(x, y, id, start_width, start_height, end_width, end_height, start_type, end_type) {
    .Call('_ggraph_cut_lines', PACKAGE = 'ggraph', x, y, id, start_width, start_height, end_width, end_height, start_type, end_type)
}

pathAttr <- function(paths, ngroups) {
    .Call('_ggraph_pathAttr', PACKAGE = 'ggraph', paths, ngroups)
}

splitTreemap <- function(parent, order, weight, width, height) {
    .Call('_ggraph_splitTreemap', PACKAGE = 'ggraph', parent, order, weight, width, height)
}

unrooted <- function(parent, order, length, daylight, tol, rotation_mod, maxiter) {
    .Call('_ggraph_unrooted', PACKAGE = 'ggraph', parent, order, length, daylight, tol, rotation_mod, maxiter)
}

#' Edge width scales
#'
#' This set of scales defines width scales for edge geoms. Of all the new edge
#' scales defined in ggraph, this is the only one not having an equivalent in
#' ggplot2. In essence it mimics the use of size in
#' [ggplot2::geom_line()] and related. As almost all edge
#' representations are lines of some sort, edge_width will be used much more
#' often than edge_size. It is not necessary to spell out that it is an edge
#' scale as the geom knows if it is drawing an edge. Just write `width` and
#' not `edge_width` in the call to geoms.
#'
#' @return A ggproto object inheriting from `Scale`
#'
#' @family scale_edge_*
#'
#' @name scale_edge_width
#' @rdname scale_edge_width
#'
NULL

#' @rdname scale_edge_width
#'
#' @inheritParams ggplot2::scale_size_continuous
#'
#' @importFrom scales rescale_pal
#' @export
scale_edge_width_continuous <- function(..., range = c(1, 6)) {
  continuous_scale('edge_width', 'width_c', rescale_pal(range), ...)
}
#' @rdname scale_edge_width
#'
#' @export
scale_edge_width <- scale_edge_width_continuous
#' @rdname scale_edge_width
#'
#' @inheritParams ggplot2::scale_size_discrete
#'
#' @importFrom scales rescale_pal
#' @export
scale_edge_width_discrete <- function(..., range = c(2, 6)) {
  discrete_scale('edge_width', 'width_d', function(n) {
    area <- seq(range[1]^2, range[2]^2, length.out = n)
    sqrt(area)
  }, ...)
}
#' @rdname scale_edge_width
#'
#' @inheritParams ggplot2::scale_size_manual
#'
#' @export
scale_edge_width_manual <- function(..., values) {
  manual_scale('edge_width', values, ...)
}
#' @rdname scale_edge_width
#'
#' @inheritParams ggplot2::scale_size_identity
#'
#' @importFrom scales identity_pal
#' @export
scale_edge_width_identity <- function(..., guide = 'none') {
  sc <- discrete_scale('edge_width', 'identity', identity_pal(), ...,
    guide = guide, super = ScaleDiscreteIdentity
  )
  sc
}

#' Create a fabric layout
#'
#' This layout is a bit unusual in that it shows nodes as horizontal line ranges
#' end edges as evenly spaced vertical spans connecting the nodes. As with the
#' matrix layout the strength comes from better scalability but its use require
#' some experience recognising the patterns that different connectivity features
#' gives rise to. As with matrix layouts the ordering of nodes have huge power
#' over the look of the plot. The `node_rank_fabric()` mimics the default
#' ordering from the original BioFabric implementation, but other ranking
#' algorithms from tidygraph can be used with the `sort.by` argument as well.
#' Fabric layouts tend to become quite wide as the graph grows which is
#' something that should be handled with care - e.g. by only zooming in on a
#' specific region.
#'
#' @param graph An `tbl_graph` object
#'
#' @param circular Ignored
#'
#' @param sort.by An expression providing the sorting of the nodes. If `NULL`
#' the nodes will be ordered by their index in the graph.
#'
#' @param shadow.edges Should shadow edges be shown.
#'
#' @return A data.frame with the columns `x`, `xmin`, `xmax`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#' Further, the edges of the graph will gain a `edge_x` variable giving the
#' horizontal position of the edge as well as a `shadow_edge` variable denoting
#' whether the edge is a shadow edge added by the layout.
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom igraph incident_edges V
#' @importFrom rlang enquo eval_tidy
#'
#' @references
#' BioFabric website: <http://www.biofabric.org>
#'
#' Longabaugh, William J.R. (2012).
#' *Combing the hairball with BioFabric: a new approach for visualization of large networks*.
#' BMC Bioinformatics, 13: 275. <https://doi.org/10.1186/1471-2105-13-275>
#'
layout_tbl_graph_fabric <- function(graph, circular = FALSE, sort.by = NULL, shadow.edges = FALSE) {
  sort.by <- enquo(sort.by)
  sort.by <- eval_tidy(sort.by, .N())
  if (!is.null(sort.by)) {
    pos <- order(order(sort.by))
  } else {
    pos <- seq_len(gorder(graph))
  }

  edges <- as_edgelist(graph, names = FALSE)
  edges <- cbind(pos[edges[,1]], pos[edges[,2]])
  first_node <- pmin(edges[,1], edges[,2])
  second_node <- pmax(edges[,1], edges[,2])
  edge_order <- order(first_node, second_node)

  if (shadow.edges) {
    shadow_order <- order(second_node, first_node)
    edge_order <- split(edge_order, factor(first_node[edge_order], seq_along(pos)))
    shadow_order <- split(shadow_order + length(second_node), factor(second_node[shadow_order], seq_along(pos)))
    edge_order <- unlist(c(shadow_order, edge_order)[matrix(seq_len(length(pos) * 2), nrow = 2, byrow = T)])
    graph <- bind_edges(graph, as_tibble(graph, active = 'edges'))
    shadow <- rep(c(FALSE, TRUE), each = length(first_node))
  } else {
    shadow <- rep_len(FALSE, length(first_node))
  }

  edge_rank <- match(seq_along(edge_order), edge_order)

  node_span <- vapply(incident_edges(graph, V(graph), mode = 'all'), function(e) {
    range(edge_rank[as.integer(e)])
  }, numeric(2))

  nodes <- new_data_frame(list(x = colMeans(node_span), xmin = node_span[1,],
                      xmax = node_span[2,], y = abs(pos - max(pos))))
  extra_data <- as_tibble(graph, active = 'nodes')
  warn_dropped_vars(nodes, extra_data)
  nodes <- cbind(nodes, extra_data[, !names(extra_data) %in% names(nodes), drop = FALSE])
  nodes$circular <- FALSE

  graph <- activate(graph, 'edges')
  graph <- mutate(graph, edge_x = edge_rank, shadow_edge = shadow)
  graph <- activate(graph, 'nodes')
  attr(nodes, 'graph') <- graph
  nodes
}

#' @rdname layout_tbl_graph_fabric
#' @importFrom igraph bfs degree
#' @importFrom tidygraph activate .G arrange
#' @export
node_rank_fabric <- function() {
  graph <- activate(.G(), 'nodes')
  graph <- mutate(graph, node_order_orig = seq_len(n()))
  graph <- arrange(graph, -degree(graph))
  node_order_orig <- pull(graph, node_order_orig)
  graph <- activate(graph, 'edges')
  graph <- arrange(graph, pmin(from, to), pmax(from, to))
  order <- as.integer(bfs(graph, 1, 'all', order = TRUE)$order)
  order <- node_order_orig[order]
  match(seq_along(order), order)
}

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid segmentsGrob polylineGrob gpar
#' @importFrom dplyr %>% group_by_ do ungroup
#' @importFrom ggforce interpolateDataFrame
#' @export
GeomEdgePath <- ggproto('GeomEdgePath', GeomPath,
  draw_panel = function(data, panel_scales, coord, arrow = NULL,
                        lineend = 'butt', linejoin = 'round', linemitre = 1,
                        na.rm = FALSE, interpolate = TRUE,
                        label_colour = 'black', label_alpha = 1, label_parse = FALSE,
                        check_overlap = FALSE, angle_calc = 'none', force_flip = TRUE,
                        label_dodge = NULL, label_push = NULL) {
    if (!anyDuplicated(data$group)) {
      message(
        'geom_edge_path: Each group consists of only one observation. ',
        'Do you need to adjust the group aesthetic?'
      )
    }
    data <- data[order(data$group), , drop = FALSE]
    data$group <- match(data$group, unique(data$group))
    if (interpolate) {
      geometries <- vapply(data, is.geometry, logical(1))
      geom_cols <- data[geometries]
      data <- cbind(interpolateDataFrame(data[!geometries]), geom_cols)
    }
    data <- coord$transform(data, panel_scales)
    zero <- coord$transform(new_data_frame(list(x = 0, y = 0)), panel_scales)
    if (nrow(data) < 2) {
      return(zeroGrob())
    }
    attr <- pathAttr(data, length(unique(data$group)))

    if (all(is.na(data$start_cap))) {
      start_captype <- 'circle'
      start_cap <- NULL
      start_cap2 <- NULL
    } else {
      scap <- data$start_cap[!duplicated(data$group)]
      start_captype <- geo_type(scap)
      start_cap <- geo_width(scap)
      start_cap2 <- geo_height(scap)
      native_start <- grepl('native', as.character(start_cap))
      native_start2 <- grepl('native', as.character(start_cap2))
      if (any(native_start) || any(native_start2)) {
        recalc <- coord$transform(
          new_data_frame(list(x = as.vector(start_cap), y = as.vector(start_cap2))),
          panel_scales
        )
        start_cap[native_start] <- unit(recalc$x - zero$x, 'npc')[native_start]
        start_cap2[native_start2] <- unit(recalc$y - zero$y, 'npc')[native_start2]
      }
    }
    if (all(is.na(data$end_cap))) {
      end_captype <- 'circle'
      end_cap <- NULL
      end_cap2 <- NULL
    } else {
      ecap <- data$end_cap[!duplicated(data$group)]
      end_captype <- geo_type(ecap)
      end_cap <- geo_width(ecap)
      end_cap2 <- geo_height(ecap)
      native_end <- grepl('native', as.character(end_cap))
      native_end2 <- grepl('native', as.character(end_cap2))
      if (any(native_end) || any(native_end2)) {
        recalc <- coord$transform(
          new_data_frame(list(x = as.vector(end_cap), y = as.vector(end_cap2))),
          panel_scales
        )
        end_cap[native_end] <- unit(recalc$x - zero$x, 'native')[native_end]
        end_cap2[native_end2] <- unit(recalc$y - zero$y, 'native')[native_end2]
      }
    }
    if ((is.null(start_cap) && !is.null(end_cap)) ||
      (!is.null(start_cap) && is.null(end_cap))) {
      if (is.null(start_cap)) start_cap <- 0
      if (is.null(end_cap)) end_cap <- 0
    }

    solid_lines <- all(attr$solid)
    constant <- all(attr$constant)
    if (!solid_lines && !constant) {
      stop('geom_edge_path: If you are using dotted or dashed lines',
        ', colour, size and linetype must be constant over the line',
        call. = FALSE
      )
    }
    gp <- gpar(
      col = alpha(data$edge_colour, data$edge_alpha),
      fill = alpha(data$edge_colour, data$edge_alpha),
      lwd = data$edge_width * .pt, lty = data$edge_linetype,
      lineend = lineend, linejoin = linejoin, linemitre = linemitre
    )
    edge_grob <- cappedPathGrob(
      x = data$x, y = data$y, id = data$group, arrow = arrow,
      start.cap = start_cap, start.cap2 = start_cap2, start.captype = start_captype,
      end.cap = end_cap, end.cap2 = end_cap2, end.captype = end_captype,
      default.units = 'native', gp = gp, constant = constant
    )
    if (any(!is.na(data$label))) {
      if (angle_calc == 'none') angle_calc <- 'rot'
      edge_ind <- split(seq_len(nrow(data)), data$group)
      edge_length <- lengths(edge_ind)
      edge_start <- c(0, cumsum(edge_length)[-length(edge_length)]) + 1
      label_pos <- data$label_pos[vapply(edge_ind, head, integer(1), n = 1)]
      label_pos <- 1 + floor((edge_length - 1) * label_pos)
      label_ind <- edge_start + label_pos
      label_data <- data[label_ind, ]
      label_data$label_colour <- if (is.na(label_colour)) {
        label_data$edge_colour
      } else {
        label_colour
      }
      label_data$label_alpha <- if (is.na(label_alpha)) {
        label_data$edge_alpha
      } else {
        label_alpha
      }
      angle_start <- ifelse(label_pos == 1, 1, label_pos - 1)
      angle_end <- ifelse(label_pos == edge_length, edge_length, label_pos + 1)
      label_x0 <- data$x[angle_start + edge_start]
      label_y0 <- data$y[angle_start + edge_start]
      label_x1 <- data$x[angle_end + edge_start]
      label_y1 <- data$y[angle_end + edge_start]
      lab <- as.character(label_data$label)
      if (label_parse) {
        lab_expressions <- sapply(X=lab, FUN=str2expression)
        lab_len <- sapply(X=lab_expressions, FUN=length)
        if (any(lab_len == 0)) {
          lab[lab_len == 0] <- "` `"
          lab <- sapply(X=lab, FUN=str2expression)
        } else {
          lab <- lab_expressions
        }
      }
      label_grob <- textAlongGrob(
        lab, label_data$x, label_data$y,
        default.units = 'native',
        hjust = label_data$hjust, vjust = label_data$vjust,
        rot = label_data$angle,
        gp = gpar(
          col = alpha(label_data$label_colour, label_data$label_alpha),
          fontsize = label_data$label_size * .pt,
          fontfamily = label_data$family,
          fontface = label_data$fontface,
          lineheight = label_data$lineheight
        ),
        check.overlap = check_overlap, rot.type = angle_calc,
        x0 = label_x0, y0 = label_y0, x1 = label_x1, y1 = label_y1,
        force.rot = force_flip, dodge = label_dodge, push = label_push
      )
      gList(edge_grob, label_grob)
    } else {
      edge_grob
    }
  },
  draw_key = function(data, params, size) {
    segmentsGrob(0.1, 0.5, 0.9, 0.5,
      gp = gpar(
        col = alpha(data$edge_colour, data$edge_alpha),
        lwd = data$edge_width * .pt,
        lty = data$edge_linetype, lineend = 'butt'
      ),
      arrow = params$arrow
    )
  },
  handle_na = function(data, params) {
    if (params$interpolate) {
      return(data)
    }
    keep <- function(x) {
      first <- match(FALSE, x, nomatch = 1) - 1
      last <- length(x) - match(FALSE, rev(x), nomatch = 1) + 1
      c(
        rep(FALSE, first),
        rep(TRUE, last - first),
        rep(FALSE, length(x) - last)
      )
    }
    missing <- !stats::complete.cases(
      data[c('x', 'y', 'edge_width', 'edge_colour', 'edge_linetype')]
    )
    kept <- stats::ave(missing, data$group, FUN = keep)
    data <- data[kept, ]
    if (!all(kept) && !params$na.rm) {
      warning('Removed ', sum(!kept), ' rows containing missing values',
        ' (geom_edge_path).',
        call. = FALSE
      )
    }
    data
  },
  default_aes = aes(
    edge_colour = 'black', edge_width = 0.5, edge_linetype = 'solid',
    edge_alpha = NA, start_cap = NA, end_cap = NA, label = NA,
    label_pos = 0.5, label_size = 3.88, angle = 0,
    hjust = 0.5, vjust = 0.5, family = '', fontface = 1,
    lineheight = 1.2
  )
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid gpar segmentsGrob
#' @export
GeomEdgeParallelPath <- ggproto('GeomEdgeParallelPath', GeomEdgePath,
  draw_panel = function(data, panel_scales, coord, arrow = NULL, sep = unit(2, 'mm'),
                        lineend = 'butt', linejoin = 'round', linemitre = 1,
                        na.rm = FALSE, interpolate = TRUE,
                        label_colour = 'black', label_alpha = 1, label_parse = FALSE,
                        check_overlap = FALSE, angle_calc = 'none', force_flip = TRUE,
                        label_dodge = NULL, label_push = NULL) {
    data <- data[order(data$group), , drop = FALSE]
    panel <- GeomEdgePath$draw_panel(data, panel_scales, coord, arrow = arrow,
                                     lineend = lineend, linejoin = linejoin,
                                     linemitre = linemitre, na.rm = na.rm,
                                     interpolate = interpolate,
                                     label_colour = label_colour,
                                     label_alpha = label_alpha,
                                     label_parse = label_parse,
                                     check_overlap = check_overlap,
                                     angle_calc = angle_calc,
                                     force_flip = force_flip,
                                     label_dodge = label_dodge,
                                     label_push = label_push)
    if (inherits(panel, 'gList')) {
      panel[[1]]$sep <- (data$.position * sep)[panel[[1]]$id]
      class(panel[[1]]) <- c('parallelPath', class(panel[[1]]))
    } else {
      panel$sep <- (data$.position[!duplicated(data$group)] * sep)[panel$id]
      class(panel) <- c('parallelPath', class(panel))
    }
    panel
  }
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid gpar segmentsGrob
#' @export
GeomEdgeSegment <- ggproto('GeomEdgeSegment', GeomSegment,
  draw_panel = function(data, panel_scales, coord, arrow = NULL, lineend = 'butt',
                          na.rm = FALSE) {
    if (is.null(data) || nrow(data) == 0 || ncol(data) == 0) {
      return(zeroGrob())
    }
    coord <- coord$transform(data, panel_scales)
    segmentsGrob(coord$x, coord$y, coord$xend, coord$yend,
      default.units = 'native',
      gp = gpar(
        col = alpha(coord$edge_colour, coord$edge_alpha),
        fill = alpha(coord$edge_colour, coord$edge_alpha),
        lwd = coord$edge_width * .pt,
        lty = coord$edge_linetype,
        lineend = lineend
      ),
      arrow = arrow
    )
  },
  draw_key = function(data, params, size) {
    segmentsGrob(0.1, 0.5, 0.9, 0.5,
      gp = gpar(
        col = alpha(data$edge_colour, data$edge_alpha),
        lwd = data$edge_width * .pt,
        lty = data$edge_linetype, lineend = 'butt'
      ),
      arrow = params$arrow
    )
  },
  default_aes = aes(
    edge_colour = 'black', edge_width = 0.5, edge_linetype = 1,
    edge_alpha = NA
  )
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
GeomEdgeParallelSegment <- ggproto('GeomEdgeParallelSegment', GeomEdgeSegment,
  draw_panel = function(data, panel_scales, coord, arrow = NULL, lineend = 'butt',
                        na.rm = FALSE, sep = unit(2, 'mm')) {
    data <- data[order(data$group), , drop = FALSE]
    panel <- GeomEdgeSegment$draw_panel(data, panel_scales, coord,
                                        arrow = arrow, lineend = lineend,
                                        na.rm = na.rm)
    panel$sep <- data$.position * sep
    class(panel) <- c('parallelPath', class(panel))
    panel
  }
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid pointsGrob unit gpar grobTree
#' @export
GeomEdgeSpanPath <- ggproto('GeomEdgeSpanPath', GeomEdgePath,
  draw_panel = function(data, panel_scales, coord, arrow = NULL, end_shape = NA,
                        lineend = 'butt', linejoin = 'round', linemitre = 1,
                        na.rm = FALSE, interpolate = TRUE,
                        label_colour = 'black', label_alpha = 1, label_parse = FALSE,
                        check_overlap = FALSE, angle_calc = 'none', force_flip = TRUE,
                        label_dodge = NULL, label_push = NULL) {
    panel <- GeomEdgePath$draw_panel(data, panel_scales, coord, arrow = arrow,
                                     lineend = lineend, linejoin = linejoin, linemitre = linemitre,
                                     na.rm = na.rm, interpolate = interpolate,
                                     label_colour = label_colour, label_alpha = label_alpha, label_parse = label_parse,
                                     check_overlap = FALSE, angle_calc = 'none', force_flip = TRUE,
                                     label_dodge = label_dodge, label_push = label_push)
    if (is.na(end_shape)) return(panel)

    data <- data[data$index == 0 | data$index == 1, ]
    end_shape <- translate_pch(end_shape)
    coords <- coord$transform(data, panel_scales)
    ends <- pointsGrob(coords$x, coords$y,
      pch = end_shape,
      size = unit(1 / (panel_scales$x.range[2] - (panel_scales$x.range[1])), 'npc'),
      gp = gpar(
        col = alpha(coords$edge_colour, coords$edge_alpha),
        fill = alpha(coords$edge_colour, coords$edge_alpha),
        lwd = 0
      )
    )
    grobTree(panel, ends)
  }
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid pointsGrob unit gpar grobTree
#' @export
GeomEdgeSpanSegment <- ggproto('GeomEdgeSpanSegment', GeomEdgeSegment,
  draw_panel = function(data, panel_scales, coord, arrow = NULL, lineend = 'butt',
                        na.rm = FALSE, end_shape = NA) {
    panel <- GeomEdgeSegment$draw_panel(data, panel_scales, coord, arrow = arrow, lineend = lineend,
                                        na.rm = na.rm)
    if (is.na(end_shape)) return(panel)

    data2 <- data
    data2$x <- data2$xend
    data2$y <- data2$yend
    data <- rbind_dfs(list(data, data2))
    data$xend <- NULL
    data$yend <- NULL

    end_shape <- translate_pch(end_shape)
    coords <- coord$transform(data, panel_scales)
    ends <- pointsGrob(coords$x, coords$y,
      pch = end_shape,
      size = unit(1 / (panel_scales$x.range[2] - (panel_scales$x.range[1])), 'npc'),
      gp = gpar(
        col = alpha(coords$edge_colour, coords$edge_alpha),
        fill = alpha(coords$edge_colour, coords$edge_alpha),
        lwd = 0
      )
    )
    grobTree(panel, ends)
  }
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid gpar pointsGrob
#' @export
GeomEdgePoint <- ggproto('GeomEdgePoint', GeomPoint,
  setup_data = function(data, params) {
    if (params$mirror) {
      data2 <- data
      data2[, c('x', 'y')] <- data2[, c('y', 'x'), drop = FALSE]
      data2$x <- abs(data2$x) * sign(data$x)
      data2$y <- abs(data2$y) * sign(data$y)
      data <- rbind_dfs(list(data, data2))
    }
    data
  },
  draw_panel = function(data, panel_scales, coord, na.rm = FALSE) {
    if (is.null(data) || nrow(data) == 0 || ncol(data) == 0) {
      return(zeroGrob())
    }
    data$edge_shape <- translate_pch(data$edge_shape)
    coords <- coord$transform(data, panel_scales)
    coords <- coords[order(coords$edge_size, decreasing = TRUE), , drop = FALSE]
    pointsGrob(coords$x, coords$y,
      pch = coords$edge_shape,
      gp = gpar(
        col = alpha(coords$edge_colour, coords$edge_alpha),
        fill = alpha(coords$edge_fill, coords$edge_alpha),
        fontsize = coords$edge_size * .pt + coords$stroke *
          .stroke / 2,
        lwd = coords$stroke * .stroke / 2
      )
    )
  },
  draw_key = function(data, params, size) {
    pointsGrob(0.5, 0.5,
      pch = data$edge_shape,
      gp = gpar(
        col = alpha(data$edge_colour, data$edge_alpha),
        fill = alpha(data$edge_fill, data$edge_alpha),
        fontsize = data$edge_size * .pt + data$stroke * .stroke / 2,
        lwd = data$stroke * .stroke / 2
      )
    )
  },
  default_aes = aes(
    edge_shape = 19, edge_colour = 'black', edge_size = 1.5,
    edge_fill = NA, edge_alpha = NA, stroke = 0.5
  ),
  extra_params = c('na.rm', 'mirror')
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid gpar rectGrob
#' @export
GeomEdgeTile <- ggproto('GeomEdgeTile', GeomTile,
  setup_data = function(data, params) {
    if (params$mirror) {
      data2 <- data
      data2[, c('x', 'y')] <- data2[, c('y', 'x'), drop = FALSE]
      data2$x <- abs(data2$x) * sign(data$x)
      data2$y <- abs(data2$y) * sign(data$y)
      data <- rbind_dfs(list(data, data2))
    }
    data
  },
  draw_panel = function(data, panel_params, coord, na.rm = FALSE) {
    if (is.null(data) || nrow(data) == 0 || ncol(data) == 0) {
      return(zeroGrob())
    }
    data$xmin <- data$x - 0.5
    data$xmax <- data$x + 0.5
    data$ymin <- data$y - 0.5
    data$ymax <- data$y + 0.5
    coords <- coord$transform(data, panel_params)
    rectGrob(
      coords$xmin, coords$ymax,
      width = coords$xmax - coords$xmin,
      height = coords$ymax - coords$ymin,
      default.units = "native",
      just = 'centre',
      gp = gpar(
        col = coords$edge_colour,
        fill = alpha(coords$edge_fill, coords$edge_alpha),
        lwd = coords$edge_size * .pt,
        lty = coords$edge_linetype,
        linejoin = 'mitre',
        # `lineend` is a workaround for Windows and intentionally kept unexposed
        # as an argument. (c.f. https://github.com/tidyverse/ggplot2/issues/3037#issuecomment-457504667)
        lineend = "square"
      )
    )
  },
  draw_key = function(data, params, size) {
    if (is.null(data$edge_size)) {
      data$edge_size <- 0.5
    }

    lwd <- min(data$edge_size, min(size) / 4)

    rectGrob(
      width = unit(1, "npc") - unit(lwd, "mm"),
      height = unit(1, "npc") - unit(lwd, "mm"),
      gp = gpar(
        col = data$edge_colour %||% NA,
        fill = alpha(data$edge_fill %||% "grey20", data$edge_alpha),
        lty = data$edge_linetype %||% 1,
        lwd = lwd * .pt,
        linejoin = "mitre",
        # `lineend` is a workaround for Windows and intentionally kept unexposed
        # as an argument. (c.f. https://github.com/tidyverse/ggplot2/issues/3037#issuecomment-457504667)
        lineend = "square"
      ))
  },
  default_aes = aes(
    edge_fill = 'grey20', edge_colour = NA, edge_size = 0.1,
    edge_linetype = 1, edge_alpha = NA
  ),
  extra_params = c('na.rm', 'mirror')
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce GeomBezier0
#' @export
GeomEdgeBezier <- ggproto('GeomEdgeBezier', GeomBezier0,
  draw_panel = function(data, panel_scales, coord, arrow = NULL,
                          lineend = 'butt', linejoin = 'round', linemitre = 1,
                          na.rm = FALSE) {
    names(data) <- sub('edge_', '', names(data))
    names(data)[names(data) == 'width'] <- 'size'
    GeomBezier0$draw_panel(data, panel_scales, coord, arrow, lineend, linejoin, linemitre, na.rm)
  },
  draw_key = function(data, params, size) {
    segmentsGrob(0.1, 0.5, 0.9, 0.5,
      gp = gpar(
        col = alpha(data$edge_colour, data$edge_alpha),
        lwd = data$edge_width * .pt,
        lty = data$edge_linetype, lineend = 'butt'
      ),
      arrow = params$arrow
    )
  },
  default_aes = aes(
    edge_colour = 'black', edge_width = 0.5, edge_linetype = 1,
    edge_alpha = NA
  ),
  handle_na = function(data, ...) {
    data
  }
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce GeomBspline0
#' @export
GeomEdgeBspline <- ggproto('GeomEdgeBspline', GeomBspline0,
  draw_panel = function(data, panel_scales, coord, arrow = NULL,
                          lineend = 'butt', linejoin = 'round', linemitre = 1,
                          na.rm = FALSE) {
    names(data) <- sub('edge_', '', names(data))
    names(data)[names(data) == 'width'] <- 'size'
    GeomBspline0$draw_panel(data, panel_scales, coord, arrow, lineend, linejoin, linemitre, na.rm)
  },
  draw_key = function(data, params, size) {
    segmentsGrob(0.1, 0.5, 0.9, 0.5,
      gp = gpar(
        col = alpha(data$edge_colour, data$edge_alpha),
        lwd = data$edge_width * .pt,
        lty = data$edge_linetype, lineend = 'butt'
      ),
      arrow = params$arrow
    )
  },
  default_aes = aes(
    edge_colour = 'black', edge_width = 0.5, edge_linetype = 1,
    edge_alpha = NA
  ),
  handle_na = function(data, ...) {
    data
  }
)

remove_loop <- function(data) {
  if (nrow(data) == 0) return(data)

  data[!(data$x == data$xend & data$y == data$yend), , drop = FALSE]
}
remove_loop2 <- function(data) {
  if (nrow(data) == 0) return(data)

  data <- data[order(data$group), ]
  loop <- data$x[c(TRUE, FALSE)] == data$x[c(FALSE, TRUE)] &
    data$y[c(TRUE, FALSE)] == data$y[c(FALSE, TRUE)]
  data[rep(!loop, each = 2), , drop = FALSE]
}

#' Create hierarchical edge bundles between node connections
#'
#' Hierarchical edge bundling is a technique to introduce some order into the
#' hairball structure that can appear when there's a lot of overplotting and
#' edge crossing in a network plot. The concept requires that the network has
#' an intrinsic hierarchical structure that defines the layout but is not shown.
#' Connections between points (that is, not edges) are then drawn so that they
#' loosely follows the underlying hierarchical structure. This results in a
#' flow-like structure where lines that partly move in the same direction will
#' be bundled together.
#'
#' @note In order to avoid excessive typing edge aesthetic names are
#' automatically expanded. Because of this it is not necessary to write
#' `edge_colour` within the `aes()` call as `colour` will
#' automatically be renamed appropriately.
#'
#' @section Aesthetics:
#' geom_conn_bundle* understands the following aesthetics. Bold aesthetics are
#' automatically set, but can be overridden.
#' \itemize{
#'  \item{\strong{x}}
#'  \item{\strong{y}}
#'  \item{\strong{group}}
#'  \item{\strong{circular}}
#'  \item{edge_colour}
#'  \item{edge_width}
#'  \item{edge_linetype}
#'  \item{edge_alpha}
#'  \item{filter}
#' }
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#'
#' @param data The result of a call to [get_con()]
#'
#' @param tension How "loose" should the bundles be. 1 will give very tight
#' bundles, while 0 will turn of bundling completely and give straight lines.
#' Defaults to 0.8
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_conn_*
#'
#' @references
#' Holten, D. (2006). *Hierarchical edge bundles: visualization
#' of adjacency relations in hierarchical data.* IEEE Transactions on
#' Visualization and Computer Graphics, **12**(5), 741-748.
#' <https://doi.org/10.1109/TVCG.2006.147>
#'
#' @examples
#' # Create a graph of the flare class system
#' library(tidygraph)
#' flareGraph <- tbl_graph(flare$vertices, flare$edges) %>%
#'   mutate(
#'     class = map_bfs_chr(node_is_root(), .f = function(node, dist, path, ...) {
#'       if (dist <= 1) {
#'         return(shortName[node])
#'       }
#'       path$result[[nrow(path)]]
#'     })
#'   )
#' importFrom <- match(flare$imports$from, flare$vertices$name)
#' importTo <- match(flare$imports$to, flare$vertices$name)
#'
#' # Use class inheritance for layout but plot class imports as bundles
#' ggraph(flareGraph, 'dendrogram', circular = TRUE) +
#'   geom_conn_bundle(aes(colour = stat(index)),
#'     data = get_con(importFrom, importTo),
#'     edge_alpha = 0.25
#'   ) +
#'   geom_node_point(aes(filter = leaf, colour = class)) +
#'   scale_edge_colour_distiller('', direction = 1, guide = 'edge_direction') +
#'   coord_fixed() +
#'   ggforce::theme_no_axes()
#' @rdname geom_conn_bundle
#' @name geom_conn_bundle
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBspline
#' @export
StatConnBundle <- ggproto('StatConnBundle', StatBspline,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    if (nrow(data) == 0) return(NULL)
    relax(data, params$tension)
  },
  setup_params = function(data, params) {
    if (is.null(params$tension)) {
      params$tension <- 0.8
    }
    if (params$tension < 0) params$tension <- 0
    if (params$tension > 1) params$tension <- 1
    params
  },
  required_aes = c('x', 'y'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n', 'tension', 'type')
)
#' @rdname geom_conn_bundle
#'
#' @export
geom_conn_bundle <- function(mapping = NULL, data = get_con(),
                             position = 'identity', arrow = NULL,
                             lineend = 'butt', show.legend = NA,
                             n = 100, tension = 0.8, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y,
                                        group = con.id))
  layer(
    data = data, mapping = mapping, stat = StatConnBundle,
    geom = GeomEdgePath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, na.rm = FALSE, n = n,
        interpolate = FALSE, tension = tension, type = 'clamped', ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBspline2
#' @export
StatConnBundle2 <- ggproto('StatConnBundle2', StatBspline2,
  setup_data = function(data, params) {
    StatConnBundle$setup_data(data, params)
  },
  setup_params = function(data, params) {
    StatConnBundle$setup_params(data, params)
  },
  required_aes = c('x', 'y'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n', 'tension', 'type')
)
#' @rdname geom_conn_bundle
#'
#' @export
geom_conn_bundle2 <- function(mapping = NULL, data = get_con(),
                              position = 'identity', arrow = NULL,
                              lineend = 'butt', show.legend = NA,
                              n = 100, tension = 0.8, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y,
                                        group = con.id))
  layer(
    data = data, mapping = mapping, stat = StatConnBundle2,
    geom = GeomEdgePath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, na.rm = FALSE, n = n,
        interpolate = TRUE, tension = tension, type = 'clamped', ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
StatConnBundle0 <- ggproto('StatConnBundle0', StatIdentity,
  setup_data = function(data, params) {
    StatConnBundle$setup_data(data, params)
  },
  setup_params = function(data, params) {
    StatConnBundle$setup_params(data, params)
  },
  required_aes = c('x', 'y'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n', 'tension', 'type')
)
#' @rdname geom_conn_bundle
#'
#' @export
geom_conn_bundle0 <- function(mapping = NULL, data = get_con(),
                              position = 'identity', arrow = NULL,
                              lineend = 'butt', show.legend = NA,
                              tension = 0.8, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y,
                                        group = con.id))
  layer(
    data = data, mapping = mapping, stat = StatConnBundle0,
    geom = GeomEdgeBspline, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, na.rm = FALSE,
        tension = tension, type = 'clamped', ...
      )
    )
  )
}
#' @importFrom utils head tail
relax <- function(data, strength) {
  formula <- function(p, startInd, endInd, pathLengths) {
    start <- rep(p[startInd], pathLengths)
    range <- rep(p[endInd] - p[startInd], pathLengths)
    ind <- unlist(lapply(pathLengths, seq_len)) - 1
    length <- rep(pathLengths, pathLengths)
    strength * p + (1 - strength) * (start + (ind / (length - 1)) * range)
  }
  idInds <- split(seq_len(nrow(data)), data$group)
  pathLengths <- lengths(idInds)
  startInd <- vapply(idInds, head, integer(1), n = 1)
  endInd <- vapply(idInds, tail, integer(1), n = 1)
  data$x <- formula(data$x, startInd, endInd, pathLengths)
  data$y <- formula(data$y, startInd, endInd, pathLengths)
  data
}

#' Draw edges as glyphs
#'
#' This geom draws edges as glyphs with their x-position defined by the
#' x-position of the start node, and the y-position defined by the y-position of
#' the end node. As such it will result in a matrix layout when used in
#' conjunction with [layout_tbl_graph_matrix()]
#'
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_point` understands the following
#' aesthetics. Bold aesthetics are automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - edge_shape
#' - edge_colour
#' - edge_size
#' - edge_alpha
#' - filter
#'
#' @inheritParams ggplot2::geom_point
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x, y, xend, yend, group and
#' circular are mapped to x, y, xend, yend, edge.id and circular in the edge
#' data.
#'
#' @param data The return of a call to `get_edges()` or a data.frame
#' giving edges in correct format (see details for for guidance on the format).
#' See [get_edges()] for more details on edge extraction.
#'
#' @param mirror Logical. Should edge points be duplicated on both sides of the
#' diagonal. Intended for undirected graphs. Default to `FALSE`
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @rdname geom_edge_point
#' @name geom_edge_point
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('zachary') %>%
#'   mutate(group = group_infomap()) %>%
#'   morph(to_split, group) %>%
#'   activate(edges) %>%
#'   mutate(edge_group = as.character(.N()$group[1])) %>%
#'   unmorph()
#'
#' ggraph(gr, 'matrix', sort.by = node_rank_hclust()) +
#'   geom_edge_point(aes(colour = edge_group), mirror = TRUE, edge_size = 3) +
#'   scale_y_reverse() +
#'   coord_fixed() +
#'   labs(edge_colour = 'Infomap Cluster') +
#'   ggtitle("Zachary' Karate Club")
NULL

#' @rdname geom_edge_point
#'
#' @export
geom_edge_point <- function(mapping = NULL, data = get_edges(),
                            position = 'identity', mirror = FALSE,
                            show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = yend))
  layer(
    data = data, mapping = mapping, stat = StatFilter, geom = GeomEdgePoint,
    position = position, show.legend = show.legend, inherit.aes = FALSE,
    params = list(na.rm = FALSE, mirror = mirror, ...)
  )
}

#' Show edges as a density map
#'
#' This geom makes it possible to add a layer showing edge presence as a density
#' map. Each edge is converted to `n` points along the line and a jitter is
#' applied. Based on this dataset a two-dimensional kernel density estimation is
#' applied and plotted as a raster image. The density is mapped to the alpha
#' level, making it possible to map a variable to the fill.
#'
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_density` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overridden.
#'
#' **x**
#' **y**
#' **xend**
#' **yend**
#' edge_fill
#' filter
#'
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{x, y}{The coordinates for each pixel in the raster}
#'  \item{density}{The density associated with the pixel}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_raster
#'
#' @param n The number of points to estimate in the x and y direction, i.e. the
#' resolution of the raster.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('bull') %>%
#'   activate(edges) %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_density(aes(fill = class)) +
#'   geom_edge_link() + geom_node_point()
#' @rdname geom_edge_density
#' @name geom_edge_density
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatLink
#' @importFrom MASS bandwidth.nrd kde2d
#' @export
StatEdgeDensity <- ggproto('StatEdgeDensity', Stat,
  compute_group = function(data, scales, na.rm = FALSE, h = NULL,
                             n = 100, bins = NULL, binwidth = NULL) {
    group <- data$group[1]
    x_range <- diff(range(c(data$x, data$xend)))
    y_range <- diff(range(c(data$y, data$yend)))
    x_extend <- x_range / 10
    y_extend <- y_range / 10
    data <- StatLink$compute_panel(data, n = 50)
    data <- PositionJitter$compute_layer(data, list(
      width = x_extend,
      height = y_extend
    ))

    if (is.null(h)) {
      h <- c(MASS::bandwidth.nrd(data$x), MASS::bandwidth.nrd(data$y))
    }
    dens <- MASS::kde2d(data$x, data$y,
      h = h, n = n,
      lims = c(
        scales$x$dimension(),
        scales$y$dimension()
      ) + c(
        -x_extend,
        x_extend,
        -y_extend,
        y_extend
      )
    )
    df <- expand.grid(x = dens$x, y = dens$y)
    df$z <- as.vector(dens$z)
    df$group <- group
    names(df) <- c('x', 'y', 'density', 'group')
    df
  },
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    data
  },
  default_aes = aes(filter = TRUE),
  required_aes = c('x', 'y', 'xend', 'yend')
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom grid gTree gList grobName rasterGrob rectGrob gpar
GeomEdgeDensity <- ggproto('GeomEdgeDensity', GeomRaster,
  draw_panel = function(self, data, panel_scales, coord, ...) {
    groups <- split(data, factor(data$group))
    max_density <- max(rowSums(do.call(
      cbind,
      lapply(groups, `[[`, i = 'density')
    )))
    grobs <- lapply(groups, function(group) {
      self$draw_group(group, panel_scales, coord,
        max.alpha = max_density,
        ...
      )
    })
    grobs <- gTree(children = do.call('gList', grobs))
    grobs$name <- grobName(grobs, 'geom_edge_density')
    grobs
  },
  draw_group = function(data, panel_scales, coord, max.alpha) {
    if (!inherits(coord, 'CoordCartesian')) {
      stop('geom_raster only works with Cartesian coordinates',
        call. = FALSE
      )
    }
    data <- coord$transform(data, panel_scales)
    x_pos <- as.integer((data$x - min(data$x)) / resolution(
      data$x,
      FALSE
    ))
    y_pos <- as.integer((data$y - min(data$y)) / resolution(
      data$y,
      FALSE
    ))
    nrow <- max(y_pos) + 1
    ncol <- max(x_pos) + 1
    raster <- matrix(NA_character_, nrow = nrow, ncol = ncol)
    raster[cbind(nrow - y_pos, x_pos + 1)] <- alpha(
      data$edge_fill,
      data$density / max.alpha
    )
    x_rng <- c(min(data$xmin, na.rm = TRUE), max(data$xmax, na.rm = TRUE))
    y_rng <- c(min(data$ymin, na.rm = TRUE), max(data$ymax, na.rm = TRUE))
    rasterGrob(raster,
      x = mean(x_rng), y = mean(y_rng),
      width = diff(x_rng), height = diff(y_rng),
      default.units = 'native', interpolate = TRUE
    )
  },
  draw_key = function(data, params, size) {
    rectGrob(gp = gpar(
      col = NA, fill = alpha(data),
      lty = 0
    ))
  },
  required_aes = c('x', 'y'),
  default_aes = aes(edge_fill = 'darkgrey')
)
#' @rdname geom_edge_density
#'
#' @importFrom ggforce StatLink
#' @export
geom_edge_density <- function(mapping = NULL, data = get_edges('short'),
                              position = 'identity', show.legend = NA,
                              n = 100, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y,
                                        xend = xend, yend = yend))
  layer(
    data = data, mapping = mapping, stat = StatEdgeDensity,
    geom = GeomEdgeDensity, position = position,
    show.legend = show.legend, inherit.aes = FALSE,
    params = expand_edge_aes(
      list(na.rm = FALSE, n = n, ...)
    )
  )
}

#' Edge linetype scales
#'
#' This set of scales defines new linetype scales for edge geoms equivalent to
#' the ones already defined by ggplot2. See
#' [ggplot2::scale_linetype()] for more information. The different
#' geoms will know whether to use edge scales or the standard scales so it is
#' not necessary to write `edge_linetype` in the call to the geom - just
#' use `linetype`.
#'
#' @return A ggproto object inheriting from `Scale`
#'
#' @family scale_edge_*
#'
#' @name scale_edge_linetype
#' @rdname scale_edge_linetype
#'
NULL

#' @rdname scale_edge_linetype
#'
#' @inheritParams ggplot2::scale_linetype
#'
#' @importFrom scales linetype_pal
#' @export
scale_edge_linetype <- function(..., na.value = 'blank') {
  discrete_scale('edge_linetype', 'linetype_d', linetype_pal(),
    na.value = na.value, ...
  )
}
#' @rdname scale_edge_linetype
#'
#' @export
scale_edge_linetype_continuous <- function(...) {
  stop('A continuous variable can not be mapped to linetype', call. = FALSE)
}
#' @rdname scale_edge_linetype
#'
#' @export
scale_edge_linetype_discrete <- scale_edge_linetype
#' @rdname scale_edge_linetype
#'
#' @inheritParams ggplot2::scale_linetype_manual
#'
#' @export
scale_edge_linetype_manual <- function(..., values) {
  manual_scale('edge_linetype', values, ...)
}
#' @rdname scale_edge_linetype
#'
#' @inheritParams ggplot2::scale_linetype_identity
#'
#' @importFrom scales identity_pal
#' @export
scale_edge_linetype_identity <- function(..., guide = 'none') {
  sc <- discrete_scale('edge_linetype', 'identity', identity_pal(), ...,
    guide = guide, super = ScaleDiscreteIdentity
  )
  sc
}

#' Draw edges as diagonals
#'
#' This geom draws edges as diagonal bezier curves. The name comes from D3.js
#' where this shape was called diagonals until it was renamed to
#' [links](https://github.com/d3/d3-shape/blob/v1.3.5/README.md#links).
#' A diagonal in this context is a quadratic bezier with the control points
#' positioned halfway between the start and end points but on the same axis.
#' This produces a pleasing fan-in, fan-out line that is mostly relevant for
#' hierarchical layouts as it implies an overall directionality in the plot.
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_diagonal` and `geom_edge_diagonal0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - **circular**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_diagonal2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **group**
#' - **circular**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_diagonal` and `geom_edge_diagonal2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#'
#' @param flipped Logical, Has the layout been flipped by reassigning the
#' mapping of x, y etc?
#'
#' @param strength The strength of the curvature of the diagonal. `0` will
#' result in a straight line while `1` will give the familiar S-shape.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' gr <- create_tree(20, 4) %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE)) %>%
#'   activate(edges) %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE))
#'
#' ggraph(gr, 'tree') +
#'   geom_edge_diagonal(aes(alpha = after_stat(index)))
#'
#' ggraph(gr, 'tree') +
#'   geom_edge_diagonal2(aes(colour = node.class))
#'
#' ggraph(gr, 'tree') +
#'   geom_edge_diagonal0(aes(colour = class))
#' @rdname geom_edge_diagonal
#' @name geom_edge_diagonal
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier
#' @export
StatEdgeDiagonal <- ggproto(
  'StatEdgeDiagonal',
  StatBezier,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data <- remove_loop(data)
    if (nrow(data) == 0) {
      return(data)
    }
    data$group <- make_unique(data$group)
    data2 <- data
    data2$x <- data2$xend
    data2$y <- data2$yend
    create_diagonal(data, data2, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'circular'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'flipped', 'n', 'strength')
)
#' @rdname geom_edge_diagonal
#'
#' @export
geom_edge_diagonal <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  flipped = FALSE,
  n = 100,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      circular = circular,
      group = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeDiagonal,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = FALSE,
        flipped = flipped,
        strength = strength,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier2
#' @export
StatEdgeDiagonal2 <- ggproto(
  'StatEdgeDiagonal2',
  StatBezier2,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data <- remove_loop2(data)
    if (nrow(data) == 0) {
      return(data)
    }
    data <- data[order(data$group), ]
    data2 <- data[c(FALSE, TRUE), ]
    data <- data[c(TRUE, FALSE), ]
    create_diagonal(data, data2, params)
  },
  required_aes = c('x', 'y', 'group', 'circular'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'flipped', 'n', 'strength')
)
#' @rdname geom_edge_diagonal
#'
#' @export
geom_edge_diagonal2 <- function(
  mapping = NULL,
  data = get_edges('long'),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  flipped = FALSE,
  n = 100,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      group = edge.id,
      circular = circular
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeDiagonal2,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = TRUE,
        flipped = flipped,
        strength = strength,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier0
#' @export
StatEdgeDiagonal0 <- ggproto(
  'StatEdgeDiagonal0',
  StatBezier0,
  setup_data = function(data, params) {
    StatEdgeDiagonal$setup_data(data, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'circular'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'flipped', 'strength')
)
#' @rdname geom_edge_diagonal
#'
#' @export
geom_edge_diagonal0 <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  flipped = FALSE,
  lineend = 'butt',
  show.legend = NA,
  ...
) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      circular = circular
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeDiagonal0,
    geom = GeomEdgeBezier,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        strength = strength,
        flipped = flipped,
        ...
      )
    )
  )
}

create_diagonal <- function(from, to, params) {
  bezier_start <- seq(1, by = 4, length.out = nrow(from))
  from$index <- bezier_start
  to$index <- bezier_start + 3
  data2 <- from
  data3 <- to
  data2$index <- bezier_start + 1
  data3$index <- bezier_start + 2
  if (any(from$circular)) {
    r0 <- sqrt(from$x[from$circular]^2 + from$y[from$circular]^2)
    r1 <- sqrt(to$x[to$circular]^2 + to$y[to$circular]^2)
    root <- r0 == 0 | r1 == 0
    r_mod <- params$strength * (r1 - r0) / 2

    data2$x[from$circular] <- from$x[from$circular] / (r0 / (r0 + r_mod))
    data2$y[from$circular] <- from$y[from$circular] / (r0 / (r0 + r_mod))
    data3$x[from$circular] <- to$x[from$circular] / (r1 / (r1 - r_mod))
    data3$y[from$circular] <- to$y[from$circular] / (r1 / (r1 - r_mod))

    data2$x[root] <- from$x[root]
    data2$y[root] <- from$y[root]
    data3$x[root] <- to$x[root]
    data3$y[root] <- to$y[root]
  }
  if (any(!from$circular)) {
    if (params$flipped) {
      h_diff <- from$x[!from$circular] - to$x[!from$circular]
      data2$x[!from$circular] <- from$x[!from$circular] -
        h_diff / 2 * params$strength
      data3$x[!from$circular] <- to$x[!from$circular] +
        h_diff / 2 * params$strength
    } else {
      h_diff <- from$y[!from$circular] - to$y[!from$circular]
      data2$y[!from$circular] <- from$y[!from$circular] -
        h_diff / 2 * params$strength
      data3$y[!from$circular] <- to$y[!from$circular] +
        h_diff / 2 * params$strength
    }
  }
  data <- vec_rbind(from, data2, data3, to)
  data[order(data$index), names(data) != 'index']
}

#' Use igraph layout algorithms for layout_tbl_graph
#'
#' This layout function makes it easy to apply one of the layout algorithms
#' supplied in igraph when plotting with ggraph. Layout names are auto completed
#' so there is no need to write `layout_with_graphopt` or
#' `layout_as_tree`, just `graphopt` and `tree` (though the
#' former will also work if you want to be super explicit). Circular layout is
#' only supported for tree-like layout (`tree` and `sugiyama`) and
#' will throw an error when applied to other layouts.
#'
#' @details
#' igraph provides a huge amount of possible layouts. They are all briefly
#' described below:
#'
#' \strong{Hierarchical layouts}
#'
#' \describe{
#'   \item{`tree`}{Uses the *Reingold-Tilford* algorithm to place the
#'   nodes below their parent with the parent centered above its children. See
#'   [igraph::as_tree()]}
#'   \item{`sugiyama`}{Designed for directed acyclic graphs (that is,
#'   hierarchies where multiple parents are allowed) it minimizes the number of
#'   crossing edges. See [igraph::with_sugiyama()]}
#' }
#'
#' \strong{Standard layouts}
#'
#' \describe{
#'   \item{`bipartite`}{Minimize edge-crossings in a simple two-row (or
#'   column) layout for bipartite graphs. See [igraph::as_bipartite()]}
#'   \item{`star`}{Place one node in the center and the rest equidistantly
#'   around it. See [igraph::as_star()]}
#'   \item{`circle`}{Place nodes in a circle in the order of their index.
#'   Consider using [layout_tbl_graph_linear()] with `circular=TRUE`
#'   for more control. See [igraph::in_circle()]}
#'   \item{`nicely`}{Tries to pick an appropriate layout. See
#'   [igraph::nicely()] for a description of the simple decision tree
#'   it uses}
#'   \item{`dh`}{Uses *Davidson and Harels* simulated annealing
#'   algorithm to place nodes. See [igraph::with_dh()]}
#'   \item{`gem`}{Place nodes on the plane using the GEM force-directed
#'   layout algorithm. See [igraph::with_gem()]}
#'   \item{`graphopt`}{Uses the Graphopt algorithm based on alternating
#'   attraction and repulsion to place nodes. See
#'   [igraph::with_graphopt()]}
#'   \item{`grid`}{Place nodes on a rectangular grid. See
#'   [igraph::on_grid()]}
#'   \item{`mds`}{Perform a multidimensional scaling of nodes using either
#'   the shortest path or a user supplied distance. See
#'   [igraph::with_mds()]}
#'   \item{`sphere`}{Place nodes uniformly on a sphere - less relevant for
#'   2D visualizations of networks. See [igraph::on_sphere()]}
#'   \item{`randomly`}{Places nodes uniformly random. See
#'   [igraph::randomly()]}
#'   \item{`fr`}{Places nodes according to the force-directed algorithm of
#'   Fruchterman and Reingold. See [igraph::with_fr()]}
#'   \item{`kk`}{Uses the spring-based algorithm by Kamada and Kawai to
#'   place nodes. See [igraph::with_kk()]}
#'   \item{`drl`}{Uses the force directed algorithm from the DrL toolbox to
#'   place nodes. See [igraph::with_drl()]}
#'   \item{`lgl`}{Uses the algorithm from Large Graph Layout to place
#'   nodes. See [igraph::with_lgl()]}
#' }
#'
#' @note This function is not intended to be used directly but by setting
#' `layout = 'igraph'` in [create_layout()]
#'
#' @param graph A `tbl_graph` object.
#'
#' @param algorithm The type of layout algorithm to apply. See *Details* or
#' [igraph::layout_()] for links to the layouts supplied by igraph.
#'
#' @param circular Logical. Should the layout be transformed to a circular
#' representation. Defaults to `FALSE`. Only applicable to
#' `algorithm = 'tree'` and `algorithm = 'sugiyama'`.
#'
#' @param offset If `circular = TRUE`, where should it begin. Defaults to
#' `pi/2` which is equivalent to 12 o'clock.
#'
#' @param use.dummy Logical. In the case of `algorithm = 'sugiyama'` should the
#' dummy-infused graph be used rather than the original. Defaults to
#' `FALSE`.
#'
#' @param ... Arguments passed on to the respective layout functions
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom igraph graph_attr components layout_as_bipartite layout_as_star layout_as_tree layout_in_circle layout_nicely layout_with_dh layout_with_drl layout_with_gem layout_with_graphopt layout_on_grid layout_with_mds layout_with_sugiyama layout_on_sphere layout_randomly layout_with_fr layout_with_kk layout_with_lgl
#' @importFrom rlang quos eval_tidy
#' @importFrom stats setNames
#'
layout_tbl_graph_igraph <- function(graph, algorithm, circular, offset = pi / 2, use.dummy = FALSE, ...) {
  algorithm <- as.igraphlayout(algorithm)
  dots <- quos(...)
  dots <- setNames(lapply(names(dots), function(nq) {
    if (nq == 'weights') {
      eval_tidy(dots[[nq]], as_tibble(graph, active = 'edges'))
    } else {
      eval_tidy(dots[[nq]])
    }
  }), names(dots))
  layout <- do.call(algorithm, c(list(graph), dots))
  if (algorithm == 'layout_with_sugiyama') {
    if (use.dummy) {
      graph <- as_tbl_graph(layout$extd_graph)
      layout <- graph_attr(layout$extd_graph, 'layout')
    } else {
      layout <- layout$layout
    }
  }
  if (algorithm == 'layout_as_tree') {
    layout[, 1] <- layout[, 1] + components(graph)$membership - 1
  }
  if ('dim' %in% names(dots) && isTRUE(dots$dim > 2)) {
    warning('Only the first two dimensions will be used despite requesting more', call. = FALSE)
  }
  extra_data <- as_tibble(graph, active = 'nodes')
  warn_dropped_vars(data.frame(x = 1, y = 1), extra_data)
  layout <- cbind(x = layout[, 1], y = layout[, 2], extra_data[, !names(extra_data) %in% c('x', 'y'), drop = FALSE])
  graph <- add_direction(graph, layout)
  if (circular) {
    if (!algorithm %in% c('layout_as_tree', 'layout_with_sugiyama')) {
      stop('Circular layout only applicable to tree and DAG layout')
    }
    radial <- radial_trans(
      r.range = rev(range(layout$y)),
      a.range = range(layout$x),
      offset = offset
    )
    coords <- radial$transform(layout$y, layout$x)
    layout$x <- coords$x
    layout$y <- coords$y
  }
  layout$circular <- circular
  attr(layout, 'graph') <- graph
  layout
}

#' Annotate nodes with text
#'
#' These geoms are equivalent in functionality to [ggplot2::geom_text()] and
#' [ggplot2::geom_label()] and allows for simple annotation of nodes.
#'
#' @section Aesthetics:
#' `geom_node_text` understands the following aesthetics. Bold aesthetics are
#' automatically set, but can be overridden. Italic aesthetics are required but
#' not set by default
#'
#' - **x**
#' - **y**
#' - *label*
#' - alpha
#' - angle
#' - colour
#' - family
#' - fontface
#' - hjust
#' - lineheight
#' - size
#' - vjust
#'
#' @inheritParams ggplot2::geom_text
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x and y are mapped to x and y in
#' the node data.
#'
#' @param nudge_x,nudge_y Horizontal and vertical adjustment to nudge labels by.
#' Useful for offsetting text from points, particularly on discrete scales.
#'
#' @param repel If `TRUE`, text labels will be repelled from each other
#' to avoid overlapping, using the `GeomTextRepel` geom from the
#' ggrepel package.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_node_*
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('bull') %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE))
#'
#' ggraph(gr, 'stress') +
#'   geom_node_point(aes(label = class))
#'
#' ggraph(gr, 'stress') +
#'   geom_node_label(aes(label = class), repel = TRUE)
#' @importFrom ggrepel GeomTextRepel
#' @export
#'
geom_node_text <- function(mapping = NULL, data = NULL, position = 'identity',
                           parse = FALSE, nudge_x = 0, nudge_y = 0,
                           check_overlap = FALSE, show.legend = NA,
                           repel = FALSE, ...) {
  if (!missing(nudge_x) || !missing(nudge_y)) {
    if (!missing(position)) {
      stop('Specify either `position` or `nudge_x`/`nudge_y`',
        call. = FALSE
      )
    }
    position <- position_nudge(nudge_x, nudge_y)
  }
  params <- list(parse = parse, na.rm = FALSE, ...)
  if (repel) {
    geom <- GeomTextRepel
  } else {
    geom <- GeomText
    params$check_overlap <- check_overlap
  }

  mapping <- aes_intersect(mapping, aes(x = x, y = y))
  layer(
    data = data, mapping = mapping, stat = StatFilter, geom = geom,
    position = position, show.legend = show.legend, inherit.aes = FALSE,
    params = params
  )
}
#' @rdname geom_node_text
#'
#' @inheritParams ggplot2::geom_label
#'
#' @importFrom ggrepel GeomLabelRepel
#' @export
#'
geom_node_label <- function(mapping = NULL, data = NULL, position = 'identity',
                            parse = FALSE, nudge_x = 0, nudge_y = 0,
                            label.padding = unit(0.25, 'lines'),
                            label.r = unit(0.15, 'lines'),
                            label.size = 0.25, show.legend = NA,
                            repel = FALSE, ...) {
  if (!missing(nudge_x) || !missing(nudge_y)) {
    if (!missing(position)) {
      stop('Specify either `position` or `nudge_x`/`nudge_y`',
        call. = FALSE
      )
    }
    position <- position_nudge(nudge_x, nudge_y)
  }
  params <- list(
    parse = parse, label.padding = label.padding,
    label.r = label.r, label.size = label.size, na.rm = FALSE, ...
  )
  if (repel) {
    geom <- GeomLabelRepel
  } else {
    geom <- GeomLabel
  }

  mapping <- aes_intersect(mapping, aes(x = x, y = y))
  layer(
    data = data, mapping = mapping, stat = StatFilter, geom = geom,
    position = position, show.legend = show.legend, inherit.aes = FALSE,
    params = params
  )
}

#' Create small multiples based on node attributes
#'
#' This function is equivalent to [ggplot2::facet_wrap()] but only
#' facets nodes. Edges are drawn if their terminal nodes are both present in a
#' panel.
#'
#' @inheritParams ggplot2::facet_wrap
#'
#' @family ggraph-facets
#'
#' @examples
#' library(tidygraph)
#' gr <- as_tbl_graph(highschool) %>%
#'   mutate(popularity = as.character(cut(centrality_degree(mode = 'in'),
#'     breaks = 3,
#'     labels = c('low', 'medium', 'high')
#'   )))
#' ggraph(gr) +
#'   geom_edge_link() +
#'   geom_node_point() +
#'   facet_nodes(~popularity)
#' @export
#'
facet_nodes <- function(facets, nrow = NULL, ncol = NULL, scales = 'fixed',
                        shrink = TRUE, labeller = 'label_value', as.table = TRUE,
                        switch = NULL, drop = TRUE, dir = 'h',
                        strip.position = 'top') {
  facet <- facet_wrap(
    facets = facets, nrow = nrow, ncol = ncol,
    scales = scales, shrink = shrink, labeller = labeller,
    as.table = as.table, switch = switch, drop = drop,
    dir = dir, strip.position = strip.position
  )
  seed <- sample(.Machine$integer.max, 1L)
  facet$params$facets[] <- lapply(seq_along(facet$params$facets), function(i) {
    rlang::quo(withr::with_seed(seed, !!facet$params$facets[[i]]))
  })
  ggproto(NULL, FacetNodes,
    shrink = shrink,
    params = facet$params
  )
}

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @export
FacetNodes <- ggproto('FacetNodes', FacetWrap,
  compute_layout = function(data, params) {
    plot_data <- data[[1]]
    data <- split(data, vapply(data, data_type, character(1)))
    facet_data <- data$node_ggraph

    panels <- FacetWrap$compute_layout(facet_data, params)
    node_placement <- if (nrow(panels) == 1) {
      list(`1` = plot_data$ggraph_index)
    } else {
      node_map <- FacetWrap$map_data(plot_data, panels, params)
      node_map <- node_map[order(node_map$.ggraph.index), , drop = FALSE]
      split(node_map$.ggraph.index, node_map$PANEL)
    }
    attr(panels, 'node_placement') <- node_placement
    panels
  },
  map_data = function(data, layout, params) {
    switch(
      data_type(data),
      edge_ggraph = {
        node_placement <- attr(layout, 'node_placement')
        edge_map <- Map(function(map, nodes) {
          map[map$from %in% nodes & map$to %in% nodes, , drop = FALSE]
        }, map = rep(list(data), length(node_placement)), nodes = node_placement)
        panel <- rep(seq_along(edge_map), vapply(edge_map, nrow, numeric(1)))
        edge_map <- rbind_dfs(edge_map)
        edge_map$PANEL <- factor(panel, levels = levels(layout$PANEL))
        edge_map
      },
      node_ggraph = , {
        FacetWrap$map_data(data, layout, params)
      }
    )
  }
)

#' @importFrom grid grob is.unit unit gTree
cappedPathGrob <- function(x, y, id = NULL, id.lengths = NULL, arrow = NULL,
                           start.cap = NULL, start.cap2 = NULL, start.captype = 'circle',
                           end.cap = NULL, end.cap2 = NULL, end.captype = 'circle',
                           default.units = 'npc', name = NULL, gp = gpar(), vp = NULL, constant = TRUE) {
  if (!is.unit(x)) {
    x <- unit(x, default.units)
  }
  if (!is.unit(y)) {
    y <- unit(y, default.units)
  }
  if (is.null(id)) {
    if (!is.null(id.lengths)) {
      id <- rep(seq_along(id.lengths), id.lengths)
    } else {
      id <- rep(1, length(x))
    }
  }
  n <- length(unique(id))
  start.cap <- validate_cap(start.cap, default.units, n)
  if (is.null(start.cap2)) {
    start.cap2 <- start.cap
  } else {
    start.cap2 <- validate_cap(start.cap2, default.units, n)
  }
  end.cap <- validate_cap(end.cap, default.units, n)
  if (is.null(end.cap2)) {
    end.cap2 <- end.cap
  } else {
    end.cap2 <- validate_cap(end.cap2, default.units, n)
  }
  if (!all(c(start.captype, end.captype) %in% c('circle', 'rect'))) {
    stop('captype must be either `circle` or `rect`', call. = FALSE)
  }
  start.captype <- rep(start.captype, length.out = n)
  end.captype <- rep(end.captype, length.out = n)

  n_points <- length(x)
  group_diff <- id[-1] != id[-n_points]
  start <- c(TRUE, group_diff)
  end <- c(group_diff, TRUE)

  if (constant) {
    gp <- lapply(gp, function(par) {
      if (length(par) == length(end)) {
        par[start]
      } else {
        par
      }
    })
  } else {
    gp <- lapply(gp, function(par) {
      if (length(par) == length(end)) {
        par[!end]
      } else {
        par
      }
    })
  }
  class(gp) <- 'gpar'

  if (is.null(start.cap) && is.null(end.cap)) {
    if (constant) {
      grob(
        x = x, y = y, id = id, id.lengths = NULL, arrow = arrow,
        name = name, gp = gp, vp = vp, cl = 'polyline'
      )
    } else {
      grob(
        x0 = x[!end], y0 = y[!end], x1 = x[!start], y1 = y[!start], id = id[!end],
        arrow = arrow, name = name, gp = gp, vp = vp, cl = 'segments'
      )
    }
  } else {
    gTree(
      x = x, y = y, id = id, arrow = arrow, constant = constant, start = start, end = end,
      start.cap = start.cap, start.cap2 = start.cap2, start.captype = start.captype,
      end.cap = end.cap, end.cap2 = end.cap2, end.captype = end.captype,
      name = name, gp = gp, vp = vp, cl = 'cappedpathgrob'
    )
  }
}
#' Dynamic capping of paths
#'
#' This function takes care of updating which parts of the paths get removed
#' when the device dimensions are updated.
#'
#' @importFrom grid convertX convertY convertWidth convertHeight unit grob makeContent setChildren gList
#' @export
#' @keywords internal
makeContent.cappedpathgrob <- function(x) {
  x_new <- convertX(x$x, 'mm', TRUE)
  y_new <- convertY(x$y, 'mm', TRUE)
  start.cap <- convertWidth(x$start.cap, 'mm', TRUE)
  end.cap <- convertWidth(x$end.cap, 'mm', TRUE)
  start.cap2 <- convertHeight(x$start.cap2, 'mm', TRUE)
  end.cap2 <- convertHeight(x$end.cap2, 'mm', TRUE)
  truncated <- cut_lines(
    x_new, y_new, as.integer(x$id), start.cap, start.cap2,
    end.cap, end.cap2, x$start.captype, x$end.captype
  )
  keep <- !is.na(truncated$x)
  if (!any(keep)) {
    lines <- zeroGrob()
  } else if (x$constant) {
    x_new <- truncated$x[keep]
    y_new <- truncated$y[keep]
    id <- x$id[keep]
    all_id <- unique(id)
    gp <- lapply(x$gp, function(par) {
      if (length(par) == 1) {
        par
      } else {
        par[all_id]
      }
    })
    class(gp) <- 'gpar'
    id <- match(id, all_id)
    lines <- grob(
      x = unit(x_new, 'mm'), y = unit(y_new, 'mm'), id = id,
      id.lengths = NULL, arrow = x$arrow, name = x$name, gp = gp,
      vp = x$vp, cl = 'polyline'
    )
  } else {
    x0 <- truncated$x[!x$end]
    y0 <- truncated$y[!x$end]
    x1 <- truncated$x[!x$start]
    y1 <- truncated$y[!x$start]
    lines <- grob(
      x0 = unit(x0, 'mm'), y0 = unit(y0, 'mm'), x1 = unit(x1, 'mm'), id = x$id[!x$end],
      y1 = unit(y1, 'mm'), arrow = x$arrow, name = x$name, gp = x$gp,
      vp = x$vp, cl = 'segments'
    )
  }
  setChildren(x, gList(lines))
}
#' @importFrom grid is.unit unit
validate_cap <- function(cap, default.units, n) {
  if (is.null(cap)) {
    return()
  }
  if (!is.unit(cap)) cap <- unit(cap, default.units)
  rep(cap, length.out = n)
}

#' Create an unrooted layout using equal-angle or equal-daylight
#'
#' When drawing unrooted trees the standard dendrogram layout is a bad fit as it
#' implicitly creates a visual root node. Instead it is possible to spread the
#' leafs out on the plane without putting any special emphasis on a particular
#' node using an unrooted layout. The standard algorithm is the equal angle
#' algorithm, but it can struggle with optimising the leaf distribution for
#' large trees trees with very uneven branch length. The equal daylight
#' algorithm modifies the output of the equal angle algorithm to better disperse
#' the leaves, at the cost of higher computational cost and the possibility of
#' edge crossings for very large unbalanced trees. For standard sized trees the
#' daylight algorithm is far superior and not too heavy so it is the default.
#'
#' @note
#' Unrooted is a layout intended for undirected trees, that is, graphs with no
#' cycles. If the provided graph does not fit this format an attempt to convert
#' it to such a format will be made.
#'
#' @param graph A tbl_graph object
#' @param daylight Should equal-daylight adjustments be made
#' @param length An expression evaluating to the branch length of each edge
#' @param tolerance The threshold for mean angular adjustment before terminating
#' the daylight adjustment
#' @param rotation_mod A modifier for the angular adjustment of each branch. Set
#' it below 1 to let the daylight adjustment progress more slowly
#' @param maxiter The maximum number of iterations in the the daylight
#' adjustment
#' @param circular ignored
#'
#' @return A data.frame with the columns `x`, `y`, `circular`, `leaf` as well as
#' any information stored as node variables in the tbl_graph object.
#'
#' @references
#' Felsenstein, J. (2004) *Drawing Trees*, in Inferring Phylogenies. Sinauer
#' Assoc., pp 573-584
#'
#' @family layout_tbl_graph_*
#'
#' @importFrom igraph as.undirected gorder
layout_tbl_graph_unrooted <- function(graph, daylight = TRUE, length = NULL, tolerance = 0.05, rotation_mod = 1, maxiter = 100, circular = FALSE) {
  extra_data <- as_tibble(graph, active = 'nodes')
  graph <- as.undirected(graph)
  length <- enquo(length)
  length <- eval_tidy(length, .E())
  hierarchy <- tree_to_hierarchy(graph, 'out', seq_len(gorder(graph)), weight = NULL, length)
  layout <- unrooted(hierarchy$parent, hierarchy$order, hierarchy$height, daylight, tolerance, rotation_mod, maxiter)[-1, ]
  layout <- new_data_frame(list(
    x = layout[, 1],
    y = layout[, 2],
    circular = FALSE,
    leaf = degree(graph) == 1
  ))
  warn_dropped_vars(layout, extra_data)
  layout <- cbind(layout, extra_data[, !names(extra_data) %in% names(layout), drop = FALSE])
  layout
}

#' @rdname ggraph
#' @aliases layout_tbl_graph
#'
#' @importFrom igraph gorder
#' @export
#'
create_layout.tbl_graph <- function(graph, layout, circular = FALSE, ...) {
  graph <- mutate(activate(graph, 'nodes'), .ggraph.orig_index = seq_len(graph_order()))
  graph <- prepare_graph(graph, layout, ...)
  .register_graph_context(graph, free = TRUE)
  if (gorder(graph) == 0) {
    layout <- new_data_frame(list(x = numeric(), y = numeric(), circular = logical()))
    layout <- cbind(layout, .N())
  } else {
    layout <- layout_to_table(layout, graph, circular = circular, ...)
  }
  layout$.ggraph.index <- seq_len(nrow(layout))
  if (is.null(attr(layout, 'graph'))) {
    attr(layout, 'graph') <- graph
  }
  attr(layout, 'circular') <- circular
  class(layout) <- c(
    'layout_tbl_graph',
    'layout_ggraph',
    'data.frame'
  )
  check_layout(layout)
}
collect_edges.layout_tbl_graph <- function(layout) {
  gr <- attr(layout, 'graph')
  edges <- as_tibble(gr, active = 'edges')
  edges$circular <- rep(attr(layout, 'circular'), nrow(edges))
  as.data.frame(edges, stringsAsFactors = FALSE)
}
#' @importFrom igraph shortest_paths
#' @importFrom rlang enquo eval_tidy
collect_connections.layout_tbl_graph <- function(layout, from, to, weight = NULL, mode = 'all') {
  from <- match(from, layout$.ggraph.orig_index)
  to <- match(to, layout$.ggraph.orig_index)
  weight <- eval_tidy(enquo(weight), collect_edges(layout))
  if (is.null(weight)) {
    weight <- NA
  }
  graph <- attr(layout, 'graph')
  to_ind <- split(seq_along(to), from)
  connections <- lapply(seq_along(to_ind), function(i) {
    paths <- shortest_paths(graph, as.integer(names(to_ind)[i]), to[to_ind[[i]]], mode = mode, weights = weight)$vpath
    lapply(paths, as.numeric)
  })
  connections <- unlist(connections, recursive = FALSE)
  connections[match(seq_along(to), unlist(to_ind))]
}

# HELPERS -----------------------------------------------------------------

is.igraphlayout <- function(type) {
  if (type %in% igraphlayouts) {
    TRUE
  } else if (any(paste0(c('as_', 'in_', 'with_', 'on_'), type) %in% igraphlayouts)) {
    TRUE
  } else {
    FALSE
  }
}
as.igraphlayout <- function(type) {
  if (type %in% igraphlayouts) {
    layout <- type
  } else {
    new_type <- paste0(c('as_', 'in_', 'with_', 'on_'), type)
    type_ind <- which(new_type %in% igraphlayouts)
    if (length(type_ind) == 0) {
      stop('Cannot find igraph layout')
    }
    layout <- new_type[type_ind]
  }
  paste0('layout_', layout)
}
#' @importFrom igraph gorder permute
prepare_graph <- function(graph, layout, direction = 'out', ...) {
  if (!is.character(layout)) {
    return(graph)
  }
  is_hierarchy <- layout %in% c(
    'dendrogram',
    'treemap',
    'circlepack',
    'partition'
  )
  if (is_hierarchy || (layout == 'auto' && with_graph(graph, graph_is_tree() || graph_is_forest()))) {
    graph <- graph_to_tree(graph, mode = direction)
    graph <- permute(graph, match(seq_len(gorder(graph)), order(node_depth(graph, direction))))
  }
  as_tbl_graph(graph)
}
#' @importFrom igraph degree unfold_tree components induced_subgraph vertex_attr vertex_attr<- is.directed simplify
graph_to_tree <- function(graph, mode) {
  if (!is.directed(graph)) {
    stop('Graph must be directed')
  }
  graph <- simplify(graph, edge.attr.comb = 'first')
  parent_dir <- if (mode == 'out') 'in' else 'out'
  comp <- components(graph, 'weak')
  graphs <- lapply(seq_len(comp$no), function(i) {
    graph <- induced_subgraph(graph, which(comp$membership == i))
    n_parents <- degree(graph, mode = parent_dir)
    if (!any(n_parents == 0)) {
      stop('No root in graph. Provide graph with one parentless node')
    }
    if (any(n_parents > 1)) {
      message('Multiple parents. Unfolding graph')
      root <- which(degree(graph, mode = parent_dir) == 0)
      if (length(root) > 1) {
        message('Multiple roots in graph. Choosing the first')
        root <- root[1]
      }
      tree <- unfold_tree(graph, mode = mode, roots = root)
      vattr <- lapply(vertex_attr(graph), `[`, i = tree$vertex_index)
      vertex_attr(tree$tree) <- vattr
      graph <- tree$tree
    }
    as_tbl_graph(graph)
  })
  do.call(bind_graphs, graphs)
}
#' @importFrom igraph gorder as_edgelist delete_vertex_attr is.named
tree_to_hierarchy <- function(graph, mode, sort.by, weight, height = NULL) {
  if (is.named(graph)) graph <- delete_vertex_attr(graph, 'name')
  parent_col <- if (mode == 'out') 1 else 2
  node_col <- if (mode == 'out') 2 else 1
  edges <- as_edgelist(graph)
  hierarchy <- new_data_frame(list(parent = rep(0, gorder(graph))))
  hierarchy$parent[edges[, node_col]] <- edges[, parent_col]
  if (is.null(sort.by)) {
    hierarchy$order <- seq_len(nrow(hierarchy)) + 1
  } else {
    hierarchy$order <- order(sort.by) + 1
  }
  if (is.null(height)) {
    hierarchy$height <- 1
  } else {
    hierarchy$height <- 0
    hierarchy$height[edges[, node_col]] <- height
  }
  leaf <- degree(graph, mode = mode) == 0
  if (is.null(weight)) {
    hierarchy$weight <- 0
    hierarchy$weight[leaf] <- 1
  } else {
    if (!is.numeric(weight)) {
      stop('Weight must be numeric')
    }
    hierarchy$weight <- weight
    if (any(hierarchy$weight[!leaf] != 0)) {
      message('Non-leaf weights ignored')
    }
    if (any(hierarchy$weight[leaf] == 0)) {
      stop('Leafs must have a weight')
    }
    hierarchy$weight[!leaf] <- 0
  }
  hierarchy <- hierarchy[c(1, seq_len(nrow(hierarchy))), ]
  hierarchy$parent[1] <- -1
  hierarchy$order[1] <- 1
  hierarchy
}
#' @importFrom igraph bfs degree gorder
node_depth <- function(graph, mode) {
  mode_rev <- switch(
    mode,
    `in` = 'out',
    out = 'in',
    stop('unknown mode')
  )
  root <- which(degree(graph, mode = mode_rev) == 0)
  depth <- rep(NA_integer_, gorder(graph))
  for (i in root) {
    depth[!is.finite(depth)] <- unname(bfs(graph, root = i, unreachable = FALSE, dist = TRUE)$dist)[!is.finite(depth)]
  }
  depth
}
#' @importFrom rlang .data
add_direction <- function(graph, pos, direction = 'out') {
  if (igraph::gsize(graph) == 0) {
    return(graph)
  }
  graph <- activate(graph, 'edges')
  graph <- mutate(graph, direction = ifelse(pos$x[.data$to] < pos$x[.data$from], 'right', 'left'))
  if (direction == 'in') {
    graph <- mutate(graph, ifelse(.data$direction == 'left', 'right', 'left'))
  }
  graph <- mutate(graph, direction = factor(.data$direction))
  graph
}

#' Convert a layout to a table
#'
#' This generic takes care of dispatching various layout types (names,
#' functions, tables) to their respective functions that will return a valid
#' layout table.
#'
#' @param layout A supported object
#' @param graph A `tbl_graph`
#' @param ... passed on to implementations
#'
#' @return A valid data.frame
#'
#' @keywords internal
#' @export
layout_to_table <- function(layout, graph, ...) {
  UseMethod('layout_to_table')
}
#' @export
layout_to_table.default <- function(layout, graph, ...) {
  stop('Unknown layout', call. = FALSE)
}
#' @export
layout_to_table.character <- function(layout, graph, circular, ...) {
  if (is.igraphlayout(layout)) {
    layout_tbl_graph_igraph(graph, layout, circular, ...)
  } else {
    layout_fun <- get(paste0('layout_tbl_graph_', layout))
    layout_fun(graph, circular = circular, ...)
  }
}
#' @export
layout_to_table.matrix <- function(layout, graph, ...) {
  layout <- new_data_frame(list(x = layout[, 1], y = layout[, 2]))
  layout_to_table(layout, graph, ...)
}
#' @export
layout_to_table.data.frame <- function(layout, graph, ...) {
  cbind(layout, as_tibble(graph, active = 'nodes'))
}
#' @export
layout_to_table.function <- function(layout, graph, circular, ...) {
  layout <- if ('circular' %in% names(formals(layout))) {
    layout(graph, circular = circular, ...)
  } else {
    layout(graph, ...)
  }
  if (!is.tbl_graph(layout) && !is.data.frame(layout)) {
    layout <- tryCatch(
      as.data.frame(layout, stringsAsFactors = FALSE),
      error = function(e) {
        tryCatch(
          as_tbl_graph(layout),
          error = function(e) {
            stop('layout function must return an object coerceble to either a data.frame or tbl_graph', call. = FALSE)
          }
        )
      }
    )
  }
  if (is.tbl_graph(layout)) {
    graph <- layout
    layout <- as_tibble(graph, active = 'nodes')
    attr(layout, 'graph') <- graph
  } else {
    layout <- cbind(layout, as_tibble(graph, active = 'nodes'))
  }
  layout
}

igraphlayouts <- c(
  'as_bipartite',
  'as_star',
  'as_tree',
  'in_circle',
  'nicely',
  'with_dh',
  'with_drl',
  'with_gem',
  'with_graphopt',
  'on_grid',
  'with_mds',
  'with_sugiyama',
  'on_sphere',
  'randomly',
  'with_fr',
  'with_kk',
  'with_lgl'
)

#' Create edge extractor function
#'
#' This function returns another function that can extract edges from a
#' ggraph_layout object. The functionality of the returned function is decided
#' by the arguments to `get_edges`. The need for `get_edges` is mainly to
#' pass to the `data` argument of the different `geom_edge_*`
#' functions in order to present them with the right kind of data. In general
#' each `geom_edge_*` has the default set correctly so there is only need
#' to modify the data argument if parallel edges should be collapsed.
#'
#' @details
#' There are two types of return formats possible for the result of the returned
#' function:
#'
#' \describe{
#'   \item{short}{In this format each edge is described in one line in the
#'   format expected for [ggplot2::geom_segment()], that is, the start
#'   node position is encoded in the `x` and `y` column and the end
#'   node position is encoded in the `xend` and `yend` column. If node
#'   parameters are added to the edge the name of the parameters will be
#'   prefixed with `node1.` for the start node and `node2.` for the
#'   end node.}
#'   \item{long}{In this format each edge consists of two rows with matching
#'   `edge.id` value. The start and end position are both encoded in the
#'   `x` and `y` column. The relative position of the rows determines
#'   which is the start and end node, the first occurring being the start node.
#'   If node parameters are added to the edge data the name of the parameters
#'   will be prefixed with `node.`.}
#' }
#'
#' Node parameters are automatically added so it is possible to format edge
#' aesthetics according to start or end node parameters, or interpolate edge
#' aesthetics between start and end node parameters. Node parameters will be
#' prefixed to avoid name clash with edge parameters. The prefix depends on the
#' format (see above).
#'
#' If the graph is not simple (it contains at most one edge between each node
#' pair) it can be collapsed so either all edges between two nodes or all edges
#' of the same direction between two nodes are merged. The edge parameters are
#' taken from the first occurring edge, so if some more sophisticated summary is
#' needed it is suggested that the graph be tidied up before plotting with
#' ggraph.
#'
#' @param format Either `'short'` (the default) or `'long'`. See
#' details for a descriptions of the differences
#'
#' @param collapse Either `'none'` (the default), `'all'` or
#' `'direction'`. Specifies whether parallel edges should be merged. See
#' details for more information
#'
#' @param ... Additional data that will be cbind'ed together with the returned
#' edge data.
#'
#' @return A data.frame with columns dependent on format as well as the graph
#' type. In addition to the columns discussed in the details section,
#' the data.frame will always contain the columns `from`, `to` and
#' `circular`, the two former giving the indexes of the start and end node
#'  and the latter if the layout is circular (needed for correct formatting of
#'  some `geom_edge_*`). The graph dependent information is:
#'
#' \describe{
#'   \item{dendrogram}{A `label` column will hold the value of the
#'   `edgetext` attribute. In addition any value stored in the
#'   `edgePar` attribute will be added. Lastly a `direction` column
#'   will hold the relative position between the start and end nodes (needed for
#'   correct formatting of [geom_edge_elbow()]).}
#'   \item{igraph}{All edge attributes of the original graph object is added as
#'   columns to the data.frame}
#' }
#'
#' @family extractors
#'
#' @export
#'
get_edges <- function(format = 'short', collapse = 'none', ...) {
  if (!collapse %in% c('none', 'all', 'direction')) {
    stop('Collapse must be either "none", "all" or "direction"')
  }
  function(layout) {
    edges <- collect_edges(layout)
    edges <- switch(
      collapse,
      none = edges,
      all = collapse_all_edges(edges),
      direction = collapse_dir_edges(edges)
    )
    edges <- switch(
      format,
      short = format_short_edges(edges, layout),
      long = format_long_edges(edges, layout),
      stop('Unknown format. Use either "short" or "long"')
    )
    edges <- do.call(
      cbind,
      c(
        list(edges),
        lapply(list(...), rep, length.out = nrow(edges)),
        list(stringsAsFactors = FALSE)
      )
    )
    attr(edges, 'type_ggraph') <- 'edge_ggraph'
    edges
  }
}
#' @rdname internal_extractors
#' @export
collect_edges <- function(layout) {
  UseMethod('collect_edges', layout)
}
collect_edges.default <- function(layout) {
  attr(layout, 'edges')
}
check_short_edges <- function(edges) {
  if (!inherits(edges, 'data.frame')) {
    stop('edges must by of class data.frame', call. = FALSE)
  }
  if (!all(c('from', 'to', 'x', 'y', 'xend', 'yend', 'circular', 'edge.id') %in% names(edges))) {
    stop('edges must contain the columns from, to, x, y, xend, yend, circular, and edge.id', call. = FALSE)
  }
  if (!is.logical(edges$circular)) {
    stop('circular column must be logical', call. = FALSE)
  }
  edges
}
check_long_edges <- function(edges) {
  if (!inherits(edges, 'data.frame')) {
    stop('edges must by of class data.frame', call. = FALSE)
  }
  if (!all(c('edge.id', 'node', 'x', 'y', 'circular') %in% names(edges))) {
    stop('edges must contain the columns edge.id, node, x, y and circular', call. = FALSE)
  }
  if (!all(range(table(edges$edge.id)) == 2)) {
    stop('Each edge must consist of two rows')
  }
  if (!is.logical(edges$circular)) {
    stop('circular column must be logical', call. = FALSE)
  }
  edges
}
add_edge_coordinates <- function(edges, layout) {
  edges$x <- layout$x[edges$from]
  edges$y <- layout$y[edges$from]
  edges$xend <- layout$x[edges$to]
  edges$yend <- layout$y[edges$to]
  edges
}
format_short_edges <- function(edges, layout) {
  edges <- add_edge_coordinates(edges, layout)
  nodes1 <- layout[edges$from, , drop = FALSE]
  names(nodes1) <- paste0('node1.', names(nodes1))
  nodes2 <- layout[edges$to, , drop = FALSE]
  names(nodes2) <- paste0('node2.', names(nodes2))
  edges <- cbind(edges, nodes1, nodes2)
  rownames(edges) <- NULL
  edges$edge.id <- seq_len(nrow(edges))
  check_short_edges(edges)
}
format_long_edges <- function(edges, layout) {
  from <- cbind(
    edge.id = seq_len(nrow(edges)),
    node = edges$from,
    layout[edges$from, c('x', 'y')],
    edges
  )
  to <- cbind(
    edge.id = seq_len(nrow(edges)),
    node = edges$to,
    layout[edges$to, c('x', 'y')],
    edges
  )
  edges <- rbind_dfs(list(from, to))
  node <- layout[edges$node, , drop = FALSE]
  names(node) <- paste0('node.', names(node))
  edges <- cbind(edges, node)
  rownames(edges) <- NULL
  check_long_edges(edges[order(edges$edge.id), ])
}
complete_edge_aes <- function(aesthetics) {
  if (is.null(aesthetics)) {
    return(aesthetics)
  }
  if (any(names(aesthetics) == 'color')) {
    names(aesthetics)[names(aesthetics) == 'color'] <- 'colour'
  }
  expand_edge_aes(aesthetics)
}
expand_edge_aes <- function(x) {
  short_names <- names(x) %in% c(
    'colour', 'color', 'fill', 'linetype', 'shape', 'size', 'width', 'alpha'
  )
  names(x)[short_names] <- paste0('edge_', names(x)[short_names])
  x
}
#' @importFrom dplyr %>% group_by top_n ungroup
collapse_all_edges <- function(edges) {
  from <- pmin(edges$from, edges$to)
  to <- pmax(edges$to, edges$from)
  id <- paste(from, to, sep = '-')
  if (anyDuplicated(id)) {
    edges$.id <- id
    edges <- edges %>%
      group_by(.data$.id) %>%
      top_n(1) %>%
      ungroup()
  }
  as.data.frame(edges, stringsAsFactors = FALSE)
}
#' @importFrom dplyr %>% group_by top_n ungroup
collapse_dir_edges <- function(edges) {
  id <- paste(edges$from, edges$to, sep = '-')
  if (anyDuplicated(id)) {
    edges$.id <- id
    edges <- edges %>%
      group_by(.data$.id) %>%
      top_n(1) %>%
      ungroup()
  }
  as.data.frame(edges, stringsAsFactors = FALSE)
}

#' Place node to emphasize group structure
#'
#' This layout is optimised for drawing small-world types of graphs often found
#' in social networks, where distinct groups are still highly connected to the
#' remaining graph. Typical layouts struggle with this as they attempt to
#' minimise the edge length of all edges equally. The backbone layout is based
#' on weighing edges based on how well they hold together communities. The end
#' result is that communities tend to stick together despite high
#' interconnectivity.
#'
#' @param graph A tbl_graph object
#' @param keep The fraction of edges to use for creating the backbone
#' @param circular ignored
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#' Further an edge attribute called `backbone` is added giving whether the edge
#' was selected as backbone.
#'
#' @references
#' Nocaj, A., Ortmann, M., & Brandes, U. (2015). *Untangling the hairballs of
#' multi-centered, small-world online social media networks.* Journal of Graph
#' Algorithms and Applications: JGAA, 19(2), 595-618.
#'
#' @family layout_tbl_graph_*
#'
#' @author The underlying algorithm is implemented in the graphlayouts package
#' by David Schoch
#'
#' @importFrom graphlayouts layout_as_backbone
#'
layout_tbl_graph_backbone <- function(graph, keep = 0.2, circular = FALSE) {
  layout <- layout_as_backbone(graph, keep = keep, backbone = TRUE)
  xy <- layout$xy
  nodes <- new_data_frame(list(x = xy[,1], y = xy[,2]))
  nodes$circular <- FALSE
  extra_data <- as_tibble(graph, active = 'nodes')
  warn_dropped_vars(nodes, extra_data)
  nodes <- cbind(nodes, extra_data[, !names(extra_data) %in% names(nodes), drop = FALSE])
  graph <- activate(graph, 'edges')
  graph <- mutate(graph, backbone = seq_len(graph_size()) %in% layout$backbone)
  attr(nodes, 'graph') <- activate(graph, 'nodes')
  nodes
}

#' Place nodes according to their eigenvalues
#'
#' This layout is based on the idea of spectral layouts where node coordinates
#' are calculated directly by decomposing a matrix representation of the graph
#' and extracting the eigenvectors.
#'
#' @param graph A tbl_graph object
#' @param type The type of matrix to extract the eigenvectors from. Either
#' `'laplacian'` or `'adjacency'`
#' @param eigenvector The eigenvector to use for coordinates. Either `'smallest'`
#' or `'largest'`
#' @param circular ignored
#'
#' @return A data.frame with the columns `x`, `y`, `circular` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @family layout_tbl_graph_*
#'
#' @author The underlying algorithm is implemented in the graphlayouts package
#' by David Schoch
#'
#' @importFrom graphlayouts layout_with_eigen
layout_tbl_graph_eigen <- function(graph, type = 'laplacian', eigenvector = 'smallest', circular = FALSE) {
  type <- match.arg(type, c('laplacian', 'adjacency'))
  eigenvector <- match.arg(eigenvector, c('smallest', 'largest'))
  xy <- layout_with_eigen(graph, type = type, ev = eigenvector)
  nodes <- new_data_frame(list(x = xy[,1],y = xy[,2]))
  nodes$circular <- FALSE
  extra_data <- as_tibble(graph, active = 'nodes')
  warn_dropped_vars(nodes, extra_data)
  nodes <- cbind(nodes, extra_data[, !names(extra_data) %in% names(nodes), drop = FALSE])
  nodes
}

#' Draw edges as straight lines between nodes
#'
#' This geom draws edges in the simplest way - as straight lines between the
#' start and end nodes. Not much more to say about that...
#'
#' @section Edge variants:
#' Many geom_edge_* layers comes in 3 flavors depending on the level of control
#' needed over the drawing. The default (no numeric postfix) generate a number
#' of points (`n`) along the edge and draws it as a path. Each point along
#' the line has a numeric value associated with it giving the position along the
#' path, and it is therefore possible to show the direction of the edge by
#' mapping to this e.g. `colour = stat(index)`. The version postfixed with a
#' "2" uses the "long" edge format (see [get_edges()]) and makes it
#' possible to interpolate node parameter between the start and end node along
#' the edge. It is considerable less performant so should only be used if this
#' is needed. The version postfixed with a "0" draws the edge in the most
#' performant way, often directly using an appropriate grob from the grid
#' package, but does not allow for gradients along the edge.
#'
#' Often it is beneficial to stop the drawing of the edge before it reaches the
#' node, for instance in cases where an arrow should be drawn and the arrowhead
#' shouldn't lay on top or below the node point. geom_edge_* and geom_edge_*2
#' supports this through the start_cap and end_cap aesthetics that takes a
#' [geometry()] specification and dynamically caps the termini of the
#' edges based on the given specifications. This means that if
#' `end_cap = circle(1, 'cm')` the edges will end at a distance of 1cm even
#' during resizing of the plot window.
#'
#' All `geom_edge_*` and `geom_edge_*2` have the ability to draw a
#' label along the edge. The reason this is not a separate geom is that in order
#' for the label to know the location of the edge it needs to know the edge type
#' etc. Labels are drawn by providing a label aesthetic. The label_pos can be
#' used to specify where along the edge it should be drawn by supplying a number
#' between 0 and 1. The label_size aesthetic can be used to control the size of
#' the label. Often it is needed to have the label written along the direction
#' of the edge, but since the actual angle is dependent on the plot dimensions
#' this cannot be calculated beforehand. Using the angle_calc argument allows
#' you to specify whether to use the supplied angle aesthetic or whether to draw
#' the label along or across the edge.
#'
#' @section Edge aesthetic name expansion:
#' In order to avoid excessive typing edge aesthetic names are
#' automatically expanded. Because of this it is not necessary to write
#' `edge_colour` within the `aes()` call as `colour` will
#' automatically be renamed appropriately.
#'
#' @section Aesthetics:
#' `geom_edge_link` and `geom_edge_link0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_link2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - **group**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_link` and `geom_edge_link2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams ggplot2::geom_path
#' @inheritParams ggplot2::geom_text
#'
#' @param mapping Set of aesthetic mappings created by [ggplot2::aes()]
#' or [ggplot2::aes_()]. By default x, y, xend, yend, group and
#' circular are mapped to x, y, xend, yend, edge.id and circular in the edge
#' data.
#'
#' @param data The return of a call to `get_edges()` or a data.frame
#' giving edges in correct format (see details for for guidance on the format).
#' See [get_edges()] for more details on edge extraction.
#'
#' @param n The number of points to create along the path.
#'
#' @param label_colour The colour of the edge label. If `NA` it will use
#' the colour of the edge.
#'
#' @param label_alpha The opacity of the edge label. If `NA` it will use
#' the opacity of the edge.
#'
#' @param label_parse If `TRUE`, the labels will be parsed into expressions
#' and displayed as described in [grDevices::plotmath()].
#'
#' @param angle_calc Either 'none', 'along', or 'across'. If 'none' the label will
#' use the angle aesthetic of the geom. If 'along' The label will be written
#' along the edge direction. If 'across' the label will be written across the
#' edge direction.
#'
#' @param force_flip Logical. If `angle_calc` is either 'along' or 'across'
#' should the label be flipped if it is on it's head. Default to `TRUE`.
#'
#' @param label_dodge A [grid::unit()] giving a fixed vertical shift
#' to add to the label in case of `angle_calc` is either 'along' or 'across'
#'
#' @param label_push A [grid::unit()] giving a fixed horizontal shift
#' to add to the label in case of `angle_calc` is either 'along' or 'across'
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('bull') %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE)) %>%
#'   activate(edges) %>%
#'   mutate(class = sample(letters[1:3], n(), replace = TRUE))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_link(aes(alpha = stat(index)))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_link2(aes(colour = node.class))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_link0(aes(colour = class))
#' @rdname geom_edge_link
#' @name geom_edge_link
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatLink
#' @export
StatEdgeLink <- ggproto('StatEdgeLink', StatLink,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    data <- remove_loop(data)
    if (nrow(data) == 0) return(NULL)
    StatLink$setup_data(data, params)
  },
  default_aes = aes(filter = TRUE)
)
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatLink2
#' @export
StatEdgeLink2 <- ggproto('StatEdgeLink2', StatLink2,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    data <- remove_loop2(data)
    if (nrow(data) == 0) return(NULL)
    StatLink2$setup_data(data, params)
  },
  default_aes = aes(filter = TRUE)
)
#' @rdname geom_edge_link
#'
#' @importFrom ggforce StatLink
#' @export
geom_edge_link <- function(mapping = NULL, data = get_edges('short'),
                           position = 'identity', arrow = NULL, n = 100,
                           lineend = 'butt', linejoin = 'round', linemitre = 1,
                           label_colour = 'black', label_alpha = 1,
                           label_parse = FALSE, check_overlap = FALSE,
                           angle_calc = 'rot', force_flip = TRUE,
                           label_dodge = NULL, label_push = NULL,
                           show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(
    x = x, y = y, xend = xend, yend = yend, group = edge.id
  ))
  layer(
    data = data, mapping = mapping, stat = StatEdgeLink,
    geom = GeomEdgePath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, linejoin = linejoin,
        linemitre = linemitre, na.rm = FALSE, n = n,
        interpolate = FALSE,
        label_colour = label_colour, label_alpha = label_alpha,
        label_parse = label_parse, check_overlap = check_overlap,
        angle_calc = angle_calc, force_flip = force_flip,
        label_dodge = label_dodge, label_push = label_push, ...
      )
    )
  )
}
#' @rdname geom_edge_link
#'
#' @importFrom ggforce StatLink2
#' @export
geom_edge_link2 <- function(mapping = NULL, data = get_edges('long'),
                            position = 'identity', arrow = NULL, n = 100,
                            lineend = 'butt', linejoin = 'round', linemitre = 1,
                            label_colour = 'black', label_alpha = 1,
                            label_parse = FALSE, check_overlap = FALSE,
                            angle_calc = 'rot', force_flip = TRUE,
                            label_dodge = NULL, label_push = NULL,
                            show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y,
                                        group = edge.id))
  layer(
    data = data, mapping = mapping, stat = StatEdgeLink2,
    geom = GeomEdgePath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, linejoin = linejoin,
        linemitre = linemitre, na.rm = FALSE, n = n,
        interpolate = TRUE,
        label_colour = label_colour, label_alpha = label_alpha,
        label_parse = label_parse, check_overlap = check_overlap,
        angle_calc = angle_calc, force_flip = force_flip,
        label_dodge = label_dodge, label_push = label_push, ...
      )
    )
  )
}
#' @rdname geom_edge_link
#'
#' @importFrom ggforce StatLink2
#' @export
geom_edge_link0 <- function(mapping = NULL, data = get_edges(),
                            position = 'identity', arrow = NULL,
                            lineend = 'butt', show.legend = NA, ...) {
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y,
                                        xend = xend, yend = yend))
  layer(
    data = data, mapping = mapping, stat = StatFilter,
    geom = GeomEdgeSegment, position = position,
    show.legend = show.legend, inherit.aes = FALSE,
    params = expand_edge_aes(
      list(arrow = arrow, lineend = lineend, na.rm = FALSE, ...)
    )
  )
}

#' Draw edges in hive plots
#'
#' This geom is only intended for use together with the hive layout. It draws
#' edges between nodes as bezier curves, with the control points positioned at
#' the same radii as the start or end point, and at a distance defined by the
#' curvature argument.
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_hive` and `geom_edge_hive0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_hive2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overridden.
#'
#' - **x**
#' - **y**
#' - **group**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_hive` and `geom_edge_hive2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#'
#' @param strength The curvature of the bezier. Defines the distance from the
#' control points to the midpoint between the start and end node. 1 means the
#' control points are positioned halfway between the nodes and the middle of the
#' two axes, while 0 means it coincide with the nodes (resulting in straight
#' lines)
#'
#' @param curvature Deprecated. Use `strength` instead.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' # Plot the flare import graph as a hive plot
#' library(tidygraph)
#' flareGr <- as_tbl_graph(flare$imports) %>%
#'   mutate(
#'     type = dplyr::case_when(
#'       centrality_degree(mode = 'in') == 0 ~ 'Source',
#'       centrality_degree(mode = 'out') == 0 ~ 'Sink',
#'       TRUE ~ 'Both'
#'     )
#'   ) %>%
#'   activate(edges) %>%
#'   mutate(
#'     type = dplyr::case_when(
#'       grepl('flare.analytics', paste(.N()$name[from], .N()$name[to])) ~ 'Analytics',
#'       TRUE ~ 'Other'
#'     )
#'   )
#'
#' ggraph(flareGr, 'hive', axis = type) +
#'   geom_edge_hive(aes(colour = type), edge_alpha = 0.1) +
#'   coord_fixed()
#' @rdname geom_edge_hive
#' @name geom_edge_hive
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier
#' @export
StatEdgeHive <- ggproto('StatEdgeHive', StatBezier,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    data <- remove_loop(data)
    if (nrow(data) == 0) return(NULL)
    data$group <- make_unique(data$group)
    data2 <- data
    data2$x <- data2$xend
    data2$y <- data2$yend
    keep <- atan2(data$y, data$x) != atan2(data2$y, data2$x)
    create_hive_bezier(data[keep, ], data2[keep, ], params)
  },
  required_aes = c('x', 'y', 'xend', 'yend'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n', 'strength', 'curvature')
)
#' @rdname geom_edge_hive
#'
#' @export
geom_edge_hive <- function(mapping = NULL, data = get_edges(),
                           position = 'identity', arrow = NULL,
                           strength = 1, n = 100, lineend = 'butt',
                           linejoin = 'round', linemitre = 1,
                           label_colour = 'black', label_alpha = 1,
                           label_parse = FALSE, check_overlap = FALSE,
                           angle_calc = 'rot', force_flip = TRUE,
                           label_dodge = NULL, label_push = NULL,
                           show.legend = NA, ..., curvature) {
  if (!missing(curvature)) {
    .Deprecated(msg = 'The curvature argument has been deprecated in favour of strength')
    strength <- curvature * 2
  }
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(
    x = x, y = y, xend = xend, yend = yend, group = edge.id
  ))
  layer(
    data = data, mapping = mapping, stat = StatEdgeHive,
    geom = GeomEdgePath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, linejoin = linejoin,
        linemitre = linemitre, na.rm = FALSE, n = n,
        interpolate = FALSE, strength = strength,
        label_colour = label_colour, label_alpha = label_alpha,
        label_parse = label_parse, check_overlap = check_overlap,
        angle_calc = angle_calc, force_flip = force_flip,
        label_dodge = label_dodge, label_push = label_push, ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier2
#' @export
StatEdgeHive2 <- ggproto('StatEdgeHive2', StatBezier2,
  setup_data = function(data, params) {
    if (any(names(data) == 'filter')) {
      if (!is.logical(data$filter)) {
        stop('filter must be logical')
      }
      data <- data[data$filter, names(data) != 'filter']
    }
    data <- remove_loop2(data)
    if (nrow(data) == 0) return(NULL)
    data <- data[order(data$group), ]
    data2 <- data[c(FALSE, TRUE), ]
    data <- data[c(TRUE, FALSE), ]
    keep <- atan2(data$y, data$x) != atan2(data2$y, data2$x)
    create_hive_bazier(data[keep, ], data2[keep, ], params)
  },
  required_aes = c('x', 'y', 'group'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n', 'strength', 'curvature')
)
#' @rdname geom_edge_hive
#'
#' @export
geom_edge_hive2 <- function(mapping = NULL, data = get_edges('long'),
                            position = 'identity', arrow = NULL,
                            strength = 1, n = 100, lineend = 'butt',
                            linejoin = 'round', linemitre = 1,
                            label_colour = 'black', label_alpha = 1,
                            label_parse = FALSE, check_overlap = FALSE,
                            angle_calc = 'rot', force_flip = TRUE,
                            label_dodge = NULL, label_push = NULL,
                            show.legend = NA, ..., curvature) {
  if (!missing(curvature)) {
    .Deprecated(msg = 'The curvature argument has been deprecated in favour of strength')
    strength <- curvature * 2
  }
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y,
                                        group = edge.id))
  layer(
    data = data, mapping = mapping, stat = StatEdgeHive2,
    geom = GeomEdgePath, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, linejoin = linejoin,
        linemitre = linemitre, na.rm = FALSE, n = n,
        interpolate = TRUE, strength = strength,
        label_colour = label_colour, label_alpha = label_alpha,
        label_parse = label_parse, check_overlap = check_overlap,
        angle_calc = angle_calc, force_flip = force_flip,
        label_dodge = label_dodge, label_push = label_push, ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier0
#' @export
StatEdgeHive0 <- ggproto('StatEdgeHive0', StatBezier0,
  setup_data = function(data, params) {
    StatEdgeHive$setup_data(data, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'strength', 'curvature')
)
#' @rdname geom_edge_hive
#'
#' @export
geom_edge_hive0 <- function(mapping = NULL, data = get_edges(),
                            position = 'identity', arrow = NULL, strength = 1,
                            lineend = 'butt', show.legend = NA, ..., curvature) {
  if (!missing(curvature)) {
    .Deprecated(msg = 'The curvature argument has been deprecated in favour of strength')
    strength <- curvature * 2
  }
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(mapping, aes(x = x, y = y,
                                        xend = xend, yend = yend))
  layer(
    data = data, mapping = mapping, stat = StatEdgeHive0,
    geom = GeomEdgeBezier, position = position, show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list(
        arrow = arrow, lineend = lineend, na.rm = FALSE,
        strength = strength, ...
      )
    )
  )
}
create_hive_bezier <- function(from, to, params) {
  bezier_start <- seq(1, by = 4, length.out = nrow(from))
  from$index <- bezier_start
  to$index <- bezier_start + 3
  data2 <- from
  data3 <- to
  data2$index <- bezier_start + 1
  data3$index <- bezier_start + 2
  from_axis <- atan2(from$y, from$x)
  from_axis[from_axis < 0] <- from_axis[from_axis < 0] + 2 * pi
  to_axis <- atan2(to$y, to$x)
  to_axis[to_axis < 0] <- to_axis[to_axis < 0] + 2 * pi
  from_first <- ifelse(from_axis < to_axis,
    to_axis - from_axis < pi,
    to_axis - from_axis < -pi
  )
  middle_axis1 <- ifelse(from_first, from_axis, to_axis)
  middle_axis2 <- ifelse(from_first, to_axis, from_axis)
  middle_axis2 <- ifelse(middle_axis2 < middle_axis1, middle_axis2 + 2 * pi, middle_axis2)
  mean_axis <- (middle_axis2 - middle_axis1) / 2
  middle_axis1 <- middle_axis1 + mean_axis * params$strength / 2
  middle_axis2 <- middle_axis2 - mean_axis * params$strength / 2
  node_r2 <- sqrt(data2$x^2 + data2$y^2)
  node_r3 <- sqrt(data3$x^2 + data3$y^2)
  data2$x <- node_r2 * cos(ifelse(from_first, middle_axis1, middle_axis2))
  data2$y <- node_r2 * sin(ifelse(from_first, middle_axis1, middle_axis2))
  data3$x <- node_r3 * cos(ifelse(from_first, middle_axis2, middle_axis1))
  data3$y <- node_r3 * sin(ifelse(from_first, middle_axis2, middle_axis1))
  data <- rbind_dfs(list(from, data2, data3, to))
  data[order(data$index), names(data) != 'index']
}

#' Draw edges as curves of different curvature
#'
#' This geom draws edges as cubic beziers with the control point positioned
#' half-way between the nodes and at an angle dependent on the presence of
#' parallel edges. This results in parallel edges being drawn in a
#' non-overlapping fashion resembling the standard approach used in
#' [igraph::plot.igraph()]. Before calculating the curvature the edges
#' are sorted by direction so that edges going the same way will be adjacent.
#' This geom is currently the only choice for non-simple graphs if edges should
#' not be overplotted.
#'
#' @inheritSection geom_edge_link Edge variants
#' @inheritSection geom_edge_link Edge aesthetic name expansion
#'
#' @section Aesthetics:
#' `geom_edge_fan` and `geom_edge_fan0` understand the following
#' aesthetics. Bold aesthetics are automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **xend**
#' - **yend**
#' - **from**
#' - **to**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_fan2` understand the following aesthetics. Bold aesthetics are
#' automatically set, but can be overwritten.
#'
#' - **x**
#' - **y**
#' - **group**
#' - **from**
#' - **to**
#' - edge_colour
#' - edge_width
#' - edge_linetype
#' - edge_alpha
#' - filter
#'
#' `geom_edge_fan` and `geom_edge_fan2` furthermore takes the following
#' aesthetics.
#'
#' - start_cap
#' - end_cap
#' - label
#' - label_pos
#' - label_size
#' - angle
#' - hjust
#' - vjust
#' - family
#' - fontface
#' - lineheight
#'
#' @section Computed variables:
#'
#' \describe{
#'  \item{index}{The position along the path (not computed for the *0 version)}
#' }
#'
#' @inheritParams geom_edge_link
#' @inheritParams ggplot2::geom_path
#'
#' @param strength Modify the width of the fans `strength > 1` will create
#' wider fans while the reverse will make them more narrow.
#'
#' @param spread Deprecated. Use `strength` instead.
#'
#' @author Thomas Lin Pedersen
#'
#' @family geom_edge_*
#'
#' @examples
#' require(tidygraph)
#' gr <- create_notable('bull') %>%
#'   convert(to_directed) %>%
#'   bind_edges(data.frame(from = c(1, 2, 2, 3), to = c(2, 1, 3, 2))) %E>%
#'   mutate(class = sample(letters[1:3], 9, TRUE)) %N>%
#'   mutate(class = sample(c('x', 'y'), 5, TRUE))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_fan(aes(alpha = after_stat(index)))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_fan2(aes(colour = node.class))
#'
#' ggraph(gr, 'stress') +
#'   geom_edge_fan0(aes(colour = class))
#' @rdname geom_edge_fan
#' @name geom_edge_fan
#'
NULL

#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier
#' @export
StatEdgeFan <- ggproto(
  'StatEdgeFan',
  StatBezier,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data <- remove_loop(data)
    if (nrow(data) == 0) {
      return(data)
    }
    data$group <- make_unique(data$group)
    data2 <- data
    data2$x <- data2$xend
    data2$y <- data2$yend
    create_fans(data, data2, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'from', 'to'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n', 'strength', 'spread')
)
#' @rdname geom_edge_fan
#'
#' @export
geom_edge_fan <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  n = 100,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...,
  spread
) {
  if (!missing(spread)) {
    lifecycle::deprecate_warn(
      '2.0.0',
      'geom_edge_fan(spread)',
      'geom_edge_fan(strength)'
    )
    strength <- spread
  }
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      from = from,
      to = to,
      group = edge.id
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeFan,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = FALSE,
        strength = strength,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier2
#' @export
StatEdgeFan2 <- ggproto(
  'StatEdgeFan2',
  StatBezier2,
  setup_data = function(data, params) {
    data <- StatFilter$setup_data(data, params)
    data <- remove_loop2(data)
    if (nrow(data) == 0) {
      return(data)
    }
    data <- data[order(data$group), ]
    data2 <- data[c(FALSE, TRUE), ]
    data <- data[c(TRUE, FALSE), ]
    create_fans(data, data2, params)
  },
  required_aes = c('x', 'y', 'group', 'from', 'to'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'n', 'strength', 'spread')
)
#' @rdname geom_edge_fan
#'
#' @export
geom_edge_fan2 <- function(
  mapping = NULL,
  data = get_edges('long'),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  n = 100,
  lineend = 'butt',
  linejoin = 'round',
  linemitre = 1,
  label_colour = 'black',
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = 'rot',
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...,
  spread
) {
  if (!missing(spread)) {
    lifecycle::deprecate_warn(
      '2.0.0',
      'geom_edge_fan2(spread)',
      'geom_edge_fan2(strength)'
    )
    strength <- spread
  }
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      group = edge.id,
      from = from,
      to = to
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeFan2,
    geom = GeomEdgePath,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        linejoin = linejoin,
        linemitre = linemitre,
        n = n,
        interpolate = TRUE,
        strength = strength,
        label_colour = label_colour,
        label_alpha = label_alpha,
        label_parse = label_parse,
        check_overlap = check_overlap,
        angle_calc = angle_calc,
        force_flip = force_flip,
        label_dodge = label_dodge,
        label_push = label_push,
        ...
      )
    )
  )
}
#' @rdname ggraph-extensions
#' @format NULL
#' @usage NULL
#' @importFrom ggforce StatBezier0
#' @export
StatEdgeFan0 <- ggproto(
  'StatEdgeFan0',
  StatBezier0,
  setup_data = function(data, params) {
    StatEdgeFan$setup_data(data, params)
  },
  required_aes = c('x', 'y', 'xend', 'yend', 'from', 'to'),
  default_aes = aes(filter = TRUE),
  extra_params = c('na.rm', 'strength', 'spread')
)
#' @rdname geom_edge_fan
#'
#' @export
geom_edge_fan0 <- function(
  mapping = NULL,
  data = get_edges(),
  position = 'identity',
  arrow = NULL,
  strength = 1,
  lineend = 'butt',
  show.legend = NA,
  ...,
  spread
) {
  if (!missing(spread)) {
    lifecycle::deprecate_warn(
      '2.0.0',
      'geom_edge_fan0(spread)',
      'geom_edge_fan0(strength)'
    )
    strength <- spread
  }
  mapping <- complete_edge_aes(mapping)
  mapping <- aes_intersect(
    mapping,
    aes(
      x = x,
      y = y,
      xend = xend,
      yend = yend,
      from = from,
      to = to
    )
  )
  layer(
    data = data,
    mapping = mapping,
    stat = StatEdgeFan0,
    geom = GeomEdgeBezier,
    position = position,
    show.legend = show.legend,
    inherit.aes = FALSE,
    params = expand_edge_aes(
      list2(
        arrow = arrow,
        lineend = lineend,
        strength = strength,
        ...
      )
    )
  )
}
#' @importFrom dplyr group_by arrange mutate n ungroup transmute
create_fans <- function(from, to, params) {
  from$.id <- paste(pmin(from$from, to$to), pmax(from$from, to$to), sep = '-')
  from$.orig_ind <- seq_len(nrow(from))
  position <- from %>%
    group_by(.data$PANEL, .data$.id) %>%
    arrange(.data$from) %>%
    mutate(position = seq_len(n()) - 0.5 - n() / 2) %>%
    mutate(position = .data$position * ifelse(.data$from < .data$to, 1, -1)) %>%
    ungroup() %>%
    arrange(.data$.orig_ind) %>%
    transmute(position = .data$position)
  position <- position$position
  max_fans <- max(table(from$.id))
  from$.id <- NULL
  from$.orig_ind <- NULL
  mean_x <- rowMeans(cbind(from$x, to$x))
  mean_y <- rowMeans(cbind(from$y, to$y))
  step_x <- -(params$strength * (to$y - from$y) / (2 * max_fans))
  step_y <- params$strength * (to$x - from$x) / (2 * max_fans)
  data <- from
  data$x <- mean_x + step_x * position
  data$y <- mean_y + step_y * position
  bezier_start <- seq(1, by = 3, length.out = nrow(from))
  from$index <- bezier_start
  to$index <- bezier_start + 2
  data$index <- bezier_start + 1
  data <- vec_rbind(from, data, to)
  data[order(data$index), names(data) != 'index']
}

#' Place nodes in circles according to centrality measure
#'
#' This layout places nodes in circles with the radii relative to a given
#' centrality measure. Under the hood it use stress majorisation to place nodes
#' optimally given the radius constraint.
#'
#' @param graph A tbl_graph object
#' @param centrality An expression evaluating to a centrality measure for the
#' nodes. See the different `centrality_*()` algorithms in tidygraph for a
#' selection.
#' @param scale Should the centrality measure be scaled between 0 and 100
#' @param tseq Transitioning steps
#' @param group An expression evaluating to a grouping of the nodes. If given
#' the layout will keep grouped nodes within an angle range of the origin
#' @param shrink shrink the reserved angle range for a group to increase the
#' gaps between groups
#' @inheritParams layout_tbl_graph_stress
#'
#' @return A data.frame with the columns `x`, `y`, `circular`, `centrality` as
#' well as any information stored as node variables in the tbl_graph object.
#'
#' @references
#' Brandes, U., & Pich, C. (2011). *More flexible radial layout.* Journal of
#' Graph Algorithms and Applications, 15(1), 157-173.
#'
#' @family layout_tbl_graph_*
#'
#' @author The underlying algorithm is implemented in the graphlayouts package
#' by David Schoch
#'
#' @importFrom graphlayouts layout_with_centrality layout_with_centrality_group
#' @importFrom rlang eval_tidy enquo
layout_tbl_graph_centrality <- function(graph, centrality, scale = TRUE,
                                        niter = 500, tolerance = 1e-4,
                                        tseq = seq(0,1,0.2), group = NULL,
                                        shrink = 10, circular = FALSE) {
  centrality <- eval_tidy(enquo(centrality), .N())
  group <- eval_tidy(enquo(group), .N())
  if (is.null(group)) {
    xy <- layout_with_centrality(graph, cent = centrality, scale = scale,
                                 iter = niter, tol = tolerance, tseq = tseq)
  } else {
    xy <- layout_with_centrality_group(graph, cent = centrality, group = group,
                                       shrink = shrink, scale = scale,
                                       iter = niter, tol = tolerance,
                                       tseq = tseq)
  }

  nodes <- data_frame0(x = xy[,1],y = xy[,2], centrality = centrality,
                       group = group, circular = FALSE)
  combine_layout_nodes(nodes, as_tibble(graph, active = 'nodes'))
}

---
title: "Nodes"
author: "Thomas Lin Pedersen"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Nodes}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include=FALSE}
set.seed(2022)
```

Nodes in a network are the entities that are connected. Sometimes these are also
referred to as vertices, but `ggraph` has opted for this nomenclature and uses it
consistently. While the nodes in a graph are the abstract concepts of entities, 
and the layout is their physical placement, the node geoms are the visual 
manifestation of the entities. Conceptually one can simply think of it in terms
of a scatter plot --- the layout provides the x and y coordinates, and these can
be used to draw nodes in different ways in the plotting window. Actually, due
to the design of `ggraph` the standard *scatterplot-like* geoms from `ggplot2`
can be used directly for plotting nodes:

```{r, message=FALSE}
library(ggraph)
library(tidygraph)

set_graph_style(plot_margin = margin(1,1,1,1))
gr <- as_tbl_graph(highschool)

ggraph(gr, layout = 'kk') + 
  geom_point(aes(x = x, y = y))
```

The reason this works is that, as discussed in the Layout vignette, layouts 
return a `data.frame` of node positions and metadata and this is used as the 
default plot data:

```{r}
head(create_layout(gr, layout = 'kk'))
```

## `geom_node_*()`
While usage of the default `ggplot2` is absolutely allowed, `ggraph` comes with
its own set of node geoms. Many of these are direct translations of `ggplot2`
own geoms like `geom_point()` so one could wonder why bother to use them. 

The first reason is to provide clear code. It is not apparent anywhere that the
standard geoms are addressing the nodes and using `geom_node_*()` makes it clear
that this layer will draw nodes. 

The second reason is that it will save typing.
Since `ggraph` is in control of the shape of the input data through the layout
calculations, it knows that *x* and *y* position is encoded in an `x` and `y` 
column. This means that `geom_node_*` can default the x and y aesthetics so 
there's no need to type them:

```{r}
ggraph(gr, layout = 'kk') + 
  geom_node_point()
```

sometimes there is a need for addressing the x and y aesthetics, which is still
possible, for instance if a partition layout should be inverted:

```{r}
gr <- tbl_graph(flare$vertices, flare$edges)

ggraph(gr, layout = 'partition') + 
  geom_node_tile(aes(y = -y, fill = depth))
```

of course this could also be accomplished by reversing the y-axis using 
`scale_y_reverse()` so this is just to illustrate that the defaults are easily
overwritten if needed.

The third reason is for the added functionality. All `ggraph` geoms get a 
`filter` aesthetic that allows you to quickly filter the input data. The use of
this can be illustrated when plotting a tree:

```{r}
ggraph(gr, layout = 'dendrogram', circular = TRUE) + 
  geom_edge_diagonal() + 
  geom_node_point(aes(filter = leaf)) + 
  coord_fixed()
```

In the above plot only the terminal nodes are drawn by filtering on the logical
leaf column provided by the dendrogram layout.

## The different node geoms
The usual suspects are of course provided in the form of `geom_node_point()` 
(showcased above), `geom_node_text()`, and `geom_node_label()`. These work as
expected, taking in the usual aesthetics (plus *filter*). Only x and y are
defaulted so everything else must be provided e.g. label which does not default
to the `name` column like is done in `igraph`. One feature sets 
`geom_node_text()` and `geom_node_label()` apart from their `ggplot2` 
counterparts: both have a `repel` argument that, when set to `TRUE`, will use 
the repel functionality provided by the 
[ggrepel](https://CRAN.R-project.org/package=ggrepel) package to avoid 
overlapping text. There is also `geom_node_voronoi()` that plots nodes as cells
from a voronoi tesselation. This is useful for e.g. showing dominance of certain
node types in an area as overlapping is avoided:

```{r}
graph <- create_notable('meredith') |> 
  mutate(group = sample(c('A', 'B'), n(), TRUE))

ggraph(graph, 'stress') + 
  geom_node_voronoi(aes(fill = group), max.radius = 1) + 
  geom_node_point() + 
  geom_edge_link() + 
  coord_fixed()
```

Apart from these geoms there's a set of geoms mainly useful for spatial
node layouts such as treemaps, partition, circle packing, and fabric. 
`geom_node_tile()` and `geom_node_range()` are the `ggraph` counterpart to 
`ggplot2`s `geom_tile()` and `geom_linerange()` while 
`geom_node_circle()` and `geom_node_arc_bar()` maps to `ggforce`s 
`geom_circle()` and `geom_arc_bar()`. Collective for these is that the spatial
dimensions of the geoms (e.g. radius, width, and height) are precalculated by
their intended layouts and defaulted by the geoms:

```{r}
ggraph(gr, layout = 'treemap', weight = size) + 
  geom_node_tile(aes(fill = depth))
```

All spatial node geoms will be center-based, meaning that the x and y value of
the layout will refer to the center of the layout and not e.g. the bottom-left
corner. This makes it easier to add labels to spatial layouts as well as using
spatial layouts in a non-spatial way:

```{r}
l <- ggraph(gr, layout = 'partition', circular = TRUE)
l + geom_node_arc_bar(aes(fill = depth)) + 
  coord_fixed()
```

```{r}
l + geom_edge_diagonal() + 
  geom_node_point(aes(colour = depth)) + 
  coord_fixed()
```

More node geoms are sure to appear in `ggraph` with time but they will generally
be quite easily comprehensible due to their strong resemblance to the standard
`ggplot2` geoms. After all it is just points on a plane...

## Want more?
Check out the other vignettes for more information on how to specify 
[layouts](Layouts.html) and draw [edges](Edges.html)...

---
title: "Edges"
author: "Thomas Lin Pedersen"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Edges}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include=FALSE}
set.seed(2022)
```

If the natural `ggplot2` equivalent to nodes is `geom_point()`, then surely the
equivalent to edges must be `geom_segment()`? Well, sort of, but there's a bit 
more to it than that.

![One does not simply draw a line between two nodes](edge_meme_wide.jpg)

While nodes are the sensible, mature, and predictably geoms, edges are the edgy 
(sorry), younger cousins that pushes the boundaries. To put it bluntly: 

> On the ggraph savannah you definitely want to be an edge!

## Meet the `geom_edge_*()` family
While the introduction might feel a bit over-the-top it is entirely true. An 
edge is an abstract concept denoting a relationship between two entities. A 
straight line is simply just one of many ways this relationship can be 
visualised. As we saw when [discussing nodes](Nodes.html)
sometimes it is not drawn at all but impied using containment or position 
(treemap, circle packing, and partition layouts), but more often it is shown
using a line of some sort. This use-case is handled by the large family of edge
geoms provided in `ggraph`. Some of the edges are general while others are 
dedicated to specific layouts. Let's creates some graphs for illustrative purposes
first:

```{r, message=FALSE}
library(ggraph)
library(tidygraph)
library(purrr)
library(rlang)

set_graph_style(plot_margin = margin(1,1,1,1))
hierarchy <- as_tbl_graph(hclust(dist(iris[, 1:4]))) |> 
  mutate(Class = map_bfs_back_chr(node_is_root(), .f = function(node, path, ...) {
    if (leaf[node]) {
      as.character(iris$Species[as.integer(label[node])])
    } else {
      species <- unique(unlist(path$result))
      if (length(species) == 1) {
        species
      } else {
        NA_character_
      }
    }
  }))

hairball <- as_tbl_graph(highschool) |> 
  mutate(
    year_pop = map_local(mode = 'in', .f = function(neighborhood, ...) {
      neighborhood %E>% pull(year) |> table() |> sort(decreasing = TRUE)
    }),
    pop_devel = map_chr(year_pop, function(pop) {
      if (length(pop) == 0 || length(unique(pop)) == 1) return('unchanged')
      switch(names(pop)[which.max(pop)],
             '1957' = 'decreased',
             '1958' = 'increased')
    }),
    popularity = map_dbl(year_pop, ~ .[1]) %|% 0
  ) |> 
  activate(edges) |> 
  mutate(year = as.character(year))
```

### Link
While you don't have to use a straight line for edges it is certainly possible
and `geom_edge_link()` is here to serve your needs:

```{r}
ggraph(hairball, layout = 'stress') + 
  geom_edge_link(aes(colour = year))
```

There's really not much more to it --- every edge is simply a straight line 
between the terminal nodes. Moving on...

### Fan
Sometimes the graph is not simple, i.e. it has multiple edges between the same
nodes. Using links is a bad choice here because edges will overlap and the
viewer will be unable to discover parallel edges. `geom_edge_fan()` got you 
covered here. If there are no parallel edges it behaves like `geom_edge_link()`
and draws a straight line, but if parallel edges exists it will spread them out
as arcs with different curvature. Parallel edges will be sorted by 
directionality prior to plotting so edges flowing in the same direction will be
plotted together:

```{r}
ggraph(hairball, layout = 'stress') + 
  geom_edge_fan(aes(colour = year))
```

### Parallel
An alternative to `geom_edge_fan()` is `geom_edge_parallel()`. It will draw 
edges as straight lines but in the case of multi-edges it will offset each edge 
a bit so they run parallel to each other. As with `geom_edge_fan()` the edges 
will be sorted by direction first. The offset is done at draw time and will thus
remain constant even during resizing:

```{r}
ggraph(hairball, layout = 'stress') + 
  geom_edge_parallel(aes(colour = year))
```

### Loops
Loops cannot be shown with regular edges as they have no length. A dedicated 
`geom_edge_loop()` exists for these cases:

```{r}
# let's make some of the student love themselves
loopy_hairball <- hairball |> 
  bind_edges(tibble::tibble(from = 1:5, to = 1:5, year = rep('1957', 5)))
ggraph(loopy_hairball, layout = 'stress') + 
  geom_edge_link(aes(colour = year), alpha = 0.25) + 
  geom_edge_loop(aes(colour = year))
```

The direction, span, and strength of the loop can all be controlled, but in 
general loops will add a lot of visual clutter to your plot unless the graph is
very simple.

### Density
This one is definitely strange, and I'm unsure of it's usefulness, but it is here
and it deserves an introduction. Consider the case where it is of interest to
see which types of edges dominates certain areas of the graph. You can colour
the edges, but edges can tend to get overplotted, thus reducing readability.
`geom_edge_density()` lets you add a shading to your plot based on the density
of edges in a certain area:

```{r}
ggraph(hairball, layout = 'stress') + 
  geom_edge_density(aes(fill = year)) + 
  geom_edge_link(alpha = 0.25)
```

### Arcs
While some insists that curved edges should be used in standard *"hairball"* 
graph visualisations it really is a poor choice, as it increases overplotting
and decreases interpretability for virtually no gain (unless complexity is 
your thing). That doesn't mean arcs have no use in graph visualizations. Linear 
and circular layouts can benefit greatly from them and `geom_edge_arc()` is 
provided precisely for this scenario:

```{r}
ggraph(hairball, layout = 'linear') + 
  geom_edge_arc(aes(colour = year))
```

Arcs behave differently in circular layouts as they will always bend towards
the center no matter the direction of the edge (the same thing can be achieved
in a linear layout by setting `fold = TRUE`).

```{r}
ggraph(hairball, layout = 'linear', circular = TRUE) + 
  geom_edge_arc(aes(colour = year)) + 
  coord_fixed()
```

### Bundling
Edge bundling is a technique to reduce clutter in a network visualization by
bundling edges that flows in the same direction. There are various ways of doing
this, many with heavy computational cost and the potential to mislead. The 
technique were initially confined to connections between nodes with a 
[hierarchical structure](#Connections) but has been expanded to general graphs.
ggraph provides 3 different bundling geoms with various up- and downsides. 

#### Force directed
This is perhaps the most classic. It treats the edges as an array of points with
the propensity to attract each other if edges are parallel. It suffers from bad
performance (though the edge bundling geoms uses memoisation to avoid 
recomputations) and can also be misleading as it doesn't use the underlying 
topology of the graph to determine if edges should be bundled, only whether they
are parallel.

```{r}
ggraph(hairball) + 
  geom_edge_bundle_force(n_cycle = 2, threshold = 0.4)
```

#### Edge path
An alternative is to let the edges follow the shortest paths rather than 
attract each other. This means the topology is being used in the bundling and
in theory lead to less misleading results. It also has the upside of being 
faster. The algorithm is iterative so that if an edge has been bundled it is
deleted from the graph where the shortest path is being searched in. In this
way the edges naturally converge towards a few "highways".

```{r}
ggraph(hairball) + 
  geom_edge_bundle_path()
```

#### Minimal
In the same vein as edge path bundling but even simpler, you can use the minimal
spanning tree of the graph as the scaffold to bundle edges along. As such, it 
changes to the hierarchical edge bundling approach, just with an implicit 
hierarchy calculated on the graph. This method is very fast but does create bias
in the output as edges will (obviously) travel along the minimal spanning tree 
thus amplifying that topology.

```{r}
ggraph(hairball) + 
  geom_edge_bundle_minimal()
```


### Elbow
Aah... The classic dendrogram with its right angle bends. Of course such
visualizations are also supported with the `geom_edge_elbow()`. It goes without
saying that this type of edge requires a layout that flows in a defined 
direction, such as a tree:

```{r}
ggraph(hierarchy, layout = 'dendrogram', height = height) + 
  geom_edge_elbow()
```

### Diagonals
If right angles aren't really your thing `ggraph` provides a smoother version in
the form of `geom_edge_diagonal()`. This edge is a quadratic bezier with control
points positioned at the same x-value as the terminal nodes and halfway 
in-between the nodes on the y-axis. The result is more organic than the elbows:

```{r}
ggraph(hierarchy, layout = 'dendrogram', height = height) + 
  geom_edge_diagonal()
```

It tends to look a bit weird with hugely unbalanced trees so use with care...

### Bends
An alternative to diagonals are bend edges which are elbow edges with a 
smoothed corner. It is implemented as a quadratic bezier with control points at
the location of the expected elbow corner:

```{r}
ggraph(hierarchy, layout = 'dendrogram', height = height) + 
  geom_edge_bend()
```

### Hive
This is certainly a very specific type of edge, intended only for use with hive
plots. It draws edges as quadratic beziers with control point positioned 
perpendicular to the axes of the hive layout:

```{r}
ggraph(hairball, layout = 'hive', axis = pop_devel, sort.by = popularity) + 
  geom_edge_hive(aes(colour = year)) + 
  geom_axis_hive(label = FALSE) + 
  coord_fixed()
```

### Span
As with the hive edge the `geom_edge_span()` is made in particular for a 
specific layout - the fabric layout. It draws the edge as a vertical line 
connecting the horizontal node lines of the layout, potentially with a terminal
shape.

```{r}
ggraph(hairball, layout = 'fabric', sort.by = node_rank_fabric()) + 
  geom_node_range(colour = 'grey') + 
  geom_edge_span(end_shape = 'circle') + 
  coord_fixed()
```

### Point and tile
It may seem weird to have edge geoms that doesn't have any span, but the matrix
layout calls for exactly that. The terminal nodes of the edge are determined by
the vertical and horizontal position of the mark, and for that reason the geom 
doesn't need any extend. The point and tile geoms serve the same purpose but are
simply different geometry types:

```{r}
ggraph(hairball, layout = 'matrix', sort.by = bfs_rank()) + 
  geom_edge_point() + 
  coord_fixed()
```

```{r}
ggraph(hairball, layout = 'matrix', sort.by = bfs_rank()) + 
  geom_edge_tile() + 
  coord_fixed()
```

## The three types of edge geoms
Almost all edge geoms comes in three variants. The basic variant (no suffix) as 
well as the variant suffixed with 2 (e.g. `geom_edge_link2()`) calculates a 
number (`n`) of points along the edge and draws it as a path. The variant 
suffixed with 0 (e.g. `geom_edge_diagonal0()`) uses the build in grid grobs to
draw the edges directly (in case of a diagonal it uses `bezierGrob()`). It might
seem strange to have so many different implementations of the same geoms but 
there's a reason to the insanity...

### Base variant
The basic edge geom is drawn by calculating a number of points along the edge
path and draw a line between these. This means that you're in control of the
detail level of curved edges and that all complex calculations happens up front.
Generally you will see better performance using the base variant rather than the
0-variant that uses grid grobs, unless you set the number of points to calculate
to something huge (50--100 is usually sufficient for a smooth look). Apart from
better performance you also get a nice bonus (you actually get several, but only
one is discussed here): The possibility of drawing a gradient along the edge.
Each calculated point gets an index value between 0 and 1 that specifies how
far along the edge it is positioned and this value can be used to e.g. map to
an alpha level to show the direction of the edge:

```{r}
ggraph(hairball, layout = 'linear') + 
  geom_edge_arc(aes(colour = year, alpha = after_stat(index))) + 
  scale_edge_alpha('Edge direction', guide = 'edge_direction')
```

### 2-variant
Like the base variant the 2-variant calculates points along the edge and draws
a path along them. The difference here is that in this variant you can map node
attributes to the edge and the aesthetics are then interpolated along the edge.
This is easier to show than to explain:

```{r}
ggraph(hierarchy, layout = 'dendrogram', height = height) + 
  geom_edge_elbow2(aes(colour = node.Class))
```

There are considerably more computation going on than in the base variant so 
unless you need to interpolate values between the terminal nodes you should go
with the base variant.

### 0-variant
This is, sadly, the boring one at the end. You don't get the luxury of smooth
gradients over the edge and often you have a considerably worse performance.
What you gain though is tack sharp resolution in the curves so if this is of 
utmost importance you are covered by this variant.

## Edge strength
Many of the edge geoms takes a strength argument that denotes their deviation 
from a straight line. Setting `strength = 0` will always result in a straight
line, while `strength = 1` is the default look. Anything in between can be used
to modify the look of the edge, while values outside that range will probably 
result in some weird looks. Some examples are shown below:

```{r}
small_tree <- create_tree(5, 2)

ggraph(small_tree, 'dendrogram') + 
  geom_edge_elbow(strength = 0.75)
```

```{r}
ggraph(small_tree, 'dendrogram') + 
  geom_edge_diagonal(strength = 0.5)
```


## Decorating edges
An edge is so much more than a line... Well at least it is also potentially an
arrow and a label. This section will go into how these can be added. To clearly
see the effect here we will use a slightly simpler graph

```{r}
# Random names - I swear
simple <- create_notable('bull') |> 
  mutate(name = c('Thomas', 'Bob', 'Hadley', 'Winston', 'Baptiste')) |> 
  activate(edges) |> 
  mutate(type = sample(c('friend', 'foe'), 5, TRUE))
```

### Arrows
While we saw above that direction can be encoded as a gradient, the good old 
arrow is still available. As with the standard `ggplot2` geoms an arrow can be 
added using the arrow argument:

```{r}
ggraph(simple, layout = 'graphopt') + 
  geom_edge_link(arrow = arrow(length = unit(4, 'mm'))) + 
  geom_node_point(size = 5)
```

I hope you think *Ugh* at the sight of this. The edges naturally extend to the
node center and nodes are thus drawn on top of the arrow heads. There's a 
solution to this in the form of the `start_cap` and `end_cap` aesthetics in the
base and 2-variant edge geoms (sorry 0-variant). This can be used to start and
stop the edge drawing at an absolute distance from the terminal nodes. Watch this:

```{r}
ggraph(simple, layout = 'graphopt') + 
  geom_edge_link(arrow = arrow(length = unit(4, 'mm')), 
                 end_cap = circle(3, 'mm')) + 
  geom_node_point(size = 5)
```

Using the `circle()`, `square()`, `ellipsis()`, and `rectangle()` helpers it is
possible to get a lot of control over how edges are capped at either end. This 
works for any edge, curved or not:

```{r}
ggraph(simple, layout = 'linear', circular = TRUE) + 
  geom_edge_arc(arrow = arrow(length = unit(4, 'mm')), 
                start_cap = circle(3, 'mm'),
                end_cap = circle(3, 'mm')) + 
  geom_node_point(size = 5) + 
  coord_fixed()
```

When plotting node labels you often want to avoid that incoming and outgoing 
edges overlaps with the labels. `ggraph` provides a helper that calculates the 
bounding rectangle of the labels and cap edges based on that:

```{r}
ggraph(simple, layout = 'graphopt') + 
  geom_edge_link(aes(start_cap = label_rect(node1.name),
                     end_cap = label_rect(node2.name)), 
                 arrow = arrow(length = unit(4, 'mm'))) + 
  geom_node_text(aes(label = name))
```

The capping of edges is dynamic and responds to resizing of the plot so the 
absolute size of the cap areas are maintained at all time.

#### A quick note on directionality
In `ggraph` there is no such thing as an undirected graph. Every edge has a 
start and an end node. For undirected graphs the start and end of edges is 
arbitrary but still exists and it is thus possible to add arrowheads to 
undirected graphs as well. This should not be done of course, but this is the
responsibility of the user as `ggraph` does not make any checks during 
rendering.

### Labels
You would expect that edge labels would be their own geom(s), but `ggraph` 
departs from the stringent grammar interpretation here. This is because the 
label placement is dependent on the choice of edge. Because of this edge 
labeling is bundled with each edge geom (but not the 0-variant) through the 
label aesthetic

```{r}
ggraph(simple, layout = 'graphopt') + 
  geom_edge_link(aes(label = type), 
                 arrow = arrow(length = unit(4, 'mm')), 
                 end_cap = circle(3, 'mm')) + 
  geom_node_point(size = 5)
```

Usually you would like the labels to run along the edges, but providing a 
fixed angle will only work at a very specific aspect ratio. Instead `ggraph`
offers to calculate the correct angle dynamically so the labels always runs
along the edge. Furthermore it can offset the label by an absolute length:

```{r}
ggraph(simple, layout = 'graphopt') + 
  geom_edge_link(aes(label = type), 
                 angle_calc = 'along',
                 label_dodge = unit(2.5, 'mm'),
                 arrow = arrow(length = unit(4, 'mm')), 
                 end_cap = circle(3, 'mm')) + 
  geom_node_point(size = 5)
```

`ggraph` offers a lot of additional customization of the edge labels but this
shows the main features. As with arrowheads labels can severely clutter your
visualization so it is only advisable on very simple graphs.

## Connections
The estranged cousin of edges are connections. While edges show the relational 
nature of the nodes in the graph structure, connections connect nodes that are 
not connected in the graph. This is done by finding the shortest path between
the two nodes. Currently the only connection geom available is 
`geom_conn_bundle()` that implements the hierarchical edge bundling technique:

```{r}
flaregraph <- tbl_graph(flare$vertices, flare$edges)
from <- match(flare$imports$from, flare$vertices$name)
to <- match(flare$imports$to, flare$vertices$name)
ggraph(flaregraph, layout = 'dendrogram', circular = TRUE) + 
  geom_conn_bundle(data = get_con(from = from, to = to), alpha = 0.1) + 
  coord_fixed()
```

The connection concept is underutilized at the moment but I expect to add more
support for this in coming releases.

## Want more?
Check out the other vignettes for more information on how to specify 
[layouts](Layouts.html) and draw [nodes](Nodes.html)...

---
title: "tidygraph and ggraph"
author: "Thomas Lin Pedersen"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
    %\VignetteIndexEntry{tidygraph and ggraph}
    %\VignetteEngine{knitr::rmarkdown}
    %\VignetteEncoding{UTF-8}
---

```{r, include=FALSE}
library(ggraph)
set_graph_style(family = 'Arial', size = 7, foreground = 'lightgrey', plot_margin = margin(0, 0, 0, 0))
set.seed(2022)
```


Following `ggraph` v2.0 the `tidygraph` package has been used as the central 
data structure. The integration goes beyond using it as a simple background 
engine and has deep implications for what you can do and how you can do it when
plotting with `ggraph`. This vignette will go into the details of the 
`ggraph`/`tidygraph` relationship — buckle up...

## Supported data structures
Prior to v2 `ggraph` had two main supported data structures, namely `dendrogram`
and `igraph`. In addition `hclust` and `network` were supported by automatic 
conversion to `dendrogram` and `igraph` respectively. Each of the two data 
structures had their own layouts and under the hood two different set of 
functionality had to be maintained to extract nodes and edges etc. In v2 and 
going forward this has been simplified and `ggraph` now uses only `tbl_graph` as
a graph representation. This does not mean that you're out of luck if you're not
buying into the whole `tidygraph` idea. Every object supported by `tidygraph` is
supported directly in `ggraph` by automatic conversion to `tbl_graph`. This 
means that `igraph`, `dendrogram`, `hclust`, and `network` is still supported in
addition to `data.tree`, `phylo`, and `graph` as well as a number of 
`data.frame`, `matrix`, and `list` representations.

The change has reduced internal code complexity quite a bit which will make it
easier to provide new features in future. From a user point of view it has the
benefit of simplifying the API in that `ggraph` doesn't really care what type of
network object you pass in - every layout and geom just works with every data
structure. Further, it simplifies how to add `ggraph` support to additional data
structures: just write an `as_tbl_graph()` method for the class!. Due to the
large support of classes and data structures in `tidygraph` this should 
relatively straightforward. If you're developer of a package that defines a 
custom network class simply export an `as_tbl_graph()` method for the class to
gain native `ggraph` (and `tidygraph`) support, or add it directly to 
`tidygraph` through a [PR](https://github.com/thomasp85/tidygraph/pulls).

This simplification for both me and the users have really been the motivation
for the integration of `tidygraph` but as it were it has also allowed or 
instigated a number of cool new features that will be explored below.

## NSE in layout specifications
In `ggraph` the initiation will need to specify a layout to use for the 
subsequent node and edge geoms. Many of these layouts use different node and 
edge variables in their calculations e.g. a node size or an edge weight. Prior 
to v2 these arguments would simply take a string naming the respective variable
to use, but following the v2 update these arguments implement Non-Standard 
Evaluation (NSE) in a manner known from both `dplyr` and `ggplot2` where it is
used inside `aes()` calls. Depending on whether the argument refers to a node or
edge value the provided expression will be evaluated in the context of nodes or
edges respectively. The bottomline is that given a network such as this:

```{r, message=FALSE}
library(tidygraph)

graph <- as_tbl_graph(
  data.frame(
    from = sample(5, 20, TRUE),
    to = sample(5, 20, TRUE),
    weight = runif(20)
  )
)
graph
```

Then, instead of writing:

```{r, eval=FALSE}
ggraph(graph, layout = 'fr', weights = "weight") + 
  geom_edge_link() + 
  geom_node_point()
```

You would simply write:

```{r}
ggraph(graph, layout = 'fr', weights = weight) + 
  geom_edge_link() + 
  geom_node_point()
```

This change means that it is much easier to experiment with modifications to
node and edge parameters affecting layouts as it is not necessary to modify the
underlying graph but only the plotting code, e.g.:

```{r}
ggraph(graph, layout = 'fr', weights = exp(weight)) + 
  geom_edge_link() + 
  geom_node_point()
```

## Access to tidygraph algorithms in ggraph code
The most important improvement resulting from the integration of `tidygraph` and
`ggraph` is that `tidygraph` algorithms are now directly usable within `ggraph` 
calls. This means that it is no longer necessary to precompute and store derived
node and edge variables on the graph in order to use them in a plot:

```{r}
graph <- create_notable('zachary')

ggraph(graph, layout = 'fr') + 
  geom_edge_link() + 
  geom_node_point(aes(size = centrality_pagerank())) + 
  theme(legend.position = 'bottom')
```

here it is not necessary to first compute the pagerank centrality and store it
as a node variable in order to plot it, and if you're interested in looking at 
one of the myriad of other centrality measures you simply change the plotting
code. This feature makes it much easier and painfree to investigate the effect
of different graph measures on your plots and is a huge benefit when iterating
on your visualisation.

Access to `tidygraph` is available within `ggraph()` and `aes()` calls, and 
within `facet` formulas. It is thus possible to use algorithms when specifying
layouts, adding aesthetics to geoms and splitting into subplots - all areas were
ease of iteration is vital:

```{r, message=FALSE}
ggraph(graph, 'matrix', sort.by = node_rank_leafsort()) + 
  geom_edge_point(aes(colour = centrality_edge_betweenness()), mirror = TRUE) + 
  theme(legend.position = 'bottom')
```

```{r}
ggraph(graph, 'fr') + 
  geom_edge_link() + 
  geom_node_point() + 
  facet_nodes(~ group_infomap())
```

---
title: "Layouts"
author: "Thomas Lin Pedersen"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Layouts}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include=FALSE}
set.seed(2022)
```

In very short terms, a layout is the vertical and horizontal placement of nodes
when plotting a particular graph structure. Conversely, a layout algorithm is an
algorithm that takes in a graph structure (and potentially some additional 
parameters) and return the vertical and horizontal position of the nodes. Often,
when people think of network visualizations, they think of node-edge diagrams
where strongly connected nodes are attempted to be plotted in close proximity.
Layouts can be a lot of other things too though --- e.g. hive plots and 
treemaps. One of the driving factors behind `ggraph` has been to develop an API
where any type of visual representation of graph structures is supported. In 
order to achieve this we first need a flexible way of defining the layout...

## The ggraph() and create_layout() functions
As the layout is a global specification of the spatial position of the nodes it
spans all layers in the plot and should thus be defined outside of calls to 
geoms or stats. In `ggraph` it is often done as part of the plot initialization
using `ggraph()` --- a function equivalent in intent to `ggplot()`. As a minimum
`ggraph()` must be passed a graph object supported by `ggraph`:

```{r, message=FALSE}
library(ggraph)
library(tidygraph)

set_graph_style(plot_margin = margin(1,1,1,1))
graph <- as_tbl_graph(highschool)

# Not specifying the layout - defaults to "auto"
ggraph(graph) + 
  geom_edge_link(aes(colour = factor(year))) + 
  geom_node_point()
```

Not specifying a layout will make `ggraph` pick one for you. This is only 
intended to get quickly up and running. The choice of layout should be 
deliberate on the part of the user as it will have a great effect on what the 
end result will communicate. From now on all calls to `ggraph()` will contain a
specification of the layout:

```{r}
ggraph(graph, layout = 'kk') + 
  geom_edge_link(aes(colour = factor(year))) + 
  geom_node_point()
```

If the layout algorithm accepts additional parameters (most do), they can be
supplied in the call to `ggraph()` as well:

```{r}
ggraph(graph, layout = 'kk', maxiter = 100) + 
  geom_edge_link(aes(colour = factor(year))) + 
  geom_node_point()
```

If any layout parameters refers to node or edge variables they must be supplied
as unquoted expression (like inside `aes()` and `tidyverse` verbs)

In addition to specifying the layout during plot creation it can also happen 
separately using `create_layout()`. This function takes the same arguments as 
`ggraph()` but returns a `layout_ggraph` object that can later be used in place
of a graph structure in ggraph call:

```{r}
layout <- create_layout(graph, layout = 'eigen')
ggraph(layout) + 
  geom_edge_link(aes(colour = factor(year))) + 
  geom_node_point()
```

Examining the return of `create_layout()` we see that it is really just a 
`data.frame` of node positions and (possible) attributes. Furthermore the 
original graph object along with other relevant information is passed along as
attributes:

```{r}
head(layout)
```

```{r}
attributes(layout)
```

As it is just a `data.frame` it means that any standard `ggplot2` call will work
by addressing the nodes. Still, use of the `geom_node_*()` family provided by 
`ggraph` is encouraged as it makes it explicit which part of the data structure
is being worked with.

## Adding support for new data sources
Out of the box `ggraph` supports `tbl_graph` objects from tidygraph natively. 
Any other type of object will be attempted to be coerced to a `tbl_graph` object
automatically. Tidygraph provide conversions for most known graph structure in R
so almost any data type is supported by ggraph by extension. If there
is wish for support for additional classes this can be achieved by providing a
`as_tbl_graph()` method for the class. If you do this, consider submitting the
method to tidygraph so others can benefit from your work.

## Layouts abound
There's a lot of different layouts in `ggraph` --- All layouts from the 
graphlayouts and igraph packages are available, and ggraph itself also provides
some of the more specialised layouts itself. All in all, ggraph provides well 
above 20 different layouts to choose from, far more than we can cover in this 
text. I urge you to explore the different layout types. Blindly running along 
with the default layouts is a sad but common mistake in network visualisation 
that can cloud or distort the insight the network might hold. If ggraph lacks 
the needed layout it is always possible to supply your own layout function that
takes a tbl_graph object and returns a data.frame of node positions, or supply
the positions directly by passing a matrix or data.frame to the layout argument.

### A note on circularity
Some layouts can be shown effectively both in a standard Cartesian projection as
well as in a polar projection. The standard approach in `ggplot2` has been to 
change the coordinate system with the addition of e.g. `coord_polar()`. This
approach --- while consistent with the grammar --- is not optimal for `ggraph`
as it does not allow layers to decide how to respond to circularity. The prime
example of this is trying to draw straight lines in a plot using 
`coord_polar()`. Instead circularity is part of the layout specification and
gets communicated to the layers with the `circular` column in the data, allowing
each layer to respond appropriately. Sometimes standard and circular 
representations of the same layout get used so often that they get different
names. In `ggraph` they'll have the same name and only differ in whether or not
`circular` is set to `TRUE`:

```{r}
# An arc diagram
ggraph(graph, layout = 'linear') + 
  geom_edge_arc(aes(colour = factor(year)))
```

```{r}
# A coord diagram
ggraph(graph, layout = 'linear', circular = TRUE) + 
  geom_edge_arc(aes(colour = factor(year))) + 
  coord_fixed()
```

```{r}
graph <- tbl_graph(flare$vertices, flare$edges)
# An icicle plot
ggraph(graph, 'partition') + 
  geom_node_tile(aes(fill = depth), size = 0.25)
```

```{r}
# A sunburst plot
ggraph(graph, 'partition', circular = TRUE) + 
  geom_node_arc_bar(aes(fill = depth), size = 0.25) + 
  coord_fixed()
```

Not every layout has a meaningful circular representation in which cases the 
`circular` argument will be ignored.

### Node-edge diagram layouts
Both `graphlayout` and `igraph` provides a range of different layout algorithms 
for classic node-edge diagrams (colloquially referred to as hairballs). Some of 
these are incredibly simple such as *randomly*, *grid*, *circle*, and *star*, 
while others tries to optimize the position of nodes based on different 
characteristics of the graph. There is no such thing as "the best layout 
algorithm" as algorithms have been optimized for different scenarios. Experiment 
with the choices at hand and remember to take the end result with a grain of 
salt, as it is just one of a range of possible "optimal node position" results. 
Below is a sample of some of the layouts available through `igraph` applied to 
the highschool graph.

```{r, fig.show='hold', results='hide'}
graph <- as_tbl_graph(highschool) |> 
  mutate(degree = centrality_degree())
lapply(c('stress', 'fr', 'lgl', 'graphopt'), function(layout) {
  ggraph(graph, layout = layout) + 
    geom_edge_link(aes(colour = factor(year)), show.legend = FALSE) +
    geom_node_point() + 
    labs(caption = paste0('Layout: ', layout))
})
```

The default plot is the `"stress"` layout that uses stress majorization to 
spread out nodes. It generally does a good job and is deterministic so that it
doesn't change upon every call (many other layouts does that as they use 
randomisation for the initial node positions). The stress layout also makes it
possible to fix the location of certain nodes in one or two dimensions making it
a very versatile starting point for your visualisation.

### Hive plots
A hive plot, while still technically a node-edge diagram, is a bit different
from the rest as it uses information pertaining to the nodes, rather than the
connection information in the graph. This means that hive plots, to a certain
extent are more interpretable as well as less vulnerable to small changes in the
graph structure. They are less common though, so use will often require some 
additional explanation.

```{r}
graph <- graph |> 
  mutate(friends = ifelse(
    centrality_degree(mode = 'in') < 5, 'few',
    ifelse(centrality_degree(mode = 'in') >= 15, 'many', 'medium')
  ))
ggraph(graph, 'hive', axis = friends, sort.by = degree) + 
  geom_edge_hive(aes(colour = factor(year))) + 
  geom_axis_hive(aes(colour = friends), size = 2, label = FALSE) + 
  coord_fixed()
```

### Focal layouts
Some layouts can put focus on a single node or a group of nodes by defining all
other positions relative to that. An example of this is the `focus` layout, but
the `centrality` layout is very akin to it:

```{r}
ggraph(graph, 'focus', focus = node_is_center()) + 
  ggforce::geom_circle(aes(x0 = 0, y0 = 0, r = r), data.frame(r = 1:5), colour = 'grey') + 
  geom_edge_link() + 
  geom_node_point() + 
  coord_fixed()
```

### Hierarchical layouts
Trees and hierarchies are an important subset of graph structures, and `ggraph`
provides a range of layouts optimized for their visual representation. Some of
these use enclosure and position rather than edges to communicate relations
(e.g. treemaps and circle packing). Still, these layouts can just as well be 
used for drawing edges if you wish to:

```{r}
graph <- tbl_graph(flare$vertices, flare$edges)
set.seed(1)
ggraph(graph, 'circlepack', weight = size) + 
  geom_node_circle(aes(fill = depth), size = 0.25, n = 50) + 
  coord_fixed()
```

```{r}
set.seed(1)
ggraph(graph, 'circlepack', weight = size) + 
  geom_edge_link() + 
  geom_node_point(aes(colour = depth)) +
  coord_fixed()
```

```{r}
ggraph(graph, 'treemap', weight = size) + 
  geom_node_tile(aes(fill = depth), size = 0.25)
```

```{r}
ggraph(graph, 'treemap', weight = size) + 
  geom_edge_link() + 
  geom_node_point(aes(colour = depth))
```

Cactustree is a layout type specifically developed for use with hierarchical 
edge bundling, but can be used on it's own as well

```{r}
ggraph(graph, 'cactustree') + 
  geom_node_circle(aes(fill = depth), size = 0.25) + 
  coord_fixed()
```

```{r}
importFrom <- match(flare$imports$from, flare$vertices$name)
importTo <- match(flare$imports$to, flare$vertices$name)

ggraph(graph, 'cactustree') + 
  geom_node_circle(aes(fill = depth), 
    size = 0.25, 
    alpha = 0.2
  ) + 
  geom_conn_bundle(aes(colour = after_stat(index)),
    data = get_con(importFrom, importTo),
    edge_alpha = 0.25
  ) +
  theme(legend.position = "none") +
  coord_fixed()
```

The most recognized tree plot is probably dendrograms though. If nothing else is
stated the height of each node is calculated based on the distance to its 
farthest sibling (the tree layout, on the other hand, puts all nodes at a 
certain depth at the same level):

```{r}
ggraph(graph, 'tree') + 
  geom_edge_diagonal()
```

The height of each branch point can be set to a variable --- e.g. the height 
provided by hclust and dendrogram objects:

```{r}
dendrogram <- hclust(dist(iris[, 1:4]))
ggraph(dendrogram, 'dendrogram', height = height) + 
  geom_edge_elbow()
```

Dendrograms are one of the layouts that are amenable for circular 
transformations, which can be effective in giving more space at the leafs of the
tree at the expense of the space given to the root:

```{r}
ggraph(dendrogram, 'dendrogram', circular = TRUE) + 
  geom_edge_elbow() + 
  coord_fixed()
```

A type of trees known especially in phylogeny is unrooted trees, where no node
is considered the root. Often a dendrogram layout will not be faithful as it
implicitly position a node at the root. To avoid that you can use the `unrooted` 
layout instead.

```{r}
tree <- create_tree(100, 2, directed = FALSE) |> 
  activate(edges) |> 
  mutate(length = runif(n()))
ggraph(tree, 'unrooted', length = length) + 
  geom_edge_link()
```

Often unrooted trees have a branch length attached - this can be passed to both
the `dendrogram` and `unrooted` layout to determine the length of each edge.

### Matrix layouts
Many node-edge diagram layouts suffer from poor scalability, where edges will 
eventually begin to overlap to the extend that the plot becomes unintellible. 
One way to combat this is by only plotting subsets of the larger plot, but
another approach is to choose a layout that avoids overlapping edges altogether.
The matrix layout places each node on a diagonal and draws edges by drawing 
points or tiles at the intersection of the vertical and horizontal position of
its terminal nodes. Using matrix layouts efficiently requires that you begin to
recognise the specific patterns that different network topologies gives rise to.
Further, it is important to recognise the large effect that the node order has 
on the look of the matrix layout:

```{r}
graph <- create_notable('zachary')
ggraph(graph, 'matrix', sort.by = node_rank_leafsort()) + 
  geom_edge_point(mirror = TRUE) + 
  coord_fixed()
```

```{r}
ggraph(graph, 'matrix', sort.by = node_rank_spectral()) + 
  geom_edge_point(mirror = TRUE) + 
  coord_fixed()
```

### Fabric layouts
Another special layout that promises scalability is the biofabric layout (here
named `fabric` to avoid it being ignored for non-biological networks). The 
fabric layout is special in that it positions nodes evenly spaced on the y-axis
and then draws edges as vertical (and by extension, parallel) lines evenly 
separated as well, connecting the nodes. Nodes are drawn as horizontal lines 
spanning the extent of the edges that departs from it. As with matrix layouts
the node ordering have a huge impact on the final look and interpreting the plot
may take some getting used to.

```{r}
ggraph(graph, 'fabric', sort.by = node_rank_fabric()) + 
  geom_node_range(colour = 'grey') + 
  geom_edge_span(end_shape = 'square') + 
  coord_fixed()
```

Fabric layouts allow something called shadow edges where all edges are 
duplicated to make it easier to follow all edges originating from each node.

```{r}
ggraph(graph, 'fabric', sort.by = node_rank_fabric(), shadow.edges =TRUE) + 
  geom_node_range(colour = 'grey') + 
  geom_edge_span(aes(filter = shadow_edge), colour ='lightblue' , end_shape = 'square') + 
  geom_edge_span(aes(filter = !shadow_edge), end_shape = 'square') + 
  coord_fixed()
```

### Spatial layouts
ggraph has a few layouts aimed at data where the node has a physical location. 
You may think this requires no layout at all and to some extent you are correct.

ggraph supports spatial networks through the sfnetworks package and grabs the 
node position and CRS from the data correctly when using the "sf" layout:

```{r}
gr <- sfnetworks::as_sfnetwork(sfnetworks::roxel)

ggraph(gr, 'sf') + 
  geom_edge_sf(aes(color = type)) + 
  geom_node_sf(size = 0.3)
```

Another type of spatial layout is the metro layout which takes in a prior 
position and attempts to make these positions fit into a grid-like structure,
well known in railroad and metro maps.

```{r}
gr <- create_notable('Walther')
# Use stress layout to come up with a initial position
prior <- create_layout(gr, 'stress')

# Optimise placement with metro layout
ggraph(gr, 'metro', x = prior$x, y = prior$y, grid_space = 1, max_movement = 50) + 
  geom_edge_link(width = 4) + 
  geom_node_point(size = 10) + 
  geom_edge_link(color = 'white', width = 1) + 
  geom_node_point(color = 'white', size = 4)
```


## Want more?
Check out the other vignettes for more information on how to draw 
[nodes](Nodes.html) and [edges](Edges.html)...

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_focus.R
\name{layout_tbl_graph_focus}
\alias{layout_tbl_graph_focus}
\title{Place nodes in circles based on distance to a specific node}
\usage{
layout_tbl_graph_focus(
  graph,
  focus,
  weights = NULL,
  niter = 500,
  tolerance = 1e-04,
  circular = TRUE
)
}
\arguments{
\item{graph}{a tbl_graph object}

\item{focus}{An expression evaluating to a selected node. Can either be a
single integer or a logical vector with a single \code{TRUE} element.}

\item{weights}{An expression evaluated on the edge data to provide edge
weights for the layout. Currently ignored for the sparse version}

\item{niter}{number of iterations during stress optimization}

\item{tolerance}{stopping criterion for stress optimization}

\item{circular}{ignored}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular}, \code{distance} as
well as any information stored as node variables in the tbl_graph object.
}
\description{
This layout constrains node placement to a radius relative to its distance to
a given node. It then uses stress majorisation to find an optimal node
distribution according to this constraint.
}
\references{
Brandes, U., & Pich, C. (2011). \emph{More flexible radial layout.} Journal of
Graph Algorithms and Applications, 15(1), 157-173.
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\author{
The underlying algorithm is implemented in the graphlayouts package
by David Schoch
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_cactustree.R
\name{layout_tbl_graph_cactustree}
\alias{layout_tbl_graph_cactustree}
\title{Calculate nodes as fractal circle buds}
\usage{
layout_tbl_graph_cactustree(
  graph,
  direction = "out",
  weight = NULL,
  scale_factor = 0.75,
  overlap = 0.5,
  upright = FALSE,
  circular = FALSE
)
}
\arguments{
\item{graph}{An \code{tbl_graph} object}

\item{direction}{The direction of the tree in the graph. \code{'out'} (default)
means that parents point towards their children, while \code{'in'} means that
children point towards their parent.}

\item{weight}{An optional node variable to use as weight. If \code{NULL} all leaf
nodes will be assigned a weight of \code{1}.}

\item{scale_factor}{A scaling factor for the circles in the layout. The
radius will be calculated as \code{total_weight ^ scale_factor} with \code{total_weight}
being the weight of the node and all it's children.}

\item{overlap}{How much is the center of child nodes offset from the
periphery of their parent as a proportion of their own radius.}

\item{upright}{Logical. Should the children of the root only be distributed
around the top half of the root or all the way around.}

\item{circular}{Logical. Should the layout be transformed to a circular
representation. Ignored.}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{r}, \code{leaf}, \code{depth},
\code{circular} as well as any information stored as node variables in the
tbl_graph object.
}
\description{
The cactustree layout is a hierarchical layout optimised for use with
hierarchical edge bundling (\code{\link[=geom_conn_bundle]{geom_conn_bundle()}}). It is a fractal layout
that places node children as circles on the periphery of their parent circle,
each circle scaled by the total weight of their children.
}
\note{
cactustree is a layout intended for trees, that is, graphs where nodes
only have one parent and zero or more children. If the provided graph does
not fit this format an attempt to convert it to such a format will be made.
}
\references{
Dang, T., Forbes, A. (2017). \emph{CactusTree: A Tree Drawing Approach
for Hierarchical Edge Bundling}. 2017 IEEE Pacific Visualization Symposium,
210-214. https://doi.org/10.1109/PACIFICVIS.2017.8031596
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_htree}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_metro}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_sf}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggraph-package.R
\docType{package}
\name{ggraph-package}
\alias{ggraph-package}
\alias{_PACKAGE}
\title{ggraph: An Implementation of Grammar of Graphics for Graphs and Networks}
\description{
\if{html}{\figure{logo.png}{options: style='float: right' alt='logo' width='120'}}

The grammar of graphics as implemented in ggplot2 is a poor fit for graph and network visualizations due to its reliance on tabular data input. ggraph is an extension of the ggplot2 API tailored to graph visualizations and provides the same flexible approach to building up plots layer by layer.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://ggraph.data-imaginist.com}
  \item \url{https://github.com/thomasp85/ggraph}
  \item Report bugs at \url{https://github.com/thomasp85/ggraph/issues}
}

}
\author{
\strong{Maintainer}: Thomas Lin Pedersen \email{thomasp85@gmail.com} (\href{https://orcid.org/0000-0002-5147-4711}{ORCID})

Other contributors:
\itemize{
  \item RStudio [copyright holder]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_eigen.R
\name{layout_tbl_graph_eigen}
\alias{layout_tbl_graph_eigen}
\title{Place nodes according to their eigenvalues}
\usage{
layout_tbl_graph_eigen(
  graph,
  type = "laplacian",
  eigenvector = "smallest",
  circular = FALSE
)
}
\arguments{
\item{graph}{A tbl_graph object}

\item{type}{The type of matrix to extract the eigenvectors from. Either
\code{'laplacian'} or \code{'adjacency'}}

\item{eigenvector}{The eigenvector to use for coordinates. Either \code{'smallest'}
or \code{'largest'}}

\item{circular}{ignored}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
}
\description{
This layout is based on the idea of spectral layouts where node coordinates
are calculated directly by decomposing a matrix representation of the graph
and extracting the eigenvectors.
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\author{
The underlying algorithm is implemented in the graphlayouts package
by David Schoch
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autograph.R
\name{autograph}
\alias{autograph}
\alias{autograph.default}
\title{Quickplot wrapper for networks}
\usage{
autograph(graph, ...)

\method{autograph}{default}(
  graph,
  ...,
  node_colour = NULL,
  edge_colour = NULL,
  node_size = NULL,
  edge_width = NULL,
  node_label = NULL,
  edge_label = NULL
)
}
\arguments{
\item{graph}{An object coercible to a tbl_graph}

\item{...}{arguments passed on to methods}

\item{node_colour, edge_colour}{Colour mapping for nodes and edges}

\item{node_size, edge_width}{Size/width mapping for nodes and edges}

\item{node_label, edge_label}{Label mapping for nodes and edges}
}
\description{
This function is intended to quickly show an overview of your network data.
While it returns a ggraph object that layers etc can be added to it is
limited in use and should not be used as a foundation for more complicated
plots. It allows colour, labeling and sizing of nodes and edges, and the
exact combination of layout and layers will depend on these as well as the
features of the network. The output of this function may be fine-tuned at any
release and should not be considered stable. If a plot should be reproducible
it should be created manually.
}
\examples{
library(tidygraph)
gr <- create_notable('herschel') \%>\%
  mutate(class = sample(letters[1:3], n(), TRUE)) \%E>\%
  mutate(weight = runif(n()))

# Standard graph
autograph(gr)

# Adding node labels will cap edges
autograph(gr, node_label = class)

# Use tidygraph calls for mapping
autograph(gr, node_size = centrality_pagerank())

# Trees are plotted as dendrograms
iris_tree <- hclust(dist(iris[1:4], method = 'euclidean'), method = 'ward.D2')
autograph(iris_tree)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_eigen.R
\name{layout_tbl_graph_eigen}
\alias{layout_tbl_graph_eigen}
\title{Place nodes according to their eigenvalues}
\usage{
layout_tbl_graph_eigen(
  graph,
  type = "laplacian",
  eigenvector = "smallest",
  circular = FALSE
)
}
\arguments{
\item{graph}{A tbl_graph object}

\item{type}{The type of matrix to extract the eigenvectors from. Either
\code{'laplacian'} or \code{'adjacency'}}

\item{eigenvector}{The eigenvector to use for coordinates. Either \code{'smallest'}
or \code{'largest'}}

\item{circular}{ignored}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
}
\description{
This layout is based on the idea of spectral layouts where node coordinates
are calculated directly by decomposing a matrix representation of the graph
and extracting the eigenvectors.
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_cactustree}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_htree}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_metro}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_sf}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\author{
The underlying algorithm is implemented in the graphlayouts package
by David Schoch
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_edge_width.R
\name{scale_edge_width}
\alias{scale_edge_width}
\alias{scale_edge_width_continuous}
\alias{scale_edge_width_discrete}
\alias{scale_edge_width_binned}
\alias{scale_edge_width_manual}
\alias{scale_edge_width_identity}
\title{Edge width scales}
\usage{
scale_edge_width_continuous(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  trans = "identity",
  guide = "legend"
)

scale_edge_width(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  trans = "identity",
  guide = "legend"
)

scale_edge_width_discrete(...)

scale_edge_width_binned(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  n.breaks = NULL,
  nice.breaks = TRUE,
  trans = "identity",
  guide = "bins"
)

scale_edge_width_manual(..., values, breaks = waiver(), na.value = NA)

scale_edge_width_identity(..., guide = "none")
}
\arguments{
\item{name}{The name of the scale. Used as the axis or legend title. If
\code{waiver()}, the default, the name of the scale is taken from the first
mapping used for that aesthetic. If \code{NULL}, the legend title will be
omitted.}

\item{breaks}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{waiver()} for the default breaks computed by the
\link[scales:new_transform]{transformation object}
\item A numeric vector of positions
\item A function that takes the limits as input and returns breaks
as output (e.g., a function returned by \code{\link[scales:breaks_extended]{scales::extended_breaks()}}).
Also accepts rlang \link[rlang:as_function]{lambda} function notation.
}}

\item{labels}{One of:
\itemize{
\item \code{NULL} for no labels
\item \code{waiver()} for the default labels computed by the
transformation object
\item A character vector giving labels (must be same length as \code{breaks})
\item An expression vector (must be the same length as breaks). See ?plotmath for details.
\item A function that takes the breaks as input and returns labels
as output. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}

\item{limits}{One of:
\itemize{
\item \code{NULL} to use the default scale range
\item A numeric vector of length two providing limits of the scale.
Use \code{NA} to refer to the existing minimum or maximum
\item A function that accepts the existing (automatic) limits and returns
new limits. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
Note that setting limits on positional scales will \strong{remove} data outside of the limits.
If the purpose is to zoom, use the limit argument in the coordinate system
(see \code{\link[ggplot2:coord_cartesian]{coord_cartesian()}}).
}}

\item{range}{a numeric vector of length 2 that specifies the minimum and
maximum size of the plotting symbol after transformation.}

\item{trans}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Deprecated in favour of
\code{transform}.}

\item{guide}{A function used to create a guide or its name. See
\code{\link[ggplot2:guides]{guides()}} for more information.}

\item{...}{
  Arguments passed on to \code{\link[ggplot2:continuous_scale]{continuous_scale}}
  \describe{
    \item{\code{minor_breaks}}{One of:
\itemize{
\item \code{NULL} for no minor breaks
\item \code{waiver()} for the default breaks (one minor break between
each major break)
\item A numeric vector of positions
\item A function that given the limits returns a vector of minor breaks. Also
accepts rlang \link[rlang:as_function]{lambda} function notation. When
the function has two arguments, it will be given the limits and major
breaks.
}}
    \item{\code{oob}}{One of:
\itemize{
\item Function that handles limits outside of the scale limits
(out of bounds). Also accepts rlang \link[rlang:as_function]{lambda}
function notation.
\item The default (\code{\link[scales:oob]{scales::censor()}}) replaces out of
bounds values with \code{NA}.
\item \code{\link[scales:oob]{scales::squish()}} for squishing out of bounds values into range.
\item \code{\link[scales:oob]{scales::squish_infinite()}} for squishing infinite values into range.
}}
    \item{\code{na.value}}{Missing values will be replaced with this value.}
    \item{\code{expand}}{For position scales, a vector of range expansion constants used to add some
padding around the data to ensure that they are placed some distance
away from the axes. Use the convenience function \code{\link[ggplot2:expansion]{expansion()}}
to generate the values for the \code{expand} argument. The defaults are to
expand the scale by 5\% on each side for continuous variables, and by
0.6 units on each side for discrete variables.}
    \item{\code{position}}{For position scales, The position of the axis.
\code{left} or \code{right} for y axes, \code{top} or \code{bottom} for x axes.}
    \item{\code{call}}{The \code{call} used to construct the scale for reporting messages.}
    \item{\code{super}}{The super class to use for the constructed scale}
  }}

\item{n.breaks}{An integer guiding the number of major breaks. The algorithm
may choose a slightly different number to ensure nice break labels. Will
only have an effect if \code{breaks = waiver()}. Use \code{NULL} to use the default
number of breaks given by the transformation.}

\item{nice.breaks}{Logical. Should breaks be attempted placed at nice values
instead of exactly evenly spaced between the limits. If \code{TRUE} (default)
the scale will ask the transformation object to create breaks, and this
may result in a different number of breaks than requested. Ignored if
breaks are given explicitly.}

\item{values}{a set of aesthetic values to map data values to. The values
will be matched in order (usually alphabetical) with the limits of the
scale, or with \code{breaks} if provided. If this is a named vector, then the
values will be matched based on the names instead. Data values that don't
match will be given \code{na.value}.}

\item{na.value}{The aesthetic value to use for missing (\code{NA}) values}
}
\value{
A ggproto object inheriting from \code{Scale}
}
\description{
This set of scales defines width scales for edge geoms. Of all the new edge
scales defined in ggraph, this is the only one not having an equivalent in
ggplot2. In essence it mimics the use of size in
\code{\link[ggplot2:geom_path]{ggplot2::geom_line()}} and related. As almost all edge
representations are lines of some sort, edge_width will be used much more
often than edge_size. It is not necessary to spell out that it is an edge
scale as the geom knows if it is drawing an edge. Just write \code{width} and
not \code{edge_width} in the call to geoms.
}
\seealso{
Other scale_edge_*: 
\code{\link{scale_edge_alpha}()},
\code{\link{scale_edge_colour}},
\code{\link{scale_edge_fill}},
\code{\link{scale_edge_linetype}()},
\code{\link{scale_edge_shape}()},
\code{\link{scale_edge_size}()},
\code{\link{scale_label_size}()}
}
\concept{scale_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggraph-package.R
\docType{package}
\name{ggraph-package}
\alias{ggraph-package}
\alias{_PACKAGE}
\title{ggraph: An Implementation of Grammar of Graphics for Graphs and Networks}
\description{
\if{html}{\figure{logo.png}{options: align='right' alt='logo' width='120'}}

The grammar of graphics as implemented in ggplot2 is a poor fit for
    graph and network visualizations due to its reliance on tabular data input.
    ggraph is an extension of the ggplot2 API tailored to graph visualizations
    and provides the same flexible approach to building up plots layer by layer.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://ggraph.data-imaginist.com}
  \item \url{https://github.com/thomasp85/ggraph}
  \item Report bugs at \url{https://github.com/thomasp85/ggraph/issues}
}

}
\author{
\strong{Maintainer}: Thomas Lin Pedersen \email{thomasp85@gmail.com} (\href{https://orcid.org/0000-0002-5147-4711}{ORCID})

Other contributors:
\itemize{
  \item RStudio [copyright holder]
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{node_angle}
\alias{node_angle}
\alias{edge_angle}
\title{Get the angle of nodes and edges}
\usage{
node_angle(x, y, degrees = TRUE)

edge_angle(x, y, xend, yend, degrees = TRUE)
}
\arguments{
\item{x, y}{A vector of positions}

\item{degrees}{Logical. Should the angle be returned in degree (\code{TRUE})
or radians (\code{FALSE}). Defaults to \code{TRUE}.}

\item{xend, yend}{The end position of the edge}
}
\value{
A vector with the angle of each node/edge
}
\description{
These helper functions makes it easy to calculate the angle associated with
nodes and edges. For nodes the angle is defined as the angle of the vector
pointing towards the node position, and is thus mainly suited for circular
layouts where it can be used to calculate the angle of labels. For edges it
is simply the angle of the vector describing the edge.
}
\examples{
require(tidygraph)
flareGraph <- tbl_graph(flare$vertices, flare$edges)

ggraph(flareGraph, 'dendrogram', circular = TRUE) +
  geom_edge_diagonal0() +
  geom_node_text(aes(filter = leaf, angle = node_angle(x, y), label = shortName),
    hjust = 'outward', size = 2
  ) +
  expand_limits(x = c(-1.3, 1.3), y = c(-1.3, 1.3))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_edge_width.R
\name{scale_edge_width}
\alias{scale_edge_width}
\alias{scale_edge_width_continuous}
\alias{scale_edge_width_discrete}
\alias{scale_edge_width_binned}
\alias{scale_edge_width_manual}
\alias{scale_edge_width_identity}
\title{Edge width scales}
\usage{
scale_edge_width_continuous(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  trans = "identity",
  guide = "legend"
)

scale_edge_width(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  trans = "identity",
  guide = "legend"
)

scale_edge_width_discrete(...)

scale_edge_width_binned(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  n.breaks = NULL,
  nice.breaks = TRUE,
  trans = "identity",
  guide = "bins"
)

scale_edge_width_manual(..., values, breaks = waiver(), na.value = NA)

scale_edge_width_identity(..., guide = "none")
}
\arguments{
\item{name}{The name of the scale. Used as the axis or legend title. If
\code{waiver()}, the default, the name of the scale is taken from the first
mapping used for that aesthetic. If \code{NULL}, the legend title will be
omitted.}

\item{breaks}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{waiver()} for the default breaks computed by the
\link[scales:new_transform]{transformation object}
\item A numeric vector of positions
\item A function that takes the limits as input and returns breaks
as output (e.g., a function returned by \code{\link[scales:breaks_extended]{scales::extended_breaks()}}).
Note that for position scales, limits are provided after scale expansion.
Also accepts rlang \link[rlang:as_function]{lambda} function notation.
}}

\item{labels}{One of the options below. Please note that when \code{labels} is a
vector, it is highly recommended to also set the \code{breaks} argument as a
vector to protect against unintended mismatches.
\itemize{
\item \code{NULL} for no labels
\item \code{waiver()} for the default labels computed by the
transformation object
\item A character vector giving labels (must be same length as \code{breaks})
\item An expression vector (must be the same length as breaks). See ?plotmath for details.
\item A function that takes the breaks as input and returns labels
as output. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}

\item{limits}{One of:
\itemize{
\item \code{NULL} to use the default scale range
\item A numeric vector of length two providing limits of the scale.
Use \code{NA} to refer to the existing minimum or maximum
\item A function that accepts the existing (automatic) limits and returns
new limits. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
Note that setting limits on positional scales will \strong{remove} data outside of the limits.
If the purpose is to zoom, use the limit argument in the coordinate system
(see \code{\link[ggplot2:coord_cartesian]{coord_cartesian()}}).
}}

\item{range}{a numeric vector of length 2 that specifies the minimum and
maximum size of the plotting symbol after transformation.}

\item{trans}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Deprecated in favour of
\code{transform}.}

\item{guide}{A function used to create a guide or its name. See
\code{\link[ggplot2:guides]{guides()}} for more information.}

\item{...}{
  Arguments passed on to \code{\link[ggplot2:continuous_scale]{continuous_scale}}
  \describe{
    \item{\code{minor_breaks}}{One of:
\itemize{
\item \code{NULL} for no minor breaks
\item \code{waiver()} for the default breaks (none for discrete, one minor break
between each major break for continuous)
\item A numeric vector of positions
\item A function that given the limits returns a vector of minor breaks. Also
accepts rlang \link[rlang:as_function]{lambda} function notation. When
the function has two arguments, it will be given the limits and major
break positions.
}}
    \item{\code{oob}}{One of:
\itemize{
\item Function that handles limits outside of the scale limits
(out of bounds). Also accepts rlang \link[rlang:as_function]{lambda}
function notation.
\item The default (\code{\link[scales:oob]{scales::censor()}}) replaces out of
bounds values with \code{NA}.
\item \code{\link[scales:oob]{scales::squish()}} for squishing out of bounds values into range.
\item \code{\link[scales:oob]{scales::squish_infinite()}} for squishing infinite values into range.
}}
    \item{\code{na.value}}{Missing values will be replaced with this value.}
    \item{\code{call}}{The \code{call} used to construct the scale for reporting messages.}
    \item{\code{super}}{The super class to use for the constructed scale}
  }}

\item{n.breaks}{An integer guiding the number of major breaks. The algorithm
may choose a slightly different number to ensure nice break labels. Will
only have an effect if \code{breaks = waiver()}. Use \code{NULL} to use the default
number of breaks given by the transformation.}

\item{nice.breaks}{Logical. Should breaks be attempted placed at nice values
instead of exactly evenly spaced between the limits. If \code{TRUE} (default)
the scale will ask the transformation object to create breaks, and this
may result in a different number of breaks than requested. Ignored if
breaks are given explicitly.}

\item{values}{a set of aesthetic values to map data values to. The values
will be matched in order (usually alphabetical) with the limits of the
scale, or with \code{breaks} if provided. If this is a named vector, then the
values will be matched based on the names instead. Data values that don't
match will be given \code{na.value}.}

\item{na.value}{The aesthetic value to use for missing (\code{NA}) values}
}
\value{
A ggproto object inheriting from \code{Scale}
}
\description{
This set of scales defines width scales for edge geoms. Of all the new edge
scales defined in ggraph, this is the only one not having an equivalent in
ggplot2. In essence it mimics the use of size in
\code{\link[ggplot2:geom_path]{ggplot2::geom_line()}} and related. As almost all edge
representations are lines of some sort, edge_width will be used much more
often than edge_size. It is not necessary to spell out that it is an edge
scale as the geom knows if it is drawing an edge. Just write \code{width} and
not \code{edge_width} in the call to geoms.
}
\seealso{
Other scale_edge_*: 
\code{\link{scale_edge_alpha}()},
\code{\link{scale_edge_colour}},
\code{\link{scale_edge_fill}},
\code{\link{scale_edge_linetype}()},
\code{\link{scale_edge_shape}()},
\code{\link{scale_edge_size}()},
\code{\link{scale_label_size}()}
}
\concept{scale_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_partition.R
\name{layout_tbl_graph_partition}
\alias{layout_tbl_graph_partition}
\title{Calculate nodes as areas dividing their parent}
\usage{
layout_tbl_graph_partition(
  graph,
  weight = NULL,
  circular = FALSE,
  height = NULL,
  sort.by = NULL,
  direction = "out",
  offset = pi/2,
  const.area = TRUE
)
}
\arguments{
\item{graph}{An \code{tbl_graph} object}

\item{weight}{An optional node variable to use as weight. Will only affect
the weight of leaf nodes as the weight of non-leaf nodes are derived from
their children.}

\item{circular}{Logical. Should the layout be transformed to a circular
representation. If \code{TRUE} the resulting layout will be a sunburst
diagram.}

\item{height}{An optional node variable to use as height. If \code{NULL}
all nodes will be given a height of 1.}

\item{sort.by}{The name of a node variable to sort the nodes by.}

\item{direction}{The direction of the tree in the graph. \code{'out'} (default)
means that parents point towards their children, while \code{'in'} means that
children point towards their parent.}

\item{offset}{If \code{circular = TRUE}, where should it begin. Defaults to
\code{pi/2} which is equivalent to 12 o'clock.}

\item{const.area}{Logical. Should 'height' be scaled for area proportionality
when using \code{circular = TRUE}. Defaults to \code{TRUE}.}
}
\value{
If \code{circular = FALSE} A data.frame with the columns \code{x},
\code{y}, \code{width}, \code{height}, \code{leaf},
\code{depth}, \code{circular} as well as any information stored as node
variables in the tbl_graph object.
If \code{circular = TRUE} A data.frame with the columns \code{x}, \code{y},
\code{r0}, \code{r}, \code{start}, \code{end}, \code{leaf},
\code{depth}, \code{circular} as well as any information stored as node
variables in the tbl_graph object.
}
\description{
The partition layout is a way to show hierarchical data in the same way as
\code{\link[=layout_tbl_graph_treemap]{layout_tbl_graph_treemap()}}. Instead of subdividing the parent area
the partition layout shows the division of a nodes children next to the area
of the node itself. As such the node positions will be very reminiscent of
a reingold-tilford tree layout but by plotting nodes as areas it better
communicate the total weight of a node by summing up all its children.
Often partition layouts are called icicle plots or sunburst diagrams (in case
a radial transform is applied).
}
\note{
partition is a layout intended for trees, that is, graphs where nodes
only have one parent and zero or more children. If the provided graph does
not fit this format an attempt to convert it to such a format will be made.
}
\references{
Kruskal, J. B., Landwehr, J. M. (1983). \emph{Icicle Plots: Better Displays
for Hierarchical Clustering}. American Statistician Vol 37(2), 162-168.
https://doi.org/10.2307/2685881
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_flare.R
\docType{data}
\name{flare}
\alias{flare}
\title{The class hierarchy of the flare visualization library}
\format{
A list of three data.frames describing the software structure of
flare:
\describe{
\item{edges}{This data.frame maps the hierarchical structure of the class
hierarchy as an edgelist, with the class in \code{from} being the superclass
of the class in \code{to}.}
\item{vertices}{This data.frame gives additional information on the classes.
It contains the full name, size and short name of each class.}
\item{imports}{This data.frame contains the class imports for each class
implementation. The \code{from} column gives the importing class and the
\code{to} column gives the import.}
}
}
\source{
The data have been adapted from the JSON downloaded from
\url{https://gist.github.com/mbostock/1044242#file-readme-flare-imports-json}
courtesy of Mike Bostock. The Flare framework is the work of the
\href{http://vis.berkeley.edu/}{UC Berkeley Visualization Lab}.
}
\usage{
flare
}
\description{
This dataset contains the graph that describes the class hierarchy for the
\href{https://blokt.com/tool/prefuse-flare}{Flare} ActionScript visualization library. It
contains both the class hierarchy as well as the import connections between
classes. This dataset has been used extensively in the D3.js documentation
and examples and are included here to make it easy to redo the examples in
ggraph.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_pmds.R
\name{layout_tbl_graph_pmds}
\alias{layout_tbl_graph_pmds}
\title{Place nodes based on a multidimensional scaling of a set of pivot nodes}
\usage{
layout_tbl_graph_pmds(graph, pivots, weights = NULL, circular = FALSE)
}
\arguments{
\item{graph}{A tbl_graph object}

\item{pivots}{The number of pivot nodes}

\item{weights}{An expression evaluated on the edge data to provide edge
weights for the layout. Currently ignored for the sparse version}

\item{circular}{ignored}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
}
\description{
This layout is similar to the 'mds' layout but uses only a subset of pivot
nodes for the mds calculation, making it considerably faster and thus suited
for large graphs
}
\references{
Brandes, U. and Pich, C. (2006). \emph{Eigensolver Methods for Progressive
Multidimensional Scaling of Large Data.} In International Symposium on Graph
Drawing (pp. 42-53). Springer
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\author{
The underlying algorithm is implemented in the graphlayouts package
by David Schoch
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_hive.R
\name{layout_tbl_graph_hive}
\alias{layout_tbl_graph_hive}
\title{Place nodes in a Hive Plot layout}
\usage{
layout_tbl_graph_hive(
  graph,
  axis,
  axis.pos = NULL,
  sort.by = NULL,
  divide.by = NULL,
  divide.order = NULL,
  normalize = TRUE,
  center.size = 0.1,
  divide.size = 0.05,
  use.numeric = FALSE,
  offset = pi/2,
  split.axes = "none",
  split.angle = pi/6,
  circular = FALSE
)
}
\arguments{
\item{graph}{An \code{tbl_graph} object}

\item{axis}{The node attribute to use for assigning nodes to axes}

\item{axis.pos}{The relative distance to the prior axis. Default
(\code{NULL}) places axes equidistant.}

\item{sort.by}{The node attribute to use for placing nodes along their axis.
Defaults (\code{NULL}) places nodes sequentially.}

\item{divide.by}{An optional node attribute to subdivide each axis by.}

\item{divide.order}{The order the axis subdivisions should appear in}

\item{normalize}{Logical. Should axis lengths be equal or reflect the number
of nodes in each axis. Defaults to \code{TRUE}.}

\item{center.size}{The size of the blank center, that is, the start position
of the axes.}

\item{divide.size}{The distance between subdivided axis segments.}

\item{use.numeric}{Logical, If the \code{sort.by} attribute is numeric,
should these values be used directly in positioning the nodes along the axes.
Defaults to \code{FALSE} which sorts the numeric values and positions them
equidistant from each other.}

\item{offset}{Change the overall rotation of the hive plot by changing the
offset of the first axis.}

\item{split.axes}{Should axes be split to show edges between nodes on the
same axis? One of:
\describe{
\item{\code{'none'}}{Do not split axes and show in-between edges}
\item{\code{'loops'}}{Only split axes that contain in-between edges}
\item{\code{'all'}}{Split all axes}
}}

\item{split.angle}{The angular distance between the two axes resulting from a
split.}

\item{circular}{Ignored.}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{r},
\code{center_size}, \code{split}, \code{axis}, \code{section}, \code{angle},
\code{circular} as well as any information stored as node variables in the
tbl_graph object.
}
\description{
Hive plots were invented by Martin Krzywinski as a perceptually uniform and
scalable alternative to standard node-edge layouts. In hive plots nodes are
positioned on axes radiating out from a center based on their own information
e.g. membership of a class, size of neighborhood, etc. Edges are then drawn
between nodes as bezier curves. As the placement of nodes is not governed by
convoluted algorithms but directly reflects the qualities of the nodes itself
the resulting plot can be easier to interpret as well as compare to other
graphs.
}
\details{
In order to be able to draw all edges without edges crossing axes you should
not assign nodes to axes based on a variable with more than three levels.
}
\references{
Krzywinski, M., Birol, I., Jones, SJM., and Marra, MA. (2012). \emph{Hive
plots-rational approach to visualizing networks}. Brief Bioinform 13 (5):
627-644. https://doi.org/10.1093/bib/bbr069

\url{http://www.hiveplot.net}
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_partition.R
\name{layout_tbl_graph_partition}
\alias{layout_tbl_graph_partition}
\title{Calculate nodes as areas dividing their parent}
\usage{
layout_tbl_graph_partition(
  graph,
  weight = NULL,
  circular = FALSE,
  height = NULL,
  sort.by = NULL,
  direction = "out",
  offset = pi/2,
  const.area = TRUE
)
}
\arguments{
\item{graph}{An \code{tbl_graph} object}

\item{weight}{An optional node variable to use as weight. Will only affect
the weight of leaf nodes as the weight of non-leaf nodes are derived from
their children.}

\item{circular}{Logical. Should the layout be transformed to a circular
representation. If \code{TRUE} the resulting layout will be a sunburst
diagram.}

\item{height}{An optional node variable to use as height. If \code{NULL}
all nodes will be given a height of 1.}

\item{sort.by}{The name of a node variable to sort the nodes by.}

\item{direction}{The direction of the tree in the graph. \code{'out'} (default)
means that parents point towards their children, while \code{'in'} means that
children point towards their parent.}

\item{offset}{If \code{circular = TRUE}, where should it begin. Defaults to
\code{pi/2} which is equivalent to 12 o'clock.}

\item{const.area}{Logical. Should 'height' be scaled for area proportionality
when using \code{circular = TRUE}. Defaults to \code{TRUE}.}
}
\value{
If \code{circular = FALSE} A data.frame with the columns \code{x},
\code{y}, \code{width}, \code{height}, \code{leaf},
\code{depth}, \code{circular} as well as any information stored as node
variables in the tbl_graph object.
If \code{circular = TRUE} A data.frame with the columns \code{x}, \code{y},
\code{r0}, \code{r}, \code{start}, \code{end}, \code{leaf},
\code{depth}, \code{circular} as well as any information stored as node
variables in the tbl_graph object.
}
\description{
The partition layout is a way to show hierarchical data in the same way as
\code{\link[=layout_tbl_graph_treemap]{layout_tbl_graph_treemap()}}. Instead of subdividing the parent area
the partition layout shows the division of a nodes children next to the area
of the node itself. As such the node positions will be very reminiscent of
a reingold-tilford tree layout but by plotting nodes as areas it better
communicate the total weight of a node by summing up all its children.
Often partition layouts are called icicle plots or sunburst diagrams (in case
a radial transform is applied).
}
\note{
partition is a layout intended for trees, that is, graphs where nodes
only have one parent and zero or more children. If the provided graph does
not fit this format an attempt to convert it to such a format will be made.
}
\references{
Kruskal, J. B., Landwehr, J. M. (1983). \emph{Icicle Plots: Better Displays
for Hierarchical Clustering}. American Statistician Vol 37(2), 162-168.
https://doi.org/10.2307/2685881
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_cactustree}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_htree}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_metro}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_sf}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_centrality.R
\name{layout_tbl_graph_centrality}
\alias{layout_tbl_graph_centrality}
\title{Place nodes in circles according to centrality measure}
\usage{
layout_tbl_graph_centrality(
  graph,
  centrality,
  scale = TRUE,
  niter = 500,
  tolerance = 1e-04,
  tseq = seq(0, 1, 0.2),
  circular = FALSE
)
}
\arguments{
\item{graph}{A tbl_graph object}

\item{centrality}{An expression evaluating to a centrality measure for the
nodes. See the different \verb{centrality_*()} algorithms in tidygraph for a
selection.}

\item{scale}{Should the centrality measure be scaled between 0 and 100}

\item{niter}{number of iterations during stress optimization}

\item{tolerance}{stopping criterion for stress optimization}

\item{tseq}{Transitioning steps}

\item{circular}{ignored}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular}, \code{centrality} as
well as any information stored as node variables in the tbl_graph object.
}
\description{
This layout places nodes in circles with the radii relative to a given
centrality measure. Under the hood it use stress majorisation to place nodes
optimally given the radius constraint.
}
\references{
Brandes, U., & Pich, C. (2011). \emph{More flexible radial layout.} Journal of
Graph Algorithms and Applications, 15(1), 157-173.
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\author{
The underlying algorithm is implemented in the graphlayouts package
by David Schoch
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_treemap.R
\name{layout_tbl_graph_treemap}
\alias{layout_tbl_graph_treemap}
\title{Calculate nodes as rectangles subdividing that of their parent}
\usage{
layout_tbl_graph_treemap(
  graph,
  algorithm = "split",
  weight = NULL,
  circular = FALSE,
  sort.by = NULL,
  direction = "out",
  height = 1,
  width = 1
)
}
\arguments{
\item{graph}{A \code{tbl_graph} object}

\item{algorithm}{The name of the tiling algorithm to use. Defaults to 'split'}

\item{weight}{An optional node variable to use as weight. Will only affect
the weight of leaf nodes as the weight of non-leaf nodes are derived from
their children.}

\item{circular}{Logical. Should the layout be transformed to a circular
representation. Ignored.}

\item{sort.by}{The name of a node variables to sort the nodes by.}

\item{direction}{The direction of the tree in the graph. \code{'out'} (default)
means that parents point towards their children, while \code{'in'} means that
children point towards their parent.}

\item{height}{The height of the bounding rectangle}

\item{width}{The width of the bounding rectangle}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{width},
\code{height}, \code{leaf}, \code{depth}, \code{circular} as well as any
information stored as node variables in the tbl_graph object.
}
\description{
A treemap is a space filling hierarchical layout that maps nodes to
rectangles. The rectangles of the children of a node is packed into the
rectangle of the node so that the size of a rectangle is a function of the
size of the children. The size of the leaf nodes can be mapped arbitrarily
(defaults to 1). Many different algorithms exists for dividing a rectangle
into smaller bits, some optimizing the aspect ratio and some focusing on the
ordering of the rectangles. See details for more discussions on this. The
treemap layout was first developed by Ben Shneiderman for visualizing disk
usage in the early '90 and has seen many improvements since.
}
\details{
Different approaches to dividing the rectangles in a treemap exists; all with
their strengths and weaknesses. Currently only the split algorithm is
implemented which strikes a good balance between aspect ratio and order
preservation, but other, more well-known, algorithms such as squarify and
slice-and-dice will eventually be implemented.

\strong{Algorithms}

\emph{Split} (default)

The Split algorithm was developed by Bjorn Engdahl in order to address the
downsides of both the original slice-and-dice algorithm (poor aspect ratio)
and the popular squarify algorithm (no ordering of nodes). It works by
finding the best cut in the ordered list of children in terms of making sure
that the two rectangles associated with the split will have optimal aspect
ratio.
}
\note{
Treemap is a layout intended for trees, that is, graphs where nodes
only have one parent and zero or more children. If the provided graph does
not fit this format an attempt to convert it to such a format will be made.
}
\references{
Engdahl, B. (2005). \emph{Ordered and unordered treemap algorithms and their
applications on handheld devices}. Master's Degree Project.

Johnson, B., & Ben Shneiderman. (1991). \emph{Tree maps: A Space-Filling
Approach to the Visualization of Hierarchical Information Structures}. IEEE
Visualization, 284-291. \doi{10.1109/VISUAL.1991.175815}
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_unrooted.R
\name{layout_tbl_graph_unrooted}
\alias{layout_tbl_graph_unrooted}
\title{Create an unrooted layout using equal-angle or equal-daylight}
\usage{
layout_tbl_graph_unrooted(
  graph,
  daylight = TRUE,
  length = NULL,
  tolerance = 0.05,
  rotation_mod = 1,
  maxiter = 100,
  circular = FALSE
)
}
\arguments{
\item{graph}{A tbl_graph object}

\item{daylight}{Should equal-daylight adjustments be made}

\item{length}{An expression evaluating to the branch length of each edge}

\item{tolerance}{The threshold for mean angular adjustment before terminating
the daylight adjustment}

\item{rotation_mod}{A modifier for the angular adjustment of each branch. Set
it below 1 to let the daylight adjustment progress more slowly}

\item{maxiter}{The maximum number of iterations in the the daylight
adjustment}

\item{circular}{ignored}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular}, \code{leaf} as well as
any information stored as node variables in the tbl_graph object.
}
\description{
When drawing unrooted trees the standard dendrogram layout is a bad fit as it
implicitly creates a visual root node. Instead it is possible to spread the
leafs out on the plane without putting any special emphasis on a particular
node using an unrooted layout. The standard algorithm is the equal angle
algorithm, but it can struggle with optimising the leaf distribution for
large trees trees with very uneven branch length. The equal daylight
algorithm modifies the output of the equal angle algorithm to better disperse
the leaves, at the cost of higher computational cost and the possibility of
edge crossings for very large unbalanced trees. For standard sized trees the
daylight algorithm is far superior and not too heavy so it is the default.
}
\note{
Unrooted is a layout intended for undirected trees, that is, graphs with no
cycles. If the provided graph does not fit this format an attempt to convert
it to such a format will be made.
}
\references{
Felsenstein, J. (2004) \emph{Drawing Trees}, in Inferring Phylogenies. Sinauer
Assoc., pp 573-584
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edge_direction.R
\name{guide_edge_direction}
\alias{guide_edge_direction}
\title{Edge direction guide}
\usage{
guide_edge_direction(
  title = NULL,
  theme = NULL,
  arrow = NULL,
  labels = NULL,
  nbin = 500,
  position = NULL,
  direction = NULL,
  reverse = FALSE,
  order = 0,
  override.aes = list(),
  ...,
  available_aes = c("edge_colour", "edge_alpha", "edge_width"),
  arrow.position = deprecated()
)
}
\arguments{
\item{title}{A character string or expression indicating a title of guide.
If \code{NULL}, the title is not shown. By default
(\code{\link[ggplot2:waiver]{waiver()}}), the name of the scale object or the name
specified in \code{\link[ggplot2:labs]{labs()}} is used for the title.}

\item{theme}{A \code{\link[ggplot2:theme]{theme}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides, and is combined with, the plot's theme.}

\item{arrow}{Logical. Should an arrow be drawn to illustrate the direction.
Defaults to \code{TRUE}. The arrow is styled with the \code{legend.axis.line} theme
element. If \code{FALSE} the direction will be indicated by the text
given in \code{labels}}

\item{labels}{A vector with two strings giving the labels to place at the
start and the end of the legend to indicate direction if \code{arrow = FALSE}}

\item{nbin}{A numeric specifying the number of bins for drawing the
colourbar. A smoother colourbar results from a larger value.}

\item{position}{A character string indicating where the legend should be
placed relative to the plot panels.}

\item{direction}{A character string indicating the direction of the guide.
One of "horizontal" or "vertical."}

\item{reverse}{logical. If \code{TRUE} the colourbar is reversed. By default,
the highest value is on the top and the lowest value is on the bottom}

\item{order}{positive integer less than 99 that specifies the order of
this guide among multiple guides. This controls the order in which
multiple guides are displayed, not the contents of the guide itself.
If 0 (default), the order is determined by a secret algorithm.}

\item{override.aes}{A list specifying aesthetic parameters of legend key.
See details and examples.}

\item{...}{ignored.}

\item{available_aes}{A vector of character strings listing the aesthetics
for which a colourbar can be drawn.}

\item{arrow.position}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} The position of the
arrow relative to the example edge. Use the \code{legend.text.position} argument
in \code{theme()} instead.}
}
\description{
This guide is intended to show the direction of edges based on the aesthetics
mapped to its progression, such as changing width, colour and opacity.
}
\examples{
gr <- tidygraph::as_tbl_graph(highschool)
ggraph(gr, layout = 'kk') +
  geom_edge_fan(aes(alpha = after_stat(index))) +
  guides(edge_alpha = guide_edge_direction())

# Use text labels instead of an arrow
ggraph(gr, layout = 'kk') +
  geom_edge_fan(aes(alpha = after_stat(index))) +
  guides(edge_alpha = guide_edge_direction(labels = c('start', 'end')))

# Style the indicator arrow
arrow_style <- element_line(linewidth = 3, arrow = grid::arrow(type = "closed"))
ggraph(gr, layout = 'kk') +
  geom_edge_fan(aes(alpha = after_stat(index))) +
  guides(
    edge_alpha = guide_edge_direction(
      theme = theme(legend.axis.line = arrow_style)
    )
  )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edge_direction.R
\name{guide_edge_direction}
\alias{guide_edge_direction}
\title{Edge direction guide}
\usage{
guide_edge_direction(
  title = NULL,
  theme = NULL,
  arrow = NULL,
  labels = NULL,
  nbin = 500,
  position = NULL,
  direction = NULL,
  reverse = FALSE,
  order = 0,
  override.aes = list(),
  ...,
  available_aes = c("edge_colour", "edge_alpha", "edge_width"),
  arrow.position = deprecated()
)
}
\arguments{
\item{title}{A character string or expression indicating a title of guide.
If \code{NULL}, the title is not shown. By default
(\code{\link[ggplot2:waiver]{waiver()}}), the name of the scale object or the name
specified in \code{\link[ggplot2:labs]{labs()}} is used for the title.}

\item{theme}{A \code{\link[ggplot2:theme]{theme}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide partially overrides, and is combined with, the plot's theme.
Arguments that apply to a single legend are respected, most of which have
the \code{legend}-prefix. Arguments that apply to combined legends
(the legend box) are ignored, including \code{legend.position},
\verb{legend.justification.*}, \code{legend.location} and \verb{legend.box.*}.}

\item{arrow}{Logical. Should an arrow be drawn to illustrate the direction.
Defaults to \code{TRUE}. The arrow is styled with the \code{legend.axis.line} theme
element. If \code{FALSE} the direction will be indicated by the text
given in \code{labels}}

\item{labels}{A vector with two strings giving the labels to place at the
start and the end of the legend to indicate direction if \code{arrow = FALSE}}

\item{nbin}{A numeric specifying the number of bins for drawing the
colourbar. A smoother colourbar results from a larger value.}

\item{position}{A character string indicating where the legend should be
placed relative to the plot panels.
One of "top", "right", "bottom", "left", or "inside".}

\item{direction}{A character string indicating the direction of the guide.
One of "horizontal" or "vertical."}

\item{reverse}{logical. If \code{TRUE} the colourbar is reversed. By default,
the highest value is on the top and the lowest value is on the bottom}

\item{order}{positive integer less than 99 that specifies the order of
this guide among multiple guides. This controls the order in which
multiple guides are displayed, not the contents of the guide itself.
If 0 (default), the order is determined by a secret algorithm.}

\item{override.aes}{A list specifying aesthetic parameters of legend key.
See details and examples.}

\item{...}{ignored.}

\item{available_aes}{A vector of character strings listing the aesthetics
for which a colourbar can be drawn.}

\item{arrow.position}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} The position of the
arrow relative to the example edge. Use the \code{legend.text.position} argument
in \code{theme()} instead.}
}
\description{
This guide is intended to show the direction of edges based on the aesthetics
mapped to its progression, such as changing width, colour and opacity.
}
\examples{
gr <- tidygraph::as_tbl_graph(highschool)
ggraph(gr, layout = 'kk') +
  geom_edge_fan(aes(alpha = after_stat(index))) +
  guides(edge_alpha = guide_edge_direction())

# Use text labels instead of an arrow
ggraph(gr, layout = 'kk') +
  geom_edge_fan(aes(alpha = after_stat(index))) +
  guides(edge_alpha = guide_edge_direction(labels = c('start', 'end')))

# Style the indicator arrow
arrow_style <- element_line(linewidth = 3, arrow = grid::arrow(type = "closed"))
ggraph(gr, layout = 'kk') +
  geom_edge_fan(aes(alpha = after_stat(index))) +
  guides(
    edge_alpha = guide_edge_direction(
      theme = theme(legend.axis.line = arrow_style)
    )
  )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_unrooted.R
\name{layout_tbl_graph_unrooted}
\alias{layout_tbl_graph_unrooted}
\title{Create an unrooted layout using equal-angle or equal-daylight}
\usage{
layout_tbl_graph_unrooted(
  graph,
  daylight = TRUE,
  length = NULL,
  tolerance = 0.05,
  rotation_mod = 1,
  maxiter = 100,
  circular = FALSE
)
}
\arguments{
\item{graph}{A tbl_graph object}

\item{daylight}{Should equal-daylight adjustments be made}

\item{length}{An expression evaluating to the branch length of each edge}

\item{tolerance}{The threshold for mean angular adjustment before terminating
the daylight adjustment}

\item{rotation_mod}{A modifier for the angular adjustment of each branch. Set
it below 1 to let the daylight adjustment progress more slowly}

\item{maxiter}{The maximum number of iterations in the the daylight
adjustment}

\item{circular}{ignored}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular}, \code{leaf} as well as
any information stored as node variables in the tbl_graph object.
}
\description{
When drawing unrooted trees the standard dendrogram layout is a bad fit as it
implicitly creates a visual root node. Instead it is possible to spread the
leafs out on the plane without putting any special emphasis on a particular
node using an unrooted layout. The standard algorithm is the equal angle
algorithm, but it can struggle with optimising the leaf distribution for
large trees with very uneven branch length. The equal daylight
algorithm modifies the output of the equal angle algorithm to better disperse
the leaves, at the cost of higher computational cost and the possibility of
edge crossings for very large unbalanced trees. For standard sized trees the
daylight algorithm is far superior and not too heavy so it is the default.
}
\note{
Unrooted is a layout intended for undirected trees, that is, graphs with no
cycles. If the provided graph does not fit this format an attempt to convert
it to such a format will be made.
}
\references{
Felsenstein, J. (2004) \emph{Drawing Trees}, in Inferring Phylogenies. Sinauer
Assoc., pp 573-584
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_cactustree}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_htree}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_metro}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_sf}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_edge_size.R
\name{scale_edge_size}
\alias{scale_edge_size}
\alias{scale_edge_size_continuous}
\alias{scale_edge_radius}
\alias{scale_edge_size_discrete}
\alias{scale_edge_size_binned}
\alias{scale_edge_size_area}
\alias{scale_edge_size_binned_area}
\alias{scale_edge_size_manual}
\alias{scale_edge_size_identity}
\title{Edge size scales}
\usage{
scale_edge_size_continuous(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  trans = "identity",
  guide = "legend"
)

scale_edge_radius(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  trans = "identity",
  guide = "legend"
)

scale_edge_size(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  trans = "identity",
  guide = "legend"
)

scale_edge_size_discrete(...)

scale_edge_size_binned(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  n.breaks = NULL,
  nice.breaks = TRUE,
  trans = "identity",
  guide = "bins"
)

scale_edge_size_area(..., max_size = 6)

scale_edge_size_binned_area(..., max_size = 6)

scale_edge_size_manual(..., values, breaks = waiver(), na.value = NA)

scale_edge_size_identity(..., guide = "none")
}
\arguments{
\item{name}{The name of the scale. Used as the axis or legend title. If
\code{waiver()}, the default, the name of the scale is taken from the first
mapping used for that aesthetic. If \code{NULL}, the legend title will be
omitted.}

\item{breaks}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{waiver()} for the default breaks computed by the
\link[scales:trans_new]{transformation object}
\item A numeric vector of positions
\item A function that takes the limits as input and returns breaks
as output (e.g., a function returned by \code{\link[scales:breaks_extended]{scales::extended_breaks()}}).
Also accepts rlang \link[rlang:as_function]{lambda} function notation.
}}

\item{labels}{One of:
\itemize{
\item \code{NULL} for no labels
\item \code{waiver()} for the default labels computed by the
transformation object
\item A character vector giving labels (must be same length as \code{breaks})
\item An expression vector (must be the same length as breaks). See ?plotmath for details.
\item A function that takes the breaks as input and returns labels
as output. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}

\item{limits}{One of:
\itemize{
\item \code{NULL} to use the default scale range
\item A numeric vector of length two providing limits of the scale.
Use \code{NA} to refer to the existing minimum or maximum
\item A function that accepts the existing (automatic) limits and returns
new limits. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
Note that setting limits on positional scales will \strong{remove} data outside of the limits.
If the purpose is to zoom, use the limit argument in the coordinate system
(see \code{\link[ggplot2:coord_cartesian]{coord_cartesian()}}).
}}

\item{range}{a numeric vector of length 2 that specifies the minimum and
maximum size of the plotting symbol after transformation.}

\item{trans}{For continuous scales, the name of a transformation object
or the object itself. Built-in transformations include "asn", "atanh",
"boxcox", "date", "exp", "hms", "identity", "log", "log10", "log1p", "log2",
"logit", "modulus", "probability", "probit", "pseudo_log", "reciprocal",
"reverse", "sqrt" and "time".

A transformation object bundles together a transform, its inverse,
and methods for generating breaks and labels. Transformation objects
are defined in the scales package, and are called \verb{<name>_trans} (e.g.,
\code{\link[scales:boxcox_trans]{scales::boxcox_trans()}}). You can create your own
transformation with \code{\link[scales:trans_new]{scales::trans_new()}}.}

\item{guide}{A function used to create a guide or its name. See
\code{\link[ggplot2:guides]{guides()}} for more information.}

\item{...}{
  Arguments passed on to \code{\link[ggplot2:continuous_scale]{continuous_scale}}
  \describe{
    \item{\code{minor_breaks}}{One of:
\itemize{
\item \code{NULL} for no minor breaks
\item \code{waiver()} for the default breaks (one minor break between
each major break)
\item A numeric vector of positions
\item A function that given the limits returns a vector of minor breaks. Also
accepts rlang \link[rlang:as_function]{lambda} function notation.
}}
    \item{\code{oob}}{One of:
\itemize{
\item Function that handles limits outside of the scale limits
(out of bounds). Also accepts rlang \link[rlang:as_function]{lambda}
function notation.
\item The default (\code{\link[scales:oob]{scales::censor()}}) replaces out of
bounds values with \code{NA}.
\item \code{\link[scales:oob]{scales::squish()}} for squishing out of bounds values into range.
\item \code{\link[scales:oob]{scales::squish_infinite()}} for squishing infinite values into range.
}}
    \item{\code{na.value}}{Missing values will be replaced with this value.}
    \item{\code{expand}}{For position scales, a vector of range expansion constants used to add some
padding around the data to ensure that they are placed some distance
away from the axes. Use the convenience function \code{\link[ggplot2:expansion]{expansion()}}
to generate the values for the \code{expand} argument. The defaults are to
expand the scale by 5\% on each side for continuous variables, and by
0.6 units on each side for discrete variables.}
    \item{\code{position}}{For position scales, The position of the axis.
\code{left} or \code{right} for y axes, \code{top} or \code{bottom} for x axes.}
    \item{\code{super}}{The super class to use for the constructed scale}
  }}

\item{n.breaks}{An integer guiding the number of major breaks. The algorithm
may choose a slightly different number to ensure nice break labels. Will
only have an effect if \code{breaks = waiver()}. Use \code{NULL} to use the default
number of breaks given by the transformation.}

\item{nice.breaks}{Logical. Should breaks be attempted placed at nice values
instead of exactly evenly spaced between the limits. If \code{TRUE} (default)
the scale will ask the transformation object to create breaks, and this
may result in a different number of breaks than requested. Ignored if
breaks are given explicitly.}

\item{max_size}{Size of largest points.}

\item{values}{a set of aesthetic values to map data values to. The values
will be matched in order (usually alphabetical) with the limits of the
scale, or with \code{breaks} if provided. If this is a named vector, then the
values will be matched based on the names instead. Data values that don't
match will be given \code{na.value}.}

\item{na.value}{The aesthetic value to use for missing (\code{NA}) values}
}
\value{
A ggproto object inheriting from \code{Scale}
}
\description{
This set of scales defines new size scales for edge geoms equivalent to the
ones already defined by ggplot2. See \code{\link[ggplot2:scale_size]{ggplot2::scale_size()}} for
more information. The different geoms will know whether to use edge scales or
the standard scales so it is not necessary to write \code{edge_size} in
the call to the geom - just use \code{size}.
}
\note{
In ggplot2 size conflates both line width and point size into one
scale. In ggraph there is also a width scale (\code{\link[=scale_edge_width]{scale_edge_width()}})
that is used for linewidth. As edges are often represented by lines the width
scale is the most common.
}
\seealso{
Other scale_edge_*: 
\code{\link{scale_edge_alpha}()},
\code{\link{scale_edge_colour}},
\code{\link{scale_edge_fill}},
\code{\link{scale_edge_linetype}()},
\code{\link{scale_edge_shape}()},
\code{\link{scale_edge_width}()},
\code{\link{scale_label_size}()}
}
\concept{scale_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_node_point.R
\name{geom_node_point}
\alias{geom_node_point}
\title{Show nodes as points}
\usage{
geom_node_point(
  mapping = NULL,
  data = NULL,
  position = "identity",
  show.legend = NA,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x and y are mapped to x and y in
the node data.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{position}{Position adjustment, either as a string naming the adjustment
(e.g. \code{"jitter"} to use \code{position_jitter}), or the result of a call to a
position adjustment function. Use the latter if you need to change the
settings of the adjustment.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}
}
\description{
This geom is equivalent in functionality to \code{\link[ggplot2:geom_point]{ggplot2::geom_point()}}
and allows for simple plotting of nodes in different shapes, colours and sizes.
}
\section{Aesthetics}{

\code{geom_node_point} understand the following aesthetics. Bold aesthetics are
automatically set, but can be overwritten.
\itemize{
\item \strong{x}
\item \strong{y}
\item alpha
\item colour
\item fill
\item shape
\item size
\item stroke
\item filter
}
}

\examples{
require(tidygraph)
gr <- create_notable('bull') \%>\%
  mutate(class = sample(letters[1:3], n(), replace = TRUE))

ggraph(gr, 'stress') + geom_node_point()
}
\seealso{
Other geom_node_*: 
\code{\link{geom_node_arc_bar}()},
\code{\link{geom_node_circle}()},
\code{\link{geom_node_range}()},
\code{\link{geom_node_sf}()},
\code{\link{geom_node_text}()},
\code{\link{geom_node_tile}()},
\code{\link{geom_node_voronoi}()}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_node_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_sf.R
\name{layout_tbl_graph_sf}
\alias{layout_tbl_graph_sf}
\title{Place nodes on their geographical space}
\usage{
layout_tbl_graph_sf(graph, circular = FALSE)
}
\arguments{
\item{graph}{An sfnetwork object}

\item{circular}{ignored}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular}.
}
\description{
This layout is built for objects of class \code{sfnetwork} and is meant to
plot a graph on its geographical space, by extracting its X and Y coordinates
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_cactustree}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_htree}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_metro}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/textAlong.R
\name{makeContent.textalong}
\alias{makeContent.textalong}
\title{Text angled according to line}
\usage{
\method{makeContent}{textalong}(x)
}
\description{
This function takes care of recalculating the angle of the text as the device
size changes
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tbl_graph.R
\name{layout_to_table}
\alias{layout_to_table}
\title{Convert a layout to a table}
\usage{
layout_to_table(layout, graph, ...)
}
\arguments{
\item{layout}{A supported object}

\item{graph}{A \code{tbl_graph}}

\item{...}{passed on to implementations}
}
\value{
A valid data.frame
}
\description{
This generic takes care of dispatching various layout types (names,
functions, tables) to their respective functions that will return a valid
layout table.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_node_tile.R
\name{geom_node_tile}
\alias{geom_node_tile}
\title{Draw the rectangles in a treemap}
\usage{
geom_node_tile(
  mapping = NULL,
  data = NULL,
  position = "identity",
  show.legend = NA,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x, y, width and height are mapped to
x, y, width and height in the node data.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{position}{Position adjustment, either as a string naming the adjustment
(e.g. \code{"jitter"} to use \code{position_jitter}), or the result of a call to a
position adjustment function. Use the latter if you need to change the
settings of the adjustment.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}
}
\description{
A treemap is a space filling layout that recursively divides a rectangle to
the children of the node. Often only the leaf nodes are drawn as nodes higher
up in the hierarchy would obscure what is below. \code{geom_treemap} is a
shorthand for \code{geom_node_treemap} as node is implicit in the case of
treemap drawing
}
\section{Aesthetics}{

\code{geom_treemap} understand the following aesthetics. Bold aesthetics are
automatically set, but can be overridden.
\itemize{
\item \strong{x}
\item \strong{y}
\item \strong{width}
\item \strong{height}
\item alpha
\item colour
\item fill
\item size
\item stroke
\item filter
}
}

\examples{
# Create a graph of the flare class system
library(tidygraph)
flareGraph <- tbl_graph(flare$vertices, flare$edges) \%>\%
  mutate(
    class = map_bfs_chr(node_is_root(), .f = function(node, dist, path, ...) {
      if (dist <= 1) {
        return(shortName[node])
      }
      path$result[[nrow(path)]]
    })
  )

ggraph(flareGraph, 'treemap', weight = size) +
  geom_node_tile(aes(fill = class, filter = leaf, alpha = depth), colour = NA) +
  geom_node_tile(aes(size = depth), colour = 'white') +
  scale_alpha(range = c(1, 0.5), guide = 'none') +
  scale_size(range = c(4, 0.2), guide = 'none')
}
\seealso{
Other geom_node_*: 
\code{\link{geom_node_arc_bar}()},
\code{\link{geom_node_circle}()},
\code{\link{geom_node_point}()},
\code{\link{geom_node_range}()},
\code{\link{geom_node_text}()},
\code{\link{geom_node_voronoi}()}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_node_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggraph.R, R/layout.R, R/tbl_graph.R
\name{ggraph}
\alias{ggraph}
\alias{create_layout}
\alias{layout_ggraph}
\alias{create_layout.default}
\alias{create_layout.layout_ggraph}
\alias{create_layout.tbl_graph}
\alias{layout_tbl_graph}
\title{Create a ggraph plot}
\usage{
ggraph(graph, layout = "auto", ...)

create_layout(graph, layout, circular, ...)

\method{create_layout}{default}(graph, layout, ...)

\method{create_layout}{layout_ggraph}(graph, ...)

\method{create_layout}{tbl_graph}(graph, layout, circular = FALSE, ...)
}
\arguments{
\item{graph}{The object containing the graph. See \emph{Details} for a list
of supported classes. Or a \code{layout_ggraph} object as returned from
\code{create_layout} in which case all subsequent arguments is ignored.}

\item{layout}{The type of layout to create. Either a valid string, a
function, a matrix, or a data.frame (see Details)}

\item{...}{Arguments passed on to the layout function.}

\item{circular}{Should the layout be transformed into a radial
representation. Only possible for some layouts. Defaults to \code{FALSE}}
}
\value{
For \code{ggraph()} an object of class gg onto which layers, scales,
etc. can be added. For \code{create_layout()} an object inheriting from
\code{layout_ggraph}. \code{layout_ggraph} itself inherits from
\code{data.frame} and can be considered as such. The data.frame contains
the node positions in the \code{x} and \code{y} column along with
additional columns generated by the specific layout, as well as node
parameters inherited from the graph. Additional information is stored as
attributes to the data.frame. The original graph object is stored in the
\code{graph} attribute and the \code{circular} attribute contains a logical
indicating whether the layout has been transformed to a circular
representation.
}
\description{
This function is the equivalent of \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}} in ggplot2.
It takes care of setting up the plot object along with creating the layout
for the plot based on the graph and the specification passed in.
Alternatively a layout can be prepared in advance using
\code{create_layout} and passed as the data argument. See \emph{Details} for
a description of all available layouts.
}
\details{
Following is a short description of the different layout types available in
ggraph. Each layout is further described in its own help pages. Any type of
regular graph/network data can be represented as a tbl_graph object. Because
of this the different layouts that can be applied to tbl_graph
objects are quite diverse, but not all layouts makes sense to all types of
graphs. It is up to the user to understand their data and choose an
appropriate layout. For standard node-edge diagrams igraph defines a
long range of different layout functions that are all available through the
\code{igraph} layout where the specific layout is specified using the
\code{algorithm} argument. In order to minimize typing all igraph algorithms
can also be passed directly into the \code{layout} argument.

Any object that has an appropriate \code{as_tbl_graph} method can be passed
into \code{ggraph()} and will automatically be converted underneath.

\describe{
\item{\code{auto}}{The default layout. See \code{\link[=layout_tbl_graph_auto]{layout_tbl_graph_auto()}} for further
details}
\item{\code{igraph}}{Use one of the internal igraph layout algorithms.
The algorithm is specified using the \code{algorithm} argument. All strings
accepted by the \code{algorithm} argument can also be supplied directly
into \code{layout}. See \code{\link[=layout_tbl_graph_igraph]{layout_tbl_graph_igraph()}} for further
details}
\item{\code{dendrogram}}{Lays out the nodes in a tree-like graph as a
dendrogram with leaves set at 0 and parents 1 unit above its tallest child.
See \code{\link[=layout_tbl_graph_dendrogram]{layout_tbl_graph_dendrogram()}} for further details}
\item{\code{manual}}{Lets the user manually specify the location of each
node. See \code{\link[=layout_tbl_graph_manual]{layout_tbl_graph_manual()}} for further details}
\item{\code{linear}}{Arranges the nodes linearly or circularly in order to
make an arc diagram. See \code{\link[=layout_tbl_graph_linear]{layout_tbl_graph_linear()}} for further
details}
\item{\code{matrix}}{Arranges nodes on a diagonal thus preparing it for use with
\code{\link[=geom_edge_point]{geom_edge_point()}} to make a matrix plot. See \code{\link[=layout_tbl_graph_matrix]{layout_tbl_graph_matrix()}}
for further details}
\item{\code{treemap}}{Creates a treemap from the graph, that is, a
space-filing subdivision of rectangles showing a weighted hierarchy. See
\code{\link[=layout_tbl_graph_treemap]{layout_tbl_graph_treemap()}} for further details}
\item{\code{circlepack}}{Creates a layout showing a hierarchy as circles
within circles. Conceptually equal to treemaps. See
\code{\link[=layout_tbl_graph_circlepack]{layout_tbl_graph_circlepack()}} for further details}
\item{\code{partition}}{Create icicle or sunburst charts, where each layer
subdivides the division given by the preceding layer. See
\code{\link[=layout_tbl_graph_partition]{layout_tbl_graph_partition()}} for further details}
\item{\code{hive}}{Positions nodes on axes spreading out from the center
based on node attributes. See \code{\link[=layout_tbl_graph_hive]{layout_tbl_graph_hive()}} for further
details}
}

Alternatively a matrix or a data.frame can be provided to the \code{layout}
argument. In the former case the first column will be used as x coordinates
and the second column will by used as y coordinates, further columns are
dropped. In the latter case the data.frame is used as the layout table and
must thus contain a numeric x and y column.

Lastly a function can be provided to the \code{layout} argument. It will be called
with the graph object as its first argument and any additional argument
passed into \code{ggraph()}/\code{create_layout()}. The function must return either a
data.frame or an object coercible to one and have an \code{x} and \code{y} column, or
an object coercible to a \code{tbl_graph}. In the latter case the node data is
extracted and used as layout (and must thus contain an \code{x} and \code{y} column)
and the graph will be added as the \code{graph} attribute.
}
\examples{
require(tidygraph)
gr <- create_notable('bull')
layout <- create_layout(gr, layout = 'igraph', algorithm = 'kk')
}
\seealso{
\code{\link[=get_edges]{get_edges()}} for extracting edge information from the
layout and \code{\link[=get_con]{get_con()}} for extracting path information.
}
\keyword{graph}
\keyword{hierarchy}
\keyword{layout}
\keyword{network}
\keyword{visualisation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_edge_arc.R
\name{geom_edge_arc}
\alias{geom_edge_arc}
\alias{geom_edge_arc2}
\alias{geom_edge_arc0}
\title{Draw edges as Arcs}
\usage{
geom_edge_arc(
  mapping = NULL,
  data = get_edges(),
  position = "identity",
  arrow = NULL,
  strength = 1,
  n = 100,
  fold = FALSE,
  lineend = "butt",
  linejoin = "round",
  linemitre = 1,
  label_colour = "black",
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = "rot",
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...,
  curvature
)

geom_edge_arc2(
  mapping = NULL,
  data = get_edges("long"),
  position = "identity",
  arrow = NULL,
  strength = 1,
  n = 100,
  fold = FALSE,
  lineend = "butt",
  linejoin = "round",
  linemitre = 1,
  label_colour = "black",
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = "rot",
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...,
  curvature
)

geom_edge_arc0(
  mapping = NULL,
  data = get_edges(),
  position = "identity",
  arrow = NULL,
  strength = 1,
  lineend = "butt",
  show.legend = NA,
  fold = fold,
  ...,
  curvature
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x, y, xend, yend, group and
circular are mapped to x, y, xend, yend, edge.id and circular in the edge
data.}

\item{data}{The return of a call to \code{get_edges()} or a data.frame
giving edges in correct format (see details for for guidance on the format).
See \code{\link[=get_edges]{get_edges()}} for more details on edge extraction.}

\item{position}{Position adjustment, either as a string naming the adjustment
(e.g. \code{"jitter"} to use \code{position_jitter}), or the result of a call to a
position adjustment function. Use the latter if you need to change the
settings of the adjustment.}

\item{arrow}{Arrow specification, as created by \code{\link[grid:arrow]{grid::arrow()}}.}

\item{strength}{The bend of the curve. 1 approximates a halfcircle while 0
will give a straight line. Negative number will change the direction of the
curve. Only used if \code{circular = FALSE}.}

\item{n}{The number of points to create along the path.}

\item{fold}{Logical. Should arcs appear on the same side of the nodes despite
different directions. Default to \code{FALSE}.}

\item{lineend}{Line end style (round, butt, square).}

\item{linejoin}{Line join style (round, mitre, bevel).}

\item{linemitre}{Line mitre limit (number greater than 1).}

\item{label_colour}{The colour of the edge label. If \code{NA} it will use
the colour of the edge.}

\item{label_alpha}{The opacity of the edge label. If \code{NA} it will use
the opacity of the edge.}

\item{label_parse}{If \code{TRUE}, the labels will be parsed into expressions
and displayed as described in \code{\link[grDevices:plotmath]{grDevices::plotmath()}}.}

\item{check_overlap}{If \code{TRUE}, text that overlaps previous text in the
same layer will not be plotted. \code{check_overlap} happens at draw time and in
the order of the data. Therefore data should be arranged by the label
column before calling \code{geom_text()}. Note that this argument is not
supported by \code{geom_label()}.}

\item{angle_calc}{Either 'none', 'along', or 'across'. If 'none' the label will
use the angle aesthetic of the geom. If 'along' The label will be written
along the edge direction. If 'across' the label will be written across the
edge direction.}

\item{force_flip}{Logical. If \code{angle_calc} is either 'along' or 'across'
should the label be flipped if it is on it's head. Default to \code{TRUE}.}

\item{label_dodge}{A \code{\link[grid:unit]{grid::unit()}} giving a fixed vertical shift
to add to the label in case of \code{angle_calc} is either 'along' or 'across'}

\item{label_push}{A \code{\link[grid:unit]{grid::unit()}} giving a fixed horizontal shift
to add to the label in case of \code{angle_calc} is either 'along' or 'across'}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}

\item{curvature}{Deprecated. Use \code{strength} instead.}
}
\description{
This geom is mainly intended for arc linear and circular diagrams (i.e. used
together with \code{\link[=layout_tbl_graph_linear]{layout_tbl_graph_linear()}}), though it can be used
elsewhere. It draws edges as arcs with a height proportional to the distance
between the nodes. Arcs are calculated as beziers. For linear layout the
placement of control points are related to the \code{curvature} argument and
the distance between the two nodes. For circular layout the control points
are placed on the same angle as the start and end node at a distance related
to the distance between the nodes.
}
\section{Aesthetics}{

\code{geom_edge_arc} and \code{geom_edge_arc0} understand the following
aesthetics. Bold aesthetics are automatically set, but can be overridden.
\itemize{
\item \strong{x}
\item \strong{y}
\item \strong{xend}
\item \strong{yend}
\item \strong{circular}
\item edge_colour
\item edge_width
\item edge_linetype
\item edge_alpha
\item filter
}

\code{geom_edge_arc2} understand the following aesthetics. Bold aesthetics are
automatically set, but can be overridden.
\itemize{
\item \strong{x}
\item \strong{y}
\item \strong{group}
\item \strong{circular}
\item edge_colour
\item edge_width
\item edge_linetype
\item edge_alpha
\item filter
}

\code{geom_edge_arc} and \code{geom_edge_arc2} furthermore takes the following
aesthetics.
\itemize{
\item start_cap
\item end_cap
\item label
\item label_pos
\item label_size
\item angle
\item hjust
\item vjust
\item family
\item fontface
\item lineheight
}
}

\section{Computed variables}{


\describe{
\item{index}{The position along the path (not computed for the *0 version)}
}
}

\section{Edge variants}{

Many geom_edge_* layers comes in 3 flavors depending on the level of control
needed over the drawing. The default (no numeric postfix) generate a number
of points (\code{n}) along the edge and draws it as a path. Each point along
the line has a numeric value associated with it giving the position along the
path, and it is therefore possible to show the direction of the edge by
mapping to this e.g. \code{colour = after_stat(index)}. The version postfixed with a
"2" uses the "long" edge format (see \code{\link[=get_edges]{get_edges()}}) and makes it
possible to interpolate node parameter between the start and end node along
the edge. It is considerable less performant so should only be used if this
is needed. The version postfixed with a "0" draws the edge in the most
performant way, often directly using an appropriate grob from the grid
package, but does not allow for gradients along the edge.

Often it is beneficial to stop the drawing of the edge before it reaches the
node, for instance in cases where an arrow should be drawn and the arrowhead
shouldn't lay on top or below the node point. geom_edge_* and geom_edge_*2
supports this through the start_cap and end_cap aesthetics that takes a
\code{\link[=geometry]{geometry()}} specification and dynamically caps the termini of the
edges based on the given specifications. This means that if
\code{end_cap = circle(1, 'cm')} the edges will end at a distance of 1cm even
during resizing of the plot window.

All \verb{geom_edge_*} and \code{geom_edge_*2} have the ability to draw a
label along the edge. The reason this is not a separate geom is that in order
for the label to know the location of the edge it needs to know the edge type
etc. Labels are drawn by providing a label aesthetic. The label_pos can be
used to specify where along the edge it should be drawn by supplying a number
between 0 and 1. The label_size aesthetic can be used to control the size of
the label. Often it is needed to have the label written along the direction
of the edge, but since the actual angle is dependent on the plot dimensions
this cannot be calculated beforehand. Using the angle_calc argument allows
you to specify whether to use the supplied angle aesthetic or whether to draw
the label along or across the edge.
}

\section{Edge aesthetic name expansion}{

In order to avoid excessive typing edge aesthetic names are
automatically expanded. Because of this it is not necessary to write
\code{edge_colour} within the \code{aes()} call as \code{colour} will
automatically be renamed appropriately.
}

\examples{
require(tidygraph)
# Make a graph with different directions of edges
gr <- create_notable('Meredith') \%>\%
  convert(to_directed) \%>\%
  mutate(class = sample(letters[1:3], n(), replace = TRUE)) \%>\%
  activate(edges) \%>\%
  mutate(
    class = sample(letters[1:3], n(), replace = TRUE),
    switch = sample(c(TRUE, FALSE), n(), replace = TRUE)
  ) \%>\%
  reroute(from = to, to = from, subset = switch)

ggraph(gr, 'linear') +
  geom_edge_arc(aes(alpha = after_stat(index)))

ggraph(gr, 'linear') +
  geom_edge_arc2(aes(colour = node.class), strength = 0.6)

ggraph(gr, 'linear', circular = TRUE) +
  geom_edge_arc0(aes(colour = class))
}
\seealso{
Other geom_edge_*: 
\code{\link{geom_edge_bend}()},
\code{\link{geom_edge_density}()},
\code{\link{geom_edge_diagonal}()},
\code{\link{geom_edge_elbow}()},
\code{\link{geom_edge_fan}()},
\code{\link{geom_edge_hive}()},
\code{\link{geom_edge_link}()},
\code{\link{geom_edge_loop}()},
\code{\link{geom_edge_parallel}()},
\code{\link{geom_edge_point}()},
\code{\link{geom_edge_span}()},
\code{\link{geom_edge_tile}()}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_node_voronoi.R
\name{geom_node_voronoi}
\alias{geom_node_voronoi}
\title{Show nodes as voronoi tiles}
\usage{
geom_node_voronoi(
  mapping = NULL,
  data = NULL,
  position = "identity",
  show.legend = NA,
  bound = NULL,
  eps = 1e-09,
  max.radius = NULL,
  normalize = FALSE,
  asp.ratio = 1,
  expand = 0,
  radius = 0,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x and y are mapped to x and y in
the node data and group set to \code{-1}.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{position}{Position adjustment, either as a string naming the adjustment
(e.g. \code{"jitter"} to use \code{position_jitter}), or the result of a call to a
position adjustment function. Use the latter if you need to change the
settings of the adjustment.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{bound}{The bounding rectangle for the tesselation or a custom polygon
to clip the tesselation to. Defaults to \code{NULL} which creates a rectangle
expanded 10\\% in all directions. If supplied as a bounding box it should be a
vector giving the bounds in the following order: xmin, xmax, ymin, ymax. If
supplied as a polygon it should either be a 2-column matrix or a data.frame
containing an \code{x} and \code{y} column.}

\item{eps}{A value of epsilon used in testing whether a quantity is zero,
mainly in the context of whether points are collinear. If anomalous errors
arise, it is possible that these may averted by adjusting the value of eps
upward or downward.}

\item{max.radius}{The maximum distance a tile can extend from the point of
origin. Will in effect clip each tile to a circle centered at the point with
the given radius. If \code{normalize = TRUE} the radius will be given relative to
the normalized values}

\item{normalize}{Should coordinates be normalized prior to calculations. If
\code{x} and \code{y} are in wildly different ranges it can lead to
tesselation and triangulation that seems off when plotted without
\code{\link[ggplot2:coord_fixed]{ggplot2::coord_fixed()}}. Normalization of coordinates solves this.
The coordinates are transformed back after calculations.}

\item{asp.ratio}{If \code{normalize = TRUE} the x values will be multiplied by this
amount after normalization.}

\item{expand}{A numeric or unit vector of length one, specifying the
expansion amount. Negative values will result in contraction instead. If the
value is given as a numeric it will be understood as a proportion of the
plot area width.}

\item{radius}{As \code{expand} but specifying the corner radius.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}
}
\description{
This geom is equivalent in functionality to \code{\link[ggforce:geom_delvor]{ggforce::geom_voronoi_tile()}}
and allows for plotting of nodes as tiles from a voronoi tesselation. As with
\code{\link[ggforce:geom_delvor]{ggforce::geom_voronoi_tile()}} it is possible to restrict the size of the
tile to a fixed radius, as well as round corners and expand/contract the
tile.
}
\section{Aesthetics}{

\code{geom_node_voronoi} understand the following aesthetics. Bold aesthetics are
automatically set, but can be overwritten.
\itemize{
\item \strong{x}
\item \strong{y}
\item alpha
\item colour
\item fill
\item shape
\item size
\item stroke
\item filter
}
}

\examples{
require(tidygraph)
gr <- create_notable('meredith') \%>\%
  mutate(group = sample(letters[1:4], n(), TRUE))

ggraph(gr) +
  geom_node_voronoi(aes(fill = group, colour = group), alpha = 0.3) +
  geom_edge_link(alpha = 0.3) +
  geom_node_point()

# Use max.radius to make the tesselation more "node"-like
ggraph(gr) +
  geom_node_voronoi(aes(fill = group, colour = group), alpha = 0.3, max.radius = 1) +
  geom_edge_link(alpha = 0.3) +
  geom_node_point()
}
\seealso{
Other geom_node_*: 
\code{\link{geom_node_arc_bar}()},
\code{\link{geom_node_circle}()},
\code{\link{geom_node_point}()},
\code{\link{geom_node_range}()},
\code{\link{geom_node_sf}()},
\code{\link{geom_node_text}()},
\code{\link{geom_node_tile}()}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_node_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_node_voronoi.R
\name{geom_node_voronoi}
\alias{geom_node_voronoi}
\title{Show nodes as voronoi tiles}
\usage{
geom_node_voronoi(
  mapping = NULL,
  data = NULL,
  position = "identity",
  show.legend = NA,
  bound = NULL,
  eps = 1e-09,
  max.radius = NULL,
  normalize = FALSE,
  asp.ratio = 1,
  expand = 0,
  radius = 0,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x and y are mapped to x and y in
the node data and group set to \code{-1}.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{bound}{The bounding rectangle for the tesselation or a custom polygon
to clip the tesselation to. Defaults to \code{NULL} which creates a rectangle
expanded 10\\% in all directions. If supplied as a bounding box it should be a
vector giving the bounds in the following order: xmin, xmax, ymin, ymax. If
supplied as a polygon it should either be a 2-column matrix or a data.frame
containing an \code{x} and \code{y} column.}

\item{eps}{A value of epsilon used in testing whether a quantity is zero,
mainly in the context of whether points are collinear. If anomalous errors
arise, it is possible that these may averted by adjusting the value of eps
upward or downward.}

\item{max.radius}{The maximum distance a tile can extend from the point of
origin. Will in effect clip each tile to a circle centered at the point with
the given radius. If \code{normalize = TRUE} the radius will be given relative to
the normalized values}

\item{normalize}{Should coordinates be normalized prior to calculations. If
\code{x} and \code{y} are in wildly different ranges it can lead to
tesselation and triangulation that seems off when plotted without
\code{\link[ggplot2:coord_fixed]{ggplot2::coord_fixed()}}. Normalization of coordinates solves this.
The coordinates are transformed back after calculations.}

\item{asp.ratio}{If \code{normalize = TRUE} the x values will be multiplied by this
amount after normalization.}

\item{expand}{A numeric or unit vector of length one, specifying the
expansion amount. Negative values will result in contraction instead. If the
value is given as a numeric it will be understood as a proportion of the
plot area width.}

\item{radius}{As \code{expand} but specifying the corner radius.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}'s \code{params} argument. These
arguments broadly fall into one of 4 categories below. Notably, further
arguments to the \code{position} argument, or aesthetics that are required
can \emph{not} be passed through \code{...}. Unknown arguments that are not part
of the 4 categories below are ignored.
\itemize{
\item Static aesthetics that are not mapped to a scale, but are at a fixed
value and apply to the layer as a whole. For example, \code{colour = "red"}
or \code{linewidth = 3}. The geom's documentation has an \strong{Aesthetics}
section that lists the available options. The 'required' aesthetics
cannot be passed on to the \code{params}. Please note that while passing
unmapped aesthetics as vectors is technically possible, the order and
required length is not guaranteed to be parallel to the input data.
\item When constructing a layer using
a \verb{stat_*()} function, the \code{...} argument can be used to pass on
parameters to the \code{geom} part of the layer. An example of this is
\code{stat_density(geom = "area", outline.type = "both")}. The geom's
documentation lists which parameters it can accept.
\item Inversely, when constructing a layer using a
\verb{geom_*()} function, the \code{...} argument can be used to pass on parameters
to the \code{stat} part of the layer. An example of this is
\code{geom_area(stat = "density", adjust = 0.5)}. The stat's documentation
lists which parameters it can accept.
\item The \code{key_glyph} argument of \code{\link[ggplot2:layer]{layer()}} may also be passed on through
\code{...}. This can be one of the functions described as
\link[ggplot2:draw_key]{key glyphs}, to change the display of the layer in the legend.
}}
}
\description{
This geom is equivalent in functionality to \code{\link[ggforce:geom_delvor]{ggforce::geom_voronoi_tile()}}
and allows for plotting of nodes as tiles from a voronoi tesselation. As with
\code{\link[ggforce:geom_delvor]{ggforce::geom_voronoi_tile()}} it is possible to restrict the size of the
tile to a fixed radius, as well as round corners and expand/contract the
tile.
}
\section{Aesthetics}{

\code{geom_node_voronoi} understand the following aesthetics. Bold aesthetics are
automatically set, but can be overwritten.
\itemize{
\item \strong{x}
\item \strong{y}
\item alpha
\item colour
\item fill
\item shape
\item size
\item stroke
\item filter
}
}

\examples{
require(tidygraph)
gr <- create_notable('meredith') \%>\%
  mutate(group = sample(letters[1:4], n(), TRUE))

ggraph(gr) +
  geom_node_voronoi(aes(fill = group, colour = group), alpha = 0.3) +
  geom_edge_link(alpha = 0.3) +
  geom_node_point()

# Use max.radius to make the tesselation more "node"-like
ggraph(gr) +
  geom_node_voronoi(aes(fill = group, colour = group), alpha = 0.3, max.radius = 1) +
  geom_edge_link(alpha = 0.3) +
  geom_node_point()
}
\seealso{
Other geom_node_*: 
\code{\link{geom_node_arc_bar}()},
\code{\link{geom_node_circle}()},
\code{\link{geom_node_point}()},
\code{\link{geom_node_range}()},
\code{\link{geom_node_sf}()},
\code{\link{geom_node_text}()},
\code{\link{geom_node_tile}()}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_node_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ggraph.R, R/layout.R, R/tbl_graph.R
\name{ggraph}
\alias{ggraph}
\alias{create_layout}
\alias{layout_ggraph}
\alias{create_layout.default}
\alias{create_layout.layout_ggraph}
\alias{create_layout.tbl_graph}
\alias{layout_tbl_graph}
\title{Create a ggraph plot}
\usage{
ggraph(graph, layout = "auto", ...)

create_layout(graph, layout, circular, ...)

\method{create_layout}{default}(graph, layout, ...)

\method{create_layout}{layout_ggraph}(graph, ...)

\method{create_layout}{tbl_graph}(graph, layout, circular = FALSE, ...)
}
\arguments{
\item{graph}{The object containing the graph. See \emph{Details} for a list
of supported classes. Or a \code{layout_ggraph} object as returned from
\code{create_layout} in which case all subsequent arguments is ignored.}

\item{layout}{The type of layout to create. Either a valid string, a
function, a matrix, or a data.frame (see Details)}

\item{...}{Arguments passed on to the layout function.}

\item{circular}{Should the layout be transformed into a radial
representation. Only possible for some layouts. Defaults to \code{FALSE}}
}
\value{
For \code{ggraph()} an object of class gg onto which layers, scales,
etc. can be added. For \code{create_layout()} an object inheriting from
\code{layout_ggraph}. \code{layout_ggraph} itself inherits from
\code{data.frame} and can be considered as such. The data.frame contains
the node positions in the \code{x} and \code{y} column along with
additional columns generated by the specific layout, as well as node
parameters inherited from the graph. Additional information is stored as
attributes to the data.frame. The original graph object is stored in the
\code{graph} attribute and the \code{circular} attribute contains a logical
indicating whether the layout has been transformed to a circular
representation.
}
\description{
This function is the equivalent of \code{\link[ggplot2:ggplot]{ggplot2::ggplot()}} in ggplot2.
It takes care of setting up the plot object along with creating the layout
for the plot based on the graph and the specification passed in.
Alternatively a layout can be prepared in advance using
\code{create_layout} and passed as the data argument. See \emph{Details} for
a description of all available layouts.
}
\details{
Following is a short description of the different layout types available in
ggraph. Each layout is further described in its own help pages. Any type of
regular graph/network data can be represented as a tbl_graph object. Because
of this the different layouts that can be applied to tbl_graph
objects are quite diverse, but not all layouts makes sense to all types of
graphs. It is up to the user to understand their data and choose an
appropriate layout. For standard node-edge diagrams igraph defines a
long range of different layout functions that are all available through the
\code{igraph} layout where the specific layout is specified using the
\code{algorithm} argument. In order to minimize typing all igraph algorithms
can also be passed directly into the \code{layout} argument.

Any object that has an appropriate \code{as_tbl_graph} method can be passed
into \code{ggraph()} and will automatically be converted underneath.

\describe{
\item{\code{auto}}{The default layout. See \code{\link[=layout_tbl_graph_auto]{layout_tbl_graph_auto()}} for further
details}
\item{\code{igraph}}{Use one of the internal igraph layout algorithms.
The algorithm is specified using the \code{algorithm} argument. All strings
accepted by the \code{algorithm} argument can also be supplied directly
into \code{layout}. See \code{\link[=layout_tbl_graph_igraph]{layout_tbl_graph_igraph()}} for further
details}
\item{\code{dendrogram}}{Lays out the nodes in a tree-like graph as a
dendrogram with leaves set at 0 and parents 1 unit above its tallest child.
See \code{\link[=layout_tbl_graph_dendrogram]{layout_tbl_graph_dendrogram()}} for further details}
\item{\code{manual}}{Lets the user manually specify the location of each
node. See \code{\link[=layout_tbl_graph_manual]{layout_tbl_graph_manual()}} for further details}
\item{\code{linear}}{Arranges the nodes linearly or circularly in order to
make an arc diagram. See \code{\link[=layout_tbl_graph_linear]{layout_tbl_graph_linear()}} for further
details}
\item{\code{matrix}}{Arranges nodes on a diagonal thus preparing it for use with
\code{\link[=geom_edge_point]{geom_edge_point()}} to make a matrix plot. See \code{\link[=layout_tbl_graph_matrix]{layout_tbl_graph_matrix()}}
for further details}
\item{\code{treemap}}{Creates a treemap from the graph, that is, a
space-filing subdivision of rectangles showing a weighted hierarchy. See
\code{\link[=layout_tbl_graph_treemap]{layout_tbl_graph_treemap()}} for further details}
\item{\code{circlepack}}{Creates a layout showing a hierarchy as circles
within circles. Conceptually equal to treemaps. See
\code{\link[=layout_tbl_graph_circlepack]{layout_tbl_graph_circlepack()}} for further details}
\item{\code{partition}}{Create icicle or sunburst charts, where each layer
subdivides the division given by the preceding layer. See
\code{\link[=layout_tbl_graph_partition]{layout_tbl_graph_partition()}} for further details}
\item{\code{hive}}{Positions nodes on axes spreading out from the center
based on node attributes. See \code{\link[=layout_tbl_graph_hive]{layout_tbl_graph_hive()}} for further
details}
\item{\code{cactustree}}{Positions nodes as circles on the periphery of their
parent circle. See \code{\link[=layout_tbl_graph_cactustree]{layout_tbl_graph_cactustree()}} for further
details}
\item{\code{backbone}}{Layout optimised for highly connected small-world graphs
such as social networks. See \code{\link[=layout_tbl_graph_backbone]{layout_tbl_graph_backbone()}} for further
details}
\item{\code{centrality}}{Place nodes around origin based on their centrality.
See \code{\link[=layout_tbl_graph_centrality]{layout_tbl_graph_centrality()}} for further
details}
\item{\code{eigen}}{Spectral layout based on the eigenvector of a matrix
representation of the graph. See \code{\link[=layout_tbl_graph_eigen]{layout_tbl_graph_eigen()}} for further
details}
\item{\code{fabric}}{Draw nodes as horizontal lines and connect them with
vertical lines if an edge exists between them. See
\code{\link[=layout_tbl_graph_fabric]{layout_tbl_graph_fabric()}} for further
details}
\item{\code{focus}}{Place nodes around a focus node based on their distance to
that node. See \code{\link[=layout_tbl_graph_focus]{layout_tbl_graph_focus()}} for further
details}
\item{\code{pmds}}{Layout based on multidimensional scaling of a set of pivot
nodes, allowing MDS layout to be used on larger graphs. See
\code{\link[=layout_tbl_graph_pmds]{layout_tbl_graph_pmds()}} for further
details}
\item{\code{stress}}{Layout based on stress minimisation with better stability
than Kamada-Kawai layout. See \code{\link[=layout_tbl_graph_stress]{layout_tbl_graph_stress()}} for further
details}
\item{\code{unrooted}}{Draws unrooted trees based on equal angle with optional
equal daylight modification. See \code{\link[=layout_tbl_graph_unrooted]{layout_tbl_graph_unrooted()}} for further
details}
\item{\code{htree}}{Draws binary trees as a space filling fractal. See
\code{\link[=layout_tbl_graph_htree]{layout_tbl_graph_htree()}} for further details}
}

Alternatively a matrix or a data.frame can be provided to the \code{layout}
argument. In the former case the first column will be used as x coordinates
and the second column will by used as y coordinates, further columns are
dropped. In the latter case the data.frame is used as the layout table and
must thus contain a numeric x and y column.

Lastly a function can be provided to the \code{layout} argument. It will be called
with the graph object as its first argument and any additional argument
passed into \code{ggraph()}/\code{create_layout()}. The function must return either a
data.frame or an object coercible to one and have an \code{x} and \code{y} column, or
an object coercible to a \code{tbl_graph}. In the latter case the node data is
extracted and used as layout (and must thus contain an \code{x} and \code{y} column)
and the graph will be added as the \code{graph} attribute.
}
\examples{
require(tidygraph)
gr <- create_notable('bull')
layout <- create_layout(gr, layout = 'igraph', algorithm = 'kk')
}
\seealso{
\code{\link[=get_edges]{get_edges()}} for extracting edge information from the
layout and \code{\link[=get_con]{get_con()}} for extracting path information.
}
\keyword{graph}
\keyword{hierarchy}
\keyword{layout}
\keyword{network}
\keyword{visualisation}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/theme_graph.R
\name{theme_graph}
\alias{theme_graph}
\alias{th_foreground}
\alias{th_no_axes}
\alias{set_graph_style}
\alias{unset_graph_style}
\title{A theme tuned for graph visualizations}
\usage{
theme_graph(
  base_family = "Arial Narrow",
  base_size = 11,
  background = "white",
  foreground = NULL,
  border = TRUE,
  text_colour = "black",
  bg_text_colour = text_colour,
  fg_text_colour = text_colour,
  title_family = base_family,
  title_size = 18,
  title_face = "bold",
  title_margin = 10,
  title_colour = bg_text_colour,
  subtitle_family = base_family,
  subtitle_size = 12,
  subtitle_face = "plain",
  subtitle_margin = 15,
  subtitle_colour = bg_text_colour,
  strip_text_family = base_family,
  strip_text_size = 10,
  strip_text_face = "bold",
  strip_text_colour = fg_text_colour,
  caption_family = base_family,
  caption_size = 9,
  caption_face = "italic",
  caption_margin = 10,
  caption_colour = bg_text_colour,
  plot_margin = margin(30, 30, 30, 30)
)

th_foreground(foreground = "grey80", fg_text_colour = NULL, border = FALSE)

th_no_axes()

set_graph_style(
  family = "Arial Narrow",
  face = "plain",
  size = 11,
  text_size = 11,
  text_colour = "black",
  ...
)

unset_graph_style()
}
\arguments{
\item{base_size, size, text_size, title_size, subtitle_size, strip_text_size, caption_size}{The size to use for the various text elements. \code{text_size} will be used as geom defaults}

\item{background}{The colour to use for the background. This theme sets all
background elements except for plot.background to \code{element_blank} so
this controls the background for all elements of the plot. Set to \code{NA}
to remove the background (thus making the plot transparent)}

\item{foreground}{The colour of foreground elements, specifically strip and
border. Set to \code{NA} to remove.}

\item{border}{Logical. Should border be drawn if a foreground colour is
provided?}

\item{text_colour, bg_text_colour, fg_text_colour, title_colour, subtitle_colour, strip_text_colour, caption_colour}{The colour of the text in the various text elements}

\item{title_margin, subtitle_margin, caption_margin}{The margin to use between the text elements and the plot area}

\item{plot_margin}{The plot margin}

\item{family, base_family, title_family, subtitle_family, strip_text_family, caption_family}{The font to use for the different elements}

\item{face, title_face, subtitle_face, strip_text_face, caption_face}{The fontface to use for the various text elements}

\item{...}{Parameters passed on the \code{theme_graph}}
}
\description{
When plotting graphs, networks, and trees the coordinate values are often of
no importance and axes are thus a distraction. \code{ggraph} comes with a
build-in theme that removes redundant elements in order to put focus on the
data. Furthermore the default behaviour is to use a narrow font so text takes
up less space. Theme colour is defined by a background and foreground colour
where the background defines the colour of the whole graphics area and the
foreground defines the colour of the strip and border. By default strip and
border is turned off as it is an unnecessary element unless facetting is
used. To add a foreground colour to a plot that is already using
\code{theme_graph} the \code{th_foreground} helper is provided. In order to
use this appearance as default use the \code{set_graph_style} function. An
added benefit of this is that it also changes the default text-related values
in the different geoms for a completely coherent look.
\code{unset_graph_style} can be used to revert the defaults back to their
default settings (that is, they are not necessarily reverted back to what
they were prior to calling \code{set_graph_style}). The \code{th_no_axes()} helper is
provided to modify an existing theme so that grid and axes are removed.
}
\examples{
library(tidygraph)
graph <- as_tbl_graph(highschool)

ggraph(graph) + geom_edge_link() + geom_node_point() + theme_graph()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_whigs.R
\docType{data}
\name{whigs}
\alias{whigs}
\title{Membership network of American Whigs}
\format{
The data is stored as an incidence matrix with persons as rows and
organizations as columns. A 0 means no membership while a one means
membership.
}
\source{
\url{https://github.com/kjhealy/revere/blob/master/data/PaulRevereAppD.csv}
adapted from:

Fischer, David H. (1995) \emph{Paul Revere's Ride}. Oxford University Press
}
\usage{
whigs
}
\description{
This dataset shows the membership of 136 colonial Americans in 5 whig
organization and is a bipartite graph. The data appeared in the appendix to
David Hackett Fischer's \emph{Paul Revere's Ride} (Oxford University Press,
1995) and compiled by Kieran Healy for the blog post
\href{https://kieranhealy.org/blog/archives/2013/06/09/using-metadata-to-find-paul-revere/}{Using Metadata to Find Paul Revere}.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geometry.R
\name{scale_type.ggraph_geometry}
\alias{scale_type.ggraph_geometry}
\title{Define default scale type for geometry}
\usage{
\method{scale_type}{ggraph_geometry}(x)
}
\description{
This function is quite useless as geometry is not meant to be scaled, but it
is a requirement for ggplot2 to handle it correctly.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_matrix.R
\name{layout_tbl_graph_matrix}
\alias{layout_tbl_graph_matrix}
\title{Place nodes on a diagonal}
\usage{
layout_tbl_graph_matrix(graph, circular = FALSE, sort.by = NULL)
}
\arguments{
\item{graph}{An \code{tbl_graph} object}

\item{circular}{Ignored}

\item{sort.by}{An expression providing the sorting of the nodes. If \code{NULL}
the nodes will be ordered by their index in the graph.}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
}
\description{
This layout puts all nodes on a diagonal, thus preparing the layout for use
with \code{\link[=geom_edge_point]{geom_edge_point()}} resulting in a matrix layout. While matrix
layouts excel in scalability, the interpretation of the visual is very
dependent on the sorting of the nodes. Different sorting algorithms have been
implemented in \code{tidygraph} and these can be used directly. Behrisch
\emph{et al.} (2016) have provided a nice overview of some of the different
sorting algorithms and what insight they might bring, along with a rundown of
different patterns to look out for.
}
\references{
Behrisch, M., Bach, B., Riche, N. H., Schreck, T., Fekete, J.-D. (2016).
\emph{Matrix Reordering Methods for Table and Network Visualization}.
Computer Graphics Forum, 35: 693–716. \url{https://doi.org/10.1111/cgf.12935}
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_treemap.R
\name{layout_tbl_graph_treemap}
\alias{layout_tbl_graph_treemap}
\title{Calculate nodes as rectangles subdividing that of their parent}
\usage{
layout_tbl_graph_treemap(
  graph,
  algorithm = "split",
  weight = NULL,
  circular = FALSE,
  sort.by = NULL,
  direction = "out",
  height = 1,
  width = 1
)
}
\arguments{
\item{graph}{A \code{tbl_graph} object}

\item{algorithm}{The name of the tiling algorithm to use. Defaults to 'split'}

\item{weight}{An optional node variable to use as weight. Will only affect
the weight of leaf nodes as the weight of non-leaf nodes are derived from
their children.}

\item{circular}{Logical. Should the layout be transformed to a circular
representation. Ignored.}

\item{sort.by}{The name of a node variables to sort the nodes by.}

\item{direction}{The direction of the tree in the graph. \code{'out'} (default)
means that parents point towards their children, while \code{'in'} means that
children point towards their parent.}

\item{height}{The height of the bounding rectangle}

\item{width}{The width of the bounding rectangle}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{width},
\code{height}, \code{leaf}, \code{depth}, \code{circular} as well as any
information stored as node variables in the tbl_graph object.
}
\description{
A treemap is a space filling hierarchical layout that maps nodes to
rectangles. The rectangles of the children of a node is packed into the
rectangle of the node so that the size of a rectangle is a function of the
size of the children. The size of the leaf nodes can be mapped arbitrarily
(defaults to 1). Many different algorithms exists for dividing a rectangle
into smaller bits, some optimizing the aspect ratio and some focusing on the
ordering of the rectangles. See details for more discussions on this. The
treemap layout was first developed by Ben Shneiderman for visualizing disk
usage in the early '90 and has seen many improvements since.
}
\details{
Different approaches to dividing the rectangles in a treemap exists; all with
their strengths and weaknesses. Currently only the split algorithm is
implemented which strikes a good balance between aspect ratio and order
preservation, but other, more well-known, algorithms such as squarify and
slice-and-dice will eventually be implemented.

\strong{Algorithms}

\emph{Split} (default)

The Split algorithm was developed by Bjorn Engdahl in order to address the
downsides of both the original slice-and-dice algorithm (poor aspect ratio)
and the popular squarify algorithm (no ordering of nodes). It works by
finding the best cut in the ordered list of children in terms of making sure
that the two rectangles associated with the split will have optimal aspect
ratio.
}
\note{
Treemap is a layout intended for trees, that is, graphs where nodes
only have one parent and zero or more children. If the provided graph does
not fit this format an attempt to convert it to such a format will be made.
}
\references{
Engdahl, B. (2005). \emph{Ordered and unordered treemap algorithms and their
applications on handheld devices}. Master's Degree Project.

Johnson, B., & Ben Shneiderman. (1991). \emph{Tree maps: A Space-Filling
Approach to the Visualization of Hierarchical Information Structures}. IEEE
Visualization, 284-291. \url{https://doi.org/10.1109/VISUAL.1991.175815}
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_matrix.R
\name{layout_tbl_graph_matrix}
\alias{layout_tbl_graph_matrix}
\title{Place nodes on a diagonal}
\usage{
layout_tbl_graph_matrix(graph, circular = FALSE, sort.by = NULL)
}
\arguments{
\item{graph}{An \code{tbl_graph} object}

\item{circular}{Ignored}

\item{sort.by}{An expression providing the sorting of the nodes. If \code{NULL}
the nodes will be ordered by their index in the graph.}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
}
\description{
This layout puts all nodes on a diagonal, thus preparing the layout for use
with \code{\link[=geom_edge_point]{geom_edge_point()}} resulting in a matrix layout. While matrix
layouts excel in scalability, the interpretation of the visual is very
dependent on the sorting of the nodes. Different sorting algorithms have been
implemented in \code{tidygraph} and these can be used directly. Behrisch
\emph{et al.} (2016) have provided a nice overview of some of the different
sorting algorithms and what insight they might bring, along with a rundown of
different patterns to look out for.
}
\references{
Behrisch, M., Bach, B., Riche, N. H., Schreck, T., Fekete, J.-D. (2016).
\emph{Matrix Reordering Methods for Table and Network Visualization}.
Computer Graphics Forum, 35: 693–716. \doi{10.1111/cgf.12935}
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_cactustree}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_htree}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_metro}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_sf}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_edge_alpha.R
\name{scale_edge_alpha}
\alias{scale_edge_alpha}
\alias{scale_edge_alpha_continuous}
\alias{scale_edge_alpha_discrete}
\alias{scale_edge_alpha_manual}
\alias{scale_edge_alpha_identity}
\title{Edge alpha scales}
\usage{
scale_edge_alpha(..., range = c(0.1, 1))

scale_edge_alpha_continuous(..., range = c(0.1, 1))

scale_edge_alpha_discrete(..., range = c(0.1, 1))

scale_edge_alpha_manual(..., values)

scale_edge_alpha_identity(..., guide = "none")
}
\arguments{
\item{...}{Other arguments passed on to \code{\link[ggplot2:continuous_scale]{continuous_scale()}}, \link[ggplot2]{binned_scale},
or \code{\link[ggplot2:discrete_scale]{discrete_scale()}} as appropriate, to control name, limits,
breaks, labels and so forth.}

\item{range}{Output range of alpha values. Must lie between 0 and 1.}

\item{values}{a set of aesthetic values to map data values to. The values
will be matched in order (usually alphabetical) with the limits of the
scale, or with \code{breaks} if provided. If this is a named vector, then the
values will be matched based on the names instead. Data values that don't
match will be given \code{na.value}.}

\item{guide}{Guide to use for this scale. Defaults to \code{"none"}.}
}
\value{
A ggproto object inheriting from \code{Scale}
}
\description{
This set of scales defines new alpha scales for edge geoms equivalent to the
ones already defined by ggplot2. See \code{\link[ggplot2:scale_alpha]{ggplot2::scale_alpha()}} for
more information. The different geoms will know whether to use edge scales or
the standard scales so it is not necessary to write \code{edge_alpha} in
the call to the geom - just use \code{alpha}.
}
\seealso{
Other scale_edge_*: 
\code{\link{scale_edge_colour}},
\code{\link{scale_edge_fill}},
\code{\link{scale_edge_linetype}()},
\code{\link{scale_edge_shape}()},
\code{\link{scale_edge_size}()},
\code{\link{scale_edge_width}()},
\code{\link{scale_label_size}()}
}
\concept{scale_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_conn_bundle.R
\name{geom_conn_bundle}
\alias{geom_conn_bundle}
\alias{geom_conn_bundle2}
\alias{geom_conn_bundle0}
\title{Create hierarchical edge bundles between node connections}
\usage{
geom_conn_bundle(
  mapping = NULL,
  data = get_con(),
  position = "identity",
  arrow = NULL,
  lineend = "butt",
  show.legend = NA,
  n = 100,
  tension = 0.8,
  ...
)

geom_conn_bundle2(
  mapping = NULL,
  data = get_con(),
  position = "identity",
  arrow = NULL,
  lineend = "butt",
  show.legend = NA,
  n = 100,
  tension = 0.8,
  ...
)

geom_conn_bundle0(
  mapping = NULL,
  data = get_con(),
  position = "identity",
  arrow = NULL,
  lineend = "butt",
  show.legend = NA,
  tension = 0.8,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x, y, xend, yend, group and
circular are mapped to x, y, xend, yend, edge.id and circular in the edge
data.}

\item{data}{The result of a call to \code{\link[=get_con]{get_con()}}}

\item{position}{Position adjustment, either as a string naming the adjustment
(e.g. \code{"jitter"} to use \code{position_jitter}), or the result of a call to a
position adjustment function. Use the latter if you need to change the
settings of the adjustment.}

\item{arrow}{Arrow specification, as created by \code{\link[grid:arrow]{grid::arrow()}}.}

\item{lineend}{Line end style (round, butt, square).}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{n}{The number of points to create along the path.}

\item{tension}{How "loose" should the bundles be. 1 will give very tight
bundles, while 0 will turn of bundling completely and give straight lines.
Defaults to 0.8}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}
}
\description{
Hierarchical edge bundling is a technique to introduce some order into the
hairball structure that can appear when there's a lot of overplotting and
edge crossing in a network plot. The concept requires that the network has
an intrinsic hierarchical structure that defines the layout but is not shown.
Connections between points (that is, not edges) are then drawn so that they
loosely follows the underlying hierarchical structure. This results in a
flow-like structure where lines that partly move in the same direction will
be bundled together.
}
\note{
In order to avoid excessive typing edge aesthetic names are
automatically expanded. Because of this it is not necessary to write
\code{edge_colour} within the \code{aes()} call as \code{colour} will
automatically be renamed appropriately.
}
\section{Aesthetics}{

geom_conn_bundle* understands the following aesthetics. Bold aesthetics are
automatically set, but can be overridden.
\itemize{
\item{\strong{x}}
\item{\strong{y}}
\item{\strong{group}}
\item{\strong{circular}}
\item{edge_colour}
\item{edge_width}
\item{edge_linetype}
\item{edge_alpha}
\item{filter}
}
}

\section{Computed variables}{


\describe{
\item{index}{The position along the path (not computed for the *0 version)}
}
}

\examples{
# Create a graph of the flare class system
library(tidygraph)
flareGraph <- tbl_graph(flare$vertices, flare$edges) \%>\%
  mutate(
    class = map_bfs_chr(node_is_root(), .f = function(node, dist, path, ...) {
      if (dist <= 1) {
        return(shortName[node])
      }
      path$result[[nrow(path)]]
    })
  )
importFrom <- match(flare$imports$from, flare$vertices$name)
importTo <- match(flare$imports$to, flare$vertices$name)

# Use class inheritance for layout but plot class imports as bundles
ggraph(flareGraph, 'dendrogram', circular = TRUE) +
  geom_conn_bundle(aes(colour = after_stat(index)),
    data = get_con(importFrom, importTo),
    edge_alpha = 0.25
  ) +
  geom_node_point(aes(filter = leaf, colour = class)) +
  scale_edge_colour_distiller('', direction = 1, guide = 'edge_direction') +
  coord_fixed() +
  ggforce::theme_no_axes()
}
\references{
Holten, D. (2006). \emph{Hierarchical edge bundles: visualization
of adjacency relations in hierarchical data.} IEEE Transactions on
Visualization and Computer Graphics, \strong{12}(5), 741-748.
\doi{10.1109/TVCG.2006.147}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_conn_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_backbone.R
\name{layout_tbl_graph_backbone}
\alias{layout_tbl_graph_backbone}
\title{Place node to emphasize group structure}
\usage{
layout_tbl_graph_backbone(graph, keep = 0.2, circular = FALSE)
}
\arguments{
\item{graph}{A tbl_graph object}

\item{keep}{The fraction of edges to use for creating the backbone}

\item{circular}{ignored}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
Further an edge attribute called \code{backbone} is added giving whether the edge
was selected as backbone.
}
\description{
This layout is optimised for drawing small-world types of graphs often found
in social networks, where distinct groups are still highly connected to the
remaining graph. Typical layouts struggle with this as they attempt to
minimise the edge length of all edges equally. The backbone layout is based
on weighing edges based on how well they hold together communities. The end
result is that communities tend to stick together despite high
interconnectivity.
}
\references{
Nocaj, A., Ortmann, M., & Brandes, U. (2015). \emph{Untangling the hairballs of
multi-centered, small-world online social media networks.} Journal of Graph
Algorithms and Applications: JGAA, 19(2), 595-618.
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\author{
The underlying algorithm is implemented in the graphlayouts package
by David Schoch
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_edge_alpha.R
\name{scale_edge_alpha}
\alias{scale_edge_alpha}
\alias{scale_edge_alpha_continuous}
\alias{scale_edge_alpha_discrete}
\alias{scale_edge_alpha_binned}
\alias{scale_edge_alpha_manual}
\alias{scale_edge_alpha_identity}
\title{Edge alpha scales}
\usage{
scale_edge_alpha(..., range = c(0.1, 1))

scale_edge_alpha_continuous(..., range = c(0.1, 1))

scale_edge_alpha_discrete(..., range = c(0.1, 1))

scale_edge_alpha_binned(..., range = c(0.1, 1))

scale_edge_alpha_manual(..., values, breaks = waiver(), na.value = NA)

scale_edge_alpha_identity(..., guide = "none")
}
\arguments{
\item{...}{Other arguments passed on to \code{\link[ggplot2:continuous_scale]{continuous_scale()}}, \code{\link[ggplot2:binned_scale]{binned_scale()}},
or \code{\link[ggplot2:discrete_scale]{discrete_scale()}} as appropriate, to control name, limits,
breaks, labels and so forth.}

\item{range}{Output range of alpha values. Must lie between 0 and 1.}

\item{values}{a set of aesthetic values to map data values to. The values
will be matched in order (usually alphabetical) with the limits of the
scale, or with \code{breaks} if provided. If this is a named vector, then the
values will be matched based on the names instead. Data values that don't
match will be given \code{na.value}.}

\item{breaks}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{waiver()} for the default breaks (the scale limits)
\item A character vector of breaks
\item A function that takes the limits as input and returns breaks
as output
}}

\item{na.value}{The aesthetic value to use for missing (\code{NA}) values}

\item{guide}{Guide to use for this scale. Defaults to \code{"none"}.}
}
\value{
A ggproto object inheriting from \code{Scale}
}
\description{
This set of scales defines new alpha scales for edge geoms equivalent to the
ones already defined by ggplot2. See \code{\link[ggplot2:scale_alpha]{ggplot2::scale_alpha()}} for
more information. The different geoms will know whether to use edge scales or
the standard scales so it is not necessary to write \code{edge_alpha} in
the call to the geom - just use \code{alpha}.
}
\seealso{
Other scale_edge_*: 
\code{\link{scale_edge_colour}},
\code{\link{scale_edge_fill}},
\code{\link{scale_edge_linetype}()},
\code{\link{scale_edge_shape}()},
\code{\link{scale_edge_size}()},
\code{\link{scale_edge_width}()},
\code{\link{scale_label_size}()}
}
\concept{scale_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_node_text.R
\name{geom_node_text}
\alias{geom_node_text}
\alias{geom_node_label}
\title{Annotate nodes with text}
\usage{
geom_node_text(
  mapping = NULL,
  data = NULL,
  position = "identity",
  parse = FALSE,
  nudge_x = 0,
  nudge_y = 0,
  check_overlap = FALSE,
  show.legend = NA,
  repel = FALSE,
  ...
)

geom_node_label(
  mapping = NULL,
  data = NULL,
  position = "identity",
  parse = FALSE,
  nudge_x = 0,
  nudge_y = 0,
  label.padding = unit(0.25, "lines"),
  label.r = unit(0.15, "lines"),
  label.size = 0.25,
  show.legend = NA,
  repel = FALSE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x and y are mapped to x and y in
the node data.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{position}{Position adjustment, either as a string, or the result of
a call to a position adjustment function. Cannot be jointy specified with
\code{nudge_x} or \code{nudge_y}.}

\item{parse}{If \code{TRUE}, the labels will be parsed into expressions and
displayed as described in \code{?plotmath}.}

\item{nudge_x, nudge_y}{Horizontal and vertical adjustment to nudge labels by.
Useful for offsetting text from points, particularly on discrete scales.}

\item{check_overlap}{If \code{TRUE}, text that overlaps previous text in the
same layer will not be plotted. \code{check_overlap} happens at draw time and in
the order of the data. Therefore data should be arranged by the label
column before calling \code{geom_text()}. Note that this argument is not
supported by \code{geom_label()}.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{repel}{If \code{TRUE}, text labels will be repelled from each other
to avoid overlapping, using the \code{GeomTextRepel} geom from the
ggrepel package.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}

\item{label.padding}{Amount of padding around label. Defaults to 0.25 lines.}

\item{label.r}{Radius of rounded corners. Defaults to 0.15 lines.}

\item{label.size}{Size of label border, in mm.}
}
\description{
These geoms are equivalent in functionality to \code{\link[ggplot2:geom_text]{ggplot2::geom_text()}} and
\code{\link[ggplot2:geom_text]{ggplot2::geom_label()}} and allows for simple annotation of nodes.
}
\section{Aesthetics}{

\code{geom_node_text} understands the following aesthetics. Bold aesthetics are
automatically set, but can be overridden. Italic aesthetics are required but
not set by default
\itemize{
\item \strong{x}
\item \strong{y}
\item \emph{label}
\item alpha
\item angle
\item colour
\item family
\item fontface
\item hjust
\item lineheight
\item size
\item vjust
}
}

\examples{
require(tidygraph)
gr <- create_notable('bull') \%>\%
  mutate(class = sample(letters[1:3], n(), replace = TRUE))

ggraph(gr, 'stress') +
  geom_node_point(aes(label = class))

ggraph(gr, 'stress') +
  geom_node_label(aes(label = class), repel = TRUE)
}
\seealso{
Other geom_node_*: 
\code{\link{geom_node_arc_bar}()},
\code{\link{geom_node_circle}()},
\code{\link{geom_node_point}()},
\code{\link{geom_node_range}()},
\code{\link{geom_node_tile}()},
\code{\link{geom_node_voronoi}()}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_node_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_backbone.R
\name{layout_tbl_graph_backbone}
\alias{layout_tbl_graph_backbone}
\title{Place node to emphasize group structure}
\usage{
layout_tbl_graph_backbone(graph, keep = 0.2, circular = FALSE)
}
\arguments{
\item{graph}{A tbl_graph object}

\item{keep}{The fraction of edges to use for creating the backbone}

\item{circular}{ignored}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
Further an edge attribute called \code{backbone} is added giving whether the edge
was selected as backbone.
}
\description{
This layout is optimised for drawing small-world types of graphs often found
in social networks, where distinct groups are still highly connected to the
remaining graph. Typical layouts struggle with this as they attempt to
minimise the edge length of all edges equally. The backbone layout is based
on weighing edges based on how well they hold together communities. The end
result is that communities tend to stick together despite high
interconnectivity.
}
\references{
Nocaj, A., Ortmann, M., & Brandes, U. (2015). \emph{Untangling the hairballs of
multi-centered, small-world online social media networks.} Journal of Graph
Algorithms and Applications: JGAA, 19(2), 595-618.
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_cactustree}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_htree}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_metro}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_sf}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\author{
The underlying algorithm is implemented in the graphlayouts package
by David Schoch
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_stress.R
\name{layout_tbl_graph_stress}
\alias{layout_tbl_graph_stress}
\alias{layout_tbl_graph_sparse_stress}
\title{Place nodes using stress majorisation}
\usage{
layout_tbl_graph_stress(
  graph,
  weights = NULL,
  niter = 500,
  tolerance = 1e-04,
  mds = TRUE,
  bbox = 50,
  circular = FALSE
)

layout_tbl_graph_sparse_stress(
  graph,
  pivots,
  weights = NULL,
  niter = 500,
  circular = FALSE
)
}
\arguments{
\item{graph}{a tbl_graph object}

\item{weights}{An expression evaluated on the edge data to provide edge
weights for the layout. Currently ignored for the sparse version}

\item{niter}{number of iterations during stress optimization}

\item{tolerance}{stopping criterion for stress optimization}

\item{mds}{should an MDS layout be used as initial layout (default: TRUE)}

\item{bbox}{constrain dimension of output. Only relevant to determine the
placement of disconnected graphs.}

\item{circular}{ignored}

\item{pivots}{The number of pivot nodes.}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
}
\description{
This layout is related to the stress-minimization algorithm known as
Kamada-Kawai (available as the 'kk' layout), but uses another optimization
strategy. It generally have better runtime, quality, and stability compared
to the Kamada-Kawai layout and is thus generally preferred. The sparse
version of the layout have better performance (especially on larger networks)
at the expense of layout quality, but will generally outperform many other
algorithms for large graphs in both runtime and quality (e.g. the 'drl'
layout from igraph).
}
\references{
Gansner, E. R., Koren, Y., & North, S. (2004). \emph{Graph drawing by stress
majorization.} In International Symposium on Graph Drawing (pp. 239-250). Springer, Berlin, Heidelberg.

Ortmann, M. and Klimenta, M. and Brandes, U. (2016). \emph{A Sparse Stress Model.} https://arxiv.org/pdf/1608.08909.pdf
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\author{
The underlying algorithm is implemented in the graphlayouts package
by David Schoch
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_edge_size.R
\name{scale_edge_size}
\alias{scale_edge_size}
\alias{scale_edge_size_continuous}
\alias{scale_edge_radius}
\alias{scale_edge_size_discrete}
\alias{scale_edge_size_binned}
\alias{scale_edge_size_area}
\alias{scale_edge_size_binned_area}
\alias{scale_edge_size_manual}
\alias{scale_edge_size_identity}
\title{Edge size scales}
\usage{
scale_edge_size_continuous(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  trans = "identity",
  guide = "legend"
)

scale_edge_radius(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  trans = "identity",
  guide = "legend"
)

scale_edge_size(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  trans = "identity",
  guide = "legend"
)

scale_edge_size_discrete(...)

scale_edge_size_binned(
  name = waiver(),
  breaks = waiver(),
  labels = waiver(),
  limits = NULL,
  range = c(1, 6),
  n.breaks = NULL,
  nice.breaks = TRUE,
  trans = "identity",
  guide = "bins"
)

scale_edge_size_area(..., max_size = 6)

scale_edge_size_binned_area(..., max_size = 6)

scale_edge_size_manual(..., values, breaks = waiver(), na.value = NA)

scale_edge_size_identity(..., guide = "none")
}
\arguments{
\item{name}{The name of the scale. Used as the axis or legend title. If
\code{waiver()}, the default, the name of the scale is taken from the first
mapping used for that aesthetic. If \code{NULL}, the legend title will be
omitted.}

\item{breaks}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{waiver()} for the default breaks computed by the
\link[scales:new_transform]{transformation object}
\item A numeric vector of positions
\item A function that takes the limits as input and returns breaks
as output (e.g., a function returned by \code{\link[scales:breaks_extended]{scales::extended_breaks()}}).
Note that for position scales, limits are provided after scale expansion.
Also accepts rlang \link[rlang:as_function]{lambda} function notation.
}}

\item{labels}{One of the options below. Please note that when \code{labels} is a
vector, it is highly recommended to also set the \code{breaks} argument as a
vector to protect against unintended mismatches.
\itemize{
\item \code{NULL} for no labels
\item \code{waiver()} for the default labels computed by the
transformation object
\item A character vector giving labels (must be same length as \code{breaks})
\item An expression vector (must be the same length as breaks). See ?plotmath for details.
\item A function that takes the breaks as input and returns labels
as output. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}

\item{limits}{One of:
\itemize{
\item \code{NULL} to use the default scale range
\item A numeric vector of length two providing limits of the scale.
Use \code{NA} to refer to the existing minimum or maximum
\item A function that accepts the existing (automatic) limits and returns
new limits. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
Note that setting limits on positional scales will \strong{remove} data outside of the limits.
If the purpose is to zoom, use the limit argument in the coordinate system
(see \code{\link[ggplot2:coord_cartesian]{coord_cartesian()}}).
}}

\item{range}{a numeric vector of length 2 that specifies the minimum and
maximum size of the plotting symbol after transformation.}

\item{trans}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Deprecated in favour of
\code{transform}.}

\item{guide}{A function used to create a guide or its name. See
\code{\link[ggplot2:guides]{guides()}} for more information.}

\item{...}{
  Arguments passed on to \code{\link[ggplot2:continuous_scale]{continuous_scale}}
  \describe{
    \item{\code{minor_breaks}}{One of:
\itemize{
\item \code{NULL} for no minor breaks
\item \code{waiver()} for the default breaks (none for discrete, one minor break
between each major break for continuous)
\item A numeric vector of positions
\item A function that given the limits returns a vector of minor breaks. Also
accepts rlang \link[rlang:as_function]{lambda} function notation. When
the function has two arguments, it will be given the limits and major
break positions.
}}
    \item{\code{oob}}{One of:
\itemize{
\item Function that handles limits outside of the scale limits
(out of bounds). Also accepts rlang \link[rlang:as_function]{lambda}
function notation.
\item The default (\code{\link[scales:oob]{scales::censor()}}) replaces out of
bounds values with \code{NA}.
\item \code{\link[scales:oob]{scales::squish()}} for squishing out of bounds values into range.
\item \code{\link[scales:oob]{scales::squish_infinite()}} for squishing infinite values into range.
}}
    \item{\code{na.value}}{Missing values will be replaced with this value.}
    \item{\code{call}}{The \code{call} used to construct the scale for reporting messages.}
    \item{\code{super}}{The super class to use for the constructed scale}
  }}

\item{n.breaks}{An integer guiding the number of major breaks. The algorithm
may choose a slightly different number to ensure nice break labels. Will
only have an effect if \code{breaks = waiver()}. Use \code{NULL} to use the default
number of breaks given by the transformation.}

\item{nice.breaks}{Logical. Should breaks be attempted placed at nice values
instead of exactly evenly spaced between the limits. If \code{TRUE} (default)
the scale will ask the transformation object to create breaks, and this
may result in a different number of breaks than requested. Ignored if
breaks are given explicitly.}

\item{max_size}{Size of largest points.}

\item{values}{a set of aesthetic values to map data values to. The values
will be matched in order (usually alphabetical) with the limits of the
scale, or with \code{breaks} if provided. If this is a named vector, then the
values will be matched based on the names instead. Data values that don't
match will be given \code{na.value}.}

\item{na.value}{The aesthetic value to use for missing (\code{NA}) values}
}
\value{
A ggproto object inheriting from \code{Scale}
}
\description{
This set of scales defines new size scales for edge geoms equivalent to the
ones already defined by ggplot2. See \code{\link[ggplot2:scale_size]{ggplot2::scale_size()}} for
more information. The different geoms will know whether to use edge scales or
the standard scales so it is not necessary to write \code{edge_size} in
the call to the geom - just use \code{size}.
}
\note{
In ggplot2 size conflates both line width and point size into one
scale. In ggraph there is also a width scale (\code{\link[=scale_edge_width]{scale_edge_width()}})
that is used for linewidth. As edges are often represented by lines the width
scale is the most common.
}
\seealso{
Other scale_edge_*: 
\code{\link{scale_edge_alpha}()},
\code{\link{scale_edge_colour}},
\code{\link{scale_edge_fill}},
\code{\link{scale_edge_linetype}()},
\code{\link{scale_edge_shape}()},
\code{\link{scale_edge_width}()},
\code{\link{scale_label_size}()}
}
\concept{scale_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_node_arc_bar.R
\name{geom_node_arc_bar}
\alias{geom_node_arc_bar}
\title{Show nodes as thick arcs}
\usage{
geom_node_arc_bar(
  mapping = NULL,
  data = NULL,
  position = "identity",
  show.legend = NA,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x and y are mapped to x0 and y0 in
the node data.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{position}{Position adjustment, either as a string naming the adjustment
(e.g. \code{"jitter"} to use \code{position_jitter}), or the result of a call to a
position adjustment function. Use the latter if you need to change the
settings of the adjustment.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}
}
\description{
This geom is equivalent in functionality to \code{\link[ggforce:geom_arc_bar]{ggforce::geom_arc_bar()}}
and allows for plotting of nodes as arcs with an inner and outer radius
scaled by the coordinate system. Its main use is currently in sunburst plots
as created with circular partition layouts
}
\section{Aesthetics}{

\code{geom_node_point} understand the following aesthetics. Bold aesthetics are
automatically set, but can be overwritten.
\itemize{
\item \strong{x0}
\item \strong{y0}
\item \strong{r0}
\item \strong{r}
\item \strong{start}
\item \strong{end}
\item alpha
\item colour
\item fill
\item shape
\item size
\item stroke
\item filter
}
}

\examples{
require(tidygraph)
gr <- tbl_graph(flare$vertices, flare$edges)
ggraph(gr, 'partition', circular = TRUE, weight = size) +
  geom_node_arc_bar()
}
\seealso{
Other geom_node_*: 
\code{\link{geom_node_circle}()},
\code{\link{geom_node_point}()},
\code{\link{geom_node_range}()},
\code{\link{geom_node_sf}()},
\code{\link{geom_node_text}()},
\code{\link{geom_node_tile}()},
\code{\link{geom_node_voronoi}()}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_node_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_stress.R
\name{layout_tbl_graph_stress}
\alias{layout_tbl_graph_stress}
\alias{layout_tbl_graph_sparse_stress}
\title{Place nodes using stress majorisation}
\usage{
layout_tbl_graph_stress(
  graph,
  weights = NULL,
  niter = 500,
  tolerance = 1e-04,
  mds = TRUE,
  bbox = 50,
  x = NULL,
  y = NULL,
  circular = FALSE
)

layout_tbl_graph_sparse_stress(
  graph,
  pivots,
  weights = NULL,
  niter = 500,
  circular = FALSE
)
}
\arguments{
\item{graph}{a tbl_graph object}

\item{weights}{An expression evaluated on the edge data to provide edge
weights for the layout. Currently ignored for the sparse version}

\item{niter}{number of iterations during stress optimization}

\item{tolerance}{stopping criterion for stress optimization}

\item{mds}{should an MDS layout be used as initial layout (default: TRUE)}

\item{bbox}{constrain dimension of output. Only relevant to determine the
placement of disconnected graphs.}

\item{x, y}{Expressions evaluated on the node data giving
coordinates along x and/or y axis to fix nodes to. You can chose to only fix
selected nodes by leaving the remaining nodes with \code{NA} values.}

\item{circular}{ignored}

\item{pivots}{The number of pivot nodes.}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
}
\description{
This layout is related to the stress-minimization algorithm known as
Kamada-Kawai (available as the 'kk' layout), but uses another optimization
strategy. It generally have better runtime, quality, and stability compared
to the Kamada-Kawai layout and is thus generally preferred. The sparse
version of the layout have better performance (especially on larger networks)
at the expense of layout quality, but will generally outperform many other
algorithms for large graphs in both runtime and quality (e.g. the 'drl'
layout from igraph).
}
\references{
Gansner, E. R., Koren, Y., & North, S. (2004). \emph{Graph drawing by stress
majorization.} In International Symposium on Graph Drawing (pp. 239-250). Springer, Berlin, Heidelberg.

Ortmann, M. and Klimenta, M. and Brandes, U. (2016). \emph{A Sparse Stress Model.} https://arxiv.org/pdf/1608.08909.pdf
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_cactustree}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_htree}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_metro}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_sf}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\author{
The underlying algorithm is implemented in the graphlayouts package
by David Schoch
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_node_arc_bar.R
\name{geom_node_arc_bar}
\alias{geom_node_arc_bar}
\title{Show nodes as thick arcs}
\usage{
geom_node_arc_bar(
  mapping = NULL,
  data = NULL,
  position = "identity",
  show.legend = NA,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x and y are mapped to x0 and y0 in
the node data.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}'s \code{params} argument. These
arguments broadly fall into one of 4 categories below. Notably, further
arguments to the \code{position} argument, or aesthetics that are required
can \emph{not} be passed through \code{...}. Unknown arguments that are not part
of the 4 categories below are ignored.
\itemize{
\item Static aesthetics that are not mapped to a scale, but are at a fixed
value and apply to the layer as a whole. For example, \code{colour = "red"}
or \code{linewidth = 3}. The geom's documentation has an \strong{Aesthetics}
section that lists the available options. The 'required' aesthetics
cannot be passed on to the \code{params}. Please note that while passing
unmapped aesthetics as vectors is technically possible, the order and
required length is not guaranteed to be parallel to the input data.
\item When constructing a layer using
a \verb{stat_*()} function, the \code{...} argument can be used to pass on
parameters to the \code{geom} part of the layer. An example of this is
\code{stat_density(geom = "area", outline.type = "both")}. The geom's
documentation lists which parameters it can accept.
\item Inversely, when constructing a layer using a
\verb{geom_*()} function, the \code{...} argument can be used to pass on parameters
to the \code{stat} part of the layer. An example of this is
\code{geom_area(stat = "density", adjust = 0.5)}. The stat's documentation
lists which parameters it can accept.
\item The \code{key_glyph} argument of \code{\link[ggplot2:layer]{layer()}} may also be passed on through
\code{...}. This can be one of the functions described as
\link[ggplot2:draw_key]{key glyphs}, to change the display of the layer in the legend.
}}
}
\description{
This geom is equivalent in functionality to \code{\link[ggforce:geom_arc_bar]{ggforce::geom_arc_bar()}}
and allows for plotting of nodes as arcs with an inner and outer radius
scaled by the coordinate system. Its main use is currently in sunburst plots
as created with circular partition layouts
}
\section{Aesthetics}{

\code{geom_node_point} understand the following aesthetics. Bold aesthetics are
automatically set, but can be overwritten.
\itemize{
\item \strong{x0}
\item \strong{y0}
\item \strong{r0}
\item \strong{r}
\item \strong{start}
\item \strong{end}
\item alpha
\item colour
\item fill
\item shape
\item size
\item stroke
\item filter
}
}

\examples{
require(tidygraph)
gr <- tbl_graph(flare$vertices, flare$edges)
ggraph(gr, 'partition', circular = TRUE, weight = size) +
  geom_node_arc_bar()
}
\seealso{
Other geom_node_*: 
\code{\link{geom_node_circle}()},
\code{\link{geom_node_point}()},
\code{\link{geom_node_range}()},
\code{\link{geom_node_sf}()},
\code{\link{geom_node_text}()},
\code{\link{geom_node_tile}()},
\code{\link{geom_node_voronoi}()}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_node_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edge_colourbar.R, R/edge_direction.R
\name{guide_train.edge_colourbar}
\alias{guide_train.edge_colourbar}
\alias{guide_train.edge_direction}
\alias{guide_merge.edge_direction}
\alias{guide_geom.edge_direction}
\alias{guide_gengrob.edge_direction}
\title{Helper methods for guides}
\usage{
\method{guide_train}{edge_colourbar}(guide, scale, aesthetic = NULL)

\method{guide_train}{edge_direction}(guide, scale, aesthetic = NULL)

\method{guide_merge}{edge_direction}(guide, new_guide)

\method{guide_geom}{edge_direction}(guide, layers, default_mapping)

\method{guide_gengrob}{edge_direction}(guide, theme)
}
\description{
Helper methods for guides
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geometry.R
\name{geometry}
\alias{geometry}
\alias{circle}
\alias{square}
\alias{ellipsis}
\alias{rectangle}
\alias{label_rect}
\alias{is.geometry}
\title{Define simple shapes for line capping}
\usage{
geometry(
  type = "circle",
  width = 1,
  height = width,
  width_unit = "cm",
  height_unit = width_unit
)

circle(radius = 1, unit = "cm")

square(length = 1, unit = "cm")

ellipsis(a = 1, b = 1, a_unit = "cm", b_unit = a_unit)

rectangle(width = 1, height = 1, width_unit = "cm", height_unit = width_unit)

label_rect(label, padding = margin(1, 1, 1.5, 1, "mm"), ...)

is.geometry(x)
}
\arguments{
\item{type}{The type of geometry to use. Currently \code{'circle'} and
\code{'rect'} is supported.}

\item{width, height, length, radius, a, b}{The dimensions of the shape.}

\item{unit, width_unit, height_unit, a_unit, b_unit}{The unit for the numbers
given.}

\item{label}{The text to be enclosed}

\item{padding}{extra size to be added around the text using the
\code{\link[ggplot2:element]{ggplot2::margin()}} function}

\item{...}{Passed on to \code{\link[grid:gpar]{grid::gpar()}}}

\item{x}{An object to test for geometry inheritance}
}
\value{
A geometry object encoding the specified shape.
}
\description{
This set of functions makes it easy to define shapes at the terminal points
of edges that are used to shorten the edges. The shapes themselves are not
drawn, but the edges will end at the boundary of the shape rather than at
the node position. This is especially relevant when drawing arrows at the
edges as the arrows will be partly obscured by the node unless the edge is
shortened. Edge shortening is dynamic and will update as the plot is resized,
making sure that the capping remains at an absolute distance to the end
point.
}
\details{
\code{geometry} is the base constructor, while the rest are helpers to save
typing. \code{circle} creates circles width a given radius, \code{square}
creates squares at a given side length, \code{ellipsis} creates ellipses with
given a and b values (width and height radii), and \code{rectangle} makes
rectangles of a given width and height. label_rect is a helper that, given
a list of strings and potentially formatting options creates a rectangle that
encloses the string.
}
\examples{
geometry(c('circle', 'rect', 'rect'), 1:3, 3:1)

circle(1:4, 'mm')

label_rect(c('some', 'different', 'words'), fontsize = 18)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_edge_density.R
\name{geom_edge_density}
\alias{geom_edge_density}
\title{Show edges as a density map}
\usage{
geom_edge_density(
  mapping = NULL,
  data = get_edges("short"),
  position = "identity",
  show.legend = NA,
  n = 100,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x, y, xend, yend, group and
circular are mapped to x, y, xend, yend, edge.id and circular in the edge
data.}

\item{data}{The return of a call to \code{get_edges()} or a data.frame
giving edges in correct format (see details for for guidance on the format).
See \code{\link[=get_edges]{get_edges()}} for more details on edge extraction.}

\item{position}{Position adjustment, either as a string naming the adjustment
(e.g. \code{"jitter"} to use \code{position_jitter}), or the result of a call to a
position adjustment function. Use the latter if you need to change the
settings of the adjustment.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{n}{The number of points to estimate in the x and y direction, i.e. the
resolution of the raster.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}
}
\description{
This geom makes it possible to add a layer showing edge presence as a density
map. Each edge is converted to \code{n} points along the line and a jitter is
applied. Based on this dataset a two-dimensional kernel density estimation is
applied and plotted as a raster image. The density is mapped to the alpha
level, making it possible to map a variable to the fill.
}
\section{Aesthetics}{

\code{geom_edge_density} understand the following aesthetics. Bold aesthetics are
automatically set, but can be overwritten.

\strong{x}
\strong{y}
\strong{xend}
\strong{yend}
edge_fill
filter
}

\section{Computed variables}{


\describe{
\item{x, y}{The coordinates for each pixel in the raster}
\item{density}{The density associated with the pixel}
}
}

\section{Edge aesthetic name expansion}{

In order to avoid excessive typing edge aesthetic names are
automatically expanded. Because of this it is not necessary to write
\code{edge_colour} within the \code{aes()} call as \code{colour} will
automatically be renamed appropriately.
}

\examples{
require(tidygraph)
gr <- create_notable('bull') \%>\%
  activate(edges) \%>\%
  mutate(class = sample(letters[1:3], n(), replace = TRUE))

ggraph(gr, 'stress') +
  geom_edge_density(aes(fill = class)) +
  geom_edge_link() + geom_node_point()
}
\seealso{
Other geom_edge_*: 
\code{\link{geom_edge_arc}()},
\code{\link{geom_edge_bend}()},
\code{\link{geom_edge_bundle_force}()},
\code{\link{geom_edge_bundle_minimal}()},
\code{\link{geom_edge_bundle_path}()},
\code{\link{geom_edge_diagonal}()},
\code{\link{geom_edge_elbow}()},
\code{\link{geom_edge_fan}()},
\code{\link{geom_edge_hive}()},
\code{\link{geom_edge_link}()},
\code{\link{geom_edge_loop}()},
\code{\link{geom_edge_parallel}()},
\code{\link{geom_edge_point}()},
\code{\link{geom_edge_sf}()},
\code{\link{geom_edge_span}()},
\code{\link{geom_edge_tile}()}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_edge_density.R
\name{geom_edge_density}
\alias{geom_edge_density}
\title{Show edges as a density map}
\usage{
geom_edge_density(
  mapping = NULL,
  data = get_edges("short"),
  position = "identity",
  show.legend = NA,
  n = 100,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x, y, xend, yend, group and
circular are mapped to x, y, xend, yend, edge.id and circular in the edge
data.}

\item{data}{The return of a call to \code{get_edges()} or a data.frame
giving edges in correct format (see details for for guidance on the format).
See \code{\link[=get_edges]{get_edges()}} for more details on edge extraction.}

\item{position}{Position adjustment, either as a string, or the result of
a call to a position adjustment function.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{n}{The number of points to estimate in the x and y direction, i.e. the
resolution of the raster.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}
}
\description{
This geom makes it possible to add a layer showing edge presence as a density
map. Each edge is converted to \code{n} points along the line and a jitter is
applied. Based on this dataset a two-dimensional kernel density estimation is
applied and plotted as a raster image. The density is mapped to the alpha
level, making it possible to map a variable to the fill.
}
\section{Aesthetics}{

\code{geom_edge_density} understand the following aesthetics. Bold aesthetics are
automatically set, but can be overridden.

\strong{x}
\strong{y}
\strong{xend}
\strong{yend}
edge_fill
filter
}

\section{Computed variables}{


\describe{
\item{x, y}{The coordinates for each pixel in the raster}
\item{density}{The density associated with the pixel}
}
}

\section{Edge aesthetic name expansion}{

In order to avoid excessive typing edge aesthetic names are
automatically expanded. Because of this it is not necessary to write
\code{edge_colour} within the \code{aes()} call as \code{colour} will
automatically be renamed appropriately.
}

\examples{
require(tidygraph)
gr <- create_notable('bull') \%>\%
  activate(edges) \%>\%
  mutate(class = sample(letters[1:3], n(), replace = TRUE))

ggraph(gr, 'stress') +
  geom_edge_density(aes(fill = class)) +
  geom_edge_link() + geom_node_point()
}
\seealso{
Other geom_edge_*: 
\code{\link{geom_edge_arc}()},
\code{\link{geom_edge_bend}()},
\code{\link{geom_edge_diagonal}()},
\code{\link{geom_edge_elbow}()},
\code{\link{geom_edge_fan}()},
\code{\link{geom_edge_hive}()},
\code{\link{geom_edge_link}()},
\code{\link{geom_edge_loop}()},
\code{\link{geom_edge_parallel}()},
\code{\link{geom_edge_point}()},
\code{\link{geom_edge_span}()},
\code{\link{geom_edge_tile}()}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\docType{import}
\name{reexports}
\alias{reexports}
\alias{scale_color_viridis}
\alias{scale_fill_viridis}
\title{Objects exported from other packages}
\keyword{internal}
\description{
These objects are imported from other packages. Follow the links
below to see their documentation.

\describe{
  \item{viridis}{\code{\link[viridis:scale_viridis]{scale_color_viridis}}, \code{\link[viridis:scale_viridis]{scale_fill_viridis}}}
}}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_circlepack.R
\name{layout_tbl_graph_circlepack}
\alias{layout_tbl_graph_circlepack}
\title{Calculate nodes as circles packed within their parent circle}
\usage{
layout_tbl_graph_circlepack(
  graph,
  weight = NULL,
  circular = FALSE,
  sort.by = NULL,
  direction = "out"
)
}
\arguments{
\item{graph}{An \code{tbl_graph} object}

\item{weight}{An optional node variable to use as weight. Will only affect
the weight of leaf nodes as the weight of non-leaf nodes are derived from
their children.}

\item{circular}{Logical. Should the layout be transformed to a circular
representation. Ignored.}

\item{sort.by}{The name of a node variable to sort the nodes by.}

\item{direction}{The direction of the tree in the graph. \code{'out'} (default)
means that parents point towards their children, while \code{'in'} means that
children point towards their parent.}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{r}, \code{leaf},
\code{depth}, \code{circular} as well as any information stored as node
variables in the tbl_graph object.
}
\description{
The circle packing algorithm is basically a treemap using circles instead of
rectangles. Due to the nature of circles they cannot be packed as efficiently
leading to increased amount of "empty space" as compared to a treemap. This
can be beneficial though, as the added empty space can aid in visually
showing the hierarchy.
}
\details{
The circle packing is based on the algorithm developed by Weixin Wang and
collaborators which tries to find the most dense packing of circles as they
are added, one by one. This makes the algorithm very dependent on the order
in which circles are added and it is possible that layouts could sometimes
be optimized by choosing a different ordering. The algorithm for finding the
enclosing circle is the randomized incremental algorithm proposed by Emo
Welzl. Both of the above algorithms are the same as used in the D3.js
implementation of circle packing and their C++ implementation in ggraph is
inspired by Mike Bostocks JavaScript implementation.
}
\note{
Circle packing is a layout intended for trees, that is, graphs where nodes
only have one parent and zero or more children. If the provided graph does
not fit this format an attempt to convert it to such a format will be made.
}
\references{
Wang, W., Wang, H. H., Dai, G., & Wang, H. (2006). \emph{Visualization of
large hierarchical data by circle packing}. Chi, 517-520.

Welzl, E. (1991). \emph{Smallest enclosing disks (balls and ellipsoids)}. New
Results and New Trends in Computer Science, 359-370.
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_axis_hive.R
\name{geom_axis_hive}
\alias{geom_axis_hive}
\title{Draw rectangular bars and labels on hive axes}
\usage{
geom_axis_hive(
  mapping = NULL,
  data = NULL,
  position = "identity",
  label = TRUE,
  axis = TRUE,
  show.legend = NA,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}}. If specified and
\code{inherit.aes = TRUE} (the default), it is combined with the default mapping
at the top level of the plot. You must supply \code{mapping} if there is no plot
mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{position}{Position adjustment, either as a string naming the adjustment
(e.g. \code{"jitter"} to use \code{position_jitter}), or the result of a call to a
position adjustment function. Use the latter if you need to change the
settings of the adjustment.}

\item{label}{Should the axes be labelled. Defaults to \code{TRUE}}

\item{axis}{Should a rectangle be drawn along the axis. Defaults to \code{TRUE}}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}
}
\description{
This function lets you annotate the axes in a hive plot with labels and
color coded bars.
}
\section{Aesthetics}{

geom_axis_hive understand the following aesthetics.
\itemize{
\item{alpha}
\item{colour}
\item{fill}
\item{size}
\item{linetype}
\item{label_size}
\item{family}
\item{fontface}
\item{lineheight}
}
}

\examples{
# Plot the flare import graph as a hive plot
library(tidygraph)
flareGr <- as_tbl_graph(flare$imports) \%>\%
  mutate(
    type = dplyr::case_when(
      centrality_degree(mode = 'in') == 0 ~ 'Source',
      centrality_degree(mode = 'out') == 0 ~ 'Sink',
      TRUE ~ 'Both'
    )
  ) \%>\%
  activate(edges) \%>\%
  mutate(
    type = dplyr::case_when(
      grepl('flare.analytics', paste(.N()$name[from], .N()$name[to])) ~ 'Analytics',
      TRUE ~ 'Other'
    )
  )
ggraph(flareGr, 'hive', axis = type) +
  geom_edge_hive(aes(colour = type), edge_alpha = 0.1) +
  geom_axis_hive(aes(colour = type)) +
  coord_fixed()
}
\author{
Thomas Lin Pedersen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RcppExports.R
\name{pack_circles}
\alias{pack_circles}
\title{Pack circles together}
\usage{
pack_circles(areas)
}
\arguments{
\item{areas}{A vector of circle areas}
}
\value{
A matrix with two columns and the same number of rows as the length
of the "areas" vector. The matrix has the following attributes added:
"enclosing_radius" giving the radius of the smallest enclosing circle, and
"front_chain" giving the terminating members of the front chain (see
Wang \emph{et al}. 2006).
}
\description{
This function is a direct interface to the circle packing algorithm used by
\code{\link{layout_tbl_graph_circlepack}}. It takes a vector of sizes and
returns the x and y position of each circle as a two-column matrix.
}
\examples{
library(ggforce)
sizes <- sample(10, 100, TRUE)

position <- pack_circles(sizes)
data <- data.frame(x = position[,1], y = position[,2], r = sqrt(sizes/pi))

ggplot() +
  geom_circle(aes(x0 = x, y0 = y, r = r), data = data, fill = 'steelblue') +
  geom_circle(aes(x0 = 0, y0 = 0, r = attr(position, 'enclosing_radius'))) +
  geom_polygon(aes(x = x, y = y),
               data = data[attr(position, 'front_chain'), ],
               fill = NA,
               colour = 'black')

}
\references{
Wang, W., Wang, H. H., Dai, G., & Wang, H. (2006). \emph{Visualization of
large hierarchical data by circle packing}. Chi, 517-520.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_axis_hive.R
\name{geom_axis_hive}
\alias{geom_axis_hive}
\title{Draw rectangular bars and labels on hive axes}
\usage{
geom_axis_hive(
  mapping = NULL,
  data = NULL,
  position = "identity",
  label = TRUE,
  axis = TRUE,
  show.legend = NA,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{aes()}} or
\code{\link[ggplot2:aes_]{aes_()}}. If specified and \code{inherit.aes = TRUE} (the
default), it is combined with the default mapping at the top level of the
plot. You must supply \code{mapping} if there is no plot mapping.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{position}{Position adjustment, either as a string, or the result of
a call to a position adjustment function.}

\item{label}{Should the axes be labelled. Defaults to \code{TRUE}}

\item{axis}{Should a rectangle be drawn along the axis. Defaults to \code{TRUE}}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}
}
\description{
This function lets you annotate the axes in a hive plot with labels and
color coded bars.
}
\section{Aesthetics}{

geom_axis_hive understand the following aesthetics.
\itemize{
\item{alpha}
\item{colour}
\item{fill}
\item{size}
\item{linetype}
\item{label_size}
\item{family}
\item{fontface}
\item{lineheight}
}
}

\examples{
# Plot the flare import graph as a hive plot
library(tidygraph)
flareGr <- as_tbl_graph(flare$imports) \%>\%
  mutate(
    type = dplyr::case_when(
      centrality_degree(mode = 'in') == 0 ~ 'Source',
      centrality_degree(mode = 'out') == 0 ~ 'Sink',
      TRUE ~ 'Both'
    )
  ) \%>\%
  activate(edges) \%>\%
  mutate(
    type = dplyr::case_when(
      grepl('flare.analytics', paste(.N()$name[from], .N()$name[to])) ~ 'Analytics',
      TRUE ~ 'Other'
    )
  )
ggraph(flareGr, 'hive', axis = type) +
  geom_edge_hive(aes(colour = type), edge_alpha = 0.1) +
  geom_axis_hive(aes(colour = type)) +
  coord_fixed()
}
\author{
Thomas Lin Pedersen
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_node_sf.R, R/nodes.R
\name{get_sf_nodes}
\alias{get_sf_nodes}
\alias{get_nodes}
\title{Create a node extractor function}
\usage{
get_sf_nodes()

get_nodes(...)
}
\arguments{
\item{...}{Additional data that should be cbind'ed together with the node
data. Accepts expressions that will be evaluated on the node data in it's
original order (irrespective of any reordering by the layout)}
}
\value{
A data.frame with the node data as well of any additional data
supplied through \code{...}
}
\description{
This function returns another function that can extract nodes from a
ggraph_layout object. As a ggraph_layout object is essentially a data.frame
of nodes it might seem useless to provide this function, but since the node
data is not necessarily available until after the \code{ggraph()} call it
can be beneficial to be able to add information to the node data on a
per-layer basis. Unlike \code{\link[=get_edges]{get_edges()}} the use of \code{get_nodes} is not
mandatory and is only required if additional data should be added to selected
node layers.
}
\seealso{
Other extractors: 
\code{\link{get_con}()},
\code{\link{get_edges}()}
}
\concept{extractors}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_node_tile.R
\name{geom_node_tile}
\alias{geom_node_tile}
\title{Draw the rectangles in a treemap}
\usage{
geom_node_tile(
  mapping = NULL,
  data = NULL,
  position = "identity",
  show.legend = NA,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x, y, width and height are mapped to
x, y, width and height in the node data.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display. To include legend keys for all levels, even
when no data exists, use \code{TRUE}.  If \code{NA}, all levels are shown in legend,
but unobserved levels are omitted.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}'s \code{params} argument. These
arguments broadly fall into one of 4 categories below. Notably, further
arguments to the \code{position} argument, or aesthetics that are required
can \emph{not} be passed through \code{...}. Unknown arguments that are not part
of the 4 categories below are ignored.
\itemize{
\item Static aesthetics that are not mapped to a scale, but are at a fixed
value and apply to the layer as a whole. For example, \code{colour = "red"}
or \code{linewidth = 3}. The geom's documentation has an \strong{Aesthetics}
section that lists the available options. The 'required' aesthetics
cannot be passed on to the \code{params}. Please note that while passing
unmapped aesthetics as vectors is technically possible, the order and
required length is not guaranteed to be parallel to the input data.
\item When constructing a layer using
a \verb{stat_*()} function, the \code{...} argument can be used to pass on
parameters to the \code{geom} part of the layer. An example of this is
\code{stat_density(geom = "area", outline.type = "both")}. The geom's
documentation lists which parameters it can accept.
\item Inversely, when constructing a layer using a
\verb{geom_*()} function, the \code{...} argument can be used to pass on parameters
to the \code{stat} part of the layer. An example of this is
\code{geom_area(stat = "density", adjust = 0.5)}. The stat's documentation
lists which parameters it can accept.
\item The \code{key_glyph} argument of \code{\link[ggplot2:layer]{layer()}} may also be passed on through
\code{...}. This can be one of the functions described as
\link[ggplot2:draw_key]{key glyphs}, to change the display of the layer in the legend.
}}
}
\description{
A treemap is a space filling layout that recursively divides a rectangle to
the children of the node. Often only the leaf nodes are drawn as nodes higher
up in the hierarchy would obscure what is below. \code{geom_treemap} is a
shorthand for \code{geom_node_treemap} as node is implicit in the case of
treemap drawing
}
\section{Aesthetics}{

\code{geom_treemap} understand the following aesthetics. Bold aesthetics are
automatically set, but can be overwritten.
\itemize{
\item \strong{x}
\item \strong{y}
\item \strong{width}
\item \strong{height}
\item alpha
\item colour
\item fill
\item size
\item stroke
\item filter
}
}

\examples{
# Create a graph of the flare class system
library(tidygraph)
flareGraph <- tbl_graph(flare$vertices, flare$edges) \%>\%
  mutate(
    class = map_bfs_chr(node_is_root(), .f = function(node, dist, path, ...) {
      if (dist <= 1) {
        return(shortName[node])
      }
      path$result[[nrow(path)]]
    })
  )

ggraph(flareGraph, 'treemap', weight = size) +
  geom_node_tile(aes(fill = class, filter = leaf, alpha = depth), colour = NA) +
  geom_node_tile(aes(linewidth = depth), colour = 'white') +
  scale_alpha(range = c(1, 0.5), guide = 'none') +
  scale_size(range = c(4, 0.2), guide = 'none')
}
\seealso{
Other geom_node_*: 
\code{\link{geom_node_arc_bar}()},
\code{\link{geom_node_circle}()},
\code{\link{geom_node_point}()},
\code{\link{geom_node_range}()},
\code{\link{geom_node_sf}()},
\code{\link{geom_node_text}()},
\code{\link{geom_node_voronoi}()}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_node_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/nodes.R
\name{get_nodes}
\alias{get_nodes}
\title{Create a node extractor function}
\usage{
get_nodes(...)
}
\arguments{
\item{...}{Additional data that should be cbind'ed together with the node
data.}
}
\value{
A data.frame with the node data as well of any additional data
supplied through \code{...}
}
\description{
This function returns another function that can extract nodes from a
ggraph_layout object. As a ggraph_layout object is essentially a data.frame
of nodes it might seem useless to provide this function, but since the node
data is not necessarily available until after the \code{ggraph()} call it
can be beneficial to be able to add information to the node data on a
per-layer basis. Unlike \code{\link[=get_edges]{get_edges()}} the use of \code{get_nodes} is not
mandatory and is only required if additional data should be added to selected
node layers.
}
\seealso{
Other extractors: 
\code{\link{get_con}()},
\code{\link{get_edges}()}
}
\concept{extractors}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_dendrogram.R
\name{layout_tbl_graph_dendrogram}
\alias{layout_tbl_graph_dendrogram}
\title{Apply a dendrogram layout to layout_tbl_graph}
\usage{
layout_tbl_graph_dendrogram(
  graph,
  circular = FALSE,
  offset = pi/2,
  height = NULL,
  length = NULL,
  repel = FALSE,
  ratio = 1,
  direction = "out"
)
}
\arguments{
\item{graph}{A \code{tbl_graph} object}

\item{circular}{Logical. Should the layout be transformed to a circular
representation. Defaults to \code{FALSE}.}

\item{offset}{If \code{circular = TRUE}, where should it begin. Defaults to
\code{pi/2} which is equivalent to 12 o'clock.}

\item{height}{The node variable holding the height of each node in the
dendrogram. If \code{NULL} it will be calculated as the maximal distance to a
leaf.}

\item{length}{An edge parameter giving the length of each edge. The node
height will be calculated from the maximal length to the root node (ignored
if \code{height} does not evaluate to \code{NULL})}

\item{repel}{Should leafs repel each other relative to the height of their
common ancestor. Will emphasize clusters}

\item{ratio}{The strength of repulsion if \code{repel = TRUE}. Higher values will
give more defined clusters}

\item{direction}{The direction to the leaves. Defaults to 'out'}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular}, \code{depth} and
\code{leaf} as well as any information stored as node variables on the
tbl_graph
}
\description{
This layout mimics the \code{\link[igraph:layout_as_tree]{igraph::layout_as_tree()}} algorithm
supplied by igraph, but puts all leaves at 0 and builds it up from there,
instead of starting from the root and building it from there. The height of
branch points are related to the maximum distance to an edge from the branch
node, or read from a node variable.
}
\note{
This function is not intended to be used directly but by setting
\code{layout = 'dendrogram'} in \code{\link[=create_layout]{create_layout()}}
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_cactustree}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_htree}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_metro}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_sf}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_dendrogram.R
\name{layout_tbl_graph_dendrogram}
\alias{layout_tbl_graph_dendrogram}
\title{Apply a dendrogram layout to layout_tbl_graph}
\usage{
layout_tbl_graph_dendrogram(
  graph,
  circular = FALSE,
  offset = pi/2,
  height = NULL,
  length = NULL,
  repel = FALSE,
  ratio = 1,
  direction = "out"
)
}
\arguments{
\item{graph}{A \code{tbl_graph} object}

\item{circular}{Logical. Should the layout be transformed to a circular
representation. Defaults to \code{FALSE}.}

\item{offset}{If \code{circular = TRUE}, where should it begin. Defaults to
\code{pi/2} which is equivalent to 12 o'clock.}

\item{height}{The node variable holding the height of each node in the
dendrogram. If \code{NULL} it will be calculated as the maximal distance to a
leaf.}

\item{length}{An edge parameter giving the length of each edge. The node
height will be calculated from the maximal length to the root node (ignored
if \code{height} does not evaluate to \code{NULL})}

\item{repel}{Should leafs repel each other relative to the height of their
common ancestor. Will emphasize clusters}

\item{ratio}{The strength of repulsion if \code{repel = TRUE}. Higher values will
give more defined clusters}

\item{direction}{The direction to the leaves. Defaults to 'out'}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular}, \code{depth} and
\code{leaf} as well as any information stored as node variables on the
tbl_graph
}
\description{
This layout mimics the \code{\link[igraph:layout_as_tree]{igraph::layout_as_tree()}} algorithm
supplied by igraph, but puts all leaves at 0 and builds it up from there,
instead of starting from the root and building it from there. The height of
branch points are related to the maximum distance to an edge from the branch
node, or read from a node variable.
}
\note{
This function is not intended to be used directly but by setting
\code{layout = 'dendrogram'} in \code{\link[=create_layout]{create_layout()}}
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/connections.R, R/edges.R
\name{internal_extractors}
\alias{internal_extractors}
\alias{collect_connections}
\alias{collect_edges}
\title{Internal data extractors}
\usage{
collect_connections(layout, from, to, ...)

collect_edges(layout)
}
\arguments{
\item{layout}{The layout data}

\item{from, to}{A numeric vector giving the indexes of the start and end nodes}

\item{...}{Additional parameters passed on to the specific method}
}
\description{
These functions exists for supporting different data structures. There is no
need to call these directly
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_edge_arc.R
\name{geom_edge_arc}
\alias{geom_edge_arc}
\alias{geom_edge_arc2}
\alias{geom_edge_arc0}
\title{Draw edges as Arcs}
\usage{
geom_edge_arc(
  mapping = NULL,
  data = get_edges(),
  position = "identity",
  arrow = NULL,
  strength = 1,
  n = 100,
  fold = FALSE,
  lineend = "butt",
  linejoin = "round",
  linemitre = 1,
  label_colour = "black",
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = "rot",
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...,
  curvature
)

geom_edge_arc2(
  mapping = NULL,
  data = get_edges("long"),
  position = "identity",
  arrow = NULL,
  strength = 1,
  n = 100,
  fold = FALSE,
  lineend = "butt",
  linejoin = "round",
  linemitre = 1,
  label_colour = "black",
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = "rot",
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...,
  curvature
)

geom_edge_arc0(
  mapping = NULL,
  data = get_edges(),
  position = "identity",
  arrow = NULL,
  strength = 1,
  lineend = "butt",
  show.legend = NA,
  fold = fold,
  ...,
  curvature
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x, y, xend, yend, group and
circular are mapped to x, y, xend, yend, edge.id and circular in the edge
data.}

\item{data}{The return of a call to \code{get_edges()} or a data.frame
giving edges in correct format (see details for for guidance on the format).
See \code{\link[=get_edges]{get_edges()}} for more details on edge extraction.}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{arrow}{Arrow specification, as created by \code{\link[grid:arrow]{grid::arrow()}}.}

\item{strength}{The bend of the curve. 1 approximates a halfcircle while 0
will give a straight line. Negative number will change the direction of the
curve. Only used if \code{circular = FALSE}.}

\item{n}{The number of points to create along the path.}

\item{fold}{Logical. Should arcs appear on the same side of the nodes despite
different directions. Default to \code{FALSE}.}

\item{lineend}{Line end style (round, butt, square).}

\item{linejoin}{Line join style (round, mitre, bevel).}

\item{linemitre}{Line mitre limit (number greater than 1).}

\item{label_colour}{The colour of the edge label. If \code{NA} it will use
the colour of the edge.}

\item{label_alpha}{The opacity of the edge label. If \code{NA} it will use
the opacity of the edge.}

\item{label_parse}{If \code{TRUE}, the labels will be parsed into expressions
and displayed as described in \code{\link[grDevices:plotmath]{grDevices::plotmath()}}.}

\item{check_overlap}{If \code{TRUE}, text that overlaps previous text in the
same layer will not be plotted. \code{check_overlap} happens at draw time and in
the order of the data. Therefore data should be arranged by the label
column before calling \code{geom_text()}. Note that this argument is not
supported by \code{geom_label()}.}

\item{angle_calc}{Either 'none', 'along', or 'across'. If 'none' the label will
use the angle aesthetic of the geom. If 'along' The label will be written
along the edge direction. If 'across' the label will be written across the
edge direction.}

\item{force_flip}{Logical. If \code{angle_calc} is either 'along' or 'across'
should the label be flipped if it is on it's head. Default to \code{TRUE}.}

\item{label_dodge}{A \code{\link[grid:unit]{grid::unit()}} giving a fixed vertical shift
to add to the label in case of \code{angle_calc} is either 'along' or 'across'}

\item{label_push}{A \code{\link[grid:unit]{grid::unit()}} giving a fixed horizontal shift
to add to the label in case of \code{angle_calc} is either 'along' or 'across'}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display. To include legend keys for all levels, even
when no data exists, use \code{TRUE}.  If \code{NA}, all levels are shown in legend,
but unobserved levels are omitted.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}'s \code{params} argument. These
arguments broadly fall into one of 4 categories below. Notably, further
arguments to the \code{position} argument, or aesthetics that are required
can \emph{not} be passed through \code{...}. Unknown arguments that are not part
of the 4 categories below are ignored.
\itemize{
\item Static aesthetics that are not mapped to a scale, but are at a fixed
value and apply to the layer as a whole. For example, \code{colour = "red"}
or \code{linewidth = 3}. The geom's documentation has an \strong{Aesthetics}
section that lists the available options. The 'required' aesthetics
cannot be passed on to the \code{params}. Please note that while passing
unmapped aesthetics as vectors is technically possible, the order and
required length is not guaranteed to be parallel to the input data.
\item When constructing a layer using
a \verb{stat_*()} function, the \code{...} argument can be used to pass on
parameters to the \code{geom} part of the layer. An example of this is
\code{stat_density(geom = "area", outline.type = "both")}. The geom's
documentation lists which parameters it can accept.
\item Inversely, when constructing a layer using a
\verb{geom_*()} function, the \code{...} argument can be used to pass on parameters
to the \code{stat} part of the layer. An example of this is
\code{geom_area(stat = "density", adjust = 0.5)}. The stat's documentation
lists which parameters it can accept.
\item The \code{key_glyph} argument of \code{\link[ggplot2:layer]{layer()}} may also be passed on through
\code{...}. This can be one of the functions described as
\link[ggplot2:draw_key]{key glyphs}, to change the display of the layer in the legend.
}}

\item{curvature}{Deprecated. Use \code{strength} instead.}
}
\description{
This geom is mainly intended for arc linear and circular diagrams (i.e. used
together with \code{\link[=layout_tbl_graph_linear]{layout_tbl_graph_linear()}}), though it can be used
elsewhere. It draws edges as arcs with a height proportional to the distance
between the nodes. Arcs are calculated as beziers. For linear layout the
placement of control points are related to the \code{curvature} argument and
the distance between the two nodes. For circular layout the control points
are placed on the same angle as the start and end node at a distance related
to the distance between the nodes.
}
\section{Aesthetics}{

\code{geom_edge_arc} and \code{geom_edge_arc0} understand the following
aesthetics. Bold aesthetics are automatically set, but can be overwritten.
\itemize{
\item \strong{x}
\item \strong{y}
\item \strong{xend}
\item \strong{yend}
\item \strong{circular}
\item edge_colour
\item edge_width
\item edge_linetype
\item edge_alpha
\item filter
}

\code{geom_edge_arc2} understand the following aesthetics. Bold aesthetics are
automatically set, but can be overwritten.
\itemize{
\item \strong{x}
\item \strong{y}
\item \strong{group}
\item \strong{circular}
\item edge_colour
\item edge_width
\item edge_linetype
\item edge_alpha
\item filter
}

\code{geom_edge_arc} and \code{geom_edge_arc2} furthermore takes the following
aesthetics.
\itemize{
\item start_cap
\item end_cap
\item label
\item label_pos
\item label_size
\item angle
\item hjust
\item vjust
\item family
\item fontface
\item lineheight
}
}

\section{Computed variables}{


\describe{
\item{index}{The position along the path (not computed for the *0 version)}
}
}

\section{Edge variants}{

Many geom_edge_* layers comes in 3 flavors depending on the level of control
needed over the drawing. The default (no numeric postfix) generate a number
of points (\code{n}) along the edge and draws it as a path. Each point along
the line has a numeric value associated with it giving the position along the
path, and it is therefore possible to show the direction of the edge by
mapping to this e.g. \code{colour = after_stat(index)}. The version postfixed with a
"2" uses the "long" edge format (see \code{\link[=get_edges]{get_edges()}}) and makes it
possible to interpolate node parameter between the start and end node along
the edge. It is considerable less performant so should only be used if this
is needed. The version postfixed with a "0" draws the edge in the most
performant way, often directly using an appropriate grob from the grid
package, but does not allow for gradients along the edge.

Often it is beneficial to stop the drawing of the edge before it reaches the
node, for instance in cases where an arrow should be drawn and the arrowhead
shouldn't lay on top or below the node point. geom_edge_* and geom_edge_*2
supports this through the start_cap and end_cap aesthetics that takes a
\code{\link[=geometry]{geometry()}} specification and dynamically caps the termini of the
edges based on the given specifications. This means that if
\code{end_cap = circle(1, 'cm')} the edges will end at a distance of 1cm even
during resizing of the plot window.

All \verb{geom_edge_*} and \code{geom_edge_*2} have the ability to draw a
label along the edge. The reason this is not a separate geom is that in order
for the label to know the location of the edge it needs to know the edge type
etc. Labels are drawn by providing a label aesthetic. The label_pos can be
used to specify where along the edge it should be drawn by supplying a number
between 0 and 1. The label_size aesthetic can be used to control the size of
the label. Often it is needed to have the label written along the direction
of the edge, but since the actual angle is dependent on the plot dimensions
this cannot be calculated beforehand. Using the angle_calc argument allows
you to specify whether to use the supplied angle aesthetic or whether to draw
the label along or across the edge.
}

\section{Edge aesthetic name expansion}{

In order to avoid excessive typing edge aesthetic names are
automatically expanded. Because of this it is not necessary to write
\code{edge_colour} within the \code{aes()} call as \code{colour} will
automatically be renamed appropriately.
}

\examples{
require(tidygraph)
# Make a graph with different directions of edges
gr <- create_notable('Meredith') \%>\%
  convert(to_directed) \%>\%
  mutate(class = sample(letters[1:3], n(), replace = TRUE)) \%>\%
  activate(edges) \%>\%
  mutate(
    class = sample(letters[1:3], n(), replace = TRUE),
    switch = sample(c(TRUE, FALSE), n(), replace = TRUE)
  ) \%>\%
  reroute(from = to, to = from, subset = switch)

ggraph(gr, 'linear') +
  geom_edge_arc(aes(alpha = after_stat(index)))

ggraph(gr, 'linear') +
  geom_edge_arc2(aes(colour = node.class), strength = 0.6)

ggraph(gr, 'linear', circular = TRUE) +
  geom_edge_arc0(aes(colour = class))
}
\seealso{
Other geom_edge_*: 
\code{\link{geom_edge_bend}()},
\code{\link{geom_edge_bundle_force}()},
\code{\link{geom_edge_bundle_minimal}()},
\code{\link{geom_edge_bundle_path}()},
\code{\link{geom_edge_density}()},
\code{\link{geom_edge_diagonal}()},
\code{\link{geom_edge_elbow}()},
\code{\link{geom_edge_fan}()},
\code{\link{geom_edge_hive}()},
\code{\link{geom_edge_link}()},
\code{\link{geom_edge_loop}()},
\code{\link{geom_edge_parallel}()},
\code{\link{geom_edge_point}()},
\code{\link{geom_edge_sf}()},
\code{\link{geom_edge_span}()},
\code{\link{geom_edge_tile}()}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_metro.R
\name{layout_tbl_graph_metro}
\alias{layout_tbl_graph_metro}
\title{Place nodes according to the standard design of metro maps}
\usage{
layout_tbl_graph_metro(
  graph,
  x,
  y,
  length = 2,
  grid_space = 0.0025,
  optimization_weights = NULL,
  max_movement = 5,
  circular = FALSE
)
}
\arguments{
\item{graph}{A tbl_graph object}

\item{x, y}{The start location of the nodes}

\item{length}{Desired multiple of grid point spacing. (\code{length * grid_space}
determines desired edge length)}

\item{grid_space}{The distance between consecitive grid points}

\item{optimization_weights}{The relative weight to be placed on the 5
criteria during optimization as a numeric vector of length 4. The criteria
are:
\itemize{
\item \verb{Angular Resolution Criterion}: The angles of incident edges at each
station should be maximized, because if there is only a small angle between
any two adjacent edges, then it can become difficult to distinguish between
them.
\item \verb{Edge Length Criterion}: The edge lengths across the whole map should be
approximately equal to ensure regular spacing between stations. It is based
on the preferred multiple, l, of the grid spacing, g. The purpose of the
criterion is to penalize edges that are longer than or shorter than lg.
\item \verb{Balanced Edge Length Criterion}: The length of edges incident to a
particular station should be similar.
\item \verb{Line Straightness Criterion}: (not yet implemented) Edges that form part
of a line should, where possible, be co-linear either side of each station
that the line passes through.
\item \verb{Octiinearity Criterion}: Each edge should be drawn horizontally,
vertically, or diagonally at 45 degree, so we penalize edges that are not
at a desired angle.
If \code{NULL} all criteria are given equal weight.
}}

\item{max_movement}{Number of grid points a station can move away rom its
original position}

\item{circular}{ignored}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
}
\description{
This layouttries to optimise the placement of nodes so that they adhere to
the classic look of metro maps. As such it optimizes on the distribution of
incident edge angles, conformity of edge length, and edge angles in multiples
of 45 degrees. As it works as a refinement of an already existing layout
(the physical location of metro stations) it requires an a priori node
location. Due to it's purpose it probably works best with planar graphs.
}
\references{
Stott, J., Rodgers, P., Martinez-Ovando, J. C., and Walker, S. G. (2011).
\emph{Automatic metro map layout using multicriteria optimization} In IEEE Trans
Vis Comput Graph 17(1) pp. 101-114. https://doi.org/10.1109/tvcg.2010.24
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_cactustree}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_htree}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_sf}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\author{
The underlying algorithm is implemented in the graphlayouts package
by David Schoch
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edge_colourbar.R
\name{guide_edge_coloursteps}
\alias{guide_edge_coloursteps}
\alias{guide_edge_colorsteps}
\title{Coloursteps legend for edges}
\usage{
guide_edge_coloursteps(
  even.steps = TRUE,
  show.limits = NULL,
  ...,
  available_aes = c("edge_colour", "edge_fill")
)

guide_edge_colorsteps(
  even.steps = TRUE,
  show.limits = NULL,
  ...,
  available_aes = c("edge_colour", "edge_fill")
)
}
\arguments{
\item{even.steps}{Should the rendered size of the bins be equal, or should
they be proportional to their length in the data space? Defaults to \code{TRUE}}

\item{show.limits}{Logical. Should the limits of the scale be shown with
labels and ticks. Default is \code{NULL} meaning it will take the value from the
scale. This argument is ignored if \code{labels} is given as a vector of
values. If one or both of the limits is also given in \code{breaks} it will be
shown irrespective of the value of \code{show.limits}.}

\item{...}{
  Arguments passed on to \code{\link[ggplot2:guide_colourbar]{ggplot2::guide_colourbar}}
  \describe{
    \item{\code{title}}{A character string or expression indicating a title of guide.
If \code{NULL}, the title is not shown. By default
(\code{\link[ggplot2:waiver]{waiver()}}), the name of the scale object or the name
specified in \code{\link[ggplot2:labs]{labs()}} is used for the title.}
    \item{\code{theme}}{A \code{\link[ggplot2:theme]{theme}} object to style the guide individually or
differently from the plot's theme settings. The \code{theme} argument in the
guide overrides, and is combined with, the plot's theme.}
    \item{\code{nbin}}{A numeric specifying the number of bins for drawing the
colourbar. A smoother colourbar results from a larger value.}
    \item{\code{display}}{A string indicating a method to display the colourbar. Can be
one of the following:
\itemize{
\item \code{"raster"} to display as a bitmap image.
\item \code{"rectangles"} to display as a series of rectangles.
\item \code{"gradient"} to display as a linear gradient.
}

Note that not all devices are able to render rasters and gradients.}
    \item{\code{raster}}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}}  A logical. If \code{TRUE} then
the colourbar is rendered as a raster object. If \code{FALSE} then the colourbar
is rendered as a set of rectangles. Note that not all graphics devices are
capable of rendering raster image.}
    \item{\code{alpha}}{A numeric between 0 and 1 setting the colour transparency of
the bar. Use \code{NA} to preserve the alpha encoded in the colour itself
(default).}
    \item{\code{draw.ulim}}{A logical specifying if the upper limit tick marks should
be visible.}
    \item{\code{draw.llim}}{A logical specifying if the lower limit tick marks should
be visible.}
    \item{\code{position}}{A character string indicating where the legend should be
placed relative to the plot panels.}
    \item{\code{direction}}{A character string indicating the direction of the guide.
One of "horizontal" or "vertical."}
    \item{\code{reverse}}{logical. If \code{TRUE} the colourbar is reversed. By default,
the highest value is on the top and the lowest value is on the bottom}
    \item{\code{order}}{positive integer less than 99 that specifies the order of
this guide among multiple guides. This controls the order in which
multiple guides are displayed, not the contents of the guide itself.
If 0 (default), the order is determined by a secret algorithm.}
  }}

\item{available_aes}{A vector of character strings listing the aesthetics
for which a colourbar can be drawn.}
}
\value{
A guide object
}
\description{
This function is equivalent to \code{\link[ggplot2:guide_coloursteps]{ggplot2::guide_coloursteps()}} but
works for edge aesthetics.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_highschool.R
\docType{data}
\name{highschool}
\alias{highschool}
\title{Friendship among high school boys}
\format{
The graph is stored as an unnamed edgelist with a year attribute.
\describe{
\item{from}{The boy answering the question}
\item{to}{The boy being the answer to the question}
\item{year}{The year the friendship was reported}
}
}
\source{
Coleman, J. S. \emph{Introduction to Mathematical Sociology}. New York: Free
Press, pp.450-451.
}
\usage{
highschool
}
\description{
This dataset shows the friendship among high school boys as assessed by the
question: "What fellows here in school do you go around with most often?".
The question was posed twice, with one year in between (1957 and 1958) and
shows the evolution in friendship between the two timepoints.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_igraph.R
\name{layout_tbl_graph_igraph}
\alias{layout_tbl_graph_igraph}
\title{Use igraph layout algorithms for layout_tbl_graph}
\usage{
layout_tbl_graph_igraph(
  graph,
  algorithm,
  circular,
  offset = pi/2,
  use.dummy = FALSE,
  ...
)
}
\arguments{
\item{graph}{A \code{tbl_graph} object.}

\item{algorithm}{The type of layout algorithm to apply. See \emph{Details} or
\code{\link[igraph:layout_]{igraph::layout_()}} for links to the layouts supplied by igraph.}

\item{circular}{Logical. Should the layout be transformed to a circular
representation. Defaults to \code{FALSE}. Only applicable to
\code{algorithm = 'tree'} and \code{algorithm = 'sugiyama'}.}

\item{offset}{If \code{circular = TRUE}, where should it begin. Defaults to
\code{pi/2} which is equivalent to 12 o'clock.}

\item{use.dummy}{Logical. In the case of \code{algorithm = 'sugiyama'} should the
dummy-infused graph be used rather than the original. Defaults to
\code{FALSE}.}

\item{...}{Arguments passed on to the respective layout functions}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
}
\description{
This layout function makes it easy to apply one of the layout algorithms
supplied in igraph when plotting with ggraph. Layout names are auto completed
so there is no need to write \code{layout_with_graphopt} or
\code{layout_as_tree}, just \code{graphopt} and \code{tree} (though the
former will also work if you want to be super explicit). Circular layout is
only supported for tree-like layout (\code{tree} and \code{sugiyama}) and
will throw an error when applied to other layouts.
}
\details{
igraph provides a huge amount of possible layouts. They are all briefly
described below:

\strong{Hierarchical layouts}

\describe{
\item{\code{tree}}{Uses the \emph{Reingold-Tilford} algorithm to place the
nodes below their parent with the parent centered above its children. See
\code{\link[igraph:layout_as_tree]{igraph::as_tree()}}}
\item{\code{sugiyama}}{Designed for directed acyclic graphs (that is,
hierarchies where multiple parents are allowed) it minimizes the number of
crossing edges. See \code{\link[igraph:layout_with_sugiyama]{igraph::with_sugiyama()}}}
}

\strong{Standard layouts}

\describe{
\item{\code{bipartite}}{Minimize edge-crossings in a simple two-row (or
column) layout for bipartite graphs. See \code{\link[igraph:layout_as_bipartite]{igraph::as_bipartite()}}}
\item{\code{star}}{Place one node in the center and the rest equidistantly
around it. See \code{\link[igraph:layout_as_star]{igraph::as_star()}}}
\item{\code{circle}}{Place nodes in a circle in the order of their index.
Consider using \code{\link[=layout_tbl_graph_linear]{layout_tbl_graph_linear()}} with \code{circular=TRUE}
for more control. See \code{\link[igraph:layout_in_circle]{igraph::in_circle()}}}
\item{\code{nicely}}{Tries to pick an appropriate layout. See
\code{\link[igraph:layout_nicely]{igraph::nicely()}} for a description of the simple decision tree
it uses}
\item{\code{dh}}{Uses \emph{Davidson and Harels} simulated annealing
algorithm to place nodes. See \code{\link[igraph:layout_with_dh]{igraph::with_dh()}}}
\item{\code{gem}}{Place nodes on the plane using the GEM force-directed
layout algorithm. See \code{\link[igraph:layout_with_gem]{igraph::with_gem()}}}
\item{\code{graphopt}}{Uses the Graphopt algorithm based on alternating
attraction and repulsion to place nodes. See
\code{\link[igraph:layout_with_graphopt]{igraph::with_graphopt()}}}
\item{\code{grid}}{Place nodes on a rectangular grid. See
\code{\link[igraph:layout_on_grid]{igraph::on_grid()}}}
\item{\code{mds}}{Perform a multidimensional scaling of nodes using either
the shortest path or a user supplied distance. See
\code{\link[igraph:layout_with_mds]{igraph::with_mds()}}}
\item{\code{sphere}}{Place nodes uniformly on a sphere - less relevant for
2D visualizations of networks. See \code{\link[igraph:layout_on_sphere]{igraph::on_sphere()}}}
\item{\code{randomly}}{Places nodes uniformly random. See
\code{\link[igraph:layout_randomly]{igraph::randomly()}}}
\item{\code{fr}}{Places nodes according to the force-directed algorithm of
Fruchterman and Reingold. See \code{\link[igraph:layout_with_fr]{igraph::with_fr()}}}
\item{\code{kk}}{Uses the spring-based algorithm by Kamada and Kawai to
place nodes. See \code{\link[igraph:layout_with_kk]{igraph::with_kk()}}}
\item{\code{drl}}{Uses the force directed algorithm from the DrL toolbox to
place nodes. See \code{\link[igraph:layout_with_drl]{igraph::with_drl()}}}
\item{\code{lgl}}{Uses the algorithm from Large Graph Layout to place
nodes. See \code{\link[igraph:layout_with_lgl]{igraph::with_lgl()}}}
}
}
\note{
This function is not intended to be used directly but by setting
\code{layout = 'igraph'} in \code{\link[=create_layout]{create_layout()}}
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cappedPath.R
\name{makeContent.cappedpathgrob}
\alias{makeContent.cappedpathgrob}
\title{Dynamic capping of paths}
\usage{
\method{makeContent}{cappedpathgrob}(x)
}
\description{
This function takes care of updating which parts of the paths get removed
when the device dimensions are updated.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_auto.R
\name{layout_tbl_graph_auto}
\alias{layout_tbl_graph_auto}
\title{Automatically pick a layout based on graph type}
\usage{
layout_tbl_graph_auto(graph, circular, ...)
}
\arguments{
\item{graph}{A tbl_graph object}

\item{circular}{Logical. Should the layout be transformed to a circular
representation. Defaults to \code{FALSE}. Only applicable if the graph is a tree
structure}

\item{...}{Arguments passed on to the chosen layout}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
}
\description{
This function infers the layout from the graph structure and is the default
when calling \code{\link[=ggraph]{ggraph()}}. If an \code{x} and \code{y} argument is passed along, the
manual layout is chosen. Otherwise if the graph is either a rooted tree or a
rooted forest the layout will be \code{dendrogram} if the nodes contains a height
variable or \code{tree} if not. If the tree is unrooted the \code{unrooted} layout will
be used. If the tree is a DAG the \code{sygiyama} layout will be used. Otherwise
the \code{stress} layout will be used (or \code{sparse_tree} if the graph contains more
than 2000 nodes).
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa.R, R/facet_edges.R, R/facet_graph.R,
%   R/facet_nodes.R, R/geom_axis_hive.R, R/geom_conn_bundle.R, R/geom_edge.R,
%   R/geom_edge_arc.R, R/geom_edge_bend.R, R/geom_edge_density.R,
%   R/geom_edge_diagonal.R, R/geom_edge_elbow.R, R/geom_edge_fan.R,
%   R/geom_edge_hive.R, R/geom_edge_link.R, R/geom_edge_loop.R,
%   R/geom_edge_parallel.R, R/geom_node_arc_bar.R, R/geom_node_circle.R,
%   R/geom_node_tile.R, R/geom_node_voronoi.R, R/ggproto-classes.R
\docType{data}
\name{StatFilter}
\alias{StatFilter}
\alias{FacetEdges}
\alias{FacetGraph}
\alias{FacetNodes}
\alias{StatAxisHive}
\alias{GeomAxisHive}
\alias{StatConnBundle}
\alias{StatConnBundle2}
\alias{StatConnBundle0}
\alias{GeomEdgePath}
\alias{GeomEdgeParallelPath}
\alias{GeomEdgeSegment}
\alias{GeomEdgeParallelSegment}
\alias{GeomEdgeSpanPath}
\alias{GeomEdgeSpanSegment}
\alias{GeomEdgePoint}
\alias{GeomEdgeTile}
\alias{GeomEdgeBezier}
\alias{GeomEdgeBspline}
\alias{StatEdgeArc}
\alias{StatEdgeArc2}
\alias{StatEdgeArc0}
\alias{StatEdgeBend}
\alias{StatEdgeBend2}
\alias{StatEdgeBend0}
\alias{StatEdgeDensity}
\alias{GeomEdgeDensity}
\alias{StatEdgeDiagonal}
\alias{StatEdgeDiagonal2}
\alias{StatEdgeDiagonal0}
\alias{StatEdgeElbow}
\alias{StatEdgeElbow2}
\alias{StatEdgeElbow0}
\alias{StatEdgeFan}
\alias{StatEdgeFan2}
\alias{StatEdgeFan0}
\alias{StatEdgeHive}
\alias{StatEdgeHive2}
\alias{StatEdgeHive0}
\alias{StatEdgeLink}
\alias{StatEdgeLink2}
\alias{StatEdgeLoop}
\alias{StatEdgeLoop0}
\alias{StatEdgeParallel}
\alias{StatEdgeParallel2}
\alias{StatEdgeParallel0}
\alias{StatNodeArcBar}
\alias{StatNodeCircle}
\alias{GeomNodeTile}
\alias{StatNodeVoronoi}
\alias{ggraph-extensions}
\title{ggraph extensions to ggplot2}
\description{
This help page lists all exported ggproto classes defined by ggraph. In
general these should be of no concern to the user as the main interface to
the functionality is, as with ggplot2, the \verb{geom_*} format. As opposed
to ggplot2 there really aren't any use for separate \verb{stat_*} functions
as they are intimately linked to each geom and mixing and matching stats and
geoms would only cause a lot of trouble.
}
\details{
Many of the \verb{geom_edge_*} geoms comes in different flavors dependent on
the functionality required. There will always be a base geom and some will
have a \code{geom_edge_*0} and \code{geom_edge_*2} version. The base geom
will, in the case of multiple versions, draw the edge as a sequence of small
segments. The different aesthetics will be repeated for each segment and a
counter will be added the enumerates the progression of segments, so that a
gradient of colour or size can be added along the edge by assigning the
respective aesthetic to \code{stat(index)}. \code{geom_edge_*2} will also draw
the edge as segments but will interpolate between the aesthetics at the end
points. This makes it possible for an edge to interpolate node properties of
its end nodes. \code{geom_edge_*2} is less performant than the base geom so
use only when interpolation is needed. \code{geom_edge_*0} is a
high-performance version that usually maps directly to a grid grob. It does
not allow for drawing gradients or interpolations though, so use only for
simple edge drawings.
}
\keyword{datasets}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_igraph.R
\name{layout_tbl_graph_igraph}
\alias{layout_tbl_graph_igraph}
\title{Use igraph layout algorithms for layout_tbl_graph}
\usage{
layout_tbl_graph_igraph(
  graph,
  algorithm,
  circular,
  offset = pi/2,
  use.dummy = FALSE,
  ...
)
}
\arguments{
\item{graph}{A \code{tbl_graph} object.}

\item{algorithm}{The type of layout algorithm to apply. See \emph{Details} or
\code{\link[igraph:layout_]{igraph::layout_()}} for links to the layouts supplied by igraph.}

\item{circular}{Logical. Should the layout be transformed to a circular
representation. Defaults to \code{FALSE}. Only applicable to
\code{algorithm = 'tree'} and \code{algorithm = 'sugiyama'}.}

\item{offset}{If \code{circular = TRUE}, where should it begin. Defaults to
\code{pi/2} which is equivalent to 12 o'clock.}

\item{use.dummy}{Logical. In the case of \code{algorithm = 'sugiyama'} should the
dummy-infused graph be used rather than the original. Defaults to
\code{FALSE}.}

\item{...}{Arguments passed on to the respective layout functions}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
}
\description{
This layout function makes it easy to apply one of the layout algorithms
supplied in igraph when plotting with ggraph. Layout names are auto completed
so there is no need to write \code{layout_with_graphopt} or
\code{layout_as_tree}, just \code{graphopt} and \code{tree} (though the
former will also work if you want to be super explicit). Circular layout is
only supported for tree-like layout (\code{tree} and \code{sugiyama}) and
will throw an error when applied to other layouts.
}
\details{
igraph provides a huge amount of possible layouts. They are all briefly
described below:

\strong{Hierarchical layouts}

\describe{
\item{\code{tree}}{Uses the \emph{Reingold-Tilford} algorithm to place the
nodes below their parent with the parent centered above its children. See
\code{\link[igraph:layout_as_tree]{igraph::as_tree()}}}
\item{\code{sugiyama}}{Designed for directed acyclic graphs (that is,
hierarchies where multiple parents are allowed) it minimizes the number of
crossing edges. See \code{\link[igraph:layout_with_sugiyama]{igraph::with_sugiyama()}}}
}

\strong{Standard layouts}

\describe{
\item{\code{bipartite}}{Minimize edge-crossings in a simple two-row (or
column) layout for bipartite graphs. See \code{\link[igraph:layout_as_bipartite]{igraph::as_bipartite()}}}
\item{\code{star}}{Place one node in the center and the rest equidistantly
around it. See \code{\link[igraph:layout_as_star]{igraph::as_star()}}}
\item{\code{circle}}{Place nodes in a circle in the order of their index.
Consider using \code{\link[=layout_tbl_graph_linear]{layout_tbl_graph_linear()}} with \code{circular=TRUE}
for more control. See \code{\link[igraph:layout_in_circle]{igraph::in_circle()}}}
\item{\code{nicely}}{Tries to pick an appropriate layout. See
\code{\link[igraph:layout_nicely]{igraph::nicely()}} for a description of the simple decision tree
it uses}
\item{\code{dh}}{Uses \emph{Davidson and Harels} simulated annealing
algorithm to place nodes. See \code{\link[igraph:layout_with_dh]{igraph::with_dh()}}}
\item{\code{gem}}{Place nodes on the plane using the GEM force-directed
layout algorithm. See \code{\link[igraph:layout_with_gem]{igraph::with_gem()}}}
\item{\code{graphopt}}{Uses the Graphopt algorithm based on alternating
attraction and repulsion to place nodes. See
\code{\link[igraph:layout_with_graphopt]{igraph::with_graphopt()}}}
\item{\code{grid}}{Place nodes on a rectangular grid. See
\code{\link[igraph:layout_on_grid]{igraph::on_grid()}}}
\item{\code{mds}}{Perform a multidimensional scaling of nodes using either
the shortest path or a user supplied distance. See
\code{\link[igraph:layout_with_mds]{igraph::with_mds()}}}
\item{\code{sphere}}{Place nodes uniformly on a sphere - less relevant for
2D visualizations of networks. See \code{\link[igraph:layout_on_sphere]{igraph::on_sphere()}}}
\item{\code{randomly}}{Places nodes uniformly random. See
\code{\link[igraph:layout_randomly]{igraph::randomly()}}}
\item{\code{fr}}{Places nodes according to the force-directed algorithm of
Fruchterman and Reingold. See \code{\link[igraph:layout_with_fr]{igraph::with_fr()}}}
\item{\code{kk}}{Uses the spring-based algorithm by Kamada and Kawai to
place nodes. See \code{\link[igraph:layout_with_kk]{igraph::with_kk()}}}
\item{\code{drl}}{Uses the force directed algorithm from the DrL toolbox to
place nodes. See \code{\link[igraph:layout_with_drl]{igraph::with_drl()}}}
\item{\code{lgl}}{Uses the algorithm from Large Graph Layout to place
nodes. See \code{\link[igraph:layout_with_lgl]{igraph::with_lgl()}}}
}
}
\note{
This function is not intended to be used directly but by setting
\code{layout = 'igraph'} in \code{\link[=create_layout]{create_layout()}}
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_cactustree}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_htree}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_metro}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_sf}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_auto.R
\name{layout_tbl_graph_auto}
\alias{layout_tbl_graph_auto}
\title{Automatically pick a layout based on graph type}
\usage{
layout_tbl_graph_auto(graph, circular, ...)
}
\arguments{
\item{graph}{A tbl_graph object}

\item{circular}{Logical. Should the layout be transformed to a circular
representation. Defaults to \code{FALSE}. Only applicable if the graph is a tree
structure}

\item{...}{Arguments passed on to the chosen layout}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
}
\description{
This function infers the layout from the graph structure and is the default
when calling \code{\link[=ggraph]{ggraph()}}. If an \code{x} and \code{y} argument is passed along, the
manual layout is chosen. Otherwise if the graph is either a rooted tree or a
rooted forest the layout will be \code{dendrogram} if the nodes contains a height
variable or \code{tree} if not. If the tree is unrooted the \code{unrooted} layout will
be used. If the tree is a DAG the \code{sygiyama} layout will be used. Otherwise
the \code{stress} layout will be used (or \code{sparse_tree} if the graph contains more
than 2000 nodes).
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_cactustree}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_htree}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_metro}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_sf}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aaa.R, R/facet_edges.R, R/facet_graph.R,
%   R/facet_nodes.R, R/geom_axis_hive.R, R/geom_conn_bundle.R, R/geom_edge.R,
%   R/geom_edge_arc.R, R/geom_edge_bend.R, R/geom_edge_bundle_force.R,
%   R/geom_edge_bundle_minimal.R, R/geom_edge_bundle_path.R,
%   R/geom_edge_density.R, R/geom_edge_diagonal.R, R/geom_edge_elbow.R,
%   R/geom_edge_fan.R, R/geom_edge_hive.R, R/geom_edge_link.R,
%   R/geom_edge_loop.R, R/geom_edge_parallel.R, R/geom_edge_sf.R,
%   R/geom_node_arc_bar.R, R/geom_node_circle.R, R/geom_node_tile.R,
%   R/geom_node_voronoi.R, R/ggproto-classes.R
\docType{data}
\name{StatFilter}
\alias{StatFilter}
\alias{StatReverse}
\alias{StatFilterSf}
\alias{FacetEdges}
\alias{FacetGraph}
\alias{FacetNodes}
\alias{StatAxisHive}
\alias{GeomAxisHive}
\alias{StatConnBundle}
\alias{StatConnBundle2}
\alias{StatConnBundle0}
\alias{GeomEdgePath}
\alias{GeomEdgeParallelPath}
\alias{GeomEdgeSegment}
\alias{GeomEdgeParallelSegment}
\alias{GeomEdgeSpanPath}
\alias{GeomEdgeSpanSegment}
\alias{GeomEdgePoint}
\alias{GeomEdgeTile}
\alias{GeomEdgeBezier}
\alias{GeomEdgeBspline}
\alias{StatEdgeArc}
\alias{StatEdgeArc2}
\alias{StatEdgeArc0}
\alias{StatEdgeBend}
\alias{StatEdgeBend2}
\alias{StatEdgeBend0}
\alias{StatEdgeBundleForce}
\alias{StatEdgeBundleForce2}
\alias{StatEdgeBundleForce0}
\alias{StatEdgeBundleMinimal}
\alias{StatEdgeBundleMinimal2}
\alias{StatEdgeBundleMinimal0}
\alias{StatEdgeBundlePath}
\alias{StatEdgeBundlePath2}
\alias{StatEdgeBundlePath0}
\alias{StatEdgeDensity}
\alias{GeomEdgeDensity}
\alias{StatEdgeDiagonal}
\alias{StatEdgeDiagonal2}
\alias{StatEdgeDiagonal0}
\alias{StatEdgeElbow}
\alias{StatEdgeElbow2}
\alias{StatEdgeElbow0}
\alias{StatEdgeFan}
\alias{StatEdgeFan2}
\alias{StatEdgeFan0}
\alias{StatEdgeHive}
\alias{StatEdgeHive2}
\alias{StatEdgeHive0}
\alias{StatEdgeLink}
\alias{StatEdgeLink2}
\alias{StatEdgeLoop}
\alias{StatEdgeLoop0}
\alias{StatEdgeParallel}
\alias{StatEdgeParallel2}
\alias{StatEdgeParallel0}
\alias{GeomEdgeSf}
\alias{StatNodeArcBar}
\alias{StatNodeCircle}
\alias{GeomNodeTile}
\alias{StatNodeVoronoi}
\alias{ggraph-extensions}
\title{ggraph extensions to ggplot2}
\description{
This help page lists all exported ggproto classes defined by ggraph. In
general these should be of no concern to the user as the main interface to
the functionality is, as with ggplot2, the \verb{geom_*} format. As opposed
to ggplot2 there really aren't any use for separate \verb{stat_*} functions
as they are intimately linked to each geom and mixing and matching stats and
geoms would only cause a lot of trouble.
}
\details{
Many of the \verb{geom_edge_*} geoms comes in different flavors dependent on
the functionality required. There will always be a base geom and some will
have a \code{geom_edge_*0} and \code{geom_edge_*2} version. The base geom
will, in the case of multiple versions, draw the edge as a sequence of small
segments. The different aesthetics will be repeated for each segment and a
counter will be added the enumerates the progression of segments, so that a
gradient of colour or size can be added along the edge by assigning the
respective aesthetic to \code{after_stat(index)}. \code{geom_edge_*2} will also draw
the edge as segments but will interpolate between the aesthetics at the end
points. This makes it possible for an edge to interpolate node properties of
its end nodes. \code{geom_edge_*2} is less performant than the base geom so
use only when interpolation is needed. \code{geom_edge_*0} is a
high-performance version that usually maps directly to a grid grob. It does
not allow for drawing gradients or interpolations though, so use only for
simple edge drawings.
}
\keyword{datasets}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/autograph.R
\name{qgraph}
\alias{qgraph}
\title{Deprecated autograph predecessor}
\usage{
qgraph(...)
}
\description{
Deprecated autograph predecessor
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/facet_graph.R
\name{facet_graph}
\alias{facet_graph}
\title{Create a grid of small multiples by node and/or edge attributes}
\usage{
facet_graph(
  facets,
  row_type = "edge",
  col_type = "node",
  margins = FALSE,
  scales = "fixed",
  space = "fixed",
  shrink = TRUE,
  labeller = "label_value",
  as.table = TRUE,
  switch = NULL,
  drop = TRUE
)
}
\arguments{
\item{facets}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} Please use \code{rows}
and \code{cols} instead.}

\item{row_type, col_type}{Either \code{'node'} or \code{'edge'}. Which data
type is being facetted in the rows and columns. Default is to facet on nodes
column wise and on edges row wise.}

\item{margins}{Either a logical value or a character
vector. Margins are additional facets which contain all the data
for each of the possible values of the faceting variables. If
\code{FALSE}, no additional facets are included (the
default). If \code{TRUE}, margins are included for all faceting
variables. If specified as a character vector, it is the names of
variables for which margins are to be created.}

\item{scales}{Are scales shared across all facets (the default,
\code{"fixed"}), or do they vary across rows (\code{"free_x"}),
columns (\code{"free_y"}), or both rows and columns (\code{"free"})?}

\item{space}{If \code{"fixed"}, the default, all panels have the same size.
If \code{"free_y"} their height will be proportional to the length of the
y scale; if \code{"free_x"} their width will be proportional to the
length of the x scale; or if \code{"free"} both height and width will
vary.  This setting has no effect unless the appropriate scales also vary.}

\item{shrink}{If \code{TRUE}, will shrink scales to fit output of
statistics, not raw data. If \code{FALSE}, will be range of raw data
before statistical summary.}

\item{labeller}{A function that takes one data frame of labels and
returns a list or data frame of character vectors. Each input
column corresponds to one factor. Thus there will be more than
one with \code{vars(cyl, am)}. Each output
column gets displayed as one separate line in the strip
label. This function should inherit from the "labeller" S3 class
for compatibility with \code{\link[ggplot2:labeller]{labeller()}}. You can use different labeling
functions for different kind of labels, for example use \code{\link[ggplot2:label_parsed]{label_parsed()}} for
formatting facet labels. \code{\link[ggplot2:label_value]{label_value()}} is used by default,
check it for more details and pointers to other options.}

\item{as.table}{If \code{TRUE}, the default, the facets are laid out like
a table with highest values at the bottom-right. If \code{FALSE}, the
facets are laid out like a plot with the highest value at the top-right.}

\item{switch}{By default, the labels are displayed on the top and
right of the plot. If \code{"x"}, the top labels will be
displayed to the bottom. If \code{"y"}, the right-hand side
labels will be displayed to the left. Can also be set to
\code{"both"}.}

\item{drop}{If \code{TRUE}, the default, all factor levels not used in the
data will automatically be dropped. If \code{FALSE}, all factor levels
will be shown, regardless of whether or not they appear in the data.}
}
\description{
This function is equivalent to \code{\link[ggplot2:facet_grid]{ggplot2::facet_grid()}} in that it
allows for building a grid of small multiples where rows and columns
correspond to a specific data value. While \code{\link[ggplot2:facet_grid]{ggplot2::facet_grid()}}
could be used it would lead to unexpected results as it is not possible to
specify whether you are referring to a node or an edge attribute. Furthermore
\code{\link[ggplot2:facet_grid]{ggplot2::facet_grid()}} will draw edges in panels even though the
panel does not contain both terminal nodes. \code{facet_graph} takes care of
all of these issues, allowing you to define which data type the rows and
columns are referencing as well as filtering the edges based on the nodes in
each panel (even when nodes are not drawn).
}
\examples{
library(tidygraph)
gr <- as_tbl_graph(highschool) \%>\%
  mutate(popularity = as.character(cut(centrality_degree(mode = 'in'),
    breaks = 3,
    labels = c('low', 'medium', 'high')
  )))
ggraph(gr) +
  geom_edge_link() +
  geom_node_point() +
  facet_graph(year ~ popularity)
}
\seealso{
Other ggraph-facets: 
\code{\link{facet_edges}()},
\code{\link{facet_nodes}()}
}
\concept{ggraph-facets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_edge_shape.R
\name{scale_edge_shape}
\alias{scale_edge_shape}
\alias{scale_edge_shape_discrete}
\alias{scale_edge_shape_continuous}
\alias{scale_edge_shape_binned}
\alias{scale_edge_shape_manual}
\alias{scale_edge_shape_identity}
\title{Edge shape scales}
\usage{
scale_edge_shape(..., solid = TRUE)

scale_edge_shape_discrete(..., solid = TRUE)

scale_edge_shape_continuous(...)

scale_edge_shape_binned(..., solid = TRUE)

scale_edge_shape_manual(..., values, breaks = waiver(), na.value = NA)

scale_edge_shape_identity(..., guide = "none")
}
\arguments{
\item{...}{
  Arguments passed on to \code{\link[ggplot2:discrete_scale]{discrete_scale}}
  \describe{
    \item{\code{palette}}{A palette function that when called with a single integer
argument (the number of levels in the scale) returns the values that
they should take (e.g., \code{\link[scales:pal_hue]{scales::pal_hue()}}).}
    \item{\code{breaks}}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{waiver()} for the default breaks (the scale limits)
\item A character vector of breaks
\item A function that takes the limits as input and returns breaks
as output. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}
    \item{\code{limits}}{One of:
\itemize{
\item \code{NULL} to use the default scale values
\item A character vector that defines possible values of the scale and their
order
\item A function that accepts the existing (automatic) values and returns
new ones. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}
    \item{\code{drop}}{Should unused factor levels be omitted from the scale?
The default, \code{TRUE}, uses the levels that appear in the data;
\code{FALSE} uses all the levels in the factor.}
    \item{\code{na.translate}}{Unlike continuous scales, discrete scales can easily show
missing values, and do so by default. If you want to remove missing values
from a discrete scale, specify \code{na.translate = FALSE}.}
    \item{\code{na.value}}{If \code{na.translate = TRUE}, what aesthetic value should the
missing values be displayed as? Does not apply to position scales
where \code{NA} is always placed at the far right.}
    \item{\code{aesthetics}}{The names of the aesthetics that this scale works with.}
    \item{\code{scale_name}}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} The name of the scale
that should be used for error messages associated with this scale.}
    \item{\code{labels}}{One of:
\itemize{
\item \code{NULL} for no labels
\item \code{waiver()} for the default labels computed by the
transformation object
\item A character vector giving labels (must be same length as \code{breaks})
\item An expression vector (must be the same length as breaks). See ?plotmath for details.
\item A function that takes the breaks as input and returns labels
as output. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}
    \item{\code{guide}}{A function used to create a guide or its name. See
\code{\link[ggplot2:guides]{guides()}} for more information.}
    \item{\code{call}}{The \code{call} used to construct the scale for reporting messages.}
    \item{\code{super}}{The super class to use for the constructed scale}
  }}

\item{solid}{Should the shapes be solid, \code{TRUE}, or hollow,
\code{FALSE}?}

\item{values}{a set of aesthetic values to map data values to. The values
will be matched in order (usually alphabetical) with the limits of the
scale, or with \code{breaks} if provided. If this is a named vector, then the
values will be matched based on the names instead. Data values that don't
match will be given \code{na.value}.}

\item{breaks}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{waiver()} for the default breaks (the scale limits)
\item A character vector of breaks
\item A function that takes the limits as input and returns breaks
as output
}}

\item{na.value}{The aesthetic value to use for missing (\code{NA}) values}

\item{guide}{Guide to use for this scale.}
}
\value{
A ggproto object inheriting from \code{Scale}
}
\description{
This set of scales defines new shape scales for edge geoms equivalent to the
ones already defined by ggplot2. See \code{\link[ggplot2:scale_shape]{ggplot2::scale_shape()}} for
more information. The different geoms will know whether to use edge scales or
the standard scales so it is not necessary to write \code{edge_shape} in
the call to the geom - just use \code{shape}.
}
\seealso{
Other scale_edge_*: 
\code{\link{scale_edge_alpha}()},
\code{\link{scale_edge_colour}},
\code{\link{scale_edge_fill}},
\code{\link{scale_edge_linetype}()},
\code{\link{scale_edge_size}()},
\code{\link{scale_edge_width}()},
\code{\link{scale_label_size}()}
}
\concept{scale_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_edge_parallel.R
\name{geom_edge_parallel}
\alias{geom_edge_parallel}
\alias{geom_edge_parallel2}
\alias{geom_edge_parallel0}
\title{Draw multi edges as parallel lines}
\usage{
geom_edge_parallel(
  mapping = NULL,
  data = get_edges(),
  position = "identity",
  arrow = NULL,
  sep = unit(2, "mm"),
  n = 100,
  lineend = "butt",
  linejoin = "round",
  linemitre = 1,
  label_colour = "black",
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = "rot",
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
)

geom_edge_parallel2(
  mapping = NULL,
  data = get_edges("long"),
  position = "identity",
  arrow = NULL,
  sep = unit(2, "mm"),
  n = 100,
  lineend = "butt",
  linejoin = "round",
  linemitre = 1,
  label_colour = "black",
  label_alpha = 1,
  label_parse = FALSE,
  check_overlap = FALSE,
  angle_calc = "rot",
  force_flip = TRUE,
  label_dodge = NULL,
  label_push = NULL,
  show.legend = NA,
  ...
)

geom_edge_parallel0(
  mapping = NULL,
  data = get_edges(),
  position = "identity",
  arrow = NULL,
  sep = unit(2, "mm"),
  lineend = "butt",
  show.legend = NA,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x, y, xend, yend, group and
circular are mapped to x, y, xend, yend, edge.id and circular in the edge
data.}

\item{data}{The return of a call to \code{get_edges()} or a data.frame
giving edges in correct format (see details for for guidance on the format).
See \code{\link[=get_edges]{get_edges()}} for more details on edge extraction.}

\item{position}{Position adjustment, either as a string, or the result of
a call to a position adjustment function.}

\item{arrow}{Arrow specification, as created by \code{\link[grid:arrow]{grid::arrow()}}.}

\item{sep}{The separation between parallel edges, given as a \code{\link[grid:unit]{grid::unit()}}}

\item{n}{The number of points to create along the path.}

\item{lineend}{Line end style (round, butt, square).}

\item{linejoin}{Line join style (round, mitre, bevel).}

\item{linemitre}{Line mitre limit (number greater than 1).}

\item{label_colour}{The colour of the edge label. If \code{NA} it will use
the colour of the edge.}

\item{label_alpha}{The opacity of the edge label. If \code{NA} it will use
the opacity of the edge.}

\item{label_parse}{If \code{TRUE}, the labels will be parsed into expressions
and displayed as described in \code{\link[grDevices:plotmath]{grDevices::plotmath()}}.}

\item{check_overlap}{If \code{TRUE}, text that overlaps previous text in the
same layer will not be plotted. \code{check_overlap} happens at draw time and in
the order of the data. Therefore data should be arranged by the label
column before calling \code{geom_label()} or \code{geom_text()}.}

\item{angle_calc}{Either 'none', 'along', or 'across'. If 'none' the label will
use the angle aesthetic of the geom. If 'along' The label will be written
along the edge direction. If 'across' the label will be written across the
edge direction.}

\item{force_flip}{Logical. If \code{angle_calc} is either 'along' or 'across'
should the label be flipped if it is on it's head. Default to \code{TRUE}.}

\item{label_dodge}{A \code{\link[grid:unit]{grid::unit()}} giving a fixed vertical shift
to add to the label in case of \code{angle_calc} is either 'along' or 'across'}

\item{label_push}{A \code{\link[grid:unit]{grid::unit()}} giving a fixed horizontal shift
to add to the label in case of \code{angle_calc} is either 'along' or 'across'}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}
}
\description{
This geom draws multi edges as parallel lines. The edges are first sorted by
direction and then shifted a fixed amount so that all edges are visible.
}
\section{Aesthetics}{

\code{geom_edge_parallel} and \code{geom_edge_parallel0} understand the following
aesthetics. Bold aesthetics are automatically set, but can be overridden.
\itemize{
\item \strong{x}
\item \strong{y}
\item \strong{xend}
\item \strong{yend}
\item \strong{from}
\item \strong{to}
\item edge_colour
\item edge_width
\item edge_linetype
\item edge_alpha
\item filter
}

\code{geom_edge_parallel2} understand the following aesthetics. Bold aesthetics are
automatically set, but can be overridden.
\itemize{
\item \strong{x}
\item \strong{y}
\item \strong{group}
\item \strong{from}
\item \strong{to}
\item edge_colour
\item edge_width
\item edge_linetype
\item edge_alpha
\item filter
}

\code{geom_edge_parallel} and \code{geom_edge_parallel2} furthermore takes the following
aesthetics.
\itemize{
\item start_cap
\item end_cap
\item label
\item label_pos
\item label_size
\item angle
\item hjust
\item vjust
\item family
\item fontface
\item lineheight
}
}

\section{Computed variables}{


\describe{
\item{index}{The position along the path (not computed for the *0 version)}
}
}

\section{Edge variants}{

Many geom_edge_* layers comes in 3 flavors depending on the level of control
needed over the drawing. The default (no numeric postfix) generate a number
of points (\code{n}) along the edge and draws it as a path. Each point along
the line has a numeric value associated with it giving the position along the
path, and it is therefore possible to show the direction of the edge by
mapping to this e.g. \code{colour = stat(index)}. The version postfixed with a
"2" uses the "long" edge format (see \code{\link[=get_edges]{get_edges()}}) and makes it
possible to interpolate node parameter between the start and end node along
the edge. It is considerable less performant so should only be used if this
is needed. The version postfixed with a "0" draws the edge in the most
performant way, often directly using an appropriate grob from the grid
package, but does not allow for gradients along the edge.

Often it is beneficial to stop the drawing of the edge before it reaches the
node, for instance in cases where an arrow should be drawn and the arrowhead
shouldn't lay on top or below the node point. geom_edge_* and geom_edge_*2
supports this through the start_cap and end_cap aesthetics that takes a
\code{\link[=geometry]{geometry()}} specification and dynamically caps the termini of the
edges based on the given specifications. This means that if
\code{end_cap = circle(1, 'cm')} the edges will end at a distance of 1cm even
during resizing of the plot window.

All \verb{geom_edge_*} and \code{geom_edge_*2} have the ability to draw a
label along the edge. The reason this is not a separate geom is that in order
for the label to know the location of the edge it needs to know the edge type
etc. Labels are drawn by providing a label aesthetic. The label_pos can be
used to specify where along the edge it should be drawn by supplying a number
between 0 and 1. The label_size aesthetic can be used to control the size of
the label. Often it is needed to have the label written along the direction
of the edge, but since the actual angle is dependent on the plot dimensions
this cannot be calculated beforehand. Using the angle_calc argument allows
you to specify whether to use the supplied angle aesthetic or whether to draw
the label along or across the edge.
}

\section{Edge aesthetic name expansion}{

In order to avoid excessive typing edge aesthetic names are
automatically expanded. Because of this it is not necessary to write
\code{edge_colour} within the \code{aes()} call as \code{colour} will
automatically be renamed appropriately.
}

\examples{
require(tidygraph)
gr <- create_notable('bull') \%>\%
  convert(to_directed) \%>\%
  bind_edges(data.frame(from = c(1, 2, 2, 3), to = c(2, 1, 3, 2))) \%E>\%
  mutate(class = sample(letters[1:3], 9, TRUE)) \%N>\%
  mutate(class = sample(c('x', 'y'), 5, TRUE))

ggraph(gr, 'stress') +
  geom_edge_parallel(aes(alpha = stat(index)))

ggraph(gr, 'stress') +
  geom_edge_parallel2(aes(colour = node.class))

ggraph(gr, 'stress') +
  geom_edge_parallel0(aes(colour = class))

# Use capping and sep to fine tune the look
ggraph(gr, 'stress') +
  geom_edge_parallel(start_cap = circle(1), end_cap = circle(1),
                     arrow = arrow(length = unit(2, 'mm')), sep = unit(4, 'mm')) +
  geom_node_point(size = 12)

}
\seealso{
Other geom_edge_*: 
\code{\link{geom_edge_arc}()},
\code{\link{geom_edge_bend}()},
\code{\link{geom_edge_density}()},
\code{\link{geom_edge_diagonal}()},
\code{\link{geom_edge_elbow}()},
\code{\link{geom_edge_fan}()},
\code{\link{geom_edge_hive}()},
\code{\link{geom_edge_link}()},
\code{\link{geom_edge_loop}()},
\code{\link{geom_edge_point}()},
\code{\link{geom_edge_span}()},
\code{\link{geom_edge_tile}()}
}
\author{
David Schoch and Thomas Lin Pedersen
}
\concept{geom_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_fabric.R
\name{layout_tbl_graph_fabric}
\alias{layout_tbl_graph_fabric}
\alias{node_rank_fabric}
\title{Create a fabric layout}
\usage{
layout_tbl_graph_fabric(
  graph,
  circular = FALSE,
  sort.by = NULL,
  shadow.edges = FALSE
)

node_rank_fabric()
}
\arguments{
\item{graph}{An \code{tbl_graph} object}

\item{circular}{Ignored}

\item{sort.by}{An expression providing the sorting of the nodes. If \code{NULL}
the nodes will be ordered by their index in the graph.}

\item{shadow.edges}{Should shadow edges be shown.}
}
\value{
A data.frame with the columns \code{x}, \code{xmin}, \code{xmax}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
Further, the edges of the graph will gain a \code{edge_x} variable giving the
horizontal position of the edge as well as a \code{shadow_edge} variable denoting
whether the edge is a shadow edge added by the layout.
}
\description{
This layout is a bit unusual in that it shows nodes as horizontal line ranges
end edges as evenly spaced vertical spans connecting the nodes. As with the
matrix layout the strength comes from better scalability but its use require
some experience recognising the patterns that different connectivity features
gives rise to. As with matrix layouts the ordering of nodes have huge power
over the look of the plot. The \code{node_rank_fabric()} mimics the default
ordering from the original BioFabric implementation, but other ranking
algorithms from tidygraph can be used with the \code{sort.by} argument as well.
Fabric layouts tend to become quite wide as the graph grows which is
something that should be handled with care - e.g. by only zooming in on a
specific region.
}
\references{
BioFabric website: \url{http://www.biofabric.org}

Longabaugh, William J.R. (2012).
\emph{Combing the hairball with BioFabric: a new approach for visualization of large networks}.
BMC Bioinformatics, 13: 275. \url{https://doi.org/10.1186/1471-2105-13-275}
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_fabric.R
\name{layout_tbl_graph_fabric}
\alias{layout_tbl_graph_fabric}
\alias{node_rank_fabric}
\title{Create a fabric layout}
\usage{
layout_tbl_graph_fabric(
  graph,
  circular = FALSE,
  sort.by = NULL,
  shadow.edges = FALSE
)

node_rank_fabric()
}
\arguments{
\item{graph}{An \code{tbl_graph} object}

\item{circular}{Ignored}

\item{sort.by}{An expression providing the sorting of the nodes. If \code{NULL}
the nodes will be ordered by their index in the graph.}

\item{shadow.edges}{Should shadow edges be shown.}
}
\value{
A data.frame with the columns \code{x}, \code{xmin}, \code{xmax}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
Further, the edges of the graph will gain a \code{edge_x} variable giving the
horizontal position of the edge as well as a \code{shadow_edge} variable denoting
whether the edge is a shadow edge added by the layout.
}
\description{
This layout is a bit unusual in that it shows nodes as horizontal line ranges
end edges as evenly spaced vertical spans connecting the nodes. As with the
matrix layout the strength comes from better scalability but its use require
some experience recognising the patterns that different connectivity features
gives rise to. As with matrix layouts the ordering of nodes have huge power
over the look of the plot. The \code{node_rank_fabric()} mimics the default
ordering from the original BioFabric implementation, but other ranking
algorithms from tidygraph can be used with the \code{sort.by} argument as well.
Fabric layouts tend to become quite wide as the graph grows which is
something that should be handled with care - e.g. by only zooming in on a
specific region.
}
\references{
BioFabric website: \url{https://biofabric.systemsbiology.net}

Longabaugh, William J.R. (2012).
\emph{Combing the hairball with BioFabric: a new approach for visualization of large networks}.
BMC Bioinformatics, 13: 275. \doi{10.1186/1471-2105-13-275}
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_cactustree}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_htree}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_metro}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_sf}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_label_size.R
\name{scale_label_size}
\alias{scale_label_size}
\alias{scale_label_size_continuous}
\alias{scale_label_size_discrete}
\alias{scale_label_size_manual}
\alias{scale_label_size_identity}
\title{Edge label size scales}
\usage{
scale_label_size_continuous(..., range = c(1, 6))

scale_label_size(..., range = c(1, 6))

scale_label_size_discrete(..., range = c(2, 6))

scale_label_size_manual(..., values)

scale_label_size_identity(..., guide = "none")
}
\arguments{
\item{...}{
  Arguments passed on to \code{\link[ggplot2:continuous_scale]{continuous_scale}}
  \describe{
    \item{\code{minor_breaks}}{One of:
\itemize{
\item \code{NULL} for no minor breaks
\item \code{waiver()} for the default breaks (one minor break between
each major break)
\item A numeric vector of positions
\item A function that given the limits returns a vector of minor breaks.
}}
    \item{\code{oob}}{One of:
\itemize{
\item Function that handles limits outside of the scale limits
(out of bounds).
\item The default (\code{\link[scales:oob]{scales::censor()}}) replaces out of
bounds values with \code{NA}.
\item \code{\link[scales:oob]{scales::squish()}} for squishing out of bounds values into range.
\item \code{\link[scales:oob]{scales::squish_infinite()}} for squishing infinite values into range.
}}
    \item{\code{na.value}}{Missing values will be replaced with this value.}
    \item{\code{expand}}{For position scales, a vector of range expansion constants used to add some
padding around the data to ensure that they are placed some distance
away from the axes. Use the convenience function \code{\link[ggplot2:expansion]{expansion()}}
to generate the values for the \code{expand} argument. The defaults are to
expand the scale by 5\% on each side for continuous variables, and by
0.6 units on each side for discrete variables.}
    \item{\code{position}}{For position scales, The position of the axis.
\code{left} or \code{right} for y axes, \code{top} or \code{bottom} for x axes.}
    \item{\code{super}}{The super class to use for the constructed scale}
  }}

\item{range}{a numeric vector of length 2 that specifies the minimum and
maximum size of the plotting symbol after transformation.}

\item{values}{a set of aesthetic values to map data values to. The values
will be matched in order (usually alphabetical) with the limits of the
scale, or with \code{breaks} if provided. If this is a named vector, then the
values will be matched based on the names instead. Data values that don't
match will be given \code{na.value}.}

\item{guide}{A function used to create a guide or its name. See
\code{\link[ggplot2:guides]{guides()}} for more information.}
}
\value{
A ggproto object inheriting from \code{Scale}
}
\description{
This set of scales defines new size scales for edge labels in order to allow
for separate sizing of edges and their labels.
}
\seealso{
Other scale_edge_*: 
\code{\link{scale_edge_alpha}()},
\code{\link{scale_edge_colour}},
\code{\link{scale_edge_fill}},
\code{\link{scale_edge_linetype}()},
\code{\link{scale_edge_shape}()},
\code{\link{scale_edge_size}()},
\code{\link{scale_edge_width}()}
}
\concept{scale_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_edge_shape.R
\name{scale_edge_shape}
\alias{scale_edge_shape}
\alias{scale_edge_shape_discrete}
\alias{scale_edge_shape_continuous}
\alias{scale_edge_shape_binned}
\alias{scale_edge_shape_manual}
\alias{scale_edge_shape_identity}
\title{Edge shape scales}
\usage{
scale_edge_shape(..., solid = TRUE)

scale_edge_shape_discrete(..., solid = TRUE)

scale_edge_shape_continuous(...)

scale_edge_shape_binned(..., solid = TRUE)

scale_edge_shape_manual(..., values, breaks = waiver(), na.value = NA)

scale_edge_shape_identity(..., guide = "none")
}
\arguments{
\item{...}{
  Arguments passed on to \code{\link[ggplot2:discrete_scale]{discrete_scale}}
  \describe{
    \item{\code{breaks}}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{waiver()} for the default breaks (the scale limits)
\item A character vector of breaks
\item A function that takes the limits as input and returns breaks
as output. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}
    \item{\code{limits}}{One of:
\itemize{
\item \code{NULL} to use the default scale values
\item A character vector that defines possible values of the scale and their
order
\item A function that accepts the existing (automatic) values and returns
new ones. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}
    \item{\code{drop}}{Should unused factor levels be omitted from the scale?
The default, \code{TRUE}, uses the levels that appear in the data;
\code{FALSE} includes the levels in the factor. Please note that to display
every level in a legend, the layer should use \code{show.legend = TRUE}.}
    \item{\code{na.translate}}{Unlike continuous scales, discrete scales can easily show
missing values, and do so by default. If you want to remove missing values
from a discrete scale, specify \code{na.translate = FALSE}.}
    \item{\code{na.value}}{If \code{na.translate = TRUE}, what aesthetic value should the
missing values be displayed as? Does not apply to position scales
where \code{NA} is always placed at the far right.}
    \item{\code{minor_breaks}}{One of:
\itemize{
\item \code{NULL} for no minor breaks
\item \code{waiver()} for the default breaks (none for discrete, one minor break
between each major break for continuous)
\item A numeric vector of positions
\item A function that given the limits returns a vector of minor breaks. Also
accepts rlang \link[rlang:as_function]{lambda} function notation. When
the function has two arguments, it will be given the limits and major
break positions.
}}
    \item{\code{labels}}{One of the options below. Please note that when \code{labels} is a
vector, it is highly recommended to also set the \code{breaks} argument as a
vector to protect against unintended mismatches.
\itemize{
\item \code{NULL} for no labels
\item \code{waiver()} for the default labels computed by the
transformation object
\item A character vector giving labels (must be same length as \code{breaks})
\item An expression vector (must be the same length as breaks). See ?plotmath for details.
\item A function that takes the breaks as input and returns labels
as output. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}
    \item{\code{guide}}{A function used to create a guide or its name. See
\code{\link[ggplot2:guides]{guides()}} for more information.}
    \item{\code{call}}{The \code{call} used to construct the scale for reporting messages.}
    \item{\code{super}}{The super class to use for the constructed scale}
  }}

\item{solid}{Should the shapes be solid, \code{TRUE}, or hollow,
\code{FALSE}?}

\item{values}{a set of aesthetic values to map data values to. The values
will be matched in order (usually alphabetical) with the limits of the
scale, or with \code{breaks} if provided. If this is a named vector, then the
values will be matched based on the names instead. Data values that don't
match will be given \code{na.value}.}

\item{breaks}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{waiver()} for the default breaks (the scale limits)
\item A character vector of breaks
\item A function that takes the limits as input and returns breaks
as output
}}

\item{na.value}{The aesthetic value to use for missing (\code{NA}) values}

\item{guide}{Guide to use for this scale.}
}
\value{
A ggproto object inheriting from \code{Scale}
}
\description{
This set of scales defines new shape scales for edge geoms equivalent to the
ones already defined by ggplot2. See \code{\link[ggplot2:scale_shape]{ggplot2::scale_shape()}} for
more information. The different geoms will know whether to use edge scales or
the standard scales so it is not necessary to write \code{edge_shape} in
the call to the geom - just use \code{shape}.
}
\seealso{
Other scale_edge_*: 
\code{\link{scale_edge_alpha}()},
\code{\link{scale_edge_colour}},
\code{\link{scale_edge_fill}},
\code{\link{scale_edge_linetype}()},
\code{\link{scale_edge_size}()},
\code{\link{scale_edge_width}()},
\code{\link{scale_label_size}()}
}
\concept{scale_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_edge_colour.R
\name{scale_edge_colour}
\alias{scale_edge_colour}
\alias{scale_edge_colour_hue}
\alias{scale_edge_colour_brewer}
\alias{scale_edge_colour_distiller}
\alias{scale_edge_colour_gradient}
\alias{scale_edge_colour_gradient2}
\alias{scale_edge_colour_gradientn}
\alias{scale_edge_colour_grey}
\alias{scale_edge_colour_identity}
\alias{scale_edge_colour_manual}
\alias{scale_edge_colour_viridis}
\alias{scale_edge_colour_steps}
\alias{scale_edge_colour_steps2}
\alias{scale_edge_colour_stepsn}
\alias{scale_edge_colour_fermenter}
\alias{scale_edge_colour_continuous}
\alias{scale_edge_colour_discrete}
\alias{scale_edge_colour_binned}
\alias{scale_edge_color_hue}
\alias{scale_edge_color_brewer}
\alias{scale_edge_color_distiller}
\alias{scale_edge_color_gradient}
\alias{scale_edge_color_gradient2}
\alias{scale_edge_color_gradientn}
\alias{scale_edge_color_grey}
\alias{scale_edge_color_identity}
\alias{scale_edge_color_manual}
\alias{scale_edge_color_continuous}
\alias{scale_edge_color_discrete}
\alias{scale_edge_color_viridis}
\alias{scale_edge_color_steps}
\alias{scale_edge_color_steps2}
\alias{scale_edge_color_stepsn}
\alias{scale_edge_color_fermenter}
\alias{scale_edge_color_binned}
\title{Edge colour scales}
\usage{
scale_edge_colour_hue(
  ...,
  h = c(0, 360) + 15,
  c = 100,
  l = 65,
  h.start = 0,
  direction = 1,
  na.value = "grey50",
  aesthetics = "edge_colour"
)

scale_edge_colour_brewer(
  ...,
  type = "seq",
  palette = 1,
  direction = 1,
  aesthetics = "edge_colour"
)

scale_edge_colour_distiller(
  ...,
  type = "seq",
  palette = 1,
  direction = -1,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
)

scale_edge_colour_gradient(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
)

scale_edge_colour_gradient2(
  ...,
  low = muted("red"),
  mid = "white",
  high = muted("blue"),
  midpoint = 0,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
)

scale_edge_colour_gradientn(
  ...,
  colours,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour",
  colors
)

scale_edge_colour_grey(
  ...,
  start = 0.2,
  end = 0.8,
  na.value = "red",
  aesthetics = "edge_colour"
)

scale_edge_colour_identity(..., guide = "none", aesthetics = "edge_colour")

scale_edge_colour_manual(
  ...,
  values,
  aesthetics = "edge_colour",
  breaks = waiver(),
  na.value = "grey50"
)

scale_edge_colour_viridis(
  ...,
  alpha = 1,
  begin = 0,
  end = 1,
  direction = 1,
  discrete = FALSE,
  option = "D",
  aesthetics = "edge_colour"
)

scale_edge_colour_steps(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
)

scale_edge_colour_steps2(
  ...,
  low = muted("red"),
  mid = "white",
  high = muted("blue"),
  midpoint = 0,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
)

scale_edge_colour_stepsn(
  ...,
  colours,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour",
  colors
)

scale_edge_colour_fermenter(
  ...,
  type = "seq",
  palette = 1,
  direction = -1,
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
)

scale_edge_colour_continuous(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
)

scale_edge_colour_discrete(
  ...,
  h = c(0, 360) + 15,
  c = 100,
  l = 65,
  h.start = 0,
  direction = 1,
  na.value = "grey50",
  aesthetics = "edge_colour"
)

scale_edge_colour_binned(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
)

scale_edge_color_hue(
  ...,
  h = c(0, 360) + 15,
  c = 100,
  l = 65,
  h.start = 0,
  direction = 1,
  na.value = "grey50",
  aesthetics = "edge_colour"
)

scale_edge_color_brewer(
  ...,
  type = "seq",
  palette = 1,
  direction = 1,
  aesthetics = "edge_colour"
)

scale_edge_color_distiller(
  ...,
  type = "seq",
  palette = 1,
  direction = -1,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
)

scale_edge_color_gradient(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
)

scale_edge_color_gradient2(
  ...,
  low = muted("red"),
  mid = "white",
  high = muted("blue"),
  midpoint = 0,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
)

scale_edge_color_gradientn(
  ...,
  colours,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour",
  colors
)

scale_edge_color_grey(
  ...,
  start = 0.2,
  end = 0.8,
  na.value = "red",
  aesthetics = "edge_colour"
)

scale_edge_color_identity(..., guide = "none", aesthetics = "edge_colour")

scale_edge_color_manual(
  ...,
  values,
  aesthetics = "edge_colour",
  breaks = waiver(),
  na.value = "grey50"
)

scale_edge_color_continuous(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
)

scale_edge_color_discrete(
  ...,
  h = c(0, 360) + 15,
  c = 100,
  l = 65,
  h.start = 0,
  direction = 1,
  na.value = "grey50",
  aesthetics = "edge_colour"
)

scale_edge_color_viridis(
  ...,
  alpha = 1,
  begin = 0,
  end = 1,
  direction = 1,
  discrete = FALSE,
  option = "D",
  aesthetics = "edge_colour"
)

scale_edge_color_steps(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
)

scale_edge_color_steps2(
  ...,
  low = muted("red"),
  mid = "white",
  high = muted("blue"),
  midpoint = 0,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
)

scale_edge_color_stepsn(
  ...,
  colours,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour",
  colors
)

scale_edge_color_fermenter(
  ...,
  type = "seq",
  palette = 1,
  direction = -1,
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
)

scale_edge_color_binned(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
)
}
\arguments{
\item{...}{
  Arguments passed on to \code{\link[ggplot2:discrete_scale]{discrete_scale}}
  \describe{
    \item{\code{palette}}{A palette function that when called with a single integer
argument (the number of levels in the scale) returns the values that
they should take (e.g., \code{\link[scales:pal_hue]{scales::pal_hue()}}).}
    \item{\code{breaks}}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{waiver()} for the default breaks (the scale limits)
\item A character vector of breaks
\item A function that takes the limits as input and returns breaks
as output. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}
    \item{\code{limits}}{One of:
\itemize{
\item \code{NULL} to use the default scale values
\item A character vector that defines possible values of the scale and their
order
\item A function that accepts the existing (automatic) values and returns
new ones. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}
    \item{\code{drop}}{Should unused factor levels be omitted from the scale?
The default, \code{TRUE}, uses the levels that appear in the data;
\code{FALSE} uses all the levels in the factor.}
    \item{\code{na.translate}}{Unlike continuous scales, discrete scales can easily show
missing values, and do so by default. If you want to remove missing values
from a discrete scale, specify \code{na.translate = FALSE}.}
    \item{\code{scale_name}}{\ifelse{html}{\href{https://lifecycle.r-lib.org/articles/stages.html#deprecated}{\figure{lifecycle-deprecated.svg}{options: alt='[Deprecated]'}}}{\strong{[Deprecated]}} The name of the scale
that should be used for error messages associated with this scale.}
    \item{\code{labels}}{One of:
\itemize{
\item \code{NULL} for no labels
\item \code{waiver()} for the default labels computed by the
transformation object
\item A character vector giving labels (must be same length as \code{breaks})
\item An expression vector (must be the same length as breaks). See ?plotmath for details.
\item A function that takes the breaks as input and returns labels
as output. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}
    \item{\code{guide}}{A function used to create a guide or its name. See
\code{\link[ggplot2:guides]{guides()}} for more information.}
    \item{\code{expand}}{For position scales, a vector of range expansion constants used to add some
padding around the data to ensure that they are placed some distance
away from the axes. Use the convenience function \code{\link[ggplot2:expansion]{expansion()}}
to generate the values for the \code{expand} argument. The defaults are to
expand the scale by 5\% on each side for continuous variables, and by
0.6 units on each side for discrete variables.}
    \item{\code{position}}{For position scales, The position of the axis.
\code{left} or \code{right} for y axes, \code{top} or \code{bottom} for x axes.}
    \item{\code{call}}{The \code{call} used to construct the scale for reporting messages.}
    \item{\code{super}}{The super class to use for the constructed scale}
  }}

\item{h}{range of hues to use, in [0, 360]}

\item{c}{chroma (intensity of colour), maximum value varies depending on
combination of hue and luminance.}

\item{l}{luminance (lightness), in [0, 100]}

\item{h.start}{hue to start at}

\item{direction}{direction to travel around the colour wheel,
1 = clockwise, -1 = counter-clockwise}

\item{na.value}{Colour to use for missing values}

\item{aesthetics}{Character string or vector of character strings listing the
name(s) of the aesthetic(s) that this scale works with. This can be useful, for
example, to apply colour settings to the \code{colour} and \code{fill} aesthetics at the
same time, via \code{aesthetics = c("colour", "fill")}.}

\item{type}{One of "seq" (sequential), "div" (diverging) or "qual"
(qualitative)}

\item{palette}{If a string, will use that named palette. If a number, will index into
the list of palettes of appropriate \code{type}. The list of available palettes can found
in the Palettes section.}

\item{values}{if colours should not be evenly positioned along the gradient
this vector gives the position (between 0 and 1) for each colour in the
\code{colours} vector. See \code{\link[scales:rescale]{rescale()}} for a convenience function
to map an arbitrary range to between 0 and 1.}

\item{space}{colour space in which to calculate gradient. Must be "Lab" -
other values are deprecated.}

\item{guide}{Type of legend. Use \code{"colourbar"} for continuous
colour bar, or \code{"legend"} for discrete colour legend.}

\item{low, high}{Colours for low and high ends of the gradient.}

\item{mid}{colour for mid point}

\item{midpoint}{The midpoint (in data value) of the diverging scale.
Defaults to 0.}

\item{colours, colors}{Vector of colours to use for n-colour gradient.}

\item{start}{grey value at low end of palette}

\item{end}{grey value at high end of palette}

\item{breaks}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{waiver()} for the default breaks (the scale limits)
\item A character vector of breaks
\item A function that takes the limits as input and returns breaks
as output
}}

\item{alpha}{The alpha transparency, a number in [0,1], see argument alpha in
\code{\link[grDevices]{hsv}}.}

\item{begin}{The (corrected) hue in [0,1] at which the color map begins.}

\item{discrete}{Generate a discrete palette? (default: \code{FALSE} -
generate continuous palette).}

\item{option}{A character string indicating the color map option to use.
Eight options are available:
\itemize{
 \item "magma" (or "A")
 \item "inferno" (or "B")
 \item "plasma" (or "C")
 \item "viridis" (or "D")
 \item "cividis" (or "E")
 \item "rocket" (or "F")
 \item "mako" (or "G")
 \item "turbo" (or "H")
}}
}
\value{
A ggproto object inheriting from \code{Scale}
}
\description{
This set of scales defines new colour scales for edge geoms equivalent to the
ones already defined by ggplot2. The parameters are equivalent to the ones
from ggplot2 so there is nothing new under the sun. The different geoms will
know whether to use edge scales or the standard scales so it is not necessary
to write \code{edge_colour} in the call to the geom - just use \code{colour}.
}
\seealso{
Other scale_edge_*: 
\code{\link{scale_edge_alpha}()},
\code{\link{scale_edge_fill}},
\code{\link{scale_edge_linetype}()},
\code{\link{scale_edge_shape}()},
\code{\link{scale_edge_size}()},
\code{\link{scale_edge_width}()},
\code{\link{scale_label_size}()}
}
\concept{scale_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geometry.R
\name{scale_type.geometry}
\alias{scale_type.geometry}
\title{Define default scale type for geometry}
\usage{
\method{scale_type}{geometry}(x)
}
\description{
This function is quite useless as geometry is not meant to be scaled, but it
is a requirement for ggplot2 to handle it correctly.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_conn_bundle.R
\name{geom_conn_bundle}
\alias{geom_conn_bundle}
\alias{geom_conn_bundle2}
\alias{geom_conn_bundle0}
\title{Create hierarchical edge bundles between node connections}
\usage{
geom_conn_bundle(
  mapping = NULL,
  data = get_con(),
  position = "identity",
  arrow = NULL,
  lineend = "butt",
  show.legend = NA,
  n = 100,
  tension = 0.8,
  ...
)

geom_conn_bundle2(
  mapping = NULL,
  data = get_con(),
  position = "identity",
  arrow = NULL,
  lineend = "butt",
  show.legend = NA,
  n = 100,
  tension = 0.8,
  ...
)

geom_conn_bundle0(
  mapping = NULL,
  data = get_con(),
  position = "identity",
  arrow = NULL,
  lineend = "butt",
  show.legend = NA,
  tension = 0.8,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x, y, xend, yend, group and
circular are mapped to x, y, xend, yend, edge.id and circular in the edge
data.}

\item{data}{The result of a call to \code{\link[=get_con]{get_con()}}}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{arrow}{Arrow specification, as created by \code{\link[grid:arrow]{grid::arrow()}}.}

\item{lineend}{Line end style (round, butt, square).}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display. To include legend keys for all levels, even
when no data exists, use \code{TRUE}.  If \code{NA}, all levels are shown in legend,
but unobserved levels are omitted.}

\item{n}{The number of points to create along the path.}

\item{tension}{How "loose" should the bundles be. 1 will give very tight
bundles, while 0 will turn of bundling completely and give straight lines.
Defaults to 0.8}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}'s \code{params} argument. These
arguments broadly fall into one of 4 categories below. Notably, further
arguments to the \code{position} argument, or aesthetics that are required
can \emph{not} be passed through \code{...}. Unknown arguments that are not part
of the 4 categories below are ignored.
\itemize{
\item Static aesthetics that are not mapped to a scale, but are at a fixed
value and apply to the layer as a whole. For example, \code{colour = "red"}
or \code{linewidth = 3}. The geom's documentation has an \strong{Aesthetics}
section that lists the available options. The 'required' aesthetics
cannot be passed on to the \code{params}. Please note that while passing
unmapped aesthetics as vectors is technically possible, the order and
required length is not guaranteed to be parallel to the input data.
\item When constructing a layer using
a \verb{stat_*()} function, the \code{...} argument can be used to pass on
parameters to the \code{geom} part of the layer. An example of this is
\code{stat_density(geom = "area", outline.type = "both")}. The geom's
documentation lists which parameters it can accept.
\item Inversely, when constructing a layer using a
\verb{geom_*()} function, the \code{...} argument can be used to pass on parameters
to the \code{stat} part of the layer. An example of this is
\code{geom_area(stat = "density", adjust = 0.5)}. The stat's documentation
lists which parameters it can accept.
\item The \code{key_glyph} argument of \code{\link[ggplot2:layer]{layer()}} may also be passed on through
\code{...}. This can be one of the functions described as
\link[ggplot2:draw_key]{key glyphs}, to change the display of the layer in the legend.
}}
}
\description{
Hierarchical edge bundling is a technique to introduce some order into the
hairball structure that can appear when there's a lot of overplotting and
edge crossing in a network plot. The concept requires that the network has
an intrinsic hierarchical structure that defines the layout but is not shown.
Connections between points (that is, not edges) are then drawn so that they
loosely follows the underlying hierarchical structure. This results in a
flow-like structure where lines that partly move in the same direction will
be bundled together.
}
\note{
In order to avoid excessive typing edge aesthetic names are
automatically expanded. Because of this it is not necessary to write
\code{edge_colour} within the \code{aes()} call as \code{colour} will
automatically be renamed appropriately.
}
\section{Aesthetics}{

geom_conn_bundle* understands the following aesthetics. Bold aesthetics are
automatically set, but can be overwritten.
\itemize{
\item{\strong{x}}
\item{\strong{y}}
\item{\strong{group}}
\item{\strong{circular}}
\item{edge_colour}
\item{edge_width}
\item{edge_linetype}
\item{edge_alpha}
\item{filter}
}
}

\section{Computed variables}{


\describe{
\item{index}{The position along the path (not computed for the *0 version)}
}
}

\examples{
# Create a graph of the flare class system
library(tidygraph)
flareGraph <- tbl_graph(flare$vertices, flare$edges) \%>\%
  mutate(
    class = map_bfs_chr(node_is_root(), .f = function(node, dist, path, ...) {
      if (dist <= 1) {
        return(shortName[node])
      }
      path$result[[nrow(path)]]
    })
  )
importFrom <- match(flare$imports$from, flare$vertices$name)
importTo <- match(flare$imports$to, flare$vertices$name)

# Use class inheritance for layout but plot class imports as bundles
ggraph(flareGraph, 'dendrogram', circular = TRUE) +
  geom_conn_bundle(aes(colour = after_stat(index)),
    data = get_con(importFrom, importTo),
    edge_alpha = 0.25
  ) +
  geom_node_point(aes(filter = leaf, colour = class)) +
  scale_edge_colour_distiller('', direction = 1, guide = 'edge_direction') +
  coord_fixed() +
  ggforce::theme_no_axes()
}
\references{
Holten, D. (2006). \emph{Hierarchical edge bundles: visualization
of adjacency relations in hierarchical data.} IEEE Transactions on
Visualization and Computer Graphics, \strong{12}(5), 741-748.
\doi{10.1109/TVCG.2006.147}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_conn_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_manual.R
\name{layout_tbl_graph_manual}
\alias{layout_tbl_graph_manual}
\title{Manually specify a layout for layout_tbl_graph}
\usage{
layout_tbl_graph_manual(graph, x, y, circular)
}
\arguments{
\item{graph}{An \code{tbl_graph} object}

\item{x, y}{Expressions with the x and y positions of the nodes}

\item{circular}{Ignored}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph.
}
\description{
This layout function lets you pass the node positions in manually. The
supplied positions must match the order of the nodes in the tbl_graph
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_cactustree}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_htree}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_metro}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_sf}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scale_edge_colour.R
\name{scale_edge_colour}
\alias{scale_edge_colour}
\alias{scale_edge_colour_hue}
\alias{scale_edge_colour_brewer}
\alias{scale_edge_colour_distiller}
\alias{scale_edge_colour_gradient}
\alias{scale_edge_colour_gradient2}
\alias{scale_edge_colour_gradientn}
\alias{scale_edge_colour_grey}
\alias{scale_edge_colour_identity}
\alias{scale_edge_colour_manual}
\alias{scale_edge_colour_viridis}
\alias{scale_edge_colour_steps}
\alias{scale_edge_colour_steps2}
\alias{scale_edge_colour_stepsn}
\alias{scale_edge_colour_fermenter}
\alias{scale_edge_colour_continuous}
\alias{scale_edge_colour_discrete}
\alias{scale_edge_colour_binned}
\alias{scale_edge_color_hue}
\alias{scale_edge_color_brewer}
\alias{scale_edge_color_distiller}
\alias{scale_edge_color_gradient}
\alias{scale_edge_color_gradient2}
\alias{scale_edge_color_gradientn}
\alias{scale_edge_color_grey}
\alias{scale_edge_color_identity}
\alias{scale_edge_color_manual}
\alias{scale_edge_color_continuous}
\alias{scale_edge_color_discrete}
\alias{scale_edge_color_viridis}
\alias{scale_edge_color_steps}
\alias{scale_edge_color_steps2}
\alias{scale_edge_color_stepsn}
\alias{scale_edge_color_fermenter}
\alias{scale_edge_color_binned}
\title{Edge colour scales}
\usage{
scale_edge_colour_hue(
  ...,
  h = c(0, 360) + 15,
  c = 100,
  l = 65,
  h.start = 0,
  direction = 1,
  na.value = "grey50",
  aesthetics = "edge_colour"
)

scale_edge_colour_brewer(
  ...,
  type = "seq",
  palette = 1,
  direction = 1,
  aesthetics = "edge_colour"
)

scale_edge_colour_distiller(
  ...,
  type = "seq",
  palette = 1,
  direction = -1,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
)

scale_edge_colour_gradient(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
)

scale_edge_colour_gradient2(
  ...,
  low = muted("red"),
  mid = "white",
  high = muted("blue"),
  midpoint = 0,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
)

scale_edge_colour_gradientn(
  ...,
  colours,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour",
  colors
)

scale_edge_colour_grey(
  ...,
  start = 0.2,
  end = 0.8,
  na.value = "red",
  aesthetics = "edge_colour"
)

scale_edge_colour_identity(..., guide = "none", aesthetics = "edge_colour")

scale_edge_colour_manual(
  ...,
  values,
  aesthetics = "edge_colour",
  breaks = waiver(),
  na.value = "grey50"
)

scale_edge_colour_viridis(
  ...,
  alpha = 1,
  begin = 0,
  end = 1,
  direction = 1,
  discrete = FALSE,
  option = "D",
  aesthetics = "edge_colour"
)

scale_edge_colour_steps(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
)

scale_edge_colour_steps2(
  ...,
  low = muted("red"),
  mid = "white",
  high = muted("blue"),
  midpoint = 0,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
)

scale_edge_colour_stepsn(
  ...,
  colours,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour",
  colors
)

scale_edge_colour_fermenter(
  ...,
  type = "seq",
  palette = 1,
  direction = -1,
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
)

scale_edge_colour_continuous(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
)

scale_edge_colour_discrete(
  ...,
  h = c(0, 360) + 15,
  c = 100,
  l = 65,
  h.start = 0,
  direction = 1,
  na.value = "grey50",
  aesthetics = "edge_colour"
)

scale_edge_colour_binned(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
)

scale_edge_color_hue(
  ...,
  h = c(0, 360) + 15,
  c = 100,
  l = 65,
  h.start = 0,
  direction = 1,
  na.value = "grey50",
  aesthetics = "edge_colour"
)

scale_edge_color_brewer(
  ...,
  type = "seq",
  palette = 1,
  direction = 1,
  aesthetics = "edge_colour"
)

scale_edge_color_distiller(
  ...,
  type = "seq",
  palette = 1,
  direction = -1,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
)

scale_edge_color_gradient(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
)

scale_edge_color_gradient2(
  ...,
  low = muted("red"),
  mid = "white",
  high = muted("blue"),
  midpoint = 0,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
)

scale_edge_color_gradientn(
  ...,
  colours,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour",
  colors
)

scale_edge_color_grey(
  ...,
  start = 0.2,
  end = 0.8,
  na.value = "red",
  aesthetics = "edge_colour"
)

scale_edge_color_identity(..., guide = "none", aesthetics = "edge_colour")

scale_edge_color_manual(
  ...,
  values,
  aesthetics = "edge_colour",
  breaks = waiver(),
  na.value = "grey50"
)

scale_edge_color_continuous(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_colourbar",
  aesthetics = "edge_colour"
)

scale_edge_color_discrete(
  ...,
  h = c(0, 360) + 15,
  c = 100,
  l = 65,
  h.start = 0,
  direction = 1,
  na.value = "grey50",
  aesthetics = "edge_colour"
)

scale_edge_color_viridis(
  ...,
  alpha = 1,
  begin = 0,
  end = 1,
  direction = 1,
  discrete = FALSE,
  option = "D",
  aesthetics = "edge_colour"
)

scale_edge_color_steps(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
)

scale_edge_color_steps2(
  ...,
  low = muted("red"),
  mid = "white",
  high = muted("blue"),
  midpoint = 0,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
)

scale_edge_color_stepsn(
  ...,
  colours,
  values = NULL,
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour",
  colors
)

scale_edge_color_fermenter(
  ...,
  type = "seq",
  palette = 1,
  direction = -1,
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
)

scale_edge_color_binned(
  ...,
  low = "#132B43",
  high = "#56B1F7",
  space = "Lab",
  na.value = "grey50",
  guide = "edge_coloursteps",
  aesthetics = "edge_colour"
)
}
\arguments{
\item{...}{
  Arguments passed on to \code{\link[ggplot2:discrete_scale]{discrete_scale}}
  \describe{
    \item{\code{breaks}}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{waiver()} for the default breaks (the scale limits)
\item A character vector of breaks
\item A function that takes the limits as input and returns breaks
as output. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}
    \item{\code{limits}}{One of:
\itemize{
\item \code{NULL} to use the default scale values
\item A character vector that defines possible values of the scale and their
order
\item A function that accepts the existing (automatic) values and returns
new ones. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}
    \item{\code{drop}}{Should unused factor levels be omitted from the scale?
The default, \code{TRUE}, uses the levels that appear in the data;
\code{FALSE} includes the levels in the factor. Please note that to display
every level in a legend, the layer should use \code{show.legend = TRUE}.}
    \item{\code{na.translate}}{Unlike continuous scales, discrete scales can easily show
missing values, and do so by default. If you want to remove missing values
from a discrete scale, specify \code{na.translate = FALSE}.}
    \item{\code{minor_breaks}}{One of:
\itemize{
\item \code{NULL} for no minor breaks
\item \code{waiver()} for the default breaks (none for discrete, one minor break
between each major break for continuous)
\item A numeric vector of positions
\item A function that given the limits returns a vector of minor breaks. Also
accepts rlang \link[rlang:as_function]{lambda} function notation. When
the function has two arguments, it will be given the limits and major
break positions.
}}
    \item{\code{labels}}{One of the options below. Please note that when \code{labels} is a
vector, it is highly recommended to also set the \code{breaks} argument as a
vector to protect against unintended mismatches.
\itemize{
\item \code{NULL} for no labels
\item \code{waiver()} for the default labels computed by the
transformation object
\item A character vector giving labels (must be same length as \code{breaks})
\item An expression vector (must be the same length as breaks). See ?plotmath for details.
\item A function that takes the breaks as input and returns labels
as output. Also accepts rlang \link[rlang:as_function]{lambda} function
notation.
}}
    \item{\code{guide}}{A function used to create a guide or its name. See
\code{\link[ggplot2:guides]{guides()}} for more information.}
    \item{\code{call}}{The \code{call} used to construct the scale for reporting messages.}
    \item{\code{super}}{The super class to use for the constructed scale}
  }}

\item{h}{range of hues to use, in [0, 360]}

\item{c}{chroma (intensity of colour), maximum value varies depending on
combination of hue and luminance.}

\item{l}{luminance (lightness), in [0, 100]}

\item{h.start}{hue to start at}

\item{direction}{direction to travel around the colour wheel,
1 = clockwise, -1 = counter-clockwise}

\item{na.value}{Colour to use for missing values}

\item{aesthetics}{Character string or vector of character strings listing the
name(s) of the aesthetic(s) that this scale works with. This can be useful, for
example, to apply colour settings to the \code{colour} and \code{fill} aesthetics at the
same time, via \code{aesthetics = c("colour", "fill")}.}

\item{type}{One of "seq" (sequential), "div" (diverging) or "qual"
(qualitative)}

\item{palette}{If a string, will use that named palette. If a number, will index into
the list of palettes of appropriate \code{type}. The list of available palettes can found
in the Palettes section.}

\item{values}{if colours should not be evenly positioned along the gradient
this vector gives the position (between 0 and 1) for each colour in the
\code{colours} vector. See \code{\link[scales:rescale]{rescale()}} for a convenience function
to map an arbitrary range to between 0 and 1.}

\item{space}{colour space in which to calculate gradient. Must be "Lab" -
other values are deprecated.}

\item{guide}{Type of legend. Use \code{"colourbar"} for continuous
colour bar, or \code{"legend"} for discrete colour legend.}

\item{low, high}{Colours for low and high ends of the gradient.}

\item{mid}{colour for mid point}

\item{midpoint}{The midpoint (in data value) of the diverging scale.
Defaults to 0.}

\item{colours, colors}{Vector of colours to use for n-colour gradient.}

\item{start}{grey value at low end of palette}

\item{end}{grey value at high end of palette}

\item{breaks}{One of:
\itemize{
\item \code{NULL} for no breaks
\item \code{waiver()} for the default breaks (the scale limits)
\item A character vector of breaks
\item A function that takes the limits as input and returns breaks
as output
}}

\item{alpha}{The alpha transparency, a number in [0,1], see argument alpha in
\code{\link[grDevices]{hsv}}.}

\item{begin}{The (corrected) hue in [0,1] at which the color map begins.}

\item{discrete}{Generate a discrete palette? (default: \code{FALSE} -
generate continuous palette).}

\item{option}{A character string indicating the color map option to use.
Eight options are available:
\itemize{
 \item "magma" (or "A")
 \item "inferno" (or "B")
 \item "plasma" (or "C")
 \item "viridis" (or "D")
 \item "cividis" (or "E")
 \item "rocket" (or "F")
 \item "mako" (or "G")
 \item "turbo" (or "H")
}}
}
\value{
A ggproto object inheriting from \code{Scale}
}
\description{
This set of scales defines new colour scales for edge geoms equivalent to the
ones already defined by ggplot2. The parameters are equivalent to the ones
from ggplot2 so there is nothing new under the sun. The different geoms will
know whether to use edge scales or the standard scales so it is not necessary
to write \code{edge_colour} in the call to the geom - just use \code{colour}.
}
\seealso{
Other scale_edge_*: 
\code{\link{scale_edge_alpha}()},
\code{\link{scale_edge_fill}},
\code{\link{scale_edge_linetype}()},
\code{\link{scale_edge_shape}()},
\code{\link{scale_edge_size}()},
\code{\link{scale_edge_width}()},
\code{\link{scale_label_size}()}
}
\concept{scale_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_manual.R
\name{layout_tbl_graph_manual}
\alias{layout_tbl_graph_manual}
\title{Manually specify a layout for layout_tbl_graph}
\usage{
layout_tbl_graph_manual(graph, x, y, circular)
}
\arguments{
\item{graph}{An \code{tbl_graph} object}

\item{x, y}{Expressions with the x and y positions of the nodes}

\item{circular}{Ignored}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph.
}
\description{
This layout function lets you pass the node positions in manually. The
supplied positions must match the order of the nodes in the tbl_graph
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_linear.R
\name{layout_tbl_graph_linear}
\alias{layout_tbl_graph_linear}
\title{Place nodes on a line or circle}
\usage{
layout_tbl_graph_linear(
  graph,
  circular,
  sort.by = NULL,
  use.numeric = FALSE,
  offset = pi/2
)
}
\arguments{
\item{graph}{An \code{tbl_graph} object}

\item{circular}{Logical. Should the layout be transformed to a circular
representation. Defaults to \code{FALSE}.}

\item{sort.by}{The name of a node variable to sort the nodes by.}

\item{use.numeric}{Logical. Should a numeric sort.by attribute be used as the
actual x-coordinates in the layout. May lead to overlapping nodes. Defaults
to FALSE}

\item{offset}{If \code{circular = TRUE}, where should it begin. Defaults to
\code{pi/2} which is equivalent to 12 o'clock.}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
}
\description{
This layout puts all nodes on a line, possibly sorted by a node attribute. If
\code{circular = TRUE} the nodes will be laid out on the unit circle instead.
In the case where the \code{sort.by} attribute is numeric, the numeric values
will be used as the x-position and it is thus possible to have uneven spacing
between the nodes.
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_node_circle.R
\name{geom_node_circle}
\alias{geom_node_circle}
\title{Show nodes as circles}
\usage{
geom_node_circle(
  mapping = NULL,
  data = NULL,
  position = "identity",
  show.legend = NA,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x and y are mapped to x0 and y0 in
the node data.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{position}{Position adjustment, either as a string naming the adjustment
(e.g. \code{"jitter"} to use \code{position_jitter}), or the result of a call to a
position adjustment function. Use the latter if you need to change the
settings of the adjustment.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}
}
\description{
This geom is equivalent in functionality to \code{\link[ggforce:geom_circle]{ggforce::geom_circle()}}
and allows for plotting of nodes as circles with a radius scaled by the
coordinate system. Because of the geoms reliance on the coordinate system
it will only produce true circles when combined with
\code{\link[ggplot2:coord_fixed]{ggplot2::coord_fixed()}}
}
\section{Aesthetics}{

\code{geom_node_circle} understand the following aesthetics. Bold aesthetics are
automatically set, but can be overridden.
\itemize{
\item \strong{x0}
\item \strong{y0}
\item \strong{r}
\item alpha
\item colour
\item fill
\item shape
\item size
\item stroke
\item filter
}
}

\examples{
require(tidygraph)
gr <- tbl_graph(flare$vertices, flare$edges)
ggraph(gr, 'circlepack', weight = size) +
  geom_node_circle() +
  coord_fixed()
}
\seealso{
Other geom_node_*: 
\code{\link{geom_node_arc_bar}()},
\code{\link{geom_node_point}()},
\code{\link{geom_node_range}()},
\code{\link{geom_node_text}()},
\code{\link{geom_node_tile}()},
\code{\link{geom_node_voronoi}()}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_node_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_edge_sf.R
\name{geom_edge_sf}
\alias{geom_edge_sf}
\title{Draw edges as LINESTRINGs in geographical space}
\usage{
geom_edge_sf(
  mapping = NULL,
  data = get_sf_edges(),
  position = "identity",
  show.legend = NA,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default geometry is mapped to the geometry in
the edge data.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{position}{Position adjustment, either as a string naming the adjustment
(e.g. \code{"jitter"} to use \code{position_jitter}), or the result of a call to a
position adjustment function. Use the latter if you need to change the
settings of the adjustment.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.

You can also set this to one of "polygon", "line", and "point" to
override the default legend.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}
}
\description{
This geom is equivalent in functionality to \code{\link[ggplot2:ggsf]{ggplot2::geom_sf()}} for \code{LINESTRING}
geometries and allows for plotting of edges in their geographical space in
different colours, linetypes and widths.
}
\section{Aesthetics}{

\code{geom_edge_sf} understand the following aesthetics.
\itemize{
\item alpha
\item colour
\item linetype
\item filter
}
}

\examples{
if (require("sfnetworks", quietly = TRUE)) {
  gr <- sfnetworks::as_sfnetwork(roxel)
  ggraph(gr, 'sf') + geom_edge_sf()
}

}
\seealso{
Other geom_edge_*: 
\code{\link{geom_edge_arc}()},
\code{\link{geom_edge_bend}()},
\code{\link{geom_edge_bundle_force}()},
\code{\link{geom_edge_bundle_minimal}()},
\code{\link{geom_edge_bundle_path}()},
\code{\link{geom_edge_density}()},
\code{\link{geom_edge_diagonal}()},
\code{\link{geom_edge_elbow}()},
\code{\link{geom_edge_fan}()},
\code{\link{geom_edge_hive}()},
\code{\link{geom_edge_link}()},
\code{\link{geom_edge_loop}()},
\code{\link{geom_edge_parallel}()},
\code{\link{geom_edge_point}()},
\code{\link{geom_edge_span}()},
\code{\link{geom_edge_tile}()}
}
\author{
Lorena Abad
}
\concept{geom_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_node_text.R
\name{geom_node_text}
\alias{geom_node_text}
\alias{geom_node_label}
\title{Annotate nodes with text}
\usage{
geom_node_text(
  mapping = NULL,
  data = NULL,
  position = "identity",
  parse = FALSE,
  nudge_x = 0,
  nudge_y = 0,
  check_overlap = FALSE,
  show.legend = NA,
  repel = FALSE,
  ...
)

geom_node_label(
  mapping = NULL,
  data = NULL,
  position = "identity",
  parse = FALSE,
  nudge_x = 0,
  nudge_y = 0,
  label.padding = unit(0.25, "lines"),
  label.r = unit(0.15, "lines"),
  show.legend = NA,
  repel = FALSE,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x and y are mapped to x and y in
the node data.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{parse}{If \code{TRUE}, the labels will be parsed into expressions and
displayed as described in \code{?plotmath}.}

\item{nudge_x, nudge_y}{Horizontal and vertical adjustment to nudge labels by.
Useful for offsetting text from points, particularly on discrete scales.}

\item{check_overlap}{If \code{TRUE}, text that overlaps previous text in the
same layer will not be plotted. \code{check_overlap} happens at draw time and in
the order of the data. Therefore data should be arranged by the label
column before calling \code{geom_text()}. Note that this argument is not
supported by \code{geom_label()}.}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display. To include legend keys for all levels, even
when no data exists, use \code{TRUE}.  If \code{NA}, all levels are shown in legend,
but unobserved levels are omitted.}

\item{repel}{If \code{TRUE}, text labels will be repelled from each other
to avoid overlapping, using the \code{GeomTextRepel} geom from the
ggrepel package.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}'s \code{params} argument. These
arguments broadly fall into one of 4 categories below. Notably, further
arguments to the \code{position} argument, or aesthetics that are required
can \emph{not} be passed through \code{...}. Unknown arguments that are not part
of the 4 categories below are ignored.
\itemize{
\item Static aesthetics that are not mapped to a scale, but are at a fixed
value and apply to the layer as a whole. For example, \code{colour = "red"}
or \code{linewidth = 3}. The geom's documentation has an \strong{Aesthetics}
section that lists the available options. The 'required' aesthetics
cannot be passed on to the \code{params}. Please note that while passing
unmapped aesthetics as vectors is technically possible, the order and
required length is not guaranteed to be parallel to the input data.
\item When constructing a layer using
a \verb{stat_*()} function, the \code{...} argument can be used to pass on
parameters to the \code{geom} part of the layer. An example of this is
\code{stat_density(geom = "area", outline.type = "both")}. The geom's
documentation lists which parameters it can accept.
\item Inversely, when constructing a layer using a
\verb{geom_*()} function, the \code{...} argument can be used to pass on parameters
to the \code{stat} part of the layer. An example of this is
\code{geom_area(stat = "density", adjust = 0.5)}. The stat's documentation
lists which parameters it can accept.
\item The \code{key_glyph} argument of \code{\link[ggplot2:layer]{layer()}} may also be passed on through
\code{...}. This can be one of the functions described as
\link[ggplot2:draw_key]{key glyphs}, to change the display of the layer in the legend.
}}

\item{label.padding}{Amount of padding around label. Defaults to 0.25 lines.}

\item{label.r}{Radius of rounded corners. Defaults to 0.15 lines.}
}
\description{
These geoms are equivalent in functionality to \code{\link[ggplot2:geom_text]{ggplot2::geom_text()}} and
\code{\link[ggplot2:geom_text]{ggplot2::geom_label()}} and allows for simple annotation of nodes.
}
\section{Aesthetics}{

\code{geom_node_text} understands the following aesthetics. Bold aesthetics are
automatically set, but can be overwritten. Italic aesthetics are required but
not set by default
\itemize{
\item \strong{x}
\item \strong{y}
\item \emph{label}
\item alpha
\item angle
\item colour
\item family
\item fontface
\item hjust
\item lineheight
\item size
\item vjust
}
}

\examples{
require(tidygraph)
gr <- create_notable('bull') \%>\%
  mutate(class = sample(letters[1:3], n(), replace = TRUE))

ggraph(gr, 'stress') +
  geom_node_point(aes(label = class))

ggraph(gr, 'stress') +
  geom_node_label(aes(label = class), repel = TRUE)
}
\seealso{
Other geom_node_*: 
\code{\link{geom_node_arc_bar}()},
\code{\link{geom_node_circle}()},
\code{\link{geom_node_point}()},
\code{\link{geom_node_range}()},
\code{\link{geom_node_sf}()},
\code{\link{geom_node_tile}()},
\code{\link{geom_node_voronoi}()}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_node_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_linear.R
\name{layout_tbl_graph_linear}
\alias{layout_tbl_graph_linear}
\title{Place nodes on a line or circle}
\usage{
layout_tbl_graph_linear(
  graph,
  circular,
  sort.by = NULL,
  use.numeric = FALSE,
  offset = pi/2,
  weight = NULL
)
}
\arguments{
\item{graph}{An \code{tbl_graph} object}

\item{circular}{Logical. Should the layout be transformed to a circular
representation. Defaults to \code{FALSE}.}

\item{sort.by}{The name of a node variable to sort the nodes by.}

\item{use.numeric}{Logical. Should a numeric sort.by attribute be used as the
actual x-coordinates in the layout. May lead to overlapping nodes. Defaults
to FALSE}

\item{offset}{If \code{circular = TRUE}, where should it begin. Defaults to
\code{pi/2} which is equivalent to 12 o'clock.}

\item{weight}{A weight for each node. Nodes will be spread out according to
their weight so that nodes with heigher weight will have more space around
them. Ignored if \code{use.numeric = TRUE}}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{circular} as
well as any information stored as node variables in the tbl_graph object.
Further, if \code{circular = FALSE} a \code{width} column and if \code{circular = TRUE} a
\code{start}, \code{end}, and \code{r0} column.
}
\description{
This layout puts all nodes on a line, possibly sorted by a node attribute. If
\code{circular = TRUE} the nodes will be laid out on the unit circle instead.
In the case where the \code{sort.by} attribute is numeric, the numeric values
will be used as the x-position and it is thus possible to have uneven spacing
between the nodes.
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_cactustree}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_htree}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_metro}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_sf}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_edge_point.R
\name{geom_edge_point}
\alias{geom_edge_point}
\title{Draw edges as glyphs}
\usage{
geom_edge_point(
  mapping = NULL,
  data = get_edges(),
  position = "identity",
  mirror = FALSE,
  show.legend = NA,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x, y, xend, yend, group and
circular are mapped to x, y, xend, yend, edge.id and circular in the edge
data.}

\item{data}{The return of a call to \code{get_edges()} or a data.frame
giving edges in correct format (see details for for guidance on the format).
See \code{\link[=get_edges]{get_edges()}} for more details on edge extraction.}

\item{position}{Position adjustment, either as a string naming the adjustment
(e.g. \code{"jitter"} to use \code{position_jitter}), or the result of a call to a
position adjustment function. Use the latter if you need to change the
settings of the adjustment.}

\item{mirror}{Logical. Should edge points be duplicated on both sides of the
diagonal. Intended for undirected graphs. Default to \code{FALSE}}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}. These are
often aesthetics, used to set an aesthetic to a fixed value, like
\code{colour = "red"} or \code{size = 3}. They may also be parameters
to the paired geom/stat.}
}
\description{
This geom draws edges as glyphs with their x-position defined by the
x-position of the start node, and the y-position defined by the y-position of
the end node. As such it will result in a matrix layout when used in
conjunction with \code{\link[=layout_tbl_graph_matrix]{layout_tbl_graph_matrix()}}
}
\section{Aesthetics}{

\code{geom_edge_point} understands the following
aesthetics. Bold aesthetics are automatically set, but can be overwritten.
\itemize{
\item \strong{x}
\item \strong{y}
\item edge_shape
\item edge_colour
\item edge_size
\item edge_alpha
\item filter
}
}

\section{Edge aesthetic name expansion}{

In order to avoid excessive typing edge aesthetic names are
automatically expanded. Because of this it is not necessary to write
\code{edge_colour} within the \code{aes()} call as \code{colour} will
automatically be renamed appropriately.
}

\examples{
require(tidygraph)
gr <- create_notable('zachary') \%>\%
  mutate(group = group_infomap()) \%>\%
  morph(to_split, group) \%>\%
  activate(edges) \%>\%
  mutate(edge_group = as.character(.N()$group[1])) \%>\%
  unmorph()

ggraph(gr, 'matrix', sort.by = node_rank_hclust()) +
  geom_edge_point(aes(colour = edge_group), mirror = TRUE, edge_size = 3) +
  scale_y_reverse() +
  coord_fixed() +
  labs(edge_colour = 'Infomap Cluster') +
  ggtitle("Zachary' Karate Club")
}
\seealso{
Other geom_edge_*: 
\code{\link{geom_edge_arc}()},
\code{\link{geom_edge_bend}()},
\code{\link{geom_edge_bundle_force}()},
\code{\link{geom_edge_bundle_minimal}()},
\code{\link{geom_edge_bundle_path}()},
\code{\link{geom_edge_density}()},
\code{\link{geom_edge_diagonal}()},
\code{\link{geom_edge_elbow}()},
\code{\link{geom_edge_fan}()},
\code{\link{geom_edge_hive}()},
\code{\link{geom_edge_link}()},
\code{\link{geom_edge_loop}()},
\code{\link{geom_edge_parallel}()},
\code{\link{geom_edge_sf}()},
\code{\link{geom_edge_span}()},
\code{\link{geom_edge_tile}()}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edges.R
\name{get_edges}
\alias{get_edges}
\title{Create edge extractor function}
\usage{
get_edges(format = "short", collapse = "none", ...)
}
\arguments{
\item{format}{Either \code{'short'} (the default) or \code{'long'}. See
details for a descriptions of the differences}

\item{collapse}{Either \code{'none'} (the default), \code{'all'} or
\code{'direction'}. Specifies whether parallel edges should be merged. See
details for more information}

\item{...}{Additional data that will be cbind'ed together with the returned
edge data.}
}
\value{
A data.frame with columns dependent on format as well as the graph
type. In addition to the columns discussed in the details section,
the data.frame will always contain the columns \code{from}, \code{to} and
\code{circular}, the two former giving the indexes of the start and end node
and the latter if the layout is circular (needed for correct formatting of
some \verb{geom_edge_*}). The graph dependent information is:

\describe{
\item{dendrogram}{A \code{label} column will hold the value of the
\code{edgetext} attribute. In addition any value stored in the
\code{edgePar} attribute will be added. Lastly a \code{direction} column
will hold the relative position between the start and end nodes (needed for
correct formatting of \code{\link[=geom_edge_elbow]{geom_edge_elbow()}}).}
\item{igraph}{All edge attributes of the original graph object is added as
columns to the data.frame}
}
}
\description{
This function returns another function that can extract edges from a
ggraph_layout object. The functionality of the returned function is decided
by the arguments to \code{get_edges}. The need for \code{get_edges} is mainly to
pass to the \code{data} argument of the different \verb{geom_edge_*}
functions in order to present them with the right kind of data. In general
each \verb{geom_edge_*} has the default set correctly so there is only need
to modify the data argument if parallel edges should be collapsed.
}
\details{
There are two types of return formats possible for the result of the returned
function:

\describe{
\item{short}{In this format each edge is described in one line in the
format expected for \code{\link[ggplot2:geom_segment]{ggplot2::geom_segment()}}, that is, the start
node position is encoded in the \code{x} and \code{y} column and the end
node position is encoded in the \code{xend} and \code{yend} column. If node
parameters are added to the edge the name of the parameters will be
prefixed with \code{node1.} for the start node and \code{node2.} for the
end node.}
\item{long}{In this format each edge consists of two rows with matching
\code{edge.id} value. The start and end position are both encoded in the
\code{x} and \code{y} column. The relative position of the rows determines
which is the start and end node, the first occurring being the start node.
If node parameters are added to the edge data the name of the parameters
will be prefixed with \code{node.}.}
}

Node parameters are automatically added so it is possible to format edge
aesthetics according to start or end node parameters, or interpolate edge
aesthetics between start and end node parameters. Node parameters will be
prefixed to avoid name clash with edge parameters. The prefix depends on the
format (see above).

If the graph is not simple (it contains at most one edge between each node
pair) it can be collapsed so either all edges between two nodes or all edges
of the same direction between two nodes are merged. The edge parameters are
taken from the first occurring edge, so if some more sophisticated summary is
needed it is suggested that the graph be tidied up before plotting with
ggraph.
}
\seealso{
Other extractors: 
\code{\link{get_con}()},
\code{\link{get_nodes}()}
}
\concept{extractors}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_edge_sf.R
\name{geom_edge_sf}
\alias{geom_edge_sf}
\title{Draw edges as LINESTRINGs in geographical space}
\usage{
geom_edge_sf(
  mapping = NULL,
  data = get_sf_edges(),
  position = "identity",
  show.legend = NA,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default geometry is mapped to the geometry in
the edge data.}

\item{data}{The data to be displayed in this layer. There are three
options:

If \code{NULL}, the default, the data is inherited from the plot
data as specified in the call to \code{\link[ggplot2:ggplot]{ggplot()}}.

A \code{data.frame}, or other object, will override the plot
data. All objects will be fortified to produce a data frame. See
\code{\link[ggplot2:fortify]{fortify()}} for which variables will be created.

A \code{function} will be called with a single argument,
the plot data. The return value must be a \code{data.frame}, and
will be used as the layer data. A \code{function} can be created
from a \code{formula} (e.g. \code{~ head(.x, 10)}).}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.

You can also set this to one of "polygon", "line", and "point" to
override the default legend.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}'s \code{params} argument. These
arguments broadly fall into one of 4 categories below. Notably, further
arguments to the \code{position} argument, or aesthetics that are required
can \emph{not} be passed through \code{...}. Unknown arguments that are not part
of the 4 categories below are ignored.
\itemize{
\item Static aesthetics that are not mapped to a scale, but are at a fixed
value and apply to the layer as a whole. For example, \code{colour = "red"}
or \code{linewidth = 3}. The geom's documentation has an \strong{Aesthetics}
section that lists the available options. The 'required' aesthetics
cannot be passed on to the \code{params}. Please note that while passing
unmapped aesthetics as vectors is technically possible, the order and
required length is not guaranteed to be parallel to the input data.
\item When constructing a layer using
a \verb{stat_*()} function, the \code{...} argument can be used to pass on
parameters to the \code{geom} part of the layer. An example of this is
\code{stat_density(geom = "area", outline.type = "both")}. The geom's
documentation lists which parameters it can accept.
\item Inversely, when constructing a layer using a
\verb{geom_*()} function, the \code{...} argument can be used to pass on parameters
to the \code{stat} part of the layer. An example of this is
\code{geom_area(stat = "density", adjust = 0.5)}. The stat's documentation
lists which parameters it can accept.
\item The \code{key_glyph} argument of \code{\link[ggplot2:layer]{layer()}} may also be passed on through
\code{...}. This can be one of the functions described as
\link[ggplot2:draw_key]{key glyphs}, to change the display of the layer in the legend.
}}
}
\description{
This geom is equivalent in functionality to \code{\link[ggplot2:ggsf]{ggplot2::geom_sf()}} for \code{LINESTRING}
geometries and allows for plotting of edges in their geographical space in
different colours, linetypes and widths.
}
\section{Aesthetics}{

\code{geom_edge_sf} understand the following aesthetics.
\itemize{
\item alpha
\item colour
\item linetype
\item filter
}
}

\examples{
if (require("sfnetworks", quietly = TRUE)) {
  gr <- sfnetworks::as_sfnetwork(roxel)
  ggraph(gr, 'sf') + geom_edge_sf()
}

}
\seealso{
Other geom_edge_*: 
\code{\link{geom_edge_arc}()},
\code{\link{geom_edge_bend}()},
\code{\link{geom_edge_bundle_force}()},
\code{\link{geom_edge_bundle_minimal}()},
\code{\link{geom_edge_bundle_path}()},
\code{\link{geom_edge_density}()},
\code{\link{geom_edge_diagonal}()},
\code{\link{geom_edge_elbow}()},
\code{\link{geom_edge_fan}()},
\code{\link{geom_edge_hive}()},
\code{\link{geom_edge_link}()},
\code{\link{geom_edge_loop}()},
\code{\link{geom_edge_parallel}()},
\code{\link{geom_edge_point}()},
\code{\link{geom_edge_span}()},
\code{\link{geom_edge_tile}()}
}
\author{
Lorena Abad
}
\concept{geom_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/edges.R, R/geom_edge_sf.R
\name{get_edges}
\alias{get_edges}
\alias{get_sf_edges}
\title{Create edge extractor function}
\usage{
get_edges(format = "short", collapse = "none", ...)

get_sf_edges()
}
\arguments{
\item{format}{Either \code{'short'} (the default) or \code{'long'}. See
details for a descriptions of the differences}

\item{collapse}{Either \code{'none'} (the default), \code{'all'} or
\code{'direction'}. Specifies whether parallel edges should be merged. See
details for more information}

\item{...}{Additional data that will be cbind'ed together with the returned
edge data. Accepts expressions that will be evaluated on the edge data}
}
\value{
A data.frame with columns dependent on format as well as the graph
type. In addition to the columns discussed in the details section,
the data.frame will always contain the columns \code{from}, \code{to} and
\code{circular}, the two former giving the indexes of the start and end node
and the latter if the layout is circular (needed for correct formatting of
some \verb{geom_edge_*}). The graph dependent information is:

\describe{
\item{dendrogram}{A \code{label} column will hold the value of the
\code{edgetext} attribute. In addition any value stored in the
\code{edgePar} attribute will be added. Lastly a \code{direction} column
will hold the relative position between the start and end nodes (needed for
correct formatting of \code{\link[=geom_edge_elbow]{geom_edge_elbow()}}).}
\item{igraph}{All edge attributes of the original graph object is added as
columns to the data.frame}
}
}
\description{
This function returns another function that can extract edges from a
ggraph_layout object. The functionality of the returned function is decided
by the arguments to \code{get_edges}. The need for \code{get_edges} is mainly to
pass to the \code{data} argument of the different \verb{geom_edge_*}
functions in order to present them with the right kind of data. In general
each \verb{geom_edge_*} has the default set correctly so there is only need
to modify the data argument if parallel edges should be collapsed.
}
\details{
There are two types of return formats possible for the result of the returned
function:

\describe{
\item{short}{In this format each edge is described in one line in the
format expected for \code{\link[ggplot2:geom_segment]{ggplot2::geom_segment()}}, that is, the start
node position is encoded in the \code{x} and \code{y} column and the end
node position is encoded in the \code{xend} and \code{yend} column. If node
parameters are added to the edge the name of the parameters will be
prefixed with \code{node1.} for the start node and \code{node2.} for the
end node.}
\item{long}{In this format each edge consists of two rows with matching
\code{edge.id} value. The start and end position are both encoded in the
\code{x} and \code{y} column. The relative position of the rows determines
which is the start and end node, the first occurring being the start node.
If node parameters are added to the edge data the name of the parameters
will be prefixed with \code{node.}.}
}

Node parameters are automatically added so it is possible to format edge
aesthetics according to start or end node parameters, or interpolate edge
aesthetics between start and end node parameters. Node parameters will be
prefixed to avoid name clash with edge parameters. The prefix depends on the
format (see above).

If the graph is not simple (it contains at most one edge between each node
pair) it can be collapsed so either all edges between two nodes or all edges
of the same direction between two nodes are merged. The edge parameters are
taken from the first occurring edge, so if some more sophisticated summary is
needed it is suggested that the graph be tidied up before plotting with
ggraph.
}
\seealso{
Other extractors: 
\code{\link{get_con}()},
\code{\link{get_sf_nodes}()}
}
\concept{extractors}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/connections.R
\name{get_con}
\alias{get_con}
\title{Create a connection extractor function}
\usage{
get_con(
  from = integer(),
  to = integer(),
  paths = NULL,
  ...,
  weight = NULL,
  mode = "all"
)
}
\arguments{
\item{from, to}{The index of the start and end nodes for the connections}

\item{paths}{A list of integer vectors giving the index of nodes defining
connections}

\item{...}{Additional information to be added to the final data output}

\item{weight}{An expression to be evaluated on the edge data to provide
weights for the shortest path calculations}

\item{mode}{Character constant, gives whether the shortest paths to or from
the given vertices should be calculated for directed graphs. If \code{out}
then the shortest paths \emph{from} the vertex, if \code{in} then \emph{to}
it will be considered. If \code{all}, the default, then the corresponding
undirected graph will be used, ie. not directed paths are searched. This
argument is ignored for undirected graphs.}
}
\value{
A function that takes a layout_ggraph object and returns the given
connections
}
\description{
Connections within the ggraph terminology are links between nodes that are
not part of the network structure itself. In that sense connections do not
affect the layout calculation in any way and will not be drawn by the
standard \verb{geom_edge_*} functions. A connection does not need to only be
defined by a start and end node, but can include intermediary nodes.
\code{get_con} helps in creating connection data by letting you specify start
and end nodes and automatically finds the shortest path within the graph
structure that connects the given points. If this is not what is needed it is
also possible to supply a list of vectors giving node indices that define a
connection.
}
\seealso{
Other extractors: 
\code{\link{get_edges}()},
\code{\link{get_nodes}()}
}
\concept{extractors}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_htree.R
\name{layout_tbl_graph_htree}
\alias{layout_tbl_graph_htree}
\title{Layout binary trees in a fractal H formation}
\usage{
layout_tbl_graph_htree(
  graph,
  sort.by = NULL,
  direction = "out",
  circular = FALSE
)
}
\arguments{
\item{graph}{An \code{tbl_graph} object}

\item{sort.by}{The name of a node variable to sort the nodes by.}

\item{direction}{The direction of the tree in the graph. \code{'out'} (default)
means that parents point towards their children, while \code{'in'} means that
children point towards their parent.}

\item{circular}{Logical. Should the layout be transformed to a circular
representation. Ignored}
}
\value{
A data.frame with the columns \code{x}, \code{y}, \code{leaf}, \code{depth}, \code{circular}
as well as any information stored as node variables in the tbl_graph object.
}
\description{
This is a spac efficient layout only useful for binary trees. It is fractal
and works by offsetting child nodes from their parent either horizontally or
vertically depending on depth. The offset is decreased at each step by a
factor of the square root of 2.
}
\note{
H Tree is a layout intended for trees, that is, graphs where nodes
only have one parent and zero or more children. If the provided graph does
not fit this format an attempt to convert it to such a format will be made.
}
\seealso{
Other layout_tbl_graph_*: 
\code{\link{layout_tbl_graph_auto}()},
\code{\link{layout_tbl_graph_backbone}()},
\code{\link{layout_tbl_graph_cactustree}()},
\code{\link{layout_tbl_graph_centrality}()},
\code{\link{layout_tbl_graph_circlepack}()},
\code{\link{layout_tbl_graph_dendrogram}()},
\code{\link{layout_tbl_graph_eigen}()},
\code{\link{layout_tbl_graph_fabric}()},
\code{\link{layout_tbl_graph_focus}()},
\code{\link{layout_tbl_graph_hive}()},
\code{\link{layout_tbl_graph_igraph}()},
\code{\link{layout_tbl_graph_linear}()},
\code{\link{layout_tbl_graph_manual}()},
\code{\link{layout_tbl_graph_matrix}()},
\code{\link{layout_tbl_graph_metro}()},
\code{\link{layout_tbl_graph_partition}()},
\code{\link{layout_tbl_graph_pmds}()},
\code{\link{layout_tbl_graph_sf}()},
\code{\link{layout_tbl_graph_stress}()},
\code{\link{layout_tbl_graph_treemap}()},
\code{\link{layout_tbl_graph_unrooted}()}
}
\concept{layout_tbl_graph_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom_edge_point.R
\name{geom_edge_point}
\alias{geom_edge_point}
\title{Draw edges as glyphs}
\usage{
geom_edge_point(
  mapping = NULL,
  data = get_edges(),
  position = "identity",
  mirror = FALSE,
  show.legend = NA,
  ...
)
}
\arguments{
\item{mapping}{Set of aesthetic mappings created by \code{\link[ggplot2:aes]{ggplot2::aes()}}
or \code{\link[ggplot2:aes_]{ggplot2::aes_()}}. By default x, y, xend, yend, group and
circular are mapped to x, y, xend, yend, edge.id and circular in the edge
data.}

\item{data}{The return of a call to \code{get_edges()} or a data.frame
giving edges in correct format (see details for for guidance on the format).
See \code{\link[=get_edges]{get_edges()}} for more details on edge extraction.}

\item{position}{A position adjustment to use on the data for this layer. This
can be used in various ways, including to prevent overplotting and
improving the display. The \code{position} argument accepts the following:
\itemize{
\item The result of calling a position function, such as \code{position_jitter()}.
This method allows for passing extra arguments to the position.
\item A string naming the position adjustment. To give the position as a
string, strip the function name of the \code{position_} prefix. For example,
to use \code{position_jitter()}, give the position as \code{"jitter"}.
\item For more information and other ways to specify the position, see the
\link[ggplot2:layer_positions]{layer position} documentation.
}}

\item{mirror}{Logical. Should edge points be duplicated on both sides of the
diagonal. Intended for undirected graphs. Default to \code{FALSE}}

\item{show.legend}{logical. Should this layer be included in the legends?
\code{NA}, the default, includes if any aesthetics are mapped.
\code{FALSE} never includes, and \code{TRUE} always includes.
It can also be a named logical vector to finely select the aesthetics to
display. To include legend keys for all levels, even
when no data exists, use \code{TRUE}.  If \code{NA}, all levels are shown in legend,
but unobserved levels are omitted.}

\item{...}{Other arguments passed on to \code{\link[ggplot2:layer]{layer()}}'s \code{params} argument. These
arguments broadly fall into one of 4 categories below. Notably, further
arguments to the \code{position} argument, or aesthetics that are required
can \emph{not} be passed through \code{...}. Unknown arguments that are not part
of the 4 categories below are ignored.
\itemize{
\item Static aesthetics that are not mapped to a scale, but are at a fixed
value and apply to the layer as a whole. For example, \code{colour = "red"}
or \code{linewidth = 3}. The geom's documentation has an \strong{Aesthetics}
section that lists the available options. The 'required' aesthetics
cannot be passed on to the \code{params}. Please note that while passing
unmapped aesthetics as vectors is technically possible, the order and
required length is not guaranteed to be parallel to the input data.
\item When constructing a layer using
a \verb{stat_*()} function, the \code{...} argument can be used to pass on
parameters to the \code{geom} part of the layer. An example of this is
\code{stat_density(geom = "area", outline.type = "both")}. The geom's
documentation lists which parameters it can accept.
\item Inversely, when constructing a layer using a
\verb{geom_*()} function, the \code{...} argument can be used to pass on parameters
to the \code{stat} part of the layer. An example of this is
\code{geom_area(stat = "density", adjust = 0.5)}. The stat's documentation
lists which parameters it can accept.
\item The \code{key_glyph} argument of \code{\link[ggplot2:layer]{layer()}} may also be passed on through
\code{...}. This can be one of the functions described as
\link[ggplot2:draw_key]{key glyphs}, to change the display of the layer in the legend.
}}
}
\description{
This geom draws edges as glyphs with their x-position defined by the
x-position of the start node, and the y-position defined by the y-position of
the end node. As such it will result in a matrix layout when used in
conjunction with \code{\link[=layout_tbl_graph_matrix]{layout_tbl_graph_matrix()}}
}
\section{Aesthetics}{

\code{geom_edge_point} understands the following
aesthetics. Bold aesthetics are automatically set, but can be overwritten.
\itemize{
\item \strong{x}
\item \strong{y}
\item edge_shape
\item edge_colour
\item edge_size
\item edge_alpha
\item filter
}
}

\section{Edge aesthetic name expansion}{

In order to avoid excessive typing edge aesthetic names are
automatically expanded. Because of this it is not necessary to write
\code{edge_colour} within the \code{aes()} call as \code{colour} will
automatically be renamed appropriately.
}

\examples{
require(tidygraph)
gr <- create_notable('zachary') \%>\%
  mutate(group = group_infomap()) \%>\%
  morph(to_split, group) \%>\%
  activate(edges) \%>\%
  mutate(edge_group = as.character(.N()$group[1])) \%>\%
  unmorph()

ggraph(gr, 'matrix', sort.by = node_rank_hclust()) +
  geom_edge_point(aes(colour = edge_group), mirror = TRUE, edge_size = 3) +
  scale_y_reverse() +
  coord_fixed() +
  labs(edge_colour = 'Infomap Cluster') +
  ggtitle("Zachary' Karate Club")
}
\seealso{
Other geom_edge_*: 
\code{\link{geom_edge_arc}()},
\code{\link{geom_edge_bend}()},
\code{\link{geom_edge_bundle_force}()},
\code{\link{geom_edge_bundle_minimal}()},
\code{\link{geom_edge_bundle_path}()},
\code{\link{geom_edge_density}()},
\code{\link{geom_edge_diagonal}()},
\code{\link{geom_edge_elbow}()},
\code{\link{geom_edge_fan}()},
\code{\link{geom_edge_hive}()},
\code{\link{geom_edge_link}()},
\code{\link{geom_edge_loop}()},
\code{\link{geom_edge_parallel}()},
\code{\link{geom_edge_sf}()},
\code{\link{geom_edge_span}()},
\code{\link{geom_edge_tile}()}
}
\author{
Thomas Lin Pedersen
}
\concept{geom_edge_*}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/connections.R
\name{get_con}
\alias{get_con}
\title{Create a connection extractor function}
\usage{
get_con(
  from = integer(),
  to = integer(),
  paths = NULL,
  ...,
  weight = NULL,
  mode = "all"
)
}
\arguments{
\item{from, to}{The index of the start and end nodes for the connections}

\item{paths}{A list of integer vectors giving the index of nodes defining
connections}

\item{...}{Additional information to be added to the final data output.
Accepts expressions that will be evaluated on the node data in it's
original order (irrespective of any reordering by the layout)}

\item{weight}{An expression to be evaluated on the edge data to provide
weights for the shortest path calculations}

\item{mode}{Character constant, gives whether the shortest paths to or from
the given vertices should be calculated for directed graphs. If \code{out}
then the shortest paths \emph{from} the vertex, if \verb{in} then \emph{to}
it will be considered. If \code{all}, the default, then the corresponding
undirected graph will be used, i.e. not directed paths are searched. This
argument is ignored for undirected graphs.}
}
\value{
A function that takes a layout_ggraph object and returns the given
connections
}
\description{
Connections within the ggraph terminology are links between nodes that are
not part of the network structure itself. In that sense connections do not
affect the layout calculation in any way and will not be drawn by the
standard \verb{geom_edge_*} functions. A connection does not need to only be
defined by a start and end node, but can include intermediary nodes.
\code{get_con} helps in creating connection data by letting you specify start
and end nodes and automatically finds the shortest path within the graph
structure that connects the given points. If this is not what is needed it is
also possible to supply a list of vectors giving node indices that define a
connection.
}
\seealso{
Other extractors: 
\code{\link{get_edges}()},
\code{\link{get_sf_nodes}()}
}
\concept{extractors}
